// Project imports:
import 'package:invoiceninja_flutter/utils/strings.dart';

abstract class LocaleCodeAware {
  LocaleCodeAware(this.localeCode);

  final String localeCode;
}

///
/// It provides localization strings to be statically used via getters
///
/// It should be used by a [LocaleCodeAware] to get the i18n strings for
/// a specified locale.
mixin LocalizationsProvider on LocaleCodeAware {
  static final Map<String, Map<String, String>> _localizedValues = {
    'en': {
      // STARTER: lang key - do not remove comment
      'show_pdf_preview': 'Show PDF Preview',
      'show_pdf_preview_help': 'Display PDF preview while editing invoices',
      'print_pdf': 'Print PDF',
      'remind_me': 'Remind Me',
      'instant_bank_pay': 'Instant Bank Pay',
      'click_selected': 'Click Selected',
      'hide_preview': 'Hide Preview',
      'edit_record': 'Edit Record',
      'credit_is_more_than_invoice':
          'The credit amount can not be more than the invoice amount',
      'giropay': 'Giropay',
      'direct_debit': 'Direct Debit',
      'please_set_a_password': 'Please set an account password',
      'set_password': 'Set Password',
      'recommend_desktop':
          'We recommend using the desktop app for the best performance',
      'recommend_mobile':
          'We recommend using the mobile app for the best performance',
      'disconnected_gateway': 'Successfully disconnected gateway',
      'disconnect': 'Disconnect',
      'add_to_invoices': 'Add to Invoices',
      'acss': 'ACSS',
      'becs': 'BECS',
      'bulk_download': 'Download',
      'persist_data_help':
          'Save data locally to enable the app to start faster, disabling may improve performance in large accounts',
      'persist_ui': 'Persist UI',
      'persist_ui_help':
          'Save UI state locally to enable the app to start at the last location, disabling may improve performance',
      'client_postal_code': 'Client Postal Code',
      'client_vat_number': 'Client VAT Number',
      'has_tasks': 'Has Tasks',
      'registration': 'Registration',
      'unauthorized_stripe_warning':
          'Please authorize Stripe to accept online payments.',
      'view_expense': 'View Expense',
      'view_statement': 'View Statement',
      'sepa': 'SEPA',
      'ideal': 'iDEAL',
      'przelewy24': 'Przelewy24',
      'eps': 'EPS',
      'fpx': 'FPX',
      'update_all_records': 'Update all records',
      'system': 'System',
      'set_default_company': 'Set Default Company',
      'updated_company': 'Successfully updated company',
      'kbc': 'KBC',
      'bancontact': 'Bancontact',
      'why_are_you_leaving': 'Help us improve by telling us why (optional)',
      'webhook_success': 'Webhook Success',
      'error_cross_client_tasks': 'Tasks must all belong to the same client',
      'error_cross_client_expenses':
          'Expenses must all belong to the same client',
      'app': 'App',
      'for_best_performance': 'For the best performance download the :app app',
      'gross_line_total': 'Gross Line Total',
      'bulk_email_invoice': 'Email Invoice',
      'bulk_email_quote': 'Email Quote',
      'bulk_email_credit': 'Email Credit',
      'from_name': 'From Name',
      'clone_to_expense': 'Clone to Expense',
      'recurring_expense': 'Recurring Expense',
      'recurring_expenses': 'Recurring Expenses',
      'new_recurring_expense': 'New Recurring Expense',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'created_recurring_expense': 'Successfully created recurring expense',
      'updated_recurring_expense': 'Successfully updated recurring expense',
      'archived_recurring_expense': 'Successfully archived recurring expense',
      'deleted_recurring_expense': 'Successfully deleted recurring expense',
      'removed_recurring_expense': 'Successfully removed recurring expense',
      'restored_recurring_expense': 'Successfully restored recurring expense',
      'search_recurring_expense': 'Search Recurring Expense',
      'search_recurring_expenses': 'Search Recurring Expenses',
      'last_sent_date': 'Last Sent Date',
      'include_drafts': 'Include Drafts',
      'include_drafts_help': 'Include draft records in reports',
      'is_invoiced': 'Is Invoiced',
      'change_plan': 'Change Plan',
      'persist_data': 'Persist Data',
      'customer_count': 'Customer Count',
      'verify_customers': 'Verify Customers',
      'google_analytics': 'Google Analytics',
      'google_analytics_tracking_id': 'Google Analytics Tracking ID',
      'decimal_comma': 'Decimal Comma',
      'use_comma_as_decimal_place': 'Use comma as decimal place in forms',
      'select_method': 'Select Method',
      'select_platform': 'Select Platform',
      'use_web_app_to_connect_gmail':
          'Please use the web app to connect to Gmail',
      'expense_tax_help': 'Item tax rates are disabled',
      'enable_markdown': 'Enable Markdown',
      'enable_markdown_help': 'Convert markdown to HTML on the PDF',
      'user_guide': 'User Guide',
      'add_second_contact': 'Add Second Contact',
      'previous_page': 'Previous Page',
      'next_page': 'Next Page',
      'export_colors': 'Export Colors',
      'import_colors': 'Import Colors',
      'clear_all': 'Clear All',
      'contrast': 'Contrast',
      'custom_colors': 'Custom Colors',
      'colors': 'Colors',
      'sidebar_active_background_color': 'Sidebar Active Background Color',
      'sidebar_active_font_color': 'Sidebar Active Font Color',
      'sidebar_inactive_background_color': 'Sidebar Inactive Background Color',
      'sidebar_inactive_font_color': 'Sidebar Inactive Font Color',
      'table_alternate_row_background_color':
          'Table Alternate Row Background Color',
      'invoice_header_background_color': 'Invoice Header Background Color',
      'invoice_header_font_color': 'Invoice Header Font Color',
      'net_subtotal': 'Net Subtotal',
      'review_app': 'Review App',
      'check_status': 'Check Status',
      'free_trial': 'Free Trial',
      'free_trial_help':
          'All accounts receive a two week trial of the Pro plan, once the trial ends your account will automatically change to the free plan.',
      'free_trial_ends_in_days':
          'The Pro plan trial ends in :count days, click to upgrade.',
      'free_trial_ends_today':
          'Today is the last day of the Pro plan trial, click to upgrade.',
      'change_email': 'Change Email',
      'client_portal_domain_hint':
          'Optionally configure a separate client portal domain',
      'tasks_shown_in_portal': 'Tasks Shown in Portal',
      'uninvoiced': 'Uninvoiced',
      'subdomain_guide':
          'The subdomain is used in the client portal to personalize links to match your brand. ie, https://your-brand.invoicing.co',
      'send_time': 'Send Time',
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\'ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Enable',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help': 'Lowercase letters, numbers and hyphens are supported',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanced settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo Code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invite',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Enable Two Factor',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Successfully refunded payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convert to Invoice',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Invoice Task',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Hide',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Column',
      'sample': 'Sample',
      'map_to': 'Map To',
      'import': 'Import',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Select File',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV File',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \':value\' to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Invoice Total',
      'quote_total': 'Quote Total',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning': 'Company is not activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Client Name',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully updated task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted expense category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived expense :value categories',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error': 'The payment amount can not be negative',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be Invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible to clients',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Recurring Invoice',
      'recurring_invoices': 'Recurring Invoices',
      'new_recurring_invoice': 'New Recurring Invoice',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help': 'Support paying a minimum amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Token Billing',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Enabled',
      'optin': 'Disabled by default',
      'optout': 'Enabled by default',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully queued invoices to be sent',
      'emailed_quotes': 'Successfully queued quotes to be sent',
      'emailed_credits': 'Successfully queued credits to be sent',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hours',
      'statement': 'Statement',
      'taxes': 'Taxes',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply_credit': 'Apply Credit',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'To',
      'health_check': 'Health Check',
      'payment_type_id': 'Payment Type',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Upcoming Invoices',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Recent Payments',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': 'Create Invoice',
      'create_quote': 'Create Quote',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create Vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Delete Quote',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Delete Invoice',
      'update_client': 'Update Client',
      'delete_client': 'Delete Client',
      'delete_payment': 'Delete Payment',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Delete Vendor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Create Task',
      'update_task': 'Update Task',
      'delete_task': 'Delete Task',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target URL',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Edit Token',
      'created_token': 'Successfully created token',
      'updated_token': 'Successfully updated token',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Successfully deleted token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Email Invoice',
      'email_quote': 'Email Quote',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use Default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Credit Amount',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search :count Documents',
      'search_designs': 'Search :count Designs',
      'search_invoices': 'Search :count Invoices',
      'search_clients': 'Search :count Clients',
      'search_products': 'Search :count Products',
      'search_quotes': 'Search :count Quotes',
      'search_credits': 'Search :count Credits',
      'search_vendors': 'Search :count Vendors',
      'search_users': 'Search :count Users',
      'search_tax_rates': 'Search :count Tax Rates',
      'search_tasks': 'Search :count Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search :count Projects',
      'search_expenses': 'Search :count Expenses',
      'search_payments': 'Search :count Payments',
      'search_groups': 'Search :count Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'Custom 1',
      'custom2': 'Custom 2',
      'custom3': 'Custom 3',
      'custom4': 'Custom 4',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': 'Invoice Sent',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Delete Account',
      'cancel_account_message':
          'Warning: This will permanently delete your account [:company], there is no undo',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company [:company], there is no undo',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'account_management': 'Account Management',
      'credit_date': 'Credit Date',
      'credit': 'Credit',
      'credits': 'Credits',
      'new_credit': 'New Credit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Successfully created credit',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Successfully archived credit',
      'deleted_credit': 'Successfully deleted credit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Successfully restored credit',
      'archived_credits': 'Successfully archived :value credits',
      'deleted_credits': 'Successfully deleted :value credits',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Current Version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Learn More',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking ID',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'New Company',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Chart',
      'count': 'Count',
      'totals': 'Totals',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Group By',
      'credit_balance': 'Credit Balance',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client ID',
      'assigned_to': 'Assigned To',
      'created_by': 'Created By',
      'assigned_to_id': 'Assigned To ID',
      'created_by_id': 'Created By ID',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Report',
      'add_company': 'Add Company',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Help',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by :value',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'Entity State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Message',
      'from': 'From',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Ensure client fee matches the gateway fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'Support Forum',
      'about': 'About',
      'documentation': 'Documentation',
      'contact_us': 'Contact Us',
      'subtotal': 'Subtotal',
      'line_total': 'Line Total',
      'item': 'Item',
      'credit_email': 'Credit Email',
      'iframe_url': 'iFrame URL',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password must be at least 8 character long',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'No client selected',
      'configure_rates': 'Configure Rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Tax Settings',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Recover Password',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Invoice Email',
      'payment_email': 'Payment Email',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Quote Email',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'User Management',
      'users': 'Users',
      'new_user': 'New User',
      'edit_user': 'Edit User',
      'created_user': 'Successfully created user',
      'updated_user': 'Successfully updated user',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Successfully deleted user',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Successfully restored user',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'General Settings',
      'invoice_options': 'Invoice Options',
      'hide_paid_to_date': 'Hide Paid to Date',
      'hide_paid_to_date_help':
          'Only display the "Paid to Date" area on your invoices once a payment has been received.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primary Color',
      'secondary_color': 'Secondary Color',
      'page_size': 'Page Size',
      'font_size': 'Font Size',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Invoice Fields',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Invoice Terms',
      'invoice_footer': 'Invoice Footer',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Two Weeks',
      'freq_four_weeks': 'Four Weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two Months',
      'freq_three_months': 'Three Months',
      'freq_four_months': 'Four Months',
      'freq_six_months': 'Six Months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two Years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Charge Taxes',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Custom CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'If enabled the contact will be able to set a password when viewing their first invoice.',
      'authorization': 'Authorization',
      'subdomain': 'Subdomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Email Signature',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable Min',
      'enable_max': 'Enable Max',
      'min_limit': 'Min Limit',
      'max_limit': 'Max Limit',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Update Address',
      'update_address_help': 'Update client\'s address with provided details',
      'rate': 'Rate',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Edit Tax Rate',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Successfully archived tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Fill Products',
      'fill_products_help':
          'Selecting a product will automatically fill in the description and cost',
      'update_products': 'Update Products',
      'update_products_help':
          'Updating an invoice will automatically update the product library',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Sunday',
      'monday': 'Monday',
      'tuesday': 'Tuesday',
      'wednesday': 'Wednesday',
      'thursday': 'Thursday',
      'friday': 'Friday',
      'saturday': 'Saturday',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': 'Military Time',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'archived_group': 'Successfully archived group',
      'deleted_group': 'Successfully deleted group',
      'restored_group': 'Successfully restored group',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Product Settings',
      'device_settings': 'Device Settings',
      'defaults': 'Defaults',
      'basic_settings': 'Basic Settings',
      'advanced_settings': 'Advanced Settings',
      'company_details': 'Company Details',
      'user_details': 'User Details',
      'localization': 'Localization',
      'online_payments': 'Online Payments',
      'tax_rates': 'Tax Rates',
      'notifications': 'Notifications',
      'import_export': 'Import | Export',
      'custom_fields': 'Custom Fields',
      'invoice_design': 'Invoice Design',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Email Settings',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Data Visualizations',
      'price': 'Price',
      'email_sign_up': 'Sign up with email',
      'google_sign_up': 'Sign up with Google',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service': 'Terms of Service',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Sign Up',
      'account_login': 'Account Login',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Requires an Enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'Pending',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Converted',
      'add_documents_to_invoice': 'Add Documents to Invoice',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert Currency',
      'mark_paid': 'Mark Paid',
      'category': 'Category',
      'address': 'Address',
      'new_vendor': 'New Vendor',
      'created_vendor': 'Successfully created vendor',
      'updated_vendor': 'Successfully updated vendor',
      'archived_vendor': 'Successfully archived vendor',
      'deleted_vendor': 'Successfully deleted vendor',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Successfully archived :value vendors',
      'deleted_vendors': 'Successfully deleted :value vendors',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'New Expense',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived :value expenses',
      'deleted_expenses': 'Successfully deleted :value expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Invoiced',
      'logged': 'Logged',
      'running': 'Running',
      'resume': 'Resume',
      'task_errors': 'Please correct any overlapping times',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'Successfully started task',
      'stopped_task': 'Successfully stopped task',
      'resumed_task': 'Successfully resumed task',
      'now': 'Now',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Timer',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Start Time',
      'end_time': 'End Time',
      'date': 'Date',
      'times': 'Times',
      'duration': 'Duration',
      'new_task': 'New Task',
      'created_task': 'Successfully created task',
      'updated_task': 'Successfully updated task',
      'archived_task': 'Successfully archived task',
      'deleted_task': 'Successfully deleted task',
      'restored_task': 'Successfully restored task',
      'archived_tasks': 'Successfully archived :value tasks',
      'deleted_tasks': 'Successfully deleted :value tasks',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :value projects',
      'deleted_projects': 'Successfully deleted :value projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'click here',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Footer',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Custom',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'View Invoice',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Edit Client',
      'edit_product': 'Edit Product',
      'edit_invoice': 'Edit Invoice',
      'edit_quote': 'Edit Quote',
      'edit_payment': 'Edit Payment',
      'edit_task': 'Edit Task',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Edit Vendor',
      'edit_project': 'Edit Project',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Billing Address',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Total Revenue',
      'average_invoice': 'Average Invoice',
      'outstanding': 'Outstanding',
      'invoices_sent': 'Invoices Sent',
      'active_clients': 'Active Clients',
      'close': 'Close',
      'email': 'Email',
      'password': 'Password',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Name',
      'logout': 'Log Out',
      'login': 'Login',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Search',
      'active': 'Active',
      'archived': 'Archived',
      'deleted': 'Deleted',
      'dashboard': 'Dashboard',
      'archive': 'Archive',
      'delete': 'Delete',
      'restore': 'Restore',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Save',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Paid to Date',
      'balance_due': 'Balance Due',
      'balance': 'Balance',
      'overview': 'Overview',
      'details': 'Details',
      'phone': 'Phone',
      'website': 'Website',
      'vat_number': 'VAT Number',
      'id_number': 'ID Number',
      'create': 'Create',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Contacts',
      'additional': 'Additional',
      'first_name': 'First Name',
      'last_name': 'Last Name',
      'add_contact': 'Add Contact',
      'are_you_sure': 'Are you sure?',
      'cancel': 'Cancel',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Product',
      'products': 'Products',
      'new_product': 'New Product',
      'created_product': 'Successfully created product',
      'updated_product': 'Successfully updated product',
      'archived_product': 'Successfully archived product',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :value products',
      'deleted_products': 'Successfully deleted :value products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Notes',
      'cost': 'Cost',
      'client': 'Client',
      'clients': 'Clients',
      'new_client': 'New Client',
      'created_client': 'Successfully created client',
      'updated_client': 'Successfully updated client',
      'archived_client': 'Successfully archived client',
      'archived_clients': 'Successfully archived :value clients',
      'deleted_client': 'Successfully deleted client',
      'deleted_clients': 'Successfully deleted :value clients',
      'restored_client': 'Successfully restored client',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Street',
      'address2': 'Apt/Suite',
      'city': 'City',
      'state': 'State/Province',
      'postal_code': 'Postal Code',
      'country': 'Country',
      'invoice': 'Invoice',
      'invoices': 'Invoices',
      'new_invoice': 'New Invoice',
      'created_invoice': 'Successfully created invoice',
      'updated_invoice': 'Successfully updated invoice',
      'archived_invoice': 'Successfully archived invoice',
      'deleted_invoice': 'Successfully deleted invoice',
      'restored_invoice': 'Successfully restored invoice',
      'archived_invoices': 'Successfully archived :value invoices',
      'deleted_invoices': 'Successfully deleted :value invoices',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Successfully queued invoice to be sent',
      'emailed_payment': 'Successfully queued payment to be sent',
      'amount': 'Amount',
      'invoice_number': 'Invoice Number',
      'invoice_date': 'Invoice Date',
      'discount': 'Discount',
      'po_number': 'PO Number',
      'terms': 'Terms',
      'public_notes': 'Public Notes',
      'private_notes': 'Private Notes',
      'frequency': 'Frequency',
      'start_date': 'Start Date',
      'end_date': 'End Date',
      'quote_number': 'Quote Number',
      'quote_date': 'Quote Date',
      'valid_until': 'Valid Until',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Description',
      'unit_cost': 'Unit Cost',
      'quantity': 'Quantity',
      'add_item': 'Add Item',
      'contact': 'Contact',
      'work_phone': 'Phone',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Due Date',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click ▶ to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Percent',
      'edit': 'Edit',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Settings',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Created At',
      'created_on': 'Created On',
      'updated_at': 'Updated At',
      'tax': 'Tax',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Sent',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Partial',
      'paid': 'Paid',
      'mark_sent': 'Mark Sent',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Done',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Dark Mode',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Activity',
      'no_records_found': 'No records found',
      'clone': 'Clone',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Payment Terms',
      'payment_date': 'Payment Date',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Cancelled',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Client Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'Reminder 1',
      'reminder2': 'Reminder 2',
      'reminder3': 'Reminder 3',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Subject',
      'body': 'Body',
      'send_email': 'Send Email',
      'email_receipt': 'Email payment receipt to the client',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Customize',
      'history': 'History',
      'payment': 'Payment',
      'payments': 'Payments',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Transaction Reference',
      'enter_payment': 'Enter Payment',
      'new_payment': 'Enter Payment',
      'created_payment': 'Successfully created payment',
      'updated_payment': 'Successfully updated payment',
      'archived_payment': 'Successfully archived payment',
      'deleted_payment': 'Successfully deleted payment',
      'restored_payment': 'Successfully restored payment',
      'archived_payments': 'Successfully archived :value payments',
      'deleted_payments': 'Successfully deleted :value payments',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Quote',
      'quotes': 'Quotes',
      'new_quote': 'New Quote',
      'created_quote': 'Successfully created quote',
      'updated_quote': 'Successfully updated quote',
      'archived_quote': 'Successfully archived quote',
      'deleted_quote': 'Successfully deleted quote',
      'restored_quote': 'Successfully restored quote',
      'archived_quotes': 'Successfully archived :value quotes',
      'deleted_quotes': 'Successfully deleted :value quotes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Expense',
      'expenses': 'Expenses',
      'vendor': 'Vendor',
      'vendors': 'Vendors',
      'task': 'Task',
      'tasks': 'Tasks',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user created client :client',
      'activity_2': ':user archived client :client',
      'activity_3': ':user deleted client :client',
      'activity_4': ':user created invoice :invoice',
      'activity_5': ':user updated invoice :invoice',
      'activity_6': ':user emailed invoice :invoice to :client',
      'activity_7': ':contact viewed invoice :invoice',
      'activity_8': ':user archived invoice :invoice',
      'activity_9': ':user deleted invoice :invoice',
      'activity_10': ':contact entered payment :payment for invoice :invoice',
      'activity_11': ':user updated payment :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote to :client',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored client :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user created expense :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled payment :payment',
      'activity_40': ':user refunded payment :payment',
      'activity_41': 'Payment :payment failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user created user',
      'activity_49': ':user updated user',
      'activity_50': ':user archived user',
      'activity_51': ':user deleted user',
      'activity_52': ':user restored user',
      'activity_53': ':user marked invoice :invoice as sent',
      'activity_54': ':user applied payment :payment to invoice :invoice',
      'activity_55': '',
      'activity_56': '',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :client',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :client',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :client',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :client',
      'activity_80': ':user created subscription',
      'activity_81': ':user updated subscription',
      'activity_82': ':user archived subscription',
      'activity_83': ':user deleted subscription',
      'activity_84': ':user restored subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Successfully queued quote to be sent',
      'emailed_credit': 'Successfully queued credit to be sent',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Select',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value 1',
      'custom_value2': 'Custom Value 2',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Invoice Number Counter',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Quote Number Counter',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice/Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Invoice Due Date',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Bill',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid Amount',
      'payment_amount': 'Payment Amount',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Bank Id',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'status_color_theme': 'Status Color Theme',
      'load_color_theme': 'Load Color Theme',
    },
    'sq': {
      'import_data': 'Importo të dhëna',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'E pacaktuar',
      'partial_value': 'Duhet të jetë më shumë se zero dhe më pak se totali',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Aktivizo',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Siguria',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Pagesë e rimbursuar',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Ktheje Ofertën në Faturë',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Faturo detyrën',
      'invoice_expense': 'Faturë shpenzimesh',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Fshih',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolona',
      'sample': 'Shembull',
      'map_to': 'Map To',
      'import': 'Importo',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Ju lutem zgjedhni një fajll',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Skedar CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Paguar pjesërisht',
      'invoice_total': 'Totali i faturës',
      'quote_total': 'Totali i Ofertës',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Paralajmerim',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Emri i klientit',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Kategoritë e shpenzimeve',
      'new_expense_category': 'Kategori e re e shpenzimeve',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category':
          'Kategoria e shpenzimeve është krijuar me sukses',
      'updated_expense_category':
          'Është perditesuar me sukses kategoria e shpenzimeve',
      'archived_expense_category':
          'Kategoria e shpenzimeve është arkivuar me sukses',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category':
          'Kategoria e shpenzimeve është rikthyer me sukses',
      'archived_expense_categories':
          ':count kategori të shpenzimeve janë arkivuar me sukses',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Duhet të faturohet',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Faturë e përsëritshme',
      'recurring_invoices': 'Fatura të përsëritshme',
      'new_recurring_invoice': 'Faturë e re e përsëritshme',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice':
          'Faturat e përsëritshme janë arkivuar me sukses',
      'deleted_recurring_invoice':
          'Faturat e përsëritshme janë fshirë  me sukses',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice':
          'Faturat e përsëritshme janë rikthyer me sukses',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Shiko portalin',
      'copy_link': 'Copy Link',
      'token_billing': 'Ruaj detajet e pagesës',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Gjithmonë',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Emri i kompanisë',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Ore',
      'statement': 'Statement',
      'taxes': 'Taksat',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Për',
      'health_check': 'Health Check',
      'payment_type_id': 'Lloji i pagesës',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Faturat e ardhshme',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Pagesat e fundit',
      'upcoming_quotes': 'Ofertat e ardhshme',
      'expired_quotes': 'Ofertat e skaduara',
      'create_client': 'Create Client',
      'create_invoice': 'Krijo faturë',
      'create_quote': 'Krijo Ofertë',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Fshi Ofertën',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Fshi faturën',
      'update_client': 'Update Client',
      'delete_client': 'Fshi Klientin',
      'delete_payment': 'Fshi Pagesën',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Fshi kompaninë',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Fshi shpenzimin',
      'create_task': 'Krijo Detyrë',
      'update_task': 'Update Task',
      'delete_task': 'Fshi Detyrën',
      'approve_quote': 'Approve Quote',
      'off': 'Ndalur',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Falas',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokenët',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokenët',
      'new_token': 'New Token',
      'edit_token': 'Edito Tokenin',
      'created_token': 'Tokeni është fshirë me sukses',
      'updated_token': 'Tokeni është perditesuar me sukses',
      'archived_token': 'Tokeni është arkivuar me sukses',
      'deleted_token': 'Tokeni është fshirë me sukses',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Dërgo faturën me email',
      'email_quote': 'Dërgo me email Ofertën',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Emri i Kontaktit',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edito kushtet e pagesës',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Shuma e kredituar',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Rimburso pagesën',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'Qytet/Shtet/Poste',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Lejet',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count invoice sent',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Fshi llogarinë',
      'cancel_account_message':
          'Vërrejtje: Kjo do të fshijë të gjitha të dhënat tuaja, ky veprim nuk ka mundësi të kthehet mbrapa.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Menaxhimi i llogarive',
      'credit_date': 'Data e kreditit',
      'credit': 'Kredi',
      'credits': 'Kredi',
      'new_credit': 'Enter Credit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Krediti është krijuar me sukses',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Krediti është arkivuar me sukses',
      'deleted_credit': 'Krediti është fshirë me sukses',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Krediti është rikhyer me sukses',
      'archived_credits': ':count kredite janë arkivuar me sukses',
      'deleted_credits': ':kredi janë fshirë me sukses',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versioni aktual',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Mëso më shumë',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Kompani e re',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reseto',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Grafik',
      'count': 'Count',
      'totals': 'Totale',
      'blank': 'Bosh',
      'day': 'Dite',
      'month': 'Muaj',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Grupo sipas',
      'credit_balance': 'Bilanci i kreditit',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'ID e klientit',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Raporte',
      'report': 'Raport',
      'add_company': 'Shto Kompani',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Ndihmë',
      'refund': 'Rimburso',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Mesazhi',
      'from': 'Nga',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentim',
      'contact_us': 'Contact Us',
      'subtotal': 'Nëntotali',
      'line_total': 'Totali i linjës',
      'item': 'Njësi',
      'credit_email': 'Credit Email',
      'iframe_url': 'Webfaqja',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Po',
      'no': 'Jo',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Shiko',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Përdorues',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Ju lutem zgjedhni një klient',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Rregullimet e Taksave',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Kalo',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Riktheni fjalëkalimin tuaj',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Orari',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Emaili i Faturës',
      'payment_email': 'Emaili i Pagesës',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Emaili i Ofertës',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Lejon përdoruesit të menaxhoj përdoruesit, të ndryshojë rregullimet dhe të modifikojë të gjitha shënimet.',
      'user_management': 'Menaxhimi i përdoruesve',
      'users': 'Përdorues',
      'new_user': 'Përdorues i ri',
      'edit_user': 'Edito përdoruesin',
      'created_user': 'Successfully created user',
      'updated_user': 'Përdoruesi është perditesuar me sukses',
      'archived_user': 'Përdoruesi është arkivuar me sukses',
      'deleted_user': 'Përdoruesi është fshirë me sukses',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Përdoruesi është rikthyer me sukses',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Rregullimet Gjenerale',
      'invoice_options': 'Opsionet e faturës',
      'hide_paid_to_date': 'Fshihe Paguar deri më tash',
      'hide_paid_to_date_help':
          'Shfaqni \"Paguar deri më tash\" në faturat tuaja pasi të jetë pranuar pagesa.',
      'invoice_embed_documents': 'Dokumentet e lidhura',
      'invoice_embed_documents_help': 'Vendos fotografinë në faturë.',
      'all_pages_header': 'Shfaqe Header',
      'all_pages_footer': 'Shfaqe Footer',
      'first_page': 'Faqja e parë',
      'all_pages': 'Të gjitha faqet',
      'last_page': 'Faqja e fundit',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Ngjyra kryesore',
      'secondary_color': 'Ngjyra dytësore',
      'page_size': 'Madhësia e faqes',
      'font_size': 'Madhësia e fontit',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Fushat e faturës',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Kushtet e faturës',
      'invoice_footer': 'Footer i Faturës',
      'quote_terms': 'Kushtet e Ofertave',
      'quote_footer': 'Footer i Ofertës',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatikisht konverto ofertën në faturë kur pranohet nga klienti.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Javore',
      'freq_two_weeks': 'Dy javore',
      'freq_four_weeks': 'Katër javore',
      'freq_monthly': 'Mujore',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Tre mujore',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Gjashtë mujore',
      'freq_annually': 'Vjetore',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Asnjëherë',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Vendos taksat',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefiks',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'CSS i ndryshushëm',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Ju mundëson të vendosni fjalëkalim për secilin kontakt. Nëse vendoset fjalëkalimi, kontakti duhet të vendos fjalëkalimin para se t\'i sheh faturat.',
      'authorization': 'Authorization',
      'subdomain': 'Subdomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Përshëndetje',
      'enable_email_markup_help':
          'Bëjeni më të lehtë për klientët tuaj të realizojnë pagesat duke vendosur schema.org markimin në emailat tuaj.',
      'plain': 'E thjeshtë',
      'light': 'E lehtë',
      'dark': 'E mbylltë',
      'email_design': 'Dizajno emailin',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Aktivizo Markimin',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Kredit kartë',
      'bank_transfer': 'Transfer bankar',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Aktivizo min',
      'enable_max': 'Aktivizo max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Perditeso Adresën',
      'update_address_help':
          'Perditeso adresën e klientit me detajet e ofruara',
      'rate': 'Norma',
      'tax_rate': 'Norma e taksave',
      'new_tax_rate': 'Normë e re e taksave',
      'edit_tax_rate': 'Edito normën e taksës',
      'created_tax_rate': 'Norma e taksës është krijuar me sukses',
      'updated_tax_rate': 'Norma e taksës është perditesuar me sukses',
      'archived_tax_rate': 'Norma e taksës është arkivuar me sukses',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Plotëso-automatikisht produktet',
      'fill_products_help':
          'Duke zgjedhur produktin, automatikisht do të plotësohen fill in the description and cost',
      'update_products': 'Perditeso-automatikisht produktet',
      'update_products_help':
          'Perditesimi i faturës automatikisht do të perditesoje librarine e produktit',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'E ç\'aktivizuar',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'E diel',
      'monday': 'E hënë',
      'tuesday': 'E marte',
      'wednesday': 'E mërkure',
      'thursday': 'E enjëte',
      'friday': 'E premte',
      'saturday': 'E shtune',
      'january': 'Janar',
      'february': 'Shkurt',
      'march': 'Mars',
      'april': 'Prill',
      'may': 'Maj',
      'june': 'Qershor',
      'july': 'Korrik',
      'august': 'Gusht',
      'september': 'Shtator',
      'october': 'Tetor',
      'november': 'Nëntor',
      'december': 'Dhjetor',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': 'Koha 24 orëshe',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Rregullimi i Produktit',
      'device_settings': 'Device Settings',
      'defaults': 'Të paracaktuara',
      'basic_settings': 'Rregullimet bazike',
      'advanced_settings': 'Rregullimi i Avansuar',
      'company_details': 'Detajet e kompanisë',
      'user_details': 'Detajet e përdoruesit',
      'localization': 'Vendore',
      'online_payments': 'Pagesat Online',
      'tax_rates': 'Normat e taksave',
      'notifications': 'Njoftimet',
      'import_export': 'Import | Export',
      'custom_fields': 'Fushat e ndryshueshme',
      'invoice_design': 'Dizajni i Faturës',
      'buy_now_buttons': 'Butonat Blej Tash',
      'email_settings': 'Rregullimi i Emailit',
      'templates_and_reminders': 'Shabllonet & Përkujtueset',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Vizualizimi i të dhënave',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Kushtet e shërbimit',
      'privacy_policy': 'Politika e Privatësisë',
      'sign_up': 'Regjistrohu',
      'account_login': 'Hyrja me llogari',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Krijo',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Shkarko',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Dokumente',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Data e shpenzimit',
      'pending': 'Në pritje',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Konvertuar',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Kursi i këmbimit',
      'convert_currency': 'Konverto valutën',
      'mark_paid': 'Mark Paid',
      'category': 'Kategoria',
      'address': 'Adresa',
      'new_vendor': 'Kompani e re',
      'created_vendor': 'Kompania është krijuar me sukses',
      'updated_vendor': 'Kompania është perditesuar me sukses',
      'archived_vendor': 'Kompania është arkivuar me sukses',
      'deleted_vendor': 'Kompania është fshirë me sukses',
      'restored_vendor': 'Kompania u rikthye me sukses',
      'archived_vendors': ':counts kompani janë arkivuar me sukses',
      'deleted_vendors': ':count kompani janë fshirë me sukses',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Enter Expense',
      'created_expense': 'Shpenzimi është krijuar me sukses',
      'updated_expense': 'Shpenzimi është perditesuar me sukses',
      'archived_expense': 'Shpenzimi është arkivuar me sukses',
      'deleted_expense': 'Shpenzimi është fshirë me sukses',
      'restored_expense': 'Shpenzimet janë rikthyer me sukses',
      'archived_expenses': 'Shpenzimet janë arkivuar me sukses',
      'deleted_expenses': 'Shpenzimet janë fshirë me sukses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Faturuar',
      'logged': 'Regjistruar',
      'running': 'Duke ndodhur',
      'resume': 'Vazhdo',
      'task_errors': 'Ju lutem korrigjoni kohët e vendosura mbi njëra-tjetrën',
      'start': 'Fillo',
      'stop': 'Ndalo',
      'started_task': 'Successfully started task',
      'stopped_task': 'Detyra është ndaluar me sukses',
      'resumed_task': 'Successfully resumed task',
      'now': 'Tash',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Kohëmatësi',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Koha e fillimit',
      'end_time': 'Koha e përfundimit',
      'date': 'Data',
      'times': 'Kohët',
      'duration': 'Kohëzgjatja',
      'new_task': 'Detyrë e re',
      'created_task': 'Detyra u krijua me sukses',
      'updated_task': 'Detyra është perditesuar me sukses',
      'archived_task': 'Detyra është arkivuar me sukses',
      'deleted_task': 'Detyra është fshirë me sukses',
      'restored_task': 'Detyra është rikthyer me sukses',
      'archived_tasks': ':count detyra janë arkivuar me sukses',
      'deleted_tasks': ':count detyra janë fshirë me sukses',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'kliko këtu',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Footer',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Shtrirja e Dates',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'E ndryshueshme',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Shiko Faturën',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Edito klientin',
      'edit_product': 'Edito produkt',
      'edit_invoice': 'Edito Faturën',
      'edit_quote': 'Edito Ofertën',
      'edit_payment': 'Edito Pagesën',
      'edit_task': 'Edito Detyrën',
      'edit_expense': 'Edito shpenzimi',
      'edit_vendor': 'Edito kompaninë',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Adresa e faturimit',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Totali i Qarkullimit',
      'average_invoice': 'Mesatarja e faturës',
      'outstanding': 'Pa paguar1',
      'invoices_sent': ':count invoices sent',
      'active_clients': 'klientë aktiv',
      'close': 'Mbyll',
      'email': 'Emaili',
      'password': 'Fjalëkalimi',
      'url': 'URL',
      'secret': 'Sekret',
      'name': 'Emri',
      'logout': 'Ç\'identifikohu',
      'login': 'Identifikohu',
      'filter': 'Filtro',
      'sort': 'Sort',
      'search': 'Kërko',
      'active': 'Aktiv',
      'archived': 'Arkivuar',
      'deleted': 'E fshirë',
      'dashboard': 'Paneli',
      'archive': 'Arkivo',
      'delete': 'Fshi',
      'restore': 'Rikthe',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Ruaj',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Paguar deri më sot',
      'balance_due': 'Bilanci aktual',
      'balance': 'Bilanci',
      'overview': 'Overview',
      'details': 'Detajet',
      'phone': 'Telefoni',
      'website': 'Website',
      'vat_number': 'Numri i TVSH',
      'id_number': 'ID numri',
      'create': 'Krijo',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontaktet',
      'additional': 'Additional',
      'first_name': 'Emri',
      'last_name': 'Mbiemri',
      'add_contact': 'Shto kontaktin',
      'are_you_sure': 'A jeni të sigurtë',
      'cancel': 'Anulo',
      'ok': 'Ok',
      'remove': 'Largo',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produktet',
      'new_product': 'Produkt i ri',
      'created_product': 'Produkti është krijuar me sukses',
      'updated_product': 'Produkti është perditesuar me sukses',
      'archived_product': 'Produkti është arkivuar me sukses',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produkt',
      'notes': 'Shënime',
      'cost': 'Kosto',
      'client': 'Klient',
      'clients': 'Klientët',
      'new_client': 'Klient i ri',
      'created_client': 'Klienti është krijuar me sukses',
      'updated_client': 'Klienti është perditesuar me sukses',
      'archived_client': 'Klienti është arkivuar me sukses',
      'archived_clients': ':count klientë janë arkivuar me sukses',
      'deleted_client': 'Klienti është fshirë me sukses',
      'deleted_clients': ':count klientë janë fshirë me sukses',
      'restored_client': 'Klienti është rikthyer me sukses',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Rruga',
      'address2': 'Apartamenti/banesa',
      'city': 'Qyteti',
      'state': 'Shteti/Provinca',
      'postal_code': 'Kodi postar',
      'country': 'Shteti',
      'invoice': 'Fatura',
      'invoices': 'Faturat',
      'new_invoice': 'Faturë e re',
      'created_invoice': 'Fatura është krijuar me sukses',
      'updated_invoice': 'Fatura është perditesuar me sukses',
      'archived_invoice': 'Fatura është arkivuar me sukses',
      'deleted_invoice': 'Fatura është fshirë me sukses',
      'restored_invoice': 'Fatura është rikthyer me sukses',
      'archived_invoices': ':count fatura janë arkivuar me sukes',
      'deleted_invoices': ':count fatura janë fshirë me sukses',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Fatura është dërguar me sukses me email',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Shuma',
      'invoice_number': 'Numri i faturës',
      'invoice_date': 'Data e faturës',
      'discount': 'Zbritje',
      'po_number': 'Numri UB',
      'terms': 'Kushtet',
      'public_notes': 'Shënime publike',
      'private_notes': 'Shënime private',
      'frequency': 'Frekuenca',
      'start_date': 'Data e fillimit',
      'end_date': 'Data e përfundimit',
      'quote_number': 'Numri i ofertës',
      'quote_date': 'Data e Ofertës',
      'valid_until': 'Valide deri',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Përshkrimi',
      'unit_cost': 'Kosto për njësi',
      'quantity': 'Sasia',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefoni',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Deri më datë',
      'partial_due_date': 'Partial Due Date',
      'status': 'Statusi',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Totali',
      'percent': 'Percent',
      'edit': 'Edito',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Rregullimet',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Taksë',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Dërguar',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'E pjesshme/depozite',
      'paid': 'Paguar',
      'mark_sent': 'Shenja është dërguar',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Përfundo',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Modeli i errët',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Aktiviteti',
      'no_records_found': 'No records found',
      'clone': 'Klono',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Kushtet e pagesës',
      'payment_date': 'Data e pagesës',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portali i klientit',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Aktivizuar',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'Përkujtuesi i parë',
      'second_reminder': 'Përkujtuesi i dytë',
      'third_reminder': 'Përkujtuesi i tretë',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Tema',
      'body': 'Përmbajtja',
      'send_email': 'Dërgo email',
      'email_receipt': 'Dërgo fletëpagesën tek klienti me email',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Parashiko',
      'customize': 'Ndrysho',
      'history': 'Historia',
      'payment': 'Pagesa',
      'payments': 'Pagesat',
      'refunded': 'Refunded',
      'payment_type': 'Lloji i pagesës',
      'transaction_reference': 'Referenca e transaksionit',
      'enter_payment': 'Cakto pagesën',
      'new_payment': 'Enter Payment',
      'created_payment': 'Pagesa është krijuar me sukses',
      'updated_payment': 'Pagesa është perditesuar me sukses',
      'archived_payment': 'Pagesa është arkivuar me sukses',
      'deleted_payment': 'Pagesa është fshirë me sukses',
      'restored_payment': 'Pagesa është rikthyer me sukses',
      'archived_payments': ':count pagesa janë arkivuar me sukses',
      'deleted_payments': ':count pagesa janë fshirë me sukses',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Ofertë',
      'quotes': 'Oferta',
      'new_quote': 'Ofertë e re',
      'created_quote': 'Oferta është krijuar me sukses',
      'updated_quote': 'Oferta është perditesuar me sukses',
      'archived_quote': 'Oferta është arkivuar me sukses',
      'deleted_quote': 'Oferta është fshirë me sukses',
      'restored_quote': 'Oferta është rikthyer me sukses',
      'archived_quotes': ': count oferta janë arkivuar me sukses',
      'deleted_quotes': ':count oferta janë fshirë me sukses',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Shpenzimet',
      'expenses': 'Shpenzimet',
      'vendor': 'Kompani',
      'vendors': 'Kompanitë',
      'task': 'Detyre',
      'tasks': 'Detyrat',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user ka krijuar klientin :client',
      'activity_2': ':user ka arkivuar klientin :client',
      'activity_3': ':user ka fshirë klientin :client',
      'activity_4': ':user ka krijuar faturën :invoice',
      'activity_5': ':user ka perditesuar faturën :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user ka arkivuar faturën :invoice',
      'activity_9': ':user ka fshirë faturën :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user ka perditesuar pagesën :payment',
      'activity_12': ':user ka arkivuar pagesën :payment',
      'activity_13': ':user ka fshirë pagesën :payment',
      'activity_14': ':user ka shtuar :credit kredit',
      'activity_15': ':user ka perditesuar :credit kredit',
      'activity_16': ':user ka arkivuar :credit kredit',
      'activity_17': ':user ka fshirë:credit kredit',
      'activity_18': ':user ka krijuar ofertë :quote',
      'activity_19': ':user ka perditesuar ofertën :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact ka shikuar ofertën :quote',
      'activity_22': ':user ka arkivuar ofertën :quote',
      'activity_23': ':user ka fshirë ofertën :quote',
      'activity_24': ':user ka rikthyer ofertën :quote',
      'activity_25': ':user ka rikthyer faturën :invoice',
      'activity_26': ':user ka rikthyer klientin :client',
      'activity_27': ':user ka rikthyer pagesën :payment',
      'activity_28': ':user ka rikthyer :credit kredit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user ka krijuar shpeznim :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) ka dështuar',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Oferta është dërguar me sukses me email',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Skaduar',
      'all': 'Të gjitha',
      'select': 'Selekto',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Numruesi i numrit të faturës',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Numruesi i numrit të ofertës',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Shkruaj',
      'invoice_amount': 'Shuma e faturës',
      'invoice_due_date': 'Deri më datë',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Faturo Automatikisht',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Emri i taksës',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Shuma e paguar',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Banka',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'bg': {
      'import_data': 'Импорт на данни',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Последно актуализиран',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Контакт - Име',
      'contact_last_name': 'Контакт - Фамилия',
      'order': 'Order',
      'unassigned': 'Неприсвоена',
      'partial_value':
          'Трябва да бъде повече от 0 и по-малко от общата стойност.',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Разреши',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Заложете subdomain или показвайте фактурите на собствения си сайт.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Сигурност',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Абонаменти',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Редакция абонамент',
      'created_subscription': 'Успешно създаден абонамент',
      'updated_subscription': 'Успешно редактиран абонамент',
      'archived_subscription': 'Успешно архивиране на абонамент',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Повторно изпращане на покана',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Върнати',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Сканирайте баркода с :link съвместимо приложение.',
      'enabled_two_factor': 'Успешно активирана Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Възстановено плащане',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Предишно тримесечие',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Конвертирай като Фактура',
      'registration_url': 'Registration URL',
      'invoice_project': 'Фактуриране на проект',
      'invoice_task': 'Фактуриране на задача',
      'invoice_expense': 'Префактуриране на разход',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Конвертирана сума',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Документи по подразбиране',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Скрий',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Колона',
      'sample': 'Пример',
      'map_to': 'Map To',
      'import': 'Импорт',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Моля изберете файл',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV файл',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Услуга',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Неплатена',
      'white_label': 'White Label',
      'delivery_note': 'Документ за доставка',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Частично дължима',
      'invoice_total': 'Тотал на фактура',
      'quote_total': 'Обща стойност на офертата',
      'credit_total': 'Общ кредит',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Внимание',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Име на клиент',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Успешно променен статус на задача',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Категории разходи',
      'new_expense_category': 'Нова категория разходи',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Успешно създаване на категория разходи',
      'updated_expense_category': 'Успешна актуализация на категория разходи',
      'archived_expense_category': 'Успешно архивиране на категория разходи',
      'deleted_expense_category': 'Успешно изтрита категория',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category':
          'Успешно възстановяване на категория разходи',
      'archived_expense_categories':
          'Успешно архивирани :count категории разходи',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Трябва да се префактурира',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Маркирай в архив',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Периодична фактура',
      'recurring_invoices': 'Периодични фактури',
      'new_recurring_invoice': 'Нова периодична фактура',
      'edit_recurring_invoice': 'Редакция на периодична фактура',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Успешно архивиране на периодична фактура',
      'deleted_recurring_invoice': 'Успешно изтриване на периодична фактура',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice':
          'Успешно възстановяване на периодична фактура',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Печалба',
      'line_item': 'Ред',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Отворени',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Преглед на портала',
      'copy_link': 'Copy Link',
      'token_billing': 'Запазване на данните на картата',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Винаги',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Етикет',
      'client_number': 'Клиентски номер',
      'auto_convert': 'Auto Convert',
      'company_name': 'Име на фирма',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Успешно изпратени фактури',
      'emailed_quotes': 'Успешно изпратени оферти',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Портал',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Часове',
      'statement': 'Извлечение',
      'taxes': 'Данъци',
      'surcharge': 'Доплащане',
      'apply_payment': 'Apply Payment',
      'apply': 'Приложи',
      'unapplied': 'Unapplied',
      'select_label': 'Избор на етикет',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'До',
      'health_check': 'Health Check',
      'payment_type_id': 'Начин на плащане',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Предстоящи фактури',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Последни плащания',
      'upcoming_quotes': 'Предстоящи оферти',
      'expired_quotes': 'Изтекли оферти',
      'create_client': 'Създаване на клиент',
      'create_invoice': 'Създай фактура',
      'create_quote': 'Създай Оферта',
      'create_payment': 'Create Payment',
      'create_vendor': 'Създаване на доставчик',
      'update_quote': 'Update Quote',
      'delete_quote': 'Изтрий Оферта',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Изтриване на фактура',
      'update_client': 'Update Client',
      'delete_client': 'Изтрий клиент',
      'delete_payment': 'Изтрий плащане',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Изтриване на доставчик',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Изтриване на разход',
      'create_task': 'Нова задача',
      'update_task': 'Update Task',
      'delete_task': 'Изтриване на задача',
      'approve_quote': 'Approve Quote',
      'off': 'Изкл.',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Безплатно',
      'plan': 'План',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API токъни',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Токън',
      'tokens': 'Токъни',
      'new_token': 'New Token',
      'edit_token': 'Редакция на токън',
      'created_token': 'Успешно създаден токън',
      'updated_token': 'Успешно актуализиран токън',
      'archived_token': 'Успешно архивиран токен',
      'deleted_token': 'Успешно изтрит токън',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Изпращане на фактура',
      'email_quote': 'Изпрати оферта по имейл',
      'email_credit': 'Email Credit',
      'email_payment': 'Имейл за плащане',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Контакт - име',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Редакция на условие за плащане',
      'created_payment_term': 'Успешно създадено условие на плащане',
      'updated_payment_term': 'Успешно актуализирано условие на плащане',
      'archived_payment_term': 'Успешно архивирано условие на плащане',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Сума на кредита',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Изключени',
      'inclusive': 'Включени',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Възстановяване на плащане',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Пълно име',
      'city_state_postal': 'Град / Щат / Пощ. код',
      'postal_city_state': 'Пощ. код / Щат / Град',
      'custom1': 'Първа колона',
      'custom2': 'Втора колона',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Изчистване на данни',
      'purge_successful': 'Успешно изчистени фирмени данни',
      'purge_data_message':
          'Внимание: Това ще изтрие данните перманентно без възможност за възстановяване.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 дни',
      'age_group_30': '30 - 60 дни',
      'age_group_60': '60 - 90 дни',
      'age_group_90': '90 - 120 дни',
      'age_group_120': '120+ дни',
      'refresh': 'Опресняване',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Детайли на фактурата',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Права за достъп',
      'none': 'Няма',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count изпратена фактура',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Въведете лиценз',
      'cancel_account': 'Изтрий Профил',
      'cancel_account_message':
          'ВНИМАНИЕ: Това действие ще изтрие перманентно вашият профил и данните в него. След това данните няма как да бъдат възстановени.',
      'delete_company': 'Изтриване на фирма',
      'delete_company_message':
          'Внимание: Това ще изтрие перманентно фирматаВи без възможност за възстановяване.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Хедър',
      'load_design': 'Зареждане на дизайн',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Предложения',
      'tickets': 'Tickets',
      'recurring_quotes': 'Периодични Оферти',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Повтарящи се разходи',
      'account_management': 'Управление на акаунтите',
      'credit_date': 'Кредитна Дата',
      'credit': 'Кредит',
      'credits': 'Кредити',
      'new_credit': 'Въведи кредит',
      'edit_credit': 'Редакция на кредит',
      'created_credit': 'Успешно създаден кредит',
      'updated_credit': 'Успешна актуализация на кредит',
      'archived_credit': 'Успешно архивиран кредит',
      'deleted_credit': 'Успешно изтрит кредит',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Успешно възстановен кредит',
      'archived_credits': 'Успешно архивирани :count кредита',
      'deleted_credits': 'Успешно изтрити :count кредита',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Текуща версия',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Научи повече',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Нова фирма',
      'added_company': 'Successfully added company',
      'company1': 'Персонализирана Компания 1',
      'company2': 'Персонализирана Компания 2',
      'company3': 'Персонализирана Компания 3',
      'company4': 'Персонализирана Компания 4',
      'product1': 'Персонализиран Продукт 1',
      'product2': 'Персонализиран Продукт 2',
      'product3': 'Персонализиран Продукт 3',
      'product4': 'Персонализиран Продукт 4',
      'client1': 'Персонализиран клиент 1',
      'client2': 'Персонализиран клиент 2',
      'client3': 'Персонализиран клиент 3',
      'client4': 'Персонализиран клиент 4',
      'contact1': 'Персонализиран контакт 1',
      'contact2': 'Персонализиран контакт 2',
      'contact3': 'Персонализиран контакт 3',
      'contact4': 'Персонализиран контакт 4',
      'task1': 'Персонализирана задача 1',
      'task2': 'Персонализирана задача 2',
      'task3': 'Персонализирана задача 3',
      'task4': 'Персонализирана задача 4',
      'project1': 'Персонализиран проект 1',
      'project2': 'Персонализиран проект 2',
      'project3': 'Персонализиран проект 3',
      'project4': 'Персонализиран проект 4',
      'expense1': 'Персонализиран разход 1',
      'expense2': 'Персонализиран разход 2',
      'expense3': 'Персонализиран разход 3',
      'expense4': 'Персонализиран разход 4',
      'vendor1': 'Персонализиран доставчик 1',
      'vendor2': 'Персонализиран доставчик 2',
      'vendor3': 'Персонализиран доставчик 3',
      'vendor4': 'Персонализиран доставчик 4',
      'invoice1': 'Персонализирана фактура 1',
      'invoice2': 'Персонализирана фактура 2',
      'invoice3': 'Персонализирана фактура 3',
      'invoice4': 'Персонализирана фактура 4',
      'payment1': 'Персонализирано плащане 1',
      'payment2': 'Персонализирано плащане 2',
      'payment3': 'Персонализирано плащане 3',
      'payment4': 'Персонализирано плащане 4',
      'surcharge1': 'Персонализирано доплащане 1',
      'surcharge2': 'Персонализирано доплащане 2',
      'surcharge3': 'Персонализирано доплащане 3',
      'surcharge4': 'Персонализирано доплащане 4',
      'group1': 'Персонализирана група 1',
      'group2': 'Персонализирана група 2',
      'group3': 'Персонализирана група 3',
      'group4': 'Персонализирана група 4',
      'reset': 'Нулиране',
      'number': 'Номер',
      'export': 'Експорт',
      'chart': 'Графика',
      'count': 'Брой',
      'totals': 'Общи суми',
      'blank': 'Празно',
      'day': 'Ден',
      'month': 'Месец',
      'year': 'Година',
      'subgroup': 'Подгрупа',
      'is_active': 'Е активен',
      'group_by': 'Групиране по',
      'credit_balance': 'Баланс на кредита',
      'contact_last_login': 'Последно влизане на контакта',
      'contact_full_name': 'Пълно име на контакта',
      'contact_phone': 'Телефон за връзка',
      'contact_custom_value1': 'Персонализирана контактна стойност 1',
      'contact_custom_value2': 'Персонализирана контактна стойност 2',
      'contact_custom_value3': 'Персонализирана контактна стойност 3',
      'contact_custom_value4': 'Персонализирана контактна стойност 4',
      'shipping_address1': 'Доставка - Улица',
      'shipping_address2': 'Доставка - Ап.',
      'shipping_city': 'Доставка - Град',
      'shipping_state': 'Доставка - Щат/Провинция',
      'shipping_postal_code': 'Доставка - Пощ. код',
      'shipping_country': 'Доставка - Държава',
      'billing_address1': 'Фактура - Улица',
      'billing_address2': 'Фактура - Ап.',
      'billing_city': 'Фактура - Град',
      'billing_state': 'Фактура - Щат/Провинция',
      'billing_postal_code': 'Фактура - Пощ. код',
      'billing_country': 'Фактура - Държава',
      'client_id': 'Client Id',
      'assigned_to': 'Присвоен на',
      'created_by': 'Създаден от :name',
      'assigned_to_id': 'Присвоен на Id',
      'created_by_id': 'Създаден от Id',
      'add_column': 'Добави колона',
      'edit_columns': 'Редактирай колони',
      'columns': 'Колони',
      'aging': 'По дата на издаване',
      'profit_and_loss': 'Печалба и загуба',
      'reports': 'Справки',
      'report': 'Справка',
      'add_company': 'Добавяне на фирма',
      'unpaid_invoice': 'Неплатена фактура',
      'paid_invoice': 'Платена фактура',
      'unapproved_quote': 'Неодобрена оферта',
      'help': 'Помощ',
      'refund': 'Възстановяване',
      'refund_date': 'Дата на възтановяване',
      'filtered_by': 'Филтрирано по',
      'contact_email': 'Имейл за връзка',
      'multiselect': 'Мултиселекция',
      'entity_state': 'Щат',
      'verify_password': 'Потвърди паролата',
      'applied': 'Приложено',
      'include_recent_errors': 'Включи последните грешки от логовете',
      'your_message_has_been_received':
          'Ние получихме съобщението Ви и ще се опитаме да отговорим незабавно.',
      'message': 'Съобщение',
      'from': 'От',
      'show_product_details': 'Показване на детайлите на продукта',
      'show_product_details_help':
          'Включване на описанието и цената в падащото меню на продукта',
      'pdf_min_requirements': 'PDF рендер-а изисква :version',
      'adjust_fee_percent': 'Настойка на процент таксата',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Конфигуриране на настройките',
      'support_forum': 'Форум за поддръжка',
      'about': 'За',
      'documentation': 'Документация',
      'contact_us': 'Свържете се с нас',
      'subtotal': 'Субтотал',
      'line_total': 'Обща цена',
      'item': 'Продукт',
      'credit_email': 'Кредитен е-мейл',
      'iframe_url': 'Сайт',
      'domain_url': 'Домейн адрес',
      'password_is_too_short': 'Паролата е твърде кратка',
      'password_is_too_easy': 'Паролата трябва да съдържа главна буква и цифра',
      'client_portal_tasks': 'Задачи от клиентския портал',
      'client_portal_dashboard': 'Табло на клиентския портал',
      'please_enter_a_value': 'Моля въведете стойност',
      'deleted_logo': 'Успешно изтрито лого',
      'yes': 'Да',
      'no': 'Не',
      'generate_number': 'Генерирай номер',
      'when_saved': 'е запазена',
      'when_sent': 'е изпратена',
      'select_company': 'Избери компания',
      'float': 'Плаващ',
      'collapse': 'Събери',
      'show_or_hide': 'Покажи/Скрий',
      'menu_sidebar': 'Меню на страничната лента',
      'history_sidebar': 'История на страничната лента',
      'tablet': 'Таблет',
      'mobile': 'Мобилно',
      'desktop': 'Десктоп',
      'layout': 'Оформление',
      'view': 'Преглед',
      'module': 'Модул',
      'first_custom': 'Първо персонализиране',
      'second_custom': 'Второ персонализиране',
      'third_custom': 'Трето персонализиране',
      'show_cost': 'Показване на цена',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Покажи цената на продуктовото поле за да проследиш печалбата',
      'show_product_quantity': 'Показване на продуктовата наличност',
      'show_product_quantity_help':
          'Покажи полето за наличност на продукта, или ще се използва стандартното поле',
      'show_invoice_quantity': 'Покажи броят на фактурите',
      'show_invoice_quantity_help':
          'Покажи реда за наличност, или ще се използва стандартен ред',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Наличност по подразбиране',
      'default_quantity_help':
          'Автоматично слагане на линията за наличност на едно',
      'one_tax_rate': 'Една данъчна ставка',
      'two_tax_rates': 'Две данъчни ставки',
      'three_tax_rates': 'Три данъчни ставки',
      'default_tax_rate': 'Данъчна ставка по подразбиране',
      'user': 'Потребител',
      'invoice_tax': 'Такса за фактура',
      'line_item_tax': 'Данък върху договорената покупка',
      'inclusive_taxes': 'Включени такси',
      'invoice_tax_rates': 'Данъчни ставки на фактури',
      'item_tax_rates': 'Данъчни ставки',
      'no_client_selected': 'Моля, изберете клиент',
      'configure_rates': 'Конфигуриране на тарифите',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Настройки за данък',
      'tax_settings_rates': 'Размер на данъци',
      'accent_color': 'Акцентен цвят',
      'switch': 'Прехвърляне',
      'comma_sparated_list': 'Лист разделен със запетаи',
      'options': 'Настройки',
      'single_line_text': 'Едноредов текст',
      'multi_line_text': 'Многоредов текст',
      'dropdown': 'Падащо меню',
      'field_type': 'Вид поле',
      'recover_password_email_sent':
          'Изпратен е e-mail за възстановяване на парола',
      'submit': 'Изпращане',
      'recover_password': 'Възстановяване на парола',
      'late_fees': 'Закъснели Такси',
      'credit_number': 'Кредит номер',
      'payment_number': 'Номер на плащане',
      'late_fee_amount': 'Стойност на таксата за закъснение',
      'late_fee_percent': 'Процент на таксата за закъснение',
      'schedule': 'График',
      'before_due_date': 'Преди крайната дата',
      'after_due_date': 'След крайната дата',
      'after_invoice_date': 'След датата на фактурата',
      'days': 'Дни',
      'invoice_email': 'Имейл за фактура',
      'payment_email': 'Имейл за плащане',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Имейл за оферта',
      'endless_reminder': 'Безкрайно подсещане',
      'filtered_by_user': 'Филтрирано по Потребител',
      'administrator': 'Администратор',
      'administrator_help':
          'Даване на права на потребителя да управлява другите потребители, да променя настойки и да редактира всички записи',
      'user_management': 'Управление на потребителите',
      'users': 'Потребители',
      'new_user': 'Нов потребител',
      'edit_user': 'Редактирай Потребител',
      'created_user': 'Успешно създаден потребител',
      'updated_user': 'Успешно обновен потребител',
      'archived_user': 'Успешно архивиране на потребител',
      'deleted_user': 'Успешно изтрит Потребител',
      'removed_user': 'Потребителя е премахнат успешно',
      'restored_user': 'Успешно възстановен Потребител',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Общи настройки',
      'invoice_options': 'Опции на фактурата',
      'hide_paid_to_date': 'Скрий \"Изплатено до момента\"',
      'hide_paid_to_date_help':
          'Покажи \"Изплатено до момента\" във фактурите, след като е получено плащане.',
      'invoice_embed_documents': 'Свързани документи',
      'invoice_embed_documents_help':
          'Включване на прикачените изображения във фактурата.',
      'all_pages_header': 'Показване на хедъра на',
      'all_pages_footer': 'Показване на футъра на',
      'first_page': 'Първа страница',
      'all_pages': 'Всички страници',
      'last_page': 'Последна страница',
      'primary_font': 'Основен Шрифт',
      'secondary_font': 'Допълнителен Шрифт',
      'primary_color': 'Основен цвят',
      'secondary_color': 'Допълнителен цвят',
      'page_size': 'Размер на страницата',
      'font_size': 'Размер на шрифта',
      'quote_design': 'Дизайн на оферта',
      'invoice_fields': 'Полета на фактурата',
      'product_fields': 'Продуктови полета',
      'invoice_terms': 'Условия на фактура',
      'invoice_footer': 'Футър на фактурата',
      'quote_terms': 'Условия на оферта',
      'quote_footer': 'Футър на оферта',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Автоматично изпращане на периодични фактури при създаването им',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Автоматично архивиране на фактури при плащането им',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Автоматично архивиране на оферти при конвертирането им',
      'auto_convert_quote': 'Автоматично конвертиране',
      'auto_convert_quote_help':
          'Автоматично конвертиране на оферта във фактура при одобрение от клиента.',
      'workflow_settings': 'Настройки на работния процес',
      'freq_daily': 'Ежедневно',
      'freq_weekly': 'Седмично',
      'freq_two_weeks': 'Две седмици',
      'freq_four_weeks': 'Четири седмици',
      'freq_monthly': 'Месечно',
      'freq_two_months': 'Два месеца',
      'freq_three_months': 'Три месеца',
      'freq_four_months': 'Четири месеца',
      'freq_six_months': 'Шест месеца',
      'freq_annually': 'Годишно',
      'freq_two_years': 'На две години',
      'freq_three_years': 'Три години',
      'never': 'Никога',
      'company': 'Фирма',
      'generated_numbers': 'Генерирани номера',
      'charge_taxes': 'Начисли данъци',
      'next_reset': 'Следващо нулиране',
      'reset_counter': 'Нулиране на брояч',
      'recurring_prefix': 'Префикс за периодични документи',
      'number_padding': 'Номер на отстояние',
      'general': 'Общ',
      'surcharge_field': 'Етикет допълнителна такса',
      'company_field': 'Поле на Фирмата',
      'company_value': 'Фирмена стойност',
      'credit_field': 'Кредитно поле',
      'invoice_field': 'Поле на фактурата',
      'invoice_surcharge': 'Допълнителна такса по фактура',
      'client_field': 'Поле на клиента',
      'product_field': 'Поле на продукта',
      'payment_field': 'Поле за плащане',
      'contact_field': 'Поле за контакти',
      'vendor_field': 'Поле на доставчика',
      'expense_field': 'Поле Разход',
      'project_field': 'Поле на проекта',
      'task_field': 'Поле Задача',
      'group_field': 'Поле за група',
      'number_counter': 'Брояч на номера',
      'prefix': 'Префикс',
      'number_pattern': 'Модел номер',
      'messages': 'Съобщения',
      'custom_css': 'Персонализиран CSS',
      'custom_javascript': 'Персонализиран JavaScript',
      'signature_on_pdf': 'Показване в PDF документа',
      'signature_on_pdf_help':
          'Показване на подписа на клиента в PDF фактурата / офертата.',
      'show_accept_invoice_terms': 'Чек-бокс за условия на фактура',
      'show_accept_invoice_terms_help':
          'Изискване клиенът да потвърди, че приема условията на фактурата',
      'show_accept_quote_terms': 'Чек-бокс за условия на офертата',
      'show_accept_quote_terms_help':
          'Изискване клиенът да потвърди, че приема условията на офертата',
      'require_invoice_signature': 'Подпис на фактурата',
      'require_invoice_signature_help': 'Изискване клиентът да подпише',
      'require_quote_signature': 'Подпис на офертата',
      'enable_portal_password': 'Защита на фактурите с парола',
      'enable_portal_password_help':
          'Дава възможност да заложите парола за всеки контакт. Ако такава е заложена, контактното лице ще трябва да я въведе преди да види фактурите,',
      'authorization': 'Оторизация',
      'subdomain': 'Subdomain',
      'domain': 'Домейн',
      'portal_mode': 'Портален режим',
      'email_signature': 'Поздрави,',
      'enable_email_markup_help':
          'Направете плащането към Вас по-лесно за клиентите си като добавите в имейлите си schema.org markup.',
      'plain': 'Изчистено',
      'light': 'Светло',
      'dark': 'Тъмно',
      'email_design': 'Дизайн на имейл',
      'attach_pdf': 'Прикачване на PDF',
      'attach_documents': 'Прикачване на Документи',
      'attach_ubl': 'Прикачване на UBL',
      'email_style': 'Стилове на имейла',
      'enable_email_markup': 'Активиране на Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Обработен',
      'credit_card': 'Кредитна карта',
      'bank_transfer': 'Банков трансфер',
      'priority': 'Приоритет',
      'fee_amount': 'Сума на таксата',
      'fee_percent': 'Процент такса',
      'fee_cap': 'Ограничение за такса',
      'limits_and_fees': 'Лимити/Такси',
      'enable_min': 'Активиране на min',
      'enable_max': 'Активиране на max',
      'min_limit': 'Мин.: :min',
      'max_limit': 'Макс.: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Лога на приемани карти',
      'credentials': 'Удостоверение за самоличност',
      'update_address': 'Актуализация на адреса',
      'update_address_help':
          'Актуализация на адреса на клиента с предоставените данни',
      'rate': 'Размер',
      'tax_rate': 'Данъчна ставка',
      'new_tax_rate': 'Нова такса',
      'edit_tax_rate': 'Редакция на данъчна ставка',
      'created_tax_rate': 'Успешно създадена данъчна ставка',
      'updated_tax_rate': 'Успешно актуализирана данъчна ставка',
      'archived_tax_rate': 'Успешно архивиране на данъчна ставка',
      'deleted_tax_rate': 'Успешно изтрита данъчна ставка',
      'restored_tax_rate': 'Успешно възтановена данъчна ставка',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Автоматично попълвай продукти',
      'fill_products_help':
          'Избирането на продукт автоматично ще попълни описанието и цената',
      'update_products': 'Автоматично обнови продукти',
      'update_products_help':
          'Промяната на фактура автоматично ще обнови продуктовия каталог',
      'convert_products': 'Конвертиране на продукти',
      'convert_products_help':
          'Автоматично конвертиране на цените на продуктите във валутата на клиента',
      'fees': 'Такси',
      'limits': 'Лимити',
      'provider': 'Доставчик',
      'company_gateway': 'Портал за плащане',
      'company_gateways': 'Портали за плащане',
      'new_company_gateway': 'Нов портал',
      'edit_company_gateway': 'Редактиране на портал',
      'created_company_gateway': 'Успешно създаден портал',
      'updated_company_gateway': 'Успешно актуализиран портал',
      'archived_company_gateway': 'Успешно архивиране на портал',
      'deleted_company_gateway': 'Успешно изтрит портал',
      'restored_company_gateway': 'Успешно възстановен портал',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Продължете редактирането',
      'discard_changes': 'Отмяна на промените',
      'default_value': 'Стойност по подразбиране',
      'disabled': 'Неактивно',
      'currency_format': 'Формат на валутата',
      'first_day_of_the_week': 'Първи ден от седмицата',
      'first_month_of_the_year': 'Първи месец на годината',
      'sunday': 'неделя',
      'monday': 'понеделник',
      'tuesday': 'вторник',
      'wednesday': 'сряда',
      'thursday': 'четвъртък',
      'friday': 'петък',
      'saturday': 'събота',
      'january': 'Януари',
      'february': 'Февруари',
      'march': 'Март',
      'april': 'Април',
      'may': 'Май',
      'june': 'Юни',
      'july': 'Юли',
      'august': 'Август',
      'september': 'Септември',
      'october': 'Октомври',
      'november': 'Ноември',
      'december': 'Декември',
      'symbol': 'Символ',
      'ocde': 'Код',
      'date_format': 'Формат на датата',
      'datetime_format': 'Формат за дата',
      'military_time': '24 часово време',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Изпращане на напомняния',
      'timezone': 'Часова зона',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Филтрирано по група',
      'filtered_by_invoice': 'Филтрирано по фактура',
      'filtered_by_client': 'Филтрирано по клиент',
      'filtered_by_vendor': 'Филтрирано по доставчик',
      'group_settings': 'Настройки на групата',
      'group': 'Групиране',
      'groups': 'Групи',
      'new_group': 'Нова Група',
      'edit_group': 'Редактира не Група',
      'created_group': 'Групата беше създадена успешно',
      'updated_group': 'Групата беше обновена успешно',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Качване на Лого',
      'uploaded_logo': 'Логото беше качено успешно',
      'logo': 'Лого',
      'saved_settings': 'Настройките бяха записани успешно',
      'product_settings': 'Настройки на продукти',
      'device_settings': 'Настройка на Устройство',
      'defaults': 'По подразбиране',
      'basic_settings': 'Основни настройки',
      'advanced_settings': 'Разширени настройки',
      'company_details': 'Данни на фирма',
      'user_details': 'Данни за потребителя',
      'localization': 'Локализация',
      'online_payments': 'Онлайн плащания',
      'tax_rates': 'Размер на данъци',
      'notifications': 'Известия',
      'import_export': 'Импорт | Експорт',
      'custom_fields': 'Собствени полета',
      'invoice_design': 'Дизайн на фактура',
      'buy_now_buttons': 'Бутони \"Купи сега\"',
      'email_settings': 'Email настройки',
      'templates_and_reminders': 'Шаблони и напомняния',
      'credit_cards_and_banks': 'Кредитни Карти & Банки',
      'data_visualizations': 'Визуализация на данни',
      'price': 'Цена',
      'email_sign_up': 'Регистрация с E-mail',
      'google_sign_up': 'Регистрация с Google',
      'thank_you_for_your_purchase': 'Благодарим за покупката!',
      'redeem': 'Осребряване',
      'back': 'Назад',
      'past_purchases': 'Минали покупки',
      'annual_subscription': 'Годишен абонамен',
      'pro_plan': 'Pro Абонамент',
      'enterprise_plan': 'Enterprise План',
      'count_users': ':count потребители',
      'upgrade': 'Обнови',
      'please_enter_a_first_name': 'Моля въведете собствено име',
      'please_enter_a_last_name': 'Моля въведете фамилно име',
      'please_agree_to_terms_and_privacy':
          'Моля съгласете се с общите условия и политиката за поверителност за да създадете профил.',
      'i_agree_to_the': 'Съгласявам се с',
      'terms_of_service_link': 'условията за ползване',
      'privacy_policy_link': 'политиката за поверителност',
      'terms_of_service': 'Условия за ползване',
      'privacy_policy': 'Политика за защита на личните данни',
      'sign_up': 'Регистрация',
      'account_login': 'Вход в профила',
      'view_website': 'Виж уебсайт',
      'create_account': 'Създай Профил',
      'email_login': 'Влизане зрез email',
      'create_new': 'Нов',
      'no_record_selected': 'Няма избрани записи',
      'error_unsaved_changes': 'Моля запазете или откажете промените',
      'download': 'Сваляне',
      'requires_an_enterprise_plan': 'Изисква \"Enterprise\" абонамент',
      'take_picture': 'Направи Снимка',
      'upload_file': 'Качване на Файл',
      'document': 'Документ',
      'documents': 'Документи',
      'new_document': 'Нов Документ',
      'edit_document': 'Редакция на Документ',
      'uploaded_document': 'Документът беше качен успешно',
      'updated_document': 'Документът беше обновен успешно',
      'archived_document': 'Документът беше архивиран успешно',
      'deleted_document': 'Документът беше изтрит успешно',
      'restored_document': 'Документът беше възстановен успешно',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Няма История',
      'expense_date': 'Дата разход',
      'pending': 'Очаквано',
      'expense_status_1': 'Логнат',
      'expense_status_2': 'Изчакващи',
      'expense_status_3': 'Фактурирано',
      'converted': 'Конвертирано',
      'add_documents_to_invoice': 'Добавяне на документ към фактура',
      'exchange_rate': 'Курс',
      'convert_currency': 'Конвертиране на валутата',
      'mark_paid': 'Маркирай платено',
      'category': 'Категория',
      'address': 'Адрес',
      'new_vendor': 'Нов доставчик',
      'created_vendor': 'Успешно създаден доставчик',
      'updated_vendor': 'Успешно обновен доставчик',
      'archived_vendor': 'Успешно архивиран доставчик',
      'deleted_vendor': 'Успешно изтрит доставчик',
      'restored_vendor': 'Успешно възстановяване на доставчик',
      'archived_vendors': 'Успешно архивирани :count доставчици',
      'deleted_vendors': 'Успешно изтрити :count доставчици',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Въведете разход',
      'created_expense': 'Успешно създаден разход',
      'updated_expense': 'Успешна актуализация на разход',
      'archived_expense': 'Успешно архивиран разход',
      'deleted_expense': 'Успешно изтрит разход',
      'restored_expense': 'Успешно възстановяване на разход',
      'archived_expenses': 'Успешно архивирани разходи',
      'deleted_expenses': 'Успешно изтрити разходи',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Копиране на адрес за доставка',
      'copy_billing': 'Копиране на адрес за фактуриране',
      'design': 'Дизайн',
      'failed_to_find_record': 'Записът не е намерен',
      'invoiced': 'Фактурирано',
      'logged': 'Логвано',
      'running': 'Стартирано',
      'resume': 'Продължаване',
      'task_errors': 'Моля, коригирайте припокриващите се интервали',
      'start': 'Старт',
      'stop': 'Стоп',
      'started_task': 'Успешно стартирана задача',
      'stopped_task': 'Успешно спряна задача',
      'resumed_task': 'Успешно възобновяване на работата по задачата',
      'now': 'Сега',
      'auto_start_tasks': 'Автоматично стартиране на задачите',
      'timer': 'Таймер',
      'manual': 'Ръчно',
      'budgeted': 'Бюджетирано',
      'start_time': 'Начало',
      'end_time': 'Край',
      'date': 'Дата',
      'times': 'Време',
      'duration': 'Продължителност',
      'new_task': 'Нова задача',
      'created_task': 'Успешно създадена задача',
      'updated_task': 'Успешно актуализирана задача',
      'archived_task': 'Успешно архивирана задача',
      'deleted_task': 'Успешно изтрита задача',
      'restored_task': 'Успешно възстановена задача',
      'archived_tasks': 'Успешно архивирани :count задачи',
      'deleted_tasks': 'Успешно изтрити :count задачи',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Моля, въведете име',
      'budgeted_hours': 'Часове по бюджет',
      'created_project': 'Успешно създаден проект',
      'updated_project': 'Успешна актуализация на проект',
      'archived_project': 'Успешно архивиран проект',
      'deleted_project': 'Успешно изтрит проект',
      'restored_project': 'Успешно възстановен проект',
      'archived_projects': 'Успешно архивирани :count проекта',
      'deleted_projects': 'Успешно изтрити :count проекта',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Нов проект',
      'thank_you_for_using_our_app':
          'Благодарим Ви, че използвате нашето приложение!',
      'if_you_like_it': 'Ако го харесвате Ви молим',
      'click_here': 'натиснете тук',
      'click_here_capital': 'Натиснете тук',
      'to_rate_it': 'да го оцените.',
      'average': 'Средно',
      'unapproved': 'Неодобрено',
      'authenticate_to_change_setting':
          'Моля, влезте в профила си за промяна на тази настойка',
      'locked': 'Блокирано',
      'authenticate': 'Вход в профила',
      'please_authenticate': 'Моля, влезте в профила си',
      'biometric_authentication': 'Биометричен вход',
      'footer': 'Футър',
      'compare': 'Сравнение',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Днес',
      'custom_range': 'Друг период',
      'date_range': 'Период',
      'current': 'Настоящ',
      'previous': 'Предишен',
      'current_period': 'Настоящ период',
      'comparison_period': 'Период за сравнение',
      'previous_period': 'Предишен период',
      'previous_year': 'Предишна година',
      'compare_to': 'Сравнение със',
      'last7_days': 'Последни 7 дни',
      'last_week': 'Предишна седмица',
      'last30_days': 'Последни 30 дни',
      'this_month': 'Този месец',
      'last_month': 'Предишния месец',
      'this_year': 'Тази година',
      'last_year': 'Предишна година',
      'custom': 'Custom',
      'clone_to_invoice': 'Копирай във фактура',
      'clone_to_quote': 'Копирай в оферта',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Прегледай Фактура',
      'convert': 'Конвертирай',
      'more': 'Още',
      'edit_client': 'Редактирай клиент',
      'edit_product': 'Редактирай продукт',
      'edit_invoice': 'Редактирай фактура',
      'edit_quote': 'Редактирай Оферта',
      'edit_payment': 'Редактирай Плащане',
      'edit_task': 'Редакция на задача',
      'edit_expense': 'Редакция на разход',
      'edit_vendor': 'Редакция на доставчик',
      'edit_project': 'Редакция на проект',
      'edit_recurring_expense': 'Редакция на повтарящ се разход',
      'edit_recurring_quote': 'Редакция на периодична оферта',
      'billing_address': 'Адрес за фактуриране',
      'shipping_address': 'Адрес за доставка',
      'total_revenue': 'Общо приходи',
      'average_invoice': 'Средна стойност на фактура',
      'outstanding': 'Оставащи',
      'invoices_sent': ':count изпратени фактури',
      'active_clients': 'активни клиенти',
      'close': 'Затвори',
      'email': 'Електронна поща',
      'password': 'Парола',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Име',
      'logout': 'Изход',
      'login': 'Вход',
      'filter': 'Филтър',
      'sort': 'Сортиране',
      'search': 'Търсене',
      'active': 'Активен',
      'archived': 'Архив',
      'deleted': 'изтрита',
      'dashboard': 'Табло',
      'archive': 'Архивирай',
      'delete': 'Изтрий',
      'restore': 'Възтанови',
      'refresh_complete': 'Обновяването завършено',
      'please_enter_your_email': 'Моля въведете вашият имейл',
      'please_enter_your_password': 'Моля въведете вашата парола',
      'please_enter_your_url': 'Моля, въведете своя URL',
      'please_enter_a_product_key': 'Моля, въведете product key',
      'ascending': 'Нарастващо',
      'descending': 'Намаляващо',
      'save': 'Запазване',
      'an_error_occurred': 'Настъпила е грешка',
      'paid_to_date': 'Платени до момента',
      'balance_due': 'Остават за плащане',
      'balance': 'Баланс',
      'overview': 'Преглед',
      'details': 'Подробности',
      'phone': 'Телефон',
      'website': 'Уебсайт',
      'vat_number': 'ДДС Номер',
      'id_number': 'ЕИК/Булстат',
      'create': 'Създаване',
      'copied_to_clipboard': 'Копирано :value в клипборда',
      'error': 'Грешка',
      'could_not_launch': 'Не може да стартира',
      'contacts': 'Контакти',
      'additional': 'Допълнително',
      'first_name': 'Първо име',
      'last_name': 'Фамилно име',
      'add_contact': 'Добави контакт',
      'are_you_sure': 'Сигурен ли сте?',
      'cancel': 'Отказ',
      'ok': 'Ок',
      'remove': 'Премахване',
      'email_is_invalid': 'Имейл адресът е невалиден',
      'product': 'Продукт',
      'products': 'Продукти',
      'new_product': 'Нов продукт',
      'created_product': 'Продуктът беше успешно създаден',
      'updated_product': 'Продуктът беше успешно променен',
      'archived_product': 'Продуктът беше успешно архивиран',
      'deleted_product': 'Успешно изтрит продукт',
      'restored_product': 'Успешно възстановен продукт',
      'archived_products': 'Успешно архивирани :count продукта',
      'deleted_products': 'Успешно изтрити :count продукта',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Продукт',
      'notes': 'Забележки',
      'cost': 'Цена',
      'client': 'Клиент',
      'clients': 'Клиенти',
      'new_client': 'Нов клиент',
      'created_client': 'Клиентът е създаден успешно',
      'updated_client': 'Успешно редактиран клиент',
      'archived_client': 'Успешно архивиран клиент',
      'archived_clients': 'Успешно архивирани :count клиенти',
      'deleted_client': 'Успешно изтрит клиент',
      'deleted_clients': 'Успешно изтрити :count клиенти',
      'restored_client': 'Успешно възстановен Клиент',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Улица',
      'address2': 'Апартамент',
      'city': 'Град',
      'state': 'Област',
      'postal_code': 'Пощенски код',
      'country': 'Държава',
      'invoice': 'Фактура',
      'invoices': 'Фактури',
      'new_invoice': 'Нова фактура',
      'created_invoice': 'Успешно създадена фактура',
      'updated_invoice': 'Успешно редактирана фактура',
      'archived_invoice': 'Успешно архивирана фактура',
      'deleted_invoice': 'Успешно изтрита фактура',
      'restored_invoice': 'Успешно възстановена фактура',
      'archived_invoices': 'Успешно архивирани :count фактури',
      'deleted_invoices': 'Успешно изтрити :count фактури',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Успешно изпратена фактура по имейл',
      'emailed_payment': 'Успешно изпратен имейл за плащане',
      'amount': 'Сума',
      'invoice_number': 'Номер на фактура',
      'invoice_date': 'Дата на фактура',
      'discount': 'Отстъпка',
      'po_number': 'номер на пощенска кутия',
      'terms': 'Условия',
      'public_notes': 'Публични забележки',
      'private_notes': 'Лични бележки',
      'frequency': 'Честота',
      'start_date': 'Начална дата',
      'end_date': 'Крайна дата',
      'quote_number': 'Номер на оферта',
      'quote_date': 'Дата на оферта',
      'valid_until': 'Валидна до',
      'items': 'Редове',
      'partial_deposit': 'Частично/Депозит',
      'description': 'Описание',
      'unit_cost': 'Ед. цена',
      'quantity': 'Количество',
      'add_item': 'Добавяне на ред',
      'contact': 'Контакт',
      'work_phone': 'Телефон',
      'total_amount': 'Обща стойност',
      'pdf': 'PDF',
      'due_date': 'Крайна дата за плащане',
      'partial_due_date': 'Частичен падеж',
      'status': 'Статус',
      'invoice_status_id': 'Статус на фактурите',
      'quote_status': 'Статус на оферта',
      'click_plus_to_add_item': 'Натиснете + за добавяне на ред',
      'click_plus_to_add_time': 'Натиснете + за да добавите време',
      'count_selected': ':count избрани',
      'total': 'Общо',
      'percent': 'Процент',
      'edit': 'Редактиране',
      'dismiss': 'Отхвърляне',
      'please_select_a_date': 'Моля изберете дата',
      'please_select_a_client': 'Моля изберете клиент',
      'please_select_an_invoice': 'Моля, изберете фактура',
      'task_rate': 'Ставка',
      'settings': 'Настройки',
      'language': 'Език',
      'currency': 'Валута',
      'created_at': 'Дата на създаване',
      'created_on': 'Created On',
      'updated_at': 'Актуализиран',
      'tax': 'Данък',
      'please_enter_an_invoice_number': 'Моля, въведете номер на фактура',
      'please_enter_a_quote_number': 'Моля, въведете номер на оферта',
      'past_due': 'Просрочено',
      'draft': 'Чернова',
      'sent': 'Изпратена',
      'viewed': 'Прегледано',
      'approved': 'Одобрено',
      'partial': 'Частично плащане / депозит',
      'paid': 'Платено',
      'mark_sent': 'Маркирай като изпратена',
      'marked_invoice_as_sent': 'Фактурата е успешно маркирана като изпратена',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Фактурите са маркирани като изпратени',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Готово',
      'please_enter_a_client_or_contact_name':
          'Моля, въведете клиент или лице за контакт',
      'dark_mode': 'Тъмен режим',
      'restart_app_to_apply_change':
          'Рестартирайте приложението за прилагане на промяната',
      'refresh_data': 'Опресняване на данни',
      'blank_contact': 'Празен контакт',
      'activity': 'Активност',
      'no_records_found': 'Няма намерени записи',
      'clone': 'Копирай',
      'loading': 'Зареждане',
      'industry': 'Бранш',
      'size': 'Размер',
      'payment_terms': 'Условия за плащане',
      'payment_date': 'Дата на плащане',
      'payment_status': 'Статус на Плащането',
      'payment_status_1': 'Изчакващи',
      'payment_status_2': 'Анулирани',
      'payment_status_3': 'Грешни',
      'payment_status_4': 'Готови',
      'payment_status_5': 'Частично възстановяване',
      'payment_status_6': 'Възстановена',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Нето',
      'client_portal': 'Клиентски портал',
      'show_tasks': 'Показване на задачи',
      'email_reminders': 'Изпращане на напомняния по имейл',
      'enabled': 'Активно',
      'recipients': 'Получатели',
      'initial_email': 'Първоначален имейл',
      'first_reminder': 'Първо напомняне',
      'second_reminder': 'Второ напомняне',
      'third_reminder': 'Трето напомняне',
      'reminder1': 'Първо напомняне',
      'reminder2': 'Второ напомняне',
      'reminder3': 'ТЪрето напомняне',
      'template': 'Шаблон',
      'send': 'Изпращане',
      'subject': 'Относно',
      'body': 'Основен текст',
      'send_email': 'Изпращане на имейл',
      'email_receipt': 'Изпращане на имейл за плащане към клиента',
      'auto_billing': 'Auto billing',
      'button': 'Бутон',
      'preview': 'Преглед',
      'customize': 'Персонализиране',
      'history': 'История',
      'payment': 'Плащане',
      'payments': 'Плащания',
      'refunded': 'Възстановена',
      'payment_type': 'Тип плащане',
      'transaction_reference': 'Основание за превод',
      'enter_payment': 'Въведи плащане',
      'new_payment': 'Въведи плащане',
      'created_payment': 'Успешно създадено плащане',
      'updated_payment': 'Успешно обновено Плащане',
      'archived_payment': 'Успешно архивирано плащане',
      'deleted_payment': 'Успешно изтрито плащане',
      'restored_payment': 'Успешно възстановенo Плащане',
      'archived_payments': 'Успешно архивирани :count плащания',
      'deleted_payments': 'Успешно изтрити :count плащания',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Оферта',
      'quotes': 'Оферти',
      'new_quote': 'Нова оферта',
      'created_quote': 'Успешно Създадена Оферта',
      'updated_quote': 'Успешно обновена Оферта',
      'archived_quote': 'Успешно архивирана Оферта',
      'deleted_quote': 'Успешно изтрита Оферта',
      'restored_quote': 'Успешно възстановена Оферта',
      'archived_quotes': 'Успешно архивирани :count Оферти',
      'deleted_quotes': 'Успешно изтрити :count Оферти',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Разход',
      'expenses': 'Разходи',
      'vendor': 'Доставчик',
      'vendors': 'Доставчици',
      'task': 'Задача',
      'tasks': 'Задачи',
      'project': 'Проект',
      'projects': 'Проекти',
      'activity_1': ':user зъздаде клиент :client',
      'activity_2': ':user архивира клиент :client',
      'activity_3': ':user изтри клиент :client',
      'activity_4': ':user създаде фактура :invoice',
      'activity_5': ':user актуализира фактура :invoice',
      'activity_6': ':user изпрати фактура :invoice, към :client, на :contact',
      'activity_7': ':contact прегледа фактура :invoice, към :client',
      'activity_8': ':user архивира фактура :invoice',
      'activity_9': ':user изтри фактура :invoice',
      'activity_10':
          ':contact въведе плащане :payment в размер на :payment_amount по фактура :invoice за :client',
      'activity_11': ':user актуализира плащане :payment',
      'activity_12': ':user архивира плащане :payment',
      'activity_13': ':user изтри плащане :payment',
      'activity_14': ':user въведе :credit credit',
      'activity_15': ':user актуализира :credit credit',
      'activity_16': ':user архивира :credit credit',
      'activity_17': ':user изтри кредит :credit',
      'activity_18': ':user създаде оферта :quote',
      'activity_19': ':user актуализира оферта :quote',
      'activity_20': ':user изпрати оферта :quote, към :client, на :contact',
      'activity_21': ':contact прегледа оферта :quote',
      'activity_22': ':user архивира оферта :quote',
      'activity_23': ':user изтри оферта :quote',
      'activity_24': ':user възстанови оферта :quote',
      'activity_25': ':user възстанови фактура :invoice',
      'activity_26': ':user възстанови клиент :client',
      'activity_27': ':user възстанови плащане :payment',
      'activity_28': ':user възстанови кредит :credit',
      'activity_29': ':contact одобри оферта :quote, към :client',
      'activity_30': ':user създаде доставчик :vendor',
      'activity_31': ':user архивира доставчик :vendor',
      'activity_32': ':user изтри доставчик :vendor',
      'activity_33': ':user възстанови доставчик :vendor',
      'activity_34': ':user създаде покупка :expense',
      'activity_35': ':user архивира покупка :expense',
      'activity_36': ':user изтри покупка :expense',
      'activity_37': ':user възстанови покупка :expense',
      'activity_39': ':user е отказал :payment_amount за плащане :payment',
      'activity_40':
          ':user е възстановил :adjustment на стойност :payment_amount за плащане :payment',
      'activity_41': 'Отказани :payment_amount по плащане (:payment)',
      'activity_42': ':user създаде задача :task',
      'activity_43': ':user актуализира задача :task',
      'activity_44': ':user архивира задача :task',
      'activity_45': ':user изтри задача :task',
      'activity_46': ':user възстанови задача :task',
      'activity_47': ':user актуализира покупка :expense',
      'activity_48': ':user актуализира тикет :ticket',
      'activity_49': ':user затвори тикет :ticket',
      'activity_50': ':user обедини тикет :ticket',
      'activity_51': ':user раздели тикет :ticket',
      'activity_52': ':contact отвори тикет :ticket',
      'activity_53': ':contact отново отвори тикет :ticket',
      'activity_54': ':user отново отвори :ticket',
      'activity_55': ':contact отговори на тикет :ticket',
      'activity_56': ':user прегледа тикет :ticket',
      'activity_57': 'Системата не успя да изпрати фактура :invoice по e-mail',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Еднократна парола',
      'emailed_quote': 'Успешно изпратена Оферта',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Офертата е успешно маркирана като изпратена',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Изтекла',
      'all': 'Всички',
      'select': 'Избери',
      'long_press_multiselect': 'Натиснете дълго за мултиселекция',
      'custom_value1': 'Персонифицирана стойност',
      'custom_value2': 'Персонифицирана стойност',
      'custom_value3': 'Персонифицирана стойност 3',
      'custom_value4': 'Персонифицирана стойност 4',
      'email_style_custom': 'Персонифициран стил на имейла',
      'custom_message_dashboard': 'Персонифицирано съобщение на таблото',
      'custom_message_unpaid_invoice':
          'Персонифицирано съобшение за неплатена фактура',
      'custom_message_paid_invoice':
          'Персонифицирано съобшение за платена фактура',
      'custom_message_unapproved_quote':
          'Персонифицирано съобшение за непотвърдена фактура',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Преводи',
      'task_number_pattern': 'Задача номер',
      'task_number_counter': 'Брояч на номер на задача',
      'expense_number_pattern': 'Разход номер',
      'expense_number_counter': 'Брояч на номер на разход',
      'vendor_number_pattern': 'Номер на доставчик',
      'vendor_number_counter': 'Брояч на номер на доставчик',
      'ticket_number_pattern': 'Билет номер',
      'ticket_number_counter': 'Брояч на номер на билет',
      'payment_number_pattern': 'Номер на плащане',
      'payment_number_counter': 'Брояч на номер на плащане',
      'invoice_number_pattern': 'Номер на фактура',
      'invoice_number_counter': 'Следващ номер на фактура',
      'quote_number_pattern': 'Номер на оферта',
      'quote_number_counter': 'Следващ номер на оферта',
      'client_number_pattern': 'Номер на кредит',
      'client_number_counter': 'Брояч на номер на кредит',
      'credit_number_pattern': 'Номер на кредит',
      'credit_number_counter': 'Брояч на номер на кредит',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Тип',
      'invoice_amount': 'Стойност на фактура',
      'invoice_due_date': 'Падеж',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Автоматично фактуриране',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Има на такса',
      'tax_amount': 'Такса',
      'tax_paid': 'Платена такса',
      'payment_amount': 'Сума на плащане',
      'age': 'Издадена преди',
      'is_running': 'Is Running',
      'time_log': 'Лог за време',
      'bank_id': 'Банка',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Категория разход',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'zh_TW': {
      'import_data': '匯入資料',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': '專業版',
      'enterprise': 'Enterprise',
      'last_updated': '上次更新時間',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': '聯絡人名字',
      'contact_last_name': '聯絡人姓氏',
      'order': 'Order',
      'unassigned': '未分配的',
      'partial_value': '必須大於零且小於總額',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': '看板',
      'enable': '啟用',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help': '設定子網域或在您的網站上顯示發票。',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': '安全',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': '訂閱',
      'new_subscription': 'New Subscription',
      'edit_subscription': '編輯訂閱資料',
      'created_subscription': '建立訂閱成功',
      'updated_subscription': '更新訂閱成功',
      'archived_subscription': '歸檔訂閱資料成功',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': '重寄邀請函',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': '已退回',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': '使用 :link 相容的 App 掃描條碼。',
      'enabled_two_factor': '啟用兩步驟驗證成功',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': '兩步驟驗證',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': '已退款的付款',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': '轉換至發票',
      'registration_url': 'Registration URL',
      'invoice_project': '發票專案',
      'invoice_task': '為任務開立發票',
      'invoice_expense': '為支出開立發票',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': '轉換的金額',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': '預設的文件',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': '隱藏',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': '欄',
      'sample': '樣本',
      'map_to': 'Map To',
      'import': '匯入',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': '請選擇一個檔案',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV 檔案',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': '服務',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': '未付款',
      'white_label': 'White Label',
      'delivery_note': '寄送註記',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': '部分應付款',
      'invoice_total': '發票總額',
      'quote_total': '報價單總計',
      'credit_total': '貸款總額',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': '警告',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': '信用卡認證編號',
      'client_name': '用戶名稱',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': '更新工作狀態成功',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': '支出類別',
      'new_expense_category': '新的支出類別',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': '成功建立支出類別',
      'updated_expense_category': '更新支出類別成功',
      'archived_expense_category': '歸檔支出類別成功',
      'deleted_expense_category': '刪除類別成功',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': '復原支出類別成功',
      'archived_expense_categories': '歸檔 :count 項支出類別成功',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': '應為此開立發票',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': '標記使用中',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': '週期性發票',
      'recurring_invoices': '週期性發票',
      'new_recurring_invoice': '新的週期性發票',
      'edit_recurring_invoice': '編輯週期性發票',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': '歸檔週期性發票成功',
      'deleted_recurring_invoice': '刪除週期性發票成功',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': '復原週期性發票成功',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': '利潤',
      'line_item': '單列品項',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': '已開啟',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': '檢視入口頁面',
      'copy_link': 'Copy Link',
      'token_billing': '儲存卡片詳細資料',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': '永遠',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': '標籤',
      'client_number': '用戶編號',
      'auto_convert': 'Auto Convert',
      'company_name': '公司名稱',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': '以電子郵件寄出發票成功',
      'emailed_quotes': '以電子郵件寄出報價單成功',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': '閘道',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': '時',
      'statement': '財務報表',
      'taxes': '各類稅金',
      'surcharge': '額外費用',
      'apply_payment': 'Apply Payment',
      'apply': '套用',
      'unapplied': 'Unapplied',
      'select_label': '選擇標籤',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': '到',
      'health_check': 'Health Check',
      'payment_type_id': '付款方式',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': '即將到期的發票',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': '最近的支付',
      'upcoming_quotes': '即將到期的報價單',
      'expired_quotes': '過期的報價單',
      'create_client': '建立用戶',
      'create_invoice': '建立發票',
      'create_quote': '建立報價單',
      'create_payment': 'Create Payment',
      'create_vendor': '建立供應商',
      'update_quote': 'Update Quote',
      'delete_quote': '刪除報價單',
      'update_invoice': 'Update Invoice',
      'delete_invoice': '刪除發票',
      'update_client': 'Update Client',
      'delete_client': '刪除用戶',
      'delete_payment': '刪除付款紀錄',
      'update_vendor': 'Update Vendor',
      'delete_vendor': '刪除供應商',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': '刪除支出',
      'create_task': '建立工作項目',
      'update_task': 'Update Task',
      'delete_task': '刪除工作項目',
      'approve_quote': 'Approve Quote',
      'off': '關',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': '免費',
      'plan': '資費案',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': '目標',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API 的安全代碼',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': '安全代碼',
      'tokens': '安全代碼',
      'new_token': 'New Token',
      'edit_token': '編輯安全代碼',
      'created_token': '安全代碼建立成功',
      'updated_token': '更新安全代碼成功',
      'archived_token': '歸檔安全代碼成功',
      'deleted_token': '刪除安全代碼成功',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': '以電子郵件寄送發票',
      'email_quote': '以電子郵件傳送報價單',
      'email_credit': 'Email Credit',
      'email_payment': '以電子郵件傳送付款資料',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': '聯絡人姓名',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': '編輯付款條件',
      'created_payment_term': '建立付款條款成功',
      'updated_payment_term': '更新付款條款成功',
      'archived_payment_term': '歸檔付款條款成功',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': '貸款金額',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': '不含',
      'inclusive': '內含',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': '已退款的支付',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': '全名',
      'city_state_postal': '城市/州省/郵遞區號',
      'postal_city_state': '城市/州省/郵遞區號',
      'custom1': '首位顧客',
      'custom2': '第二名顧客',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': '清除資料',
      'purge_successful': '清除公司資料成功',
      'purge_data_message': '警告: 這將永久性地抹除您的資料；沒有恢復的可能。',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 天',
      'age_group_30': '30 - 60 天',
      'age_group_60': '60 - 90 天',
      'age_group_90': '90 - 120 天',
      'age_group_120': '120 天以上',
      'refresh': '更新',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': '發票詳細內容',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': '權限',
      'none': '無',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': '已寄出 :count 份發票',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': '套用授權',
      'cancel_account': '刪除帳戶',
      'cancel_account_message': '警告: 這將永久刪除您的帳戶，而且無法恢復。',
      'delete_company': '刪除公司資料',
      'delete_company_message': '警告: 這將永久刪除您的公司資料，而且不可能復原。',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': '頁首',
      'load_design': '載入設計',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': '提案',
      'tickets': '票證',
      'recurring_quotes': '週期性報價單',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': '週期性支出',
      'account_management': '帳號管理',
      'credit_date': '貸款日期',
      'credit': '貸款',
      'credits': '貸款',
      'new_credit': '輸入貸款資料',
      'edit_credit': '編輯貸款資料',
      'created_credit': '建立貸款資料完成',
      'updated_credit': '更新貸款資料成功',
      'archived_credit': '歸檔貸款資料成功',
      'deleted_credit': '刪除貸款資料成功',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': '復原貸款資料成功',
      'archived_credits': '歸檔 :count 筆貸款資料成功',
      'deleted_credits': '刪除 :count 筆貸款資料成功',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': '目前版本',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': '瞭解更多',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': '新的公司資料',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': '重設',
      'number': 'Number',
      'export': '匯出',
      'chart': '圖表',
      'count': 'Count',
      'totals': '總計',
      'blank': '空白',
      'day': '日',
      'month': '月',
      'year': '年',
      'subgroup': '次群組',
      'is_active': 'Is Active',
      'group_by': '分組方式',
      'credit_balance': '貸款餘額',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': '聯絡人電話',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': '送貨地址之街道',
      'shipping_address2': '送貨地址之大樓/套房',
      'shipping_city': '送貨地址之城市',
      'shipping_state': '送貨地址之州/省',
      'shipping_postal_code': '送貨地址之郵遞區號',
      'shipping_country': '送貨地址之國家',
      'billing_address1': '帳單地址之街/路',
      'billing_address2': '帳單地址之大樓/套房',
      'billing_city': '帳單地址之城市',
      'billing_state': '帳單地址之州/省',
      'billing_postal_code': '帳單地址之郵遞區號',
      'billing_country': '帳單地址之國家',
      'client_id': '用戶 Id',
      'assigned_to': '分配給',
      'created_by': '由 :name 建立',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': '欄',
      'aging': '帳齡',
      'profit_and_loss': '利潤與損失',
      'reports': '報告',
      'report': '報告',
      'add_company': '新增公司資料',
      'unpaid_invoice': '未付款之發票',
      'paid_invoice': '已付款之發票',
      'unapproved_quote': '未同意之報價單',
      'help': '說明',
      'refund': '退款',
      'refund_date': 'Refund Date',
      'filtered_by': '篩選依據',
      'contact_email': '聯絡人電子郵件',
      'multiselect': 'Multiselect',
      'entity_state': '狀態',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': '訊息',
      'from': '從',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': '調整百分比以計入費用',
      'configure_settings': 'Configure Settings',
      'support_forum': '支援討論區',
      'about': 'About',
      'documentation': '文件',
      'contact_us': '聯絡我們',
      'subtotal': '小計',
      'line_total': '總計',
      'item': '品項',
      'credit_email': 'Credit Email',
      'iframe_url': '網站',
      'domain_url': 'Domain URL',
      'password_is_too_short': '密碼太短',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': '是',
      'no': '否',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': '行動裝置',
      'desktop': '電腦桌面',
      'layout': 'Layout',
      'view': '檢視',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': '使用者',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': '請選取一個用戶',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': '稅額設定',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': '選項',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': '提交',
      'recover_password': '重設您的密碼',
      'late_fees': '滯納金',
      'credit_number': '貸款編號',
      'payment_number': '付款號碼',
      'late_fee_amount': '逾期費用金額',
      'late_fee_percent': '逾期費用率',
      'schedule': '時間表',
      'before_due_date': '到期日之前',
      'after_due_date': '到期日之後',
      'after_invoice_date': '發票日之後',
      'days': '日',
      'invoice_email': '發票電子郵件',
      'payment_email': '付款資料電子郵件',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': '報價單電子郵件',
      'endless_reminder': '不終止的提醒函',
      'filtered_by_user': '依使用者篩選',
      'administrator': '管理者',
      'administrator_help': '允許使用者管理所有使用者、改變設定、修改所有紀錄',
      'user_management': '管理使用者',
      'users': '使用者',
      'new_user': '新使用者',
      'edit_user': '編輯使用者',
      'created_user': '已成功建立使用者',
      'updated_user': '更新使用者資料成功',
      'archived_user': '歸檔使用者資料成功',
      'deleted_user': '刪除使用者成功',
      'removed_user': 'Successfully removed user',
      'restored_user': '復原使用者資料成功',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': '一般設定',
      'invoice_options': '發票選項',
      'hide_paid_to_date': '隱藏迄今之付款金額',
      'hide_paid_to_date_help': '一旦收到付款，僅在您的發票上顯示「迄今之付款金額」。',
      'invoice_embed_documents': '嵌入的文件',
      'invoice_embed_documents_help': '在發票上附加圖片。',
      'all_pages_header': '顯示頁首於',
      'all_pages_footer': '顯示頁尾於',
      'first_page': '第一頁',
      'all_pages': '所有頁面',
      'last_page': '最後一頁',
      'primary_font': '主要字型',
      'secondary_font': '次要字型',
      'primary_color': '主要色彩',
      'secondary_color': '次要色彩',
      'page_size': '頁面尺寸',
      'font_size': '字型大小',
      'quote_design': '報價單設計',
      'invoice_fields': '發票欄位',
      'product_fields': '產品欄位',
      'invoice_terms': '發票之條款',
      'invoice_footer': '發票頁尾',
      'quote_terms': '報價單條款',
      'quote_footer': '報價單頁尾',
      'auto_email_invoice': '自動電子郵件',
      'auto_email_invoice_help': '週期性發票建立後，自動以電子郵件寄出。',
      'auto_archive_invoice': '自動歸檔',
      'auto_archive_invoice_help': '發票已付款後，自動將它們歸檔。',
      'auto_archive_quote': '自動歸檔',
      'auto_archive_quote_help': '報價單轉換後，自動將它們歸檔。',
      'auto_convert_quote': '自動轉換',
      'auto_convert_quote_help': '在用戶核准後自動將報價單轉換為發票。',
      'workflow_settings': '工作流程設定',
      'freq_daily': '每天',
      'freq_weekly': '每星期',
      'freq_two_weeks': '兩星期',
      'freq_four_weeks': '四星期',
      'freq_monthly': '每月',
      'freq_two_months': '兩個月',
      'freq_three_months': '三個月',
      'freq_four_months': '四個月',
      'freq_six_months': '六個月',
      'freq_annually': 'Annually',
      'freq_two_years': '兩年',
      'freq_three_years': 'Three Years',
      'never': '永不',
      'company': '公司',
      'generated_numbers': '自動產生之號碼',
      'charge_taxes': '附收稅款',
      'next_reset': '下一次重設',
      'reset_counter': '重設計數器',
      'recurring_prefix': '用以標示週期性的前置符號',
      'number_padding': '數字填充',
      'general': '一般',
      'surcharge_field': '附加費欄位',
      'company_field': '公司欄位',
      'company_value': '公司值',
      'credit_field': '信用欄位',
      'invoice_field': '發票欄位',
      'invoice_surcharge': '發票額外費用',
      'client_field': '用戶欄位',
      'product_field': '產品欄位',
      'payment_field': '付款欄位',
      'contact_field': '聯絡人欄位',
      'vendor_field': '供應商欄位',
      'expense_field': '支出欄位',
      'project_field': '專案欄位',
      'task_field': '任務欄位',
      'group_field': '群組欄位',
      'number_counter': '數字計數器',
      'prefix': '前置符號',
      'number_pattern': '數字模式',
      'messages': '訊息',
      'custom_css': '自訂樣式表',
      'custom_javascript': '自訂 JavaScript',
      'signature_on_pdf': '在 PDF 檔案上顯示',
      'signature_on_pdf_help': '在發票/報價單 PDF 顯示用戶簽名。',
      'show_accept_invoice_terms': '發票條款核取方塊',
      'show_accept_invoice_terms_help': '要求用戶確認他們接受發票條款。',
      'show_accept_quote_terms': '報價單條款核取方塊',
      'show_accept_quote_terms_help': '要求用戶確認他們接受報價條款。',
      'require_invoice_signature': '發票簽名',
      'require_invoice_signature_help': '要求用戶提供其簽名。',
      'require_quote_signature': '報價單簽名',
      'enable_portal_password': '用以保護發票的密碼',
      'enable_portal_password_help':
          '使您能夠為每位聯絡人設定密碼。若設定密碼，聯絡人將會在查看發票之前被要求輸入密碼。',
      'authorization': '授權',
      'subdomain': '子網域',
      'domain': '網域',
      'portal_mode': '入口網站模式',
      'email_signature': '向您致意，',
      'enable_email_markup_help': '透過在電子郵件中加入 schema.org 標記，使您的用戶更輕鬆地支付您的費用。',
      'plain': '純文字',
      'light': '淺色',
      'dark': '深色',
      'email_design': '電子郵件的設計',
      'attach_pdf': '附加 PDF 檔案',
      'attach_documents': '附加文件',
      'attach_ubl': '附加 UBL',
      'email_style': '電子郵件樣式',
      'enable_email_markup': '啟用網頁標示',
      'reply_to_email': '回覆電子郵件',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': '電子郵件密件副本',
      'processed': '處理',
      'credit_card': '信用卡',
      'bank_transfer': '銀行轉帳',
      'priority': '優先順序',
      'fee_amount': '費用金額',
      'fee_percent': '費用百分比',
      'fee_cap': '費用上限',
      'limits_and_fees': '限額/費用',
      'enable_min': '啟用最小值',
      'enable_max': '啟用最大值',
      'min_limit': '最小值: :min',
      'max_limit': '最大值: :max',
      'min': '最小值',
      'max': '最大值',
      'accepted_card_logos': '接受的卡片標誌',
      'credentials': '認證',
      'update_address': '更新地址',
      'update_address_help': '使用提供的詳細資料更新用戶的地址',
      'rate': '率',
      'tax_rate': '稅率',
      'new_tax_rate': '新稅率',
      'edit_tax_rate': '編輯稅率',
      'created_tax_rate': '已成功地建立稅率',
      'updated_tax_rate': '更新稅率成功',
      'archived_tax_rate': '歸檔稅率資料成功',
      'deleted_tax_rate': '成功刪除稅率',
      'restored_tax_rate': '成功恢復稅率',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': '自動填入之產品項目',
      'fill_products_help': '選擇產品將自動填寫描述和成本',
      'update_products': '自動更新產品',
      'update_products_help': '更新發票時會自動 更新產品資料庫',
      'convert_products': '轉換產品',
      'convert_products_help': '自動將產品價格轉換為用戶的貨幣',
      'fees': '費用',
      'limits': '限制',
      'provider': '供應商',
      'company_gateway': '付款閘道',
      'company_gateways': '付款閘道',
      'new_company_gateway': '新增閘道',
      'edit_company_gateway': '編輯閘道',
      'created_company_gateway': '建立閘道資料成功',
      'updated_company_gateway': '更新閘道資料成功',
      'archived_company_gateway': '封存閘道資料成功',
      'deleted_company_gateway': '刪除閘道資料成功',
      'restored_company_gateway': '復原閘道成功',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': '繼續編輯',
      'discard_changes': '放棄變更',
      'default_value': '預設值',
      'disabled': '已停用',
      'currency_format': '貨幣格式',
      'first_day_of_the_week': '每星期的第一天',
      'first_month_of_the_year': '年度的第一個月',
      'sunday': '星期日',
      'monday': '星期一',
      'tuesday': '星期二',
      'wednesday': '星期三',
      'thursday': '星期四',
      'friday': '星期五',
      'saturday': '星期六',
      'january': '一月',
      'february': '二月',
      'march': '三月',
      'april': '四月',
      'may': '五月',
      'june': '六月',
      'july': '七月',
      'august': '八月',
      'september': '九月',
      'october': '十月',
      'november': '十一月',
      'december': '十二月',
      'symbol': '符號',
      'ocde': '代碼',
      'date_format': '日期格式',
      'datetime_format': '日期時間格式',
      'military_time': '24 小時制',
      'military_time_help': '24 Hour Display',
      'send_reminders': '傳送提醒',
      'timezone': '時區',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': '依群組篩選',
      'filtered_by_invoice': '依發票篩選',
      'filtered_by_client': '依用戶端篩選',
      'filtered_by_vendor': '依供應商篩選',
      'group_settings': '群組設定',
      'group': '群組',
      'groups': '群組',
      'new_group': '新增群組',
      'edit_group': '編輯群組',
      'created_group': '已成功建立群組',
      'updated_group': '已成功更新群組',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': '上傳徽標',
      'uploaded_logo': '已成功上傳徽標',
      'logo': '標誌',
      'saved_settings': '已成功儲存設定',
      'product_settings': '產品設定',
      'device_settings': '裝置設定',
      'defaults': '預設值',
      'basic_settings': '基本設定',
      'advanced_settings': '進階設定',
      'company_details': '公司之詳細資料',
      'user_details': '使用者詳細資料',
      'localization': '本地化',
      'online_payments': '線上付款',
      'tax_rates': '稅率',
      'notifications': '注意事項',
      'import_export': '匯入 | 匯出',
      'custom_fields': '自訂欄位',
      'invoice_design': '發票設計',
      'buy_now_buttons': '現在即購買按鈕',
      'email_settings': '電子郵件設定',
      'templates_and_reminders': '範本與提醒',
      'credit_cards_and_banks': '信用卡 & 銀行',
      'data_visualizations': '資料視覺化',
      'price': '價格',
      'email_sign_up': '電子郵件註冊',
      'google_sign_up': 'Google 註冊',
      'thank_you_for_your_purchase': '感謝您的購買!',
      'redeem': '兌換',
      'back': '返回',
      'past_purchases': '過去購買',
      'annual_subscription': '年度訂閱',
      'pro_plan': '專業方案',
      'enterprise_plan': '企業方案',
      'count_users': ':count users',
      'upgrade': '升級',
      'please_enter_a_first_name': '請輸入名字',
      'please_enter_a_last_name': '請輸入姓氏',
      'please_agree_to_terms_and_privacy': '請同意服務條款和隱私政策以建立帳戶。',
      'i_agree_to_the': '我同意',
      'terms_of_service_link': '服務條款',
      'privacy_policy_link': '隱私政策',
      'terms_of_service': '服務條款',
      'privacy_policy': '隱私權政策',
      'sign_up': '登入',
      'account_login': '登入帳戶',
      'view_website': '檢視網站',
      'create_account': '建立帳戶',
      'email_login': '電子郵件登入',
      'create_new': '建立新的',
      'no_record_selected': '未選取任何記錄',
      'error_unsaved_changes': '請儲存或取消您的變更',
      'download': '下載',
      'requires_an_enterprise_plan': '需要企業方案',
      'take_picture': '拍照',
      'upload_file': '上傳檔案',
      'document': '文件',
      'documents': '文件',
      'new_document': '新新文件',
      'edit_document': '編輯文件',
      'uploaded_document': '已成功上載文件',
      'updated_document': '已成功更新文件',
      'archived_document': '已成功封存文件',
      'deleted_document': '已成功刪除文件',
      'restored_document': '已成功還原文件',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': '無歷史記錄',
      'expense_date': '支出日期',
      'pending': '擱置',
      'expense_status_1': '已登入',
      'expense_status_2': '擱置',
      'expense_status_3': '已開立發票的',
      'converted': '已轉換',
      'add_documents_to_invoice': '新增文件至發票',
      'exchange_rate': '匯率',
      'convert_currency': '轉換貨幣單位',
      'mark_paid': '標記已付',
      'category': '類別',
      'address': '地址',
      'new_vendor': '新供應商',
      'created_vendor': '建立供應商資料成功',
      'updated_vendor': '更新供應商資料成功',
      'archived_vendor': '歸檔供應商資料成功',
      'deleted_vendor': '刪除供應商成功',
      'restored_vendor': '復原供應商成功',
      'archived_vendors': '歸檔 :count 筆供應商資料成功',
      'deleted_vendors': '刪除 :count 筆供應商成功',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': '輸入支出',
      'created_expense': '已成功建立支出',
      'updated_expense': '更新支出資料成功',
      'archived_expense': '歸檔支出項目成功',
      'deleted_expense': '刪除支出項目成功',
      'restored_expense': '復原支出資料成功',
      'archived_expenses': '歸檔支出項目成功',
      'deleted_expenses': '刪除支出項目成功',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': '複製送貨地址',
      'copy_billing': '複製帳單地址',
      'design': '設計',
      'failed_to_find_record': '找不到記錄',
      'invoiced': '已開立發票的',
      'logged': '已登入',
      'running': '執行中',
      'resume': '繼續',
      'task_errors': '請修正所有的重疊時段',
      'start': '開始',
      'stop': '停止',
      'started_task': '展開任務成功',
      'stopped_task': '停止任務成功',
      'resumed_task': '復原任務成功',
      'now': '現在',
      'auto_start_tasks': '自動啟動任務',
      'timer': '計時器',
      'manual': '手動',
      'budgeted': '預算',
      'start_time': '開始時間',
      'end_time': '結束時間',
      'date': '日期',
      'times': '時段',
      'duration': '時間長度',
      'new_task': '新任務',
      'created_task': '建立工作項目成功',
      'updated_task': '更新工作項目成功',
      'archived_task': '歸檔任務資料成功',
      'deleted_task': '刪除任務成功',
      'restored_task': '復原任務資料成功',
      'archived_tasks': '歸檔 :count 項任務成功',
      'deleted_tasks': '刪除 :count 項任務成功',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': '請輸入姓名',
      'budgeted_hours': '列入預算的小時',
      'created_project': '建立專案成功',
      'updated_project': '成功更新的專案',
      'archived_project': '歸檔專案項目成功',
      'deleted_project': 'Successfully deleted project',
      'restored_project': '復原專案成功',
      'archived_projects': '歸檔 :count 項專案成功',
      'deleted_projects': '刪除 :count 件專案成功',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': '新專案',
      'thank_you_for_using_our_app': '感謝您使用我們的應用程式!',
      'if_you_like_it': '如果您喜歡，請',
      'click_here': '按一下此處',
      'click_here_capital': 'Click here',
      'to_rate_it': '給它評分。',
      'average': '平均',
      'unapproved': '未同意',
      'authenticate_to_change_setting': '請進行身份驗證以變更這個設定',
      'locked': '鎖定',
      'authenticate': '身份驗證',
      'please_authenticate': '請驗證',
      'biometric_authentication': '生物識別驗證',
      'footer': '頁尾',
      'compare': '比較',
      'hosted_login': '託管登入',
      'selfhost_login': 'Selfhost 登入',
      'google_sign_in': 'Login with Google',
      'today': '今天',
      'custom_range': '自訂範圍',
      'date_range': '日期範圍',
      'current': '目前',
      'previous': '以前',
      'current_period': '目前期限',
      'comparison_period': '比較期限',
      'previous_period': '上一期限',
      'previous_year': '上一年度',
      'compare_to': '比較',
      'last7_days': '最近 7 天',
      'last_week': '上個星期',
      'last30_days': '最近 30 天',
      'this_month': '本月',
      'last_month': '上個月',
      'this_year': '今年',
      'last_year': '下個月',
      'custom': '自訂',
      'clone_to_invoice': '再製到發票',
      'clone_to_quote': '再製到報價單',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': '檢視發票',
      'convert': '轉換',
      'more': '更多',
      'edit_client': '編輯用戶',
      'edit_product': '編輯產品資料',
      'edit_invoice': '編輯發票',
      'edit_quote': '編輯報價單',
      'edit_payment': '編輯付款資料',
      'edit_task': '編輯工作項目',
      'edit_expense': '編輯支出',
      'edit_vendor': '編輯供應商',
      'edit_project': '編輯專案',
      'edit_recurring_expense': '編輯週期性支出',
      'edit_recurring_quote': '編輯週期性報價單',
      'billing_address': '帳單地址',
      'shipping_address': '送貨位址',
      'total_revenue': '總收入',
      'average_invoice': '平均銷售額',
      'outstanding': '未付清的',
      'invoices_sent': '已寄出 :count 份發票',
      'active_clients': '使用中用戶',
      'close': '關閉',
      'email': '電子郵件',
      'password': '密碼',
      'url': 'URL',
      'secret': '秘密',
      'name': '姓名',
      'logout': '登出',
      'login': '登入',
      'filter': '篩選器',
      'sort': '排序',
      'search': '搜尋',
      'active': '使用中',
      'archived': '已歸檔',
      'deleted': '已刪除',
      'dashboard': '儀表板',
      'archive': '歸檔',
      'delete': '刪除',
      'restore': '復原',
      'refresh_complete': '重新整理完成',
      'please_enter_your_email': '請輸入您的電子郵件',
      'please_enter_your_password': '請輸入您的密碼',
      'please_enter_your_url': '請輸入您的網址',
      'please_enter_a_product_key': '請輸入產品金鑰',
      'ascending': '遞增',
      'descending': '遞減',
      'save': '儲存',
      'an_error_occurred': '發生錯誤',
      'paid_to_date': '已付',
      'balance_due': '到期餘額',
      'balance': '差額',
      'overview': '總覽',
      'details': '詳細資料',
      'phone': '電話',
      'website': '網站',
      'vat_number': '加值稅編號',
      'id_number': 'ID 編號',
      'create': '建立',
      'copied_to_clipboard': '複製 :value 到剪貼簿',
      'error': '錯誤',
      'could_not_launch': '無法啟動',
      'contacts': '聯絡人',
      'additional': '額外',
      'first_name': '名字',
      'last_name': '姓氏',
      'add_contact': '新增聯絡資料',
      'are_you_sure': '您確定嗎?',
      'cancel': '取消',
      'ok': '正常',
      'remove': '刪除',
      'email_is_invalid': '電子郵件無效',
      'product': '產品',
      'products': '產品',
      'new_product': '新產品',
      'created_product': '建立產品資料成功',
      'updated_product': '成功更新的產品資料',
      'archived_product': '歸檔產品資料成功',
      'deleted_product': '已成功刪除產品資料',
      'restored_product': '復原產品資料成功',
      'archived_products': '歸檔 :count 項產品資料成功',
      'deleted_products': '刪除 :count 筆產品資料成功',
      'restored_products': 'Successfully restored :value products',
      'product_key': '產品',
      'notes': '註記',
      'cost': '成本',
      'client': '用戶',
      'clients': '用戶',
      'new_client': '新用戶',
      'created_client': '建立用戶資料成功',
      'updated_client': '更新用戶資料成功',
      'archived_client': '歸檔用戶資料成功',
      'archived_clients': '歸檔 :count 個用戶資料成功',
      'deleted_client': '刪除用戶資料成功',
      'deleted_clients': '刪除 :count 筆用戶資料成功',
      'restored_client': '復原用戶資料成功',
      'restored_clients': 'Successfully restored :value clients',
      'address1': '街道',
      'address2': '大樓/套房',
      'city': '城市',
      'state': '州/省',
      'postal_code': '郵遞區號',
      'country': '國家',
      'invoice': '發票',
      'invoices': '發票',
      'new_invoice': '新發票',
      'created_invoice': '製作完成的發票',
      'updated_invoice': '更新發票成功',
      'archived_invoice': '歸檔發票資料成功',
      'deleted_invoice': '刪除發票成功',
      'restored_invoice': '復原發票成功',
      'archived_invoices': '歸檔 :count 筆發票資料成功',
      'deleted_invoices': '刪除 :count 筆發票成功',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': '以電子郵件寄出發票成功',
      'emailed_payment': '以電子郵件寄出付款成功',
      'amount': '金額',
      'invoice_number': '發票號碼',
      'invoice_date': '發票開立日期',
      'discount': '折扣',
      'po_number': '郵遞區號',
      'terms': '條款',
      'public_notes': '公開註記',
      'private_notes': '私人註記',
      'frequency': '頻率',
      'start_date': '開始日期',
      'end_date': '結束日期',
      'quote_number': '報價單編號',
      'quote_date': '報價單日期',
      'valid_until': '有效至',
      'items': '個項目',
      'partial_deposit': '存款',
      'description': '描述',
      'unit_cost': '單位成本',
      'quantity': '數量',
      'add_item': '加入項目',
      'contact': '聯絡人',
      'work_phone': '電話',
      'total_amount': '總金額',
      'pdf': 'PDF',
      'due_date': '應付款日期',
      'partial_due_date': '部分截止日期',
      'status': '狀態',
      'invoice_status_id': '發票狀態',
      'quote_status': '報價單狀態',
      'click_plus_to_add_item': '按一下 + 來加入項目',
      'click_plus_to_add_time': '按一下 + 來加入項目',
      'count_selected': ':count 項已選取',
      'total': '總計',
      'percent': '百分比',
      'edit': '編輯',
      'dismiss': '撤銷',
      'please_select_a_date': '請選取日期',
      'please_select_a_client': '請選取一個用戶',
      'please_select_an_invoice': '請選取發票',
      'task_rate': '任務費率',
      'settings': '設定',
      'language': '語言',
      'currency': '貨幣',
      'created_at': '建立日期',
      'created_on': 'Created On',
      'updated_at': '更新',
      'tax': '稅',
      'please_enter_an_invoice_number': '請輸入發票編號',
      'please_enter_a_quote_number': '請輸入報價單編號',
      'past_due': '過去到期',
      'draft': '草稿',
      'sent': '已傳送',
      'viewed': '已檢視',
      'approved': '已核准',
      'partial': '存款',
      'paid': '已付款',
      'mark_sent': '標記已傳送',
      'marked_invoice_as_sent': '標記發票為已傳送成功',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': '完成',
      'please_enter_a_client_or_contact_name': '請輸入用戶或連絡人姓名',
      'dark_mode': '黑暗模式',
      'restart_app_to_apply_change': '重新啟動應用程式以套用變更',
      'refresh_data': '重新整理資料',
      'blank_contact': '空白連絡人',
      'activity': '活動',
      'no_records_found': '找不到記錄',
      'clone': '再製',
      'loading': '載入中',
      'industry': '工業',
      'size': '大小',
      'payment_terms': '付款條件',
      'payment_date': '付款日期',
      'payment_status': '付款狀態',
      'payment_status_1': '擱置',
      'payment_status_2': '作廢',
      'payment_status_3': '失敗',
      'payment_status_4': '完成',
      'payment_status_5': '部分退款',
      'payment_status_6': '退款',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': '淨額',
      'client_portal': '用戶門戶頁面',
      'show_tasks': '顯示任務',
      'email_reminders': '電子郵件提醒',
      'enabled': '啟用',
      'recipients': '收件人',
      'initial_email': '最初的電子郵件',
      'first_reminder': '首次提醒',
      'second_reminder': '第二次提醒',
      'third_reminder': '第三次提醒',
      'reminder1': '首次提醒',
      'reminder2': '第二次提醒',
      'reminder3': '第三次提醒',
      'template': '範本',
      'send': '傳送',
      'subject': '主旨',
      'body': '內文',
      'send_email': '寄送電子郵件',
      'email_receipt': '以電子郵件傳送付款收據給用戶',
      'auto_billing': '自動計費',
      'button': '按鈕',
      'preview': '預覽',
      'customize': '自訂',
      'history': '歷程紀錄',
      'payment': '付款',
      'payments': '付款',
      'refunded': '退款',
      'payment_type': '付款方式',
      'transaction_reference': '轉帳資料',
      'enter_payment': '輸入付款資料',
      'new_payment': '輸入付款資料',
      'created_payment': '已建立完成的付款資料',
      'updated_payment': '更新付款資料成功',
      'archived_payment': '歸檔付款資料成功',
      'deleted_payment': '刪除付款資料成功',
      'restored_payment': '復原付款資料成功',
      'archived_payments': '歸檔 :count 筆付款資料成功',
      'deleted_payments': '刪除 :count 筆付款資料成功',
      'restored_payments': 'Successfully restored :value payments',
      'quote': '報價單',
      'quotes': '報價單',
      'new_quote': '新報價單',
      'created_quote': '報價單建立成功',
      'updated_quote': '報價單更新成功',
      'archived_quote': '歸檔報價單成功',
      'deleted_quote': '報價單刪除成功',
      'restored_quote': '復原報價單成功',
      'archived_quotes': '歸檔 :count 份報價單成功',
      'deleted_quotes': '刪除 :count 筆報價單成功',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': '支出',
      'expenses': '支出',
      'vendor': '供應商',
      'vendors': '供應商',
      'task': '任務',
      'tasks': '任務',
      'project': '專案',
      'projects': '專案',
      'activity_1': ':user 已建立用戶 :client',
      'activity_2': ':user 已將用戶 :client 歸檔',
      'activity_3': ':user 已刪除用戶 :client',
      'activity_4': ':user 已建立發票 :invoice',
      'activity_5': ':user 已更新發票 :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user 已將發票 :invoice 歸檔',
      'activity_9': ':user 已刪除發票 :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user 已更新付款資料 :payment',
      'activity_12': ':user 已將付款資料 :payment 歸檔',
      'activity_13': ':user 已刪除付款資料 :payment',
      'activity_14': ':user 已輸入貸款資料 :credit',
      'activity_15': ':user 更新貸款 :credit',
      'activity_16': ':user 已將 :credit 貸款資料歸檔',
      'activity_17': ':user 已刪除 :credit 貸款資料',
      'activity_18': ':user 已建立報價單 :quote',
      'activity_19': ':user 已更新報價單 :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact 已檢視報價單 :quote',
      'activity_22': ':user 已將報價單 :quote 歸檔',
      'activity_23': ':user 已刪除發票 :quote',
      'activity_24': ':user 已復原報價單 :quote',
      'activity_25': ':user 已復原發票 :invoice',
      'activity_26': ':user 已復原用戶 :client 資料',
      'activity_27': ':user 已復原付款資料 :payment',
      'activity_28': ':user 已復原 :credit 貸款資料',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user 已建立供應商 :vendor',
      'activity_31': ':user 已將供應商 :vendor 歸檔',
      'activity_32': ':user 已刪除供應商 :vendor',
      'activity_33': ':user 已復原供應商 :vendor',
      'activity_34': ':user 已建立支出 :expense',
      'activity_35': ':user 已將支出 :expense 歸檔',
      'activity_36': ':user 已刪除支出 :expense',
      'activity_37': ':user 已復原支出 :expense',
      'activity_39': ':user 已取消一項 :payment_amount 的付款 :payment',
      'activity_40': ':user 獲得一筆金額 :payment_amount 付款 :payment 的退款 :adjustment',
      'activity_41': ':payment_amount 的付款 (:payment) 失敗',
      'activity_42': ':user 已建立任務 :task',
      'activity_43': ':user 已將任務 :task 更新',
      'activity_44': ':user 已將任務 :task 歸檔',
      'activity_45': ':user 已刪除任務 :task',
      'activity_46': ':user 已將任務 :task復原',
      'activity_47': ':user 已將支出 :expense 更新',
      'activity_48': ':user 已更新票證 :ticket',
      'activity_49': ':user 已關閉票證 :ticket',
      'activity_50': ':user 已合併票證 :ticket',
      'activity_51': ':user 拆分票證 :ticket',
      'activity_52': ':contact 已開啟票證 :ticket',
      'activity_53': ':contact 已重新開啟票證 :ticket',
      'activity_54': ':user 已重新開啟票證 :ticket',
      'activity_55': ':contact 已回覆票證 :ticket',
      'activity_56': ':user 已檢視票證 :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': '一次性密碼',
      'emailed_quote': '以電子郵件寄出報價單成功',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': '標記報價單為已傳送成功',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': '過期',
      'all': '全部',
      'select': '選擇',
      'long_press_multiselect': '長按多選',
      'custom_value1': '自訂值',
      'custom_value2': '自訂值',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': '發票號碼計數器',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': '報價單編號計數器',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': '類型',
      'invoice_amount': '發票金額',
      'invoice_due_date': '應付款日期',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': '自動帳單',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': '稅名',
      'tax_amount': '稅金金額',
      'tax_paid': '已付稅',
      'payment_amount': '付款金額',
      'age': '年齡',
      'is_running': 'Is Running',
      'time_log': '時間日誌',
      'bank_id': '銀行',
      'expense_category_id': 'Expense Category ID',
      'expense_category': '支出類別',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'hr': {
      'import_data': 'Uvezi podatke',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Zadnje ažuriranje',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Mora biti veće od nula i manje od zbroja',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Omogući',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Postavi poddomenu ili prikazujte račune na vlastitoj web stranici.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Prošlo tromjesječje',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Konverzija računa',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Fakturiraj zadatak',
      'invoice_expense': 'Trošak računa',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Podržani događaji',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Sakrij',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolona',
      'sample': 'Uzorak',
      'map_to': 'Map To',
      'import': 'Uvoz',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Molim odaberite datoteku',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV datoteka',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Neplaćeno',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Račun sveukupno',
      'quote_total': 'Ponuda sveukupno',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Klijent',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Kategorije troškova',
      'new_expense_category': 'Nova kategorija troškova',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Treba biti fakturiran',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Označi kao aktivno',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Redovni račun',
      'recurring_invoices': 'Redovni računi',
      'new_recurring_invoice': 'Novi redovni račun',
      'edit_recurring_invoice': 'Uredi ponavljajući račun',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Uspješno arhiviran redoviti račun',
      'deleted_recurring_invoice': 'Uspješno obrisan redoviti račun',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Uspješno obnovljen redoviti račun',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email poslan',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Neuspjeh',
      'quota_exceeded': 'Kvota premašena',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'Zapisnici sustava',
      'view_portal': 'View Portal',
      'copy_link': 'Kopiraj link',
      'token_billing': 'Pohrani detalje kartice',
      'welcome_to_invoice_ninja': 'Dobrodošli u Invoice Ninja',
      'always': 'Always',
      'optin': 'Dragovoljno sudjeluj',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Automatski pretvoriti',
      'company_name': 'Company Name',
      'reminder1_sent': 'Podsjetnik 1 poslan',
      'reminder2_sent': 'Podsjetnik 2 poslan',
      'reminder3_sent': 'Podsjetnik 3 poslan',
      'reminder_last_sent': 'Podsjetnik 4 poslan',
      'pdf_page_info': 'Stranica :current od :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'Pogled u aplikaciji Stripe',
      'rows_per_page': 'Redova po stranici',
      'hours': 'sati',
      'statement': 'Izvješće o stanju duga',
      'taxes': 'Porezi',
      'surcharge': 'Surcharge',
      'apply_payment': 'Izvrši plaćanje',
      'apply': 'Apply',
      'unapplied': 'Neprovedeni',
      'select_label': 'Select Label',
      'custom_labels': 'Prilagođene oznake',
      'record_type': 'Vrsta zapisa',
      'record_name': 'Ime zapisa',
      'file_type': 'Vrsta datoteke',
      'height': 'Visina',
      'width': 'Širina',
      'to': 'Prima',
      'health_check': 'Provjera zdravlja',
      'payment_type_id': 'Tip uplate',
      'last_login_at': 'Posljednja prijava u',
      'company_key': 'Ključ tvrtke',
      'storefront': 'Storefront',
      'storefront_help':
          'Omogućite aplikacijama trećih strana za stvaranje računa',
      'client_created': 'Klijent stvoren',
      'online_payment_email': 'E-pošta za internetsko plaćanje',
      'manual_payment_email': 'E-pošta za ručno plaćanje',
      'completed': 'Dovršeno',
      'gross': 'Bruto',
      'net_amount': 'Neto iznos',
      'net_balance': 'Neto saldo',
      'client_settings': 'Postavke klijenta',
      'selected_invoices': 'Odabrani računi',
      'selected_payments': 'Odabrane transkacije',
      'selected_quotes': 'Odabrane ponude',
      'selected_tasks': 'Odabrani zadaci',
      'selected_expenses': 'Odabrani troškovi',
      'upcoming_invoices': 'Dolazni računi',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nedavne uplate',
      'upcoming_quotes': 'Nadolazeće ponude',
      'expired_quotes': 'Istekle ponude',
      'create_client': 'Create Client',
      'create_invoice': 'Kreiraj račun',
      'create_quote': 'Kreiraj ponudu',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Ažuriraj ponudi',
      'delete_quote': 'Obriši ponudu',
      'update_invoice': 'Ažuriraj račun',
      'delete_invoice': 'Obriši račun',
      'update_client': 'Ažuriraj klijenta',
      'delete_client': 'Obriši klijenta',
      'delete_payment': 'Obriši uplatu',
      'update_vendor': 'Ažuriraj dobavljača',
      'delete_vendor': 'Obriši dobavljača',
      'create_expense': 'Stvori trošak',
      'update_expense': 'Ažuriraj trošak',
      'delete_expense': 'Obriši trošak',
      'create_task': 'Kreiraj zadatak',
      'update_task': 'Ažuriraj zadatak',
      'delete_task': 'Obriši zadatak',
      'approve_quote': 'Odobri ponudu',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Istječe u',
      'free': 'Slobodan',
      'plan': 'Plan',
      'show_sidebar': 'Pokaži bočnu traku',
      'hide_sidebar': 'Sakrij bočnu traku',
      'event_type': 'Vrsta događaja',
      'target_url': 'Target',
      'copy': 'Kopiraj',
      'must_be_online':
          'Ponovo pokrenite aplikaciju nakon povezivanja s internetom',
      'crons_not_enabled': 'CRON zadatak mora biti postavljen',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Pretraži :count Webhooks',
      'search_webhook': 'Pretraži 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Novi Webhook',
      'edit_webhook': 'Uredi Webhook',
      'created_webhook': 'Webhook uspješno stvoren',
      'updated_webhook': 'Webhook uspješno ažuriran',
      'archived_webhook': 'Webhook uspješno arhiviran',
      'deleted_webhook': 'Webhook uspješno izbrisan',
      'removed_webhook': 'Webhook uspješno uklonjen',
      'restored_webhook': 'Webhook uspješno vraćen',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API tokeni',
      'api_docs': 'API Docs',
      'search_tokens': 'Pretraži :count tokena',
      'search_token': 'Pretraži 1 token',
      'token': 'Token',
      'tokens': 'Tokeni',
      'new_token': 'Novi token',
      'edit_token': 'Uredi token',
      'created_token': 'Uspješno kreiran token',
      'updated_token': 'Uspješno ažuriran token',
      'archived_token': 'Uspješno arhiviran token',
      'deleted_token': 'Uspješno obrisan token',
      'removed_token': 'Token uspješno uklonjen',
      'restored_token': 'Token uspješno vraćen',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Registracija klijenta',
      'client_registration_help':
          'Omogućite klijentima da se sami registriraju na portalu',
      'customize_and_preview': 'Prilagodba i pregled',
      'email_invoice': 'Pošalji e-poštom',
      'email_quote': 'Šalji ponudu e-poštom',
      'email_credit': 'Email Credit',
      'email_payment': 'Pošalji uplatu e-mailom',
      'client_email_not_set': 'Klijent nema postavljenu adresu e-pošte',
      'ledger': 'Ledger',
      'view_pdf': 'Pogledaj PDF',
      'all_records': 'Svi zapisi',
      'owned_by_user': 'Vlasništvo korisnika',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Upotrijebi zadanu vrijednost',
      'reminder_endless': 'Beskrajni podsjetnici',
      'number_of_days': 'Broj dana',
      'configure_payment_terms': 'Konfiguriraj rokove plaćanja',
      'payment_term': 'Rok plaćanja',
      'new_payment_term': 'Novi rok plaćanja',
      'edit_payment_term': 'Uredi uvjete plaćanja',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Uspješno izbrisan rok plaćanja',
      'removed_payment_term': 'Uspješno uklonjen rok plaćanja',
      'restored_payment_term': 'Uspješno vraćen rok plaćanja',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Prijavite se e-poštom',
      'change': 'Promijeni',
      'change_to_mobile_layout': 'Promijeni na mobilni izgled?',
      'change_to_desktop_layout': 'Promijeni na izgled stolnog računala',
      'send_from_gmail': 'Pošalji s Gmaila',
      'reversed': 'Stornirano',
      'cancelled': 'Otkazani',
      'credit_amount': 'Iznos kredita',
      'quote_amount': 'Iznos Ponude',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Sakri Izbornik',
      'show_menu': 'Prikaži Izbornik',
      'partially_refunded': 'Djelomičan Povrat',
      'search_documents': 'Pretraži Dokumente',
      'search_designs': 'Pretraži Dizajne',
      'search_invoices': 'Pretraži Račune',
      'search_clients': 'Pretraži Klijente',
      'search_products': 'Pretraži proizvode',
      'search_quotes': 'Pretraži Ponude',
      'search_credits': 'Search Credits',
      'search_vendors': 'Pretraži Dobavljača',
      'search_users': 'Pretraži Korisnike',
      'search_tax_rates': 'Pretraži porezne stope',
      'search_tasks': 'Pretraži Zadatke',
      'search_settings': 'Pretraži Postavke',
      'search_projects': 'Pretraži projekte',
      'search_expenses': 'Pretraži troškove',
      'search_payments': 'Pretraži Uplate',
      'search_groups': 'Pretraži Grupe',
      'search_company': 'Pretraži Poduzeće',
      'search_document': 'Pretraži 1 dokument',
      'search_design': 'Pretraži 1 dizajn',
      'search_invoice': 'Pretraži 1 račun',
      'search_client': 'Pretraži 1 klijenta',
      'search_product': 'Pretraži 1 proizvod',
      'search_quote': 'Pretraži 1 ponudu',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Pretraži 1 dobavljača',
      'search_user': 'Pretraži 1 korisnika',
      'search_tax_rate': 'Pretraži 1 poreznu stopu',
      'search_task': 'Pretraži 1 zadatka',
      'search_project': 'Pretraži 1 projekta',
      'search_expense': 'Pretraži 1 troška',
      'search_payment': 'Pretraži 1 transakciju',
      'search_group': 'Pretraži 1 grupu',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Uspješno otkazani račun',
      'cancelled_invoices': 'Uspješno otkazani račun',
      'reversed_invoice': 'Uspješno otkazani računi',
      'reversed_invoices': 'Uspješno storniran račun',
      'reverse': 'Storniraj',
      'full_name': 'Ime i prezime',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Neobavezno',
      'license': 'Licenca',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Stanje računa',
      'age_group_0': '0 - 30 dana',
      'age_group_30': '30 - 60 dana',
      'age_group_60': '60 - 90 dana',
      'age_group_90': '90 - 120 dana',
      'age_group_120': '120+ dana',
      'refresh': 'Refresh',
      'saved_design': 'Uspješno spremljen dizajn',
      'client_details': 'Pojedinosti o klijentu',
      'company_address': 'Adresa tvrtke',
      'invoice_details': 'Detalji računa',
      'quote_details': 'Pojedinosti o ponudi',
      'credit_details': 'Credit Details',
      'product_columns': 'Stupci proizvoda',
      'task_columns': 'Stupci zadatka',
      'add_field': 'Dodaj polje',
      'all_events': 'Svi događaji',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'U vlasništvu',
      'payment_success': 'Uspjeh plaćanja',
      'payment_failure': 'Neuspjeh plaćanja',
      'invoice_sent': ':count invoice sent',
      'quote_sent': 'Ponuda poslana',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Račun pregledan',
      'quote_viewed': 'Ponuda pogledana',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Ponuda odobrena',
      'receive_all_notifications': 'Primi sve obavijesti',
      'purchase_license': 'Kupi licencu',
      'apply_license': 'Apply License',
      'cancel_account': 'Izbriši korisnički račun',
      'cancel_account_message':
          'Pozor: Ovo će trajno obrisati sve vaše podatke, nema povratka.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Ponuda uspješno pretvorena',
      'credit_design': 'Credit Design',
      'includes': 'Uključuje',
      'header': 'Zaglavlje',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Prilagođeni dizajni',
      'designs': 'Dizajni',
      'new_design': 'Novi dizajn',
      'edit_design': 'Uredi dizajn',
      'created_design': 'Dizajn uspješno stvoren',
      'updated_design': 'Dizajn uspješno ažuriran',
      'archived_design': 'Dizajn uspješno arhiviran',
      'deleted_design': 'Dizajn uspješno izbrisan',
      'removed_design': 'Dizajn uspješno uklonjen',
      'restored_design': 'Dizajn uspješno vraćen',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Prijedlozi',
      'tickets': 'Radni nalozi',
      'recurring_quotes': 'Ponavljajuće ponude',
      'recurring_tasks': 'Ponavljajući zadaci',
      'recurring_expenses': 'Redovni troškovi',
      'account_management': 'Upravljanje računima',
      'credit_date': 'Datum kredita',
      'credit': 'Kredit',
      'credits': 'Krediti',
      'new_credit': 'Dodaj kredit',
      'edit_credit': 'Uredi kredit',
      'created_credit': 'Uspješno kreiran kredit',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Uspješno arhiviran kredit',
      'deleted_credit': 'Uspješno obrisan kredit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Uspješno obnovljen kredit',
      'archived_credits': 'Uspješno arhivirano :count kredita',
      'deleted_credits': 'Uspješno obrisano :count kredita',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Trenutna verzija',
      'latest_version': 'Najnovija verzija',
      'update_now': 'Ažuriraj sada',
      'a_new_version_is_available': 'Dostupna je nova verzija web aplikacije',
      'update_available': 'Ažuriranje dostupno',
      'app_updated': 'Ažuriranje je uspješno završeno',
      'learn_more': 'Više informacija',
      'integrations': 'Integracije',
      'tracking_id': 'Broj za praćenje',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Novo poduzeće',
      'added_company': 'Tvrtka je uspješno dodana',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Resetiraj',
      'number': 'Broj',
      'export': 'Izvoz',
      'chart': 'Karte',
      'count': 'Zbroj',
      'totals': 'Zbrojevi',
      'blank': 'Blank',
      'day': 'Dan',
      'month': 'Mjesec',
      'year': 'Godina',
      'subgroup': 'Subgroup',
      'is_active': 'Je aktivan',
      'group_by': 'Grupiraj po',
      'credit_balance': 'Stanje kredita',
      'contact_last_login': 'Zadnje prijavljivanje kontakta',
      'contact_full_name': 'Puno ime kontakta',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Prilagođena vrijednost 1 kontakta',
      'contact_custom_value2': 'Prilagođena vrijednost 2 kontakta',
      'contact_custom_value3': 'Prilagođena vrijednost 3 kontakta',
      'contact_custom_value4': 'Prilagođena vrijednost 4 kontakta',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Dodijeljeno za',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Dodijeljeno ID-u',
      'created_by_id': 'Stvorio ID',
      'add_column': 'Dodaj stupac',
      'edit_columns': 'Uredi stupce',
      'columns': 'Kolone',
      'aging': 'Izvan dospijeća',
      'profit_and_loss': 'Profit i Trošak',
      'reports': 'Izvješća',
      'report': 'Izvješća',
      'add_company': 'Dodaj poduzeće',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Pomoć',
      'refund': 'Refund',
      'refund_date': 'Datum povrata novca',
      'filtered_by': 'Filtrirano po',
      'contact_email': 'Contact Email',
      'multiselect': 'Višestruki odabir',
      'entity_state': 'Kanton',
      'verify_password': 'Potvrdi lozinku',
      'applied': 'Primijenjeno',
      'include_recent_errors': 'Uključite nedavne pogreške iz zapisnika',
      'your_message_has_been_received':
          'Primili smo vašu poruku i pokušat ćemo brzo odgovoriti.',
      'message': 'Poruka',
      'from': 'Šalje',
      'show_product_details': 'Prikaži detalje o proizvodu',
      'show_product_details_help':
          'Uključite opis i cijenu u padajući izbornik proizvoda',
      'pdf_min_requirements': 'PDF renderer zahtijeva :version',
      'adjust_fee_percent': 'Prilagodite postotak naknade',
      'adjust_fee_percent_help':
          'Prilagodite postotak da biste uzeli u obzir naknadu',
      'configure_settings': 'Konfigurirajte postavke',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentacija',
      'contact_us': 'Kontaktirajte nas',
      'subtotal': 'Sveukupno',
      'line_total': 'Ukupno',
      'item': 'Stavka',
      'credit_email': 'Credit Email',
      'iframe_url': 'Web mjesto',
      'domain_url': 'URL domene',
      'password_is_too_short': 'Lozinka je prekratka',
      'password_is_too_easy':
          'Lozinka mora sadržavati barem jedno veliko slovo i broj',
      'client_portal_tasks': 'Zadaci klijentskog portala',
      'client_portal_dashboard': 'Nadzorna ploča klijentskog portala',
      'please_enter_a_value': 'Molimo unesite vrijednost',
      'deleted_logo': 'Logo je uspješno izbrisan',
      'yes': 'Da',
      'no': 'Ne',
      'generate_number': 'Generiraj broj',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Odaberite tvrtku',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Pokaži/Sakrij',
      'menu_sidebar': 'Bočna traka izbornika',
      'history_sidebar': 'Bočna traka povijesti',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Raspored',
      'view': 'Pregled',
      'module': 'Module',
      'first_custom': 'Prvi stupac',
      'second_custom': 'Drugi stupac',
      'third_custom': 'Treći stupac',
      'show_cost': 'Prikaži trošak',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Prikaži polje troškova proizvoda za praćenje marže / dobiti',
      'show_product_quantity': 'Prikaži količinu proizvoda',
      'show_product_quantity_help':
          'Prikaži polje s količinom proizvoda, inače zadano 1',
      'show_invoice_quantity': 'Prikaži količinu računa',
      'show_invoice_quantity_help':
          'Prikaži polje za količinu stavke, inače zadano 1',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Zadana količina',
      'default_quantity_help': 'Količina stavke retka automatski postavi na 1',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Korisnik',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Postavke poreza',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Opcije',
      'single_line_text': 'Tekst u jednom retku',
      'multi_line_text': 'Tekst s više redaka',
      'dropdown': 'Padajući izbornik',
      'field_type': 'Vrsta polja',
      'recover_password_email_sent': 'Poslan je e-mail za oporavak lozinke',
      'submit': 'Submit',
      'recover_password': 'Obnovite vašu zaporku',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Broj transakcije',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Raspored',
      'before_due_date': 'Prije datuma dospijeća',
      'after_due_date': 'Nakon datuma dospijeća',
      'after_invoice_date': 'Nakon datuma računa',
      'days': 'Dani',
      'invoice_email': 'E-pošta računa',
      'payment_email': 'E-pošta uplate',
      'partial_payment': 'Djelomično plaćanje',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'E-pošta za djelomično plaćanje',
      'quote_email': 'E-pošta ponude',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtrirano po korisniku',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Upravljanje korisnicima',
      'users': 'Korisnici',
      'new_user': 'Novi korisnik',
      'edit_user': 'Uredi korisnika',
      'created_user': 'Uspješno stvoren korisnik',
      'updated_user': 'Korisnik je uspješno ažuriran',
      'archived_user': 'Uspješno arhiviran korisnik',
      'deleted_user': 'Korisnik je uspješno obrisan',
      'removed_user': 'Korisnik je uspješno uklonjen',
      'restored_user': 'Uspješno obnovljen korisnik',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Opće postavke',
      'invoice_options': 'Opcije računa',
      'hide_paid_to_date': 'Sakrij datum plaćanja',
      'hide_paid_to_date_help':
          'Prikažite \"Datum plaćanja\" na računima, onda kada je uplata primljena.',
      'invoice_embed_documents': 'Ugrađeni dokumenti',
      'invoice_embed_documents_help': 'Ubaci dodane dokumente u račun.',
      'all_pages_header': 'Prikaži zaglavlje na',
      'all_pages_footer': 'Prikaži podnožje na',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primarni font',
      'secondary_font': 'Sekundarni font',
      'primary_color': 'Primarna boja',
      'secondary_color': 'Sekundarna boja',
      'page_size': 'Page Size',
      'font_size': 'Veličina fonta',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Polja računa',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Uvjeti računa',
      'invoice_footer': 'Podnožje računa',
      'quote_terms': 'Uvjeti ponude',
      'quote_footer': 'Podnožje ponude',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatski konvertirajte ponudu u račun nakon što je odobrena od strane klijenta.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Two weeks',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Tri godine',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generirani brojevi',
      'charge_taxes': 'Naplati poreze',
      'next_reset': 'Slijedeći reset',
      'reset_counter': 'Resetiraj brojač',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'Općenito',
      'surcharge_field': 'Polje doplate',
      'company_field': 'Company Field',
      'company_value': 'Vrijednost tvrtke',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Polje transakcije',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Polje Grupe',
      'number_counter': 'Brojač brojeva',
      'prefix': 'Prefiks',
      'number_pattern': 'Uzorak broja',
      'messages': 'Messages',
      'custom_css': 'Prilagođeni CSS',
      'custom_javascript': 'Prilagođeni JavaScript',
      'signature_on_pdf': 'Pokaži na PDF-u',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Poddomena',
      'domain': 'Domain',
      'portal_mode': 'Način rada Portal',
      'email_signature': 'Srdačno,',
      'enable_email_markup_help':
          'Olakšajte svojim klijentima plaćanje dodavanjem schema.org markupa vašoj e-pošti.',
      'plain': 'Obično',
      'light': 'Svijetlo',
      'dark': 'Tamno',
      'email_design': 'Dizajn e-pošte',
      'attach_pdf': 'Priložite PDF',
      'attach_documents': 'Priložite dokumente',
      'attach_ubl': 'Priložite UBL',
      'email_style': 'Stil e-pošte',
      'enable_email_markup': 'Omogući markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Obrađeno',
      'credit_card': 'Kreditna kartica',
      'bank_transfer': 'Bankovni prijenos',
      'priority': 'Prioritet',
      'fee_amount': 'Iznos naknade',
      'fee_percent': 'Postotak naknade',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limiti/Naknade',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Ažuriraj adresu',
      'update_address_help': 'Ažuriraj adresu klijenta uz osigurane detalje',
      'rate': 'Stopa',
      'tax_rate': 'Porezna stopa',
      'new_tax_rate': 'Nova porezna stopa',
      'edit_tax_rate': 'Uredi poreznu stopu',
      'created_tax_rate': 'Uspješno kreirana porezna stopa',
      'updated_tax_rate': 'Uspješno ažurirana porezna stopa',
      'archived_tax_rate': 'Uspješno arhivirana porezna stopa',
      'deleted_tax_rate': 'Uspješno izbrisana porezna stopa',
      'restored_tax_rate': 'Uspješno vraćena porezna stopa',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Proizvodi sa samoispunom',
      'fill_products_help':
          'Odabir proizvoda će automatski ispuniti opis i cijenu',
      'update_products': 'Proizvidi sa autoažuriranjem',
      'update_products_help':
          'Ažuriranje računa automatski ažurirati registar proizvoda',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Dobavljač',
      'company_gateway': 'Sustav online plaćanja',
      'company_gateways': 'Sustavi online plaćanja',
      'new_company_gateway': 'Novi sustav online plaćanja',
      'edit_company_gateway': 'Uredi sustav online plaćanja',
      'created_company_gateway': 'Uspješno stvoren Sustav online plaćanja',
      'updated_company_gateway': 'Uspješno ažuriran sustav online plaćanja',
      'archived_company_gateway': 'Uspješno arhiviran sustav online plaćanja',
      'deleted_company_gateway': 'Uspješno izbrisan sustav online plaćanja',
      'restored_company_gateway': 'Uspješno vraćen sustav online plaćanja',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Nastavi uređivati',
      'discard_changes': 'Discard Changes',
      'default_value': 'Zadana vrijednost',
      'disabled': 'Onemogućeno',
      'currency_format': 'Format valute',
      'first_day_of_the_week': 'Prvi dan u tjednu',
      'first_month_of_the_year': 'Prvi mjesec u godini',
      'sunday': 'Nedjelja',
      'monday': 'Ponedjeljak',
      'tuesday': 'Utorak',
      'wednesday': 'Srijeda',
      'thursday': 'Četvrtak',
      'friday': 'Petak',
      'saturday': 'Subota',
      'january': 'Siječanj',
      'february': 'Veljača',
      'march': 'Ožujak',
      'april': 'Travanj',
      'may': 'Svibanj',
      'june': 'Lipanj',
      'july': 'Srpanj',
      'august': 'Kolovoz',
      'september': 'Rujan',
      'october': 'Listopad',
      'november': 'Studeni',
      'december': 'Prosinac',
      'symbol': 'Simbol',
      'ocde': 'Code',
      'date_format': 'Format datuma',
      'datetime_format': 'Format vremena',
      'military_time': '24 satno vrijeme',
      'military_time_help': '24-satni prikaz',
      'send_reminders': 'Pošalji podsjetnike',
      'timezone': 'Vremenska zona',
      'filtered_by_project': 'Filtrirano po Projektu',
      'filtered_by_group': 'Filtrirano po grupi',
      'filtered_by_invoice': 'Filtrirano po računu',
      'filtered_by_client': 'Filtrirano po klijentu',
      'filtered_by_vendor': 'Filtrirano po dobavljaču',
      'group_settings': 'Postavke grupe',
      'group': 'Group',
      'groups': 'Grupe',
      'new_group': 'Nova grupa',
      'edit_group': 'Uredi grupu',
      'created_group': 'Grupa je uspješno stvorena',
      'updated_group': 'Grupa je uspješno ažurirana',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Prenesi logo',
      'uploaded_logo': 'Uspješno preneseni logo',
      'logo': 'Logo',
      'saved_settings': 'Postavke uspješno spremljene',
      'product_settings': 'Postavke proizvoda',
      'device_settings': 'Postavke uređaja',
      'defaults': 'Zadano',
      'basic_settings': 'Osnovne postavke',
      'advanced_settings': 'Napredne postavke',
      'company_details': 'Detalji poduzeća',
      'user_details': 'Detalji korisnika',
      'localization': 'Lokalizacija',
      'online_payments': 'Online uplate',
      'tax_rates': 'Porezne stope',
      'notifications': 'Obavijesti',
      'import_export': 'Uvoz | Izvoz',
      'custom_fields': 'Prilagođena polja',
      'invoice_design': 'Dizajn računa',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Postavke e-pošte',
      'templates_and_reminders': 'Predlošci & podsjetnici',
      'credit_cards_and_banks': 'Kreditne kartice i banke',
      'data_visualizations': 'Vizualizacije podataka',
      'price': 'Cijena',
      'email_sign_up': 'Registrirajte se e-poštom',
      'google_sign_up': 'Registrirajte se putem Google računa',
      'thank_you_for_your_purchase': 'Hvala vam na kupnji!',
      'redeem': 'Redeem',
      'back': 'Natrag',
      'past_purchases': 'Prošle kupnje',
      'annual_subscription': 'Godišnja pretplata',
      'pro_plan': 'Pro plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count korisnika',
      'upgrade': 'Nadogradi',
      'please_enter_a_first_name': 'Molimo unesite ime',
      'please_enter_a_last_name': 'Molimo unesite prezime',
      'please_agree_to_terms_and_privacy':
          'Molimo vas da se složite s uvjetima pružanja usluge i pravilima o privatnosti za stvaranje računa.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'uvjetima pružanja usluge',
      'privacy_policy_link': 'politika privatnosti',
      'terms_of_service': 'Uvjeti korištenja usluge',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Prijava',
      'account_login': 'Korisnička prijava',
      'view_website': 'Pogledajte web stranicu',
      'create_account': 'Otvori račun',
      'email_login': 'Prijava putem e-pošte',
      'create_new': 'Create New',
      'no_record_selected': 'Nije odabran nijedan zapis',
      'error_unsaved_changes': 'Spremite ili poništite svoje promjene',
      'download': 'Preuzmi',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Fotografiraj',
      'upload_file': 'Prenesi datoteku',
      'document': 'Document',
      'documents': 'Dokumenti',
      'new_document': 'Novi Dokument',
      'edit_document': 'Uredi Dokument',
      'uploaded_document': 'Uspješno preneseni dokument',
      'updated_document': 'Uspješno ažurirani dokument',
      'archived_document': 'Uspješno arhiviran dokument',
      'deleted_document': 'Uspješno izbrisani dokument',
      'restored_document': 'Uspješno vraćeni dokument',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Nema povijesti',
      'expense_date': 'Datum troška',
      'pending': 'Na čekanju',
      'expense_status_1': 'Evidentirano',
      'expense_status_2': 'U obradi',
      'expense_status_3': 'Fakturirano',
      'converted': 'Konvertirano',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Tečaj',
      'convert_currency': 'Konvertiraj valutu',
      'mark_paid': 'Označi uplatu',
      'category': 'Kategorija',
      'address': 'Adresa',
      'new_vendor': 'Novi dobavljač',
      'created_vendor': 'Uspješno kreiran dobavljač',
      'updated_vendor': 'Uspješno ažuriran dobavljač',
      'archived_vendor': 'Uspješno arhiviran dobavljač',
      'deleted_vendor': 'Uspješno obrisan dobavljač',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Uspješno arhivirano :count dobavljača',
      'deleted_vendors': 'Uspješno obrisano :count dobavljača',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Novi trošak',
      'created_expense': 'Uspješno kreiran trošak',
      'updated_expense': 'Uspješno ažuriran trošak',
      'archived_expense': 'Uspješno arhiviran trošak',
      'deleted_expense': 'Uspješno obrisan trošak',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Uspješno arhivirani troškovi',
      'deleted_expenses': 'Uspješno obrisan trošak',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Dizajn',
      'failed_to_find_record': 'Pronalaženje zapisa nije uspjelo',
      'invoiced': 'Fakturirano',
      'logged': 'Logirano',
      'running': 'Pokrenuto',
      'resume': 'Nastavi',
      'task_errors': 'Molimo korigirajte preklopna vremena',
      'start': 'Početak',
      'stop': 'Završetak',
      'started_task': 'Successfully started task',
      'stopped_task': 'Uspješno završen zadatak',
      'resumed_task': 'Successfully resumed task',
      'now': 'Sada',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Štoperica',
      'manual': 'Ručno',
      'budgeted': 'Budžet',
      'start_time': 'Početno vrijeme',
      'end_time': 'Završno vrijeme',
      'date': 'Datum',
      'times': 'Vremena',
      'duration': 'Trajanje',
      'new_task': 'Novi zadatak',
      'created_task': 'Uspješno kreiran zadatak',
      'updated_task': 'Uspješno ažuriran zadatak',
      'archived_task': 'Uspješno arhiviran zadatak',
      'deleted_task': 'Uspješno obrisan zadatak',
      'restored_task': 'Uspješno obnovljen zadatak',
      'archived_tasks': 'Uspješno arhivirano :count zadataka',
      'deleted_tasks': 'Uspješno obrisano :count zadataka',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Molimo unesite ime',
      'budgeted_hours': 'Dogovoreno radnih sati',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Novi projekt',
      'thank_you_for_using_our_app': 'Hvala vam što koristite našu aplikaciju!',
      'if_you_like_it': 'Ako vam se sviđa, molim vas',
      'click_here': 'kliknite ovdje',
      'click_here_capital': 'Kliknite ovdje',
      'to_rate_it': 'da bi ju ocijenili.',
      'average': 'Prosjek',
      'unapproved': 'Neodobreno',
      'authenticate_to_change_setting':
          'Potvrdite autentičnost da biste promijenili ovu postavku',
      'locked': 'Zaključano',
      'authenticate': 'Provjera autentičnosti',
      'please_authenticate': 'Molimo provjerite autentičnost',
      'biometric_authentication': 'Biometrijska provjera autentičnosti',
      'footer': 'Podnožje',
      'compare': 'Usporedi',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Prijavite se s Google računom',
      'today': 'Danas',
      'custom_range': 'Prilagođeni Raspon',
      'date_range': 'Raspon datuma',
      'current': 'Trenutni',
      'previous': 'Prijašnji',
      'current_period': 'Tekuće Razdoblje',
      'comparison_period': 'Razdoblje usporedbe',
      'previous_period': 'Prethodno razdoblje',
      'previous_year': 'Prethodna godina',
      'compare_to': 'Usporedi s',
      'last7_days': 'Zadnjih 7 dana',
      'last_week': 'Prošli tjedan',
      'last30_days': 'Zadnjih 30 dana',
      'this_month': 'Ovaj mjesec',
      'last_month': 'Prošli mjesec',
      'this_year': 'Ova godina',
      'last_year': 'Prošla godina',
      'custom': 'Prilagođeno',
      'clone_to_invoice': 'Kloniraj u Račune',
      'clone_to_quote': 'Kloniraj u Ponude',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Pregled računa',
      'convert': 'Pretvori',
      'more': 'Više',
      'edit_client': 'Uredi klijenta',
      'edit_product': 'Uredi proizvod',
      'edit_invoice': 'Uredi račun',
      'edit_quote': 'Uredi ponudu',
      'edit_payment': 'Uredi uplatu',
      'edit_task': 'Uredi zadatak',
      'edit_expense': 'Uredi trošak',
      'edit_vendor': 'Uredi dobavljača',
      'edit_project': 'Uredi projekt',
      'edit_recurring_expense': 'Uredi redovne troškove',
      'edit_recurring_quote': 'Uredi ponavljajuću ponudu',
      'billing_address': 'Adresa računa',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Ukupni prihod',
      'average_invoice': 'Prosječni račun',
      'outstanding': 'Dospijeva',
      'invoices_sent': ':count invoices sent',
      'active_clients': 'aktivni klijenti',
      'close': 'Zatvori',
      'email': 'E-pošta',
      'password': 'Zaporka',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Ime',
      'logout': 'Odjava',
      'login': 'Prijava',
      'filter': 'Filter',
      'sort': 'Poredak',
      'search': 'Pretraga',
      'active': 'Aktivan',
      'archived': 'Arhivirano',
      'deleted': 'Obrisano',
      'dashboard': 'Kontrolna ploča',
      'archive': 'Arhiva',
      'delete': 'Obriši',
      'restore': 'Obnovi',
      'refresh_complete': 'Osvježavanje završeno',
      'please_enter_your_email': 'Molimo upišite vašu email adresu',
      'please_enter_your_password': 'Molimo upišite vašu zaporku',
      'please_enter_your_url': 'Molimo unesite URL',
      'please_enter_a_product_key': 'Molimo upišite šifru proizvoda',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Pohrani',
      'an_error_occurred': 'Dogodila se pogreška',
      'paid_to_date': 'Plaćeno na vrijeme',
      'balance_due': 'Stanje duga',
      'balance': 'Potraživanje',
      'overview': 'Pregled',
      'details': 'Detalji',
      'phone': 'Telefon',
      'website': 'Web mjesto',
      'vat_number': 'OIB',
      'id_number': 'ID broj',
      'create': 'Kreiraj',
      'copied_to_clipboard': 'Kopirao :value u međuspremnik',
      'error': 'Greška',
      'could_not_launch': 'Pokretanje nije uspjelo',
      'contacts': 'Kontakti',
      'additional': 'Dodatno',
      'first_name': 'Ime',
      'last_name': 'Prezime',
      'add_contact': 'Dodaj kontakt',
      'are_you_sure': 'Da li ste sigurni?',
      'cancel': 'Odustani',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email adresa je pogrešna',
      'product': 'Proizvod',
      'products': 'Proizvodi',
      'new_product': 'Novi proizvod / usluga',
      'created_product': 'Proizvod je uspješno kreiran',
      'updated_product': 'Proizvod je uspješno ažuriran',
      'archived_product': 'Proizvod je uspješno arhiviran',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Proizvod',
      'notes': 'Bilješke',
      'cost': 'Cijena',
      'client': 'Klijent',
      'clients': 'Klijenti',
      'new_client': 'Novi klijent',
      'created_client': 'Klijent je uspješno kreiran',
      'updated_client': 'Uspješno ažuriranje klijenta',
      'archived_client': 'Uspješno arhiviran klijent',
      'archived_clients': 'Uspješno arhivirano :count klijenata',
      'deleted_client': 'Uspješno obrisan klijent',
      'deleted_clients': 'Uspješno obrisano :count klijenata',
      'restored_client': 'Uspješno obnovljen  klijent',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Ulica i kućni broj',
      'address2': 'Kat/Oznaka',
      'city': 'Grad',
      'state': 'Županija',
      'postal_code': 'Poštanski broj',
      'country': 'Zemlja',
      'invoice': 'Račun',
      'invoices': 'Računi',
      'new_invoice': 'Novi račun',
      'created_invoice': 'Uspješno kreiran račun',
      'updated_invoice': 'Uspješno ažuriran račun',
      'archived_invoice': 'Uspješno arhiviran račun',
      'deleted_invoice': 'Uspješno obrisan račun',
      'restored_invoice': 'Uspješno obnovljen račun',
      'archived_invoices': 'Uspješno arhivirano :count računa',
      'deleted_invoices': 'Uspješno obrisano :count računa',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Račun uspješno poslan e-poštom',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Iznos',
      'invoice_number': 'Broj računa',
      'invoice_date': 'Datum računa',
      'discount': 'Popust',
      'po_number': 'Broj narudžbe',
      'terms': 'Uvjeti',
      'public_notes': 'Javne bilješke',
      'private_notes': 'Privatne bilješke',
      'frequency': 'Frekvencija',
      'start_date': 'Početni datum',
      'end_date': 'Završni datum',
      'quote_number': 'Broj ponude',
      'quote_date': 'Datum ponude',
      'valid_until': 'Vrijedi do',
      'items': 'Stavke',
      'partial_deposit': 'Djelomično/Depozit',
      'description': 'Opis',
      'unit_cost': 'Jedinična cijena',
      'quantity': 'Količina',
      'add_item': 'Dodaj stavku',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Ukupan iznos',
      'pdf': 'PDF',
      'due_date': 'Datum dospijeća',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Status računa',
      'quote_status': 'Status ponude',
      'click_plus_to_add_item': 'Kliknite + za dodavanje stavke',
      'click_plus_to_add_time': 'Pritisnite + za dodavanje vremena',
      'count_selected': ':count odabrano',
      'total': 'Sveukupno',
      'percent': 'Percent',
      'edit': 'Uredi',
      'dismiss': 'Odbaci',
      'please_select_a_date': 'Molimo odaberite datum',
      'please_select_a_client': 'Molimo odaberite klijenta',
      'please_select_an_invoice': 'Molimo odaberite račun',
      'task_rate': 'Satnica',
      'settings': 'Postavke',
      'language': 'Jezik',
      'currency': 'Currency',
      'created_at': 'Datum kreiranja',
      'created_on': 'Stvoreno u',
      'updated_at': 'Ažurirano',
      'tax': 'Porez',
      'please_enter_an_invoice_number': 'Molimo upišite broj računa',
      'please_enter_a_quote_number': 'Molimo upišite broj ponude',
      'past_due': 'Past Due',
      'draft': 'Skica',
      'sent': 'Poslano',
      'viewed': 'Pregledano',
      'approved': 'Odobreno',
      'partial': 'Partial/Deposit',
      'paid': 'Plaćeno',
      'mark_sent': 'Označi kao poslano',
      'marked_invoice_as_sent': 'Račun je uspješno označen kao poslan',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Računi su uspješno označeni kao poslani',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Dovršeno',
      'please_enter_a_client_or_contact_name':
          'Molimo upišite ime klijenta ili kontakta',
      'dark_mode': 'Tamni prikaz',
      'restart_app_to_apply_change':
          'Ponovno pokrenite aplikaciju za primjenu promjena',
      'refresh_data': 'Osvježi podatke',
      'blank_contact': 'Prazan kontakt',
      'activity': 'Aktivnost',
      'no_records_found': 'Nije pronađen zapis',
      'clone': 'Kloniraj',
      'loading': 'Loading',
      'industry': 'Industrija',
      'size': 'Veličina',
      'payment_terms': 'Uvjeti plaćanja',
      'payment_date': 'Datum uplate',
      'payment_status': 'Status uplate',
      'payment_status_1': 'U tijeku',
      'payment_status_2': 'Poništeno',
      'payment_status_3': 'Neuspješno',
      'payment_status_4': 'Završeno',
      'payment_status_5': 'Djelimični povrat',
      'payment_status_6': 'Povrat',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Neto',
      'client_portal': 'Klijentski portal',
      'show_tasks': 'Prikaži zadatke',
      'email_reminders': 'Email podsjetnici',
      'enabled': 'Enabled',
      'recipients': 'Primatelji',
      'initial_email': 'Prvi Email',
      'first_reminder': 'Prvi podsjetnik',
      'second_reminder': 'Drugi podsjetnik',
      'third_reminder': 'Treći podsjetnik',
      'reminder1': 'Prvi podsjetnik',
      'reminder2': 'Drugi podsjetnik',
      'reminder3': 'Treći podsjetnik',
      'template': 'Predložak',
      'send': 'Pošalji',
      'subject': 'Naslov',
      'body': 'Tijelo',
      'send_email': 'Slanje e-pošte',
      'email_receipt': 'Pošalji e-poštom račun klijentu',
      'auto_billing': 'Automatska naplata',
      'button': 'Gumb',
      'preview': 'Preview',
      'customize': 'Prilagodi',
      'history': 'Povijest',
      'payment': 'Uplata',
      'payments': 'Uplate',
      'refunded': 'Povrat',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Referenca transakcije',
      'enter_payment': 'Unesi uplatu',
      'new_payment': 'Unesi uplatu',
      'created_payment': 'Uspješno kreirana uplata',
      'updated_payment': 'Uspješno ažurirana uplata',
      'archived_payment': 'Uspješno arhivirana uplata',
      'deleted_payment': 'Uspješno obrisana uplata',
      'restored_payment': 'Uspješno obnovljena uplata',
      'archived_payments': 'Uspješno arhivirana :count uplata',
      'deleted_payments': 'Uspješno obrisano :count uplata',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Ponuda',
      'quotes': 'Ponude',
      'new_quote': 'Nova ponuda',
      'created_quote': 'Ponuda uspješno kreirana',
      'updated_quote': 'Ponuda je uspješno ažurirana',
      'archived_quote': 'Ponuda uspješno arhivirana',
      'deleted_quote': 'Ponuda uspješno obrisana',
      'restored_quote': 'Uspješno obnovljena ponuda',
      'archived_quotes': 'Uspješno arhivirano :count ponuda',
      'deleted_quotes': 'Uspješno obrisano :count ponuda',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Trošak',
      'expenses': 'Troškovi',
      'vendor': 'Dobavljač',
      'vendors': 'Dobavljači',
      'task': 'Task',
      'tasks': 'Zadaci',
      'project': 'Projekt',
      'projects': 'Projekti',
      'activity_1': ':user kreirao klijenta :client',
      'activity_2': ':user arhivirao klijenta :client',
      'activity_3': ':user obrisao klijenta :client',
      'activity_4': ':user kreirao račun :invoice',
      'activity_5': ':user ažurirao račun :invoice',
      'activity_6': ':user poslao e-poštom račun :invoice za :contact',
      'activity_7': ':contact pregledao račun :invoice',
      'activity_8': ':user arhivirao račun :invoice',
      'activity_9': ':user obrisao račun :invoce',
      'activity_10': ':contact upisao uplatu :payment za :invoice',
      'activity_11': ':user ažurirao uplatu :payment',
      'activity_12': ':user ahivirao uplatu :payment',
      'activity_13': ':user obrisao uplatu :payment',
      'activity_14': ':user upisao :credit kredit',
      'activity_15': ':user ažurirao :credit kredit',
      'activity_16': ':user arhivirao :credit kredit',
      'activity_17': ':user obrisao :credit kredit',
      'activity_18': ':user kreirao ponudu :quote',
      'activity_19': ':user ažurirao ponudu :quote',
      'activity_20': ':user poslao e-poštom ponudu :quote za :contact',
      'activity_21': ':contact pregledao ponudu :quote',
      'activity_22': ':user arhivirao ponudu :quote',
      'activity_23': ':user obrisao ponudu :quote',
      'activity_24': ':user obnovio ponudu :quote',
      'activity_25': ':user obnovio račun :invoice',
      'activity_26': ':user obnovio klijenta :client',
      'activity_27': ':user obnovio uplatu :payment',
      'activity_28': ':user obnovio :credit kredit',
      'activity_29': ':contact odobrio ponudu :quote',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user kreirao trošak :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': 'Korisnik :user je ažurirao radni nalog :ticket',
      'activity_49': 'Korisnik :user je zatvorio radni nalog :ticket',
      'activity_50': 'Korisnik :user je spojio radni nalog :ticket',
      'activity_51': 'Korisnik :user je razdijelio radni nalog :ticket',
      'activity_52': 'Kontakt :contact je otvorio radni nalog :ticket',
      'activity_53': 'Kontakt :contact je ponovno otvorio radni nalog :ticket',
      'activity_54': 'Korisnik :user je ponovno otvorio radni nalog :ticket',
      'activity_55': 'Kontakt :contact je odgovorio na radni nalog :ticket',
      'activity_56': 'Korisnik :user je pregledao radni nalog :ticket',
      'activity_57': 'Sustav nije uspio poslati račun e-poštom :invoice',
      'activity_58': ':user je stornirao račun :invoice',
      'activity_59': ':user otkazao račun :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Ponuda uspješno poslana e-poštom',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Ponuda je uspješno označena kao poslana',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Isteklo',
      'all': 'Svi',
      'select': 'Odaberi',
      'long_press_multiselect': 'Dugo pritisnite za višestruku odabir',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Prilagođena vrijednost 3',
      'custom_value4': 'Prilagođena vrijednost 4',
      'email_style_custom': 'Prilagođeni stil e-pošte',
      'custom_message_dashboard': 'Prilagođena poruka nadzorne ploče',
      'custom_message_unpaid_invoice': 'Prilagođena poruka neplaćenog računa',
      'custom_message_paid_invoice': 'Prilagođena poruka plaćenog računa',
      'custom_message_unapproved_quote':
          'Prilagođena poruka ne odobrene ponude',
      'lock_invoices': 'Zaključaj račune',
      'translations': 'Prijevodi',
      'task_number_pattern': 'Uzorak broja zadatka',
      'task_number_counter': 'Brojač broja zadatka',
      'expense_number_pattern': 'Uzorak broja troškova',
      'expense_number_counter': 'Brojač broja troškova',
      'vendor_number_pattern': 'Uzorak broja dobavljača',
      'vendor_number_counter': 'Brojač brojeva dobavljača',
      'ticket_number_pattern': 'Uzorak broja radnog naloga',
      'ticket_number_counter': 'Brojač broj radnog naloga',
      'payment_number_pattern': 'Uzorak broja transakcije',
      'payment_number_counter': 'Brojač broja transakcije',
      'invoice_number_pattern': 'Uzorak broja računa',
      'invoice_number_counter': 'Brojač računa',
      'quote_number_pattern': 'Uzorak broja ponude',
      'quote_number_counter': 'Brojač ponuda',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Poništi datum brojača',
      'counter_padding': 'Ispuna broja brojača',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Prikaz u tablici',
      'show_list': 'Prikaz u listi',
      'client_city': 'Grad klijenta',
      'client_state': 'Županija klijenta',
      'client_country': 'Država klijenta',
      'client_is_active': 'Klijent je aktivan',
      'client_balance': 'Stanje računa klijenta',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tip',
      'invoice_amount': 'Iznos računa',
      'invoice_due_date': 'Datum valute',
      'tax_rate1': 'Porezna stopa 1',
      'tax_rate2': 'Porezna stopa 2',
      'tax_rate3': 'Porezna stopa 3',
      'auto_bill': 'Auto račun',
      'archived_at': 'Arhivirano u',
      'has_expenses': 'Ima troškove',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Izbrisan',
      'vendor_city': 'Grad dobavljača',
      'vendor_state': 'Županija dobavljača',
      'vendor_country': 'Država dobavljača',
      'is_approved': 'Odobreno je',
      'tax_name': 'Ime porezne stope',
      'tax_amount': 'Iznos poreza',
      'tax_paid': 'Plaćeno poreza',
      'payment_amount': 'Iznos uplate',
      'age': 'Dospijeće',
      'is_running': 'Is Running',
      'time_log': 'Dnevnik vremena',
      'bank_id': 'Banka',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'cs': {
      'import_data': 'Importovat data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Profi',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Musí být větší než nula  a méně než součet',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Umožnit',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Změnit na fakturu',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Fakturační úloha',
      'invoice_expense': 'Fakturovat náklady',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Skrýt',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Sloupec',
      'sample': 'Vzorek',
      'map_to': 'Map To',
      'import': 'Importovat',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Prosím zvolte soubor',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV soubor',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Dodací list',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Částečně splaceno',
      'invoice_total': 'Celková částka',
      'quote_total': 'Celkem',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV kód',
      'client_name': 'Jméno klienta',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Má být fakturován',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'První den v měsíci',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Pravidelná faktura',
      'recurring_invoices': 'Pravidelné faktury',
      'new_recurring_invoice': 'Nová pravidelná faktura',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Pravidelná faktura úspěšně archivována',
      'deleted_recurring_invoice': 'Pravidelná faktura smazána',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Pravidelná faktura obnovena',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Ukládat platební údaje',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hodiny',
      'statement': 'Statement',
      'taxes': 'Daně',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Komu',
      'health_check': 'Health Check',
      'payment_type_id': 'Typ platby',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Nadcházející faktury',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Poslední platby',
      'upcoming_quotes': 'Nadcházející nabídky',
      'expired_quotes': 'Prošlé nabídky',
      'create_client': 'Create Client',
      'create_invoice': 'Vytvořit fakturu',
      'create_quote': 'Vytvořit nabídku',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Smazat nabídku',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Smazat fakturu',
      'update_client': 'Update Client',
      'delete_client': 'Smazat klienta',
      'delete_payment': 'Smazat platbu',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Smazat dodavatele',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Smazat náklad',
      'create_task': 'Vytvořit úlohu',
      'update_task': 'Update Task',
      'delete_task': 'Smazat úlohu',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Zdarma',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokeny',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokeny',
      'new_token': 'New Token',
      'edit_token': 'Editovat token',
      'created_token': 'Token úspěšně vytvořen',
      'updated_token': 'Token úspěšně změněn',
      'archived_token': 'Token úspěšně archivován',
      'deleted_token': 'Token úspěšně smazán',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Poslat fakturu emailem',
      'email_quote': 'Odeslat nabídku emailem',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Editovat platební podmínky',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Počet kreditu',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Práva',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count faktura odeslána',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Smazat účet',
      'cancel_account_message':
          'Varování: Toto permanentně odstraní Váš účet. Tato akce je nevratná.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Hlavička',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Pravidelné nabídky',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Datum kreditu',
      'credit': 'Kredit',
      'credits': 'Kredity',
      'new_credit': 'Zadat kredit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Kredit úspěšně vytvořen',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Kredit úspěšně archivován',
      'deleted_credit': 'Kredit úspěšně smazán',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Kredit úspěšně obnoven',
      'archived_credits': ':count kreditů bylo úspěšně archivováno',
      'deleted_credits': ':count kreditů bylo úspěšně smazáno',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Současná verze',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Více informací',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nová firma',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Resetovat',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Graf',
      'count': 'Count',
      'totals': 'Celkem',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Měsíc',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Seskupené podle',
      'credit_balance': 'Zůstatek kreditu',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Report',
      'add_company': 'Přidat firmu',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Pomoc',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Zpráva',
      'from': 'Od',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentace',
      'contact_us': 'Contact Us',
      'subtotal': 'Mezisoučet',
      'line_total': 'Celkem',
      'item': 'Položka',
      'credit_email': 'Credit Email',
      'iframe_url': 'Web',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Ano',
      'no': 'Ne',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Zobrazit',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Uživatel',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Nastavení daní',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Obnovit vaše heslo',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Rozvrh',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Email pro fakturu',
      'payment_email': 'Email pro platbu',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Email pro nabídku',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrátor',
      'administrator_help':
          'Povolit uživatelům spravovat další uživatele, měnit nastavení a všechny záznamy',
      'user_management': 'Správa uživatelů',
      'users': 'Uživatelé',
      'new_user': 'Nový uživatel',
      'edit_user': 'Upravit uživatele',
      'created_user': 'Successfully created user',
      'updated_user': 'Uživatel úspěšně změněn',
      'archived_user': 'Užival úspěšně archivován',
      'deleted_user': 'Uživatel úspěšně smazán',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Uživatel úspěšně obnoven',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Obecné nastavení',
      'invoice_options': 'Možnosti faktury',
      'hide_paid_to_date': 'Skrýt Zaplaceno ke dni',
      'hide_paid_to_date_help':
          'Zobrazit na faktuře \"Zaplaceno ke dni\" pouze když přijde platba.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Zobrazit hlavičku',
      'all_pages_footer': 'Zobrazit patičku',
      'first_page': 'první stránka',
      'all_pages': 'všechny stránky',
      'last_page': 'poslední stránka',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Základní barva',
      'secondary_color': 'Druhá barva',
      'page_size': 'Page Size',
      'font_size': 'Velikost fontu',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Pole na faktuře',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Fakturační podmínky',
      'invoice_footer': 'Patička faktury',
      'quote_terms': 'Podmínky nabídky',
      'quote_footer': 'Patička nabídky',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automaticky zkonvertovat nabídku na fakturu po schválení klientem.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'týdně',
      'freq_two_weeks': 'Two weeks',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Měsíčně',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Ročně',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Použít daně',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Pole produktu',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Volitelné CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Umožní Vám nastavit heslo pro každý kontakt. Pokud heslo nastavíte, tak kontakt ho bude pro zobrazení faktury vždy použít.',
      'authorization': 'Schválení',
      'subdomain': 'subdoména',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'S pozdravem,',
      'enable_email_markup_help':
          'Přidejte si mikroznačky schema.org do emailu a usnadněte tak vašim klientům platby.',
      'plain': 'Prostý text',
      'light': 'Světlý',
      'dark': 'Tmavý',
      'email_design': 'Vzhled emailu',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Umožnit mikroznačky',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Změnit adresu',
      'update_address_help': 'Změnit adresu klienta podle poskytnutých detailů',
      'rate': 'Sazba',
      'tax_rate': 'Daňová sazba',
      'new_tax_rate': 'Nová sazba daně',
      'edit_tax_rate': 'Editovat daňovou sazbu',
      'created_tax_rate': 'Daňová sazba úspěšně vytvořena',
      'updated_tax_rate': 'Daňová sazba úspěšně změněna',
      'archived_tax_rate': 'Daňová sazba úspěšně archivována',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Automaticky předvyplnit produkty',
      'fill_products_help': 'Výběr produktu automaticky vyplní popis a cenu',
      'update_products': 'Automaticky aktualizovat produkty',
      'update_products_help':
          'Změna na faktuře automaticky aktualizuje katalog produktů',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Nepovolen',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'První den v týdnu',
      'first_month_of_the_year': 'První měsíc v roce',
      'sunday': 'Neděle',
      'monday': 'Pondělí',
      'tuesday': 'Úterý',
      'wednesday': 'Středa',
      'thursday': 'Čtvrtek',
      'friday': 'Pátek',
      'saturday': 'Sobota',
      'january': 'Leden',
      'february': 'Únor',
      'march': 'Březen',
      'april': 'Duben',
      'may': 'Květen',
      'june': 'Červen',
      'july': 'Červenc',
      'august': 'Srpen',
      'september': 'Září',
      'october': 'Říjen',
      'november': 'Listopad',
      'december': 'Prosinec',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 hodinový čas',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Nastavení produktu',
      'device_settings': 'Device Settings',
      'defaults': 'Výchozí',
      'basic_settings': 'Základní nastavení',
      'advanced_settings': 'Pokročilé nastavení',
      'company_details': 'Detaily firmy',
      'user_details': 'Uživatelské detaily',
      'localization': 'Lokalizace',
      'online_payments': 'Online platby',
      'tax_rates': 'Sazby daně',
      'notifications': 'Oznámení',
      'import_export': 'Import | Export',
      'custom_fields': 'Volitelná pole',
      'invoice_design': 'Vzhled faktur',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Nastavení emailu',
      'templates_and_reminders': 'Šablony & Připomínky',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Vizualizace dat',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Obchodní podmínky',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Zaregistrovat se',
      'account_login': 'Přihlášení k účtu',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Stáhnout',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Datum nákladu',
      'pending': 'Nevyřízený',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Zkonvertováno',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Měnový kurz',
      'convert_currency': 'Zkonvertovat měnu',
      'mark_paid': 'Mark Paid',
      'category': 'Category',
      'address': 'Adresa',
      'new_vendor': 'Nový dodavatel',
      'created_vendor': 'Dodavatel úspěšně vytvořen',
      'updated_vendor': 'Dodavatel úspěšně aktualizován',
      'archived_vendor': 'Dodavatel úspěšně archivován',
      'deleted_vendor': 'Dodavatel úspěšně smazán',
      'restored_vendor': 'Dodavatel úspěšně obnoven',
      'archived_vendors': ':count dodavatelů bylo úspěšně archivováno',
      'deleted_vendors': ':count dodavatelů bylo úspěšně smazáno',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Enter Expense',
      'created_expense': 'Náklad úspěšně vytvořen',
      'updated_expense': 'Náklad úspěšně změněn',
      'archived_expense': 'Náklad úspěšně archivován',
      'deleted_expense': 'Náklad úspěšně smazán',
      'restored_expense': 'Náklady úspěšně obnoveny',
      'archived_expenses': 'Náklady úspěšně archivovány',
      'deleted_expenses': 'Náklad úspěšně smazán',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Fakturováno',
      'logged': 'Přihlášen',
      'running': 'Bežící',
      'resume': 'Pokračovat',
      'task_errors': 'Prosím opravte překrývající se časy',
      'start': 'Začátek',
      'stop': 'Konec',
      'started_task': 'Successfully started task',
      'stopped_task': 'Úloha úspěšně zastavena',
      'resumed_task': 'Successfully resumed task',
      'now': 'Nyní',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Časovač',
      'manual': 'Manuální',
      'budgeted': 'Budgeted',
      'start_time': 'Počáteční čas',
      'end_time': 'Čas konce',
      'date': 'Datum',
      'times': 'Časy',
      'duration': 'Trvání',
      'new_task': 'Nový úloha',
      'created_task': 'Úloha úspěšně vytvořena',
      'updated_task': 'Úloha úspěšně změněna',
      'archived_task': 'Úloha úspěšně archivována',
      'deleted_task': 'Úloha úspěšně smazána',
      'restored_task': 'Úloha úspěšně obnovena',
      'archived_tasks': 'Úspěšně archivováno :count úloh',
      'deleted_tasks': 'Úspěšně smazáno  :count úloh',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'klikněte zde',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Patička',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Tento měsíc',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Volitelné',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Zobrazit fakturu',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Editovat klienta',
      'edit_product': 'Upravit produkt',
      'edit_invoice': 'Editovat fakturu',
      'edit_quote': 'Upravit nabídku',
      'edit_payment': 'Editovat platbu',
      'edit_task': 'Editovat úlohu',
      'edit_expense': 'Editovat náklad',
      'edit_vendor': 'Editovat dodavatele',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Fakturační adresa',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Celkové příjmy',
      'average_invoice': 'Průměrná faktura',
      'outstanding': 'Nezaplaceno',
      'invoices_sent': ':count faktur odesláno',
      'active_clients': 'aktivní klienti',
      'close': 'Zavřít',
      'email': 'Email',
      'password': 'Heslo',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Jméno',
      'logout': 'Odhlásit se',
      'login': 'Přihlášení',
      'filter': 'Filtr',
      'sort': 'Sort',
      'search': 'Vyhledat',
      'active': 'Aktivní',
      'archived': 'Archivováno',
      'deleted': 'Smazáno',
      'dashboard': 'Hlavní panel',
      'archive': 'Archivovat',
      'delete': 'Smazat',
      'restore': 'Obnovit',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Uložit',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Zaplaceno ke dni',
      'balance_due': 'Zbývá zaplatit',
      'balance': 'Zůstatek',
      'overview': 'Overview',
      'details': 'Detaily',
      'phone': 'Telefon',
      'website': 'Web',
      'vat_number': 'DIČ',
      'id_number': 'IČO',
      'create': 'Vytvořit',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakty',
      'additional': 'Additional',
      'first_name': 'Jméno',
      'last_name': 'Příjmení',
      'add_contact': 'Přidat kontakt',
      'are_you_sure': 'Jste si jisti?',
      'cancel': 'Zrušit',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produkty',
      'new_product': 'Nový produkt',
      'created_product': 'Produkt úspěšně vytvořen',
      'updated_product': 'Produkt úspěšně aktualizován',
      'archived_product': 'Produkt úspěšně archivován',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Poznámky',
      'cost': 'Cena',
      'client': 'Klient',
      'clients': 'Klienti',
      'new_client': 'Nový klient',
      'created_client': 'Klient úspěšně vytvořen',
      'updated_client': 'Klient úspěšně aktualizován',
      'archived_client': 'Klient úspěšně archivován',
      'archived_clients': ':count klientů bylo úspěšně archivováno',
      'deleted_client': 'Klient úspěšně smazán',
      'deleted_clients': ':count klientů bylo úspěšně smazáno',
      'restored_client': 'Klient úspěšně obnoven',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Ulice',
      'address2': 'Pokoj',
      'city': 'Město',
      'state': 'Oblast',
      'postal_code': 'PSČ',
      'country': 'Země',
      'invoice': 'Faktura',
      'invoices': 'Faktury',
      'new_invoice': 'Nová faktura',
      'created_invoice': 'Faktura úspěšně vytvořena',
      'updated_invoice': 'Faktura úspěšně aktualizována',
      'archived_invoice': 'Faktura úspěšně archivována',
      'deleted_invoice': 'Faktura úspěšně smazána',
      'restored_invoice': 'Faktura úspěšně obnovena',
      'archived_invoices': ':count faktur úspěšně archivováno',
      'deleted_invoices': ':count faktur úspěšně smazáno',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Faktura úspěšně odeslána',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Částka',
      'invoice_number': 'Číslo faktury',
      'invoice_date': 'Datum vystavení',
      'discount': 'Sleva',
      'po_number': 'Číslo objednávky',
      'terms': 'Podmínky',
      'public_notes': 'Veřejné poznámky',
      'private_notes': 'Soukromé poznámky',
      'frequency': 'Frekvence',
      'start_date': 'Počáteční datum',
      'end_date': 'Konečné datum',
      'quote_number': 'Číslo nabídky',
      'quote_date': 'Datum nabídky',
      'valid_until': 'Platí do',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Popis',
      'unit_cost': 'Jedn. cena',
      'quantity': 'Množství',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Datum splatnosti',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Celkem',
      'percent': 'Percent',
      'edit': 'Upravit',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Nastavení',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'DPH',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Odesláno',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Záloha',
      'paid': 'Zaplacené',
      'mark_sent': 'Značka odesláno',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Hotovo',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Tmavý mód',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Aktivita',
      'no_records_found': 'No records found',
      'clone': 'Duplikovat',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Platební podmínky',
      'payment_date': 'Datum platby',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Klientský portál',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'První připomínka',
      'second_reminder': 'Druhá připomínka',
      'third_reminder': 'Třetí připomínka',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Předmět',
      'body': 'Tělo',
      'send_email': 'Odeslat email',
      'email_receipt': 'Odeslat potvrzení platby klientovi',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Přizpůsobení',
      'history': 'Historie',
      'payment': 'Platba',
      'payments': 'Platby',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Odkaz na transakci',
      'enter_payment': 'Zadat platbu',
      'new_payment': 'Zadat platbu',
      'created_payment': 'Platba úspěšně vytvořena',
      'updated_payment': 'Platba úspěšně změněna',
      'archived_payment': 'Platba úspěšně archivována',
      'deleted_payment': 'Platba úspěšně smazána',
      'restored_payment': 'Platba úspěšně obnovena',
      'archived_payments': ':count plateb úspěšně archivováno',
      'deleted_payments': ':count plateb bylo úspěšně smazáno',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Nabídka',
      'quotes': 'Nabídky',
      'new_quote': 'Nová nabídka',
      'created_quote': 'Nabídka úspěšně vytvořena',
      'updated_quote': 'Nabídka úspěšně aktualizována',
      'archived_quote': 'Nabídka úspěšně archivována',
      'deleted_quote': 'Nabídka úspěšně smazána',
      'restored_quote': 'Nabídka úspěšně obnovena',
      'archived_quotes': ':count nabídek bylo úspěšně archivováno',
      'deleted_quotes': ':count nabídek bylo úspěšně smazáno',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Náklad',
      'expenses': 'Náklady',
      'vendor': 'Dodavatel',
      'vendors': 'Dodavatelé',
      'task': 'Task',
      'tasks': 'Úlohy',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user vytvořil klienta :client',
      'activity_2': ':user archivoval klienta :client',
      'activity_3': ':user smazal klienta :client',
      'activity_4': ':user vytvořil fakturu :invoice',
      'activity_5': ':user změnil fakturu :invoice',
      'activity_6':
          ':user poslal email s fakturou :invoice pro :client na :contact',
      'activity_7': 'Klient :contact zobrazil fakturu :invoice pro :client',
      'activity_8': ':user archivoval fakturu :invoice',
      'activity_9': ':user smazal fakturu :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user změnil platbu :payment',
      'activity_12': ':user archivoval platbu :payment',
      'activity_13': ':user smazal platbu :payment',
      'activity_14': ':user zadal :credit kredit',
      'activity_15': ':user změnil :credit kredit',
      'activity_16': ':user archivoval :credit kredit',
      'activity_17': ':user smazal :credit kredit',
      'activity_18': ':user vytvořil nabídku :quote',
      'activity_19': ':user změnil nabídku :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact zobrazil nabídku :quote',
      'activity_22': ':user archivoval nabídku :quote',
      'activity_23': ':user smazal nabídku :quote',
      'activity_24': ':user obnovil nabídku :quote',
      'activity_25': ':user obnovil fakturu :invoice',
      'activity_26': ':user obnovil klienta :client',
      'activity_27': ':user obnovil platbu :payment',
      'activity_28': ':user obnovil :credit kredit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user vytvořil výdaj :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user aktualizoval tiket :ticket',
      'activity_49': ':user uzavřel tiket :ticket',
      'activity_50': ':user sloučil tiket :ticket',
      'activity_51': ':user rozdělil tiket :ticket',
      'activity_52': ':contact vytvořil tiket :ticket',
      'activity_53': ':contact znovu otevřel tiket :ticket',
      'activity_54': ':user znovu otevřel tiket :ticket',
      'activity_55': ':contact odpověděl na tiket :ticket',
      'activity_56': ':user zobrazil tiket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Nabídka úspěšně odeslána',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expirované',
      'all': 'All',
      'select': 'Zvolit',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Číselná řada faktur',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Číselná řada nabídek',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Typ',
      'invoice_amount': 'Částka faktury',
      'invoice_due_date': 'Datum splatnosti',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Automatické fakturování',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Název daně',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Částka k platbě',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Banka',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'da': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Senest opdateret',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Ikke tilknyttet',
      'partial_value': 'Skal være større end nul og mindre end totalen',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Aktivér',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Angiv subdomænet eller vis fakturaen på din egen hjemmeside.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sikkerhed',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Abonnementet blev opdateret',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunderet betaling',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Forrige kvartal',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Konvertér til en faktura',
      'registration_url': 'Registration URL',
      'invoice_project': 'Fakturér projekt',
      'invoice_task': 'Fakturer opgave',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Skjul',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolonne',
      'sample': 'Eksempel',
      'map_to': 'Map To',
      'import': 'Importer',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Venligst vælg en fil',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Vælg CSV-fil',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Ikke betalt',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Faktura total',
      'quote_total': 'Tilbud total',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Advarsel',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'Kontrolcifre',
      'client_name': 'Kundenavn',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Udgiftskategorier',
      'new_expense_category': 'Ny udgiftskategori',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Udgiftskategori oprettet',
      'updated_expense_category': 'Ajourført udgiftskategori',
      'archived_expense_category': 'Udgiftskategori arkiveret',
      'deleted_expense_category': 'Sletning af kategori er gennemført',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Udgiftskategori genoprettet',
      'archived_expense_categories': '.count udgiftskategori(er) arkiveret',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'Paypal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Markér som aktiv',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Gentaget faktura',
      'recurring_invoices': 'Gentagende fakturaer',
      'new_recurring_invoice': 'Ny gentaget fakture',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Fortjeneste',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Se Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Gem kort detaljer',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Altid',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Klientnummer',
      'auto_convert': 'Auto Convert',
      'company_name': 'Firma navn',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Timer',
      'statement': 'Statement',
      'taxes': 'Skatter',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Til',
      'health_check': 'Health Check',
      'payment_type_id': 'Betalingsmetode',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Kommende fakturaer',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nylige betalinger',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': 'Opret faktura',
      'create_quote': 'Opret tilbud',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Slet tilbud',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Slet faktura',
      'update_client': 'Update Client',
      'delete_client': 'Slet kunde',
      'delete_payment': 'Slet betaling',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Slet sælger',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Opret opgave',
      'update_task': 'Update Task',
      'delete_task': 'Slet opgave',
      'approve_quote': 'Approve Quote',
      'off': 'Deaktiver',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Token\'s',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Token\'s',
      'new_token': 'New Token',
      'edit_token': 'Redigér token',
      'created_token': 'Token oprettet',
      'updated_token': 'Token opdateret',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Token slettet',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Send faktura som e-mail',
      'email_quote': 'E-mail tilbuddet',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Kontakt navn',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kreditbeløb',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Eksklusiv',
      'inclusive': 'Inklusiv',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refunder betaling',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'By/Postnummer',
      'postal_city_state': 'Postnummer/By/Region',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Advarsel: Dette vil slette dine data permanent, der er ingen måder at fortryde.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 dage',
      'age_group_30': '30 - 60 dage',
      'age_group_60': '60 - 90 dage',
      'age_group_90': '90 - 120 dage',
      'age_group_120': '120+ dage',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Fakturadetaljer',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count fakturaer sendt',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Anvend licens',
      'cancel_account': 'Annuller konto',
      'cancel_account_message':
          'ADVARSEL: Dette vil permanent slette din konto, der er INGEN mulighed for at fortryde.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Hoved',
      'load_design': 'Indlæs design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Projektforslag',
      'tickets': 'Sager',
      'recurring_quotes': 'Gentagne tilbud',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Kreditdato',
      'credit': 'Kredit',
      'credits': 'Kreditter',
      'new_credit': 'Indtast kredit',
      'edit_credit': 'Redigér kredit',
      'created_credit': 'Kredit oprettet',
      'updated_credit': 'Opdatering af kredit gennemført',
      'archived_credit': 'Kredit arkiveret',
      'deleted_credit': 'Kredit slettet',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Kredit genskabt',
      'archived_credits': 'Arkiverede :count kreditter',
      'deleted_credits': 'Slettede :count kreditter',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Nuværende version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Lær mere',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nyt firma',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Number',
      'export': 'Eksport',
      'chart': 'Diagram',
      'count': 'Count',
      'totals': 'Totaler',
      'blank': 'Blank',
      'day': 'Dag',
      'month': 'Måned',
      'year': 'År',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Gruppér efter',
      'credit_balance': 'Kreditsaldo',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Kontakttelefon',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Klients ID',
      'assigned_to': 'Assigned to',
      'created_by': 'Oprettet af :navn',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Kolonner',
      'aging': 'Aging',
      'profit_and_loss': 'Fortjeneste og tab',
      'reports': 'Rapporter',
      'report': 'Rapport',
      'add_company': 'Tilføj firma',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Hjælp',
      'refund': 'Refunder',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'E-mailkontakt',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Besked',
      'from': 'Fra',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentation',
      'contact_us': 'Kontakt os',
      'subtotal': 'Subtotal',
      'line_total': 'Sum',
      'item': 'Produkttype',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Vælg venligst en kunde',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Tax Settings',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Skift',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Indsend',
      'recover_password': 'Generhverv din adgangskode',
      'late_fees': 'Late Fees',
      'credit_number': 'Kreditnummer',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dage',
      'invoice_email': 'Faktura e-mail',
      'payment_email': 'Betalings e-mail',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Tilbuds e-mail',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Brugerhåndtering',
      'users': 'Brugere',
      'new_user': 'New User',
      'edit_user': 'Rediger bruger',
      'created_user': 'Successfully created user',
      'updated_user': 'Bruger opdateret',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Bruger slettet',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Bruger genskabt',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'General Settings',
      'invoice_options': 'Fakturaindstillinger',
      'hide_paid_to_date': 'Skjul delbetalinger',
      'hide_paid_to_date_help':
          'Vis kun delbetalinger hvis der er forekommet en delbetaling.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show header on',
      'all_pages_footer': 'Show footer on',
      'first_page': 'first page',
      'all_pages': 'all pages',
      'last_page': 'last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primær Farve',
      'secondary_color': 'Sekundær Farve',
      'page_size': 'Page Size',
      'font_size': 'Font Størrelse',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Faktura felt',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Vilkår for fakturaen',
      'invoice_footer': 'Faktura fodnoter',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto konvertering',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daglig',
      'freq_weekly': 'Ugentlig',
      'freq_two_weeks': 'To uger',
      'freq_four_weeks': 'Fire uger',
      'freq_monthly': 'Månedlig',
      'freq_two_months': 'To måneder',
      'freq_three_months': 'Tre måneder',
      'freq_four_months': 'Fire måneder',
      'freq_six_months': 'Seks måneder',
      'freq_annually': 'Årlig',
      'freq_two_years': 'To år',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Dannede numre',
      'charge_taxes': 'Inkluder skat',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Projektfelt',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Præfix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Brugerdefineret CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Afkrydsningsfelt for fakturavilkår',
      'show_accept_invoice_terms_help':
          'Bed kunden om at bekræfte, at de accepterer fakturavilkårene.',
      'show_accept_quote_terms': 'Tilbuds Betingelser Afkrydsningsfelt',
      'show_accept_quote_terms_help':
          'Bed kunden om at bekræfte, at de accepterer tilbudsbetingelserne.',
      'require_invoice_signature': 'Fakturasignatur',
      'require_invoice_signature_help':
          'Kræv at klienten giver deres underskrift.',
      'require_quote_signature': 'Tilbuds underskrift',
      'enable_portal_password': 'Adgangskodebeskyttet Fakturaer',
      'enable_portal_password_help':
          'Lader dig indtaste en adgangskode til hver kontakt. Hvis en adgangskode ikke er lavet, vil kontakten blive pålagt at indtaste en adgangskode før det er muligt at se fakturaer.',
      'authorization': 'Autorisation',
      'subdomain': 'Underdomain',
      'domain': 'Domæne',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Venlig hilsen,',
      'enable_email_markup_help':
          'Gør det lettere for dine klienter at betale dig ved at tilføje schema.org markup i dine e-mails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Brug HTML markup sprog',
      'reply_to_email': 'Svar-til e-mail',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC-email',
      'processed': 'Processed',
      'credit_card': 'Kreditkort',
      'bank_transfer': 'Bankoverførsel',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Aktivér minimum',
      'enable_max': 'Aktivér maksimum',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Opdater adresse',
      'update_address_help': 'Opdater kundens adresse med de opgivne detaljer',
      'rate': 'Sats',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Edit tax rate',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Successfully archived the tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Automatisk-udfyld produkter',
      'fill_products_help':
          'Valg af produkt vil automatisk udfylde beskrivelse og pris',
      'update_products': 'Automatisk opdatering af produkter',
      'update_products_help':
          'En opdatering af en faktura vil automatisk opdaterer Produkt biblioteket',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Gebyrer',
      'limits': 'Grænser',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Sunday',
      'monday': 'Monday',
      'tuesday': 'Tuesday',
      'wednesday': 'Wednesday',
      'thursday': 'Thursday',
      'friday': 'Friday',
      'saturday': 'Saturday',
      'january': 'Januar',
      'february': 'Februar',
      'march': 'Marts',
      'april': 'April',
      'may': 'Maj',
      'june': 'Juni',
      'july': 'Juli',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Hour Time',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Produkt Indstillinger',
      'device_settings': 'Device Settings',
      'defaults': 'Standarder',
      'basic_settings': 'Basic Settings',
      'advanced_settings': 'Avancerede indstillinger',
      'company_details': 'Virksomhedsinformation',
      'user_details': 'User Details',
      'localization': 'Lokalisering',
      'online_payments': 'Onlinebetaling',
      'tax_rates': 'Momssatser',
      'notifications': 'Påmindelser',
      'import_export': 'Import/Eksport',
      'custom_fields': 'Brugerdefineret felt',
      'invoice_design': 'Fakturadesign',
      'buy_now_buttons': '\"Køb nu\" knapper',
      'email_settings': 'E-mail-indstillinger',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Datavisualisering',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Vilkår for brug',
      'privacy_policy': 'Privatlivspolitik',
      'sign_up': 'Registrer dig',
      'account_login': 'Konto Log ind',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Opret ny',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Hent',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'Afventer',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Konverteret',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Markér som betalt',
      'category': 'Kategori',
      'address': 'Adresse',
      'new_vendor': 'Ny sælger',
      'created_vendor': 'Sælger oprettet',
      'updated_vendor': 'Sælger opdateret succesfuldt',
      'archived_vendor': 'Gennemførte arkivering af sælger',
      'deleted_vendor': 'Sletning af sælger gennemført',
      'restored_vendor': 'Genskabelse af sælger gennemført',
      'archived_vendors': 'Gennemførte arkivering af :count sælgere',
      'deleted_vendors': 'Gennemførte sletning af :count sælgere',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Indtast udgift',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Successfully deleted expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Faktureret',
      'logged': 'Ajourført',
      'running': 'Kører',
      'resume': 'Genoptag',
      'task_errors': 'Ret venligst de overlappende tider',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'Successfully started task',
      'stopped_task': 'Opgave stoppet',
      'resumed_task': 'Successfully resumed task',
      'now': 'Nu',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Tidtager',
      'manual': 'Manuelt',
      'budgeted': 'Budgeted',
      'start_time': 'Start Tidspunkt',
      'end_time': 'Slut tidspunkt',
      'date': 'Dato',
      'times': 'Gange',
      'duration': 'Varighed',
      'new_task': 'Ny opgave',
      'created_task': 'Opgave oprettet',
      'updated_task': 'Opgave opdateret',
      'archived_task': 'Opgave arkiveret',
      'deleted_task': 'Opgave slettet',
      'restored_task': 'Opgave genskabt',
      'archived_tasks': 'Antal arkiverede opgaver: :count',
      'deleted_tasks': 'Antal opgaver slettet: :count',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Projektet blev oprettet',
      'updated_project': 'Projektet blev opdateret',
      'archived_project': 'Projektet blev arktiveret',
      'deleted_project': 'Projektet blev slettet',
      'restored_project': 'Projektet blev genskabt',
      'archived_projects': ':count projekter blev arkiveret',
      'deleted_projects': ':count projekter blev slettet',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nyt projekt',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'Klik her',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Fod',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Valgfri periode',
      'date_range': 'Dato område',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Denne måned',
      'last_month': 'Forrige måned',
      'this_year': 'Dette år',
      'last_year': 'Forrige år',
      'custom': 'Brugertilpasset',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Se faktura',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Rediger kunde',
      'edit_product': 'Rediger produkt',
      'edit_invoice': 'Rediger faktura',
      'edit_quote': 'Rediger tilbud',
      'edit_payment': 'Redigér betaling',
      'edit_task': 'Redigér opgave',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Redigér sælger',
      'edit_project': 'Redigér projekt',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Faktura adresse',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Samlede indtægter',
      'average_invoice': 'Gennemsnitlig fakturaer',
      'outstanding': 'Forfaldne',
      'invoices_sent': ':count fakturaer sendt',
      'active_clients': 'aktive kunder',
      'close': 'Luk',
      'email': 'E-mail',
      'password': 'Kodeord',
      'url': 'URL',
      'secret': 'Hemmelighed',
      'name': 'Navn',
      'logout': 'Log ud',
      'login': 'Log ind',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Søg',
      'active': 'Aktiv',
      'archived': 'Archived',
      'deleted': 'Slettet',
      'dashboard': 'Oversigt',
      'archive': 'Arkiv',
      'delete': 'Slet',
      'restore': 'Genskab',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Gem',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Betalt pr. d.d.',
      'balance_due': 'Udestående beløb',
      'balance': 'Balance',
      'overview': 'Overview',
      'details': 'Detaljer',
      'phone': 'Telefon',
      'website': 'Hjemmeside',
      'vat_number': 'CVR/SE-nummer',
      'id_number': 'CVR/SE-nummer',
      'create': 'Opret',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakter',
      'additional': 'Additional',
      'first_name': 'Fornavn',
      'last_name': 'Efternavn',
      'add_contact': 'Tilføj kontakt',
      'are_you_sure': 'Er du sikker?',
      'cancel': 'Annuller',
      'ok': 'Ok',
      'remove': 'Fjern',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produkter',
      'new_product': 'New Product',
      'created_product': 'Produkt oprettet',
      'updated_product': 'Produkt opdateret',
      'archived_product': 'Produkt arkiveret',
      'deleted_product': 'Sletning af produkt gennemført',
      'restored_product': 'Genskabelse af produkt gennemført',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Sletning af :count produkter gennemført',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produkt',
      'notes': 'Notes',
      'cost': 'Cost',
      'client': 'Kunde',
      'clients': 'Kunder',
      'new_client': 'Ny kunde',
      'created_client': 'Kunde oprettet succesfuldt',
      'updated_client': 'Kunde opdateret',
      'archived_client': 'Kunde arkiveret',
      'archived_clients': 'Arkiverede :count kunder',
      'deleted_client': 'Kunde slettet',
      'deleted_clients': 'Slettede :count kunder',
      'restored_client': 'Kunde genskabt',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Gade',
      'address2': 'Nummer',
      'city': 'By',
      'state': 'Område',
      'postal_code': 'Postnummer',
      'country': 'Country',
      'invoice': 'Faktura',
      'invoices': 'Fakturaer',
      'new_invoice': 'Ny faktura',
      'created_invoice': 'Faktura oprettet',
      'updated_invoice': 'Faktura opdateret',
      'archived_invoice': 'Faktura arkiveret',
      'deleted_invoice': 'Faktura slettet',
      'restored_invoice': 'Faktura genskabt',
      'archived_invoices': 'Arkiverede :count fakturaer',
      'deleted_invoices': 'Slettede :count fakturaer',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'E-mail faktura sendt',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Beløb',
      'invoice_number': 'Fakturanummer',
      'invoice_date': 'Faktureringsdato',
      'discount': 'Rabat',
      'po_number': 'Ordrenummer',
      'terms': 'Vilkår',
      'public_notes': 'Public Notes',
      'private_notes': 'Private notater',
      'frequency': 'Frekvens',
      'start_date': 'Startdato',
      'end_date': 'Slutdato',
      'quote_number': 'Tilbuds nummer',
      'quote_date': 'Tilbuds dato',
      'valid_until': 'Gyldig indtil',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Beskrivelse',
      'unit_cost': 'Enhedspris',
      'quantity': 'Stk.',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Betalingsfrist',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Procent',
      'edit': 'Rediger',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Indstillinger',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Oprettelsesdato',
      'created_on': 'Created On',
      'updated_at': 'Opdateret',
      'tax': 'Moms',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Kladde',
      'sent': 'Sendt',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Udbetaling',
      'paid': 'Betalt',
      'mark_sent': 'Markér som sendt',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Færdig',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Mørk tilstand',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Aktivitet',
      'no_records_found': 'No records found',
      'clone': 'Kopiér',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Betalingsvilkår',
      'payment_date': 'Betalingsdato',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Client Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Aktiveret',
      'recipients': 'Modtagere',
      'initial_email': 'Indledende e-mail',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Skabelon',
      'send': 'Send',
      'subject': 'Subject',
      'body': 'Body',
      'send_email': 'Send e-mail',
      'email_receipt': 'Send e-mail kvittering til kunden',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Customize',
      'history': 'Historie',
      'payment': 'Betaling',
      'payments': 'Betalinger',
      'refunded': 'Refunded',
      'payment_type': 'Betalingstype',
      'transaction_reference': 'Transaktionsreference',
      'enter_payment': 'Tilføj betaling',
      'new_payment': 'Indtast betaling',
      'created_payment': 'Betaling oprettet',
      'updated_payment': 'Betaling opdateret',
      'archived_payment': 'Betaling arkiveret',
      'deleted_payment': 'Betaling slettet',
      'restored_payment': 'Betaling genskabt',
      'archived_payments': 'Arkiverede :count betalinger',
      'deleted_payments': 'Slettede :count betalinger',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Pristilbud',
      'quotes': 'Pristilbud',
      'new_quote': 'Nyt tilbud',
      'created_quote': 'Tilbud oprettet',
      'updated_quote': 'Tilbud opdateret',
      'archived_quote': 'Tilbud arkiveret',
      'deleted_quote': 'Tilbud slettet',
      'restored_quote': 'Tilbud genskabt',
      'archived_quotes': 'Arkiverede :count tilbud',
      'deleted_quotes': 'Slettede :count tilbud',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Expense',
      'expenses': 'Udgifter',
      'vendor': 'Sælger',
      'vendors': 'Sælgere',
      'task': 'Opgave',
      'tasks': 'Opgaver',
      'project': 'Projekt',
      'projects': 'Projekter',
      'activity_1': ':user created client :client',
      'activity_2': ':user arkiverede kunde :client',
      'activity_3': ':user slettede kunde :client',
      'activity_4': ':user oprettede faktura :invoice',
      'activity_5': ':user ajourførte faktura :invoice',
      'activity_6':
          ':user emailede fakturaen :invoice for :client til :contact',
      'activity_7': ':contact læste faktura :invoice for :client',
      'activity_8': ':user arkiverede faktura :invoice',
      'activity_9': ':user slettede faktura :invoice',
      'activity_10':
          ':contact indtastede betaling :payment for :payment_amout i fakturaen :invoice for :client',
      'activity_11': ':user ajourførte betaling :payment',
      'activity_12': ':user arkiverede betaling :payment',
      'activity_13': ':user slettede betaling :payment',
      'activity_14': ':user indtastede :credit kredit',
      'activity_15': ':user ajourførte :credit kredit',
      'activity_16': ':user arkiverede :credit kredit',
      'activity_17': ':user slettede :credit kredit',
      'activity_18': ':user oprettede tilbud :quote',
      'activity_19': ':user ajourførte tilbud :quote',
      'activity_20': ':user emailede tilbuddet :quote for :client til :contact',
      'activity_21': ':contact læste tilbud :quote',
      'activity_22': ':user arkiverede tilbud :quote',
      'activity_23': ':user slettede tilbud:quote',
      'activity_24': ':user genoprettede tilbud :quote',
      'activity_25': ':user genoprettede faktura :invoice',
      'activity_26': ':user genoprettede kunde :client',
      'activity_27': ':user genoprettede betaling :payment',
      'activity_28': ':user genoprettede :credit kredit',
      'activity_29': ':contact godkendte tilbuddet :quote for :client',
      'activity_30': ':user oprettede sælger :vendor',
      'activity_31': ':user arkiverede sælger :vendor',
      'activity_32': ':user slettede sælgeren :vendor',
      'activity_33': ':user genskabte sælgeren :vendor',
      'activity_34': ':user oprettede udgiften :expense',
      'activity_35': ':user arkiverede udgiften :expense',
      'activity_36': ':user slettede udgiften :expense',
      'activity_37': ':user genskabte udgiften :expense',
      'activity_39': ':user annullerede en :payment_amount betaling :payment',
      'activity_40':
          ':bruger refunderet :justering af en :betaling_beløb betaling :betaling',
      'activity_41': ':payment_amount betaling (:betaling) mislykkedes',
      'activity_42': ':user oprettede opgaven :task',
      'activity_43': ':user opdaterede opgaven :task',
      'activity_44': ':user arkiverede opgaven :task',
      'activity_45': ':user slettede opgave :task',
      'activity_46': ':user genoprettede opgave :task',
      'activity_47': ':user ajourførte udgift :expense',
      'activity_48': ':user opdaterede sagen :ticket',
      'activity_49': ':user lukkede sagen :ticket',
      'activity_50': ':user sammenflettede sagen :ticket',
      'activity_51': ':user opdelte sagen :ticket',
      'activity_52': ':contact åbnede sagen :ticket',
      'activity_53': ':contact genåbnede sagen :ticket',
      'activity_54': ':user genåbnede sagen :ticket',
      'activity_55': ':contact besvarede sagen :ticket',
      'activity_56': ':user læste sagen :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Tilbud sendt som e-mail',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'Alle',
      'select': 'Vælg',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Fakturanummer-tæller',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Tilbuds nummer-tæller',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Due Date',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Bill',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Beløb',
      'age': 'Alder',
      'is_running': 'Is Running',
      'time_log': 'Tids log',
      'bank_id': 'bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Udgiftskategori',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'nl': {
      'import_data': 'Importeer data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Laatst bijgewerkt',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact voornaam',
      'contact_last_name': 'Contact achternaam',
      'order': 'Order',
      'unassigned': 'Niet toegewezen',
      'partial_value': 'Moet groter zijn dan nul en minder dan het totaal',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Activeer',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Stel het subdomein in of toon de factuur op uw eigen website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Beveiliging',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Abonnementen',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Abonnement wijzigen',
      'created_subscription': 'Het abonnement is aangemaakt',
      'updated_subscription': 'Het abonnement is gewijzigd',
      'archived_subscription': 'Het abonnement is gearchiveerd',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Gebruik laatste e-mail',
      'activate_company': 'Activeer bedrijf',
      'activate_company_help':
          'Schakel e-mails, terugkerende facturen en meldingen in',
      'an_error_occurred_try_again':
          'Er is een fout opgetreden, probeer het opnieuw',
      'please_first_set_a_password': 'Stel eerst een wachtwoord in',
      'changing_phone_disables_two_factor':
          'Waarschuwing: als u uw telefoonnummer wijzigt, wordt 2FA uitgeschakeld',
      'help_translate': 'Help vertalen',
      'please_select_a_country': 'Selecteer een land',
      'resend_invite': 'Uitnodiging opnieuw versturen',
      'disabled_two_factor': '2FA succesvol uitgeschakeld',
      'connected_google': 'Account succesvol verbonden',
      'disconnected_google': 'Account succesvol losgekoppeld',
      'delivered': 'Afgeleverd',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'Bekijk documenten',
      'enter_phone_to_enable_two_factor':
          'Geef een mobiel telefoonnummer op om tweefactor authenticatie in te schakelen',
      'send_sms': 'Verzend SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Scan de streepjescode met een :link compatibele app.',
      'enabled_two_factor': 'Tweestaps-authenticatie ingeschakeld',
      'connect_google': 'Verbind met Google',
      'disconnect_google': 'Verwijder Google',
      'enable_two_factor': 'Tweestaps-authenticatie',
      'disable_two_factor': 'Schakel twee factor authenticatie uit',
      'require_password_with_social_login':
          'Vereis wachtwoord met sociale login',
      'stay_logged_in': 'Blijf ingelogd',
      'session_about_to_expire': 'Waarschuwing: uw sessie loopt bijna af',
      'count_hours': ':count uren',
      'count_day': '1 dag',
      'count_days': ':count dagen',
      'web_session_timeout': 'Time-out van websessie',
      'security_settings': 'Veiligheidsinstellingen',
      'resend_email': 'Email opnieuw verzenden',
      'confirm_your_email_address': 'Bevestig je e-mailadres',
      'refunded_payment': 'Gecrediteerde betaling',
      'partially_unapplied': 'Gedeeltelijk niet toegepast',
      'select_a_gmail_user':
          'Selecteer een gebruiker die is geverifieerd met Gmail',
      'list_long_press': 'Lijst lang indrukken',
      'show_actions': 'Toon acties',
      'start_multiselect': 'Start Multi select',
      'email_sent_to_confirm_email':
          'Er is een e-mail verzonden om het e-mailadres te bevestigen',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'Dit kwartaal',
      'last_quarter': 'Laatste Kwartaal',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Zet om naar factuur',
      'registration_url': 'Registratie link',
      'invoice_project': 'Factureer project',
      'invoice_task': 'Factureer taak',
      'invoice_expense': 'Factureer uitgave',
      'search_payment_term': 'Zoek betalingstermijn 1',
      'search_payment_terms': 'Zoek :count betalingstermijnen',
      'save_and_preview': 'Opslaan en bekijk voorbeeld',
      'save_and_email': 'Opslaan en verstuur email',
      'supported_events': 'Ondersteunde gebeurtenissen',
      'converted_amount': 'Omgezet bedrag',
      'converted_balance': 'Omgekeerd balans',
      'converted_paid_to_date': '\"Reeds betaald\" omzetten',
      'converted_credit_balance': 'Omgerekend creditsaldo',
      'converted_total': 'Totaal omzetten',
      'is_sent': 'Is verzonden',
      'default_documents': 'Standaard documenten',
      'document_upload': 'Document uploaden',
      'document_upload_help': 'Laat klanten documenten uploaden',
      'expense_total': 'Totale uitgave',
      'enter_taxes': 'Voer belastingen in',
      'by_rate': 'Op tarief',
      'by_amount': 'Op bedrag',
      'enter_amount': 'Voer bedrag in',
      'before_taxes': 'Voor BTW',
      'after_taxes': 'Na BTW',
      'color': 'Kleur',
      'show': 'Tonen',
      'hide': 'Verbergen',
      'empty_columns': 'Lege kolommen',
      'debug_mode_is_enabled': 'Foutopsporingsmodus is ingeschakeld',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recente taken',
      'recent_expenses': 'Recente uitgaven',
      'upcoming_expenses': 'Aankomende uitgaven',
      'update_app': 'Update App',
      'started_import': 'Succesvol begonnen met importeren',
      'duplicate_column_mapping': 'Dubbele kolommapping',
      'uses_inclusive_taxes': 'Gebruik inclusieve belastingen',
      'is_amount_discount': 'Is bedrag korting',
      'column': 'Kolom',
      'sample': 'Voorbeeld',
      'map_to': 'Map naar',
      'import': 'Importeer',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Selecteer een bestand',
      'no_file_selected': 'Geen bestand geselecteerd',
      'csv_file': 'Selecteer CSV bestand',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave boekhouding',
      'zoho': 'Zoho',
      'accounting': 'Boekhouding',
      'required_files_missing': 'Geef alle CSV\'s op.',
      'import_type': 'Importeer type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Bekijk Licenties',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Editor volledig scherm',
      'sidebar_editor': 'Zijbalk Editor',
      'please_type_to_confirm': 'Typ \":value\" om te bevestigen',
      'purge': 'Wissen',
      'service': 'Service',
      'clone_to': 'Dupliceer naar',
      'clone_to_other': 'Dupliceert naar andere',
      'labels': 'Labels',
      'add_custom': 'Aangepast toevoegen',
      'payment_tax': 'Betalingsbelasting',
      'unpaid': 'Onbetaald',
      'white_label': 'White Label',
      'delivery_note': 'Afleveringsbon',
      'sent_invoices_are_locked': 'Verzonden facturen zijn vergrendeld',
      'paid_invoices_are_locked': 'Betaalde facturen zijn vergrendeld',
      'source_code': 'Broncode',
      'app_platforms': 'App-platforms',
      'invoice_late': 'Factuur te laat',
      'quote_expired': 'Offerte verlopen',
      'partial_due': 'Te betalen voorschot',
      'invoice_total': 'Factuur totaal',
      'quote_total': 'Offertetotaal',
      'credit_total': 'Totaal krediet',
      'recurring_invoice_total': 'Factuur totaal',
      'actions': 'Acties',
      'expense_number': 'Uitgave nummer',
      'task_number': 'Taaknummer',
      'project_number': 'Projectnummer',
      'project_name': 'Project naam',
      'warning': 'Waarschuwing',
      'view_settings': 'Instellingen tonen',
      'company_disabled_warning':
          'Waarschuwing: dit bedrijf is nog niet geactiveerd',
      'late_invoice': 'Late factuur',
      'expired_quote': 'Verlopen offerte',
      'remind_invoice': 'Herinnering Factuur',
      'cvv': 'CVV',
      'client_name': 'Klantnaam',
      'client_phone': 'Klant telefoon',
      'required_fields': 'Verreisde velden',
      'calculated_rate': 'Berekend tarief',
      'default_task_rate': 'Standaard taak tarief',
      'clear_cache': 'Maak cache leeg',
      'sort_order': 'Sorteer volgorde',
      'task_status': 'Status',
      'task_statuses': 'Taak status',
      'new_task_status': 'Nieuwe taak status',
      'edit_task_status': 'Taak status aanpassen',
      'created_task_status': 'Succesvol een taak status aangemaakt',
      'updated_task_status': 'De taak is gewijzigd',
      'archived_task_status': 'Succesvol een taak status gearchiveerd',
      'deleted_task_status': 'Succesvol een taak status verwijderd',
      'removed_task_status': 'Succesvol een taak status verwijderd',
      'restored_task_status': 'Succesvol een taak status hersteld',
      'archived_task_statuses': 'Succesvol taakstatussen :value gearchiveerd',
      'deleted_task_statuses': 'Succesvol taak statussen :value verwijderd',
      'restored_task_statuses': 'Succesvol taak statussen :value hersteld',
      'search_task_status': 'Zoek 1 taak status',
      'search_task_statuses': 'Zoek :count taak statussen',
      'show_tasks_table': 'Taken tabel tonen',
      'show_tasks_table_help':
          'Weergeef de taken wanneer een factuur wordt aangemaakt',
      'invoice_task_timelog': 'Factuur taak tijdlog',
      'invoice_task_timelog_help':
          'Voeg de tijd omschrijvingen toe aan de factuur producten',
      'invoice_task_datelog': 'Factuur taak datumlog',
      'invoice_task_datelog_help':
          'Voeg datumdetails toe aan de factuurregelitems',
      'auto_start_tasks_help': 'Start taken voordat het wordt opgeslagen',
      'configure_statuses': 'Status instellen',
      'task_settings': 'Taak instellingen',
      'configure_categories': 'Categorieën instellen',
      'expense_categories': 'Uitgavecategorie',
      'new_expense_category': 'Nieuwe uitgavecategorie',
      'edit_expense_category': 'Bewerk uitgavencategorie',
      'created_expense_category': 'De uitgaven categorie is aangemaakt',
      'updated_expense_category': 'De uitgaven categorie is gewijzigd',
      'archived_expense_category': 'De uitgaven categorie is gearchiveerd',
      'deleted_expense_category': 'De categorie is verwijderd',
      'removed_expense_category': 'De uitgavencategorie is verwijderd',
      'restored_expense_category': 'De uitgaven categorie hersteld',
      'archived_expense_categories': ':count uitgave-categorieën gearchiveerd',
      'deleted_expense_categories':
          'Succesvol uitgave categorieën :value verwijderd',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Zoek 1 uitgavencategorie',
      'search_expense_categories': 'Zoek :count uitgave categorieën',
      'use_available_credits': 'Gebruik beschikbaar krediet',
      'show_option': 'Toon optie',
      'negative_payment_error':
          'Het kredietbedrag mag niet hoger zijn als het te betalen bedrag',
      'view_changes': 'Bekijk wijzigingen',
      'force_update': 'Forceer een update',
      'force_update_help':
          'De applicatie draait op de laatste versie, maar wellicht zijn er nog een aantal fixes beschikbaar.',
      'mark_paid_help': 'Volg de uitgave dat betaald is',
      'should_be_invoiced': 'Moet worden gefactureerd',
      'should_be_invoiced_help': 'Maak het mogelijk de uitgave te factureren',
      'add_documents_to_invoice_help': 'Laat de documenten zien',
      'convert_currency_help': 'Stel een ruilwaarde in van de valuta',
      'expense_settings': 'Uitgave instellingen',
      'clone_to_recurring': 'Maak een kopie voor herhaling',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'Gebruiker Veld',
      'variables': 'Variabelen',
      'show_password': 'Wachtwoord weergeven',
      'hide_password': 'Wachtwoord verbergen',
      'copy_error': 'Fout kopiëren',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Automatisch betalen ingeschakeld',
      'total_taxes': 'Totale belasting',
      'line_taxes': 'Regelitem belastingen',
      'total_fields': 'Totaal velden',
      'stopped_recurring_invoice': 'Herhalend factuur succesvol stopgezet',
      'started_recurring_invoice': 'Herhalend factuur succesvol gestart',
      'resumed_recurring_invoice': 'Herhalend factuur succesvol hervat',
      'gateway_refund': 'Gateway terugbetaling',
      'gateway_refund_help':
          'Verwerk een terugbetaling via de betalingsgateway',
      'due_date_days': 'Verloopdatum',
      'paused': 'Gepauzeerd',
      'mark_active': 'Markeer als actief',
      'day_count': 'Dag :count',
      'first_day_of_the_month': 'Eerste dag van de maand',
      'last_day_of_the_month': 'Laatste dag van de maand',
      'use_payment_terms': 'Gebruik betalingseisen',
      'endless': 'Eindeloos',
      'next_send_date': 'Volgende verzenddatum',
      'remaining_cycles': 'Resterende keren',
      'recurring_invoice': 'Terugkerende factuur',
      'recurring_invoices': 'Terugkerende facturen',
      'new_recurring_invoice': 'Nieuwe terugkerende factuur',
      'edit_recurring_invoice': 'Bewerk terugkerende factuur',
      'created_recurring_invoice': 'Herhalend factuur succesvol aangemaakt',
      'updated_recurring_invoice': 'Herhalend factuur succesvol bijgewerkt',
      'archived_recurring_invoice': 'De terugkerende factuur is gearchiveerd',
      'deleted_recurring_invoice': 'De terugkerende factuur is verwijderd',
      'removed_recurring_invoice': 'Herhalend factuur succesvol verwijderd',
      'restored_recurring_invoice': 'De terugkerende factuur is hersteld',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Zoek 1 herhalend factuur',
      'search_recurring_invoices': 'Zoek :count herhalende facturen',
      'send_date': 'Verzenddatum',
      'auto_bill_on': 'Automatische betaling aan',
      'minimum_under_payment_amount': 'Minimum onder het te betalen bedrag',
      'profit': 'Winst',
      'line_item': 'Regelitem',
      'allow_over_payment': 'Toestaan te betalen boven het te betalen bedrag',
      'allow_over_payment_help':
          'Draag bij aan extra betalen om fooi te accepteren',
      'allow_under_payment': 'Toestaan te betalen onder het te betalen bedrag',
      'allow_under_payment_help':
          'Ondersteun het betalen van een minimaal gedeeltelijk / aanbetalingsbedrag',
      'test_mode': 'Test modus',
      'opened': 'Geopend',
      'payment_reconciliation_failure': 'Koppelen mislukt',
      'payment_reconciliation_success': 'Koppelen gelukt',
      'gateway_success': 'Gateway geslaagd',
      'gateway_failure': 'Gateway gefaald',
      'gateway_error': 'Gateway fout',
      'email_send': 'E-mail verzonden',
      'email_retry_queue': 'E-mail wachtrij voor opnieuw versturen',
      'failure': 'Fout',
      'quota_exceeded': 'Limiet bereikt',
      'upstream_failure': 'Upload mislukt',
      'system_logs': 'Systeem log',
      'view_portal': 'Toon portaal',
      'copy_link': 'Link kopiëren',
      'token_billing': 'Kaartgegevens opslaan',
      'welcome_to_invoice_ninja': 'Welkom bij Invoice Ninja',
      'always': 'Altijd',
      'optin': 'Inschrijven',
      'optout': 'Uitschrijven',
      'label': 'Label',
      'client_number': 'Klantnummer',
      'auto_convert': 'Automatisch omzetten',
      'company_name': 'Bedrijfsnaam',
      'reminder1_sent': '1ste herinnering verstuurd',
      'reminder2_sent': '2de herinnering verstuurd',
      'reminder3_sent': '3de herinnering verstuurd',
      'reminder_last_sent': 'Laatste herinnering verstuurd',
      'pdf_page_info': 'Pagina :current van :total',
      'emailed_invoices': 'De facturen zijn gemaild',
      'emailed_quotes': 'De offertes zijn gemaild',
      'emailed_credits': 'Krediet is succesvol gemaild',
      'gateway': 'Gateway',
      'view_in_stripe': 'Bekijk in Stripe',
      'rows_per_page': 'Regels per pagina',
      'hours': 'Uren',
      'statement': 'Overzicht',
      'taxes': 'Belastingen',
      'surcharge': 'Toeslag',
      'apply_payment': 'Betaling toepassen',
      'apply': 'Toepassen',
      'unapplied': 'Niet toegepast',
      'select_label': 'Selecteer label',
      'custom_labels': 'Aangepaste labels',
      'record_type': 'Record Type',
      'record_name': 'Record naam',
      'file_type': 'Bestandstype',
      'height': 'Hoogte',
      'width': 'Breedte',
      'to': 'Aan',
      'health_check': 'Health Check',
      'payment_type_id': 'Betalingstype',
      'last_login_at': 'Voor het laatst ingelogd',
      'company_key': 'Bedrijfssleutel',
      'storefront': 'Storefront',
      'storefront_help':
          'Activeer third-party applicaties om facturen te maken',
      'client_created': 'Klant aangemaakt',
      'online_payment_email': 'Online betalingsmail',
      'manual_payment_email': 'Handmatige betalingsmail',
      'completed': 'Voltooid',
      'gross': 'Bruto',
      'net_amount': 'Netto bedrag',
      'net_balance': 'Netto balans',
      'client_settings': 'Klantinstellingen',
      'selected_invoices': 'Geselecteerde facturen',
      'selected_payments': 'Geselecteerde betalingen',
      'selected_quotes': 'Geselecteerde offertes',
      'selected_tasks': 'Geselecteerde taken',
      'selected_expenses': 'Geselecteerde uitgaves',
      'upcoming_invoices': 'Aankomende facturen',
      'past_due_invoices': 'Verlopen facturen',
      'recent_payments': 'Recente betalingen',
      'upcoming_quotes': 'Eerstvolgende offertes',
      'expired_quotes': 'Verlopen offertes',
      'create_client': 'Klant aanmaken',
      'create_invoice': 'Factuur aanmaken',
      'create_quote': 'Maak offerte aan',
      'create_payment': 'Creëer betaling',
      'create_vendor': 'Leverancier aanmaken',
      'update_quote': 'Offerte bijwerken',
      'delete_quote': 'Verwijder offerte',
      'update_invoice': 'Factuur bijwerken',
      'delete_invoice': 'Verwijder factuur',
      'update_client': 'Klant bijwerken',
      'delete_client': 'Verwijder klant',
      'delete_payment': 'Verwijder betaling',
      'update_vendor': 'Leverancier bijwerken',
      'delete_vendor': 'Verwijder leverancier',
      'create_expense': 'Creëer uitgave',
      'update_expense': 'Uitgave bijwerken',
      'delete_expense': 'Verwijder uitgave',
      'create_task': 'Taak aanmaken',
      'update_task': 'Taak bijwerken',
      'delete_task': 'Verwijder taak',
      'approve_quote': 'Offerte goedkeuren',
      'off': 'Uit',
      'when_paid': 'Wanneer betaald',
      'expires_on': 'Verloopt op',
      'free': 'Gratis',
      'plan': 'Abonnement',
      'show_sidebar': 'Laat zijbalk zien',
      'hide_sidebar': 'Verberg zijbalk',
      'event_type': 'Event Type',
      'target_url': 'Doel',
      'copy': 'Kopieer',
      'must_be_online':
          'Herstart alsjeblieft de applicatie wanneer er verbinding is met het internet',
      'crons_not_enabled': 'De crons moeten geactiveerd worden',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Zoek :count webhooks',
      'search_webhook': 'Zoek 1 webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Nieuwe webhook',
      'edit_webhook': 'Webhook bijwerken',
      'created_webhook': 'Webhook succesvol aangemaakt',
      'updated_webhook': 'Webhook succesvol bijgewerkt',
      'archived_webhook': 'Webhook succesvol gearchiveerd',
      'deleted_webhook': 'Webhook succesvol verwijderd',
      'removed_webhook': 'Webhook succesvol verwijderd',
      'restored_webhook': 'Webhook succesvol hersteld',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Zoek :count tokens',
      'search_token': 'Zoek 1 token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'Nieuwe token',
      'edit_token': 'Wijzig token',
      'created_token': 'Het token is aangemaakt',
      'updated_token': 'Het token is gewijzigd',
      'archived_token': 'Het token is gearchiveerd',
      'deleted_token': 'Het token is verwijderd',
      'removed_token': 'Token succesvol verwijderd',
      'restored_token': 'Token succesvol hersteld',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Klant registratie',
      'client_registration_help':
          'Zelfregistratie voor klanten in het portaal toestaan',
      'customize_and_preview': 'Pas aan & Weergeven',
      'email_invoice': 'E-mail factuur',
      'email_quote': 'E-mail offerte',
      'email_credit': 'E-mail Krediet',
      'email_payment': 'E-mail betaling',
      'client_email_not_set': 'Er is geen e-mailadres ingesteld voor de klant',
      'ledger': 'Grootboek',
      'view_pdf': 'Bekijk PDF',
      'all_records': 'Alle gegevens',
      'owned_by_user': 'Owned door gebruiker',
      'credit_remaining': 'Resterend krediet',
      'contact_name': 'Contactnaam',
      'use_default': 'Gebruik standaard',
      'reminder_endless': 'Eindeloze herinneringen',
      'number_of_days': 'Aantal dagen',
      'configure_payment_terms': 'Betalingsvoorwaarden configureren',
      'payment_term': 'Betalingstermijn',
      'new_payment_term': 'Nieuwe betalingstermijn',
      'edit_payment_term': 'Bewerk betalingstermijn',
      'created_payment_term': 'De betalingstermijn is aangemaakt',
      'updated_payment_term': 'De betalingstermijn is gewijzigd',
      'archived_payment_term': 'De betalingstermijn is gearchiveerd',
      'deleted_payment_term': 'betalingstermijn met succes verwijderd',
      'removed_payment_term': 'betalingstermijn met succes verwijderd',
      'restored_payment_term': 'betalingstermijn met succes hersteld',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Log in met e-mail',
      'change': 'Aanpassen',
      'change_to_mobile_layout': 'Verander naar de mobiele layout?',
      'change_to_desktop_layout': 'Verander naar de bureaublad layout?',
      'send_from_gmail': 'Verzonden vanaf Gmail',
      'reversed': 'Teruggedraaid',
      'cancelled': 'Geannuleerd',
      'credit_amount': 'Kredietbedrag',
      'quote_amount': 'Offertebedrag',
      'hosted': 'Gehost',
      'selfhosted': 'Zelf-Gehost',
      'exclusive': 'Exclusief',
      'inclusive': 'Inclusief',
      'hide_menu': 'Verberg menu',
      'show_menu': 'Toon Menu',
      'partially_refunded': 'Gedeeltelijk terugbetaald',
      'search_documents': 'Documenten zoeken',
      'search_designs': 'Ontwerpen zoeken',
      'search_invoices': 'Facturen zoeken',
      'search_clients': 'Klanten zoeken',
      'search_products': 'Producten zoeken',
      'search_quotes': 'Offertes zoeken',
      'search_credits': 'Zoek Krediet',
      'search_vendors': 'Zoek Leveranciers',
      'search_users': 'Zoek Gebruikers',
      'search_tax_rates': 'Zoek Belastingstarieven',
      'search_tasks': 'Zoek Taken',
      'search_settings': 'Zoek Instellingen',
      'search_projects': 'Zoek Projecten',
      'search_expenses': 'Zoek Uitgaven',
      'search_payments': 'Zoek Betalingen',
      'search_groups': 'Zoek Groepen',
      'search_company': 'Zoek Bedrijf',
      'search_document': 'Zoek 1 document',
      'search_design': 'Zoek 1 ontwerp',
      'search_invoice': 'Zoek 1 factuur',
      'search_client': 'Zoek 1 klant',
      'search_product': 'Zoek 1 product',
      'search_quote': 'Zoek 1 offerte',
      'search_credit': 'Zoek 1 krediet',
      'search_vendor': 'Zoek 1 leverancier',
      'search_user': 'Zoek 1 gebruiker',
      'search_tax_rate': 'Zoek 1 BTW-tarief',
      'search_task': 'Zoek 1 taak',
      'search_project': 'Zoek 1 project',
      'search_expense': 'Zoek 1 uitgave',
      'search_payment': 'Zoek 1 betaling',
      'search_group': 'Zoek 1 groep',
      'refund_payment': 'Terugbetalen',
      'cancelled_invoice': 'Factuur succesvol geannuleerd',
      'cancelled_invoices': 'Facturen succesvol geannuleerd',
      'reversed_invoice': 'Factuur succesvol teruggedraaid',
      'reversed_invoices': 'Facturen succesvol teruggedraaid',
      'reverse': 'Terugdraaien',
      'full_name': 'Volledige naam',
      'city_state_postal': 'Postcode',
      'postal_city_state': 'Provincie',
      'custom1': 'Eerste aangepaste',
      'custom2': 'Tweede aangepaste',
      'custom3': 'Derde aangepaste',
      'custom4': 'Vierde aangepaste',
      'optional': 'Optioneel',
      'license': 'Licentie',
      'purge_data': 'Wis gegevens',
      'purge_successful': 'De bedrijfsgegevens zijn gewist',
      'purge_data_message':
          'Waarschuwing: Dit zal uw gegevens verwijderen. Er is geen manier om dit ongedaan te maken.',
      'invoice_balance': 'Factuur balans',
      'age_group_0': '0 - 30 dagen',
      'age_group_30': '30 - 60 dagen',
      'age_group_60': '60 - 90 dagen',
      'age_group_90': '90 - 120 dagen',
      'age_group_120': '120+ dagen',
      'refresh': 'Verversen',
      'saved_design': 'Ontwerp opgeslagen',
      'client_details': 'Klantgegevens',
      'company_address': 'Bedrijfs-adres',
      'invoice_details': 'Factuur details',
      'quote_details': 'Offerte Details',
      'credit_details': 'Kredietgegevens',
      'product_columns': 'Product kolommen',
      'task_columns': 'Taak kolommen',
      'add_field': 'Veld toevoegen',
      'all_events': 'Alle gebeurtenissen',
      'permissions': 'Rechten',
      'none': 'Geen',
      'owned': 'Eigendom',
      'payment_success': 'Betaling is gelukt',
      'payment_failure': 'Betalingsfout',
      'invoice_sent': ':count factuur verzonden',
      'quote_sent': 'Offerte Verzonden',
      'credit_sent': 'Factuur verzonden',
      'invoice_viewed': 'Factuur bekeken',
      'quote_viewed': 'Offerte Bekeken',
      'credit_viewed': 'Krediet bekeken',
      'quote_approved': 'Offerte Goedgekeurd',
      'receive_all_notifications': 'Ontvang alle notificaties',
      'purchase_license': 'Licentie aanschaffen',
      'apply_license': 'Activeer licentie',
      'cancel_account': 'Account verwijderen',
      'cancel_account_message':
          'Waarschuwing: Dit zal uw account verwijderen. Er is geen manier om dit ongedaan te maken.',
      'delete_company': 'Verwijder bedrijf',
      'delete_company_message':
          'Waarschuwing: Hiermee verwijder je permanent je bedrijf, dit kan niet worden ontdaan.',
      'enabled_modules': 'Ingeschakelde modules',
      'converted_quote': 'Offerte omgezet',
      'credit_design': 'Krediet ontwerp',
      'includes': 'Inclusief',
      'header': 'Koptekst',
      'load_design': 'Laad ontwerp',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Aangepaste Ontwerpen',
      'designs': 'Ontwerpen',
      'new_design': 'Nieuw ontwerp',
      'edit_design': 'Ontwerp aanpassen',
      'created_design': 'Ontwerp aangemaakt',
      'updated_design': 'Ontwerp bijgewerkt',
      'archived_design': 'Ontwerp gearchiveerd',
      'deleted_design': 'Ontwerp verwijderd',
      'removed_design': 'Ontwerp verwijderd',
      'restored_design': 'Ontwerp teruggehaald',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Voorstellen',
      'tickets': 'Tickets',
      'recurring_quotes': 'Terugkerende offertes',
      'recurring_tasks': 'Terugkerende Taken',
      'recurring_expenses': 'Terugkerende uitgaven',
      'account_management': 'Accountbeheer',
      'credit_date': 'Kredietdatum',
      'credit': 'Krediet',
      'credits': 'Kredietnota\'s',
      'new_credit': 'Nieuwe kredietnota',
      'edit_credit': 'Wijzig krediet',
      'created_credit': 'De kredietnota is aangemaakt',
      'updated_credit': 'Het krediet is gewijzigd',
      'archived_credit': 'De kredietnota is gearchiveerd',
      'deleted_credit': 'De kredietnota is verwijderd',
      'removed_credit': 'Krediet is verwijders',
      'restored_credit': 'De kredietnota is hersteld',
      'archived_credits': 'Succesvol :count kredietnota\'s gearchiveerd',
      'deleted_credits': 'Succesvol :count kredietnota\'s verwijderd',
      'restored_credits': ':value aan krediet succesvol hersteld',
      'current_version': 'Huidige versie',
      'latest_version': 'Laatste versie',
      'update_now': 'Nu updaten',
      'a_new_version_is_available':
          'Een nieuwe versie van de web applicatie is beschikbaar',
      'update_available': 'Update beschikbaar',
      'app_updated': 'Update met succes voltooid',
      'learn_more': 'Kom meer te weten',
      'integrations': 'Integraties',
      'tracking_id': 'Tracering Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Krediet voettekst',
      'credit_terms': 'Kredietvoorwaarden',
      'new_company': 'Nieuw bedrijf',
      'added_company': 'Bedrijf toegevoegd',
      'company1': 'Aangepast bedrijf 1',
      'company2': 'Aangepast bedrijf 2',
      'company3': 'Aangepast bedrijf 3',
      'company4': 'Aangepast bedrijf 4',
      'product1': 'Aangepast product 1',
      'product2': 'Aangepast product 2',
      'product3': 'Aangepast product 3',
      'product4': 'Aangepast product 4',
      'client1': 'Aangepast cliënt 1',
      'client2': 'Aangepast cliënt 2',
      'client3': 'Aangepast cliënt 3',
      'client4': 'Aangepast cliënt 4',
      'contact1': 'Aangepast Contact 1',
      'contact2': 'Aangepast Contact 2',
      'contact3': 'Aangepast Contact 3',
      'contact4': 'Aangepast Contact 4',
      'task1': 'Aangepaste Taak 1',
      'task2': 'Aangepaste Taak 2',
      'task3': 'Aangepaste Taak 3',
      'task4': 'Aangepaste Taak 4',
      'project1': 'Aangepast Project 1',
      'project2': 'Aangepast Project 2',
      'project3': 'Aangepast Project 3',
      'project4': 'Aangepast Project 4',
      'expense1': 'Aangepaste Uitgave 1',
      'expense2': 'Aangepaste Uitgave 2',
      'expense3': 'Aangepaste Uitgave 3',
      'expense4': 'Aangepaste Uitgave 4',
      'vendor1': 'Aangepaste Aanbieder 1',
      'vendor2': 'Aangepaste Aanbieder 2',
      'vendor3': 'Aangepaste Aanbieder 3',
      'vendor4': 'Aangepaste Aanbieder 4',
      'invoice1': 'Aangepaste Factuur 1',
      'invoice2': 'Aangepaste Factuur 2',
      'invoice3': 'Aangepaste Factuur 3',
      'invoice4': 'Aangepaste Factuur 4',
      'payment1': 'Aangepaste Betaling 1',
      'payment2': 'Aangepaste Betaling 2',
      'payment3': 'Aangepaste Betaling 3',
      'payment4': 'Aangepaste Betaling 4',
      'surcharge1': 'Aangepaste Toeslag 1',
      'surcharge2': 'Aangepaste Toeslag 2',
      'surcharge3': 'Aangepaste Toeslag 3',
      'surcharge4': 'Aangepaste Toeslag 4',
      'group1': 'Aangepaste Groep 1',
      'group2': 'Aangepaste Groep 2',
      'group3': 'Aangepaste Groep 3',
      'group4': 'Aangepaste Groep 4',
      'reset': 'Reset',
      'number': 'Nummer',
      'export': 'Exporteer',
      'chart': 'Grafiek',
      'count': 'Telling',
      'totals': 'Totalen',
      'blank': 'Blanco',
      'day': 'Dag',
      'month': 'Maand',
      'year': 'Jaar',
      'subgroup': 'Subgroep',
      'is_active': 'Is actief',
      'group_by': 'Groepeer per',
      'credit_balance': 'Kredietsaldo',
      'contact_last_login': 'Contact laatste Login',
      'contact_full_name': 'Contact Volledige Naam',
      'contact_phone': 'Contact telefoon',
      'contact_custom_value1': 'Contact aangepaste waarde 1',
      'contact_custom_value2': 'Contact aangepaste waarde 2',
      'contact_custom_value3': 'Contact aangepaste waarde 3',
      'contact_custom_value4': 'Contact aangepaste waarde 4',
      'shipping_address1': 'Leveringsstraat',
      'shipping_address2': 'Leverings Apt/Suite',
      'shipping_city': 'Leveringsstad',
      'shipping_state': 'Leverings Staat/Provincie',
      'shipping_postal_code': 'Leverings Postcode',
      'shipping_country': 'Leveringsland',
      'billing_address1': 'Facturatie straat',
      'billing_address2': 'Facturatie Apt/Suite',
      'billing_city': 'Facturatiestad',
      'billing_state': 'Facturatie Staat/Provincie',
      'billing_postal_code': 'Facturatie Postcode',
      'billing_country': 'Facturatieland',
      'client_id': 'Klantnummer',
      'assigned_to': 'Toegewezen aan',
      'created_by': 'Aangemaakt door :name',
      'assigned_to_id': 'Toegekend aan ID',
      'created_by_id': 'Gemaakt door ID',
      'add_column': 'Voeg kolom toe',
      'edit_columns': 'Wijzig kolom',
      'columns': 'Kolommen',
      'aging': 'Toekomst',
      'profit_and_loss': 'Winst en verlies',
      'reports': 'Rapporten',
      'report': 'Rapport',
      'add_company': 'Bedrijf toevoegen',
      'unpaid_invoice': 'Onbetaalde factuur',
      'paid_invoice': 'Betaalde factuur',
      'unapproved_quote': 'Niet goedgekeurde offerte',
      'help': 'Help',
      'refund': 'Terugbetaling',
      'refund_date': 'Terugbetaling datum',
      'filtered_by': 'Gefilterd op',
      'contact_email': 'Contact e-mail',
      'multiselect': 'Multiselectie',
      'entity_state': 'Staat',
      'verify_password': 'Verifieer wachtwoord',
      'applied': 'Toegepast',
      'include_recent_errors': 'Voeg recente fouten uit de logboeken toe',
      'your_message_has_been_received':
          'We hebben uw bericht ontvangen, en zullen zo spoedig mogelijk reageren.',
      'message': 'Bericht',
      'from': 'Van',
      'show_product_details': 'toon product details',
      'show_product_details_help':
          'Neem de beschrijving en kosten op in de vervolgkeuzelijst met producten',
      'pdf_min_requirements': 'De PDF renderaar vereist :version',
      'adjust_fee_percent': 'Pas Vergoedingspercentage Aan',
      'adjust_fee_percent_help':
          'Pas percentage aan om rekening te houden met de kosten',
      'configure_settings': 'Instellingen configureren',
      'support_forum': 'Support Forum',
      'about': 'Over',
      'documentation': 'Documentatie',
      'contact_us': 'Contacteer ons',
      'subtotal': 'Subtotaal',
      'line_total': 'Totaal',
      'item': 'Artikel',
      'credit_email': 'Krediet E-mail',
      'iframe_url': 'Website',
      'domain_url': 'Domein URL',
      'password_is_too_short': 'Wachtwoord is te kort',
      'password_is_too_easy':
          'Het wachtwoord moet een hoofdletter en een nummer bevatten',
      'client_portal_tasks': 'Klantenportaal taken',
      'client_portal_dashboard': 'Klantenportaal dashboard',
      'please_enter_a_value': 'Voer alstublieft een waarde in',
      'deleted_logo': 'Logo verwijderd',
      'yes': 'Ja',
      'no': 'Nee',
      'generate_number': 'Genereer nummer',
      'when_saved': 'Als opgeslagen',
      'when_sent': 'Als verzonden',
      'select_company': 'Selecteer Bedrijf',
      'float': 'Float',
      'collapse': 'Inklappen',
      'show_or_hide': 'Laten zien/Verbergen',
      'menu_sidebar': 'Menu Zijbalk',
      'history_sidebar': 'Geschiedenis Zijbalk',
      'tablet': 'Tablet',
      'mobile': 'Mobiel',
      'desktop': 'Bureaublad',
      'layout': 'Indeling',
      'view': 'Bekijken',
      'module': 'Module',
      'first_custom': 'Eerste aangepaste',
      'second_custom': 'Tweede aangepaste',
      'third_custom': 'Derde aangepaste',
      'show_cost': 'Toon kosten',
      'show_product_cost': 'Laat product kosten zien',
      'show_cost_help':
          'Toon het kostenveld van een product om de opmaak / winst te volgen',
      'show_product_quantity': 'Toon product hoeveelheid',
      'show_product_quantity_help':
          'Toon aantallen voor producten, anders de standaard versie',
      'show_invoice_quantity': 'Toon factuur aantallen',
      'show_invoice_quantity_help':
          'Toon aantallen voor regelitem, anders de standaard versie',
      'show_product_discount': 'Toon product korting',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Standaard aantallen',
      'default_quantity_help': 'Stel de producthoeveelheid automatisch in op 1',
      'one_tax_rate': 'Eerste BTW-tarief',
      'two_tax_rates': 'Tweede BTW-tarief',
      'three_tax_rates': 'Derde BTW-tarief',
      'default_tax_rate': 'Standaard BTW-tarief',
      'user': 'Gebruiker',
      'invoice_tax': 'Factuur BTW-tarief',
      'line_item_tax': 'Regelitem BTW-tarief',
      'inclusive_taxes': 'Inclusief belasting',
      'invoice_tax_rates': 'Factuur belastingtarief',
      'item_tax_rates': 'Product belastingtarief',
      'no_client_selected': 'Gelieve een klant te selecteren',
      'configure_rates': 'Tarieven instellen',
      'configure_gateways': 'Configureer Gateways',
      'tax_settings': 'BTW-instellingen',
      'tax_settings_rates': 'BTW-tarieven',
      'accent_color': 'Accent Kleur',
      'switch': 'Overschakelen',
      'comma_sparated_list': 'Komma gescheiden lijst',
      'options': 'Opties',
      'single_line_text': 'Eenregelige tekst',
      'multi_line_text': 'Multi-regelige tekst',
      'dropdown': 'Dropdwon',
      'field_type': 'Veld type',
      'recover_password_email_sent': 'Een wachtwoord herstel mail is verzonden',
      'submit': 'Opslaan',
      'recover_password': 'Wachtwoord vergeten?',
      'late_fees': 'Late vergoedingen',
      'credit_number': 'Kredietnummer',
      'payment_number': 'Betalingsnummer',
      'late_fee_amount': 'Late vergoedingsbedrag',
      'late_fee_percent': 'Late vergoedingspercentage',
      'schedule': 'Schema',
      'before_due_date': 'Voor de vervaldatum',
      'after_due_date': 'Na de vervaldatum',
      'after_invoice_date': 'na de factuurdatum',
      'days': 'Dagen',
      'invoice_email': 'Factuurmail',
      'payment_email': 'Betalingsmail',
      'partial_payment': 'Gedeeltelijke betaling',
      'payment_partial': 'Gedeeltelijke betaling',
      'partial_payment_email': 'E-mail voor gedeeltelijke betaling',
      'quote_email': 'Offertemail',
      'endless_reminder': 'Eindeloze taak',
      'filtered_by_user': 'Gefilterd door gebruiker',
      'administrator': 'Beheerder',
      'administrator_help':
          'Geef gebruiker de toestemming om andere gebruikers te beheren, instellingen te wijzigen en alle regels te bewerken.',
      'user_management': 'Gebruikersbeheer',
      'users': 'Gebruikers',
      'new_user': 'Nieuwe Gebruiker',
      'edit_user': 'Bewerk gebruiker',
      'created_user': 'De gebruiker is aangemaakt',
      'updated_user': 'De gebruiker is gewijzigd',
      'archived_user': 'De gebruiker is gearchiveerd',
      'deleted_user': 'De gebruiker is verwijderd',
      'removed_user': 'Gebruiker verwijderd',
      'restored_user': 'De gebruiker is hersteld',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Algemene instellingen',
      'invoice_options': 'Factuuropties',
      'hide_paid_to_date': 'Verberg \"Reeds betaald\"',
      'hide_paid_to_date_help':
          'Toon alleen het \"Reeds betaald\" gebied op je facturen als er een betaling gemaakt is.',
      'invoice_embed_documents': 'Documenten invoegen',
      'invoice_embed_documents_help':
          'Bijgevoegde afbeeldingen weergeven in de factuur.',
      'all_pages_header': 'Toon header op',
      'all_pages_footer': 'Toon footer op',
      'first_page': 'eerste pagina',
      'all_pages': 'alle pagina\'s',
      'last_page': 'laatste pagina',
      'primary_font': 'Primair lettertype',
      'secondary_font': 'Secundair lettertype',
      'primary_color': 'Primaire kleur',
      'secondary_color': 'Secundaire kleur',
      'page_size': 'Paginagrootte',
      'font_size': 'Tekstgrootte',
      'quote_design': 'Offerte ontwerp',
      'invoice_fields': 'Factuurvelden',
      'product_fields': 'Productvelden',
      'invoice_terms': 'Factuur voorwaarden',
      'invoice_footer': 'Factuurvoettekst',
      'quote_terms': 'Offertevoorwaarden',
      'quote_footer': 'Offertevoettekst',
      'auto_email_invoice': 'Automatisch e-mailen',
      'auto_email_invoice_help':
          'Verzend terugkerende facturen automatisch wanneer ze worden gemaakt.',
      'auto_archive_invoice': 'Automatisch archiveren',
      'auto_archive_invoice_help':
          'Facturen automatisch archiveren wanneer ze worden betaald.',
      'auto_archive_quote': 'Automatisch archiveren',
      'auto_archive_quote_help':
          'Offertes automatisch archiveren wanneer ze zijn omgezet.',
      'auto_convert_quote': 'Automatisch omzetten',
      'auto_convert_quote_help':
          'Zet een offerte automatisch om in een factuur zodra deze door een klant wordt goedgekeurd.',
      'workflow_settings': 'Workflow instellingen',
      'freq_daily': 'Dagelijks',
      'freq_weekly': 'Wekelijks',
      'freq_two_weeks': 'Twee weken',
      'freq_four_weeks': 'Vier weken',
      'freq_monthly': 'Maandelijks',
      'freq_two_months': 'Twee maanden',
      'freq_three_months': 'Drie maanden',
      'freq_four_months': 'Vier maanden',
      'freq_six_months': 'Zes maanden',
      'freq_annually': 'Jaarlijks',
      'freq_two_years': 'Twee jaar',
      'freq_three_years': 'Drie jaar',
      'never': 'Nooit',
      'company': 'Bedrijf',
      'generated_numbers': 'Gegenereerde nummers',
      'charge_taxes': 'BTW berekenen',
      'next_reset': 'Volgende reset',
      'reset_counter': 'Teller resetten',
      'recurring_prefix': 'Terugkerend voorvoegsel',
      'number_padding': 'Nummer afstand',
      'general': 'Algemeen',
      'surcharge_field': 'Extra toeslag veld',
      'company_field': 'Bedrijf veld',
      'company_value': 'Bedrijfswaarde',
      'credit_field': 'Credit veld',
      'invoice_field': 'Factuur veld',
      'invoice_surcharge': 'Factuurkost',
      'client_field': 'Klant veld',
      'product_field': 'Productveld',
      'payment_field': 'Betaalveld',
      'contact_field': 'Contact veld',
      'vendor_field': 'Leverancier veld',
      'expense_field': 'Uitgave veld',
      'project_field': 'Project veld',
      'task_field': 'Taak veld',
      'group_field': 'Groepsveld',
      'number_counter': 'Nummerteller',
      'prefix': 'Voorvoegsel',
      'number_pattern': 'Nummer patroon',
      'messages': 'Berichten',
      'custom_css': 'Aangepaste CSS',
      'custom_javascript': 'Zelfgeschreven JavaScript',
      'signature_on_pdf': 'Weergeven op PDF',
      'signature_on_pdf_help':
          'Toon de handtekening van de klant op de factuur/offerte PDF.',
      'show_accept_invoice_terms': 'Factuurvoorwaarden checkbox',
      'show_accept_invoice_terms_help':
          'Verplicht de klant om akkoord te gaan met de factuurvoorwaarden.',
      'show_accept_quote_terms': 'Offertevoorwaarden checkbox',
      'show_accept_quote_terms_help':
          'Verplicht de klant om akkoord te gaan met de offertevoorwaarden.',
      'require_invoice_signature': 'Factuur handtekening',
      'require_invoice_signature_help':
          'Verplicht de klant om zijn handtekening te zetten.',
      'require_quote_signature': 'Offerte handtekening',
      'enable_portal_password': 'Facturen beveiligen met een wachtwoord',
      'enable_portal_password_help':
          'Geeft u de mogelijkheid om een wachtwoord in te stellen voor elke contactpersoon. Als er een wachtwoord is ingesteld moet de contactpersoon het wachtwoord invoeren voordat deze facturen kan bekijken.',
      'authorization': 'Autorisatie',
      'subdomain': 'Subdomein',
      'domain': 'Domein',
      'portal_mode': 'portaalmodus',
      'email_signature': 'Met vriendelijke groeten,',
      'enable_email_markup_help':
          'Maak het gemakkelijker voor uw klanten om te betalen door scherma.org opmaak toe te voegen aan uw e-mails.',
      'plain': 'Platte tekst',
      'light': 'Licht',
      'dark': 'Donker',
      'email_design': 'E-mail Ontwerp',
      'attach_pdf': 'PDF bijlvoegen',
      'attach_documents': 'Document bijvoegen',
      'attach_ubl': 'UBL bijvoegen',
      'email_style': 'Email opmaak',
      'enable_email_markup': 'Opmaak inschakelen',
      'reply_to_email': 'Antwoord naar e-mail',
      'reply_to_name': 'Antwoordnaam',
      'bcc_email': 'BCC Email',
      'processed': 'Verwerkt',
      'credit_card': 'Creditcard',
      'bank_transfer': 'Overschrijving',
      'priority': 'Prioriteit',
      'fee_amount': 'Vergoedingsbedrag',
      'fee_percent': 'Vergoedingspercentage',
      'fee_cap': 'Maximale vergoeding',
      'limits_and_fees': 'limiet/vergoedingen',
      'enable_min': 'Min inschakelen',
      'enable_max': 'Max inschakelen',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Geaccepteerde kaart logo\'s',
      'credentials': 'Gegevens',
      'update_address': 'Adres aanpassen',
      'update_address_help':
          'Pas het adres van de klant aan met de ingevulde gegevens',
      'rate': 'Tarief',
      'tax_rate': 'BTW-tarief',
      'new_tax_rate': 'Nieuw BTW-tarief',
      'edit_tax_rate': 'Bewerk tarief',
      'created_tax_rate': 'Het tarief is aangemaakt',
      'updated_tax_rate': 'Het tarief is bijgewerkt',
      'archived_tax_rate': 'Het tarief is gearchiveerd',
      'deleted_tax_rate': 'De BTW heffing is verwijderd',
      'restored_tax_rate': 'De BTW heffing is teruggezet',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Producten Automatisch aanvullen',
      'fill_products_help':
          'Een product selecteren zal automatisch de beschrijving en kosten instellen',
      'update_products': 'Producten automatisch wijzigen',
      'update_products_help':
          'Het wijzigen van een factuur zal automatisch de producten aanpassen',
      'convert_products': 'Producten omzetten',
      'convert_products_help':
          'Productprijzen automatisch converteren naar het valuta van de klant',
      'fees': 'Transactiekosten',
      'limits': 'Limieten',
      'provider': 'Provider',
      'company_gateway': 'Betalingsgateway',
      'company_gateways': 'Betalingsgateway',
      'new_company_gateway': 'Nieuwe instantie aanmaken',
      'edit_company_gateway': 'Huidige instantie bewerken',
      'created_company_gateway': 'De nieuwe instantie is aangemaakt',
      'updated_company_gateway': 'De nieuwe instantie is bijgewerkt',
      'archived_company_gateway': 'De nieuwe instantie is gearchiveerd',
      'deleted_company_gateway': 'De nieuwe instantie is verwijderd',
      'restored_company_gateway': 'De nieuwe instantie is hersteld',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Bewerk verder',
      'discard_changes': 'Wis Wijzigingen',
      'default_value': 'Standaard waarde',
      'disabled': 'Uitgeschakeld',
      'currency_format': 'Munt formaat',
      'first_day_of_the_week': 'Eerste dag van de week',
      'first_month_of_the_year': 'Eerste maand van het jaar',
      'sunday': 'Zondag',
      'monday': 'Maandag',
      'tuesday': 'Dinsdag',
      'wednesday': 'Woensdag',
      'thursday': 'Donderdag',
      'friday': 'Vrijdag',
      'saturday': 'Zaterdag',
      'january': 'januari',
      'february': 'februari',
      'march': 'maart',
      'april': 'april',
      'may': 'mei',
      'june': 'juni',
      'july': 'juli',
      'august': 'augustus',
      'september': 'september',
      'october': 'oktober',
      'november': 'november',
      'december': 'december',
      'symbol': 'Symbool',
      'ocde': 'Code',
      'date_format': 'Datum formaat',
      'datetime_format': 'Datum/tijd opmaak',
      'military_time': '24-uurs klok',
      'military_time_help': '24-uurs weergave',
      'send_reminders': 'Verstuur herinneringen',
      'timezone': 'Tijdzone',
      'filtered_by_project': 'Gefilterd op project',
      'filtered_by_group': 'Filteren op groep',
      'filtered_by_invoice': 'Filteren op factuur',
      'filtered_by_client': 'Filteren op klant',
      'filtered_by_vendor': 'Filteren op leverancier',
      'group_settings': 'Groepsinstellingen',
      'group': 'Groep',
      'groups': 'Groep',
      'new_group': 'Nieuwe groep',
      'edit_group': 'Wijzig groep',
      'created_group': 'Nieuwe groep aangemaakt',
      'updated_group': 'Groep gewijzigd',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload logo',
      'uploaded_logo': 'Het logo is opgeslagen',
      'logo': 'Logo',
      'saved_settings': 'De instellingen zijn opgeslagen',
      'product_settings': 'Productinstellingen',
      'device_settings': 'Apparaatinstellingen',
      'defaults': 'Standaardwaarden',
      'basic_settings': 'Basisinstellingen',
      'advanced_settings': 'Geavanceerde instellingen',
      'company_details': 'Bedrijfsdetails',
      'user_details': 'Gebruikersgegevens',
      'localization': 'Lokalisatie',
      'online_payments': 'Online betalingen',
      'tax_rates': 'BTW-tarieven',
      'notifications': 'Notificaties',
      'import_export': 'Importeer/Exporteer',
      'custom_fields': 'Aangepaste velden',
      'invoice_design': 'Factuurontwerp',
      'buy_now_buttons': 'Koop nu knoppen',
      'email_settings': 'E-mailinstellingen',
      'templates_and_reminders': 'Sjablonen en herinneringen',
      'credit_cards_and_banks': 'Credit Cards & Banken',
      'data_visualizations': 'Datavisualisaties',
      'price': 'Prijs',
      'email_sign_up': 'Aanmelden voor email',
      'google_sign_up': 'Aanmelden bij Google',
      'thank_you_for_your_purchase': 'Bedankt voor uw aankoop!',
      'redeem': 'Verzilver',
      'back': 'Terug',
      'past_purchases': 'Voorbije aankopen',
      'annual_subscription': 'Jaarlijks abonnement',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count gebruikers',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Vul een voornaam in aub',
      'please_enter_a_last_name': 'Vul een naam in aub',
      'please_agree_to_terms_and_privacy':
          'Ga akkoord met de servicevoorwaarden en het privacybeleid om een account aan te maken.',
      'i_agree_to_the': 'Ik ga akkoord met',
      'terms_of_service_link': 'de servicevoorwaarden',
      'privacy_policy_link': 'het privacybeleid',
      'terms_of_service': 'Gebruiksvoorwaarden',
      'privacy_policy': 'Privacybeleid',
      'sign_up': 'Aanmelden',
      'account_login': 'Accountlogin',
      'view_website': 'Bekijk website',
      'create_account': 'Account aanmaken',
      'email_login': 'Email login',
      'create_new': 'Nieuwe aanmaken',
      'no_record_selected': 'Geen records geselecteerd',
      'error_unsaved_changes': 'Bewaar of annuleer de wijzigingen',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Vereist een enterprise plan',
      'take_picture': 'Maak foto',
      'upload_file': 'Upload bestand',
      'document': 'Document',
      'documents': 'Documenten',
      'new_document': 'Nieuw document',
      'edit_document': 'Bewerk Document',
      'uploaded_document': 'Document is geupload',
      'updated_document': 'Het document is bijgewerkt',
      'archived_document': 'Het document is gearchiveerd',
      'deleted_document': 'Het document is verwijderd',
      'restored_document': 'Het document is hersteld',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Geen geschiedenis',
      'expense_date': 'Uitgave datum',
      'pending': 'In afwachting',
      'expense_status_1': 'Gelogged',
      'expense_status_2': 'In afwachting',
      'expense_status_3': 'Gefactureerd',
      'converted': 'Omgezet',
      'add_documents_to_invoice': 'Voeg documenten toe aan factuur',
      'exchange_rate': 'Wisselkoers',
      'convert_currency': 'Reken valuta om',
      'mark_paid': 'Markeer als betaald',
      'category': 'Categorie',
      'address': 'Adres',
      'new_vendor': 'Nieuwe leverancier',
      'created_vendor': 'De leverancier is aangemaakt',
      'updated_vendor': 'De leverancier is gewijzigd',
      'archived_vendor': 'De leverancier is gearchiveerd',
      'deleted_vendor': 'De leverancier is verwijderd',
      'restored_vendor': 'De leverancier is hersteld',
      'archived_vendors': 'Succesvol :count leveranciers gearchiveerd',
      'deleted_vendors': 'Succesvol :count leveranciers verwijderd',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Nieuwe uitgave',
      'created_expense': 'De uitgave is aangemaakt',
      'updated_expense': 'De uitgave is gewijzigd',
      'archived_expense': 'De uitgave is gearchiveerd',
      'deleted_expense': 'De uitgave is verwijderd',
      'restored_expense': 'De uitgave is hersteld',
      'archived_expenses': 'De uitgaven zijn gearchiveerd',
      'deleted_expenses': 'De uitgaven zijn verwijderd',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Levering kopiëren',
      'copy_billing': 'Facturatie kopiëren',
      'design': 'Ontwerp',
      'failed_to_find_record': 'Geen gegeven gevonden',
      'invoiced': 'Gefactureerd',
      'logged': 'Gelogd',
      'running': 'Lopend',
      'resume': 'Doorgaan',
      'task_errors': 'Pas overlappende tijden aan a.u.b..',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'De taak is gestart',
      'stopped_task': 'De taak is gestopt',
      'resumed_task': 'Taak hervat',
      'now': 'Nu',
      'auto_start_tasks': 'Automatisch Startende Taken',
      'timer': 'Timer',
      'manual': 'Manueel',
      'budgeted': 'Begroot',
      'start_time': 'Starttijd',
      'end_time': 'Eindtijd',
      'date': 'Datum',
      'times': 'Tijden',
      'duration': 'Duur',
      'new_task': 'Nieuwe taak',
      'created_task': 'De taak is aangemaakt',
      'updated_task': 'De taak is gewijzigd',
      'archived_task': 'De taak is gearchiveerd',
      'deleted_task': 'De taak is verwijderd',
      'restored_task': 'De taak is hersteld',
      'archived_tasks': 'Succesvol :count taken gearchiveerd',
      'deleted_tasks': 'Succesvol :count taken verwijderd',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Geef a.u.b. een naam op',
      'budgeted_hours': 'Begrote uren',
      'created_project': 'Het project is aangemaakt',
      'updated_project': 'Het project is gewijzigd',
      'archived_project': 'Het project is gearchiveerd',
      'deleted_project': 'Het project is verwijderd',
      'restored_project': 'Het project is hersteld',
      'archived_projects': 'Succesvol :count projecten gearchiveerd',
      'deleted_projects': 'Succesvol :count projecten verwijderd',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nieuw project',
      'thank_you_for_using_our_app': 'Bedankt voor het gebruik van onze app!',
      'if_you_like_it': 'Als je het leuk vindt alsjeblieft',
      'click_here': 'Klik hier',
      'click_here_capital': 'Klik hier',
      'to_rate_it': 'om een score te geven.',
      'average': 'Gemiddeld',
      'unapproved': 'Afgekeurd',
      'authenticate_to_change_setting':
          'Gelieve te authenticeren om deze instelling te wijzigen',
      'locked': 'Vergrendeld',
      'authenticate': 'Authenticeer',
      'please_authenticate': 'Gelieve te authenticeren',
      'biometric_authentication': 'Biometrische authenticatie',
      'footer': 'Voettekst',
      'compare': 'Vergelijk',
      'hosted_login': 'Hosted login',
      'selfhost_login': 'Self-Host login',
      'google_sign_in': 'Log in met Google',
      'today': 'Vandaag',
      'custom_range': 'Aangepast bereik',
      'date_range': 'Datumbereik',
      'current': 'Huidige',
      'previous': 'Vorige',
      'current_period': 'Huidige Periode',
      'comparison_period': 'Periode om mee te vergelijken',
      'previous_period': 'Vorige Periode',
      'previous_year': 'Vorig jaar',
      'compare_to': 'Vergelijk met',
      'last7_days': 'Laatste 7 dagen',
      'last_week': 'Afgelopen week',
      'last30_days': 'Laatste 30 Dagen',
      'this_month': 'Deze maand',
      'last_month': 'Vorige maand',
      'this_year': 'Dit jaar',
      'last_year': 'Vorig jaar',
      'custom': 'Aangepast',
      'clone_to_invoice': 'Dupliceer als factuur',
      'clone_to_quote': 'Dupliceer als offerte',
      'clone_to_credit': 'Klonen naar krediet',
      'view_invoice': 'Bekijk factuur',
      'convert': 'Converteer',
      'more': 'Meer',
      'edit_client': 'Wijzig klant',
      'edit_product': 'Wijzig product',
      'edit_invoice': 'Wijzig factuur',
      'edit_quote': 'Bewerk offerte',
      'edit_payment': 'Bewerk betaling',
      'edit_task': 'Wijzig taak',
      'edit_expense': 'Bewerk uitgave',
      'edit_vendor': 'Bewerk leverancier',
      'edit_project': 'Wijzig project',
      'edit_recurring_expense': 'Terugkerende uitgave bewerken',
      'edit_recurring_quote': 'Bewerk terugkerende offerte',
      'billing_address': 'Factuuradres',
      'shipping_address': 'Leveringsadres',
      'total_revenue': 'Totale inkomsten',
      'average_invoice': 'Gemiddelde factuur',
      'outstanding': 'Uitstaand',
      'invoices_sent': 'facturen verzonden',
      'active_clients': 'Actieve klanten',
      'close': 'Sluiten',
      'email': 'E-mail',
      'password': 'Wachtwoord',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Naam',
      'logout': 'Afmelden',
      'login': 'Login',
      'filter': 'Filter',
      'sort': 'Sorteer',
      'search': 'Zoeken',
      'active': 'Actief',
      'archived': 'Gearchiveerd',
      'deleted': 'Verwijderd',
      'dashboard': 'Dashboard',
      'archive': 'Archiveer',
      'delete': 'Verwijder',
      'restore': 'Herstel',
      'refresh_complete': 'Verversen afgerond',
      'please_enter_your_email': 'Gelieve uw e-maildres in te vullen',
      'please_enter_your_password': 'Gelieve uw wachtwoord in te voeren',
      'please_enter_your_url': 'Gelieve uw URL in te voeren',
      'please_enter_a_product_key': 'Gelieve een productcode in te voeren',
      'ascending': 'Oplopend',
      'descending': 'Aflopend',
      'save': 'Opslaan',
      'an_error_occurred': 'Er is een fout opgetreden',
      'paid_to_date': 'Reeds betaald',
      'balance_due': 'Te voldoen',
      'balance': 'Saldo',
      'overview': 'Overzicht',
      'details': 'Details',
      'phone': 'Telefoon',
      'website': 'Website',
      'vat_number': 'BTW-nummer',
      'id_number': 'Identificatienummer',
      'create': 'Aanmaken',
      'copied_to_clipboard': 'Waarde :value naar klembord gekopieerd',
      'error': 'Fout',
      'could_not_launch': 'Kon niet starten',
      'contacts': 'Contactpersonen',
      'additional': 'Extra',
      'first_name': 'Voornaam',
      'last_name': 'Achternaam',
      'add_contact': 'Contact toevoegen',
      'are_you_sure': 'Weet je het zeker?',
      'cancel': 'Annuleren',
      'ok': 'OK',
      'remove': 'Verwijderen',
      'email_is_invalid': 'E-mailadres is incorrect',
      'product': 'Product',
      'products': 'Producten',
      'new_product': 'Nieuw product',
      'created_product': 'Het product is aangemaakt',
      'updated_product': 'Het product is gewijzigd',
      'archived_product': 'Het product is gearchiveerd',
      'deleted_product': 'Het product is verwijderd',
      'restored_product': 'Het product is hersteld',
      'archived_products': 'Succesvol :count producten gearchiveerd',
      'deleted_products': 'Succesvol :count producten verwijderd',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Notities',
      'cost': 'Kosten',
      'client': 'Klant',
      'clients': 'Klanten',
      'new_client': 'Nieuwe klant',
      'created_client': 'De klant is aangemaakt',
      'updated_client': 'De klant is bijgewerkt',
      'archived_client': 'De klant is gearchiveerd',
      'archived_clients': 'Succesvol  :count klanten gearchiveerd',
      'deleted_client': 'De klant is verwijderd',
      'deleted_clients': 'Succesvol :count klanten verwijderd',
      'restored_client': 'De klant is hersteld',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Straat',
      'address2': 'Apt/Suite',
      'city': 'Plaats',
      'state': 'Provincie',
      'postal_code': 'Postcode',
      'country': 'Land',
      'invoice': 'Factuur',
      'invoices': 'Facturen',
      'new_invoice': 'Nieuwe factuur',
      'created_invoice': 'De factuur is aangemaakt',
      'updated_invoice': 'De factuur is gewijzigd',
      'archived_invoice': 'De factuur is gearchiveerd',
      'deleted_invoice': 'De factuur is verwijderd',
      'restored_invoice': 'De factuur is hersteld',
      'archived_invoices': 'Succesvol :count facturen gearchiveerd',
      'deleted_invoices': 'De :count facturen zijn verwijderd',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'De factuur is gemaild',
      'emailed_payment': 'De betaling is per mail verstuurd',
      'amount': 'Bedrag',
      'invoice_number': 'Factuurnummer',
      'invoice_date': 'Factuurdatum',
      'discount': 'Korting',
      'po_number': 'Bestelnummer',
      'terms': 'Voorwaarden',
      'public_notes': 'Publieke opmerkingen',
      'private_notes': 'Prive notities',
      'frequency': 'Frequentie',
      'start_date': 'Startdatum',
      'end_date': 'Einddatum',
      'quote_number': 'Offertenummer',
      'quote_date': 'Offertedatum',
      'valid_until': 'Geldig tot',
      'items': 'Artikelen',
      'partial_deposit': 'Voorschot',
      'description': 'Omschrijving',
      'unit_cost': 'Eenheidsprijs',
      'quantity': 'Aantal',
      'add_item': 'Artikel toevoegen',
      'contact': 'Contact',
      'work_phone': 'Telefoon',
      'total_amount': 'Totaal hoeveelheid',
      'pdf': 'PDF',
      'due_date': 'Vervaldatum',
      'partial_due_date': 'Gedeeltelijke vervaldatum',
      'status': 'Status',
      'invoice_status_id': 'Factuurstatus',
      'quote_status': 'Offertestatus',
      'click_plus_to_add_item': 'Klik op + om een artikel toe te voegen',
      'click_plus_to_add_time': 'Klik + om tijd toe te voegen',
      'count_selected': ':count geselecteerd',
      'total': 'Totaal',
      'percent': 'Procent',
      'edit': 'Bewerk',
      'dismiss': 'Seponeren',
      'please_select_a_date': 'Gelieve een datum selecteren',
      'please_select_a_client': 'Gelieve een klant te selecteren',
      'please_select_an_invoice': 'Selecteer een factuur',
      'task_rate': 'Taak tarief',
      'settings': 'Instellingen',
      'language': 'Taal',
      'currency': 'Munteenheid',
      'created_at': 'Aanmaakdatum',
      'created_on': 'Aangemaakt op',
      'updated_at': 'Bijgewerkt',
      'tax': 'Belasting',
      'please_enter_an_invoice_number':
          'Gelieve een factuurnummer in te voeren',
      'please_enter_a_quote_number': 'Gelieve een offertenummer in te voeren',
      'past_due': 'Verlopen',
      'draft': 'Concept',
      'sent': 'Verzonden',
      'viewed': 'Bekenen',
      'approved': 'Goedgekeurd',
      'partial': 'Voorschot',
      'paid': 'Betaald',
      'mark_sent': 'Markeer als verzonden',
      'marked_invoice_as_sent': 'De factuur is gemarkeerd als verzonden',
      'marked_invoice_as_paid': 'Factuur succesvol gemarkeerd als verzonden',
      'marked_invoices_as_sent': 'Facturen gemarkeerd als verzonden',
      'marked_invoices_as_paid': 'Facturen succesvol gemarkeerd als verzonden',
      'done': 'Klaar',
      'please_enter_a_client_or_contact_name':
          'Gelieve een bedrijfsnaam of contactpersoon in te voeren',
      'dark_mode': 'Donkere modus',
      'restart_app_to_apply_change':
          'Herstart de applicatie om de wijziging toe te passen',
      'refresh_data': 'Gegevens verversen',
      'blank_contact': 'Leeg contact',
      'activity': 'Activiteit',
      'no_records_found': 'Geen gegevens gevonden',
      'clone': 'Dupliceer',
      'loading': 'Laden',
      'industry': 'Industrie',
      'size': 'Grootte',
      'payment_terms': 'Betalingsvoorwaarden',
      'payment_date': 'Betalingsdatum',
      'payment_status': 'Betaalstatus',
      'payment_status_1': 'In afwachting',
      'payment_status_2': 'Ongeldig',
      'payment_status_3': 'Mislukt',
      'payment_status_4': 'Voltooid',
      'payment_status_5': 'Deels terugbetaald',
      'payment_status_6': 'Gecrediteerd',
      'payment_status_-1': 'Niet toegepast',
      'payment_status_-2': 'Gedeeltelijk niet toegepast',
      'net': 'Betaaltermijn',
      'client_portal': 'Klantenportaal',
      'show_tasks': 'Toon taken',
      'email_reminders': 'E-mail herinneringen',
      'enabled': 'Ingeschakeld',
      'recipients': 'Ontvangers',
      'initial_email': 'Initiële e-mail',
      'first_reminder': 'Eerste herinnering',
      'second_reminder': 'Tweede herinnering',
      'third_reminder': 'Derde herinnering',
      'reminder1': 'Eerste herinnering',
      'reminder2': 'Tweede herinnering',
      'reminder3': 'Derde herinnering',
      'template': 'Sjabloon',
      'send': 'Verstuur',
      'subject': 'Onderwerp',
      'body': 'Tekst',
      'send_email': 'Verstuur e-mail',
      'email_receipt': 'Mail betalingsbewijs naar de klant',
      'auto_billing': 'Automatisch incasseren',
      'button': 'Knop',
      'preview': 'Voorbeeld',
      'customize': 'Aanpassen',
      'history': 'Geschiedenis',
      'payment': 'Betaling',
      'payments': 'Betalingen',
      'refunded': 'Gecrediteerd',
      'payment_type': 'Betalingswijze',
      'transaction_reference': 'Transactie referentie',
      'enter_payment': 'Voer betaling in',
      'new_payment': 'Nieuwe betaling',
      'created_payment': 'De betaling is aangemaakt',
      'updated_payment': 'De betaling is gewijzigd',
      'archived_payment': 'De betaling is gearchiveerd',
      'deleted_payment': 'De betaling is verwijderd',
      'restored_payment': 'De betaling is hersteld',
      'archived_payments': 'Succesvol :count betalingen gearchiveerd',
      'deleted_payments': 'Succesvol :count betalingen verwijderd',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Offerte',
      'quotes': 'Offertes',
      'new_quote': 'Nieuwe offerte',
      'created_quote': 'De offerte is aangemaakt',
      'updated_quote': 'De offerte is gewijzigd',
      'archived_quote': 'De offerte is gearchiveerd',
      'deleted_quote': 'De offerte is verwijderd',
      'restored_quote': 'De offerte is hersteld',
      'archived_quotes': 'Succesvol :count offertes gearchiveerd',
      'deleted_quotes': 'Succesvol :count offertes verwijderd',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Uitgave',
      'expenses': 'Uitgaven',
      'vendor': 'Leverancier',
      'vendors': 'Leveranciers',
      'task': 'Taak',
      'tasks': 'Taken',
      'project': 'Project',
      'projects': 'Projecten',
      'activity_1': ':user heeft klant :client aangemaakt',
      'activity_2': ':user heeft klant :client gearchiveerd',
      'activity_3': ':user heeft klant :client verwijderd',
      'activity_4': ':user heeft factuur :invoice aangemaakt',
      'activity_5': ':user heeft factuur :invoice bijgewerkt',
      'activity_6':
          ':user heeft factuur :invoice voor :client naar :contact verstuurd',
      'activity_7': ':contact heeft factuur :invoice voor :client bekeken',
      'activity_8': ':user heeft factuur :invoice gearchiveerd',
      'activity_9': ':user heeft factuur :invoice verwijderd',
      'activity_10':
          ':contact heeft betaling :payment van :payment_amount ingevoerd voor factuur :invoice voor :client',
      'activity_11': ':user heeft betaling :payment bijgewerkt',
      'activity_12': ':user heeft betaling :payment gearchiveerd',
      'activity_13': ':user heeft betaling :payment verwijderd',
      'activity_14': ':user heeft :credit krediet ingevoerd',
      'activity_15': ':user heeft :credit krediet bijgewerkt',
      'activity_16': ':user heeft :credit krediet gearchiveerd',
      'activity_17': ':user heeft :credit krediet verwijderd',
      'activity_18': ':user heeft offerte :quote aangemaakt',
      'activity_19': ':user heeft offerte :quote bijgewerkt',
      'activity_20':
          ':user heeft offerte :quote voor :client verstuurd naar :contact',
      'activity_21': ':contact heeft offerte :quote bekeken',
      'activity_22': ':user heeft offerte :quote gearchiveerd',
      'activity_23': ':user heeft offerte :quote verwijderd',
      'activity_24': ':user heeft offerte :quote hersteld',
      'activity_25': ':user heeft factuur :invoice hersteld',
      'activity_26': ':user heeft klant :client hersteld',
      'activity_27': ':user heeft betaling :payment hersteld',
      'activity_28': ':user heeft :credit krediet hersteld',
      'activity_29': ':contact heeft offerte :quote goedgekeurd voor :client',
      'activity_30': ':user heeft leverancier :vendor aangemaakt',
      'activity_31': ':user heeft leverancier :vendor gearchiveerd',
      'activity_32': ':user heeft leverancier :vendor verwijderd',
      'activity_33': ':user heeft leverancier :vendor hersteld',
      'activity_34': ':user heeft uitgave :expense aangemaakt',
      'activity_35': ':user heeft uitgave :expense gearchiveerd',
      'activity_36': ':user heeft uitgave :expense verwijderd',
      'activity_37': ':user heeft uitgave :expense hersteld',
      'activity_39':
          ':user heeft een a :payment_amount betaling geannuleerd :payment',
      'activity_40':
          ':user heeft :adjustment van een :payment_amount betaling :payment',
      'activity_41': 'Betaling van :payment_amount mislukt (:payment)',
      'activity_42': ':user heeft taak :task aangemaakt',
      'activity_43': ':user heeft taak :task bijgewerkt',
      'activity_44': ':user heeft taak :task gearchiveerd',
      'activity_45': ':user heeft taak :task verwijderd',
      'activity_46': ':user heeft taak :task hersteld',
      'activity_47': ':user heeft uitgave :expense bijgewerkt',
      'activity_48': ':user heeft ticket :ticket bijgewerkt',
      'activity_49': ':user heeft ticket :ticket gesloten',
      'activity_50': ':user heeft ticket :ticket samengevoegd',
      'activity_51': ':user heeft ticket :ticket gesplitst',
      'activity_52': ':contact heeft ticket :ticket geopend',
      'activity_53': ':contact heeft ticket :ticket heropend',
      'activity_54': ':user heeft ticket :ticket heropend',
      'activity_55': ':contact heeft op ticket :ticket gereageerd',
      'activity_56': ':user heeft ticket :ticket bekeken',
      'activity_57': 'Systeem kon de factuur niet mailen :invoice',
      'activity_58': ':gebruiker teruggedraaide factuur :factuur',
      'activity_59': ':gebruiker geannuleerde factuur :factuur',
      'activity_60': ':contact heeft de offerte :quote bekeken',
      'activity_61': ':user heeft de klant :client aangepast',
      'activity_62': ':user heeft de leverancier :vendor gewijzigd',
      'activity_63':
          ':user heeft de eerste herinnering voor factuur :invoice naar :contact verzonden',
      'activity_64':
          ':user heeft de tweede herinnering voor factuur :invoice naar :contact verzonden',
      'activity_65':
          ':user heeft de derde herinnering voor factuur :invoice naar :contact verzonden',
      'activity_66':
          ':user heeft eindeloze herinneringen voor factuur :invoice naar :contact verzonden',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Eenmalig wachtwoord',
      'emailed_quote': 'De offerte is gemaild',
      'emailed_credit': 'Krediet is verzonden',
      'marked_quote_as_sent': 'De offerte is  gemarkeerd als verzonden',
      'marked_credit_as_sent': 'Krediet is gemarkeerd als verzonden',
      'expired': 'Verlopen',
      'all': 'Alles',
      'select': 'Selecteer',
      'long_press_multiselect': 'Lang indrukken multiselect',
      'custom_value1': 'Aangepaste waarde',
      'custom_value2': 'Aangepaste waarde',
      'custom_value3': 'Aangepaste waarde 3',
      'custom_value4': 'Aangepaste waarde 4',
      'email_style_custom': 'Aangepaste Email Stijl',
      'custom_message_dashboard': 'Aangepast bericht Dashboard',
      'custom_message_unpaid_invoice': 'Aangepast bericht Onbetaalde Factuur',
      'custom_message_paid_invoice': 'Aangepast bericht Betaalde Factuur',
      'custom_message_unapproved_quote':
          'Aangepast bericht Niet goedgekeurde Offerte',
      'lock_invoices': 'Vergrendel facturen',
      'translations': 'Vertalingen',
      'task_number_pattern': 'Taaknummer patroon',
      'task_number_counter': 'Taaknummer teller',
      'expense_number_pattern': 'Uitgave nummer patroon',
      'expense_number_counter': 'Uitgave nummer teller',
      'vendor_number_pattern': 'Leverancier nummer patroon',
      'vendor_number_counter': 'Leverancier nummer teller',
      'ticket_number_pattern': 'Ticket nummer patroon',
      'ticket_number_counter': 'Ticket nummer teller',
      'payment_number_pattern': 'Betalingsnummer patroon',
      'payment_number_counter': 'Betalingsnummer teller',
      'invoice_number_pattern': 'Factuur nummer patroon',
      'invoice_number_counter': 'Factuurnummerteller',
      'quote_number_pattern': 'Offertenummer teller',
      'quote_number_counter': 'Offertenummerteller',
      'client_number_pattern': 'Kredietnummer patroon',
      'client_number_counter': 'Kredietnummer teller',
      'credit_number_pattern': 'Kredietnummer patroon',
      'credit_number_counter': 'Kredietnummer teller',
      'reset_counter_date': 'Teller datum resetten',
      'counter_padding': 'Teller patroon',
      'shared_invoice_quote_counter': 'Gedeelde factuur offerte teller',
      'default_tax_name_1': 'Standaard BTW naam 1',
      'default_tax_rate_1': 'Standaard BTW-tarief 1',
      'default_tax_name_2': 'Standaard BTW naam 2',
      'default_tax_rate_2': 'Standaard BTW-tarief 2',
      'default_tax_name_3': 'Standaard BTW naam 3',
      'default_tax_rate_3': 'Standaard BTW-tarief 3',
      'email_subject_invoice': 'E-mail factuur onderwerp',
      'email_subject_quote': 'E-mail offerte onderwerp',
      'email_subject_payment': 'E-mail betaling onderwerp',
      'email_subject_payment_partial':
          'E-mail gedeeltelijke betalingsonderwerp',
      'show_table': 'Weergeef als tabel',
      'show_list': 'Weergeef als lijst',
      'client_city': 'Klant stad',
      'client_state': 'Klant provincie',
      'client_country': 'Land van de klant',
      'client_is_active': 'Klant is actief',
      'client_balance': 'Klanten balans',
      'client_address1': 'Klant straat',
      'client_address2': 'Klant apt/suite',
      'vendor_address1': 'Leverancier straatnaam',
      'vendor_address2': 'Leverancier Apt / Suite',
      'client_shipping_address1': 'Klant leveringsadres',
      'client_shipping_address2': 'Klant leverings Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Factuurbedrag',
      'invoice_due_date': 'Vervaldatum',
      'tax_rate1': 'BTW-tarief 1',
      'tax_rate2': 'BTW-tarief 2',
      'tax_rate3': 'BTW-tarief 3',
      'auto_bill': 'Automatische incasso',
      'archived_at': 'Gearchiveerd op',
      'has_expenses': 'Heeft uitgaves',
      'custom_taxes1': 'Aangepaste Belastingen 1',
      'custom_taxes2': 'Aangepaste Belastingen 2',
      'custom_taxes3': 'Aangepaste Belastingen 3',
      'custom_taxes4': 'Aangepaste Belastingen 4',
      'custom_surcharge1': 'Aangepaste Toeslag 1',
      'custom_surcharge2': 'Aangepaste Toeslag 2',
      'custom_surcharge3': 'Aangepaste Toeslag 3',
      'custom_surcharge4': 'Aangepaste Toeslag 4',
      'is_deleted': 'Is verwijderd',
      'vendor_city': 'Stad van de klant',
      'vendor_state': 'Leverancier provincie',
      'vendor_country': 'Land van de verkoper',
      'is_approved': 'Is goedgekeurd',
      'tax_name': 'Belasting naam',
      'tax_amount': 'BTW',
      'tax_paid': 'Betaalde Belasting',
      'payment_amount': 'Betalingsbedrag',
      'age': 'Leeftijd',
      'is_running': 'Word uitgevoerd',
      'time_log': 'Tijdschema',
      'bank_id': 'Bank',
      'expense_category_id': 'Uitgave categorie ID',
      'expense_category': 'Uitgavecategorie',
      'invoice_currency_id': 'Factuur valuta ID',
      'tax_name1': 'BTW naam 1',
      'tax_name2': 'BTW naam 2',
      'tax_name3': 'BTW naam 3',
      'transaction_id': 'Transactie ID',
      'color_theme': 'Kleuren thema',
    },
    'en_AU': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Enable',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Customer ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the barcode with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convert to Invoice',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Invoice Task',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable customers to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Colour',
      'show': 'Show',
      'hide': 'Hide',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Column',
      'sample': 'Sample',
      'map_to': 'Map To',
      'import': 'Import',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Please select a file',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV file',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Invoice Total',
      'quote_total': 'Quote Total',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not been activated yet ',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Customer Name',
      'client_phone': 'Customer Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Time Log',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark as Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Recurring Invoice',
      'recurring_invoices': 'Recurring Invoices',
      'new_recurring_invoice': 'New Recurring Invoice',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Billing On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Save card details',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Customer Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hours',
      'statement': 'Statement',
      'taxes': 'Taxes',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'To',
      'health_check': 'Health Check',
      'payment_type_id': 'Payment Type',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Allow third-party apps to create invoices',
      'client_created': 'Customer Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Customer Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Upcoming Invoices',
      'past_due_invoices': 'Overdue Invoices',
      'recent_payments': 'Recent Payments',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Customer',
      'create_invoice': 'Create Invoice',
      'create_quote': 'Create Quote',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create supplier',
      'update_quote': 'Update Quote',
      'delete_quote': 'Delete Quote',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Delete Invoice',
      'update_client': 'Update Customer',
      'delete_client': 'Delete Customer',
      'delete_payment': 'Delete Payment',
      'update_vendor': 'Update Supplier',
      'delete_vendor': 'Delete Supplier',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Create Task',
      'update_task': 'Update Task',
      'delete_task': 'Delete Task',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Edit Token',
      'created_token': 'Successfully created token',
      'updated_token': 'Successfully updated token',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Successfully deleted token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Customer Registration',
      'client_registration_help':
          'Enable customers to self register in the portal',
      'customize_and_preview': 'Customise & Preview',
      'email_invoice': 'Email Invoice',
      'email_quote': 'Email Quote',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Customer does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Credit Amount',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Customers',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Suppliers',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Customer',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Supplier',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postcode',
      'postal_city_state': 'Postcode/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Customer Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count invoice sent',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Delete Account',
      'cancel_account_message':
          'Warning: This will permanently delete your account, there is no undo.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Credit Date',
      'credit': 'Credit',
      'credits': 'Credits',
      'new_credit': 'Enter Credit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Successfully created credit',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Successfully archived credit',
      'deleted_credit': 'Successfully deleted credit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Successfully restored credit',
      'archived_credits': 'Successfully archived :count credits',
      'deleted_credits': 'Successfully deleted :count credits',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Current version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Learn more',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'New Company',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Customer 1',
      'client2': 'Custom Customer 2',
      'client3': 'Custom Customer 3',
      'client4': 'Custom Customer 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Supplier 1',
      'vendor2': 'Custom Supplier 2',
      'vendor3': 'Custom Supplier 3',
      'vendor4': 'Custom Supplier 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Chart',
      'count': 'Count',
      'totals': 'Totals',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Group by',
      'credit_balance': 'Credit Balance',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Delivery Street',
      'shipping_address2': 'Delivery Unit/Suite',
      'shipping_city': 'Delivery Town/Suburb',
      'shipping_state': 'Delivery State',
      'shipping_postal_code': 'Delivery Postcode',
      'shipping_country': 'Delivery Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Unit/Suite',
      'billing_city': 'Billing Town/Suburb',
      'billing_state': 'Billing State',
      'billing_postal_code': 'Billing Postcode',
      'billing_country': 'Billing Country',
      'client_id': 'Customer Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Report',
      'add_company': 'Add Company',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Help',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multi-select',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Message',
      'from': 'From',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Documentation',
      'contact_us': 'Contact Us',
      'subtotal': 'Subtotal',
      'line_total': 'Line Total',
      'item': 'Item',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Customer Portal Tasks',
      'client_portal_dashboard': 'Customer Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive of Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a customer',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Tax Settings',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Colour',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Recover your password',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Invoice Email',
      'payment_email': 'Payment Email',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Quote Email',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'User Management',
      'users': 'Users',
      'new_user': 'New User',
      'edit_user': 'Edit User',
      'created_user': 'Successfully created user',
      'updated_user': 'Successfully updated user',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Successfully deleted user',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Successfully restored user',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'General Settings',
      'invoice_options': 'Invoice Options',
      'hide_paid_to_date': 'Hide Paid to Date',
      'hide_paid_to_date_help':
          'Only display the \"Paid to Date\" area on your invoices once a payment has been received.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primary Colour',
      'secondary_color': 'Secondary Colour',
      'page_size': 'Page Size',
      'font_size': 'Font Size',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Invoice Fields',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Invoice Terms',
      'invoice_footer': 'Invoice Footer',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a customer.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Fortnightly',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Charge taxes',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Customer Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Supplier Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Custom CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the customer signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require customer to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require customer to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require customer to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorisation',
      'subdomain': 'Subdomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Regards,',
      'enable_email_markup_help':
          'Make it easier for your customers to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percentage',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Update Address',
      'update_address_help': 'Update customer\'s address with provided details',
      'rate': 'Rate',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Edit tax rate',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Successfully archived the tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-fill products',
      'fill_products_help':
          'Selecting a product will automatically fill in the description and price',
      'update_products': 'Auto-update products',
      'update_products_help':
          'Updating an invoice will automatically update the product library',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the customer\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Sunday',
      'monday': 'Monday',
      'tuesday': 'Tuesday',
      'wednesday': 'Wednesday',
      'thursday': 'Thursday',
      'friday': 'Friday',
      'saturday': 'Saturday',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Hour Time',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Customer',
      'filtered_by_vendor': 'Filtered by Supplier',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Product Settings',
      'device_settings': 'Device Settings',
      'defaults': 'Defaults',
      'basic_settings': 'Basic Settings',
      'advanced_settings': 'Advanced Settings',
      'company_details': 'Company Details',
      'user_details': 'User Details',
      'localization': 'Localisation',
      'online_payments': 'Online Payments',
      'tax_rates': 'Tax Rates',
      'notifications': 'Notifications',
      'import_export': 'Import | Export',
      'custom_fields': 'Custom Fields',
      'invoice_design': 'Invoice Design',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Email Settings',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Data Visualisations',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Terms of Service',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Sign Up',
      'account_login': 'Account Login',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'Pending',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Converted',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Mark as Paid',
      'category': 'Category',
      'address': 'Address',
      'new_vendor': 'New Supplier',
      'created_vendor': 'Successfully created supplier',
      'updated_vendor': 'Successfully updated supplier',
      'archived_vendor': 'Successfully archived supplier',
      'deleted_vendor': 'Successfully deleted supplier',
      'restored_vendor': 'Successfully restored supplier',
      'archived_vendors': 'Successfully archived :count suppliers',
      'deleted_vendors': 'Successfully deleted :count suppliers',
      'restored_vendors': 'Successfully restored :value suppliers',
      'new_expense': 'Enter Expense',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Successfully deleted expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Delivery',
      'copy_billing': 'Copy Billing Address',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Invoiced',
      'logged': 'Logged',
      'running': 'Running',
      'resume': 'Resume',
      'task_errors': 'Please correct any overlapping times',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'Successfully started task',
      'stopped_task': 'Successfully stopped task',
      'resumed_task': 'Successfully resumed task',
      'now': 'Now',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Timer',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Start Time',
      'end_time': 'End Time',
      'date': 'Date',
      'times': 'Times',
      'duration': 'Duration',
      'new_task': 'New Task',
      'created_task': 'Successfully created task',
      'updated_task': 'Successfully updated task',
      'archived_task': 'Successfully archived task',
      'deleted_task': 'Successfully deleted task',
      'restored_task': 'Successfully restored task',
      'archived_tasks': 'Successfully archived :count tasks',
      'deleted_tasks': 'Successfully deleted :count tasks',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'click here',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Footer',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Custom',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'View Invoice',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Edit Customer',
      'edit_product': 'Edit Product',
      'edit_invoice': 'Edit Invoice',
      'edit_quote': 'Edit Quote',
      'edit_payment': 'Edit Payment',
      'edit_task': 'Edit Task',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Edit Supplier',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Billing Address',
      'shipping_address': 'Delivery Address',
      'total_revenue': 'Total Revenue',
      'average_invoice': 'Average Invoice',
      'outstanding': 'Outstanding',
      'invoices_sent': ':count invoices sent',
      'active_clients': 'active customers',
      'close': 'Close',
      'email': 'Email',
      'password': 'Password',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Name',
      'logout': 'Log Out',
      'login': 'Login',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Search',
      'active': 'Active',
      'archived': 'Archived',
      'deleted': 'Deleted',
      'dashboard': 'Dashboard',
      'archive': 'Archive',
      'delete': 'Delete',
      'restore': 'Restore',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Save',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Paid to Date',
      'balance_due': 'Balance Due',
      'balance': 'Balance',
      'overview': 'Overview',
      'details': 'Details',
      'phone': 'Phone',
      'website': 'Website',
      'vat_number': 'ABN',
      'id_number': 'ACN',
      'create': 'Create',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Customer Contacts',
      'additional': 'Additional',
      'first_name': 'First Name',
      'last_name': 'Last Name',
      'add_contact': 'Add contact',
      'are_you_sure': 'Are you sure?',
      'cancel': 'Cancel',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Product',
      'products': 'Products',
      'new_product': 'New Product',
      'created_product': 'Successfully created product',
      'updated_product': 'Successfully updated product',
      'archived_product': 'Successfully archived product',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Notes',
      'cost': 'Price',
      'client': 'Customer',
      'clients': 'Customers',
      'new_client': 'New Customer',
      'created_client': 'Successfully created customer',
      'updated_client': 'Successfully updated customer',
      'archived_client': 'Successfully archived customer',
      'archived_clients': 'Successfully archived :count customers',
      'deleted_client': 'Successfully deleted customer',
      'deleted_clients': 'Successfully deleted :count customers',
      'restored_client': 'Successfully restored customer',
      'restored_clients': 'Successfully restored :value customers',
      'address1': 'Street',
      'address2': 'Unit/Suite',
      'city': 'Town/Suburb',
      'state': 'State',
      'postal_code': 'Postcode',
      'country': 'Country',
      'invoice': 'Invoice',
      'invoices': 'Invoices',
      'new_invoice': 'New Invoice',
      'created_invoice': 'Successfully created invoice',
      'updated_invoice': 'Successfully updated invoice',
      'archived_invoice': 'Successfully archived invoice',
      'deleted_invoice': 'Successfully deleted invoice',
      'restored_invoice': 'Successfully restored invoice',
      'archived_invoices': 'Successfully archived :count invoices',
      'deleted_invoices': 'Successfully deleted :count invoices',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Successfully emailed invoice',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Amount',
      'invoice_number': 'Invoice Number',
      'invoice_date': 'Invoice Date',
      'discount': 'Discount',
      'po_number': 'PO Number',
      'terms': 'Payment Terms',
      'public_notes': 'Public Notes',
      'private_notes': 'Private Notes',
      'frequency': 'Frequency',
      'start_date': 'Start Date',
      'end_date': 'End Date',
      'quote_number': 'Quote Number',
      'quote_date': 'Quote Date',
      'valid_until': 'Valid Until',
      'items': 'Items',
      'partial_deposit': 'Partial Payment/Deposit',
      'description': 'Description',
      'unit_cost': 'Unit Cost',
      'quantity': 'Quantity',
      'add_item': 'Add Item',
      'contact': 'Contact',
      'work_phone': 'Phone',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Due Date',
      'partial_due_date': 'Partial Payment Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Percent',
      'edit': 'Edit',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a customer',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Settings',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'GST',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Overdue',
      'draft': 'Draft',
      'sent': 'Sent',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Partial Payment/Deposit',
      'paid': 'Paid',
      'mark_sent': 'Mark as Sent',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Done',
      'please_enter_a_client_or_contact_name':
          'Please enter a customer or contact name',
      'dark_mode': 'Dark Mode',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Activity',
      'no_records_found': 'No records found',
      'clone': 'Clone',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Payment Terms',
      'payment_date': 'Payment Date',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Customer Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Subject',
      'body': 'Body',
      'send_email': 'Send Email',
      'email_receipt': 'Email payment receipt to the customer',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Customise',
      'history': 'History',
      'payment': 'Payment',
      'payments': 'Payments',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Transaction Reference',
      'enter_payment': 'Enter Payment',
      'new_payment': 'Enter Payment',
      'created_payment': 'Successfully created payment',
      'updated_payment': 'Successfully updated payment',
      'archived_payment': 'Successfully archived payment',
      'deleted_payment': 'Successfully deleted payment',
      'restored_payment': 'Successfully restored payment',
      'archived_payments': 'Successfully archived :count payments',
      'deleted_payments': 'Successfully deleted :count payments',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Quote',
      'quotes': 'Quotes',
      'new_quote': 'New Quote',
      'created_quote': 'Successfully created quote',
      'updated_quote': 'Successfully updated quote',
      'archived_quote': 'Successfully archived quote',
      'deleted_quote': 'Successfully deleted quote',
      'restored_quote': 'Successfully restored quote',
      'archived_quotes': 'Successfully archived :count quotes',
      'deleted_quotes': 'Successfully deleted :count quotes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Expense',
      'expenses': 'Expenses',
      'vendor': 'Supplier',
      'vendors': 'Suppliers',
      'task': 'Task',
      'tasks': 'Tasks',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user created customer :client',
      'activity_2': ':user archived customer :client',
      'activity_3': ':user deleted customer :client',
      'activity_4': ':user created invoice :invoice',
      'activity_5': ':user updated invoice :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user archived invoice :invoice',
      'activity_9': ':user deleted invoice :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user updated payment :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored customer :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created supplier :vendor',
      'activity_31': ':user archived supplier :vendor',
      'activity_32': ':user deleted supplier :vendor',
      'activity_33': ':user restored supplier :vendor',
      'activity_34': ':user created expense :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated customer :client',
      'activity_62': ':user updated supplier :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Successfully emailed quote',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Select',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Supplier Number Pattern',
      'vendor_number_counter': 'Supplier Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Invoice Number Counter',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Quote Number Counter',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Customer Suburb',
      'client_state': 'Customer State',
      'client_country': 'Customer Country',
      'client_is_active': 'Customer is Active',
      'client_balance': 'Customer Balance',
      'client_address1': 'Customer Street',
      'client_address2': 'Customer Unit/Suite',
      'vendor_address1': 'Supplier Street',
      'vendor_address2': 'Supplier Unit/Suite',
      'client_shipping_address1': 'Customer Delivery Street',
      'client_shipping_address2': 'Customer Delivery Unit/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Due Date',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Bill',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Supplier Suburb',
      'vendor_state': 'Supplier State',
      'vendor_country': 'Supplier Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'GST Amount',
      'tax_paid': 'GST Paid',
      'payment_amount': 'Payment Amount',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Colour Theme',
    },
    'en_GB': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Enable',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convert to Invoice',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Invoice Task',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Hide',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Column',
      'sample': 'Sample',
      'map_to': 'Map To',
      'import': 'Import',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Please select a file',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV file',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Invoice Total',
      'quote_total': 'Quote Total',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Client Name',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Recurring Invoice',
      'recurring_invoices': 'Recurring Invoices',
      'new_recurring_invoice': 'New Recurring Invoice',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Save card details',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hours',
      'statement': 'Statement',
      'taxes': 'Taxes',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'To',
      'health_check': 'Health Check',
      'payment_type_id': 'Payment Type',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Upcoming Invoices',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Recent Payments',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': 'Create Invoice',
      'create_quote': 'Create Quote',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Delete Quote',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Delete Invoice',
      'update_client': 'Update Client',
      'delete_client': 'Delete Client',
      'delete_payment': 'Delete Payment',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Delete Vendor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Create Task',
      'update_task': 'Update Task',
      'delete_task': 'Delete Task',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Edit Token',
      'created_token': 'Successfully created token',
      'updated_token': 'Successfully updated token',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Successfully deleted token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Email Invoice',
      'email_quote': 'Email Quote',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Credit Amount',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count invoice sent',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Delete Account',
      'cancel_account_message':
          'Warning: This will permanently delete your account, there is no undo.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Credit Date',
      'credit': 'Credit',
      'credits': 'Credits',
      'new_credit': 'Enter Credit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Successfully created credit',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Successfully archived credit',
      'deleted_credit': 'Successfully deleted credit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Successfully restored credit',
      'archived_credits': 'Successfully archived :count credits',
      'deleted_credits': 'Successfully deleted :count credits',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Current version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Learn more',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'New Company',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Chart',
      'count': 'Count',
      'totals': 'Totals',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Group by',
      'credit_balance': 'Credit Balance',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Report',
      'add_company': 'Add Company',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Help',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Message',
      'from': 'From',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Documentation',
      'contact_us': 'Contact Us',
      'subtotal': 'Subtotal',
      'line_total': 'Line Total',
      'item': 'Item',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Tax Settings',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Recover your password',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Invoice Email',
      'payment_email': 'Payment Email',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Quote Email',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'User Management',
      'users': 'Users',
      'new_user': 'New User',
      'edit_user': 'Edit User',
      'created_user': 'Successfully created user',
      'updated_user': 'Successfully updated user',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Successfully deleted user',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Successfully restored user',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'General Settings',
      'invoice_options': 'Invoice Options',
      'hide_paid_to_date': 'Hide Paid to Date',
      'hide_paid_to_date_help':
          'Only display the \"Paid to Date\" area on your invoices once a payment has been received.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primary Colour',
      'secondary_color': 'Secondary Colour',
      'page_size': 'Page Size',
      'font_size': 'Font Size',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Invoice Fields',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Invoice Terms',
      'invoice_footer': 'Invoice Footer',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Fortnightly',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Charge taxes',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Custom CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorisation',
      'subdomain': 'Subdomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Regards,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Update Address',
      'update_address_help': 'Update client\'s address with provided details',
      'rate': 'Rate',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Edit tax rate',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Successfully archived the tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-fill products',
      'fill_products_help':
          'Selecting a product will automatically fill in the description and cost',
      'update_products': 'Auto-update products',
      'update_products_help':
          'Updating an invoice will automatically update the product library',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Sunday',
      'monday': 'Monday',
      'tuesday': 'Tuesday',
      'wednesday': 'Wednesday',
      'thursday': 'Thursday',
      'friday': 'Friday',
      'saturday': 'Saturday',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Hour Time',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Product Settings',
      'device_settings': 'Device Settings',
      'defaults': 'Defaults',
      'basic_settings': 'Basic Settings',
      'advanced_settings': 'Advanced Settings',
      'company_details': 'Company Details',
      'user_details': 'User Details',
      'localization': 'Localisation',
      'online_payments': 'Online Payments',
      'tax_rates': 'Tax Rates',
      'notifications': 'Notifications',
      'import_export': 'Import | Export',
      'custom_fields': 'Custom Fields',
      'invoice_design': 'Invoice Design',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Email Settings',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Data Visualisations',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Terms of Service',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Sign Up',
      'account_login': 'Account Login',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'Pending',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Converted',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Mark Paid',
      'category': 'Category',
      'address': 'Address',
      'new_vendor': 'New Vendor',
      'created_vendor': 'Successfully created vendor',
      'updated_vendor': 'Successfully updated vendor',
      'archived_vendor': 'Successfully archived vendor',
      'deleted_vendor': 'Successfully deleted vendor',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Successfully archived :count vendors',
      'deleted_vendors': 'Successfully deleted :count vendors',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Enter Expense',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Successfully deleted expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Invoiced',
      'logged': 'Logged',
      'running': 'Running',
      'resume': 'Resume',
      'task_errors': 'Please correct any overlapping times',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'Successfully started task',
      'stopped_task': 'Successfully stopped task',
      'resumed_task': 'Successfully resumed task',
      'now': 'Now',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Timer',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Start Time',
      'end_time': 'End Time',
      'date': 'Date',
      'times': 'Times',
      'duration': 'Duration',
      'new_task': 'New Task',
      'created_task': 'Successfully created task',
      'updated_task': 'Successfully updated task',
      'archived_task': 'Successfully archived task',
      'deleted_task': 'Successfully deleted task',
      'restored_task': 'Successfully restored task',
      'archived_tasks': 'Successfully archived :count tasks',
      'deleted_tasks': 'Successfully deleted :count tasks',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'click here',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Footer',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Custom',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'View Invoice',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Edit Client',
      'edit_product': 'Edit Product',
      'edit_invoice': 'Edit Invoice',
      'edit_quote': 'Edit Quote',
      'edit_payment': 'Edit Payment',
      'edit_task': 'Edit Task',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Edit Vendor',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Billing Address',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Total Revenue',
      'average_invoice': 'Average Invoice',
      'outstanding': 'Outstanding',
      'invoices_sent': ':count invoices sent',
      'active_clients': 'active clients',
      'close': 'Close',
      'email': 'Email',
      'password': 'Password',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Name',
      'logout': 'Log Out',
      'login': 'Login',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Search',
      'active': 'Active',
      'archived': 'Archived',
      'deleted': 'Deleted',
      'dashboard': 'Dashboard',
      'archive': 'Archive',
      'delete': 'Delete',
      'restore': 'Restore',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Save',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Paid to Date',
      'balance_due': 'Balance Due',
      'balance': 'Balance',
      'overview': 'Overview',
      'details': 'Details',
      'phone': 'Phone',
      'website': 'Website',
      'vat_number': 'VAT Number',
      'id_number': 'ID Number',
      'create': 'Create',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Contacts',
      'additional': 'Additional',
      'first_name': 'First Name',
      'last_name': 'Last Name',
      'add_contact': 'Add contact',
      'are_you_sure': 'Are you sure?',
      'cancel': 'Cancel',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Product',
      'products': 'Products',
      'new_product': 'New Product',
      'created_product': 'Successfully created product',
      'updated_product': 'Successfully updated product',
      'archived_product': 'Successfully archived product',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Notes',
      'cost': 'Cost',
      'client': 'Client',
      'clients': 'Clients',
      'new_client': 'New Client',
      'created_client': 'Successfully created client',
      'updated_client': 'Successfully updated client',
      'archived_client': 'Successfully archived client',
      'archived_clients': 'Successfully archived :count clients',
      'deleted_client': 'Successfully deleted client',
      'deleted_clients': 'Successfully deleted :count clients',
      'restored_client': 'Successfully restored client',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Street',
      'address2': 'Apt/Suite',
      'city': 'City',
      'state': 'State/Province',
      'postal_code': 'Postal Code',
      'country': 'Country',
      'invoice': 'Invoice',
      'invoices': 'Invoices',
      'new_invoice': 'New Invoice',
      'created_invoice': 'Successfully created invoice',
      'updated_invoice': 'Successfully updated invoice',
      'archived_invoice': 'Successfully archived invoice',
      'deleted_invoice': 'Successfully deleted invoice',
      'restored_invoice': 'Successfully restored invoice',
      'archived_invoices': 'Successfully archived :count invoices',
      'deleted_invoices': 'Successfully deleted :count invoices',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Successfully emailed invoice',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Amount',
      'invoice_number': 'Invoice Number',
      'invoice_date': 'Invoice Date',
      'discount': 'Discount',
      'po_number': 'PO Number',
      'terms': 'Terms',
      'public_notes': 'Public Notes',
      'private_notes': 'Private Notes',
      'frequency': 'Frequency',
      'start_date': 'Start Date',
      'end_date': 'End Date',
      'quote_number': 'Quote Number',
      'quote_date': 'Quote Date',
      'valid_until': 'Valid Until',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Description',
      'unit_cost': 'Unit Cost',
      'quantity': 'Quantity',
      'add_item': 'Add Item',
      'contact': 'Contact',
      'work_phone': 'Phone',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Due Date',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Percent',
      'edit': 'Edit',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Settings',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Tax',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Sent',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Partial/Deposit',
      'paid': 'Paid',
      'mark_sent': 'Mark Sent',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Done',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Dark Mode',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Activity',
      'no_records_found': 'No records found',
      'clone': 'Clone',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Payment Terms',
      'payment_date': 'Payment Date',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Client Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Subject',
      'body': 'Body',
      'send_email': 'Send Email',
      'email_receipt': 'Email payment receipt to the client',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Customise',
      'history': 'History',
      'payment': 'Payment',
      'payments': 'Payments',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Transaction Reference',
      'enter_payment': 'Enter Payment',
      'new_payment': 'Enter Payment',
      'created_payment': 'Successfully created payment',
      'updated_payment': 'Successfully updated payment',
      'archived_payment': 'Successfully archived payment',
      'deleted_payment': 'Successfully deleted payment',
      'restored_payment': 'Successfully restored payment',
      'archived_payments': 'Successfully archived :count payments',
      'deleted_payments': 'Successfully deleted :count payments',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Quote',
      'quotes': 'Quotes',
      'new_quote': 'New Quote',
      'created_quote': 'Successfully created quote',
      'updated_quote': 'Successfully updated quote',
      'archived_quote': 'Successfully archived quote',
      'deleted_quote': 'Successfully deleted quote',
      'restored_quote': 'Successfully restored quote',
      'archived_quotes': 'Successfully archived :count quotes',
      'deleted_quotes': 'Successfully deleted :count quotes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Expense',
      'expenses': 'Expenses',
      'vendor': 'Vendor',
      'vendors': 'Vendors',
      'task': 'Task',
      'tasks': 'Tasks',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user created client :client',
      'activity_2': ':user archived client :client',
      'activity_3': ':user deleted client :client',
      'activity_4': ':user created invoice :invoice',
      'activity_5': ':user updated invoice :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user archived invoice :invoice',
      'activity_9': ':user deleted invoice :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user updated payment :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored client :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user created expense :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Successfully emailed quote',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Select',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Invoice Number Counter',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Quote Number Counter',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Due Date',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Bill',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Payment Amount',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'fi': {
      'import_data': 'Tuo tietoja',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'viime päivitetty',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'kontakti ensimmäinen nimi',
      'contact_last_name': 'kontakti viime nimi',
      'order': 'Order',
      'unassigned': 'Kohdistamaton',
      'partial_value':
          'Täytyy olla suurempi kuin nolla ja vähemmän kuin kaikki yhteensä',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Ota käyttöön',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Aseta  alidomain tai näytä  lasku omalla verkkosivullasi.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'muokkaa tilaus',
      'created_subscription': 'onnistuneesti luotu tilaus',
      'updated_subscription': 'onnistuneesti päivitetty tilaus',
      'archived_subscription': 'onnistuneesti arkistoitu tilaus',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan  bar koodi  a :link compatible app.',
      'enabled_two_factor':
          'Kaksivaiheinen tunnistautuminen otettu onnistuneesti käyttöön',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Kaksivaiheinen tunnistautuminen',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Hyvitetty maksu',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'viime Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Muuta laskuksi',
      'registration_url': 'Registration URL',
      'invoice_project': 'Lasku projekti',
      'invoice_task': 'Laskuta tehtävä',
      'invoice_expense': 'Lasku kulu',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted määrä',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'oletus Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Piilota',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Sarake',
      'sample': 'Otos',
      'map_to': 'Map To',
      'import': 'Tuo',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'ystävällisesti valitsee tiedosto',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV tiedosto',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Maksamaton',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Huom',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Maksettava',
      'quote_total': 'Tarjouksen loppusumma',
      'credit_total': 'luotto yhteensä',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Varoitus',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Asiakkaan nimi',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Onnistuneesti päivitetty tehtävän tila',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Lisää aikatieto laskun tuoteriville',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'kulu kategoriat',
      'new_expense_category': 'uusi kulu kategoria',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'onnistuneesti luotu kulukategoria',
      'updated_expense_category': 'onnistuneesti päivitetty kulukategoria',
      'archived_expense_category': 'onnistuneesti arkistoitu kulu kategoria',
      'deleted_expense_category': 'onnistuneesti poistettu category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'onnistuneesti palautettu kulukategoria',
      'archived_expense_categories':
          'onnistuneesti arkistoitu :count kulu kategoria',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Pitäisi laskuttaa',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Merkitse aktiiviseksi',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Päättymätön',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Toistuva lasku',
      'recurring_invoices': 'Toistuvat laskut',
      'new_recurring_invoice': 'Uusi toistuva lasku',
      'edit_recurring_invoice': 'muokkaa toistuva Lasku',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Toistuva lasku arkistoitu onnistuneesti',
      'deleted_recurring_invoice': 'Toistuva lasku poistettu onnistuneesti',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Toistuva lasku palautettu onnistuneesti',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Näytä Portaali',
      'copy_link': 'Copy Link',
      'token_billing': 'Tallenna korttitiedot',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'asiakas numero',
      'auto_convert': 'Auto Convert',
      'company_name': 'yritys nimi',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'onnistuneesti emailed laskut',
      'emailed_quotes': 'Lähetetty onnistuneesti tarjoukset sähköpostitse',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Maksunvälittäjä',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Tuntia',
      'statement': 'tiliote',
      'taxes': 'Verot',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Käytä',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Vastaanottaja',
      'health_check': 'Health Check',
      'payment_type_id': 'Maksun tyyppi',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Valitut tarjoukset',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Erääntyvät laskut',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Viimeisimmät maksut',
      'upcoming_quotes': 'Tulevat tarjoukset',
      'expired_quotes': 'Vanhentuneet tarjoukset',
      'create_client': 'luo asiakas',
      'create_invoice': 'Luo lasku',
      'create_quote': 'Luo tarjous',
      'create_payment': 'Create Payment',
      'create_vendor': 'Luo kauppias',
      'update_quote': 'Päivitä tarjous',
      'delete_quote': 'Poista tarjous',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Poista lasku',
      'update_client': 'Update Client',
      'delete_client': 'Poista asiakas',
      'delete_payment': 'Poista maksu',
      'update_vendor': 'Päivitä kauppias',
      'delete_vendor': 'Poista kauppias',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Poista kulu',
      'create_task': 'Luo tehtävä',
      'update_task': 'Update Task',
      'delete_task': 'Poista tehtävä',
      'approve_quote': 'Hyväksy tarjous',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Ilmainen',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API-salasanat',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokenit',
      'new_token': 'New Token',
      'edit_token': 'Muokkaa tokenia',
      'created_token': 'Token luotu onnistuneesti',
      'updated_token': 'Token päivitetty onnistuneesti',
      'archived_token': 'Token arkistoitu onnistuneesti',
      'deleted_token': 'Token poistettu onnistuneesti',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Lähetä lasku sähköpostitse',
      'email_quote': 'Lähetä tarjous sähköpostitse',
      'email_credit': 'Email Credit',
      'email_payment': 'Email maksu',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Yhteyshenkilön nimi',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Muokkaa maksuaikaa',
      'created_payment_term': 'onnistuneesti luotu maksu ehto',
      'updated_payment_term': 'onnistuneesti päivitetty maksu ehto',
      'archived_payment_term': 'onnistuneesti arkistoitu maksu ehto',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Hyvityksen määrä',
      'quote_amount': 'Tarjouksen summa',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Hae tarjouksia',
      'search_credits': 'Search Credits',
      'search_vendors': 'Hae kauppiaita',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Hae 1 tarjous',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Hae 1 kauppias',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Hyvitysmaksu',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full nimi',
      'city_state_postal': 'Kaupunki/Alue/Postitoimipaikka',
      'postal_city_state': 'Postal/kaupunki/State',
      'custom1': 'ensimmäinen muokattu',
      'custom2': 'toinen muokattu',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'onnistuneesti purged yritys data',
      'purge_data_message':
          'Warning: tämä will pysyvästi erase sinun data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 päivää',
      'age_group_30': '30 - 60 päivää',
      'age_group_60': '60 - 90 päivää',
      'age_group_90': '90 - 120 päivää',
      'age_group_120': '120+ päivää',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Asiakkaan tiedot',
      'company_address': 'Company Address',
      'invoice_details': 'Laskun tiedot',
      'quote_details': 'Tarjouksen tiedot',
      'credit_details': 'Hyvityksen tiedot',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Oikeudet',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count lasku lähetetty',
      'quote_sent': 'Tarjous lähetetty',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Tarjous luettu',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Tarjous hyväksytty',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Käytä lisenssi',
      'cancel_account': 'Poista tili',
      'cancel_account_message':
          'Varoitus: Tämä poistaa tilisi pysyvästi. Tietoja ei pysty palauttamaan.',
      'delete_company': 'Poista yritys',
      'delete_company_message':
          'Warning: tämä will pysyvästi poista sinun yritys, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Tarjous on onnistuneesti muunnettu',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Ylätunniste',
      'load_design': 'Load malli',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Ehdotukset',
      'tickets': 'Tickets',
      'recurring_quotes': 'Toistuvat tarjoukset',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'toistuva kulut',
      'account_management': 'Käyttäjätilin hallinta',
      'credit_date': 'Hyvityksen päivämäärä',
      'credit': 'Luotto',
      'credits': 'Hyvitykset',
      'new_credit': 'Anna hyvitys',
      'edit_credit': 'muokkaa luotto',
      'created_credit': 'Hyvitys on luotu onnistuneesti',
      'updated_credit': 'onnistuneesti päivitetty luotto',
      'archived_credit': 'Hyvitys on arkistoitu onnistuneesti',
      'deleted_credit': 'Hyvitys on poistettu onnistuneesti',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Hyvitys palautettu onnistuneesti',
      'archived_credits': ':count hyvitys(tä) arkistoitu onnistuneesti',
      'deleted_credits': ':count hyvitys(tä) poistettu onnistuneesti',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Nykyinen versio',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Lue lisää',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Uusi yritys',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Nollaa',
      'number': 'Number',
      'export': 'Vienti',
      'chart': 'Kaavio',
      'count': 'Count',
      'totals': 'Yhteensä',
      'blank': 'Tyhjä',
      'day': 'Päivä',
      'month': 'Kuukausi',
      'year': 'Vuosi',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Niputa',
      'credit_balance': 'Hyvityksen saldo',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'kontakti puhelin',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Toimitus: Katu',
      'shipping_address2': 'Toimitus: Asunto/huoneisto',
      'shipping_city': 'Toimitus: Kaupunki',
      'shipping_state': 'Toimitus: Maakunta',
      'shipping_postal_code': 'Toimitus: Postinumero',
      'shipping_country': 'Toimitus: Maa',
      'billing_address1': 'Laskutus: Katu',
      'billing_address2': 'Laskutus: Asunto/huoneisto',
      'billing_city': 'Laskutus: Kaupunki',
      'billing_state': 'Laskutus: Maakunta',
      'billing_postal_code': 'Laskutus: Postinumero',
      'billing_country': 'Laskutus: Maa',
      'client_id': 'Asiakkaan tunniste',
      'assigned_to': 'Assigned',
      'created_by': 'luotu by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit ja Loss',
      'reports': 'Raportit',
      'report': 'Raportti',
      'add_company': 'Lisää yritys',
      'unpaid_invoice': 'Maksamatonl lasku',
      'paid_invoice': 'Paid Lasku',
      'unapproved_quote': 'Hyväksymätön tarjous',
      'help': 'Ohje',
      'refund': 'Hyvitys',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'kontakti Email',
      'multiselect': 'Multiselect',
      'entity_state': 'Osavaltio',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Viesti',
      'from': 'Lähettäjä',
      'show_product_details': 'Näytä tuotteen tiedot',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent  tili  palkkio',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentaatio',
      'contact_us': 'kontakti Us',
      'subtotal': 'Välisumma',
      'line_total': 'Rivin summa',
      'item': 'Tuote',
      'credit_email': 'Credit Email',
      'iframe_url': 'Verkkosivu',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'salasana on liian lyhyt',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Kyllä',
      'no': 'Ei',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Näytä',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Käyttäjä',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Valitse asiakas',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Veroasetukset',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Valinnat',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Palauta salasana',
      'late_fees': 'Viivästysmaksut',
      'credit_number': 'luotto numero',
      'payment_number': 'maksu numero',
      'late_fee_amount': 'Late palkkio määrä',
      'late_fee_percent': 'Late palkkio Percent',
      'schedule': 'Aikataulu',
      'before_due_date': 'Ennen  eräpäivää',
      'after_due_date': 'Eräpäivän jälkeen',
      'after_invoice_date': 'Laskun päiväyksen jälkeen',
      'days': 'Päivää',
      'invoice_email': 'Laskusähköposti',
      'payment_email': 'Maksusähköposti',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Osittainen maksu',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Tarjoussähköposti',
      'endless_reminder': 'Endless muistutus',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Ylläpitäjä',
      'administrator_help':
          'Allow käyttäjä  manage users, change asetus ja modify kaikki records',
      'user_management': 'Käyttäjänhallinta',
      'users': 'Käyttäjät',
      'new_user': 'Uusi käyttäjä',
      'edit_user': 'Muokkaa käyttäjä',
      'created_user': 'Onnistuneesti luotu käyttäjä',
      'updated_user': 'Käyttäjä on päivitetty onnistuneesti',
      'archived_user': 'Käyttäjä arkistoitu onnistuneesti',
      'deleted_user': 'Käyttäjä on poistettu onnistuneesti',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Käyttäjä palautettu onnistuneesti',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Yleiset asetukset',
      'invoice_options': 'Laskun valinnat',
      'hide_paid_to_date': 'Piilota \"Maksettu tähän asti\"',
      'hide_paid_to_date_help':
          'Näytä \"Maksettava päivämäärään mennessä\" kenttä laskuillasi vain maksetuilla laskuilla.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Sisällytä liitetyt kuvat laskuun.',
      'all_pages_header': 'näytä Header on',
      'all_pages_footer': 'Näytä alatunniste',
      'first_page': 'ensimmäinen page',
      'all_pages': 'All pages',
      'last_page': 'viime page',
      'primary_font': 'Ensisijainen kirjasin',
      'secondary_font': 'toissijainen kirjasin',
      'primary_color': 'Pääväri',
      'secondary_color': 'Apuväri',
      'page_size': 'Page Size',
      'font_size': 'Fontin koko',
      'quote_design': 'Tarjouksen muotoilu',
      'invoice_fields': 'Laskun kentät',
      'product_fields': 'Tuote kentät',
      'invoice_terms': 'Laskun ehdot',
      'invoice_footer': 'Laskun alatunniste',
      'quote_terms': 'Tarjouksen ehdot',
      'quote_footer': 'Tarjouksen alatunniste',
      'auto_email_invoice': 'automaattinen Email',
      'auto_email_invoice_help':
          'automaattisesti sähköposti toistuva laskut when they on luotu.',
      'auto_archive_invoice': 'automaattinen Arkistoi',
      'auto_archive_invoice_help':
          'automaattisesti archive laskut when they on paid.',
      'auto_archive_quote': 'automaattinen Arkistoi',
      'auto_archive_quote_help':
          'Arkistoi tarjoukset automaattisesti kun ne on muunnettu laskuiksi.',
      'auto_convert_quote': 'Automaattinen muunnos',
      'auto_convert_quote_help':
          'Muunna tarjous automaattisesti laskuksi, kun asiakas on hyväksynyt tarjouksen.',
      'workflow_settings': 'Workflow asetukset',
      'freq_daily': 'päivittäin',
      'freq_weekly': 'viikoittain',
      'freq_two_weeks': 'Kaksi viikkoa',
      'freq_four_weeks': 'neljä viikkoa',
      'freq_monthly': 'Kuukausittain',
      'freq_two_months': 'Kaksi kuukautta',
      'freq_three_months': 'kolme kuukautta',
      'freq_four_months': 'neljä kuukautta',
      'freq_six_months': 'Six kuukautta',
      'freq_annually': 'Vuosittain',
      'freq_two_years': 'Kaksi vuotta',
      'freq_three_years': 'Three Years',
      'never': 'Ei koskaan',
      'company': 'yritys',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Veloita veroa',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'toistuva etuliite',
      'number_padding': 'numero Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge kenttä',
      'company_field': 'yritys kenttä',
      'company_value': 'yritys Value',
      'credit_field': 'luotto kenttä',
      'invoice_field': 'Lasku kenttä',
      'invoice_surcharge': 'Lasku Surcharge',
      'client_field': 'asiakas kenttä',
      'product_field': 'Tuote kenttä',
      'payment_field': 'maksu kenttä',
      'contact_field': 'kontakti kenttä',
      'vendor_field': 'Kauppias kenttä',
      'expense_field': 'kulu kenttä',
      'project_field': 'Projekti kenttä',
      'task_field': 'Tehtävä kenttä',
      'group_field': 'ryhmä kenttä',
      'number_counter': 'numero Counter',
      'prefix': 'Etuliite',
      'number_pattern': 'numero Pattern',
      'messages': 'Viestit',
      'custom_css': 'Mukautettu CSS',
      'custom_javascript': 'Muokautettu JavaScript',
      'signature_on_pdf': 'näytä on PDF',
      'signature_on_pdf_help':
          'Näytä asiakkaan allekirjoitus lasku-/tarjous-PDF:ssä.',
      'show_accept_invoice_terms': 'Laskun ehdot valintaruutu',
      'show_accept_invoice_terms_help':
          'Vaadi asiakasta  vahvistamaan, että hän hyväksyy laskun ehdot.',
      'show_accept_quote_terms': 'Tarjouksen ehdot valintaruutu',
      'show_accept_quote_terms_help':
          'Vaadi asiakasta  vahvistamaan, että hän hyväksyy tarjouksen ehdot.',
      'require_invoice_signature': 'Laskun allekirjoitus',
      'require_invoice_signature_help':
          'Vaadi asiakasta täyttämään allekirjoitus.',
      'require_quote_signature': 'Tarjouksen allekirjoitus',
      'enable_portal_password': 'salasana suojaa laskut',
      'enable_portal_password_help':
          'Mahdollistaa, että voit antaa salasanan jokaiselle yhteyshenkilölle. Jos salasana on asetettu, yhteyshenkilön tulee kirjautua sen avulla sisään voidakseen tarkastella laskuja.',
      'authorization': 'Valtuutus',
      'subdomain': 'Alidomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Ystävällisesti,',
      'enable_email_markup_help':
          'Tee asiakkaillesi helpommaksi maksaa laskusi ottamalla käyttöön schema.org -merkintä  sähköposteissasi.',
      'plain': 'Yksinkertainen',
      'light': 'Vaalea',
      'dark': 'Tumma',
      'email_design': 'Sähköpostin muotoilu',
      'attach_pdf': 'Liitä PDF',
      'attach_documents': 'Liitä asiakirjoja',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Ota käyttöön merkintä',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Luottokortti',
      'bank_transfer': 'Pankkisiirto',
      'priority': 'Priority',
      'fee_amount': 'palkkio määrä',
      'fee_percent': 'Palkkio prosentti',
      'fee_cap': 'palkkio Cap',
      'limits_and_fees': 'Limits/palkkiot',
      'enable_min': 'Ota käyttöön min',
      'enable_max': 'Ota käyttöön max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted kortti Logos',
      'credentials': 'Tunnukset',
      'update_address': 'Päivitä osoite',
      'update_address_help': 'Päivitä asiakkaan osoite annetuilla tiedoilla',
      'rate': 'Kanta',
      'tax_rate': 'Verokanta',
      'new_tax_rate': 'Uusi veromäärä',
      'edit_tax_rate': 'Muokkaa verokantaa',
      'created_tax_rate': 'Verokanta luotu onnistuneesti',
      'updated_tax_rate': 'Verokanta päivitetty onnistuneesti',
      'archived_tax_rate': 'Verokanta arkistoitu onnistuneesti',
      'deleted_tax_rate': 'Verokanta onnistuneesti poistettu',
      'restored_tax_rate': 'Verokanta onnistuneesti palautettu',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Lisää automaattisesti tuotteita',
      'fill_products_help':
          'Tuotteen valinta täyttää kuvauksen ja hinnan automaattisesti',
      'update_products': 'Päivitä automaattisesti tuotteet',
      'update_products_help':
          'Laskun päivittäminen päivittää tuotetietokannan automaattisesti',
      'convert_products': 'Convert tuotteet',
      'convert_products_help':
          'Muunna automaattisesti tuotehinnat asiakkaan valuuttaan',
      'fees': 'palkkiot',
      'limits': 'Limits',
      'provider': 'Tarjoaja',
      'company_gateway': 'maksu Gateway',
      'company_gateways': 'maksu Gateways',
      'new_company_gateway': 'uusi Gateway',
      'edit_company_gateway': 'muokkaa Gateway',
      'created_company_gateway': 'onnistuneesti luotu gateway',
      'updated_company_gateway': 'onnistuneesti päivitetty gateway',
      'archived_company_gateway': 'onnistuneesti arkistoitu gateway',
      'deleted_company_gateway': 'onnistuneesti poistettu gateway',
      'restored_company_gateway': 'onnistuneesti palautettu gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Jatka muokkausta',
      'discard_changes': 'Discard Changes',
      'default_value': 'Oletus arvo',
      'disabled': 'Pois käytöstä',
      'currency_format': 'Valuutan muoto',
      'first_day_of_the_week': 'Viikon ensimmäinen päivä',
      'first_month_of_the_year': 'Vuoden ensimmäinen kuukausi',
      'sunday': 'sunnuntai',
      'monday': 'Maanantai',
      'tuesday': 'Tiistai',
      'wednesday': 'Keskiviikko',
      'thursday': 'Torstai',
      'friday': 'Perjantai',
      'saturday': 'Lauantai',
      'january': 'Tammikuu',
      'february': 'Helmikuu',
      'march': 'Maaliskuu',
      'april': 'Huhtikuu',
      'may': 'Toukokuu',
      'june': 'Kesäkuu',
      'july': 'Heinäkuu',
      'august': 'Elokuu',
      'september': 'Syyskuu',
      'october': 'Lokakuu',
      'november': 'Marraskuu',
      'december': 'Joulukuu',
      'symbol': 'Symboli',
      'ocde': 'Koodi',
      'date_format': 'Date Format',
      'datetime_format': 'Päivä-Aika esitysmuoto',
      'military_time': '24 tunnin aika',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'lähetä muistutukset',
      'timezone': 'Aikavyöhyke',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by ryhmä',
      'filtered_by_invoice': 'Filtered by Lasku',
      'filtered_by_client': 'Filtered by asiakas',
      'filtered_by_vendor': 'Suodatettu: Kauppias',
      'group_settings': 'ryhmä asetukset',
      'group': 'ryhmä',
      'groups': 'ryhmät',
      'new_group': 'uusi ryhmä',
      'edit_group': 'muokkaa ryhmä',
      'created_group': 'onnistuneesti luotu ryhmä',
      'updated_group': 'onnistuneesti päivitetty ryhmä',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Lataa Logo',
      'uploaded_logo': 'Logo onnistuneesti ladattu palvelimelle',
      'logo': 'Logo',
      'saved_settings': 'onnistuneesti saved asetus',
      'product_settings': 'Tuoteasetukset',
      'device_settings': 'Device asetukset',
      'defaults': 'Oletusasetukset',
      'basic_settings': 'Perusasetukset',
      'advanced_settings': 'Lisääasetuksia',
      'company_details': 'Yrityksen tiedot',
      'user_details': 'Käyttäjätiedot',
      'localization': 'Lokalisointi',
      'online_payments': 'Online maksut',
      'tax_rates': 'Verokannat',
      'notifications': 'Sähköposti-ilmoitukset',
      'import_export': 'Tuonti | Vienti',
      'custom_fields': 'Mukautetut kentät',
      'invoice_design': 'Laskun muotoilu',
      'buy_now_buttons': 'Osta nyt napit',
      'email_settings': 'Sähköpostin asetukset',
      'templates_and_reminders': 'Pohjat ja muistutukset',
      'credit_cards_and_banks': 'luotto Cards & Banks',
      'data_visualizations': 'Datan visualisaatiot',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'kiitos you  sinun purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual tilaus',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count käyttäjää',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Anna  etunimi',
      'please_enter_a_last_name': 'Anna  sukunimi',
      'please_agree_to_terms_and_privacy':
          'Ole hyvä ja hyväksy palveluehtomme sekä tietosuojakäytäntömme luodaksesi käyttäjätilin.',
      'i_agree_to_the': 'Hyväksyn',
      'terms_of_service_link': 'palvelun ehdot',
      'privacy_policy_link': 'Tietosuojakäytäntö',
      'terms_of_service': 'Käyttöehdot',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Rekisteröidy',
      'account_login': 'Tiliin kirjautuminen',
      'view_website': 'Näytä verkkosivu',
      'create_account': 'Luo käyttäjätili',
      'email_login': 'Email Login',
      'create_new': 'luo uusi',
      'no_record_selected': 'ei record selected',
      'error_unsaved_changes': 'save tai peruuta sinun muutokset',
      'download': 'Lataa',
      'requires_an_enterprise_plan': 'Requires  enterprise plan',
      'take_picture': 'Ota kuva',
      'upload_file': 'Lataa tiedosto palvelimelle',
      'document': 'Document',
      'documents': 'Asiakirjat',
      'new_document': 'Uusi asiakirja',
      'edit_document': 'Muokkaa asiakirjaa',
      'uploaded_document': 'onnistuneesti lähetetty dokumentti',
      'updated_document': 'onnistuneesti päivitetty dokumentti',
      'archived_document': 'onnistuneesti arkistoitu dokumentti',
      'deleted_document': 'onnistuneesti poistettu dokumentti',
      'restored_document': 'onnistuneesti palautettu dokumentti',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'ei History',
      'expense_date': 'Kulun päivämäärä',
      'pending': 'Odottaa vastausta',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Muunnettu',
      'add_documents_to_invoice': 'Lisää asiakirjoja laskuun',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Muunna valuutta',
      'mark_paid': 'Merkitse maksetuksi',
      'category': 'Kategoria',
      'address': 'Osoite',
      'new_vendor': 'Uusi kauppias',
      'created_vendor': 'Kauppias luotin onnistuneesti',
      'updated_vendor': 'Kauppias on päivitetty onnistuneesti',
      'archived_vendor': 'Kauppias on arkistoitu onnistuneesti',
      'deleted_vendor': 'Kauppias on poistettu onnistuneesti',
      'restored_vendor': 'Onnistuneesti palautettu kauppias',
      'archived_vendors': ':count kauppias(ta) arkistoitu onnistuneesti',
      'deleted_vendors': ':count kauppias(ta) poistettu onnistuneesti',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Lisää kulu',
      'created_expense': 'onnistuneesti luotu kulu',
      'updated_expense': 'onnistuneesti päivitetty kulu',
      'archived_expense': 'Kulu arkistoitu onnistuneesti',
      'deleted_expense': 'Kulu poistettu onnistuneesti',
      'restored_expense': 'onnistuneesti palautettu kulu',
      'archived_expenses': 'onnistuneesti arkistoitu kulut',
      'deleted_expenses': 'onnistuneesti poistettu kulut',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Kopioi toimitus',
      'copy_billing': 'Kopioi laskutus',
      'design': 'malli',
      'failed_to_find_record': 'Failed  find record',
      'invoiced': 'Laskutettu',
      'logged': 'Kirjattu',
      'running': 'Käynnissä',
      'resume': 'Jatka',
      'task_errors': 'Ole hyvä ja korjaa päällekäiset ajat',
      'start': 'Aloitus',
      'stop': 'Lopetus',
      'started_task': 'Onnistuneesti aloitettu tehtävä',
      'stopped_task': 'Tehtävä lopetettu onnistuneesti',
      'resumed_task': 'Onnistuneesti jatkettu tehtävää',
      'now': 'Nyt',
      'auto_start_tasks': 'Automaattinen tehtävien aloitus',
      'timer': 'Ajastin',
      'manual': 'Manuaalinen',
      'budgeted': 'Budjetoitu',
      'start_time': 'Aloitusaika',
      'end_time': 'Lopetusaika',
      'date': 'Päivämäärä',
      'times': 'Ajat',
      'duration': 'Kesto',
      'new_task': 'Uusi tehtävä',
      'created_task': 'Tehtävä luotu onnistuneesti',
      'updated_task': 'Tehtävä päivitetty onnistuneesti',
      'archived_task': 'Tehtävä arkistoitu onnistuneesti',
      'deleted_task': 'Tehtävä poistettu onnistuneesti',
      'restored_task': 'Tehtävä palautettu onnistuneesti',
      'archived_tasks': ':count tehtävää arkistoitu onnistuneesti',
      'deleted_tasks': ':count tehtävää poistettu onnistuneesti',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Ole hyvä ja anna nimi',
      'budgeted_hours': 'Budjetoidut työtunnit',
      'created_project': 'Onnistuneesti luotu projekti',
      'updated_project': 'Onnistuneesti päivitetty projekti',
      'archived_project': 'Onnistuneesti arkistoitu projekti',
      'deleted_project': 'Projekti poistettu onnistuneesti',
      'restored_project': 'Onnistuneesti palautettu projekti',
      'archived_projects': 'Onnistuneesti arkistoitu :count projekti(a)',
      'deleted_projects': 'Onnistuneesti poistettu :count projekti(a)',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Uusi projekti',
      'thank_you_for_using_our_app': 'kiitos you  using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'klikkaa tästä',
      'click_here_capital': 'Click here',
      'to_rate_it': 'rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting': 'authenticate  change this asetus',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Alatunniste',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'tänään',
      'custom_range': 'muokattu Range',
      'date_range': 'Päivämääräväli',
      'current': 'nykyinen',
      'previous': 'Previous',
      'current_period': 'nykyinen kausi',
      'comparison_period': 'Comparison kausi',
      'previous_period': 'Previous kausi',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare',
      'last7_days': 'viime 7 päivää',
      'last_week': 'viime viikko',
      'last30_days': 'viime 30 päivää',
      'this_month': 'tämä kuukausi',
      'last_month': 'viime kuukausi',
      'this_year': 'tämä Year',
      'last_year': 'viime Year',
      'custom': 'Mukautettu',
      'clone_to_invoice': 'kloonaa  Lasku',
      'clone_to_quote': 'Kopioi  tarjous',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Katso lasku',
      'convert': 'Convert',
      'more': 'Lisää',
      'edit_client': 'Muokkaa asiakas',
      'edit_product': 'Muokkaa tuote',
      'edit_invoice': 'Muokkaa laskua',
      'edit_quote': 'Muokkaa tarjousta',
      'edit_payment': 'Muokkaa maksua',
      'edit_task': 'Muokkaa tehtävä',
      'edit_expense': 'muokkaa kulu',
      'edit_vendor': 'Muokkaa kauppiasta',
      'edit_project': 'Muokkaa projektia',
      'edit_recurring_expense': 'muokkaa toistuva kulu',
      'edit_recurring_quote': 'Muokkaa toistuvaa tarjousta',
      'billing_address': 'Laskutusosoite',
      'shipping_address': 'Toimitusosoite',
      'total_revenue': 'Kokonaistulot',
      'average_invoice': 'Laskujen keskiarvo',
      'outstanding': 'Maksamattomat laskut',
      'invoices_sent': ':count laskua lähetetty',
      'active_clients': 'Aktiiviset asiakkaat',
      'close': 'Sulje',
      'email': 'Sähköposti',
      'password': 'Salasana',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Nimi',
      'logout': 'Kirjaudu ulos',
      'login': 'Kirjaudu sisään',
      'filter': 'Suodata',
      'sort': 'Sort',
      'search': 'Etsi',
      'active': 'Aktiivinen',
      'archived': 'Arkistoitu',
      'deleted': 'Poistettu',
      'dashboard': 'Hallintapaneeli',
      'archive': 'Arkisto',
      'delete': 'Poista',
      'restore': 'Palauta',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Anna sähköpostiosoitteesi',
      'please_enter_your_password': 'Anna salasanasi',
      'please_enter_your_url': 'Anna sinun URL-osoitteesi',
      'please_enter_a_product_key': 'Anna tuoteavain',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Tallenna',
      'an_error_occurred': 'virhe occurred',
      'paid_to_date': 'Maksettu tähän mennessä',
      'balance_due': 'Avoin lasku',
      'balance': 'Saldo',
      'overview': 'Yleiskatsaus',
      'details': 'Tiedot',
      'phone': 'Puhelin',
      'website': 'Kotisivu',
      'vat_number': 'ALV-numero',
      'id_number': 'Asiakasnumero',
      'create': 'Luo',
      'copied_to_clipboard': 'Copied :arvo    clipboard',
      'error': 'Virhe',
      'could_not_launch': 'Could not launch',
      'contacts': 'Yhteystiedot',
      'additional': 'Lisäksi',
      'first_name': 'Etunimi',
      'last_name': 'Sukunimi',
      'add_contact': 'Lisää yhteystieto',
      'are_you_sure': 'Oletko varma?',
      'cancel': 'Peruuta',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Tuote',
      'products': 'Tuotteet',
      'new_product': 'Uusi tuote',
      'created_product': 'Tuote on luotu onnistuneesti',
      'updated_product': 'Tuote on päivitetty onnistuneesti',
      'archived_product': 'Tuote on arkistoitu onnistuneesti',
      'deleted_product': 'onnistuneesti poistettu tuote',
      'restored_product': 'onnistuneesti palautettu tuote',
      'archived_products': 'onnistuneesti arkistoitu :count tuotteet',
      'deleted_products': 'onnistuneesti poistettu :count tuotteet',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Tuote',
      'notes': 'Viestit',
      'cost': 'Hinta',
      'client': 'Asiakas',
      'clients': 'Asiakkaat',
      'new_client': 'Uusi asiakas',
      'created_client': 'Luotin onnistuneesti asiakas',
      'updated_client': 'Asiakas on päivitetty onnistuneesti',
      'archived_client': 'Asiakas on arkistoitu onnistuneesti',
      'archived_clients': ':count asiakas(ta) arkistoitu onnistuneesti',
      'deleted_client': 'Asiakas on poistettu onnistuneesti',
      'deleted_clients': ':count asiakas(ta) poistettu onnistuneesti',
      'restored_client': 'Asiakas palautettu onnistuneesti',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Katu',
      'address2': 'Asunto',
      'city': 'Kaupunki',
      'state': 'Lääni',
      'postal_code': 'Postinumero',
      'country': 'Maa',
      'invoice': 'Lasku',
      'invoices': 'Laskut',
      'new_invoice': 'Uusi lasku',
      'created_invoice': 'Lasku luotiin onnistuneesti',
      'updated_invoice': 'Lasku päivitettiin onnistuneesti',
      'archived_invoice': 'Lasku arkistoitiin onnistuneesti',
      'deleted_invoice': 'Lasku poistettiin onnistuneesti',
      'restored_invoice': 'Lasku palautettu onnistuneesti',
      'archived_invoices': ':count asiakas(ta) arkistoitu onnistuneesti',
      'deleted_invoices': ':count laskua poistettiin onnistuneesti',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Lasku lähetettiin onnistuneesti',
      'emailed_payment': 'onnistuneesti emailed maksu',
      'amount': 'Määrä',
      'invoice_number': 'Laskun numero',
      'invoice_date': 'Laskun päivämäärä',
      'discount': 'Alennus',
      'po_number': 'Tilaus numero',
      'terms': 'Ehdot',
      'public_notes': 'Julkiset muistiinpanot',
      'private_notes': 'Yksityiset muistiinpanot',
      'frequency': 'Kuinka usein',
      'start_date': 'Alkamispäivämäärä',
      'end_date': 'Loppupäivämäärä',
      'quote_number': 'Tarjous numero',
      'quote_date': 'Tarjouksen päivämäärä',
      'valid_until': 'Voimassa',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Kuvaus',
      'unit_cost': 'Kappalehinta',
      'quantity': 'Määrä',
      'add_item': 'Lisää nimike',
      'contact': 'Yhteyshenkilö',
      'work_phone': 'Puhelin',
      'total_amount': 'yhteensä määrä',
      'pdf': 'PDF',
      'due_date': 'Eräpäivä',
      'partial_due_date': 'Partial eräpäivä',
      'status': 'Tila',
      'invoice_status_id': 'Lasku tila',
      'quote_status': 'Tarjouksen tila',
      'click_plus_to_add_item': 'Napsauta +  lisätäksesi nimikkeen',
      'click_plus_to_add_time': 'Napsauta + lisätäksesi ajan',
      'count_selected': ':count selected',
      'total': 'Loppusumma',
      'percent': 'Prosentti',
      'edit': 'Muokkaa',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'valitse  päivämäärä',
      'please_select_a_client': 'valitse  asiakas',
      'please_select_an_invoice': 'valitse  lasku',
      'task_rate': 'Tehtävän luokitus',
      'settings': 'Asetukset',
      'language': 'Language',
      'currency': 'Valuutta',
      'created_at': 'Luotu',
      'created_on': 'Created On',
      'updated_at': 'päivitetty',
      'tax': 'Vero',
      'please_enter_an_invoice_number': 'Ayötä  laskunumero',
      'please_enter_a_quote_number': 'Ole hyvä ja anna tarjouksen numero',
      'past_due': 'Past Due',
      'draft': 'Luonnos',
      'sent': 'Lähetetty',
      'viewed': 'Nähty',
      'approved': 'Approved',
      'partial': 'Osittainen/Talletus',
      'paid': 'Maksettu',
      'mark_sent': 'Merkitse lähetetyksi',
      'marked_invoice_as_sent': 'Onnistuneesti merkitty lasku lähetetyksi',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Valmis',
      'please_enter_a_client_or_contact_name':
          'Anna  asiakkaan tai yhteyshenkilön nimi',
      'dark_mode': 'Tumma tila',
      'restart_app_to_apply_change':
          'Uudelleenkäynnistä sovellus ottaaksesi muutoksen käyttöön',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank kontakti',
      'activity': 'Toiminta',
      'no_records_found': 'ei records found',
      'clone': 'Kopioi',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Maksuehdot',
      'payment_date': 'Maksun päivämäärä',
      'payment_status': 'maksu tila',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Asiakasportaali',
      'show_tasks': 'Näytä tehtävät',
      'email_reminders': 'Email muistutukset',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'Ensimmäinen muistutus',
      'second_reminder': 'Toinen muistutus',
      'third_reminder': 'Kolmas muistutus',
      'reminder1': 'ensimmäinen muistutus',
      'reminder2': 'toinen muistutus',
      'reminder3': 'Third muistutus',
      'template': 'Malli',
      'send': 'lähetä',
      'subject': 'Otsikko',
      'body': 'Sisältö',
      'send_email': 'Lähetä sähköposti',
      'email_receipt': 'Lähetä maksukuitti sähköpostilla asiakkaalle',
      'auto_billing': 'Automaattinen laskutus',
      'button': 'Button',
      'preview': 'Esikatselu',
      'customize': 'Mukauta',
      'history': 'Historia',
      'payment': 'Maksu',
      'payments': 'Maksut',
      'refunded': 'Refunded',
      'payment_type': 'Maksutyyppi',
      'transaction_reference': 'Tapahtuman viite',
      'enter_payment': 'Kirjaa maksu',
      'new_payment': 'Uusi maksutapahtuma',
      'created_payment': 'Maksu on luotu onnistuneesti',
      'updated_payment': 'Maksu päivitetty onnistuneesti',
      'archived_payment': 'Maksu on arkistoitu onnistuneesti',
      'deleted_payment': 'Maksu on poistettu onnistuneesti',
      'restored_payment': 'Maksu palautettu onnistuneesti',
      'archived_payments': ':count maksu(a) arkistoitu onnistuneesti',
      'deleted_payments': ':count maksu(a) poistettu onnistuneesti',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Tarjous',
      'quotes': 'Tarjoukset',
      'new_quote': 'Uusi tarjous',
      'created_quote': 'Tarjous on luotu onnistuneesti',
      'updated_quote': 'Tarjous on päivitetty onnistuneesti',
      'archived_quote': 'Tarjous on arkistoitu onnistuneesti',
      'deleted_quote': 'Tarjous on poistettu onnistuneesti',
      'restored_quote': 'Tarjous palautettu onnistuneesti',
      'archived_quotes': ':count tarjous(ta) arkistoitu onnistuneesti',
      'deleted_quotes': ':count tarjous(ta) poistettu onnistuneesti',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Kulu',
      'expenses': 'Kulut',
      'vendor': 'Kauppias',
      'vendors': 'Kauppiaat',
      'task': 'Tehtävä',
      'tasks': 'Tehtävät',
      'project': 'Projekti',
      'projects': 'Projektit',
      'activity_1': ':käyttäjä loi asiakkaan :client',
      'activity_2': ':käyttäjä arkistoi asiakkaan :client',
      'activity_3': ':käyttäjä poisti asiakkaan :client',
      'activity_4': ':käyttäjä loi laskun :invoice',
      'activity_5': ':käyttäjä päivitti laskun :invoice',
      'activity_6': ':käyttäjä emailed lasku :lasku for :asiakas   :kontakti',
      'activity_7': ':kontakti katsoi lasku :lasku for :asiakas',
      'activity_8': ':käyttäjä arkistoi laskun :invoice',
      'activity_9': ':käyttäjä poisti laskun :invoice',
      'activity_10':
          ':kontakti entered maksu :maksu for :payment_amount on lasku :lasku for :asiakas',
      'activity_11': ':käyttäjä päivitti maksun :maksu',
      'activity_12': ':käyttäjä arkistoi maksun :maksu',
      'activity_13': ':käyttäjä poisti maksun :maksu',
      'activity_14': ':käyttäjä syötti :luotto hyvityksen',
      'activity_15': ':käyttäjä päivitti :luotto hyvityksen',
      'activity_16': ':käyttäjä arkistoi :luotto hyvityksen',
      'activity_17': ':käyttäjä poisti :luotto hyvityksen',
      'activity_18': ':user loi tarjouksen :quote',
      'activity_19': ':user päivitti tarjouksen :quote',
      'activity_20':
          ':user lähetti sähköpostitse tarjouksen :quote asiakkaan :client yhteyshenkilölle :contact',
      'activity_21': ':contact luki tarjouksen :quote',
      'activity_22': ':user arkistoi tarjouksen :quote',
      'activity_23': ':user poisti tarjouksen :quote',
      'activity_24': ':user palautti tarjouksen :quote',
      'activity_25': ':käyttäjä palautti laskun :invoice',
      'activity_26': ':käyttäjä palautti asiakkaan :client',
      'activity_27': ':käyttäjä palautti maksun :maksu',
      'activity_28': ':käyttäjä palautti hyvityksen :luotto',
      'activity_29': ':contact hyväksyi tarjouksen :quote asiakkaalle :client',
      'activity_30': ':käyttäjä loi kauppiaan :vendor',
      'activity_31': ':käyttäjä arkistoi kauppiaan :vendor',
      'activity_32': ':käyttäjä poisti kauppiaan :vendor',
      'activity_33': ':käyttäjä palautti kauppiaan :vendor',
      'activity_34': ':käyttäjä loi kulun :kulu',
      'activity_35': ':käyttäjä arkistoi kulun :kulu',
      'activity_36': ':käyttäjä poisti kulun :kulu',
      'activity_37': ':käyttäjä palautti kulun :kulu',
      'activity_39': ':käyttäjä perui :payment_amount maksun :maksu',
      'activity_40':
          ':käyttäjä refunded :adjustment  a :payment_amount maksu :maksu',
      'activity_41': ':payment_amount maksu (:maksu) failed',
      'activity_42': ':käyttäjä loi tehtävän :tehtävä',
      'activity_43': ':käyttäjä päivitti tehtävän :tehtävä',
      'activity_44': ':käyttäjä arkistoi tehtävän :tehtävä',
      'activity_45': ':käyttäjä poisti tehtävän :tehtävä',
      'activity_46': ':käyttäjä palautti tehtävän :tehtävä',
      'activity_47': ':käyttäjä päivitti kulun :kulu',
      'activity_48': ':käyttäjä päivitti tehtävän :tiketti',
      'activity_49': ':käyttäjä sulki tiketin :tiketti',
      'activity_50': ':käyttäjä mergesi tiketin :tiketti',
      'activity_51': ':käyttäjä jakoi tiketin :tiketti',
      'activity_52': ':kontakti avasi tiketin :tiketti',
      'activity_53': ':kontakti reopened tiketti :tiketti',
      'activity_54': ':käyttäjä reopened tiketti :tiketti',
      'activity_55': ':kontakti vastasi tiketti :tiketti',
      'activity_56': ':käyttäjä katsoi tiketti :tiketti',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Kertakäyttöinen salasana',
      'emailed_quote': 'Tarjous on lähetetty onnistuneesti',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Tarjous on onnistuneesti merkitty lähetetyksi',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Vanhentunut',
      'all': 'Kaikki',
      'select': 'Valitse',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'muokattu Value',
      'custom_value2': 'Mukautettu arvo',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Oma Hyväksymätön tarjous -viesti',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Kauppiaan numerolaskuri',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Laskun järjestysnumero',
      'quote_number_pattern': 'Tarjouksen numeroinnin kuvio',
      'quote_number_counter': 'Tarjouksen järjestysnumero',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Jaettu lasku tarjous laskuri',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Tarjoussähköpostin otsikko',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tyyppi',
      'invoice_amount': 'Lasku määrä',
      'invoice_due_date': 'Eräpäivä',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Automaattinen laskutus',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Kauppiaan kaupunki',
      'vendor_state': 'Kauppiaan alue',
      'vendor_country': 'Kauppiaan maa',
      'is_approved': 'Is Approved',
      'tax_name': 'veronimi',
      'tax_amount': 'vero määrä',
      'tax_paid': 'vero Paid',
      'payment_amount': 'Maksun määrä',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Aikaloki',
      'bank_id': 'Pankki',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Kulujen kategoria',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Veron nimi 1',
      'tax_name2': 'Veron nimi 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'fr': {
      'import_data': 'Importer des données',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Dernière mise à jour',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Prénom du contact',
      'contact_last_name': 'Nom du contact',
      'order': 'Order',
      'unassigned': 'Non assigné',
      'partial_value': 'Doit être supérieur à zéro et inférieur au total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Activé(e)',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Définissez un sous-domaine ou affichez la facture sur votre propre site web.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sécurité',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Abonnements',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Éditer l\'abonnement',
      'created_subscription': 'Abonnement créé avec succès',
      'updated_subscription': 'Abonnement mis à jour avec succès',
      'archived_subscription': 'Abonnement archivé avec succès',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Renvoyer une invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Rebondi',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Scannez le code à barres avec une application compatible :link',
      'enabled_two_factor':
          'Authentification à deux facteurs activée avec succès',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Authentification à 2 facteurs',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Paiement remboursé',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Dernier trimestre',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convertir en facture',
      'registration_url': 'Registration URL',
      'invoice_project': 'Facturer le projet',
      'invoice_task': 'Facturer la tâche',
      'invoice_expense': 'Facturer la dépense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Montant converti',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Documents par défaut',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Cacher',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Colonne',
      'sample': 'Exemple',
      'map_to': 'Map To',
      'import': 'Importer',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Veuillez sélectionner un fichier',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Sélectionner un fichier CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'URL Webhook',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Non payé',
      'white_label': 'White Label',
      'delivery_note': 'Bon de livraison',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Solde partiel',
      'invoice_total': 'Montant total',
      'quote_total': 'Montant du devis',
      'credit_total': 'Total Crédit',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Avertissement',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'Cryptogramme visuel',
      'client_name': 'Nom du client',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Statut de tâche mis à jour avec succès',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'catégories de dépense',
      'new_expense_category': 'Nouvelle catégorie de dépense',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Catégorie de dépense créée avec succès',
      'updated_expense_category':
          'Catégorie de dépense mise à jour avec succès',
      'archived_expense_category': 'Catégorie de dépense archivée avec succès',
      'deleted_expense_category': 'La catégorie a été supprimée avec succès',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Catégorie de dépense restaurée avec succès',
      'archived_expense_categories':
          ':count catégorie(s) de dépense archivée(s) avec succès',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Devrait être facturé',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Marquer comme actif',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Facture récurrente',
      'recurring_invoices': 'Factures récurrentes',
      'new_recurring_invoice': 'Nouvelle facture récurrente',
      'edit_recurring_invoice': 'Editer facture récurrente',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Facture récurrente archivée avec succès',
      'deleted_recurring_invoice': 'Facture récurrente supprimée avec succès',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Facture récurrente restaurée avec succès',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Ligne d\'article',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Ouvert(e)',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Voir le portail',
      'copy_link': 'Copy Link',
      'token_billing': 'Enregister les détails de paiement',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Toujours',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Intitulé',
      'client_number': 'Numéro de client',
      'auto_convert': 'Auto Convert',
      'company_name': 'Nom de l\'entreprise',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Les factures ont été envoyées par email avec succès',
      'emailed_quotes': 'Les offres ont été envoyées par courriel avec succès',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Passerelle',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Heures',
      'statement': 'Relevé',
      'taxes': 'Taxes',
      'surcharge': 'Majoration',
      'apply_payment': 'Apply Payment',
      'apply': 'Appliquer',
      'unapplied': 'Unapplied',
      'select_label': 'Sélection intitulé',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'À',
      'health_check': 'Health Check',
      'payment_type_id': 'Type de paiement',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Factures à venir',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Paiements récents',
      'upcoming_quotes': 'Devis à venir',
      'expired_quotes': 'Devis expirés',
      'create_client': 'Créer un client',
      'create_invoice': 'Créer une facture',
      'create_quote': 'Créer un devis',
      'create_payment': 'Create Payment',
      'create_vendor': 'Créer un fournisseur',
      'update_quote': 'Update Quote',
      'delete_quote': 'Supprimer ce devis',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Supprimer la facture',
      'update_client': 'Update Client',
      'delete_client': 'Supprimer ce client',
      'delete_payment': 'Supprimer ce paiement',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Supprimer ce fournisseur',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Supprimer la dépense',
      'create_task': 'Créer une tâche',
      'update_task': 'Update Task',
      'delete_task': 'Supprimer la tâche',
      'approve_quote': 'Approve Quote',
      'off': 'Fermé',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Gratuit',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Cible',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'Jetons d\'API',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Jeton',
      'tokens': 'Jetons',
      'new_token': 'New Token',
      'edit_token': 'Éditer ce jeton',
      'created_token': 'Jeton créé avec succès',
      'updated_token': 'Jeton mis à jour avec succès',
      'archived_token': 'Jeton archivé avec succès',
      'deleted_token': 'Jeton supprimé avec succès',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Envoyer la facture par courriel',
      'email_quote': 'Envoyer ce devis par courriel',
      'email_credit': 'Email Credit',
      'email_payment': 'Reçu du paiement par courriel',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Nom du contact',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Éditer la condition de paiement',
      'created_payment_term': 'Conditions de paiement créées avec succès',
      'updated_payment_term': 'Conditions de paiement mises à jour avec succès',
      'archived_payment_term': 'Conditions de paiement archivées avec succès',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Montant du crédit',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusif',
      'inclusive': 'Inclusif',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Remboursement du paiement',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Nom complet',
      'city_state_postal': 'Ville/ Province (Département)/ CP',
      'postal_city_state': 'Ville/Province (Département)/Code postal',
      'custom1': 'Personnalisé1',
      'custom2': 'Personnalisé2',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optionnel',
      'license': 'Licence',
      'purge_data': 'Purger les données',
      'purge_successful':
          'Les données de l\'entreprise ont été purgées avec succès',
      'purge_data_message':
          'Attention : Cette action va supprimer vos données et est irréversible',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 jours',
      'age_group_30': '30 -60 jours',
      'age_group_60': '60 - 90 jours',
      'age_group_90': '90 - 120 jours',
      'age_group_120': '120+ jours',
      'refresh': 'Rafraîchir',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Détails de la facture',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'Aucun(e)',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count facture envoyée',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Activer la licence',
      'cancel_account': 'Supprimer le compte',
      'cancel_account_message':
          'Attention : Ceci va supprimer définitivement votre compte, il n\'y a pas d\'annulation possible.',
      'delete_company': 'Supprimer la société',
      'delete_company_message':
          'Attention : Ceci supprimera définitivement votre société, il n\'y a pas d\'annulation.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'En-tête',
      'load_design': 'Charger un modèle',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Propositions',
      'tickets': 'Tickets',
      'recurring_quotes': 'Devis récurrent',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Dépenses récurrentes',
      'account_management': 'Gestion des comptes',
      'credit_date': 'Date d\'avoir',
      'credit': 'Crédit',
      'credits': 'Crédits',
      'new_credit': 'Entrer un crédit',
      'edit_credit': 'Éditer le crédit',
      'created_credit': 'Crédit créé avec succès',
      'updated_credit': 'Le crédit a été mis à jour avec succès',
      'archived_credit': 'Crédit archivé avec succès',
      'deleted_credit': 'Crédit supprimé avec succès',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Crédit restauré avec succès',
      'archived_credits': ':count crédits archivés avec succès',
      'deleted_credits': ':count crédits supprimés avec succès',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Version actuelle',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'En savoir plus',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nouveau compte',
      'added_company': 'L\'entreprise a été ajoutée',
      'company1': 'Champ personnalisé Entreprise 1',
      'company2': 'Champ personnalisé Entreprise 2',
      'company3': 'Champ personnalisé Entreprise 3',
      'company4': 'Champ personnalisé Entreprise 4',
      'product1': 'Champ personnalisé Produit 1',
      'product2': 'Champ personnalisé Produit 2',
      'product3': 'Champ personnalisé Produit 3',
      'product4': 'Champ personnalisé Produit 4',
      'client1': 'Champ personnalisé Client 1',
      'client2': 'Client personnalisé 2',
      'client3': 'Client personnalisé 3',
      'client4': 'Client personnalisé 4',
      'contact1': 'Champ personnalisé Contact 1',
      'contact2': 'Champ personnalisé Contact 2',
      'contact3': 'Champ personnalisé Contact 3',
      'contact4': 'Champ personnalisé Contact 4',
      'task1': 'Champ personnalisé Tâche 1',
      'task2': 'Champ personnalisé Tâche 2',
      'task3': 'Champ personnalisé Tâche 3',
      'task4': 'Champ personnalisé Tâche 4',
      'project1': 'Champ personnalisé Projet 1',
      'project2': 'Champ personnalisé Projet 2',
      'project3': 'Champ personnalisé Projet 3',
      'project4': 'Champ personnalisé Projet 4',
      'expense1': 'Champ personnalisé Dépense 1',
      'expense2': 'Champ personnalisé Dépense 2',
      'expense3': 'Champ personnalisé Dépense 3',
      'expense4': 'Champ personnalisé Dépense 4',
      'vendor1': 'Fournisseur personnalisé 1',
      'vendor2': 'Fournisseur personnalisé 2',
      'vendor3': 'Fournisseur personnalisé 3',
      'vendor4': 'Fournisseur personnalisé 4',
      'invoice1': 'Champ personnalisé Facture 1',
      'invoice2': 'Champ personnalisé Facture 2',
      'invoice3': 'Champ personnalisé Facture 3',
      'invoice4': 'Champ personnalisé Facture 4',
      'payment1': 'Champ personnalisé Paiement 1',
      'payment2': 'Champ personnalisé Paiement 2',
      'payment3': 'Champ personnalisé Paiement 3',
      'payment4': 'Champ personnalisé Paiement 4',
      'surcharge1': 'Autre frais 1',
      'surcharge2': 'Autre frais 2',
      'surcharge3': 'Autre frais 3',
      'surcharge4': 'Autre frais 4',
      'group1': 'Champ personnalisé Groupe 1',
      'group2': 'Champ personnalisé Groupe 2',
      'group3': 'Champ personnalisé Groupe 3',
      'group4': 'Champ personnalisé Groupe 4',
      'reset': 'Remettre à zéro',
      'number': 'Nombre',
      'export': 'Exporter',
      'chart': 'Graphique',
      'count': 'Compte',
      'totals': 'Totaux',
      'blank': 'Vide',
      'day': 'Jour',
      'month': 'Mois',
      'year': 'Année',
      'subgroup': 'Sous-groupe',
      'is_active': 'Actif',
      'group_by': 'Grouper par',
      'credit_balance': 'Solde du crédit',
      'contact_last_login': 'Dernière connexion du contact',
      'contact_full_name': 'Nom du contact',
      'contact_phone': 'Téléphone du contact',
      'contact_custom_value1': 'Valeur champ personnalisé Contact 1',
      'contact_custom_value2': 'Valeur champ personnalisé Contact 2',
      'contact_custom_value3': 'Valeur champ personnalisé Contact 3',
      'contact_custom_value4': 'Valeur champ personnalisé Contact 4',
      'shipping_address1': 'Rue',
      'shipping_address2': 'Appt/Bâtiment',
      'shipping_city': 'Ville',
      'shipping_state': 'Région/Département',
      'shipping_postal_code': 'Code postal',
      'shipping_country': 'Pays',
      'billing_address1': 'Rue',
      'billing_address2': 'Appt/Bâtiment',
      'billing_city': 'Ville',
      'billing_state': 'Région/Département',
      'billing_postal_code': 'Code postal',
      'billing_country': 'Pays',
      'client_id': 'ID du client',
      'assigned_to': 'Assigné à',
      'created_by': 'Créé par :name',
      'assigned_to_id': 'Assigné à ID',
      'created_by_id': 'Créé par ID',
      'add_column': 'Ajouter une colonne',
      'edit_columns': 'Éditer les colonnes',
      'columns': 'Colonnes',
      'aging': 'Vieillissement',
      'profit_and_loss': 'Profits et Pertes',
      'reports': 'Rapports',
      'report': 'Rapport',
      'add_company': 'Ajouter compte',
      'unpaid_invoice': 'Facture impayée',
      'paid_invoice': 'Facture payée',
      'unapproved_quote': 'Devis non-approuvé',
      'help': 'Aide',
      'refund': 'Remboursement',
      'refund_date': 'Date du remboursement',
      'filtered_by': 'Filtré par',
      'contact_email': 'Courriel du contact',
      'multiselect': 'Sélection multiple',
      'entity_state': 'État',
      'verify_password': 'Vérifier le mot de passe',
      'applied': 'Publié',
      'include_recent_errors': 'Contient les erreurs récentes des journaux',
      'your_message_has_been_received':
          'Nous avons reçu votre message et répondrons dans les meilleurs délais',
      'message': 'Message',
      'from': 'De',
      'show_product_details': 'Voir les détails du produit',
      'show_product_details_help':
          'Veuillez inclure la description et le coût dans la liste déroulante du produit',
      'pdf_min_requirements':
          'Le générateur de PDF nécessite la version :version',
      'adjust_fee_percent': 'Ajuster le pourcentage de frais',
      'adjust_fee_percent_help': 'Ajuster le frais de pourcentage au compte',
      'configure_settings': 'Modifier les paramètres',
      'support_forum': 'forum de support',
      'about': 'À propos',
      'documentation': 'Documentation',
      'contact_us': 'Nous joindre',
      'subtotal': 'Sous-total',
      'line_total': 'Total',
      'item': 'Article',
      'credit_email': 'Courriel de crédit',
      'iframe_url': 'Site internet',
      'domain_url': 'URL du domaine',
      'password_is_too_short': 'Mot de passe trop court',
      'password_is_too_easy':
          'Le mot de passe doit comporter au moins une majuscule et un nombre',
      'client_portal_tasks': 'Tâche du portail client',
      'client_portal_dashboard': 'Tableau de bord du portail client',
      'please_enter_a_value': 'Saisissez une valeur',
      'deleted_logo': 'Le logo a été supprimé',
      'yes': 'Oui',
      'no': 'Non',
      'generate_number': 'Générer un nombre',
      'when_saved': 'Lors de la sauvegarde',
      'when_sent': 'Lors de l\'envoi',
      'select_company': 'Sélectionner une entreprise',
      'float': 'Flottant',
      'collapse': 'Réduire',
      'show_or_hide': 'Afficher/cacher',
      'menu_sidebar': 'Barre latérale du menu',
      'history_sidebar': 'Historique latéral',
      'tablet': 'Tablette',
      'mobile': 'Mobile',
      'desktop': 'Bureau',
      'layout': 'Présentation',
      'view': 'Voir',
      'module': 'Module',
      'first_custom': 'Premier personnalisé',
      'second_custom': 'Second personnalisé',
      'third_custom': 'Troisième personnalisé',
      'show_cost': 'Voir le coût',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Afficher un champ coût du produit pour suivre la marge',
      'show_product_quantity': 'Voir la quantité du produit',
      'show_product_quantity_help':
          'Afficher un champ de quantité du produit, sinon en choisir un par défaut',
      'show_invoice_quantity': 'Voir la quantité sur la facture',
      'show_invoice_quantity_help':
          'Afficher un champ de quantité pour la position, sinon en choisir un par défaut',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Quantité par défaut',
      'default_quantity_help':
          'Mettre automatiquement la quantité de la position à un',
      'one_tax_rate': 'Un taux de taxe',
      'two_tax_rates': 'Deux taux de taxe',
      'three_tax_rates': 'Trois taux de taxe',
      'default_tax_rate': 'Taux de taxe par défaut',
      'user': 'Utilisateur',
      'invoice_tax': 'Taxe de la facture',
      'line_item_tax': 'Taxe de la position',
      'inclusive_taxes': 'Taxes incluses',
      'invoice_tax_rates': 'Taux de taxe de la facture',
      'item_tax_rates': 'Taux de taxe de la position',
      'no_client_selected': 'Veuillez sélectionner un client',
      'configure_rates': 'Configurer les taux',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Réglages des taxes',
      'tax_settings_rates': 'Taux de taxes',
      'accent_color': 'Couleur de mise en évidence',
      'switch': 'Changer',
      'comma_sparated_list': 'Liste séparée par des virgules',
      'options': 'Options',
      'single_line_text': 'Texte sur une ligne',
      'multi_line_text': 'Texte multi-lignes',
      'dropdown': 'Liste déroulante',
      'field_type': 'Type du champ',
      'recover_password_email_sent':
          'Un courriel de récupération du mot de passe a été envoyé',
      'submit': 'Envoyer',
      'recover_password': 'Récupérer votre mot de passe',
      'late_fees': 'Frais de retard',
      'credit_number': 'Numéro d\'avoir',
      'payment_number': 'Numéro de paiement',
      'late_fee_amount': 'Montant de pénalité de retard',
      'late_fee_percent': 'Pourcentage de pénalité de retard',
      'schedule': 'Planification',
      'before_due_date': 'Avant la date d\'échéance',
      'after_due_date': 'Après la date d\'échéance',
      'after_invoice_date': 'Après la date de facturation',
      'days': 'Jours',
      'invoice_email': 'Courriel de facture',
      'payment_email': 'Courriel de paiement',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Courriel de devis',
      'endless_reminder': 'Rappel sans fin',
      'filtered_by_user': 'Filtré par utilisateur',
      'administrator': 'Administrateur',
      'administrator_help':
          'Permettre à l\'utilisateur de gérer les utilisateurs, modifier les paramètres et de modifier tous les enregistrements',
      'user_management': 'Gestion des utilisateurs',
      'users': 'Utilisateurs',
      'new_user': 'Nouvel utilisateur',
      'edit_user': 'Éditer l\'utilisateur',
      'created_user': 'Utilisateur créé avec succès avec succès',
      'updated_user': 'Utilisateur mis à jour avec succès',
      'archived_user': 'Utilisateur archivé avec succès',
      'deleted_user': 'Utilisateur supprimé avec succès',
      'removed_user': 'L\'utilisateur a été supprimé',
      'restored_user': 'Commande restaurée avec succès',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Paramètres généraux',
      'invoice_options': 'Options de facturation',
      'hide_paid_to_date': 'Masquer \"Payé à ce jour\"',
      'hide_paid_to_date_help':
          'Afficher la ligne \"Payé à ce jour\" sur vos factures seulement une fois qu\'un paiement a été reçu.',
      'invoice_embed_documents': 'Documents intégrés',
      'invoice_embed_documents_help':
          'Inclure l\'image attachée dans la facture.',
      'all_pages_header': 'Voir les en-têtes sur',
      'all_pages_footer': 'Voir les pieds de page sur',
      'first_page': 'Première page',
      'all_pages': 'Toutes les pages',
      'last_page': 'Dernière page',
      'primary_font': 'Police principale',
      'secondary_font': 'Police secondaire',
      'primary_color': 'Couleur principale',
      'secondary_color': 'Couleur secondaire',
      'page_size': 'Taille de Page',
      'font_size': 'Taille de police',
      'quote_design': 'Mise en page des Devis',
      'invoice_fields': 'Champs de facture',
      'product_fields': 'Champs de produit',
      'invoice_terms': 'Conditions de facturation',
      'invoice_footer': 'Pied de facture',
      'quote_terms': 'Conditions des devis',
      'quote_footer': 'Pied de page des devis',
      'auto_email_invoice': 'Envoyer automatiquement par courriel',
      'auto_email_invoice_help':
          'Envoyer automatiquement par courriel les factures récurrentes lorsqu\'elles sont créés.',
      'auto_archive_invoice': 'Archiver automatiquement',
      'auto_archive_invoice_help':
          'Archiver automatiquement les factures lorsqu\'elles sont payées.',
      'auto_archive_quote': 'Archiver automatiquement',
      'auto_archive_quote_help':
          'Archiver automatiquement les devis lorsqu\'ils sont convertis.',
      'auto_convert_quote': 'Convertir automatiquement',
      'auto_convert_quote_help':
          'Convertir automatiquement un devis en facture dès qu\'il est approuvé par le client.',
      'workflow_settings': 'Paramètres de flux de travail',
      'freq_daily': 'Quotidien',
      'freq_weekly': 'Hebdomadaire',
      'freq_two_weeks': 'Deux semaines',
      'freq_four_weeks': 'Quatre semaines',
      'freq_monthly': 'Mensuelle',
      'freq_two_months': 'Deux mois',
      'freq_three_months': 'Trimestrielle',
      'freq_four_months': 'Quatre mois',
      'freq_six_months': 'Six mois',
      'freq_annually': 'Annuelle',
      'freq_two_years': 'Deux ans',
      'freq_three_years': 'Trois ans',
      'never': 'Jamais',
      'company': 'Entreprise',
      'generated_numbers': 'Numéros générés',
      'charge_taxes': 'Taxe supplémentaire',
      'next_reset': 'Prochaine remise à zéro',
      'reset_counter': 'Remettre le compteur à zéro',
      'recurring_prefix': 'Préfixe récurrent',
      'number_padding': 'Marge interne du nombre',
      'general': 'Général',
      'surcharge_field': 'Champ Surcharge',
      'company_field': 'Champ d\'entreprise',
      'company_value': 'Valeur de compagnie',
      'credit_field': 'Champ de Crédit',
      'invoice_field': 'Champ de facture',
      'invoice_surcharge': 'Majoration de facture',
      'client_field': 'Champ de client',
      'product_field': 'Champ de produit',
      'payment_field': 'Champ de Paiement',
      'contact_field': 'Champ de contact',
      'vendor_field': 'Champ de fournisseur',
      'expense_field': 'Champ de dépense',
      'project_field': 'Champ de projet',
      'task_field': 'Champ de tâche',
      'group_field': 'Champ de Groupe',
      'number_counter': 'Compteur de nombre',
      'prefix': 'Préfixe',
      'number_pattern': 'Modèle de nombre',
      'messages': 'Messages',
      'custom_css': 'CSS personnalisé',
      'custom_javascript': 'JavaScript personnalisé',
      'signature_on_pdf': 'Afficher sur le PDF',
      'signature_on_pdf_help':
          'Afficher la signature du client sur la facture / le devis PDF.',
      'show_accept_invoice_terms':
          'Case à cocher pour les conditions de facturation',
      'show_accept_invoice_terms_help':
          'Exiger que le client confirme qu\'il accepte les conditions de facturation',
      'show_accept_quote_terms': 'Case à cocher pour les conditions d\'offre',
      'show_accept_quote_terms_help':
          'Exiger que le client confirme qu\'il accepte les conditions de l\'offre',
      'require_invoice_signature': 'Signature de facture',
      'require_invoice_signature_help': 'Exiger que le client signe',
      'require_quote_signature': 'Signature de l\'offre',
      'enable_portal_password': 'Protéger les factures avec un mot de passe',
      'enable_portal_password_help':
          'Autoriser la création d\'un mot de passe pour chaque contact. Si un mot de passe est créé, le contact devra entrer un mot de passe avant de voir les factures.',
      'authorization': 'Autorisation',
      'subdomain': 'Sous-domaine',
      'domain': 'Domaine',
      'portal_mode': 'Mode portail',
      'email_signature': 'Cordialement,',
      'enable_email_markup_help':
          'Rendez le règlement de vos clients plus facile en ajoutant les markup schema.org à vos courriels.',
      'plain': 'Brut',
      'light': 'Clair',
      'dark': 'Sombre',
      'email_design': 'Modèle de courriel',
      'attach_pdf': 'Joindre PDF',
      'attach_documents': 'Joindre les Documents',
      'attach_ubl': 'Joindre UBL',
      'email_style': 'Style d\'email',
      'enable_email_markup': 'Activer le balisage',
      'reply_to_email': 'Adresse de réponse',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Courriel CCI',
      'processed': 'Traité',
      'credit_card': 'Carte de Crédit',
      'bank_transfer': 'Virement bancaire',
      'priority': 'Prioritée',
      'fee_amount': 'Montant des frais',
      'fee_percent': 'Pourcentage des frais',
      'fee_cap': 'Limite des frais',
      'limits_and_fees': 'Limites/Frais',
      'enable_min': 'Activer min',
      'enable_max': 'Activer max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Logos des cartes acceptées',
      'credentials': 'Identifiants',
      'update_address': 'Mettre à jour l\'adresse',
      'update_address_help':
          'Mettre à jour l\'adresse du client avec les détails fournis',
      'rate': 'Taux',
      'tax_rate': 'Taux de taxe',
      'new_tax_rate': 'Nouveau taux de taxe',
      'edit_tax_rate': 'Éditer le taux de taxe',
      'created_tax_rate': 'Taux de taxe créé avec succès',
      'updated_tax_rate': 'Taux de taxe mis à jour avec succès',
      'archived_tax_rate': 'Taux de taxe archivé avec succès',
      'deleted_tax_rate': 'Le taux de taxe a été supprimé avec succès',
      'restored_tax_rate': 'Le taux de taxe a été restauré avec succès',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Remplissage auto des produits',
      'fill_products_help':
          'La sélection d’un produit entrainera la MAJ de la description et du prix',
      'update_products': 'Mise à jour auto des produits',
      'update_products_help':
          'La mise à jour d\'une facture entraîne la mise à jour des produits',
      'convert_products': 'Convertir les produits',
      'convert_products_help':
          'Convertir automatiquement les prix des produits dans la devise du client',
      'fees': 'Frais',
      'limits': 'Limites',
      'provider': 'Fournisseur',
      'company_gateway': 'Passerelle de paiement',
      'company_gateways': 'Passerelles de paiements',
      'new_company_gateway': 'Nouvelle passerelle',
      'edit_company_gateway': 'Éditer la passerelle',
      'created_company_gateway': 'La passerelle a été créée avec succès',
      'updated_company_gateway': 'La passerelle a été mise à jour avec succès',
      'archived_company_gateway': 'La passerelle a été archivée avec succès',
      'deleted_company_gateway': 'La passerelle a été supprimée avec succès',
      'restored_company_gateway': 'La passerelle a été restaurée avec succès',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continuer l\'édition',
      'discard_changes': 'Ignorer les modifications',
      'default_value': 'Valeur Par Défaut',
      'disabled': 'Désactivé',
      'currency_format': 'Format de devise',
      'first_day_of_the_week': 'Premier Jour de la Semaine',
      'first_month_of_the_year': 'Premier mois de l\'Année',
      'sunday': 'Dimanche',
      'monday': 'Lundi',
      'tuesday': 'Mardi',
      'wednesday': 'Mercredi',
      'thursday': 'Jeudi',
      'friday': 'Vendredi',
      'saturday': 'Samedi',
      'january': 'Janvier',
      'february': 'Février',
      'march': 'Mars',
      'april': 'Avril',
      'may': 'Mai',
      'june': 'Juin',
      'july': 'Juillet',
      'august': 'Août',
      'september': 'Septembre',
      'october': 'Octobre',
      'november': 'Novembre',
      'december': 'Décembre',
      'symbol': 'Symbole',
      'ocde': 'Code',
      'date_format': 'Format de la date',
      'datetime_format': 'Format date/heure',
      'military_time': '24H',
      'military_time_help': 'Affichage sur 24h',
      'send_reminders': 'Envoyer des rappels',
      'timezone': 'Fuseau horaire',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtrer par groupe',
      'filtered_by_invoice': 'Filtré par Facture',
      'filtered_by_client': 'Filtré par Client',
      'filtered_by_vendor': 'Filtré par Vendeur',
      'group_settings': 'Paramètres de groupe',
      'group': 'Groupe',
      'groups': 'Groupes',
      'new_group': 'Nouveau Groupe',
      'edit_group': 'Éditer le groupe',
      'created_group': 'Le groupe a été créé avec succès',
      'updated_group': 'Le groupe a été mis à jour avec succès',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Envoyer le logo',
      'uploaded_logo': 'Le logo a été envoyé avec succès',
      'logo': 'Logo',
      'saved_settings': 'Les paramètres ont été sauvegardés avec succès',
      'product_settings': 'Réglages du produit',
      'device_settings': 'Paramètres de l\'appareil',
      'defaults': 'Valeurs par défaut',
      'basic_settings': 'Paramètres de base',
      'advanced_settings': 'Paramètres avancés',
      'company_details': 'Informations sur l’entreprise',
      'user_details': 'Utilisateur',
      'localization': 'Localisation',
      'online_payments': 'Paiements en ligne',
      'tax_rates': 'Taux de taxe',
      'notifications': 'Notifications',
      'import_export': 'Importer/Exporter',
      'custom_fields': 'Champs personnalisés',
      'invoice_design': 'Modèle de facture',
      'buy_now_buttons': 'Boutons Achetez maintenant',
      'email_settings': 'Paramètres de courriel',
      'templates_and_reminders': 'Modèles & Rappels',
      'credit_cards_and_banks': 'Cartes de crédit et banques',
      'data_visualizations': 'Visualisation des données',
      'price': 'Prix',
      'email_sign_up': 'Inscription par email',
      'google_sign_up': 'Inscription avec Google',
      'thank_you_for_your_purchase': 'Merci pour votre achat !',
      'redeem': 'Rembourser',
      'back': 'Retour',
      'past_purchases': 'Achats antérieurs',
      'annual_subscription': 'Abonnement annuel',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count utilisateur(s)',
      'upgrade': 'Mettre à niveau',
      'please_enter_a_first_name': 'Veuillez entrer un prénom',
      'please_enter_a_last_name': 'Veuillez entrer un nom',
      'please_agree_to_terms_and_privacy':
          'Veuillez accepter les conditions d\'utilisation et la politique de confidentialité pour créer un compte.',
      'i_agree_to_the': 'J\'accepte les',
      'terms_of_service_link': 'Conditions d\'utilisation',
      'privacy_policy_link': 'Politique de confidentialité',
      'terms_of_service': 'Conditions d\'utilisation',
      'privacy_policy': 'Politique de confidentialité',
      'sign_up': 'S’enregistrer',
      'account_login': 'Connexion à votre compte',
      'view_website': 'Voir le site Web',
      'create_account': 'Créer un compte',
      'email_login': 'Email de connexion',
      'create_new': 'Créer',
      'no_record_selected': 'Aucun enregistrement sélectionné',
      'error_unsaved_changes':
          'Veuillez enregistrer ou annuler vos modifications',
      'download': 'Télécharger',
      'requires_an_enterprise_plan': 'Χρειάζεται πλάνο επιχείρησης',
      'take_picture': 'Φωτογραφίσετε',
      'upload_file': 'Envoyer un fichier',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'Νέο Έγγραφο',
      'edit_document': 'Εκδώσετε Έγγραφο',
      'uploaded_document': 'Le document a été envoyé avec succès',
      'updated_document': 'Document mis à jour avec succès',
      'archived_document': 'Document archivé avec succès',
      'deleted_document': 'Le document a été supprimé avec succès',
      'restored_document': 'Le document a été restauré avec succès',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Κανένα Ιστορικό',
      'expense_date': 'Date de la dépense',
      'pending': 'En attente',
      'expense_status_1': 'Σύνδεση',
      'expense_status_2': 'Σε εκκρεμότητα',
      'expense_status_3': 'Με τιμολόγιο',
      'converted': 'Converti',
      'add_documents_to_invoice': 'Ajouter un document à la facture',
      'exchange_rate': 'Taux de change',
      'convert_currency': 'Convertir la devise',
      'mark_paid': 'Marquer comme payé',
      'category': 'Catégorie',
      'address': 'Adresse',
      'new_vendor': 'Nouveau fournisseur',
      'created_vendor': 'Fournisseur créé avec succès',
      'updated_vendor': 'Founisseur mis à jour avec succès',
      'archived_vendor': 'Fournisseur archivé avec succès',
      'deleted_vendor': 'Fournisseur supprimé avec succès',
      'restored_vendor': 'Fournisseur restauré avec succès',
      'archived_vendors': ':count fournisseurs archivés avec succès',
      'deleted_vendors': ':count fournisseurs supprimés avec succès',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Saisir une dépense',
      'created_expense': 'Dépense créée avec succès',
      'updated_expense': 'Dépense mise à jour avec succès',
      'archived_expense': 'Dépense archivée avec succès',
      'deleted_expense': 'Dépense supprimée avec succès',
      'restored_expense': 'Dépense restaurée avec succès',
      'archived_expenses': 'Dépenses archivées avec succès',
      'deleted_expenses': 'Dépenses supprimées avec succès',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copier expédition',
      'copy_billing': 'Copier facturation',
      'design': 'Design',
      'failed_to_find_record': 'Élément non trouvé',
      'invoiced': 'Facturé',
      'logged': 'Enregistré',
      'running': 'En cours',
      'resume': 'Reprendre',
      'task_errors': 'Merci de corriger les horaires conflictuels',
      'start': 'Début',
      'stop': 'Fin',
      'started_task': 'Tâche démarrée avec succès',
      'stopped_task': 'Tâche stoppée avec succès',
      'resumed_task': 'Tâche relancée avec succès',
      'now': 'Maintenant',
      'auto_start_tasks': 'Démarrer automatiquement les tâches',
      'timer': 'Compteur',
      'manual': 'Manuel',
      'budgeted': 'Budgétisé',
      'start_time': 'Début',
      'end_time': 'Heure de fin',
      'date': 'Date',
      'times': 'Horaires',
      'duration': 'Durée',
      'new_task': 'Nouvelle tâche',
      'created_task': 'Tâche créée avec succès',
      'updated_task': 'Tâche mise à jour avec succès',
      'archived_task': 'Tâche archivée avec succès',
      'deleted_task': 'Tâche supprimée avec succès',
      'restored_task': 'Tâche restaurée avec succès',
      'archived_tasks': ':count tâches archivées avec succès',
      'deleted_tasks': ':count tâches supprimées avec succès',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Veuillez entrer un nom',
      'budgeted_hours': 'Heures budgétées',
      'created_project': 'Le projet a été créé avec succès',
      'updated_project': 'Le projet a été mis à  jour avec succès',
      'archived_project': 'Le projet a été archivé avec succès',
      'deleted_project': 'Le projet a été supprimé avec succès',
      'restored_project': 'Le projet a été rétabli avec succès',
      'archived_projects': ':count projet(s) a (ont) été archivé(s)',
      'deleted_projects':
          ':count projet(s) a (ont) été supprimé(s) avec succès',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nouveau projet',
      'thank_you_for_using_our_app': 'Merci d\'utiliser notre app !',
      'if_you_like_it': 'Si vous appréciez, merci de',
      'click_here': 'cliquer ici',
      'click_here_capital': 'Cliquer ici',
      'to_rate_it': 'pour évaluer notre app.',
      'average': 'Moyenne',
      'unapproved': 'Non approuvé',
      'authenticate_to_change_setting':
          'Veuillez vous connecter pour changer ce paramètre',
      'locked': 'Verrouillé',
      'authenticate': 'Connexion',
      'please_authenticate': 'Veuillez vous connecter',
      'biometric_authentication': 'Connexion biométrique',
      'footer': 'Pied de page',
      'compare': 'Comparer',
      'hosted_login': 'Authentification Hosted',
      'selfhost_login': 'Authentification Selfhost',
      'google_sign_in': 'Login with Google',
      'today': 'Aujourd\'hui',
      'custom_range': 'Intervalle personnalisé',
      'date_range': 'Intervalle de dates',
      'current': 'Actuel',
      'previous': 'Précédent',
      'current_period': 'Période actuelle',
      'comparison_period': 'Comparaison de période',
      'previous_period': 'Période précédente',
      'previous_year': 'Année précédente',
      'compare_to': 'Comparer à',
      'last7_days': '7 derniers jours',
      'last_week': 'Semaine dernière',
      'last30_days': '30 derniers jours',
      'this_month': 'Mois en cours',
      'last_month': 'Mois dernier',
      'this_year': 'Cette année',
      'last_year': 'Dernière année',
      'custom': 'Personnalisé',
      'clone_to_invoice': 'Dupliquer la facture',
      'clone_to_quote': 'Dupliquer en devis',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Voir la facture',
      'convert': 'Convertir',
      'more': 'Plus',
      'edit_client': 'Modifier ce client',
      'edit_product': 'Éditer ce produit',
      'edit_invoice': 'Modifier la facture',
      'edit_quote': 'Éditer ce devis',
      'edit_payment': 'Éditer le paiement',
      'edit_task': 'Éditer la tâche',
      'edit_expense': 'Éditer la dépensee',
      'edit_vendor': 'Éditer le fournisseur',
      'edit_project': 'Editer le projet',
      'edit_recurring_expense': 'Éditer la dépense récurrente',
      'edit_recurring_quote': 'Editer devis récurrent',
      'billing_address': 'Adresse de facturation',
      'shipping_address': 'Adresse de Livraison',
      'total_revenue': 'Revenu total',
      'average_invoice': 'Facture moyenne',
      'outstanding': 'Impayé',
      'invoices_sent': ':count factures envoyées',
      'active_clients': 'clients actifs',
      'close': 'Fermer',
      'email': 'Courriel',
      'password': 'Mot de passe',
      'url': 'URL',
      'secret': 'Clé secrète',
      'name': 'Nom',
      'logout': 'Se déconnecter',
      'login': 'Connexion',
      'filter': 'Filtrer',
      'sort': 'Trier',
      'search': 'Rechercher',
      'active': 'Actif',
      'archived': 'Archivé',
      'deleted': 'Supprimé',
      'dashboard': 'Tableau de bord',
      'archive': 'Archiver',
      'delete': 'Supprimer',
      'restore': 'Restaurer',
      'refresh_complete': 'Rafraichissement terminé',
      'please_enter_your_email': 'Entrez votre adresse e-mail',
      'please_enter_your_password': 'Entez votre mot de passe',
      'please_enter_your_url': 'Entrez votre URL',
      'please_enter_a_product_key': 'Entrez la clé produit',
      'ascending': 'Ascendant',
      'descending': 'Descendant',
      'save': 'Sauvegarder',
      'an_error_occurred': 'Une erreur s\'est produite',
      'paid_to_date': 'Payé à ce jour',
      'balance_due': 'Montant dû',
      'balance': 'Solde',
      'overview': 'Vue d\'ensemble',
      'details': 'Détails',
      'phone': 'Téléphone',
      'website': 'Site Web',
      'vat_number': 'Numéro de TVA',
      'id_number': 'Numéro ID',
      'create': 'Créer',
      'copied_to_clipboard': ':value a été copié au presse-papier',
      'error': 'Erreur',
      'could_not_launch': 'Lancement impossible',
      'contacts': 'Informations de contact',
      'additional': 'Additionnel',
      'first_name': 'Prénom',
      'last_name': 'Nom',
      'add_contact': 'Ajouter un contact',
      'are_you_sure': 'Voulez-vous vraiment effectuer cette action ?',
      'cancel': 'Annuler',
      'ok': 'Ok',
      'remove': 'Supprimer',
      'email_is_invalid': 'L\'adresse de courriel n\'est pas correcte',
      'product': 'Produit',
      'products': 'Produits',
      'new_product': 'Nouvel article',
      'created_product': 'Produit créé avec succès',
      'updated_product': 'Produit mis à jour avec succès',
      'archived_product': 'Produit archivé avec succès',
      'deleted_product': 'Le produit a été supprimé avec succès',
      'restored_product': 'Le produit a été rétabli avec succès',
      'archived_products': ':count produits archivés avec succès',
      'deleted_products': ':count produit(s) supprimé(s) avec succès',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produit',
      'notes': 'Notes',
      'cost': 'Coût',
      'client': 'Client',
      'clients': 'Clients',
      'new_client': 'Nouveau client',
      'created_client': 'Client créé avec succès',
      'updated_client': 'Client modifié avec succès',
      'archived_client': 'Client archivé avec succès',
      'archived_clients': ':count clients archivés avec succès',
      'deleted_client': 'Client supprimé avec succès',
      'deleted_clients': ':count clients supprimés avec succès',
      'restored_client': 'Client restauré avec succès',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Rue',
      'address2': 'Appt/Bâtiment',
      'city': 'Ville',
      'state': 'Région/Département',
      'postal_code': 'Code postal',
      'country': 'Pays',
      'invoice': 'Facture',
      'invoices': 'Factures',
      'new_invoice': 'Nouvelle facture',
      'created_invoice': 'Facture créée avec succès',
      'updated_invoice': 'Facture modifiée avec succès',
      'archived_invoice': 'Facture archivée avec succès',
      'deleted_invoice': 'Facture supprimée avec succès',
      'restored_invoice': 'Facture restaurée avec succès',
      'archived_invoices': ':count factures archivées avec succès',
      'deleted_invoices': ':count factures supprimées avec succès',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Facture envoyée par courriel avec succès',
      'emailed_payment': 'Paiement envoyé par email avec succès',
      'amount': 'Montant',
      'invoice_number': 'Numéro de facture',
      'invoice_date': 'Date de facture',
      'discount': 'Remise',
      'po_number': 'N° de Bon de Commande',
      'terms': 'Conditions',
      'public_notes': 'Note publique',
      'private_notes': 'Notes personnelles',
      'frequency': 'Fréquence',
      'start_date': 'Date de début',
      'end_date': 'Date de fin',
      'quote_number': 'Devis numéro',
      'quote_date': 'Date du devis',
      'valid_until': 'Valide jusqu\'au',
      'items': 'Articles',
      'partial_deposit': 'Depot Partial',
      'description': 'Description',
      'unit_cost': 'Coût unitaire',
      'quantity': 'Quantité',
      'add_item': 'Ajouter Article',
      'contact': 'Contact',
      'work_phone': 'Téléphone',
      'total_amount': 'Montant Total',
      'pdf': 'Fichier PDF',
      'due_date': 'Date d\'échéance',
      'partial_due_date': 'Paiement partiel',
      'status': 'Statut',
      'invoice_status_id': 'Etat de Facture',
      'quote_status': 'État du devis',
      'click_plus_to_add_item': 'Cliquer pour ajouter un article (objet)',
      'click_plus_to_add_time': 'Cliquez sur + pour ajouter du temps',
      'count_selected': 'nombre selectionne',
      'total': 'Total',
      'percent': 'Pourcent',
      'edit': 'Éditer',
      'dismiss': 'Quitter',
      'please_select_a_date': 'Sélectionnez une date',
      'please_select_a_client': 'Sélectionnez un client',
      'please_select_an_invoice': 'Sélectionnez une facture',
      'task_rate': 'Taux de tâche',
      'settings': 'Paramètres',
      'language': 'Langue',
      'currency': 'Devise',
      'created_at': 'Date de création',
      'created_on': 'Created On',
      'updated_at': 'Mis à jour',
      'tax': 'Taxe',
      'please_enter_an_invoice_number': 'Sélectionnez un numéro de facture',
      'please_enter_a_quote_number': 'Sélectionner un numéro de devis',
      'past_due': 'En retard',
      'draft': 'Brouillon',
      'sent': 'Envoyé',
      'viewed': 'Vu',
      'approved': 'Approuvé',
      'partial': 'Partiel/dépôt',
      'paid': 'Payé',
      'mark_sent': 'Marquer comme envoyé',
      'marked_invoice_as_sent': 'Facture marquee comme envoyee avec succes',
      'marked_invoice_as_paid': 'Facture marquée comme envoyée',
      'marked_invoices_as_sent': 'Les factures ont été marquées envoyées',
      'marked_invoices_as_paid': 'Factures marquées comme envoyées',
      'done': 'Terminé',
      'please_enter_a_client_or_contact_name':
          'Veuillez introduire un nom de client',
      'dark_mode': 'Mode sombre',
      'restart_app_to_apply_change':
          'Recommencer k\'app pour introduire l\'app change',
      'refresh_data': 'Rafraîchir les données',
      'blank_contact': 'Details pour contacter la Banque',
      'activity': 'Activité',
      'no_records_found': 'Pas d\'archives trouves',
      'clone': 'Dupliquer',
      'loading': 'Chargement',
      'industry': 'Champ',
      'size': 'Taille',
      'payment_terms': 'Conditions de paiement',
      'payment_date': 'Date du paiement',
      'payment_status': 'État du paiement',
      'payment_status_1': 'En attente',
      'payment_status_2': 'Annulé',
      'payment_status_3': 'Échoué',
      'payment_status_4': 'Complété',
      'payment_status_5': 'Partiellement remboursé',
      'payment_status_6': 'Remboursement',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portail client',
      'show_tasks': 'Afficher des taches',
      'email_reminders': 'Messages de rappel par courriel',
      'enabled': 'Activé',
      'recipients': 'Destinataires',
      'initial_email': 'Courriel initial',
      'first_reminder': 'Premier rappel',
      'second_reminder': 'Second rappel',
      'third_reminder': 'Troisième rappel',
      'reminder1': 'Premier Message de Rappel',
      'reminder2': 'Deuxieme Message de Rappel',
      'reminder3': 'Troisieme Message de Rappel',
      'template': 'Modèle',
      'send': 'Envoyer',
      'subject': 'Sujet',
      'body': 'Corps',
      'send_email': 'Envoyer courriel',
      'email_receipt': 'Envoyer le reçu par courriel au client',
      'auto_billing': 'Debit Automatique',
      'button': 'Bouton',
      'preview': 'Prévisualisation',
      'customize': 'Personnaliser',
      'history': 'Historique',
      'payment': 'Paiement',
      'payments': 'Paiements',
      'refunded': 'Remboursé',
      'payment_type': 'Type de paiement',
      'transaction_reference': 'Référence transaction',
      'enter_payment': 'Saisissez un paiement',
      'new_payment': 'Entrer un paiement',
      'created_payment': 'Paiement créé avec succès',
      'updated_payment': 'Paiement mis à jour avec succès',
      'archived_payment': 'Paiement archivé avec succès',
      'deleted_payment': 'Paiement supprimé avec succès',
      'restored_payment': 'Paiement restauré avec succès',
      'archived_payments': ':count paiement archivés avec succès',
      'deleted_payments': ':count paiements supprimés avec succès',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Devis',
      'quotes': 'Devis',
      'new_quote': 'Nouveau devis',
      'created_quote': 'Devis créé avec succès',
      'updated_quote': 'Devis mis à jour avec succès',
      'archived_quote': 'Devis archivé avec succès',
      'deleted_quote': 'Devis supprimé avec succès',
      'restored_quote': 'Devis restauré avec succès',
      'archived_quotes': ':count devis archivés avec succès',
      'deleted_quotes': ':count devis supprimés avec succès',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Dépense',
      'expenses': 'Dépenses',
      'vendor': 'Fournisseur',
      'vendors': 'Fournisseurs',
      'task': 'Tâche',
      'tasks': 'Tâches',
      'project': 'Projet',
      'projects': 'Projets',
      'activity_1': ':user a créé le client :client',
      'activity_2': ':user a archivé le client :client',
      'activity_3': ':user a supprimé le client :client',
      'activity_4': ':user a créé la facture :invoice',
      'activity_5': ':user a mis à jour la facture :invoice',
      'activity_6': ':user a mailé la facture :invoice pour :client à :contact',
      'activity_7': ':contact a vu la facture :invoice pour :client',
      'activity_8': ':user a archivé la facture :invoice',
      'activity_9': ':user a supprimé la facture :invoice',
      'activity_10':
          ':contact a saisi un paiement :payment concernant :invoice pour :client',
      'activity_11': ':user a mis à jour le moyen de paiement :payment',
      'activity_12': ':user a archivé le moyen de paiement :payment',
      'activity_13': ':user a supprimé le moyen de paiement :payment',
      'activity_14': ':user a entré le crédit :credit',
      'activity_15': ':user a mis à jour le crédit :credit',
      'activity_16': ':user a archivé le crédit :credit',
      'activity_17': ':user a supprimé le crédit :credit',
      'activity_18': ':user a créé le devis :quote',
      'activity_19': ':user a mis à jour le devis :quote',
      'activity_20': ':user a mailé un devis :quote pour :client à :contact',
      'activity_21': ':contact a lu le devis :quote',
      'activity_22': ':user a archivé le devis :quote',
      'activity_23': ':user a supprimé le devis :quote',
      'activity_24': ':user a restauré le devis :quote',
      'activity_25': ':user a restauré la facture :invoice',
      'activity_26': ':user a restauré le client :client',
      'activity_27': ':user a restauré le paiement :payment',
      'activity_28': ':user a restauré le crédit :credit',
      'activity_29': ':contact a approuvé le devis :quote pour :client',
      'activity_30': ':user a créé le fournisseur :vendor',
      'activity_31': ':user a archivé le fournisseur :vendor',
      'activity_32': ':user a supprimé le fournisseur :vendor',
      'activity_33': ':user a restauré le fournisseur :vendor',
      'activity_34': ':user a créé la dépense :expense',
      'activity_35': ':user a archivé la dépense :expense',
      'activity_36': ':user a supprimé la dépense :expense',
      'activity_37': ':user a restauré la dépense :expense',
      'activity_39': ':user a annulé un paiement de :payment_amount (:payment)',
      'activity_40':
          ':user a remboursé :adjustment d\'un paiement de :payment_amount (:payment)',
      'activity_41': 'Le paiement de :payment_amount a échoué (:payment)',
      'activity_42': ':user a créé la tâche :task',
      'activity_43': ':user a mis à jour la tâche :task',
      'activity_44': ':user a archivé la tâche :task',
      'activity_45': ':user a supprimé la tâche :task',
      'activity_46': ':user a restauré la tâche :task',
      'activity_47': ':user a mis à jour la dépense :expense',
      'activity_48': ':user a mis à jour le ticket :ticket',
      'activity_49': ':user a fermé le ticket :ticket',
      'activity_50': ':user a fusionner le ticket :ticket',
      'activity_51': ':user a divisé le :ticket',
      'activity_52': ':contact a ouvert le ticket :ticket',
      'activity_53': ':contact a ré-ouvert le ticket :ticket',
      'activity_54': ':user a ré-ouvert le ticket :ticket',
      'activity_55': ':contact a répondu au ticket :ticket',
      'activity_56': ':user a visualisé le ticket :ticket',
      'activity_57': 'La facture :invoice n\'a pu être envoyée',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Mot de passe à usage unique',
      'emailed_quote': 'Devis envoyé par courriel avec succès',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Le devis sélectionné a été envoyé avec succès',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expiré',
      'all': 'Tous',
      'select': 'Sélectionner',
      'long_press_multiselect': 'Multisélection par pression longue',
      'custom_value1': 'Valeur Personnalisée 1',
      'custom_value2': 'Valeur Personnalisée 2',
      'custom_value3': 'Valeur personnalisée 3',
      'custom_value4': 'Valeur personnalisée 4',
      'email_style_custom': 'Style de courriel personnalisé',
      'custom_message_dashboard': 'Message personnalisé du tableau de bord',
      'custom_message_unpaid_invoice':
          'Message personnalisé pour une facture impayée',
      'custom_message_paid_invoice':
          'Message personnalisé pour un paiement de facture',
      'custom_message_unapproved_quote':
          'Message personnalisé pour un devis refusé',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Traductions',
      'task_number_pattern': 'Modèle de numéro de tâche',
      'task_number_counter': 'Modèle de compteur de tâche',
      'expense_number_pattern': 'Modèle de numéro de dépense',
      'expense_number_counter': 'Modèle de compteur de dépense',
      'vendor_number_pattern': 'Modèle de numéro de fournisseur',
      'vendor_number_counter': 'Modèle de compteur de fournisseur',
      'ticket_number_pattern': 'Modèle de numéro de ticket',
      'ticket_number_counter': 'Modèle de compteur de ticket',
      'payment_number_pattern': 'Modèle de numéro de paiement',
      'payment_number_counter': 'Modèle de compteur de paiement',
      'invoice_number_pattern': 'Modèle de numéro de facture',
      'invoice_number_counter': 'Compteur du numéro de facture',
      'quote_number_pattern': 'Modèle de numéro de devis',
      'quote_number_counter': 'Compteur du numéro de devis',
      'client_number_pattern': 'Modèle de numéro de crédit',
      'client_number_counter': 'Modèle de compteur de crédit',
      'credit_number_pattern': 'Modèle de numéro de crédit',
      'credit_number_counter': 'Modèle de compteur de crédit',
      'reset_counter_date': 'Remise à zéro du compteur de date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Nom par défaut de la taxe 1',
      'default_tax_rate_1': 'Taux par défaut de la taxe 1',
      'default_tax_name_2': 'Nom par défaut de la taxe 2',
      'default_tax_rate_2': 'Taux par défaut de la taxe 2',
      'default_tax_name_3': 'Nom par défaut de la taxe 3',
      'default_tax_rate_3': 'Taux par défaut de la taxe 3',
      'email_subject_invoice': 'Sujet du courriel de la facture',
      'email_subject_quote': 'Sujet du courriel du devis',
      'email_subject_payment': 'Sujet du courriel du paiement',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Ville du client',
      'client_state': 'Région du client',
      'client_country': 'Pays du client',
      'client_is_active': 'Le client est actif',
      'client_balance': 'Solde du client',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Montant de la facture',
      'invoice_due_date': 'Date limite',
      'tax_rate1': 'Taux de taxe 1',
      'tax_rate2': 'Taux de taxe 2',
      'tax_rate3': 'Taux de taxe 3',
      'auto_bill': 'Facturation automatique',
      'archived_at': 'Archivé le',
      'has_expenses': 'Dépenses en cours',
      'custom_taxes1': 'Autres taxes 1',
      'custom_taxes2': 'Autres taxes 2',
      'custom_taxes3': 'Autres taxes 3',
      'custom_taxes4': 'Autres taxes 4',
      'custom_surcharge1': 'Autre frais 1',
      'custom_surcharge2': 'Autre frais 2',
      'custom_surcharge3': 'Autre frais 3',
      'custom_surcharge4': 'Autre frais 4',
      'is_deleted': 'Supprimé',
      'vendor_city': 'Ville du fournisseur',
      'vendor_state': 'Région du fournisseur',
      'vendor_country': 'Pays du fournisseur',
      'is_approved': 'Is Approved',
      'tax_name': 'Nom de la taxe',
      'tax_amount': 'Montant de la taxe',
      'tax_paid': 'Taxe payée',
      'payment_amount': 'Montant du paiement',
      'age': 'Ancienneté',
      'is_running': 'Is Running',
      'time_log': 'Journal de temps',
      'bank_id': 'Banque',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Catégorie de dépense',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'fr_CA': {
      'import_data': 'Importer les données',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user a créé une facture récurrente :recurring_invoice',
      'activity_101':
          ':user a mis à jour une facture récurrente :recurring_invoice',
      'activity_102':
          ':user a archivé une facture récurrente :recurring_invoice',
      'activity_103':
          ':user a supprimé une facture récurrente :recurring_invoice',
      'activity_104':
          ':user a restauré une facture récurrente :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Dernière mise à jour',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Prénom du contact',
      'contact_last_name': 'Nom du contact',
      'order': 'Order',
      'unassigned': 'Non assigné',
      'partial_value': 'Doit être plus grand que zéro et moins que le total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Autoriser',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Définissez le sous-domaine ou affichez la facture sur votre site web.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Code promo',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sécurité',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Abonnement',
      'subscriptions': 'Abonnements',
      'new_subscription': 'Nouvel abonnement',
      'edit_subscription': 'Éditer l\'abonnement',
      'created_subscription': 'Abonnement créé avec succès',
      'updated_subscription': 'Abonnement mis à jour avec succès',
      'archived_subscription': 'Abonnement archivé avec succès',
      'deleted_subscription': 'L\'abonnement a été supprimé avec succès',
      'removed_subscription': 'L\'abonnement a été retiré avec succès',
      'restored_subscription': 'L\'abonnement a été restauré avec succès',
      'search_subscription': 'Recherche de 1 abonnement',
      'search_subscriptions': 'Recherche :count abonnements',
      'subdomain_is_not_available': 'Le sous-domaine n\'est pas disponible',
      'connect_gmail': 'Connectez Gmail',
      'disconnect_gmail': 'Déconnecter Gmail',
      'connected_gmail': 'Gmail a été connecté avec succès',
      'disconnected_gmail': 'Gmail a été déconnecté avec succès',
      'update_fail_help':
          'Les modifications apportées au code de base peuvent bloquer la mise à jour, vous pouvez exécuter cette commande pour annuler les modifications:',
      'client_id_number': 'Numéro d\'identification du client',
      'count_minutes': ':count minutes',
      'password_timeout': 'Délai d\'expiration du mot de passe',
      'shared_invoice_credit_counter':
          'Compteur partagé pour les factures et les crédits',
      'use_last_email': 'Utiliser le dernier e-mail',
      'activate_company': 'Activer la société',
      'activate_company_help':
          'Activez les courriels, les factures récurrentes et les notifications',
      'an_error_occurred_try_again':
          'Une erreur s\'est produite, veuillez réessayer',
      'please_first_set_a_password':
          'Veuillez d\'abord définir un mot de passe',
      'changing_phone_disables_two_factor':
          'Attention: modifier votre numéro de téléphone désactivera l\'authentification à deux facteurs (A2F)',
      'help_translate': 'Aide à la traduction',
      'please_select_a_country': 'Veuillez sélectionner un pays',
      'resend_invite': 'Renvoyer l\'invitation',
      'disabled_two_factor':
          'L\'authentification à deux facteurs (A2F) a été désactivée avec succès',
      'connected_google': 'Le compte a été connecté avec succès',
      'disconnected_google': 'Le comte a été déconnecté avec succès',
      'delivered': 'Livré',
      'bounced': 'Rejetés',
      'spam': 'Pourriel',
      'view_docs': 'Afficher la documentation',
      'enter_phone_to_enable_two_factor':
          'Veuillez fournir un numéro de téléphone mobile pour activer l\'authentification à deux facteurs',
      'send_sms': 'Envoyer un SMS',
      'sms_code': 'Code SMS',
      'two_factor_setup_help':
          'Scannez le code barre avec une :link app compatible.',
      'enabled_two_factor':
          'Vous avez activé authentification à deux facteurs avec succès.',
      'connect_google': 'Connectez Google',
      'disconnect_google': 'Déconnecter Google',
      'enable_two_factor': 'Authentification à deux facteurs',
      'disable_two_factor': 'Désactiver l\'authentification à deux facteurs',
      'require_password_with_social_login':
          'Requiert un mot de passe avec une connexion de réseau social',
      'stay_logged_in': 'Restez connecté',
      'session_about_to_expire':
          'Avertissement: Votre session va expirer bientôt',
      'count_hours': ':count heures',
      'count_day': '1 jour',
      'count_days': ':count jours',
      'web_session_timeout': 'Expiration de la session web',
      'security_settings': 'Paramètres de sécurité',
      'resend_email': 'Renvoyer le courriel',
      'confirm_your_email_address': 'Veuillez confirmer votre adresse courriel',
      'refunded_payment': 'Paiement remboursé',
      'partially_unapplied': 'Partiellement non-appliquée',
      'select_a_gmail_user':
          'Veuillez sélectionner un utilisateur authentifié avec Gmail',
      'list_long_press': 'Longue pression pour liste',
      'show_actions': 'Afficher les actions',
      'start_multiselect': 'Démarrer la multisélection',
      'email_sent_to_confirm_email':
          'Un courriel a été envoyé pour confirmer l\'adresse courriel',
      'counter_pattern_error':
          'Pour utiliser :client_counter veuillez ajouter soit :client_number ou :client_id_number pour éviter les conflits',
      'this_quarter': 'Ce trimestre',
      'last_quarter': 'Dernier trimestre',
      'to_update_run': 'Pour mettre à jour l\'exécution',
      'convert_to_invoice': 'Convertir en facture',
      'registration_url': 'URL d\'enregistrement',
      'invoice_project': 'Facturer le projet',
      'invoice_task': 'Facturer la tâche',
      'invoice_expense': 'Facture de dépense',
      'search_payment_term': 'Rechercher 1 délai de paiement',
      'search_payment_terms': 'Rechercher :count délais de paiement',
      'save_and_preview': 'Enregistrer et prévisualiser',
      'save_and_email': 'Enregistrer et envoyer par courriel',
      'supported_events': 'Événements pris en charge',
      'converted_amount': 'Montant converti',
      'converted_balance': 'Solde converti',
      'converted_paid_to_date': 'Payé à ce jour converti',
      'converted_credit_balance': 'Solde de crédit converti',
      'converted_total': 'Total converti',
      'is_sent': 'Est Envoyé',
      'default_documents': 'Documents par défaut',
      'document_upload': 'Téléversement de document',
      'document_upload_help':
          'Autoriser les clients à téléverser des documents',
      'expense_total': 'Total des dépenses',
      'enter_taxes': 'Saisir les taxes',
      'by_rate': 'Par taux',
      'by_amount': 'Par montant',
      'enter_amount': 'Entrer le montant',
      'before_taxes': 'Avant taxes',
      'after_taxes': 'Après taxes',
      'color': 'Couleur',
      'show': 'Voir',
      'hide': 'Cacher',
      'empty_columns': 'Colonnes vides',
      'debug_mode_is_enabled': 'Mode debug activé',
      'debug_mode_is_enabled_help':
          'Avertissement: Pour usage local seulement. Fuites de données possible. En savoir plus.',
      'running_tasks': 'Tâches en cours',
      'recent_tasks': 'Tâches récentes',
      'recent_expenses': 'Dépenses récentes',
      'upcoming_expenses': 'Dépenses à venir',
      'update_app': 'Mettre à jour l\'App',
      'started_import': 'L\'importation a démarré avec succès',
      'duplicate_column_mapping': 'Dupliquer le mappage de colonnes',
      'uses_inclusive_taxes': 'Utiliser taxes incluses',
      'is_amount_discount': 'Est Montant rabais',
      'column': 'Colonne',
      'sample': 'Exemple',
      'map_to': 'Mapper vers',
      'import': 'Importer',
      'first_row_as_column_names':
          'Utiliser première rangée comme noms de colonnes',
      'select_file': 'Veuillez sélectionner un fichier',
      'no_file_selected': 'Aucun fichier sélectionné',
      'csv_file': 'Fichier CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Comptabilité',
      'required_files_missing': 'Veuillez fournir tous les CSV.',
      'import_type': 'Type d\'importation',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Voir les licences',
      'webhook_url': 'URL Webhook',
      'fullscreen_editor': 'Éditeur plein écran',
      'sidebar_editor': 'Éditeur de barre latérale',
      'please_type_to_confirm': 'Veuillez saisir \":value\" pour confirmer',
      'purge': 'Purger',
      'service': 'Service',
      'clone_to': 'Cloner vers',
      'clone_to_other': 'Cloner vers Autre',
      'labels': 'Étiquettes',
      'add_custom': 'Ajout personnalisé',
      'payment_tax': 'Paiement de taxe',
      'unpaid': 'Impayé',
      'white_label': 'Sans marque',
      'delivery_note': 'Note de livraison',
      'sent_invoices_are_locked': 'Les factures envoyées sont verrouillées',
      'paid_invoices_are_locked': 'Les factures payées sont verrouillées',
      'source_code': 'Code source',
      'app_platforms': 'Plateformes d\'app',
      'invoice_late': 'facture en retard',
      'quote_expired': 'Soumission expirée',
      'partial_due': 'Montant partiel du',
      'invoice_total': 'Montant total',
      'quote_total': 'Total de la soumission',
      'credit_total': 'Total du crédit',
      'recurring_invoice_total': 'Total de facture',
      'actions': 'Actions',
      'expense_number': 'Numéro de dépense',
      'task_number': 'Numéro de tâche',
      'project_number': 'Numéro de projet',
      'project_name': 'Nom du projet',
      'warning': 'Avertissement',
      'view_settings': 'Voir les paramètres',
      'company_disabled_warning':
          'Avertissement: Cette entreprise n\'a pas encore été activée',
      'late_invoice': 'Facture en retard',
      'expired_quote': 'Soumission expirée',
      'remind_invoice': 'Rappeler la facture',
      'cvv': 'CVV',
      'client_name': 'Nom du client',
      'client_phone': 'Téléphone du client',
      'required_fields': 'Champs requis',
      'calculated_rate': 'Taux calculé',
      'default_task_rate': 'Taux de tâche par défaut',
      'clear_cache': 'Vider le cache',
      'sort_order': 'Ordre de tri',
      'task_status': 'État',
      'task_statuses': 'États de tâche',
      'new_task_status': 'Nouvel état de tâche',
      'edit_task_status': 'Édition de l\'état de tâche',
      'created_task_status': 'L\'état de tâche a été créé avec succès',
      'updated_task_status':
          'Le statut de la tâche a été mis à jour avec succès',
      'archived_task_status': 'L\'état de tâche a été archivé avec succès',
      'deleted_task_status': 'L\'état de tâche a été supprimé avec succès',
      'removed_task_status': 'L\'état de tâche a été retiré avec succès',
      'restored_task_status': 'L\'état de tâche a été restauré avec succès',
      'archived_task_statuses':
          'Les :value états de tâche ont été archivés succès',
      'deleted_task_statuses':
          'Les :value états de tâche ont été supprimés avec succès',
      'restored_task_statuses':
          'Les :value états de tâche ont été restaurés avec succès',
      'search_task_status': 'Recherche 1 état de tâche',
      'search_task_statuses': 'Recherche :count états de tâche',
      'show_tasks_table': 'Afficher le tableau des tâches',
      'show_tasks_table_help':
          'Toujours afficher la section des tâches lors de la création de factures',
      'invoice_task_timelog': 'Facturer le journal du temps des tâches',
      'invoice_task_timelog_help':
          'Ajouter les détails de temps aux lignes d\'articles des factures',
      'invoice_task_datelog': 'Facturer le journal des dates des tâches',
      'invoice_task_datelog_help':
          'Ajouter les détails de date aux lignes d\'articles des factures',
      'auto_start_tasks_help': 'Démarrer les tâches avant de sauvegarder',
      'configure_statuses': 'Configurer les états',
      'task_settings': 'Paramètres des tâches',
      'configure_categories': 'Configurer les catégories',
      'expense_categories': 'Catégories de dépense',
      'new_expense_category': 'Nouvelle catégorie de dépense',
      'edit_expense_category': 'Éditer la catégorie Dépense',
      'created_expense_category':
          'La catégorie de dépense a été créé avec succès',
      'updated_expense_category':
          'La catégorie de dépense a été mise à jour avec succès',
      'archived_expense_category':
          'La catégorie de dépense a été archivée avec succès',
      'deleted_expense_category': 'La catégorie a été supprimée avec succès',
      'removed_expense_category':
          'La catégorie de dépense a été retirée avec succès',
      'restored_expense_category':
          'La catégorie de dépense a été rétablie avec succès',
      'archived_expense_categories':
          ':count catégories de dépense ont été archivées avec succès',
      'deleted_expense_categories':
          'Les :value catégories de dépense ont été supprimées avec succès',
      'restored_expense_categories':
          'Les :value catégories de dépense ont été restaurées avec succès',
      'search_expense_category': 'Recherche 1 catégorie de dépense',
      'search_expense_categories': 'Recherche :count catégorie de dépense',
      'use_available_credits': 'Utiliser les crédits disponibles',
      'show_option': 'Afficher les options',
      'negative_payment_error':
          'Le montant du crédit ne peut pas excéder le montant du paiement',
      'view_changes': 'Visualiser les changements',
      'force_update': 'Forcer la mise à jour',
      'force_update_help':
          'Vous êtes sur la dernière version, mais il peut y avoir encore quelques mises à jour en cours',
      'mark_paid_help': 'Suivez les dépenses qui ont été payées',
      'should_be_invoiced': 'Devrait être facturée',
      'should_be_invoiced_help': 'Activer la facturation de la dépense',
      'add_documents_to_invoice_help': 'Rend visibles les documents',
      'convert_currency_help': 'Définir un taux d\'échange',
      'expense_settings': 'Paramètres des dépenses',
      'clone_to_recurring': 'Cloner en récurrence',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'Champs utilisateur',
      'variables': 'Variables',
      'show_password': 'Afficher le mot de passe',
      'hide_password': 'Masquer le mot de passe',
      'copy_error': 'Erreur de copie',
      'capture_card': 'Carte saisie',
      'auto_bill_enabled': 'Autofacturation activée',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Ligne Taxes',
      'total_fields': 'Total Champs',
      'stopped_recurring_invoice':
          'La facture récurrente a été arrêtée avec succès',
      'started_recurring_invoice':
          'La facture récurrente a été démarrée avec succès',
      'resumed_recurring_invoice':
          'La facture récurrente a été reprise avec succès',
      'gateway_refund': 'Remboursement de passerelle',
      'gateway_refund_help':
          'Procéder au remboursement avec la passerelle de paiement',
      'due_date_days': 'Date d\'échéance',
      'paused': 'En pause',
      'mark_active': 'Cocher actif',
      'day_count': 'Jour :count',
      'first_day_of_the_month': 'Premier jour du mois',
      'last_day_of_the_month': 'Dernier jour du mois',
      'use_payment_terms': 'Utiliser les délais de paiement',
      'endless': 'Sans fin',
      'next_send_date': 'Prochaine date d\'envoi',
      'remaining_cycles': 'Cycles restants',
      'recurring_invoice': 'Facture récurrente',
      'recurring_invoices': 'Factures récurrentes',
      'new_recurring_invoice': 'Nouvelle facture récurrente',
      'edit_recurring_invoice': 'Éditer la facture récurrente',
      'created_recurring_invoice':
          'La facture récurrente a été créée avec succès',
      'updated_recurring_invoice':
          'La facture récurrente a été mise à jour avec succès',
      'archived_recurring_invoice':
          'La facture récurrente a été archivée avec succès',
      'deleted_recurring_invoice':
          'La facture récurrente a été supprimée avec succès',
      'removed_recurring_invoice':
          'La facture récurrente a été retirée avec succès',
      'restored_recurring_invoice':
          'La facture récurrente a été restaurée avec succès',
      'archived_recurring_invoices':
          'Les :value factures récurrentes ont été archivées avec succès',
      'deleted_recurring_invoices':
          'Les :value factures récurrentes ont été supprimées avec succès',
      'restored_recurring_invoices':
          'Les :value factures récurrentes ont été restaurées avec succès',
      'search_recurring_invoice': 'Recherche 1 facture récurrente',
      'search_recurring_invoices': 'Recherche :count factures récurrentes',
      'send_date': 'Date d\'envoi',
      'auto_bill_on': 'Autofacturer le',
      'minimum_under_payment_amount': 'Montant minimum de sous-paiement',
      'profit': 'Profit',
      'line_item': 'Ligne d\'article',
      'allow_over_payment': 'Accepter Sur-paiement',
      'allow_over_payment_help':
          'Accepter paiement supplémentaire pour pourboire',
      'allow_under_payment': 'Accepter Sous-paiement',
      'allow_under_payment_help':
          'Accepter paiement au minimum le montant partiel/dépôt',
      'test_mode': 'Mode test',
      'opened': 'Ouverts',
      'payment_reconciliation_failure': 'Conciliation non réussie',
      'payment_reconciliation_success': 'Conciliation réussie',
      'gateway_success': 'Passerelle réussie',
      'gateway_failure': 'Échec de passerelle',
      'gateway_error': 'Erreur de passerelle',
      'email_send': 'Envoi de courriel',
      'email_retry_queue': 'File d\'envoi de courriel',
      'failure': 'Échec',
      'quota_exceeded': 'Quota dépassé',
      'upstream_failure': 'Échec en amont',
      'system_logs': 'Journaux système',
      'view_portal': 'Voir le portail',
      'copy_link': 'Copier le lien',
      'token_billing': 'Sauvegarder les informations de carte de crédit',
      'welcome_to_invoice_ninja': 'Bienvenue dans Invoice Ninja',
      'always': 'Toujours',
      'optin': 'Adhésion',
      'optout': 'Désadhésion',
      'label': 'Libellé',
      'client_number': 'Numéro de client',
      'auto_convert': 'Conversion automatique',
      'company_name': 'Nom de l\'entreprise',
      'reminder1_sent': 'Rappel 1 envoyé',
      'reminder2_sent': 'Rappel 2 envoyé',
      'reminder3_sent': 'Rappel 3 envoyé',
      'reminder_last_sent': 'Dernier envoi de rappel',
      'pdf_page_info': 'Page :current de :total',
      'emailed_invoices':
          'Les factures ont été envoyées par courriel avec succès',
      'emailed_quotes':
          'Les soumissions ont été envoyées par courriel avec succès',
      'emailed_credits': 'Les crédits ont été envoyés par courriel avec succès',
      'gateway': 'Passerelle',
      'view_in_stripe': 'Voir dans Stripe',
      'rows_per_page': 'Rangées par page',
      'hours': 'Heures',
      'statement': 'Relevé',
      'taxes': 'Taxes',
      'surcharge': 'surcharge',
      'apply_payment': 'Appliquer le paiement',
      'apply': 'Appliquer',
      'unapplied': 'Non appliqué',
      'select_label': 'Sélectionnez le libellé',
      'custom_labels': 'Libellés personnalisés',
      'record_type': 'Type d\'enregistrement',
      'record_name': 'Non d\'enregistrement',
      'file_type': 'Type de fichier',
      'height': 'Hauteur',
      'width': 'Largeur',
      'to': 'à',
      'health_check': 'État de santé',
      'payment_type_id': 'Type de paiement',
      'last_login_at': 'Dernière connexion à',
      'company_key': 'Clé d\'entreprise',
      'storefront': 'Vitrine',
      'storefront_help':
          'Activer les applications externes à créer des factures',
      'client_created': 'Client créé',
      'online_payment_email': 'Courriel de paiement en ligne',
      'manual_payment_email': 'Courriel de paiement manuel',
      'completed': 'Complété',
      'gross': 'Brut',
      'net_amount': 'Montant net',
      'net_balance': 'Solde net',
      'client_settings': 'Paramètres clients',
      'selected_invoices': 'Factures sélectionnées',
      'selected_payments': 'Paiements sélectionnés',
      'selected_quotes': 'Soumissions sélectionnées',
      'selected_tasks': 'Tâches sélectionnées',
      'selected_expenses': 'Dépenses sélectionnées',
      'upcoming_invoices': 'Factures à venir',
      'past_due_invoices': 'Factures impayées',
      'recent_payments': 'Paiements reçus',
      'upcoming_quotes': 'Soumissions à venir',
      'expired_quotes': 'Soumissions expirées',
      'create_client': 'Créer un client',
      'create_invoice': 'Créer une facture',
      'create_quote': 'Créer une soumission',
      'create_payment': 'Créer un paiement',
      'create_vendor': 'Créer un fournisseur',
      'update_quote': 'Mettre à jour la soumission',
      'delete_quote': 'Supprimer la soumission',
      'update_invoice': 'Mettre à jour la facture',
      'delete_invoice': 'Supprimer la facture',
      'update_client': 'Mettre à jour le client',
      'delete_client': 'Supprimer ce client',
      'delete_payment': 'Supprimer ce paiement',
      'update_vendor': 'Mettre à jour le fournisseur',
      'delete_vendor': 'Supprimer le fournisseur',
      'create_expense': 'Créer une dépense',
      'update_expense': 'Mettre à jour la dépense',
      'delete_expense': 'Supprimer la dépense',
      'create_task': 'Créer une Tâche',
      'update_task': 'Mettre à jour la tâche',
      'delete_task': 'Supprimer la Tâche',
      'approve_quote': 'Approuver la tâche',
      'off': 'Fermé',
      'when_paid': 'Lors du paiement',
      'expires_on': 'Expiration le',
      'free': 'Gratuit',
      'plan': 'Plan',
      'show_sidebar': 'Afficher la barre latérale',
      'hide_sidebar': 'Masquer la barre latérale',
      'event_type': 'Type d\'événement',
      'target_url': 'Cible',
      'copy': 'Copier',
      'must_be_online':
          'Veuillez redémarrer l\'application lorsque vous serez connecté à internet',
      'crons_not_enabled': 'Les crons doivent être activés',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Recherche de :count Webhooks',
      'search_webhook': 'Recherche de 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Nouveau Webhook',
      'edit_webhook': 'Éditer le Webhook',
      'created_webhook': 'Le webhook a été créé avec succès',
      'updated_webhook': 'Le webhook a été mis à jour avec succès',
      'archived_webhook': 'Le webhook a été archivé avec succès',
      'deleted_webhook': 'Le webhook a été supprimé avec succès',
      'removed_webhook': 'Le webhook a été retiré avec succès',
      'restored_webhook': 'Le webhook a été restauré avec succès',
      'archived_webhooks': 'Les :value webhooks ont été archivés avec succès',
      'deleted_webhooks': 'Les :value webhooks ont été supprimés avec succès',
      'removed_webhooks': 'Les :value webhooks ont été retirés avec succès',
      'restored_webhooks': 'Les :value webhooks ont été restaurés avec succès',
      'api_tokens': 'Jetons API',
      'api_docs': 'Docs API',
      'search_tokens': 'Recherche de :count jetons',
      'search_token': 'Recherche de 1 jeton',
      'token': 'Jeton',
      'tokens': 'Jetons',
      'new_token': 'Nouveau jeton',
      'edit_token': 'Éditer le jeton',
      'created_token': 'Le jeton a été créé avec succès',
      'updated_token': 'Le jeton a été mis à jour avec succès',
      'archived_token': 'Le jeton a été archivé avec succès',
      'deleted_token': 'Le jeton a été supprimé avec succès',
      'removed_token': 'Le jeton a été retiré avec succès',
      'restored_token': 'Le jeton a été restauré avec succès',
      'archived_tokens': 'Les :value jetons ont été archivés avec succès',
      'deleted_tokens': 'Les :value jetons ont été supprimés avec succès',
      'restored_tokens': 'Les :value jetons ont été restaurés avec succès',
      'client_registration': 'Enregistrement d\'un client',
      'client_registration_help':
          'Autoriser le client à s\'inscrire sur le portail',
      'customize_and_preview': 'Personnaliser et prévisualiser',
      'email_invoice': 'Envoyer la facture par courriel',
      'email_quote': 'Envoyer la soumission par courriel',
      'email_credit': 'Crédit par courriel',
      'email_payment': 'Courriel de paiement',
      'client_email_not_set': 'Le client n\'a pas d\'adresse courriel définie',
      'ledger': 'Grand livre',
      'view_pdf': 'Voir PDF',
      'all_records': 'Tous les enregistrements',
      'owned_by_user': 'Propriété de l\'utilisateur',
      'credit_remaining': 'Crédit restant',
      'contact_name': 'Nom du contact',
      'use_default': 'Utiliser le défaut',
      'reminder_endless': 'Rappels infinis',
      'number_of_days': 'Nombre de jours',
      'configure_payment_terms': 'Configuration des délais de paiements',
      'payment_term': 'Délai de paiement',
      'new_payment_term': 'Nouveau délai de paiement',
      'edit_payment_term': 'Modifier le délai de paiement',
      'created_payment_term': 'Le délai de paiement a été cré avec succès',
      'updated_payment_term':
          'Le délai de paiement a été mis à jour avec succès',
      'archived_payment_term': 'Le délai de paiement a été archivé avec succès',
      'deleted_payment_term': 'Le délai de paiement a été supprimé avec succès',
      'removed_payment_term': 'Le délai de paiement a été retiré avec succès',
      'restored_payment_term':
          'Le délai de paiement a été restauré avec succès',
      'archived_payment_terms':
          'Les :value délais de paiement ont été archivés avec succès',
      'deleted_payment_terms':
          'Les :value délais de paiement ont été supprimés avec succès',
      'restored_payment_terms':
          'Les :value délais de paiement ont été restaurés avec succès',
      'email_sign_in': 'Connexion par courriel',
      'change': 'Basculer',
      'change_to_mobile_layout': 'Basculer vers l\'affichage mobile',
      'change_to_desktop_layout': 'Basculer vers l\'affichage ordinateur',
      'send_from_gmail': 'Envoyer avec Gmail',
      'reversed': 'Inversé',
      'cancelled': 'Annulé',
      'credit_amount': 'Montant du crédit',
      'quote_amount': 'Montant de la soumission',
      'hosted': 'Hébergé',
      'selfhosted': 'Auto-hébergé',
      'exclusive': 'Exclusif',
      'inclusive': 'Inclusif',
      'hide_menu': 'Masquer le menu',
      'show_menu': 'Afficher le menu',
      'partially_refunded': 'Partiellement remboursé',
      'search_documents': 'Recherche de documents',
      'search_designs': 'Recherche de designs',
      'search_invoices': 'Recherche de factures',
      'search_clients': 'Recherche de clients',
      'search_products': 'Recherche de produits',
      'search_quotes': 'Recherche de soumissions',
      'search_credits': 'Recherche de crédits',
      'search_vendors': 'Recherche de fournisseurs',
      'search_users': 'Recherche d\'utilisateurs',
      'search_tax_rates': 'Recherche de taux de taxe',
      'search_tasks': 'Recherche de tâches',
      'search_settings': 'Recherche de paramètres',
      'search_projects': 'Recherche de projets',
      'search_expenses': 'Recherche de dépenses',
      'search_payments': 'Recherche de paiements',
      'search_groups': 'Recherche de groupes',
      'search_company': 'Recherche d\'entreprises',
      'search_document': 'Recherche de 1 document',
      'search_design': 'Recherche de 1 design',
      'search_invoice': 'Recherche de 1 facture',
      'search_client': 'Recherche de 1 client',
      'search_product': 'Recherche de 1 produit',
      'search_quote': 'Recherche de 1 soumission',
      'search_credit': 'Recherche de 1 crédit',
      'search_vendor': 'Recherche de 1 entreprise',
      'search_user': 'Recherche de 1 utilisateur',
      'search_tax_rate': 'Recherche de 1 taux de taxe',
      'search_task': 'Recherche de 1 tâche',
      'search_project': 'Recherche de 1 projet',
      'search_expense': 'Recherche de 1 dépense',
      'search_payment': 'Recherche de 1 paiement',
      'search_group': 'Recherche de 1 groupe',
      'refund_payment': 'Remboursement',
      'cancelled_invoice': 'La facture a été annulée avec succès',
      'cancelled_invoices': 'Les factures ont été annulées avec succès',
      'reversed_invoice': 'La facture a été inversée avec succès',
      'reversed_invoices': 'Les factures ont été inversées avec succès',
      'reverse': 'Inverse',
      'full_name': 'Nom complet',
      'city_state_postal': 'Ville/Prov/CP',
      'postal_city_state': 'Ville/Province/Code postal',
      'custom1': 'Personnalisation 1',
      'custom2': 'Personnalisation 2',
      'custom3': 'Troisième personnalisé',
      'custom4': 'Quatrième personnalisée',
      'optional': 'Optionnel',
      'license': 'Licence',
      'purge_data': 'Purger les données',
      'purge_successful':
          'Toutes les données de l\'entreprise ont été purgées avec succès',
      'purge_data_message':
          'Avertissement: Cette action est irréversible et va supprimer vos données de façon définitive.',
      'invoice_balance': 'Solde de facture',
      'age_group_0': '0 - 30 jours',
      'age_group_30': '30 - 60 jours',
      'age_group_60': '60 - 90 jours',
      'age_group_90': '90 - 120 jours',
      'age_group_120': '120+ jours',
      'refresh': 'Actualiser',
      'saved_design': 'Le modèle a été sauvegardé avec succès',
      'client_details': 'Informations du client',
      'company_address': 'Adresse de l\'entreprise',
      'invoice_details': 'Détails de facture',
      'quote_details': 'Informations de la soumission',
      'credit_details': 'Informations de crédit',
      'product_columns': 'Colonnes produit',
      'task_columns': 'Colonnes tâches',
      'add_field': 'Ajouter un champ',
      'all_events': 'Ajouter un événement',
      'permissions': 'Permissions',
      'none': 'Aucun',
      'owned': 'Propriétaire',
      'payment_success': 'Paiement réussi',
      'payment_failure': 'Le paiement a échoué',
      'invoice_sent': ':count facture envoyée',
      'quote_sent': 'Soumission envoyée',
      'credit_sent': 'Crédit envoyé',
      'invoice_viewed': 'Facture visualisée',
      'quote_viewed': 'Soumission visualisée',
      'credit_viewed': 'Crédit visualisé',
      'quote_approved': 'Soumission approuvée',
      'receive_all_notifications': 'Recevoir toutes les notifications',
      'purchase_license': 'Acheter une licence',
      'apply_license': 'Activer la licence',
      'cancel_account': 'Supprimer le compte',
      'cancel_account_message':
          'Avertissement: cette action est irréversible et supprimera votre compte de façon définitive.',
      'delete_company': 'Supprimer l\'entreprise',
      'delete_company_message':
          'Avertissement: Cette entreprise sera définitivement supprimée.',
      'enabled_modules': 'Modules activés',
      'converted_quote': 'La soumission a été convertie avec succès',
      'credit_design': 'Design de crédit',
      'includes': 'Inclue',
      'header': 'Entête',
      'load_design': 'Charger le design',
      'css_framework': 'Framework CSS',
      'custom_designs': 'Designs personnalisés',
      'designs': 'Designs',
      'new_design': 'Nouveau design',
      'edit_design': 'Éditer le design',
      'created_design': 'Le modèle a été créé avec succès',
      'updated_design': 'Le modèle a été mis à jour avec succès',
      'archived_design': 'Le modèle a été archivé avec succès',
      'deleted_design': 'Le modèle a été supprimé avec succès',
      'removed_design': 'Le modèle a été retiré avec succès',
      'restored_design': 'Le modèle a été restauré avec succès',
      'archived_designs': 'Les :value modèles ont été archivés avec succès',
      'deleted_designs': 'Les :value modèles ont été supprimés avec succès',
      'restored_designs': 'Les :value modèles ont été restaurés avec succès',
      'proposals': 'Propositions',
      'tickets': 'Billets',
      'recurring_quotes': 'Soumissions récurrentes',
      'recurring_tasks': 'Tâches récurrentes',
      'recurring_expenses': 'Dépenses récurrentes',
      'account_management': 'Gestion du compte',
      'credit_date': 'Date du crédit',
      'credit': 'Crédit',
      'credits': 'Crédits',
      'new_credit': 'Inscrire un crédit',
      'edit_credit': 'Éditer le crédit',
      'created_credit': 'Le crédit a été créé avec succès',
      'updated_credit': 'Le crédit a été mis à jour avec succès',
      'archived_credit': 'Le crédit a été archivé avec succès',
      'deleted_credit': 'Le crédit a été supprimé avec succès',
      'removed_credit': 'Le crédit a été retiré avec succès',
      'restored_credit': 'Le crédit a été restauré avec succès',
      'archived_credits': ':count crédits ont archivés avec succès',
      'deleted_credits': ':count crédits ont été supprimés avec succès',
      'restored_credits': 'Les :value crédits ont été restaurés avec succès',
      'current_version': 'Version courante',
      'latest_version': 'Dernière version',
      'update_now': 'Mettre à jour',
      'a_new_version_is_available':
          'Une nouvelle version de l\'application web est disponible',
      'update_available': 'Mise à jour disponible',
      'app_updated': 'La mise à jour a été complétée avec succès',
      'learn_more': 'En savoir plus',
      'integrations': 'Intégrations',
      'tracking_id': 'ID de suivi',
      'slack_webhook_url': 'URL du Webhook Slack',
      'credit_footer': 'Pied de page pour crédit',
      'credit_terms': 'Conditions d\'utilisation pour crédit',
      'new_company': 'Nouvelle entreprise',
      'added_company': 'L\'entreprise a été ajoutée avec succès',
      'company1': 'Entreprise personnalisée 1',
      'company2': 'Entreprise personnalisée 2',
      'company3': 'Entreprise personnalisée 3',
      'company4': 'Entreprise personnalisée 4',
      'product1': 'Produit personnalisé 1',
      'product2': 'Produit personnalisé 2',
      'product3': 'Produit personnalisé 3',
      'product4': 'Produit personnalisé 4',
      'client1': 'Client personnalisé 1',
      'client2': 'Client personnalisé 2',
      'client3': 'Client personnalisé 3',
      'client4': 'Client personnalisé 4',
      'contact1': 'Contact personnalisé 1',
      'contact2': 'Contact personnalisé 2',
      'contact3': 'Contact personnalisé 3',
      'contact4': 'Contact personnalisé 4',
      'task1': 'Tâche personnalisée 1',
      'task2': 'Tâche personnalisée 2',
      'task3': 'Tâche personnalisée 3',
      'task4': 'Tâche personnalisée 4',
      'project1': 'Projet personnalisé 1',
      'project2': 'Projet personnalisé 2',
      'project3': 'Projet personnalisé 3',
      'project4': 'Projet personnalisé 4',
      'expense1': 'Dépense personnalisée 1',
      'expense2': 'Dépense personnalisée 2',
      'expense3': 'Dépense personnalisée 3',
      'expense4': 'Dépense personnalisée 4',
      'vendor1': 'Fournisseur personnalisé 1',
      'vendor2': 'Fournisseur personnalisé 2',
      'vendor3': 'Fournisseur personnalisé 3',
      'vendor4': 'Fournisseur personnalisé 4',
      'invoice1': 'Facture personnalisée 1',
      'invoice2': 'Facture personnalisée 2',
      'invoice3': 'Facture personnalisée 3',
      'invoice4': 'Facture personnalisée 4',
      'payment1': 'Paiement personnalisé 1',
      'payment2': 'Facture personnalisée 2',
      'payment3': 'Facture personnalisée 3',
      'payment4': 'Facture personnalisée 4',
      'surcharge1': 'Surcharge personnalisée 1',
      'surcharge2': 'Surcharge personnalisée 2',
      'surcharge3': 'Surcharge personnalisée 3',
      'surcharge4': 'Surcharge personnalisée 4',
      'group1': 'Groupe personnalisé 1',
      'group2': 'Groupe personnalisé 2',
      'group3': 'Groupe personnalisé 3',
      'group4': 'Groupe personnalisé 4',
      'reset': 'Remise à zéro',
      'number': 'Numéro',
      'export': 'Exporter',
      'chart': 'Graphique',
      'count': 'Compteur',
      'totals': 'Totaux',
      'blank': 'Vide',
      'day': 'Jour',
      'month': 'Mois',
      'year': 'Année',
      'subgroup': 'Sous-groupe',
      'is_active': 'Actif',
      'group_by': 'Grouper par',
      'credit_balance': 'Solde du crédit',
      'contact_last_login': 'Dernière connexion du contact',
      'contact_full_name': 'Nom complet du contact',
      'contact_phone': 'Téléphone du contact',
      'contact_custom_value1': 'Valeur personnalisée du contact 1',
      'contact_custom_value2': 'Valeur personnalisée du contact 2',
      'contact_custom_value3': 'Valeur personnalisée du contact 3',
      'contact_custom_value4': 'Valeur personnalisée du contact 4',
      'shipping_address1': 'Rue de livraison',
      'shipping_address2': 'App. de livraison',
      'shipping_city': 'Ville de livraison',
      'shipping_state': 'Province de livraison',
      'shipping_postal_code': 'Code postal de livraison',
      'shipping_country': 'Pays de livraison',
      'billing_address1': 'Rue de facturation',
      'billing_address2': 'App. de facturation',
      'billing_city': 'Ville de facturation',
      'billing_state': 'Province de facturation',
      'billing_postal_code': 'Code postal de facturation',
      'billing_country': 'Pays de facturation',
      'client_id': 'ID du client',
      'assigned_to': 'Assigné à',
      'created_by': 'Créé par :name',
      'assigned_to_id': 'Assigné à ID',
      'created_by_id': 'Créé par ID',
      'add_column': 'Ajouter colonne',
      'edit_columns': 'Éditer colonne',
      'columns': 'Colonnes',
      'aging': 'Impayés',
      'profit_and_loss': 'Profit et perte',
      'reports': 'Rapports',
      'report': 'Rapport',
      'add_company': 'Ajouter une entreprise',
      'unpaid_invoice': 'Facture impayée',
      'paid_invoice': 'Facture payée',
      'unapproved_quote': 'Soumission non approuvée',
      'help': 'Aide',
      'refund': 'Rembousement',
      'refund_date': 'Date de remboursement',
      'filtered_by': 'Filtrée par',
      'contact_email': 'Courriel du contact',
      'multiselect': 'Sélection multiple',
      'entity_state': 'Statut',
      'verify_password': 'Vérifier le mot de passe',
      'applied': 'Publié',
      'include_recent_errors':
          'Inclut les erreurs récentes des journaux système',
      'your_message_has_been_received':
          'Nous avons reçu votre message et vous répondrons rapidement.',
      'message': 'Message',
      'from': 'De',
      'show_product_details': 'Afficher les détails du produit',
      'show_product_details_help':
          'Veuillez inclure la description et le coût dans la liste déroulante du produit',
      'pdf_min_requirements': 'Le moteur de rendu PDF nécessite :version',
      'adjust_fee_percent': 'Ajuster le pourcentage de frais',
      'adjust_fee_percent_help': 'Ajuster le frais de pourcentage au compte',
      'configure_settings': 'Configurer les paramètres',
      'support_forum': 'Forum de support',
      'about': 'À propos',
      'documentation': 'Documentation',
      'contact_us': 'Nous joindre',
      'subtotal': 'Sous-total',
      'line_total': 'Total',
      'item': 'Article',
      'credit_email': 'Courriel pour le crédit',
      'iframe_url': 'Site web',
      'domain_url': 'URL de domaine',
      'password_is_too_short': 'Le mot de passe est trop court',
      'password_is_too_easy':
          'Le mot de passe doit contenir une majuscule et un nombre',
      'client_portal_tasks': 'Tâches du portail client',
      'client_portal_dashboard': 'Tableau de bord du portail client',
      'please_enter_a_value': 'Veuillez saisir une valeur',
      'deleted_logo': 'Le logo a été supprimé avec succès',
      'yes': 'Oui',
      'no': 'Non',
      'generate_number': 'Générer un nombre',
      'when_saved': 'Lors de la sauvegarde',
      'when_sent': 'Lors de l\'envoi',
      'select_company': 'Sélectionnez une entreprise',
      'float': 'Flottant',
      'collapse': 'Réduire',
      'show_or_hide': 'Afficher/masquer',
      'menu_sidebar': 'Menu latéral',
      'history_sidebar': 'Historique latéral',
      'tablet': 'Tablette',
      'mobile': 'Mobile',
      'desktop': 'Fixe',
      'layout': 'Affichage',
      'view': 'Visualiser',
      'module': 'Module',
      'first_custom': 'Premier personnalisé',
      'second_custom': 'Second personnalisé',
      'third_custom': 'Troisième latéral',
      'show_cost': 'Afficher le coût',
      'show_product_cost': 'Afficher le montant du produit',
      'show_cost_help':
          'Afficher un champ de coût du produit pour suivre le profit',
      'show_product_quantity': 'Afficher la quantité de produit',
      'show_product_quantity_help':
          'Afficher un champ Quantité de produit. 1 par défaut.',
      'show_invoice_quantity': 'Afficher la quantité de facture',
      'show_invoice_quantity_help':
          'Afficher un champ Quantité d\'article par ligne. 1 par défaut.',
      'show_product_discount': 'Afficher le rabais de produit',
      'show_product_discount_help':
          'Afficher un champ rabais de ligne d\'article',
      'default_quantity': 'Quantité par défaut',
      'default_quantity_help':
          'Définit automatiquement la quantité d\'article par ligne à 1.',
      'one_tax_rate': 'Un taux de taxe',
      'two_tax_rates': 'Deux taux de taxe',
      'three_tax_rates': 'Trois taux de taxes',
      'default_tax_rate': 'Taux de taxe par défaut',
      'user': 'Utilisateur',
      'invoice_tax': 'Taxe de facture',
      'line_item_tax': 'Taxe d\'article par ligne',
      'inclusive_taxes': 'Taxes incluses',
      'invoice_tax_rates': 'Taux de taxe de facture',
      'item_tax_rates': 'Taux de taxe par article',
      'no_client_selected': 'Veuillez sélectionner un client',
      'configure_rates': 'Configuration des taux',
      'configure_gateways': 'Configurer les passerelles',
      'tax_settings': 'Paramètres des taxes',
      'tax_settings_rates': 'Taux de taxe',
      'accent_color': 'Couleur de mise en évidence',
      'switch': 'Changer',
      'comma_sparated_list': 'Liste séparée par virgule',
      'options': 'Options',
      'single_line_text': 'Ligne de texte simple',
      'multi_line_text': 'Multiligne de texte',
      'dropdown': 'Liste déroulante',
      'field_type': 'Type de champ',
      'recover_password_email_sent':
          'Un courriel a été envoyé pour la récupération du mot de passe',
      'submit': 'Envoyer',
      'recover_password': 'Récupérez votre mot de passe',
      'late_fees': 'Frais de retard',
      'credit_number': 'Numéro de crédit',
      'payment_number': 'Numéro de paiement',
      'late_fee_amount': 'Frais de retard',
      'late_fee_percent': 'Pourcentage de frais de retard',
      'schedule': 'Calendrier',
      'before_due_date': 'Avant l\'échéance',
      'after_due_date': 'Après l\'échéance',
      'after_invoice_date': 'Après la date de facturation',
      'days': 'Jours',
      'invoice_email': 'Courriel de facturation',
      'payment_email': 'Courriel de paiement',
      'partial_payment': 'Paiement partiel',
      'payment_partial': 'Paiement partiel',
      'partial_payment_email': 'Courriel du paiement partiel',
      'quote_email': 'Courriel de soumission',
      'endless_reminder': 'Rappel perpétuel',
      'filtered_by_user': 'Filtré par utilisateur',
      'administrator': 'Administrateur',
      'administrator_help':
          'Permet à un utilisateur de gérer d\'autres utilisateurs, modifier les paramètres et tous les enregistrements.',
      'user_management': 'Gestion des utilisateurs',
      'users': 'Utilisateurs',
      'new_user': 'Nouvel utilisateur',
      'edit_user': 'Éditer l\'utilisateur',
      'created_user': 'L\'utilisateur a été créé avec succès',
      'updated_user': 'L\'utilisateur a été mis à jour avec succès',
      'archived_user': 'L\'utilisateur a été archivé avec succès',
      'deleted_user': 'L\'utilisateur a été supprimé avec succès',
      'removed_user': 'L\'utilisateur a été retiré avec succès',
      'restored_user': 'L\'utilisateur a été restauré avec succès',
      'archived_users': 'Les :value utilisateurs ont été archivés avec succès',
      'deleted_users': 'Les :value utilisateurs ont été supprimés avec succès',
      'removed_users': 'Les :value utilisateurs ont été retirés avec succès',
      'restored_users': 'Les :value utilisateurs ont été restaurés avec succès',
      'general_settings': 'Paramètres généraux',
      'invoice_options': 'Options de facture',
      'hide_paid_to_date': 'Masquer \"Payé à ce jour\"',
      'hide_paid_to_date_help':
          'Afficher seulement la ligne \"Payé à ce jour\" sur les factures pour lesquelles il y a au moins un paiement.',
      'invoice_embed_documents': 'Documents intégrés',
      'invoice_embed_documents_help':
          'Inclure les images jointes dans la facture.',
      'all_pages_header': 'Afficher l\'entête sur',
      'all_pages_footer': 'Afficher le pied de page sur',
      'first_page': 'première page',
      'all_pages': 'toutes les pages',
      'last_page': 'dernière page',
      'primary_font': 'Fonte principale',
      'secondary_font': 'Fonte secondaire',
      'primary_color': 'Couleur principale',
      'secondary_color': 'Couleur secondaire',
      'page_size': 'Taille de page',
      'font_size': 'Taille de police',
      'quote_design': 'Design de soumission',
      'invoice_fields': 'Champs de facture',
      'product_fields': 'Champs produit',
      'invoice_terms': 'Conditions de facturation',
      'invoice_footer': 'Pied de facture',
      'quote_terms': 'Conditions de soumission',
      'quote_footer': 'Pied de soumission par défaut',
      'auto_email_invoice': 'Envoi automatique',
      'auto_email_invoice_help':
          'Envoi automatiquement les factures récurrentes lorsqu\'elles sont créées.',
      'auto_archive_invoice': 'Autoarchivage',
      'auto_archive_invoice_help':
          'Archive automatiquement les soumissions lorsqu\'elles sont converties.',
      'auto_archive_quote': 'Autoarchivage',
      'auto_archive_quote_help':
          'Archive automatiquement les soumissions lorsqu\'elles sont converties.',
      'auto_convert_quote': 'Autoconversion',
      'auto_convert_quote_help':
          'Convertir automatiquement une soumission en facture lorsque le client l\'accepte.',
      'workflow_settings': 'Paramètres de flux de travail',
      'freq_daily': 'Quotidienne',
      'freq_weekly': 'Hebdomadaire',
      'freq_two_weeks': 'Aux deux semaines',
      'freq_four_weeks': 'Aux quatre semaines',
      'freq_monthly': 'Mensuelle',
      'freq_two_months': 'Deux mois',
      'freq_three_months': 'Trimestrielle',
      'freq_four_months': '4 mois',
      'freq_six_months': 'Semestrielle',
      'freq_annually': 'Annuelle',
      'freq_two_years': 'Deux ans',
      'freq_three_years': 'Trois ans',
      'never': 'Jamais',
      'company': 'Entreprise',
      'generated_numbers': 'Nombres générés',
      'charge_taxes': 'Facturer les taxes',
      'next_reset': 'Prochaine remise à zéro',
      'reset_counter': 'Remettre à zéro le compteur',
      'recurring_prefix': 'Préfixe récurrent',
      'number_padding': 'Marge interne du nombre',
      'general': 'Général',
      'surcharge_field': 'Champ Surcharge',
      'company_field': 'Champ Entreprise',
      'company_value': 'Valeur de compagnie',
      'credit_field': 'Champ Crédit',
      'invoice_field': 'Champ Facture',
      'invoice_surcharge': 'Surcharge de facture',
      'client_field': 'Champ Client',
      'product_field': 'Champ Produit',
      'payment_field': 'Champ Paiement',
      'contact_field': 'Champ Contact',
      'vendor_field': 'Champ Fournisseur',
      'expense_field': 'Champ Dépense',
      'project_field': 'Champ Projet',
      'task_field': 'Champ Tâche',
      'group_field': 'Champ Groupe',
      'number_counter': 'Compteur de nombre',
      'prefix': 'Préfixe',
      'number_pattern': 'Modèle de nombre',
      'messages': 'Messages',
      'custom_css': 'CSS personnalisé',
      'custom_javascript': 'JavaScript personnalisé',
      'signature_on_pdf': 'Afficher sur le PDF',
      'signature_on_pdf_help':
          'Afficher la signature du client sur la facture/soumission PDF.',
      'show_accept_invoice_terms':
          'Case à cocher pour les conditions de facturation',
      'show_accept_invoice_terms_help':
          'Requiert du client qu\'il confirme et accepte les conditions de facturation',
      'show_accept_quote_terms':
          'Case à cocher pour les conditions de soumssion',
      'show_accept_quote_terms_help':
          'Requiert du client qu\'il confirme et accepte les conditions de soumission',
      'require_invoice_signature': 'Signature de facture',
      'require_invoice_signature_help': 'Requiert une signature du client',
      'require_quote_signature': 'Signature de soumission',
      'enable_portal_password': 'Protéger les factures avec un mot de passe',
      'enable_portal_password_help':
          'Permet de spécifier un mot de passe pour chaque contact. Si un mot de passe est spécifié, le contact devra saisir ce mot de passe pour visualiser ses factures.',
      'authorization': 'Autorisation',
      'subdomain': 'sous-domaine',
      'domain': 'Domaine',
      'portal_mode': 'Mode portail',
      'email_signature': 'Cordialement,',
      'enable_email_markup_help':
          'rendez le paiement plus facile à vos client en ajoutant à vos courriel, le marquage de schema.org.',
      'plain': 'Ordinaire',
      'light': 'Clair',
      'dark': 'Foncé',
      'email_design': 'Modèle de courriel',
      'attach_pdf': 'Joindre un PDF',
      'attach_documents': 'Joindre un document',
      'attach_ubl': 'Joindre UBL',
      'email_style': 'Style de courriel',
      'enable_email_markup': 'Autoriser le marquage',
      'reply_to_email': 'Courriel de réponse',
      'reply_to_name': 'Nom de Répondre À',
      'bcc_email': 'Courriel CCI',
      'processed': 'Traité',
      'credit_card': 'Carte de crédit',
      'bank_transfer': 'Virement bancaire',
      'priority': 'Priorité',
      'fee_amount': 'Montant des frais',
      'fee_percent': 'Pourcentage des frais',
      'fee_cap': 'Limite des frais',
      'limits_and_fees': 'Limites/Frais',
      'enable_min': 'Activer min',
      'enable_max': 'Activer max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Logos des cartes acceptées',
      'credentials': 'Identifiants',
      'update_address': 'Mise à jour de l\'adresse',
      'update_address_help':
          'Met à jour l\'adresse du client avec les informations fournies',
      'rate': 'Taux',
      'tax_rate': 'Taux de taxe',
      'new_tax_rate': 'Nouveau taux de taxe',
      'edit_tax_rate': 'Éditer le taux de taxe',
      'created_tax_rate': 'Le taux de taxe a été créé avec succès',
      'updated_tax_rate': 'Le taux de taxe a été mis à jour avec succès',
      'archived_tax_rate': 'Le taux de taxe a été archivé avec succès',
      'deleted_tax_rate': 'Le taux de taxe a été supprimé avec succès',
      'restored_tax_rate': 'Le taux de taxe a été restauré avec succès',
      'archived_tax_rates':
          'Les :value taux de taxes ont été archivés avec succès',
      'deleted_tax_rates':
          'Les :value taux de taxes ont été supprimés avec succès',
      'restored_tax_rates':
          'Les :value taux de taxes ont été restaurés avec succès',
      'fill_products': 'Remplissage automatique des produits',
      'fill_products_help':
          'La sélection d\'un produit entrainera la mise à jour de la description et du prix',
      'update_products': 'Mise à jour automatique des produits',
      'update_products_help':
          'La mise à jour d\'une facture entraîne la mise à jour des produits',
      'convert_products': 'Convertir les produits',
      'convert_products_help':
          'Convertir automatiquement le prix des produits dans la devise du client',
      'fees': 'Frais',
      'limits': 'Limites',
      'provider': 'Fournisseur',
      'company_gateway': 'Passerelle de paiement',
      'company_gateways': 'Passerelles de paiement',
      'new_company_gateway': 'Nouvelle passerelle',
      'edit_company_gateway': 'Éditer la passerelle',
      'created_company_gateway': 'La passerelle a été créée avec succès',
      'updated_company_gateway': 'La passerelle a été mise à jour avec succès',
      'archived_company_gateway': 'La passerelle a été archivée avec succès',
      'deleted_company_gateway': 'La passerelle a été supprimée avec succès',
      'restored_company_gateway': 'La passerelle a été restaurée avec succès',
      'archived_company_gateways':
          'Les :value passerelles ont été archivées avec succès',
      'deleted_company_gateways':
          'Les :value passerelles ont été supprimées avec succès',
      'restored_company_gateways':
          'Les :value passerelles ont été restaurées avec succès',
      'continue_editing': 'Continuez l\'édition',
      'discard_changes': 'Annuler les changements',
      'default_value': 'Valeur par défaut',
      'disabled': 'Désactivé',
      'currency_format': 'Format de devise',
      'first_day_of_the_week': 'Premier jour de la semaine',
      'first_month_of_the_year': 'Premier mois de l\'année',
      'sunday': 'Dimanche',
      'monday': 'Lundi',
      'tuesday': 'Mardi',
      'wednesday': 'Mercredi',
      'thursday': 'Jeudi',
      'friday': 'Vendredi',
      'saturday': 'Samedi',
      'january': 'Janvier',
      'february': 'Février',
      'march': 'Mars',
      'april': 'Avril',
      'may': 'Mai',
      'june': 'Juin',
      'july': 'Juillet',
      'august': 'Août',
      'september': 'Septembre',
      'october': 'Octobre',
      'november': 'Novembre',
      'december': 'Décembre',
      'symbol': 'Symbole',
      'ocde': 'Code',
      'date_format': 'Format de date',
      'datetime_format': 'Format date/heure',
      'military_time': 'Format d\'heure 24 h',
      'military_time_help': 'Affichage 24h',
      'send_reminders': 'Envoyer des rappels',
      'timezone': 'Fuseau horaire',
      'filtered_by_project': 'Filtrer par projet',
      'filtered_by_group': 'Filtrer par groupe',
      'filtered_by_invoice': 'Filtrer par facture',
      'filtered_by_client': 'Filtrer par client',
      'filtered_by_vendor': 'Filtrer par fournisseur',
      'group_settings': 'Paramètres des groupes',
      'group': 'Groupe',
      'groups': 'Groupes',
      'new_group': 'Nouveau groupe',
      'edit_group': 'Éditer le groupe',
      'created_group': 'Le groupe a été créé avec succès',
      'updated_group': 'Le groupe a été mis à jour avec succès',
      'archived_groups': 'Les :value groupes ont été archivés avec succès',
      'deleted_groups': 'Les :value groupes ont été supprimés avec succès',
      'restored_groups': 'Les :value groupes ont été restaurés avec succès',
      'upload_logo': 'Téléverser le logo',
      'uploaded_logo': 'Le logo a été téléversé avec succès',
      'logo': 'Logo',
      'saved_settings': 'Les paramètres ont été sauvegardés avec succès',
      'product_settings': 'Paramètres des produits',
      'device_settings': 'Paramètres de l\'appareil',
      'defaults': 'Pré-définis',
      'basic_settings': 'Paramètres de base',
      'advanced_settings': 'Paramètres avancés',
      'company_details': 'Informations sur l\'entreprise',
      'user_details': 'Profil utilisateur',
      'localization': 'Paramètres régionaux',
      'online_payments': 'Paiements en ligne',
      'tax_rates': 'Taux de taxe',
      'notifications': 'Notifications',
      'import_export': 'Importer / Exporter',
      'custom_fields': 'Champs personnalisés',
      'invoice_design': 'Modèles de factures',
      'buy_now_buttons': 'Boutons Achetez maintenant',
      'email_settings': 'Paramètres de courriel',
      'templates_and_reminders': 'Modèles et rappels',
      'credit_cards_and_banks': 'Cartes de crédit et banques',
      'data_visualizations': 'Visualisation des données',
      'price': 'Prix',
      'email_sign_up': 'Inscription par courriel',
      'google_sign_up': 'Inscription avec Google',
      'thank_you_for_your_purchase': 'Merci de votre achat!',
      'redeem': 'Rembourser',
      'back': 'Retour',
      'past_purchases': 'Achats précédents',
      'annual_subscription': 'Abonnement annuel',
      'pro_plan': 'Plan Pro',
      'enterprise_plan': 'Plan Entreprise',
      'count_users': ':count utilisateurs',
      'upgrade': 'Mettre à niveau',
      'please_enter_a_first_name': 'Veuillez entrer votre prénom',
      'please_enter_a_last_name': 'Veuillez entrer votre nom',
      'please_agree_to_terms_and_privacy':
          'Vous devez accepter les conditions et la politique de confidentialité pour créer un compte.',
      'i_agree_to_the': 'J\'accepte',
      'terms_of_service_link': 'les conditions',
      'privacy_policy_link': 'la politique de confidentialité',
      'terms_of_service': 'Conditions d\'utilisation',
      'privacy_policy': 'Politique de confidentialité',
      'sign_up': 'Inscription',
      'account_login': 'Connexion',
      'view_website': 'Visiter le site web',
      'create_account': 'Créer un compte',
      'email_login': 'Courriel de connexion',
      'create_new': 'Créer',
      'no_record_selected': 'Aucun enregistrement sélectionné',
      'error_unsaved_changes':
          'Veuillez sauvegarder ou annuler vos modifications',
      'download': 'Télécharger',
      'requires_an_enterprise_plan': 'Le plan Entreprise est requis',
      'take_picture': 'Prendre un photo',
      'upload_file': 'Téléverser un fichier',
      'document': 'Justificatifs',
      'documents': 'Documents',
      'new_document': 'Nouveau document',
      'edit_document': 'Éditer un document',
      'uploaded_document': 'Le document a été téléversé avec succès',
      'updated_document': 'Le document a été mis à jour avec succès',
      'archived_document': 'Le document a été archivé avec succès',
      'deleted_document': 'Le document a été supprimé avec succès',
      'restored_document': 'Le document a été restauré avec succès',
      'archived_documents': 'Les :value documents ont été archivés avec succès',
      'deleted_documents': 'Les :value documents ont été supprimés avec succès',
      'restored_documents':
          'Les :value documents ont été restaurés avec succès',
      'no_history': 'Aucun historique',
      'expense_date': 'Date de la dépense',
      'pending': 'En attente',
      'expense_status_1': 'Connecté',
      'expense_status_2': 'En attente',
      'expense_status_3': 'Facturé',
      'converted': 'Convertie',
      'add_documents_to_invoice': 'Ajouter des documents aux factures',
      'exchange_rate': 'Taux de change',
      'convert_currency': 'Conversion de devise',
      'mark_paid': 'Marquer payée',
      'category': 'Catégorie',
      'address': 'Adresse',
      'new_vendor': 'Nouveau fournisseur',
      'created_vendor': 'Le fournisseur a été créé avec succès',
      'updated_vendor': 'Le fournisseur a été mis à jour avec succès',
      'archived_vendor': 'Le fournisseur a été archivé avec succès',
      'deleted_vendor': 'Le fournisseur a été supprimé avec succès',
      'restored_vendor': 'Le fournisseur  a été restauré avec succès',
      'archived_vendors': ':count fournisseurs ont été archivés avec succès',
      'deleted_vendors': ':count fournisseurs ont été supprimés avec succès',
      'restored_vendors':
          'Les :value fournisseurs ont été restaurés avec succès',
      'new_expense': 'Entrer une dépense',
      'created_expense': 'La dépense a été créée avec succès',
      'updated_expense': 'La dépense a été mise à jour avec succès',
      'archived_expense': 'La dépense a été archivée avec succès',
      'deleted_expense': 'La dépense a été supprimée avec succès',
      'restored_expense': 'La dépense a été restaurée avec succès',
      'archived_expenses': 'Les dépenses ont été archivées avec succès',
      'deleted_expenses': 'Les dépenses ont été supprimées avec succès',
      'restored_expenses': 'Les :value dépenses ont été restaurées avec succès',
      'copy_shipping': 'Copier livraison',
      'copy_billing': 'Copier facturation',
      'design': 'Conception',
      'failed_to_find_record': 'Enregistrement introuvable',
      'invoiced': 'Facturée',
      'logged': 'Enregistrée',
      'running': 'En cours',
      'resume': 'Continuer',
      'task_errors': 'Veuillez corriger les plages de temps qui se chevauchent',
      'start': 'Démarrer',
      'stop': 'Arrêter',
      'started_task': 'La tâche a démaré avec succès',
      'stopped_task': 'La tâche a été arrêtée avec succès',
      'resumed_task': 'La tâche a été reprise avec succès',
      'now': 'Maintenant',
      'auto_start_tasks': 'Démarrage de tâches automatique',
      'timer': 'Minuteur',
      'manual': 'Manuel',
      'budgeted': 'Budgété',
      'start_time': 'Démarrée à',
      'end_time': 'Arrêtée à',
      'date': 'Date',
      'times': 'Temps',
      'duration': 'Durée',
      'new_task': 'Nouvelle tâche',
      'created_task': 'La tâche a été créée avec succès',
      'updated_task': 'La tâche a été modifiée avec succès',
      'archived_task': 'La tâche a été archivée avec succès',
      'deleted_task': 'La tâche a été supprimée avec succès',
      'restored_task': 'La tâche a été restaurée avec succès',
      'archived_tasks': ':count tâches ont été archivées avec succès',
      'deleted_tasks': ':count tâches ont été supprimées avec succès',
      'restored_tasks': 'Les :value tâches ont été restaurées avec succès',
      'please_enter_a_name': 'Veuillez entrer un nom',
      'budgeted_hours': 'Heures budgétées',
      'created_project': 'Le projet a été créé avec succès',
      'updated_project': 'Le projet a été mis à  jour avec succès',
      'archived_project': 'Le projet a été archivé avec succès',
      'deleted_project': 'Le projet a été supprimé avec succès',
      'restored_project': 'Le projet a été restauré avec succès',
      'archived_projects': ':count projets ont été archivés avec succès',
      'deleted_projects': ':count projets ont été supprimés avec succès',
      'restored_projects': 'Les :value projets ont été restaurés avec succès',
      'new_project': 'Nouveau projet',
      'thank_you_for_using_our_app': 'Merci d\'utiliser notre app!',
      'if_you_like_it': 'Si vous appréciez, merci',
      'click_here': 'cliquer ici',
      'click_here_capital': 'Cliquez ici',
      'to_rate_it': 'd\'évaluer notre app.',
      'average': 'Moyenne',
      'unapproved': 'Non approuvé',
      'authenticate_to_change_setting':
          'Veuillez vous connecter pour changer ce paramètre',
      'locked': 'Verrouillé',
      'authenticate': 'Connexion',
      'please_authenticate': 'Veuillez vous connecter',
      'biometric_authentication': 'Connexion biométrique',
      'footer': 'Pied de page',
      'compare': 'Comparer',
      'hosted_login': 'Connexion hébergée',
      'selfhost_login': 'Connexion autohébergée',
      'google_sign_in': 'Connexion avec Google',
      'today': 'Aujourd\'hui',
      'custom_range': 'Personnalisé',
      'date_range': 'Intervalle de dates',
      'current': 'En cours',
      'previous': 'Précédent',
      'current_period': 'Période en cours',
      'comparison_period': 'Période de comparaison',
      'previous_period': 'Période précédente',
      'previous_year': 'Année précédente',
      'compare_to': 'Comparer à',
      'last7_days': '7 derniers jours',
      'last_week': 'Dernière semaine',
      'last30_days': '30 derniers jours',
      'this_month': 'Mois en cours',
      'last_month': 'Mois dernier',
      'this_year': 'Cette année',
      'last_year': 'Dernière année',
      'custom': 'Personnalisé',
      'clone_to_invoice': 'Cloner en facture',
      'clone_to_quote': 'Cloner en soumission',
      'clone_to_credit': 'Cloner au crédit',
      'view_invoice': 'Voir la facture',
      'convert': 'Convertir',
      'more': 'Plus',
      'edit_client': 'Modifier le client',
      'edit_product': 'Modifier le produit',
      'edit_invoice': 'Modifier la facture',
      'edit_quote': 'Modifier la soumission',
      'edit_payment': 'Modifier le paiement',
      'edit_task': 'Éditer la tâche',
      'edit_expense': 'Éditer la dépense',
      'edit_vendor': 'Éditer le fournisseur',
      'edit_project': 'Éditer le projet',
      'edit_recurring_expense': 'Éditer la dépense récurrente',
      'edit_recurring_quote': 'Éditer la soumission récurrente',
      'billing_address': 'Adresse de facturation',
      'shipping_address': 'Adresse de livraison',
      'total_revenue': 'Revenu total',
      'average_invoice': 'Facture moyenne',
      'outstanding': 'Impayées',
      'invoices_sent': ':count factures envoyées',
      'active_clients': 'clients actifs',
      'close': 'Fermer',
      'email': 'Courriel',
      'password': 'Mot de passe',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Nom',
      'logout': 'Déconnexion',
      'login': 'Connexion',
      'filter': 'Filtrer',
      'sort': 'Trier',
      'search': 'Rechercher',
      'active': 'Actif',
      'archived': 'Archivé',
      'deleted': 'Supprimé',
      'dashboard': 'Tableau de bord',
      'archive': 'Archiver',
      'delete': 'Supprimer',
      'restore': 'Restaurer',
      'refresh_complete': 'Actualisation complétée',
      'please_enter_your_email': 'Veuillez saisir votre courriel',
      'please_enter_your_password': 'Veuillez saisir votre mot de passe',
      'please_enter_your_url': 'Veuillez saisir votre URL',
      'please_enter_a_product_key': 'Veuillez saisir la clé de produit',
      'ascending': 'Ascendant',
      'descending': 'Descendant',
      'save': 'Sauvegarder',
      'an_error_occurred': 'Il y a eu une erreur',
      'paid_to_date': 'Payé à ce jour',
      'balance_due': 'Solde dû',
      'balance': 'Solde',
      'overview': 'Survol',
      'details': 'Détails',
      'phone': 'Téléphone',
      'website': 'Site web',
      'vat_number': 'N° de taxe',
      'id_number': 'N° d\'entreprise',
      'create': 'Créer',
      'copied_to_clipboard': ':value a été copié au presse-papier',
      'error': 'Erreur',
      'could_not_launch': 'Lancement impossible',
      'contacts': 'Contacts',
      'additional': 'Additionnel',
      'first_name': 'Prénom',
      'last_name': 'Nom',
      'add_contact': 'Ajouter un contact',
      'are_you_sure': 'Voulez-vous vraiment effectuer cette action?',
      'cancel': 'Annuler',
      'ok': 'Ok',
      'remove': 'Retirer',
      'email_is_invalid': 'Le courriel est invalide',
      'product': 'Produit',
      'products': 'Produits',
      'new_product': 'Nouveau produit',
      'created_product': 'Le produit a été créé avec succès',
      'updated_product': 'Le produit a été mis à jour avec succès',
      'archived_product': 'Le produit a été archivé avec succès',
      'deleted_product': 'Le produit a été supprimé avec succès',
      'restored_product': 'Le produit a été restauré avec succès',
      'archived_products': ':count produits archivés avec succès',
      'deleted_products': ':count produits ont été supprimés avec succès',
      'restored_products': 'Les :value produits ont été restaurés avec succès',
      'product_key': 'Produit',
      'notes': 'Notes',
      'cost': 'Coût',
      'client': 'Client',
      'clients': 'Clients',
      'new_client': 'Nouveau client',
      'created_client': 'Le client a été créé avec succès',
      'updated_client': 'Le client a été modifié avec succès',
      'archived_client': 'Le client a été archivé avec succès',
      'archived_clients': ':count clients archivés avec succès',
      'deleted_client': 'Le client a été supprimé avec succès',
      'deleted_clients': ':count clients ont été supprimés avec succès',
      'restored_client': 'Le client a été restauré avec succès',
      'restored_clients': 'Les :value clients ont été restaurés avec succès',
      'address1': 'Rue',
      'address2': 'Appartement / Bureau',
      'city': 'Ville',
      'state': 'État / Province',
      'postal_code': 'Code postal',
      'country': 'Pays',
      'invoice': 'Facture',
      'invoices': 'Factures',
      'new_invoice': 'Nouvelle facture',
      'created_invoice': 'La facture a été créée avec succès',
      'updated_invoice': 'La facture a été modifiée avec succès',
      'archived_invoice': 'La facture a été archivée avec succès',
      'deleted_invoice': 'La facture a été supprimée avec succès',
      'restored_invoice': 'La facture a été restaurée avec succès',
      'archived_invoices': ':count factures ont été archivées avec succès',
      'deleted_invoices': ':count factures supprimées avec succès',
      'restored_invoices': 'Les :value factures ont été restaurées avec succès',
      'emailed_invoice': 'La facture a été envoyée par courriel avec succès',
      'emailed_payment': 'Le paiement a été envoyé par courriel avec succès',
      'amount': 'Montant',
      'invoice_number': 'N° de facture',
      'invoice_date': 'Date de facturation',
      'discount': 'Escompte',
      'po_number': 'N° de bon de commande',
      'terms': 'Modalités',
      'public_notes': 'Notes publiques',
      'private_notes': 'Notes personnelles',
      'frequency': 'Fréquence',
      'start_date': 'Date de début',
      'end_date': 'Date de fin',
      'quote_number': 'N° de soumission',
      'quote_date': 'Date de soumission',
      'valid_until': 'Valide jusqu\'au',
      'items': 'Articles',
      'partial_deposit': 'Partiel/dépôt',
      'description': 'Description',
      'unit_cost': 'Coût unitaire',
      'quantity': 'Quantité',
      'add_item': 'Ajouter un article',
      'contact': 'Contact',
      'work_phone': 'Téléphone',
      'total_amount': 'Montant total',
      'pdf': 'PDF',
      'due_date': 'Échéance',
      'partial_due_date': 'Date d\'échéance paiement partiel',
      'status': 'Statut',
      'invoice_status_id': 'État de la facture',
      'quote_status': 'État de la soumission',
      'click_plus_to_add_item': 'Cliquez + pour ajouter un article',
      'click_plus_to_add_time': 'Cliquez sur + pour ajouter du temps',
      'count_selected': ':count sélectionnés',
      'total': 'Total',
      'percent': 'Pourcent',
      'edit': 'Modifier',
      'dismiss': 'Annuler',
      'please_select_a_date': 'Veuillez saisir une date',
      'please_select_a_client': 'Veuillez sélectionner un client',
      'please_select_an_invoice': 'Veuillez sélectionner une facture',
      'task_rate': 'Taux  de tâche',
      'settings': 'Paramètres',
      'language': 'Langue',
      'currency': 'Devise',
      'created_at': 'Créé le',
      'created_on': 'Créé le',
      'updated_at': 'Mis à jour',
      'tax': 'Taxe',
      'please_enter_an_invoice_number': 'Veuillez saisir un numéro de facture',
      'please_enter_a_quote_number': 'Veuillez saisir un numéro de soumission',
      'past_due': 'En souffrance',
      'draft': 'Brouillon',
      'sent': 'Envoyé',
      'viewed': 'Vue',
      'approved': 'Approuvée',
      'partial': 'Partiel / dépôt',
      'paid': 'Payé',
      'mark_sent': 'Marquer comme envoyé',
      'marked_invoice_as_sent':
          'La facture a été marquée comme envoyée avec succès',
      'marked_invoice_as_paid':
          'La facture a été marquée comme envoyée avec succès',
      'marked_invoices_as_sent':
          'Les factures ont été marquées comme envoyées avec succès',
      'marked_invoices_as_paid':
          'Les factures ont été marquées comme envoyées avec succès',
      'done': 'Valider',
      'please_enter_a_client_or_contact_name':
          'Veuillez saisir un nom de client ou de contact',
      'dark_mode': 'Mode foncé',
      'restart_app_to_apply_change':
          'Redémarrez l\'app pour mettre à jour les changements',
      'refresh_data': 'Actualiser les données',
      'blank_contact': 'Contact vide',
      'activity': 'Activité',
      'no_records_found': 'Aucun enregistrement trouvé',
      'clone': 'Dupliquer',
      'loading': 'Chargement',
      'industry': 'Entreprise',
      'size': 'Taille',
      'payment_terms': 'Délais de paiement',
      'payment_date': 'Date du paiement',
      'payment_status': 'État du paiement',
      'payment_status_1': 'Em attente',
      'payment_status_2': 'Annulée',
      'payment_status_3': 'Échouée',
      'payment_status_4': 'Complétée',
      'payment_status_5': 'Partiellement remboursée',
      'payment_status_6': 'Remboursée',
      'payment_status_-1': 'Non appliqué',
      'payment_status_-2': 'Partiellement non-appliquée',
      'net': 'Net',
      'client_portal': 'Portail client',
      'show_tasks': 'Afficher les tâches',
      'email_reminders': 'Courriel de rappel',
      'enabled': 'Activé',
      'recipients': 'destinataires',
      'initial_email': 'Courriel initial',
      'first_reminder': '1er rappel',
      'second_reminder': '2e rappel',
      'third_reminder': '3e rappel',
      'reminder1': 'Premier rappel',
      'reminder2': 'Deuxième rappel',
      'reminder3': 'Troisième rappel',
      'template': 'Modèle',
      'send': 'Envoyer',
      'subject': 'Sujet',
      'body': 'Message',
      'send_email': 'Envoyer un courriel',
      'email_receipt': 'Envoyer le reçu de paiement par courriel au client',
      'auto_billing': 'Facturation automatique',
      'button': 'Bouton',
      'preview': 'PRÉVISUALISATION',
      'customize': 'Personnalisation',
      'history': 'Historique',
      'payment': 'Paiement',
      'payments': 'Paiements',
      'refunded': 'Remboursée',
      'payment_type': 'Type de paiement',
      'transaction_reference': 'N° de référence',
      'enter_payment': 'Inscrire un paiement',
      'new_payment': 'Inscrire un paiement',
      'created_payment': 'Le paiement a été créé avec succès',
      'updated_payment': 'Le paiement a été mis à jour avec succès',
      'archived_payment': 'Le paiement a été archivé avec succès',
      'deleted_payment': 'Le paiement a été supprimé avec succès',
      'restored_payment': 'Le paiement a été restauré avec succès',
      'archived_payments': ':count paiements ont été archivés avec succès',
      'deleted_payments': ':count paiements ont été supprimés avec succès',
      'restored_payments': 'Les :value paiements ont été restaurés avec succès',
      'quote': 'Soumission',
      'quotes': 'Soumissions',
      'new_quote': 'Nouvelle soumission',
      'created_quote': 'La soumission a été créée avec succès',
      'updated_quote': 'La soumission a été mise à jour avec succès',
      'archived_quote': 'La soumission a été archivée avec succès',
      'deleted_quote': 'La soumission a été supprimée avec succès',
      'restored_quote': 'La soumission a été restaurée avec succès',
      'archived_quotes': ':count soumissions ont été archivées avec succès',
      'deleted_quotes': ':count soumissions ont été supprimées avec succès',
      'restored_quotes':
          'Les :value soumissions ont été restaurées avec succès',
      'expense': 'Dépense',
      'expenses': 'Dépenses',
      'vendor': 'Fournisseur',
      'vendors': 'Fournisseurs',
      'task': 'Tâche',
      'tasks': 'Tâches',
      'project': 'Projet',
      'projects': 'Projets',
      'activity_1': ':user a créé le client :client',
      'activity_2': ':user a archivé le client :client',
      'activity_3': ':user a supprimé le client :client',
      'activity_4': ':user a créé la facture :invoice',
      'activity_5': ':user a mis à jour la facture :invoice',
      'activity_6':
          ':user a envoyé par courriel la facture :invoice pour :client à :contact',
      'activity_7': ':contact a visualisé la facture :invoice pour :client',
      'activity_8': ':user a archivé la facture :invoice',
      'activity_9': ':user a supprimé la facture :invoice',
      'activity_10':
          ':contact a saisi le paiement :payment de :payment_amount de la facture :invoice pour :client',
      'activity_11': ':user a mis à jour le paiement :payment',
      'activity_12': ':user a archivé le paiement :payment',
      'activity_13': ':user a supprimé le paiement :payment',
      'activity_14': ':user a saisi le crédit :credit',
      'activity_15': ':user a mis à jour le crédit :credit',
      'activity_16': ':user a archivé le crédit :credit',
      'activity_17': ':user a supprimé le crédit :credit',
      'activity_18': ':user a créé la soumission :quote',
      'activity_19': ':user a mis à jour la soumission :quote',
      'activity_20':
          ':user a envoyé par courriel la soumission :quote pour :client à :contact',
      'activity_21': ':contact a visualisé la soumission :quote',
      'activity_22': ':user a archivé la soumission :quote',
      'activity_23': ':user a supprimé la soumission :quote',
      'activity_24': ':user a restauré la soumission :quote',
      'activity_25': ':user a restauré la facture :invoice',
      'activity_26': ':user a restauré le client :client',
      'activity_27': ':user a restauré le paiement :payment',
      'activity_28': ':user a restauré le crédit :credit',
      'activity_29': ':contact a approuvé la soumission :quote pour :client',
      'activity_30': ':user a créé le fournisseur :vendor',
      'activity_31': ':user a archivé le fournisseur :vendor',
      'activity_32': ':user a supprimé le fournisseur :vendor',
      'activity_33': ':user a restauré le fournisseur :vendor',
      'activity_34': ':user a créé la dépense :expense',
      'activity_35': ':user a archivé la dépense :expense',
      'activity_36': ':user a supprimé la dépense :expense',
      'activity_37': ':user a restauré la dépense :expense',
      'activity_39': ':user a annulé un paiement :payment de :payment_amount',
      'activity_40':
          ':user a remboursé :adjustment d\'un paiement :payment de :payment_amount',
      'activity_41': 'Le paiement de :payment_amount a échoué (:payment)',
      'activity_42': ':user a créé la tâche :task',
      'activity_43': ':user a mis à jour la tâche :task',
      'activity_44': ':user a archivé la tâche :task',
      'activity_45': ':user a supprimé la tâche :task',
      'activity_46': ':user a restauré la tâche :task',
      'activity_47': ':user a mis à jour la dépense :expense',
      'activity_48': ':user a mis à jour le billet :ticket',
      'activity_49': ':user a fermé le billet :ticket',
      'activity_50': ':user a fusionné le billet :ticket',
      'activity_51': ':user a scindé le billet :ticket',
      'activity_52': ':contact a ouvert le billet :ticket',
      'activity_53': ':contact a réouvert le billet :ticket',
      'activity_54': ':user a réouvert le billet :ticket',
      'activity_55': ':contact a répondu au billet :ticket',
      'activity_56': ':user a vu le billet :ticket',
      'activity_57':
          'Le système n\'a pas pu envoyer le courriel de la facture :invoice',
      'activity_58': ':user a inversé la facture :invoice',
      'activity_59': ':user a annulé la facture :invoice',
      'activity_60': ':contact a vu la soumission :quote',
      'activity_61': ':user a mis à jour le client :client',
      'activity_62': ':user a mis à jour le fournisseur :vendor',
      'activity_63':
          ':user a envoyé le premier rappel pour la facture :invoice de :contact',
      'activity_64':
          ':user a envoyé le deuxième rappel pour la facture :invoice de :contact',
      'activity_65':
          ':user a envoyé le troisième rappel pour la facture :invoice de :contact',
      'activity_66':
          ':user a envoyé un rappel sans fin pour la facture :invoice de :contact',
      'activity_80': ':user a créé l\'abonnement :subscription',
      'activity_81': ':user a mis à jour l\'abonnement :subscription',
      'activity_82': ':user a archivé l\'abonnement :subscription',
      'activity_83': ':user a supprimé l\'abonnement :subscription',
      'activity_84': ':user a restauré l\'abonnement :subscription',
      'one_time_password': 'Mot de passe à usage unique',
      'emailed_quote': 'La soumission a été envoyée avec succès',
      'emailed_credit': 'Le crédit a envoyé par courriel avec succès',
      'marked_quote_as_sent':
          'La soumission a été marquée comme envoyée avec succès',
      'marked_credit_as_sent':
          'Le crédit a été marqué comme envoyé avec succès',
      'expired': 'Expiré',
      'all': 'Tous',
      'select': 'Sélectionner',
      'long_press_multiselect': 'Multisélection par pression longue',
      'custom_value1': 'Valeur par défaut',
      'custom_value2': 'Valeur par défaut',
      'custom_value3': 'Valeur personnalisée 3',
      'custom_value4': 'Valeur personnalisée 4',
      'email_style_custom': 'Style de courriel personnalisé',
      'custom_message_dashboard': 'Message personnalisé du tableau de bord',
      'custom_message_unpaid_invoice':
          'Message personnalisé pour facture impayée',
      'custom_message_paid_invoice': 'Message personnalisé pour facture payée',
      'custom_message_unapproved_quote':
          'Message personnalisé pour soumission non approuvée',
      'lock_invoices': 'Verrouiller les factures',
      'translations': 'Traductions',
      'task_number_pattern': 'Modèle du numéro de tâche',
      'task_number_counter': 'Compteur du numéro de tâche',
      'expense_number_pattern': 'Modèle du numéro de dépense',
      'expense_number_counter': 'Compteur du numéro de dépense',
      'vendor_number_pattern': 'Modèle du numéro de fournisseur',
      'vendor_number_counter': 'Compteur du numéro de fournisseur',
      'ticket_number_pattern': 'Modèle du numéro de billet',
      'ticket_number_counter': 'Compteur du numéro de billet',
      'payment_number_pattern': 'Modèle du numéro de paiement',
      'payment_number_counter': 'Compteur du numéro de paiement',
      'invoice_number_pattern': 'Modèle du numéro de facture',
      'invoice_number_counter': 'Compteur du numéro de facture',
      'quote_number_pattern': 'Modèle du numéro de soumission',
      'quote_number_counter': 'Compteur du numéro de soumission',
      'client_number_pattern': 'Modèle du numéro de crédit',
      'client_number_counter': 'Compteur du numéro de crédit',
      'credit_number_pattern': 'Modèle du numéro de crédit',
      'credit_number_counter': 'Compteur du numéro de crédit',
      'reset_counter_date': 'Remise à zéro du compteur de date',
      'counter_padding': 'Espacement du compteur',
      'shared_invoice_quote_counter':
          'Compteur partagé pour les factures et les soumissions',
      'default_tax_name_1': 'Nom de taxe par défaut 1',
      'default_tax_rate_1': 'Taux de taxe par défaut 1',
      'default_tax_name_2': 'Nom de taxe par défaut 2',
      'default_tax_rate_2': 'Taux de taxe par défaut 2',
      'default_tax_name_3': 'Nom de taxe par défaut 3',
      'default_tax_rate_3': 'Taux de taxe par défaut 3',
      'email_subject_invoice': 'Objet du courriel de facture',
      'email_subject_quote': 'Objet du courriel de soumission',
      'email_subject_payment': 'Objet du courriel de paiement',
      'email_subject_payment_partial': 'Sujet du courriel de paiement partiel',
      'show_table': 'Affiche la table',
      'show_list': 'Afficher la liste',
      'client_city': 'Ville du client',
      'client_state': 'Province du client',
      'client_country': 'Pays du client',
      'client_is_active': 'Client actif',
      'client_balance': 'Solde du client',
      'client_address1': 'Rue du clients',
      'client_address2': 'Apt/Suite',
      'vendor_address1': 'Rue du fournisseur',
      'vendor_address2': 'App du fournisseur',
      'client_shipping_address1': 'Rue d\'expédition',
      'client_shipping_address2': 'Expédition Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Montant de la facture',
      'invoice_due_date': 'Échéance',
      'tax_rate1': 'Taux de taxe 1',
      'tax_rate2': 'Taux de taxe 2',
      'tax_rate3': 'Taux de taxe 3',
      'auto_bill': 'Facturation automatique',
      'archived_at': 'Archivé à',
      'has_expenses': 'A Dépenses',
      'custom_taxes1': 'Taxes personnalisées 1',
      'custom_taxes2': 'Taxes personnalisées 2',
      'custom_taxes3': 'Taxes personnalisées 3',
      'custom_taxes4': 'Taxes personnalisées 4',
      'custom_surcharge1': 'Surcharge personnalisée 1',
      'custom_surcharge2': 'Surcharge personnalisée 2',
      'custom_surcharge3': 'Surcharge personnalisée 3',
      'custom_surcharge4': 'Surcharge personnalisée 4',
      'is_deleted': 'Est supprimé',
      'vendor_city': 'Ville du fournisseur',
      'vendor_state': 'Province du fournisseur',
      'vendor_country': 'Pays du fournisseur',
      'is_approved': 'Est approuvé',
      'tax_name': 'Nom de la taxe',
      'tax_amount': 'Montant de taxe',
      'tax_paid': 'Taxe payée',
      'payment_amount': 'Montant du paiement',
      'age': 'Âge',
      'is_running': 'En cours',
      'time_log': 'Journal de temps',
      'bank_id': 'Banque',
      'expense_category_id': 'ID de catégorie de dépense',
      'expense_category': 'Catégorie de dépense',
      'invoice_currency_id': 'ID de la devise de facturation',
      'tax_name1': 'Nom de la taxe 1',
      'tax_name2': 'Nom de la taxe 2',
      'tax_name3': 'Nom de taxe 3',
      'transaction_id': 'ID de transaction',
      'color_theme': 'Couleur de thème',
    },
    'de': {
      'import_data': 'Importiere Daten',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Zuletzt aktualisiert',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Kontakt Vorname',
      'contact_last_name': 'Kontakt Nachname',
      'order': 'Order',
      'unassigned': 'Nicht zugewiesen',
      'partial_value':
          'Muss grösser als Null und kleiner als der Gesamtbetrag sein',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Aktivieren',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Passen Sie die Rechnungslink-Subdomäne an oder stellen Sie die Rechnung auf Ihrer eigenen Webseite zur Verfügung.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Gutscheincode',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sicherheit',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Abonnement',
      'subscriptions': 'Abonnements',
      'new_subscription': 'Neues Abonnement',
      'edit_subscription': 'Abonnement bearbeiten',
      'created_subscription': 'Abonnement erfolgreich erstellt',
      'updated_subscription': 'Abonnement erfolgreich aktualisiert',
      'archived_subscription': 'Abonnement erfolgreich archiviert',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Mit Gmail verbinden',
      'disconnect_gmail': 'von Gmail trennen',
      'connected_gmail': 'Mit Gmail erfolgreich verbunden',
      'disconnected_gmail': 'Von Gmail erfolgreich getrennt',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Kundennummer',
      'count_minutes': ':count Minuten',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again':
          'Ein Fehler ist aufgetreten, bitte versuchen Sie es erneut.',
      'please_first_set_a_password': 'Bitte vergeben Sie zuerst ein Passwort.',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Hilf mit beim Übersetzen',
      'please_select_a_country': 'Bitte wählen Sie ein Land',
      'resend_invite': 'Einladung erneut versenden',
      'disabled_two_factor': '2FA erfolgreich deaktiviert',
      'connected_google': 'Konto erfolgreich verbunden.',
      'disconnected_google': 'Konto erfolgreich getrennt.',
      'delivered': 'zugestellt',
      'bounced': 'Abpraller',
      'spam': 'Spam',
      'view_docs': 'Dokumentation ansehen.',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'SMS senden',
      'sms_code': 'SMS-Code',
      'two_factor_setup_help': 'Barcode mit :link kompatibler App scannen.',
      'enabled_two_factor':
          'Zwei-Faktor-Authentifizierung erfolgreich aktiviert',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Zwei-Faktor-Authentifizierung',
      'disable_two_factor': 'Zwei-Faktor-Authentifizierung deaktivieren',
      'require_password_with_social_login':
          'Password mit Verknüpfung zu Sozialmedia notwendig',
      'stay_logged_in': 'Eingeloggt bleiben',
      'session_about_to_expire': 'Warnung: Ihre Sitzung läuft bald ab',
      'count_hours': ':count Stunden',
      'count_day': '1 Tag',
      'count_days': ':count Tage',
      'web_session_timeout': 'Web-Sitzungs-Timeout',
      'security_settings': 'Sicherheitseinstellungen',
      'resend_email': 'Bestätigungsemail erneut versenden',
      'confirm_your_email_address': 'Bitte bestätigen Sie Ihre E-Mail-Adresse',
      'refunded_payment': 'Zahlung erstattet',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Zeige Aktionen',
      'start_multiselect': 'Mehrfachauswahl',
      'email_sent_to_confirm_email':
          'Eine E-Mail wurde versandt um Ihre E-Mail-Adresse zu bestätigen.',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'Dieses Quartal',
      'last_quarter': 'Letztes Quartal',
      'to_update_run': 'Zum Änderungslauf',
      'convert_to_invoice': 'In Rechnung umwandeln',
      'registration_url': 'Registrierungs-URL',
      'invoice_project': 'Projekt berechnen',
      'invoice_task': 'Aufgabe in Rechnung stellen',
      'invoice_expense': 'Ausgabe abrechnen',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Speichern und Vorschau anzeigen',
      'save_and_email': 'Speichern und verschicken',
      'supported_events': 'Unterstützte Ereignisse',
      'converted_amount': 'Umgerechneter Betrag',
      'converted_balance': 'Guthabenstand',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Gesendet',
      'default_documents': 'Standard-Dokumente',
      'document_upload': 'Dokument hochladen',
      'document_upload_help': 'Erlaube Kunden Dokumente hochzuladen',
      'expense_total': 'Ausgabensumme',
      'enter_taxes': 'Steuersätze eingeben',
      'by_rate': 'Nach Satz',
      'by_amount': 'Nach Betrag',
      'enter_amount': 'Betrag eingeben',
      'before_taxes': 'Vor Steuern',
      'after_taxes': 'Nach Steuern',
      'color': 'Farbe',
      'show': 'anzeigen',
      'hide': 'Verbergen',
      'empty_columns': 'Leere Spalten',
      'debug_mode_is_enabled': 'Der Entwicklungsmodus ist aktiviert',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Laufende Aufgaben',
      'recent_tasks': 'Kürzliche Aufgaben',
      'recent_expenses': 'Kürzliche Ausgaben',
      'upcoming_expenses': 'Zukünftige Ausgaben',
      'update_app': 'App aktualisieren',
      'started_import': 'Import erfolgreich gestartet',
      'duplicate_column_mapping': 'Dupliziere Spaltenzuordnung',
      'uses_inclusive_taxes': 'Benutzt Inklusive Steuern',
      'is_amount_discount': 'Ist Betrag ermäßigt',
      'column': 'Spalte',
      'sample': 'Beispiel',
      'map_to': 'Zuordnen',
      'import': 'Importieren',
      'first_row_as_column_names': 'Benutze erste Zeile als Spaltenüberschrift',
      'select_file': 'Bitte wähle eine Datei',
      'no_file_selected': 'Keine Datei ausgewählt',
      'csv_file': 'Wähle CSV Datei',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Buchhaltung',
      'required_files_missing': 'Bitte geben Sie alle CSV-Dateien an.',
      'import_type': 'Import Typ',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Lizenzen anzeigen',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Vollbild Editor',
      'sidebar_editor': 'Seitenmenü Editor',
      'please_type_to_confirm':
          'Bitte geben Sie \":value\" zur Bestätigung ein',
      'purge': 'Bereinigen',
      'service': 'Dienst',
      'clone_to': 'Duplizieren zu',
      'clone_to_other': 'Duplizieren zu anderem',
      'labels': 'Beschriftung',
      'add_custom': 'Beschriftung hinzufügen',
      'payment_tax': 'Steuer-Zahlung',
      'unpaid': 'Unbezahlt',
      'white_label': 'White Label',
      'delivery_note': 'Lieferschein',
      'sent_invoices_are_locked': 'Versendete Rechnungen sind gesperrt',
      'paid_invoices_are_locked': 'Bezahlte Rechnungen sind gesperrt',
      'source_code': 'Quellcode',
      'app_platforms': 'Applikations Platform',
      'invoice_late': 'Rechnung in Verzug',
      'quote_expired': 'Angebot abgelaufen',
      'partial_due': 'Anzahlung',
      'invoice_total': 'Rechnungsbetrag',
      'quote_total': 'Angebotssumme',
      'credit_total': 'Gesamtguthaben',
      'recurring_invoice_total': 'Gesamtbetrag',
      'actions': 'Aktionen',
      'expense_number': 'Ausgabennummer',
      'task_number': 'Aufgabennummer',
      'project_number': 'Projektnummer',
      'project_name': 'Projektname',
      'warning': 'Warnung',
      'view_settings': 'Einstellungen anzeigen',
      'company_disabled_warning':
          'Warnung: diese Firma wurde noch nicht aktiviert',
      'late_invoice': 'Rechnung in Verzug',
      'expired_quote': 'Abgelaufenes Angebot',
      'remind_invoice': 'Rechnungserinnerung',
      'cvv': 'Kartenprüfziffer',
      'client_name': 'Kunde',
      'client_phone': 'Kunden Telefon',
      'required_fields': 'Benötigte Felder',
      'calculated_rate': 'Berechneter Satz',
      'default_task_rate': 'Standard-Steuersatz',
      'clear_cache': 'Zwischenspeicher leeren',
      'sort_order': 'Sortierreihenfolge',
      'task_status': 'Status',
      'task_statuses': 'Aufgaben Status',
      'new_task_status': 'Neuer Aufgaben Status',
      'edit_task_status': 'Aufgaben Status bearbeiten',
      'created_task_status': 'Aufgaben Status erfolgreich erstellt',
      'updated_task_status': 'Aufgabenstatus erfolgreich aktualisiert',
      'archived_task_status': 'Aufgaben Status erfolgreich archiviert',
      'deleted_task_status': 'Aufgaben Status erfolgreich gelöscht',
      'removed_task_status': 'Aufgaben Status erfolgreich entfernt',
      'restored_task_status': 'Aufgaben Status erfolgreich wiederhergestellt',
      'archived_task_statuses': ':value Aufgaben Stati erfolgreich archiviert',
      'deleted_task_statuses': ':value Aufgaben Stati erfolgreich gelöscht',
      'restored_task_statuses':
          ':value Aufgaben Stati erfolgreich wiederhergestellt',
      'search_task_status': 'Suche 1 Aufgaben Status',
      'search_task_statuses': 'Suche :count Aufgaben Status',
      'show_tasks_table': 'Zeige Aufgaben Tabelle',
      'show_tasks_table_help':
          'Beim Erstellen von Rechnungen immer die Aufgabenauswahl anzeigen',
      'invoice_task_timelog': 'Aufgaben Zeiterfassung in Rechnung stellen',
      'invoice_task_timelog_help':
          'Zeitdetails in der Rechnungsposition ausweisen',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Beginne Aufgabe vor dem Speichern',
      'configure_statuses': 'Stati bearbeiten',
      'task_settings': 'Aufgaben Einstellungen',
      'configure_categories': 'Kategorien bearbeiten',
      'expense_categories': 'Ausgabenkategorien',
      'new_expense_category': 'Neue Ausgabenkategorie',
      'edit_expense_category': 'Ausgaben Kategorie bearbeiten',
      'created_expense_category': 'Ausgabenkategorie erfolgreich erstellt',
      'updated_expense_category': 'Ausgabenkategorie erfolgreich aktualisiert',
      'archived_expense_category': 'Ausgabenkategorie erfolgreich archiviert',
      'deleted_expense_category': 'Kategorie erfolgreich gelöscht',
      'removed_expense_category': 'Ausgaben Kategorie erfolgreich entfernt',
      'restored_expense_category':
          'Ausgabenkategorie erfolgreich wiederhergestellt',
      'archived_expense_categories':
          ':count Ausgabenkategorien erfolgreich archiviert',
      'deleted_expense_categories':
          ':value Ausgabenkategorien erfolgreich gelöscht',
      'restored_expense_categories':
          ':value Ausgabenkategorien erfolgreich wiederhergestellt',
      'search_expense_category': 'Suche 1 Ausgabenkategorie',
      'search_expense_categories': 'Suche :count Ausgabenkategorie',
      'use_available_credits': 'Verfügbares Guthaben verwenden',
      'show_option': 'Zeige Option',
      'negative_payment_error':
          'Der Guthabenbetrag darf den Zahlungsbetrag nicht übersteigen',
      'view_changes': 'Änderungen anzeigen',
      'force_update': 'Aktualisierungen erzwingen',
      'force_update_help':
          'Du benutzt die aktuellste Version, aber es stehen noch ausstehende Fehlerbehebungen zur Verfügung',
      'mark_paid_help': 'Verfolge ob Ausgabe bezahlt wurde',
      'should_be_invoiced': 'Sollte in Rechnung gestellt werden',
      'should_be_invoiced_help':
          'Ermögliche diese Ausgabe in Rechnung zu stellen',
      'add_documents_to_invoice_help': 'Dokumente sichtbar machen',
      'convert_currency_help': 'Wechselkurs setzen',
      'expense_settings': 'Ausgaben Einstellungen',
      'clone_to_recurring': 'Duplizieren zu Widerkehrend',
      'crypto': 'Verschlüsselung',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'SOFORT-Überweisung',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'Benutzer Feld',
      'variables': 'Variablen',
      'show_password': 'Zeige Passwort',
      'hide_password': 'Verstecke Passwort',
      'copy_error': 'Kopier Fehler',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Automatische Rechnungsstellung aktivieren',
      'total_taxes': 'Gesamt Steuern',
      'line_taxes': 'Belegposition Steuer',
      'total_fields': 'Gesamt Felder',
      'stopped_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich gestoppt',
      'started_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich gestartet',
      'resumed_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich fortgesetzt',
      'gateway_refund': 'Zahlungsanbieter Rückerstattung',
      'gateway_refund_help':
          'Bearbeite die Rückerstattung über den Zahlungsanbieter',
      'due_date_days': 'Fälligkeitsdatum',
      'paused': 'Pausiert',
      'mark_active': 'Markiere aktiv',
      'day_count': 'Tag :count',
      'first_day_of_the_month': 'Erster Tag des Monats',
      'last_day_of_the_month': 'Letzter Tag des Monats',
      'use_payment_terms': 'Benutze Zahlungsbedingung',
      'endless': 'Endlos',
      'next_send_date': 'Nächstes Versanddatum',
      'remaining_cycles': 'Verbleibende Durchgänge',
      'recurring_invoice': 'Wiederkehrende Rechnung',
      'recurring_invoices': 'Wiederkehrende Rechnungen',
      'new_recurring_invoice': 'Neue wiederkehrende Rechnung',
      'edit_recurring_invoice': 'Bearbeite wiederkehrende Rechnung',
      'created_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich erstellt',
      'updated_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich aktualisiert',
      'archived_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich archiviert',
      'deleted_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich gelöscht',
      'removed_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich entfernt',
      'restored_recurring_invoice':
          'Wiederkehrende Rechnung erfolgreich wiederhergestellt',
      'archived_recurring_invoices':
          ':value Wiederkehrende Rechnung erfolgreich archiviert',
      'deleted_recurring_invoices':
          ':value Wiederkehrende Rechnungen erfolgreich gelöscht',
      'restored_recurring_invoices':
          ':value Wiederkehrende Rechnungen erfolgreich wiederhergestellt',
      'search_recurring_invoice': 'Suche 1 wiederkehrende Rechnung',
      'search_recurring_invoices': 'Suche :count Wiederkehrende Rechnungen',
      'send_date': 'Versanddatum',
      'auto_bill_on': 'Automatische Rechnungsstellung zum',
      'minimum_under_payment_amount': 'Minimaler Unterzahlungsbetrag',
      'profit': 'Profit',
      'line_item': 'Posten',
      'allow_over_payment': 'Überzahlung zulassen',
      'allow_over_payment_help':
          'Überzahlungen zulassen, beispielsweise Trinkgelder',
      'allow_under_payment': 'Unterzahlung zulassen',
      'allow_under_payment_help': 'Teilzahlungen zulassen',
      'test_mode': 'Test Modus',
      'opened': 'Geöffnet',
      'payment_reconciliation_failure': 'Fehler bei Kontenabstimmung',
      'payment_reconciliation_success': 'Kontenabstimmung erfolgreich',
      'gateway_success': 'Zahlungsanbieter erfolgreich',
      'gateway_failure': 'Zahlungsanbieter Fehler',
      'gateway_error': 'Zahlungsanbieter Fehler',
      'email_send': 'E-Mail gesendet',
      'email_retry_queue': 'E-Mail Wiederholungswarteschlange',
      'failure': 'Fehler',
      'quota_exceeded': 'Quota erreicht',
      'upstream_failure': 'Upstream Fehler',
      'system_logs': 'System-Log',
      'view_portal': 'Portal anzeigen',
      'copy_link': 'Link kopieren',
      'token_billing': 'Kreditkarte merken',
      'welcome_to_invoice_ninja': 'Willkommen bei Invoice Ninja',
      'always': 'Immer',
      'optin': 'Anmelden',
      'optout': 'Abmelden',
      'label': 'Label',
      'client_number': 'Kundennummer',
      'auto_convert': 'Automatisch konvertieren',
      'company_name': 'Firmenname',
      'reminder1_sent': 'Erste Erinnerung verschickt',
      'reminder2_sent': 'Zweite Erinnerung verschickt',
      'reminder3_sent': 'Dritte Erinnerung verschickt',
      'reminder_last_sent': 'Letzte Erinnerung verschickt',
      'pdf_page_info': 'Seite :current von :total',
      'emailed_invoices': 'Rechnungen erfolgreich versendet',
      'emailed_quotes': 'Angebote erfolgreich versendet',
      'emailed_credits': 'Guthaben erfolgreich per E-Mail versendet',
      'gateway': 'Provider',
      'view_in_stripe': 'In Stripe anzeigen',
      'rows_per_page': 'Einträge pro Seite',
      'hours': 'Stunden',
      'statement': 'Bericht',
      'taxes': 'Steuern',
      'surcharge': 'Gebühr',
      'apply_payment': 'Zahlungen anwenden',
      'apply': 'Anwenden',
      'unapplied': 'unangewendet',
      'select_label': 'Bezeichnung wählen',
      'custom_labels': 'Eigene Beschriftungen',
      'record_type': 'Eintragstyp',
      'record_name': 'Eintragsname',
      'file_type': 'Dateityp',
      'height': 'Höhe',
      'width': 'Breite',
      'to': 'An',
      'health_check': 'Systemprüfung',
      'payment_type_id': 'Zahlungsart',
      'last_login_at': 'Letzter Login',
      'company_key': 'Firmen Schlüssel',
      'storefront': 'Storefront',
      'storefront_help':
          'Drittanbieter Applikationen erlauben Rechnungen zu erstellen',
      'client_created': 'Kunde wurde erstellt',
      'online_payment_email': 'Online-Zahlung E-Mail Adresse',
      'manual_payment_email': 'manuelle Zahlung E-Mail Adresse',
      'completed': 'Abgeschlossen',
      'gross': 'Gesamtbetrag',
      'net_amount': 'Netto Betrag',
      'net_balance': 'Netto Betrag',
      'client_settings': 'Kundeneinstellungen',
      'selected_invoices': 'Ausgewählte Rechnungen',
      'selected_payments': 'Ausgewählte Zahlungen',
      'selected_quotes': 'Ausgewählte Angebote',
      'selected_tasks': 'Ausgewählte Aufgaben',
      'selected_expenses': 'Ausgewählte Ausgaben',
      'upcoming_invoices': 'Ausstehende Rechnungen',
      'past_due_invoices': 'Überfällige Rechnungen',
      'recent_payments': 'Kürzliche Zahlungen',
      'upcoming_quotes': 'Ausstehende Angebote',
      'expired_quotes': 'Abgelaufene Angebote',
      'create_client': 'Kunden erstellen',
      'create_invoice': 'Rechnung erstellen',
      'create_quote': 'Angebot erstellen',
      'create_payment': 'Zahlung erstellen',
      'create_vendor': 'Lieferanten erstellen',
      'update_quote': 'Angebot aktualisieren',
      'delete_quote': 'Angebot löschen',
      'update_invoice': 'Rechnung aktualisieren',
      'delete_invoice': 'Rechnung löschen',
      'update_client': 'Kunde aktualisieren',
      'delete_client': 'Kunde löschen',
      'delete_payment': 'Zahlung löschen',
      'update_vendor': 'Lieferant aktualisieren',
      'delete_vendor': 'Lieferant Löschen',
      'create_expense': 'Ausgabe erstellen',
      'update_expense': 'Ausgabe aktualisieren',
      'delete_expense': 'Ausgabe Löschen',
      'create_task': 'Aufgabe erstellen',
      'update_task': 'Aufgabe aktualisieren',
      'delete_task': 'Aufgabe löschen',
      'approve_quote': 'Angebot annehmen',
      'off': 'Aus',
      'when_paid': 'Bei Zahlung',
      'expires_on': 'Gültig bis',
      'free': 'Kostenlos',
      'plan': 'Plan',
      'show_sidebar': 'Zeige Seitenmenü',
      'hide_sidebar': 'Verstecke Seitenmenu',
      'event_type': 'Ereignistyp',
      'target_url': 'Ziel',
      'copy': 'kopieren',
      'must_be_online':
          'Bitte starten Sie die App sobald Sie mit dem Internet verbunden sind',
      'crons_not_enabled': 'Die Crons müssen aktiviert werden',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Suche :count Webhooks',
      'search_webhook': 'Suche 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Neuer Webhook',
      'edit_webhook': 'Webhook bearbeiten',
      'created_webhook': 'Webhook erfolgreich erstellt',
      'updated_webhook': 'Webhook erfolgreich aktualisiert',
      'archived_webhook': 'Webhook erfolgreich archiviert',
      'deleted_webhook': 'Webhook erfolgreich gelöscht',
      'removed_webhook': 'Webhook erfolgreich entfernt',
      'restored_webhook': 'Webhook erfolgreich wiederhergestellt',
      'archived_webhooks': ':value Webhooks erfolgreich archiviert',
      'deleted_webhooks': ':value Webhooks erfolgreich gelöscht',
      'removed_webhooks': ':value Webhooks erfolgreich entfernt',
      'restored_webhooks': ':value Webhooks erfolgreich wiederhergestellt',
      'api_tokens': 'API Token',
      'api_docs': 'API Doku',
      'search_tokens': 'Suche :count Token',
      'search_token': 'Suche 1 Token',
      'token': 'Token',
      'tokens': 'Token',
      'new_token': 'Neues Token',
      'edit_token': 'Token bearbeiten',
      'created_token': 'Token erfolgreich erstellt',
      'updated_token': 'Token erfolgreich aktualisiert',
      'archived_token': 'Token erfolgreich archiviert',
      'deleted_token': 'Token erfolgreich gelöscht',
      'removed_token': 'Token erfolgreich entfernt',
      'restored_token': 'Token erfolgreich wiederhergestellt',
      'archived_tokens': ':count Token erfolgreich archiviert',
      'deleted_tokens': ':count Token erfolgreich gelöscht',
      'restored_tokens': ':value Token erfolgreich wiederhergestellt',
      'client_registration': 'Kunden Registration',
      'client_registration_help':
          'Den Kunden ermöglichen, sich selbst im Portal zu registrieren.',
      'customize_and_preview': 'Anpassung und Vorschau',
      'email_invoice': 'Rechnung versenden',
      'email_quote': 'Angebot per E-Mail senden',
      'email_credit': 'Guthaben per E-Mail versenden',
      'email_payment': 'Sende Zahlungs eMail',
      'client_email_not_set':
          'Es wurde noch keine E-Mail Adresse beim Kunden eingetragen.',
      'ledger': 'Hauptbuch',
      'view_pdf': 'Zeige PDF',
      'all_records': 'Alle Einträge',
      'owned_by_user': 'Eigentümer',
      'credit_remaining': 'Verbleibendes Guthaben',
      'contact_name': 'Name des Kontakts',
      'use_default': 'Benutze Standardwert',
      'reminder_endless': 'Endlose Reminder',
      'number_of_days': 'Anzahl Tage',
      'configure_payment_terms': 'Zahlungsbedingungen bearbeiten',
      'payment_term': 'Zahlungsbedingung',
      'new_payment_term': 'Neue Zahlungsbedingung',
      'edit_payment_term': 'Bearbeite Zahlungsbedingungen',
      'created_payment_term': 'Zahlungsbedingung erfolgreich erstellt',
      'updated_payment_term': 'Zahlungsbedingung erfolgreich aktualisiert',
      'archived_payment_term': 'Zahlungsbedingung erfolgreich archiviert',
      'deleted_payment_term': 'Zahlungsbedingung erfolgreich gelöscht',
      'removed_payment_term': 'Zahlungsbedingung erfolgreich entfernt',
      'restored_payment_term':
          'Zahlungsbedingungen erfolgreich wiederhergestellt',
      'archived_payment_terms':
          ':value Zahlungsbedingungen erfolgreich archiviert',
      'deleted_payment_terms':
          ':value Zahlungsbedingungen erfolgreich gelöscht',
      'restored_payment_terms':
          ':value Zahlungsbedingungen erfolgreich wiederhergestellt',
      'email_sign_in': 'Mit E-Mail anmelden',
      'change': 'Ändern',
      'change_to_mobile_layout': 'Möchten Sie zur mobilen Ansicht wechseln?',
      'change_to_desktop_layout': 'Möchten Sie zur Desktopansicht wechseln?',
      'send_from_gmail': 'Mit Gmail versenden',
      'reversed': 'Umgekehrt',
      'cancelled': 'Storniert',
      'credit_amount': 'Guthabenbetrag',
      'quote_amount': 'Angebotsbetrag',
      'hosted': 'Gehostet',
      'selfhosted': 'Selbstgehostet',
      'exclusive': 'Exklusive',
      'inclusive': 'Inklusive',
      'hide_menu': 'Menü ausblenden',
      'show_menu': 'Menü einblenden',
      'partially_refunded': 'Teilweise erstattet',
      'search_documents': 'Suche nach Dokumenten',
      'search_designs': 'Suche nach Designs',
      'search_invoices': 'Suche Rechnungen',
      'search_clients': 'Suche Kunden',
      'search_products': 'Suche Produkte',
      'search_quotes': 'Suche Angebote',
      'search_credits': 'Suche Guthaben',
      'search_vendors': 'Suche Lieferanten',
      'search_users': 'Suche Benutzer',
      'search_tax_rates': 'Suche Steuersatz',
      'search_tasks': 'Suche Aufgaben',
      'search_settings': 'Suche Einstellungen',
      'search_projects': 'Suche nach Projekten',
      'search_expenses': 'Suche Ausgaben',
      'search_payments': 'Suche Zahlungen',
      'search_groups': 'Suche nach Gruppen',
      'search_company': 'Suche Firma',
      'search_document': 'Suche 1 Dokument',
      'search_design': 'Suche 1 Design',
      'search_invoice': 'Suche 1 Angebot',
      'search_client': 'Suche 1 Kunden',
      'search_product': 'Suche 1 Produkt',
      'search_quote': 'Suche 1 Angebot',
      'search_credit': 'Suche 1 Guthaben',
      'search_vendor': 'Suche 1 Hersteller',
      'search_user': 'Suche 1 Benutzer',
      'search_tax_rate': 'Suche 1 Steuersatz',
      'search_task': 'Suche 1 Aufgabe',
      'search_project': 'Suche 1 Projekt',
      'search_expense': 'Suche 1 Ausgabe',
      'search_payment': 'Suche 1 Zahlung',
      'search_group': 'Suche 1 Gruppen',
      'refund_payment': 'Zahlung erstatten',
      'cancelled_invoice': 'Rechnung erfolgreich storniert',
      'cancelled_invoices': 'Rechnungen erfolgreich storniert',
      'reversed_invoice': 'Rechnung erfolgreich zurückgebucht',
      'reversed_invoices': 'Rechnungen erfolgreich zurückgebucht',
      'reverse': 'Rückbuchung',
      'full_name': 'Voller Name',
      'city_state_postal': 'Stadt / Bundesland / PLZ',
      'postal_city_state': 'Plz/Stadt/Staat',
      'custom1': 'Benutzerdefiniert 1',
      'custom2': 'Benutzerdefiniert 2',
      'custom3': 'Benutzerdefiniert 3',
      'custom4': 'Benutzerdefiniert 3',
      'optional': 'optional',
      'license': 'Lizenz',
      'purge_data': 'Daten säubern',
      'purge_successful': 'Die Kontodaten wurden erfolgreich gelöscht',
      'purge_data_message':
          'Achtung: Alle Daten werden vollständig gelöscht. Dieser Vorgang kann nicht rückgängig gemacht werden.',
      'invoice_balance': 'Rechnungssaldo',
      'age_group_0': '0 - 30 Tage',
      'age_group_30': '30 - 60 Tage',
      'age_group_60': '60 - 90 Tage',
      'age_group_90': '90 - 120 Tage',
      'age_group_120': '120+ Tage',
      'refresh': 'Aktualisieren',
      'saved_design': 'Design erfolgreich gespeichert',
      'client_details': 'Kundeninformationen',
      'company_address': 'Firmenadresse',
      'invoice_details': 'Rechnungsdetails',
      'quote_details': 'Kostenvoranschlag-Details',
      'credit_details': 'Gutschrift Details',
      'product_columns': 'Produktspalten',
      'task_columns': 'Aufgabenspalten',
      'add_field': 'Feld hinzufügen',
      'all_events': 'Alle Ereignisse',
      'permissions': 'Berechtigungen',
      'none': 'Nichts',
      'owned': 'Eigentümer',
      'payment_success': 'Bezahlung erfolgreich',
      'payment_failure': 'Bezahlung fehlgeschlagen',
      'invoice_sent': ':count Rechnung versendet',
      'quote_sent': 'Kostenvoranschlag versendet',
      'credit_sent': 'Guthaben gesendet',
      'invoice_viewed': 'Rechnung angesehen',
      'quote_viewed': 'Kostenvoranschlag angesehen',
      'credit_viewed': 'Guthaben angesehen',
      'quote_approved': 'Kostenvoranschlag angenommen',
      'receive_all_notifications': 'Empfange alle Benachrichtigungen',
      'purchase_license': 'Lizenz kaufen',
      'apply_license': 'Lizenz anwenden',
      'cancel_account': 'Konto kündigen',
      'cancel_account_message':
          'Warnung: Diese Aktion wird dein Konto unwiderruflich löschen.',
      'delete_company': 'Firma löschen',
      'delete_company_message':
          'Achtung: Dadurch wird Ihre Firma unwiderruflich gelöscht. Es gibt kein Zurück.',
      'enabled_modules': 'Module aktivieren',
      'converted_quote': 'Angebot erfolgreichen konvertiert',
      'credit_design': 'Guthaben Design',
      'includes': 'Beinhaltet',
      'header': 'Kopf',
      'load_design': 'Designvorlage laden',
      'css_framework': 'CSS-Framework',
      'custom_designs': 'Benutzerdefinierte Designs',
      'designs': 'Designs',
      'new_design': 'Neues Design',
      'edit_design': 'Design bearbeiten',
      'created_design': 'Design erfolgreich erstellt',
      'updated_design': 'Design erfolgreich aktualisiert',
      'archived_design': 'Design erfolgreich archiviert',
      'deleted_design': 'Design erfolgreich gelöscht',
      'removed_design': 'Design erfolgreich entfernt',
      'restored_design': 'Design erfolgreich wiederhergestellt',
      'archived_designs': ':value Designs erfolgreich archiviert',
      'deleted_designs': ':value Designs erfolgreich gelöscht',
      'restored_designs': ':value Designs erfolgreich wiederhergestellt',
      'proposals': 'Vorschläge',
      'tickets': 'Tickets',
      'recurring_quotes': 'Wiederkehrende Angebote',
      'recurring_tasks': 'Wiederkehrende Aufgabe',
      'recurring_expenses': 'Wiederkehrende Ausgaben',
      'account_management': 'Kontoverwaltung',
      'credit_date': 'Guthabendatum',
      'credit': 'Gutschrift',
      'credits': 'Guthaben',
      'new_credit': 'Guthaben eingeben',
      'edit_credit': 'Saldo bearbeiten',
      'created_credit': 'Guthaben erfolgreich erstellt',
      'updated_credit': 'Saldo erfolgreich aktualisiert',
      'archived_credit': 'Guthaben erfolgreich archiviert',
      'deleted_credit': 'Guthaben erfolgreich gelöscht',
      'removed_credit': 'Guthaben erfolgreich entfernt',
      'restored_credit': 'Guthaben erfolgreich wiederhergestellt',
      'archived_credits': ':count Guthaben erfolgreich archiviert',
      'deleted_credits': ':count Guthaben erfolgreich gelöscht',
      'restored_credits': ':value Guthaben erfolgreich archiviert',
      'current_version': 'Aktuelle Version',
      'latest_version': 'Neueste Version',
      'update_now': 'Jetzt aktualisieren',
      'a_new_version_is_available':
          'Eine neue Version der Webapp ist verfügbar.',
      'update_available': 'Update verfügbar',
      'app_updated': 'Update erfolgreich',
      'learn_more': 'Mehr erfahren',
      'integrations': 'Integrationen',
      'tracking_id': 'Sendungsnummer',
      'slack_webhook_url': 'Slack-Webhook-URL',
      'credit_footer': 'Guthaben-Fußzeile',
      'credit_terms': 'Gutschrift Bedingungen',
      'new_company': 'Neues Konto',
      'added_company': 'Erfolgreich Firma hinzugefügt',
      'company1': 'Benutzerdefinierte Firma 1',
      'company2': 'Benutzerdefinierte Firma 2',
      'company3': 'Benutzerdefinierte Firma 3',
      'company4': 'Benutzerdefinierte Firma 4',
      'product1': 'Benutzerdefiniertes Produkt 1',
      'product2': 'Benutzerdefiniertes Produkt 2',
      'product3': 'Benutzerdefiniertes Produkt 3',
      'product4': 'Benutzerdefiniertes Produkt 4',
      'client1': 'Benutzerdefinierter Kunde 1',
      'client2': 'Benutzerdefinierter Kunde 2',
      'client3': 'Benutzerdefinierter Kunde 3',
      'client4': 'Benutzerdefinierter Kunde 4',
      'contact1': 'Benutzerdefinierter Kontakt 1',
      'contact2': 'Benutzerdefinierter Kontakt 2',
      'contact3': 'Benutzerdefinierter Kontakt 3',
      'contact4': 'Benutzerdefinierter Kontakt 4',
      'task1': 'Benutzerdefinierte Aufgabe 1',
      'task2': 'Benutzerdefinierte Aufgabe 2',
      'task3': 'Benutzerdefinierte Aufgabe 3',
      'task4': 'Benutzerdefinierte Aufgabe 4',
      'project1': 'Benutzerdefiniertes Projekt 1',
      'project2': 'Benutzerdefiniertes Projekt 2',
      'project3': 'Benutzerdefiniertes Projekt 3',
      'project4': 'Benutzerdefiniertes Projekt 4',
      'expense1': 'Benutzerdefinierte Ausgabe 1',
      'expense2': 'Benutzerdefinierte Ausgabe 2',
      'expense3': 'Benutzerdefinierte Ausgabe 3',
      'expense4': 'Benutzerdefinierte Ausgabe 4',
      'vendor1': 'Benutzerdefinierter Lieferant 1',
      'vendor2': 'Benutzerdefinierter Lieferant 2',
      'vendor3': 'Benutzerdefinierter Lieferant 3',
      'vendor4': 'Benutzerdefinierter Lieferant 4',
      'invoice1': 'Benutzerdefinierte Rechnung 1',
      'invoice2': 'Benutzerdefinierte Rechnung 2',
      'invoice3': 'Benutzerdefinierte Rechnung 3',
      'invoice4': 'Benutzerdefinierte Rechnung 4',
      'payment1': 'Benutzerdefinierte Zahlung 1',
      'payment2': 'Benutzerdefinierte Zahlung 2',
      'payment3': 'Benutzerdefinierte Zahlung 3',
      'payment4': 'Benutzerdefinierte Zahlung 4',
      'surcharge1': 'Benutzerdefinierter Zuschlag 1',
      'surcharge2': 'Benutzerdefinierter Zuschlag 2',
      'surcharge3': 'Benutzerdefinierter Zuschlag 3',
      'surcharge4': 'Benutzerdefinierter Zuschlag 4',
      'group1': 'Benutzerdefinierte Gruppe 1',
      'group2': 'Benutzerdefinierte Gruppe 2',
      'group3': 'Benutzerdefinierte Gruppe 3',
      'group4': 'Benutzerdefinierte Gruppe 4',
      'reset': 'Zurücksetzen',
      'number': 'Nummer',
      'export': 'Exportieren',
      'chart': 'Diagramm',
      'count': 'Anzahl',
      'totals': 'Summe',
      'blank': 'Leer',
      'day': 'Tag',
      'month': 'Monat',
      'year': 'Jahr',
      'subgroup': 'Untergruppe',
      'is_active': 'Ist aktiv',
      'group_by': 'Gruppieren nach',
      'credit_balance': 'Guthabenstand',
      'contact_last_login': 'Letzter Login des Kontakts',
      'contact_full_name': 'Vollständiger Name des Kontakts',
      'contact_phone': 'Telefonnummer des Kontakts',
      'contact_custom_value1': 'Kontakt Benutzerdefinierter Wert 1',
      'contact_custom_value2': 'Kontakt Benutzerdefinierter Wert 2',
      'contact_custom_value3': 'Kontakt Benutzerdefinierter Wert 3',
      'contact_custom_value4': 'Kontakt Benutzerdefinierter Wert 4',
      'shipping_address1': 'Strasse Versandanschrift',
      'shipping_address2': 'Versand Adresszusatz',
      'shipping_city': 'Stadt Versandanschrift',
      'shipping_state': 'Versand Bundesland',
      'shipping_postal_code': 'Postleitzahl Versandanschrift',
      'shipping_country': 'Lieferungsland',
      'billing_address1': 'Strasse Rechnungsanschrift',
      'billing_address2': 'Rechnung Adresszusatz',
      'billing_city': 'Stadt Rechnungsanschrift',
      'billing_state': 'Rechnung Bundesland',
      'billing_postal_code': 'Postleitzahl Rechnungsanschrift',
      'billing_country': 'Rechnungsland',
      'client_id': 'Kundennummer',
      'assigned_to': 'Zugewiesen an',
      'created_by': 'Erstellt von :name',
      'assigned_to_id': 'Zugewiesen zur ID',
      'created_by_id': 'Erstellt von ID',
      'add_column': 'Spalte hinzufügen',
      'edit_columns': 'Spalten bearbeiten',
      'columns': 'Spalten',
      'aging': 'Versendet',
      'profit_and_loss': 'Gewinn und Verlust',
      'reports': 'Berichte',
      'report': 'Bericht',
      'add_company': 'Konto hinzufügen',
      'unpaid_invoice': 'Unbezahlte Rechnung',
      'paid_invoice': 'Bezahlte Rechnung',
      'unapproved_quote': 'Nicht genehmigtes Angebot',
      'help': 'Hilfe',
      'refund': 'Erstattung',
      'refund_date': 'Erstattungsdatum',
      'filtered_by': 'Gefiltert nach',
      'contact_email': 'E-Mail-Adresse des Kontakts',
      'multiselect': 'Mehrfachauswahl',
      'entity_state': 'Status',
      'verify_password': 'Passwort überprüfen',
      'applied': 'Angewendet',
      'include_recent_errors': 'Kürzliche Fehler aus den Logs einfügen',
      'your_message_has_been_received':
          'Wir haben ihre Nachricht erhalten und bemühen uns schnellstmöglich zu antworten.',
      'message': 'Nachricht',
      'from': 'Von',
      'show_product_details': 'Produktdetails anzeigen',
      'show_product_details_help':
          'Beschreibung und Kosten in die Produkt-Dropdown-Liste einfügen',
      'pdf_min_requirements': 'Der PDF-Renderer benötigt :version',
      'adjust_fee_percent': 'Anpassungszuschlag Prozent',
      'adjust_fee_percent_help': 'Gebühren Prozentsatz an das Konto anpassen',
      'configure_settings': 'Einstellungen bearbeiten',
      'support_forum': 'Support-Forum',
      'about': 'Über',
      'documentation': 'Dokumentation',
      'contact_us': 'Kontaktieren Sie uns',
      'subtotal': 'Zwischensumme',
      'line_total': 'Summe',
      'item': 'Artikel',
      'credit_email': 'Guthaben E-Mail',
      'iframe_url': 'Webseite',
      'domain_url': 'Domain-URL',
      'password_is_too_short': 'Das Passwort ist zu kurz',
      'password_is_too_easy':
          'Das Passwort muss einen Großbuchstaben und eine Nummer enthalten',
      'client_portal_tasks': 'Kundenportal-Aufgaben',
      'client_portal_dashboard': 'Kundenportal-Übersicht',
      'please_enter_a_value': 'Bitte einen Wert eingeben',
      'deleted_logo': 'Logo erfolgreich gelöscht',
      'yes': 'Ja',
      'no': 'Nein',
      'generate_number': 'Nummer generieren',
      'when_saved': 'Wenn gespeichert',
      'when_sent': 'Wenn gesendet',
      'select_company': 'Firma auswählen',
      'float': 'Schwebend',
      'collapse': 'Einklappen',
      'show_or_hide': 'Anzeigen/verstecken',
      'menu_sidebar': 'Menüleiste',
      'history_sidebar': 'Verlaufs-Seitenleiste',
      'tablet': 'Tablet',
      'mobile': 'Mobil',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Ansehen',
      'module': 'Modul',
      'first_custom': 'Erste benutzerdefinierte',
      'second_custom': 'Zweite benutzerdefinierte',
      'third_custom': 'Dritte benutzerdefinierte',
      'show_cost': 'Kosten anzeigen',
      'show_product_cost': 'Produktkosten anzeigen',
      'show_cost_help':
          'Feld für Einkaufspreis anzeigen, um Gewinnspanne zu verfolgen',
      'show_product_quantity': 'Produktanzahl anzeigen',
      'show_product_quantity_help':
          'Zeigen ein Mengenangabe Feld, sonst den Standardwert 1',
      'show_invoice_quantity': 'Rechnungsanzahl anzeigen',
      'show_invoice_quantity_help':
          'Zeige ein Rechnungsposten Anzahlfeld, sonst den Standardwert 1',
      'show_product_discount': 'Produktermäßigung anzeigen',
      'show_product_discount_help': 'Zeige Rabattfeld in Belegposition',
      'default_quantity': 'Standardanzahl',
      'default_quantity_help':
          'Setze den Rechnungsposten automatisch auf Anzahl 1',
      'one_tax_rate': 'Ein Steuersatz',
      'two_tax_rates': 'Zwei Steuersätze',
      'three_tax_rates': 'Drei Steuersätze',
      'default_tax_rate': 'Standard-Steuersatz',
      'user': 'Benutzer',
      'invoice_tax': 'Rechnungssteuer',
      'line_item_tax': 'Belegposition Steuer',
      'inclusive_taxes': 'Inklusive Steuern',
      'invoice_tax_rates': 'Rechnungs-Steuersätze',
      'item_tax_rates': 'Element-Steuersätze',
      'no_client_selected': 'Bitte wählen Sie einen Kunden',
      'configure_rates': 'Steuersätze bearbeiten',
      'configure_gateways': 'Zahlungsanbieter bearbeiten',
      'tax_settings': 'Steuer-Einstellungen',
      'tax_settings_rates': 'Steuersätze',
      'accent_color': 'Akzent-Farbe',
      'switch': 'Switch',
      'comma_sparated_list': 'Komma-separierte Liste',
      'options': 'Optionen',
      'single_line_text': 'Einzeiliger Text',
      'multi_line_text': 'Mehrzeiliger Text',
      'dropdown': 'Dropdown',
      'field_type': 'Feldtyp',
      'recover_password_email_sent':
          'Eine Passwort-Wiederherstellungs-Mail wurde versendet',
      'submit': 'Abschicken',
      'recover_password': 'Passwort wiederherstellen',
      'late_fees': 'Verspätungszuschläge',
      'credit_number': 'Gutschriftnummer',
      'payment_number': 'Zahlungsnummer',
      'late_fee_amount': 'Höhe des Verspätungszuschlags',
      'late_fee_percent': 'Verspätungszuschlag Prozent',
      'schedule': 'Zeitgesteuert',
      'before_due_date': 'Vor dem Fälligkeitsdatum',
      'after_due_date': 'Nach dem Fälligkeitsdatum',
      'after_invoice_date': 'Nach dem Rechnungsdatum',
      'days': 'Tage',
      'invoice_email': 'Rechnungsmail',
      'payment_email': 'Zahlungsmail',
      'partial_payment': 'Teilzahlung',
      'payment_partial': 'Teilzahlung',
      'partial_payment_email': 'Teilzahlungsmail',
      'quote_email': 'Angebotsmail',
      'endless_reminder': 'Endlose Erinnnerung',
      'filtered_by_user': 'Gefiltert nach Benutzer',
      'administrator': 'Administrator',
      'administrator_help':
          'Dem Benutzer erlauben, andere Benutzer zu administrieren, Einstellungen zu ändern und alle Einträge zu bearbeiten',
      'user_management': 'Benutzerverwaltung',
      'users': 'Benutzer',
      'new_user': 'Neuer Benutzer',
      'edit_user': 'Benutzer bearbeiten',
      'created_user': 'Benutzer erfolgreich erstellt',
      'updated_user': 'Benutzer erfolgreich aktualisiert',
      'archived_user': 'Benutzer erfolgreich archiviert',
      'deleted_user': 'Benutzer erfolgreich gelöscht',
      'removed_user': 'Benutzer erfolgreich entfernt',
      'restored_user': 'Benutzer erfolgreich wiederhergestellt',
      'archived_users': ':value Benutzer erfolgreich archiviert',
      'deleted_users': ':value Benutzer erfolgreich gelöscht',
      'removed_users': ':value Benutzer erfolgreich entfernt',
      'restored_users': ':value Benutzer erfolgreich wiederhergestellt',
      'general_settings': 'Allgemeine Einstellungen',
      'invoice_options': 'Rechnungsoptionen',
      'hide_paid_to_date': '\"Bereits gezahlt\" ausblenden',
      'hide_paid_to_date_help':
          '\"Bereits gezahlt\" nur anzeigen, wenn eine Zahlung eingegangen ist.',
      'invoice_embed_documents': 'Dokumente einbetten',
      'invoice_embed_documents_help':
          'Bildanhänge zu den Rechnungen hinzufügen.',
      'all_pages_header': 'Zeige Kopf auf',
      'all_pages_footer': 'Zeige Fußzeilen auf',
      'first_page': 'Erste Seite',
      'all_pages': 'Alle Seiten',
      'last_page': 'Letzte Seite',
      'primary_font': 'Primäre Schriftart',
      'secondary_font': 'Sekundäre Schriftart',
      'primary_color': 'Primäre Farbe',
      'secondary_color': 'Sekundäre Farbe',
      'page_size': 'Seitengröße',
      'font_size': 'Schriftgröße',
      'quote_design': 'Angebots-Layout',
      'invoice_fields': 'Rechnungsfelder',
      'product_fields': 'Produktfelder',
      'invoice_terms': 'Rechnungsbedingungen',
      'invoice_footer': 'Rechnungsfußzeile',
      'quote_terms': 'Angebotsbedingungen',
      'quote_footer': 'Angebots-Fußzeile',
      'auto_email_invoice': 'Automatische Email',
      'auto_email_invoice_help':
          'Senden Sie wiederkehrende Rechnungen automatisch per E-Mail, wenn sie erstellt werden.',
      'auto_archive_invoice': 'Automatisches Archiv',
      'auto_archive_invoice_help':
          'Archivieren Sie Rechnungen automatisch, wenn sie bezahlt sind.',
      'auto_archive_quote': 'Automatisches Archiv',
      'auto_archive_quote_help':
          'Archivieren Sie Angebote automatisch, wenn sie konvertiert werden.',
      'auto_convert_quote': 'Automatisch konvertieren',
      'auto_convert_quote_help':
          'Das Angebot automatisch in eine Rechnung umwandeln wenn es vom Kunden angenommen wird.',
      'workflow_settings': 'Workflow Einstellungen',
      'freq_daily': 'Täglich',
      'freq_weekly': 'Wöchentlich',
      'freq_two_weeks': 'Zweiwöchentlich',
      'freq_four_weeks': 'Vierwöchentlich',
      'freq_monthly': 'Monatlich',
      'freq_two_months': 'Zwei Monate',
      'freq_three_months': 'Dreimonatlich',
      'freq_four_months': 'Vier Monate',
      'freq_six_months': 'Halbjährlich',
      'freq_annually': 'Jährlich',
      'freq_two_years': 'Zwei Jahre',
      'freq_three_years': 'Drei Jahre',
      'never': 'Niemals',
      'company': 'Firma',
      'generated_numbers': 'Generierte Nummern',
      'charge_taxes': 'Steuern erheben',
      'next_reset': 'Nächster Reset',
      'reset_counter': 'Zähler-Reset',
      'recurring_prefix': 'Wiederkehrender Präfix',
      'number_padding': 'Nummernabstand',
      'general': 'Allgemein',
      'surcharge_field': 'Zuschlagsfeld',
      'company_field': 'Firmenfeld',
      'company_value': 'Firmenwert',
      'credit_field': 'Kredit-Feld',
      'invoice_field': 'Rechnungsfeld',
      'invoice_surcharge': 'Rechnungsgebühr',
      'client_field': 'Kundenfeld',
      'product_field': 'Produktfeld',
      'payment_field': 'Zahlungs-Feld',
      'contact_field': 'Kontaktfeld',
      'vendor_field': 'Lieferantenfeld',
      'expense_field': 'Ausgabenfeld',
      'project_field': 'Projektfeld',
      'task_field': 'Aufgabenfeld',
      'group_field': 'Gruppen-Feld',
      'number_counter': 'Nummernzähler',
      'prefix': 'Präfix',
      'number_pattern': 'Nummernschema',
      'messages': 'Nachrichten',
      'custom_css': 'Benutzerdefiniertes CSS',
      'custom_javascript': 'Benutzerdefiniertes JavaScript',
      'signature_on_pdf': 'Auf PDF anzeigen',
      'signature_on_pdf_help':
          'Unterschrift des Kunden auf dem Angebots/Rechnungs PDF anzeigen.',
      'show_accept_invoice_terms': 'Checkbox für Rechnungsbedingungen',
      'show_accept_invoice_terms_help':
          'Erfordern Sie die Bestätigung der Rechnungsbedingungen durch den Kunden.',
      'show_accept_quote_terms': 'Checkbox für Angebotsbedingungen',
      'show_accept_quote_terms_help':
          'Erfordern Sie die Bestätigung der Angebotsbedingungen durch den Kunden.',
      'require_invoice_signature': 'Rechnungsunterschrift',
      'require_invoice_signature_help':
          'Erfordern Sie die Unterschrift des Kunden bei Rechnungen.',
      'require_quote_signature': 'Angebotsunterschrift',
      'enable_portal_password': 'Rechnungen mit Passwort schützen',
      'enable_portal_password_help':
          'Erlaubt Ihnen ein Passwort für jeden Kontakt zu erstellen. Wenn ein Passwort erstellt wurde, muss der Kunde dieses eingeben, bevor er eine Rechnung ansehen darf.',
      'authorization': 'Genehmigung',
      'subdomain': 'Subdomäne',
      'domain': 'Domäne',
      'portal_mode': 'Portalmodus',
      'email_signature': 'Mit freundlichen Grüßen,',
      'enable_email_markup_help':
          'Machen Sie es einfacher für Ihre Kunden zu bezahlen, indem Sie schema.org Markup zu Ihren E-Mails hinzufügen.',
      'plain': 'Einfach',
      'light': 'Hell',
      'dark': 'Dunkel',
      'email_design': 'E-Mail-Design',
      'attach_pdf': 'PDF anhängen',
      'attach_documents': 'Dokumente anhängen',
      'attach_ubl': 'UBL anhängen',
      'email_style': 'E-Mail-Stil',
      'enable_email_markup': 'Markup erlauben',
      'reply_to_email': 'Antwort-E-Mail-Adresse',
      'reply_to_name': 'Name der Antwortadresse',
      'bcc_email': 'BCC E-Mail',
      'processed': 'Verarbeitet',
      'credit_card': 'Kreditkarte',
      'bank_transfer': 'Überweisung',
      'priority': 'Priorität',
      'fee_amount': 'Zuschlag Betrag',
      'fee_percent': 'Zuschlag Prozent',
      'fee_cap': 'Gebührenobergrenze',
      'limits_and_fees': 'Grenzwerte/Gebühren',
      'enable_min': 'Min aktivieren',
      'enable_max': 'Max aktivieren',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Logos der akzeptierten Kreditkarten',
      'credentials': 'Zugangsdaten',
      'update_address': 'Adresse aktualisieren',
      'update_address_help':
          'Kundenadresse mit den gemachten Angaben aktualisieren',
      'rate': 'Satz',
      'tax_rate': 'Steuersatz',
      'new_tax_rate': 'Neuer Steuersatz',
      'edit_tax_rate': 'Steuersatz bearbeiten',
      'created_tax_rate': 'Steuersatz erstellt',
      'updated_tax_rate': 'Steuersatz aktualisiert',
      'archived_tax_rate': 'Steuersatz archiviert',
      'deleted_tax_rate': 'Steuersatz erfolgreich gelöscht',
      'restored_tax_rate': 'Steuersatz erfolgreich wiederhergestellt',
      'archived_tax_rates': ':value Steuersätze erfolgreich archiviert',
      'deleted_tax_rates': ':value Steuersätze erfolgreich gelöscht',
      'restored_tax_rates': ':value Steuersätze erfolgreich wiederhergestellt',
      'fill_products': 'Produkte automatisch ausfüllen',
      'fill_products_help':
          'Beim Auswählen eines Produktes werden automatisch Beschreibung und Kosten ausgefüllt',
      'update_products': 'Produkte automatisch aktualisieren',
      'update_products_help':
          'Beim Aktualisieren einer Rechnung werden die Produkte automatisch aktualisiert',
      'convert_products': 'Produkte konvertieren',
      'convert_products_help':
          'Produktpreise automatisch in die Währung des Kunden konvertieren',
      'fees': 'Gebühren',
      'limits': 'Grenzwerte',
      'provider': 'Anbieter',
      'company_gateway': 'Zahlungs-Gateway',
      'company_gateways': 'Zahlungs-Gateways',
      'new_company_gateway': 'Neues Gateway',
      'edit_company_gateway': 'Gateway bearbeiten',
      'created_company_gateway': 'Gateway erfolgreich erstellt',
      'updated_company_gateway': 'Gateway erfolgreich aktualisiert',
      'archived_company_gateway': 'Gateway erfolgreich archiviert',
      'deleted_company_gateway': 'Gateway erfolgreich gelöscht',
      'restored_company_gateway': 'Gateway erfolgreich wiederhergestellt',
      'archived_company_gateways':
          ':value Zahlungsanbieter erfolgreich archiviert',
      'deleted_company_gateways':
          ':value Zahlungsanbieter erfolgreich gelöscht',
      'restored_company_gateways':
          ':value Zahlungsanbieter erfolgreich wiederhergestellt',
      'continue_editing': 'Weiterbearbeiten',
      'discard_changes': 'Änderungen verwerfen',
      'default_value': 'Standardwert',
      'disabled': 'Deaktiviert',
      'currency_format': 'Währungsformat',
      'first_day_of_the_week': 'Erster Tag der Woche',
      'first_month_of_the_year': 'Erster Monat des Jahres',
      'sunday': 'Sonntag',
      'monday': 'Montag',
      'tuesday': 'Dienstag',
      'wednesday': 'Mittwoch',
      'thursday': 'Donnerstag',
      'friday': 'Freitag',
      'saturday': 'Samstag',
      'january': 'Januar',
      'february': 'Februar',
      'march': 'März',
      'april': 'April',
      'may': 'Mai',
      'june': 'Juni',
      'july': 'Juli',
      'august': 'August',
      'september': 'September',
      'october': 'Oktober',
      'november': 'November',
      'december': 'Dezember',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Datumsformat',
      'datetime_format': 'Datums-/Zeitformat',
      'military_time': '24-Stunden-Zeit',
      'military_time_help': '24-Stunden-Anzeige',
      'send_reminders': 'Erinnerungen senden',
      'timezone': 'Zeitzone',
      'filtered_by_project': 'Nach Projekt filtern',
      'filtered_by_group': 'Gefiltert nach Gruppe',
      'filtered_by_invoice': 'Gefiltert nach Rechnung',
      'filtered_by_client': 'Gefiltert nach Kunde',
      'filtered_by_vendor': 'Gefiltert nach Lieferant',
      'group_settings': 'Gruppeneinstellungen',
      'group': 'Gruppe',
      'groups': 'Gruppen',
      'new_group': 'Neue Gruppe',
      'edit_group': 'Gruppe bearbeiten',
      'created_group': 'Gruppe erfolgreich erstellt',
      'updated_group': 'Gruppe erfolgreich aktualisiert',
      'archived_groups': ':value Gruppen erfolgreich archiviert',
      'deleted_groups': ':value Gruppen erfolgreich gelöscht',
      'restored_groups': ':value Gruppen erfolgreich wiederhergestellt',
      'upload_logo': 'Logo hochladen',
      'uploaded_logo': 'Logo erfolgreich hochgeladen',
      'logo': 'Logo',
      'saved_settings': 'Einstellungen erfolgreich gespeichert',
      'product_settings': 'Produkt-Einstellungen',
      'device_settings': 'Geräteeinstellungen',
      'defaults': 'Standards',
      'basic_settings': 'Allgemeine Einstellungen',
      'advanced_settings': 'Erweiterte Einstellungen',
      'company_details': 'Firmendaten',
      'user_details': 'Benutzerdaten',
      'localization': 'Lokalisierung',
      'online_payments': 'Online-Zahlungen',
      'tax_rates': 'Steuersätze',
      'notifications': 'Benachrichtigungen',
      'import_export': 'Import/Export',
      'custom_fields': 'Benutzerdefinierte Felder',
      'invoice_design': 'Rechnungsdesign',
      'buy_now_buttons': '\"Kaufe jetzt\"-Buttons',
      'email_settings': 'E-Mail-Einstellungen',
      'templates_and_reminders': 'Vorlagen & Erinnerungen',
      'credit_cards_and_banks': 'Kreditkarten & Banken',
      'data_visualizations': 'Datenvisualisierungen',
      'price': 'Preis',
      'email_sign_up': 'E-Mail-Registrierung',
      'google_sign_up': 'Registrierung via Google',
      'thank_you_for_your_purchase': 'Vielen Dank für Ihren Kauf!',
      'redeem': 'Einlösen',
      'back': 'Zurück',
      'past_purchases': 'Vergangene Käufe',
      'annual_subscription': 'Jahres-Abonnement',
      'pro_plan': 'Pro-Tarif',
      'enterprise_plan': 'Enterprise-Tarif',
      'count_users': ':count Benutzer',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Bitte geben Sie einen Vornamen ein',
      'please_enter_a_last_name': 'Bitte geben Sie einen Nachnamen ein',
      'please_agree_to_terms_and_privacy':
          'Bitte stimmen Sie den Nutzungsbedingungen und der Datenschutzerklärung zu, um ein Konto zu erstellen.',
      'i_agree_to_the': 'Ich stimme den',
      'terms_of_service_link': 'Nutzungsbedingungen',
      'privacy_policy_link': 'Datenschutzerklärung',
      'terms_of_service': 'Service-Bedingungen',
      'privacy_policy': 'Datenschutzerklärung',
      'sign_up': 'Anmeldung',
      'account_login': 'Konto Login',
      'view_website': 'Webseite anschauen',
      'create_account': 'Konto erstellen',
      'email_login': 'E-Mail-Anmeldung',
      'create_new': 'Neu...',
      'no_record_selected': 'Kein Eintrag ausgewählt',
      'error_unsaved_changes':
          'Bitte speichern oder verwerfen Sie Ihre Änderungen',
      'download': 'Downloaden',
      'requires_an_enterprise_plan': 'Benötigt einen Enterprise Plan',
      'take_picture': 'Bild aufnehmen',
      'upload_file': 'Datei hochladen',
      'document': 'Dokument',
      'documents': 'Dokumente',
      'new_document': 'Neues Dokument',
      'edit_document': 'Dokument bearbeiten',
      'uploaded_document': 'Dokument erfolgreich hochgeladen',
      'updated_document': 'Dokument erfolgreich aktualisiert',
      'archived_document': 'Dokument erfolgreich archiviert',
      'deleted_document': 'Dokument erfolgreich gelöscht',
      'restored_document': 'Dokument erfolgreich wiederhergestellt',
      'archived_documents': ':value Dokumente erfolgreich archiviert',
      'deleted_documents': ':value Dokumente erfolgreich gelöscht',
      'restored_documents': ':value Dokumente erfolgreich wiederhergestellt',
      'no_history': 'Kein Verlauf',
      'expense_date': 'Ausgabendatum',
      'pending': 'Ausstehend',
      'expense_status_1': 'Aufgezeichnet',
      'expense_status_2': 'Ausstehend',
      'expense_status_3': 'Fakturiert',
      'converted': 'Umgewandelt',
      'add_documents_to_invoice': 'Fügen Sie Dokumente zur Rechnung hinzu',
      'exchange_rate': 'Wechselkurs',
      'convert_currency': 'Währung umrechnen',
      'mark_paid': 'Als bezahlt markieren',
      'category': 'Kategorie',
      'address': 'Adresse',
      'new_vendor': 'Neuer Lieferant',
      'created_vendor': 'Lieferant erfolgreich erstellt',
      'updated_vendor': 'Lieferant erfolgreich aktualisiert',
      'archived_vendor': 'Lieferant erfolgreich archiviert',
      'deleted_vendor': 'Lieferant erfolgreich gelöscht',
      'restored_vendor': 'Lieferant erfolgreich wiederhergestellt',
      'archived_vendors': ':count Lieferanten erfolgreich archiviert',
      'deleted_vendors': ':count Lieferanten erfolgreich gelöscht',
      'restored_vendors': ':value Lieferanten erfolgreich wiederhergestellt',
      'new_expense': 'Ausgabe eingeben',
      'created_expense': 'Ausgabe erfolgreich erstellt',
      'updated_expense': 'Ausgabe erfolgreich aktualisiert',
      'archived_expense': 'Ausgabe erfolgreich archiviert',
      'deleted_expense': 'Ausgabe erfolgreich gelöscht',
      'restored_expense': 'Ausgabe erfolgreich wiederhergestellt',
      'archived_expenses': 'Ausgaben erfolgreich archiviert',
      'deleted_expenses': 'Ausgaben erfolgreich gelöscht',
      'restored_expenses': ':value Ausgaben erfolgreich wiederhergestellt',
      'copy_shipping': 'Versand kopieren',
      'copy_billing': 'Zahlung kopieren',
      'design': 'Design',
      'failed_to_find_record': 'Eintrag konnte nicht gefunden werden',
      'invoiced': 'In Rechnung gestellt',
      'logged': 'Protokolliert',
      'running': 'Läuft',
      'resume': 'Fortfahren',
      'task_errors': 'Bitte korrigieren Sie alle überlappenden Zeiten',
      'start': 'Starten',
      'stop': 'Anhalten',
      'started_task': 'Aufgabe erfolgreich gestartet',
      'stopped_task': 'Aufgabe erfolgreich angehalten',
      'resumed_task': 'Aufgabe fortgesetzt',
      'now': 'Jetzt',
      'auto_start_tasks': 'Aufgaben für den automatischen Start',
      'timer': 'Zeitmesser',
      'manual': 'Manuell',
      'budgeted': 'Budgetiert',
      'start_time': 'Startzeit',
      'end_time': 'Endzeit',
      'date': 'Datum',
      'times': 'Zeiten',
      'duration': 'Dauer',
      'new_task': 'Neue Aufgabe',
      'created_task': 'Aufgabe erfolgreich erstellt',
      'updated_task': 'Aufgabe erfolgreich aktualisiert',
      'archived_task': 'Aufgabe erfolgreich archiviert',
      'deleted_task': 'Aufgabe erfolgreich gelöscht',
      'restored_task': 'Aufgabe erfolgreich wiederhergestellt',
      'archived_tasks': ':count Aufgaben wurden erfolgreich archiviert',
      'deleted_tasks': ':count Aufgaben wurden erfolgreich gelöscht',
      'restored_tasks': ':value Aufgaben erfolgreich wiederhergestellt',
      'please_enter_a_name': 'Bitte geben Sie einen Namen ein',
      'budgeted_hours': 'Budgetierte Stunden',
      'created_project': 'Projekt erfolgreich erstellt',
      'updated_project': 'Projekt erfolgreich aktualisiert',
      'archived_project': 'Projekt erfolgreich archiviert',
      'deleted_project': 'Projekt erfolgreich gelöscht',
      'restored_project': 'Projekt erfolgreich wiederhergestellt',
      'archived_projects': 'Erfolgreich :count Projekte archiviert',
      'deleted_projects': 'Erfolgreich :count Projekte gelöscht',
      'restored_projects': ':value Projekte erfolgreich wiederhergestellt',
      'new_project': 'neues Projekt',
      'thank_you_for_using_our_app': 'Vielen Dank, dass Sie unsere App nutzen!',
      'if_you_like_it': 'Wenn es dir gefällt, bitte',
      'click_here': 'hier klicken',
      'click_here_capital': 'Klicke hier',
      'to_rate_it': ', um es zu bewerten.',
      'average': 'Durchschnittlich',
      'unapproved': 'Nicht genehmigt',
      'authenticate_to_change_setting':
          'Bitte authentifizieren Sie sich, um diese Einstellung zu ändern.',
      'locked': 'Gesperrt',
      'authenticate': 'Authentifizieren',
      'please_authenticate': 'Bitte authentifizieren Sie sich',
      'biometric_authentication': 'Biometrische Authentifizierung',
      'footer': 'Fußzeile',
      'compare': 'Vergleiche',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Anmeldung mit Google',
      'today': 'Heute',
      'custom_range': 'Benutzerdefinierter Bereich',
      'date_range': 'Datumsbereich',
      'current': 'Aktuell',
      'previous': 'Vorherige',
      'current_period': 'Aktuelle Periode',
      'comparison_period': 'Vergleichsperiode',
      'previous_period': 'Vorherige Periode',
      'previous_year': 'Vorjahr',
      'compare_to': 'Vergleiche mit',
      'last7_days': 'Letzte 7 Tage',
      'last_week': 'Letzte Woche',
      'last30_days': 'Letzte 30 Tage',
      'this_month': 'Dieser Monat',
      'last_month': 'Letzter Monat',
      'this_year': 'Dieses Jahr',
      'last_year': 'Letztes Jahr',
      'custom': 'Benutzerdefiniert',
      'clone_to_invoice': 'Klone in Rechnung',
      'clone_to_quote': 'Klone in Angebot',
      'clone_to_credit': 'Duplizieren in Gutschrift',
      'view_invoice': 'Rechnung anschauen',
      'convert': 'Konvertiere',
      'more': 'Mehr',
      'edit_client': 'Kunde bearbeiten',
      'edit_product': 'Produkt bearbeiten',
      'edit_invoice': 'Rechnung bearbeiten',
      'edit_quote': 'Angebot bearbeiten',
      'edit_payment': 'Zahlung bearbeiten',
      'edit_task': 'Aufgabe bearbeiten',
      'edit_expense': 'Ausgabe Bearbeiten',
      'edit_vendor': 'Lieferant Bearbeiten',
      'edit_project': 'Projekt bearbeiten',
      'edit_recurring_expense': 'Wiederkehrende Ausgabe bearbeiten',
      'edit_recurring_quote': 'Bearbeite wiederkehrendes Angebot',
      'billing_address': 'Rechnungsadresse',
      'shipping_address': 'Lieferadresse',
      'total_revenue': 'Gesamteinnahmen',
      'average_invoice': 'Durchschnittlicher Rechnungsbetrag',
      'outstanding': 'Ausstehend',
      'invoices_sent': ':count Rechnungen versendet',
      'active_clients': 'aktive Kunden',
      'close': 'Schließen',
      'email': 'E-Mail',
      'password': 'Passwort',
      'url': 'URL',
      'secret': 'Passwort',
      'name': 'Name',
      'logout': 'Abmelden',
      'login': 'Login',
      'filter': 'Filter',
      'sort': 'Sortierung',
      'search': 'Suche',
      'active': 'Aktiv',
      'archived': 'Archiviert',
      'deleted': 'Gelöscht',
      'dashboard': 'Dashboard',
      'archive': 'Archivieren',
      'delete': 'löschen',
      'restore': 'Wiederherstellen',
      'refresh_complete': 'Aktualisieren beendet',
      'please_enter_your_email': 'Bitte geben Sie Ihre E-Mail-Adresse ein',
      'please_enter_your_password': 'Bitte geben Sie Ihr Passwort ein',
      'please_enter_your_url': 'Bitte geben Sie Ihre URL ein',
      'please_enter_a_product_key':
          'Bitte geben Sie Ihren Produkt schlüssel ein',
      'ascending': 'Aufsteigend',
      'descending': 'Absteigend',
      'save': 'Speichern',
      'an_error_occurred': 'Ein Fehler ist aufgetreten',
      'paid_to_date': 'Bereits gezahlt',
      'balance_due': 'Offener Betrag',
      'balance': 'Saldo',
      'overview': 'Übersicht',
      'details': 'Details',
      'phone': 'Telefon',
      'website': 'Webseite',
      'vat_number': 'USt-IdNr.',
      'id_number': 'Kundennummer',
      'create': 'Erstellen',
      'copied_to_clipboard': ':value in die Zwischenablage kopiert',
      'error': 'Fehler',
      'could_not_launch': 'Konnte nicht gestartet werden',
      'contacts': 'Kontakte',
      'additional': 'Zusätzlich',
      'first_name': 'Vorname',
      'last_name': 'Nachname',
      'add_contact': 'Kontakt hinzufügen',
      'are_you_sure': 'Sind Sie sicher?',
      'cancel': 'Abbrechen',
      'ok': 'Ok',
      'remove': 'Entfernen',
      'email_is_invalid': 'E-Mail ist ungültig',
      'product': 'Produkt',
      'products': 'Produkte',
      'new_product': 'Neues Produkt',
      'created_product': 'Produkt erfolgreich erstellt',
      'updated_product': 'Produkt erfolgreich aktualisiert',
      'archived_product': 'Produkt erfolgreich archiviert',
      'deleted_product': 'Produkt erfolgreich gelöscht',
      'restored_product': 'Produkt erfolgreich wiederhergestellt',
      'archived_products': 'Archivierung erfolgreich :Produktzähler',
      'deleted_products': 'Erfolgreich :count Produkte gelöscht',
      'restored_products': ':value Produkte erfolgreich wiederhergestellt',
      'product_key': 'Produkt',
      'notes': 'Notizen',
      'cost': 'Kosten',
      'client': 'Kunde',
      'clients': 'Kunden',
      'new_client': 'Neuer Kunde',
      'created_client': 'Kunde erfolgreich angelegt',
      'updated_client': 'Kunde erfolgreich aktualisiert',
      'archived_client': 'Kunde erfolgreich archiviert',
      'archived_clients': ':count Kunden erfolgreich archiviert',
      'deleted_client': 'Kunde erfolgreich gelöscht',
      'deleted_clients': ':count Kunden erfolgreich gelöscht',
      'restored_client': 'Kunde erfolgreich wiederhergestellt',
      'restored_clients': ':value Kunden erfolgreich wiederhergestellt',
      'address1': 'Straße',
      'address2': 'Adresszusatz',
      'city': 'Stadt',
      'state': 'Bundesland',
      'postal_code': 'Postleitzahl',
      'country': 'Land',
      'invoice': 'Rechnung',
      'invoices': 'Rechnungen',
      'new_invoice': 'Neue Rechnung',
      'created_invoice': 'Rechnung erfolgreich erstellt',
      'updated_invoice': 'Rechnung erfolgreich aktualisiert',
      'archived_invoice': 'Rechnung erfolgreich archiviert',
      'deleted_invoice': 'Rechnung erfolgreich gelöscht',
      'restored_invoice': 'Rechnung erfolgreich wiederhergestellt',
      'archived_invoices': ':count Rechnungen erfolgreich archiviert',
      'deleted_invoices': ':count Rechnungen erfolgreich gelöscht',
      'restored_invoices': ':value Rechnungen erfolgreich wiederhergestellt',
      'emailed_invoice': 'Rechnung erfolgreich versendet',
      'emailed_payment': 'Zahlungs eMail erfolgreich gesendet',
      'amount': 'Betrag',
      'invoice_number': 'Rechnungsnummer',
      'invoice_date': 'Rechnungsdatum',
      'discount': 'Rabatt',
      'po_number': 'Bestellnummer',
      'terms': 'Bedingungen',
      'public_notes': 'Öffentliche Notizen',
      'private_notes': 'Private Notizen',
      'frequency': 'Häufigkeit',
      'start_date': 'Startdatum',
      'end_date': 'Enddatum',
      'quote_number': 'Angebotsnummer',
      'quote_date': 'Angebotsdatum',
      'valid_until': 'Gültig bis',
      'items': 'Element',
      'partial_deposit': 'Teil-/Anzahlung',
      'description': 'Beschreibung',
      'unit_cost': 'Einzelpreis',
      'quantity': 'Menge',
      'add_item': 'Artikel hinzufügen',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Gesamtbetrag',
      'pdf': 'PDF',
      'due_date': 'Fälligkeitsdatum',
      'partial_due_date': 'Teilzahlungsziel',
      'status': 'Status',
      'invoice_status_id': 'Rechnungs Status',
      'quote_status': 'Angebotsstatus',
      'click_plus_to_add_item':
          'Klicken Sie auf +, um ein Element hinzuzufügen.',
      'click_plus_to_add_time': 'Klicken Sie auf +, um die Zeit hinzuzufügen.',
      'count_selected': ':count ausgewählt',
      'total': 'Gesamt',
      'percent': 'Prozent',
      'edit': 'Bearbeiten',
      'dismiss': 'Verwerfen',
      'please_select_a_date': 'Bitte wählen Sie ein Datum',
      'please_select_a_client': 'Bitte wählen Sie einen Kunden',
      'please_select_an_invoice': 'Bitte wählen Sie eine Rechnung aus',
      'task_rate': 'Kosten für Tätigkeit',
      'settings': 'Einstellungen',
      'language': 'Sprache',
      'currency': 'Währung',
      'created_at': 'Erstellt am',
      'created_on': 'Erstellt am',
      'updated_at': 'Aktualisiert',
      'tax': 'Steuer',
      'please_enter_an_invoice_number':
          'Bitte geben Sie eine Rechnungs Nummer ein',
      'please_enter_a_quote_number': 'Bitte geben Sie eine Angebots Nummer ein',
      'past_due': 'Überfällig',
      'draft': 'Entwurf',
      'sent': 'Versendet',
      'viewed': 'Angesehen',
      'approved': 'Bestätigt',
      'partial': 'Teil-/Anzahlung',
      'paid': 'Bezahlt',
      'mark_sent': 'Als versendet markieren',
      'marked_invoice_as_sent': 'Rechnung erfolgreich als versendet markiert',
      'marked_invoice_as_paid':
          'Die Rechnung wurde erfolgreich als \"versendet\" markiert',
      'marked_invoices_as_sent':
          'Erfolgreich Rechnungen als versendet markiert',
      'marked_invoices_as_paid':
          'Die Rechnung wurde erfolgreich als \"versendet\" markiert',
      'done': 'Erledigt',
      'please_enter_a_client_or_contact_name':
          'Bitte geben Sie einen Kunden- oder Kontaktnamen ein',
      'dark_mode': 'Dunkler Modus',
      'restart_app_to_apply_change':
          'Starten Sie die App neu, um die Änderung zu übernehmen.',
      'refresh_data': 'Daten aktualisieren',
      'blank_contact': 'Leerer Kontakt',
      'activity': 'Aktivität',
      'no_records_found': 'Kein Einträge gefunden',
      'clone': 'Duplizieren',
      'loading': 'Lädt',
      'industry': 'Kategorie',
      'size': 'Größe',
      'payment_terms': 'Zahlungsbedingungen',
      'payment_date': 'Zahlungsdatum',
      'payment_status': 'Zahlungsstatus',
      'payment_status_1': 'Ausstehend',
      'payment_status_2': 'entwertet',
      'payment_status_3': 'Fehlgeschlagen',
      'payment_status_4': 'Abgeschlossen',
      'payment_status_5': 'Teilweise erstattet',
      'payment_status_6': 'Erstattet',
      'payment_status_-1': 'nicht angewendet',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Netto',
      'client_portal': 'Kunden-Portal',
      'show_tasks': 'Aufgaben anzeigen',
      'email_reminders': 'E-Mail Erinnerungen',
      'enabled': 'Aktiviert',
      'recipients': 'Empfänger',
      'initial_email': 'Initiale E-Mail',
      'first_reminder': 'Erste Erinnerung',
      'second_reminder': 'Zweite Erinnerung',
      'third_reminder': 'Dritte Erinnerung',
      'reminder1': 'Erste Erinnerung',
      'reminder2': 'Zweite Erinnerung',
      'reminder3': 'Dritte Erinnerung',
      'template': 'Vorlage',
      'send': 'Senden',
      'subject': 'Betreff',
      'body': 'Inhalt',
      'send_email': 'E-Mail senden',
      'email_receipt': 'Zahlungsbestätigung an Kunden per E-Mail senden',
      'auto_billing': 'Automatische Rechnungsstellung',
      'button': 'Knopf',
      'preview': 'Vorschau',
      'customize': 'Anpassen',
      'history': 'Verlauf',
      'payment': 'Zahlung',
      'payments': 'Zahlungen',
      'refunded': 'Erstattet',
      'payment_type': 'Zahlungsart',
      'transaction_reference': 'Abwicklungsreferenz',
      'enter_payment': 'Zahlung eingeben',
      'new_payment': 'Zahlung eingeben',
      'created_payment': 'Zahlung erfolgreich erstellt',
      'updated_payment': 'Zahlung erfolgreich aktualisiert',
      'archived_payment': 'Zahlung erfolgreich archiviert',
      'deleted_payment': 'Zahlung erfolgreich gelöscht',
      'restored_payment': 'Zahlung erfolgreich wiederhergestellt',
      'archived_payments': ':count Zahlungen erfolgreich archiviert',
      'deleted_payments': ':count Zahlungen erfolgreich gelöscht',
      'restored_payments': ':value Zahlungen erfolgreich wiederhergestellt',
      'quote': 'Angebot',
      'quotes': 'Angebote',
      'new_quote': 'Neues Angebot',
      'created_quote': 'Angebot erfolgreich erstellt',
      'updated_quote': 'Angebot erfolgreich aktualisiert',
      'archived_quote': 'Angebot erfolgreich archiviert',
      'deleted_quote': 'Angebot erfolgreich gelöscht',
      'restored_quote': 'Angebot erfolgreich wiederhergestellt',
      'archived_quotes': ':count Angebote erfolgreich archiviert',
      'deleted_quotes': ':count Angebote erfolgreich gelöscht',
      'restored_quotes': ':value Angebote erfolgreich wiederhergestellt',
      'expense': 'Ausgabe',
      'expenses': 'Ausgaben',
      'vendor': 'Lieferant',
      'vendors': 'Lieferanten',
      'task': 'Aufgabe',
      'tasks': 'Zeiterfassung',
      'project': 'Projekt',
      'projects': 'Projekte',
      'activity_1': ':user erstellte Kunde :client',
      'activity_2': ':user archivierte Kunde :client',
      'activity_3': ':user löschte Kunde :client',
      'activity_4': ':user erstellte Rechnung :invoice',
      'activity_5': ':user aktualisierte Rechnung :invoice',
      'activity_6': ':user mailte Rechnung :invoice für :client an :contact',
      'activity_7': ':contact schaute Rechnung :invoice für :client an',
      'activity_8': ':user archivierte Rechnung :invoice',
      'activity_9': ':user löschte Rechnung :invoice',
      'activity_10':
          ':contact gab Zahlungsinformation :payment über :payment_amount für Rechnung :invoice für Kunde :client',
      'activity_11': ':user aktualisierte Zahlung :payment',
      'activity_12': ':user archivierte Zahlung :payment',
      'activity_13': ':user löschte Zahlung :payment',
      'activity_14': ':user gab :credit Guthaben ein',
      'activity_15': ':user aktualisierte :credit Guthaben',
      'activity_16': ':user archivierte :credit Guthaben',
      'activity_17': ':user löschte :credit Guthaben',
      'activity_18': ':user erstellte Angebot :quote',
      'activity_19': ':user aktualisierte Angebot :quote',
      'activity_20': ':user mailte Angebot :quote für :client an :contact',
      'activity_21': ':contact schaute Angebot :quote an',
      'activity_22': ':user archivierte Angebot :quote',
      'activity_23': ':user löschte Angebot :quote',
      'activity_24': ':user stellte Angebot :quote wieder her',
      'activity_25': ':user stellte Rechnung :invoice wieder her',
      'activity_26': ':user stellte Kunde :client wieder her',
      'activity_27': ':user stellte Zahlung :payment wieder her',
      'activity_28': ':user stellte Guthaben :credit wieder her',
      'activity_29': ':contact akzeptierte Angebot :quote für :client',
      'activity_30': ':user hat Lieferant :vendor erstellt',
      'activity_31': ':user hat Lieferant :vendor archiviert',
      'activity_32': ':user hat Lieferant :vendor gelöscht',
      'activity_33': ':user hat Lieferant :vendor wiederhergestellt',
      'activity_34': ':user erstellte Ausgabe :expense',
      'activity_35': ':user hat Ausgabe :expense archiviert',
      'activity_36': ':user hat Ausgabe :expense gelöscht',
      'activity_37': ':user hat Ausgabe :expense wiederhergestellt',
      'activity_39':
          ':user brach eine Zahlung über :payment_amount ab :payment',
      'activity_40':
          ':user hat :adjustment von :payment_amount der Zahlung :payment zurück erstattet',
      'activity_41': ':payment_amount Zahlung (:payment) schlug fehl',
      'activity_42': ':user hat Aufgabe :task erstellt',
      'activity_43': ':user hat Aufgabe :task bearbeitet',
      'activity_44': ':user hat Aufgabe :task archiviert',
      'activity_45': ':user hat Aufgabe :task gelöscht',
      'activity_46': ':user hat Aufgabe :task wiederhergestellt',
      'activity_47': ':user hat Ausgabe :expense bearbeitet',
      'activity_48': ':user hat Ticket :ticket bearbeitet',
      'activity_49': ':user hat Ticket :ticket geschlossen',
      'activity_50': ':user hat Ticket :ticket zusammengeführt',
      'activity_51': ':user hat Ticket :ticket aufgeteilt',
      'activity_52': ':contact hat Ticket :ticket geöffnet',
      'activity_53': ':contact hat Ticket :ticket wieder geöffnet',
      'activity_54': ':user hat Ticket :ticket wieder geöffnet',
      'activity_55': ':contact hat auf Ticket :ticket geantwortet',
      'activity_56': ':user  hat Ticket :ticket angesehen',
      'activity_57':
          'Das System konnte die Rechnung :invoice nicht per E-Mail versenden',
      'activity_58': ':user buchte Rechnung :invoice zurück',
      'activity_59': ':user brach Rechnung :invoice ab',
      'activity_60': ':contact schaute Angebot :quote an',
      'activity_61': ':user hat Kunde :client aktualisiert',
      'activity_62': ':user hat Lieferant :vendor aktualisiert',
      'activity_63':
          ':user mailte erste Erinnerung für Rechnung :invoice an :contact',
      'activity_64':
          ':user mailte zweite Erinnerung für Rechnung :invoice an :contact',
      'activity_65':
          ':user mailte dritte Erinnerung für Rechnung :invoice an :contact',
      'activity_66':
          ':user mailte endlose Erinnerung für Rechnung :invoice an :contact',
      'activity_80': ':user hat Abonnement :subscription erstellt',
      'activity_81': ':user hat Abonnement :subscription geändert',
      'activity_82': ':user hat Abonnement :subscription archiviert',
      'activity_83': ':user hat Abonnement :subscription gelöscht',
      'activity_84': ':user hat Abonnement :subscription wiederhergestellt',
      'one_time_password': 'Einmaliges Passwort',
      'emailed_quote': 'Angebot erfolgreich versendet',
      'emailed_credit': 'Guthaben erfolgreich per E-Mail versendet',
      'marked_quote_as_sent': 'Angebot erfolgreich als versendet markiert',
      'marked_credit_as_sent': 'Guthaben erfolgreich als versendet markiert',
      'expired': 'Abgelaufen',
      'all': 'Alle',
      'select': 'Wählen',
      'long_press_multiselect': 'Mehrfachauswahl durch langes Drücken',
      'custom_value1': 'Benutzerdefinierten Wert',
      'custom_value2': 'Benutzerdefinierten Wert',
      'custom_value3': 'Benutzerdefinierter Wert 3',
      'custom_value4': 'Benutzerdefinierter Wert 4',
      'email_style_custom': 'Benutzer definierter E-Mail-Stil',
      'custom_message_dashboard': 'Benutzerdefinierte Dashboard-Nachricht',
      'custom_message_unpaid_invoice':
          'Benutzerdefinierte Nachricht für unbezahlte Rechnung',
      'custom_message_paid_invoice':
          'Benutzerdefinierte Nachricht für bezahlte Rechnung',
      'custom_message_unapproved_quote':
          'Benutzerdefinierte Nachricht für nicht genehmigten Kostenvoranschlag',
      'lock_invoices': 'Rechnung sperren',
      'translations': 'Übersetzungen',
      'task_number_pattern': 'Aufgabennummernschema',
      'task_number_counter': 'Aufgabennummernzähler',
      'expense_number_pattern': 'Ausgabennummernschema',
      'expense_number_counter': 'Ausgabennummernzähler',
      'vendor_number_pattern': 'Lieferantennummernschema',
      'vendor_number_counter': 'Lieferantennummernzähler',
      'ticket_number_pattern': 'Ticketnummernschema',
      'ticket_number_counter': 'Ticketnummernzähler',
      'payment_number_pattern': 'Zahlungsnummernschema',
      'payment_number_counter': 'Zahlungsnummernzähler',
      'invoice_number_pattern': 'Rechnungsnummernschema',
      'invoice_number_counter': 'Zähler für Rechnungsnummer',
      'quote_number_pattern': 'Kostenvoranschlags-Nummernschema',
      'quote_number_counter': 'Zähler für Angebotsnummer',
      'client_number_pattern': 'Gutschriftnummernschema',
      'client_number_counter': 'Gutschriftnummernzähler',
      'credit_number_pattern': 'Gutschriftnummernzähler',
      'credit_number_counter': 'Gutschriftnummernzähler',
      'reset_counter_date': 'Zählerdatum zurücksetzen',
      'counter_padding': 'Zähler-Innenabstand',
      'shared_invoice_quote_counter':
          'Gemeinsamen Nummernzähler für Rechnungen und Angebote verwenden',
      'default_tax_name_1': 'Standard-Steuername 1',
      'default_tax_rate_1': 'Standard-Steuersatz 1',
      'default_tax_name_2': 'Standard-Steuername 2',
      'default_tax_rate_2': 'Standard-Steuersatz 2',
      'default_tax_name_3': 'Standard-Steuername 3',
      'default_tax_rate_3': 'Standard-Steuersatz 3',
      'email_subject_invoice': 'EMail Rechnung Betreff',
      'email_subject_quote': 'EMail Angebot Betreff',
      'email_subject_payment': 'EMail Zahlung Betreff',
      'email_subject_payment_partial': 'EMail Teilzahlung Betreff',
      'show_table': 'Zeige Tabelle',
      'show_list': 'Zeige Liste',
      'client_city': 'Kunden-Stadt',
      'client_state': 'Kunden-Bundesland/Kanton',
      'client_country': 'Kunden-Land',
      'client_is_active': 'Kunde ist aktiv',
      'client_balance': 'Kunden Betrag',
      'client_address1': 'Straße des Kunden',
      'client_address2': 'Adresszusatz',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Strasse Kundenlieferanschrift',
      'client_shipping_address2': 'Addresszusatz Kundenlieferadresse',
      'type': 'Typ',
      'invoice_amount': 'Rechnungssumme',
      'invoice_due_date': 'Fälligkeitsdatum',
      'tax_rate1': 'Steuersatz 1',
      'tax_rate2': 'Steuersatz 2',
      'tax_rate3': 'Steuersatz 3',
      'auto_bill': 'Automatische Verrechnung',
      'archived_at': 'Archiviert um',
      'has_expenses': 'Hat Ausgaben',
      'custom_taxes1': 'Benutzerdefinierte Steuern 1',
      'custom_taxes2': 'Benutzerdefinierte Steuern 2',
      'custom_taxes3': 'Benutzerdefinierte Steuern 3',
      'custom_taxes4': 'Benutzerdefinierte Steuern 4',
      'custom_surcharge1': 'Benutzerdefinierter Zuschlag 1',
      'custom_surcharge2': 'Benutzerdefinierter Zuschlag 2',
      'custom_surcharge3': 'Benutzerdefinierter Zuschlag 3',
      'custom_surcharge4': 'Benutzerdefinierter Zuschlag 4',
      'is_deleted': 'ist gelöscht',
      'vendor_city': 'Lieferanten-Stadt',
      'vendor_state': 'Lieferanten-Bundesland/Kanton',
      'vendor_country': 'Lieferanten-Land',
      'is_approved': 'Wurde angenommen',
      'tax_name': 'Steuersatz Name',
      'tax_amount': 'Steuerwert',
      'tax_paid': 'Steuern bezahlt',
      'payment_amount': 'Zahlungsbetrag',
      'age': 'Alter',
      'is_running': 'Läuft derzeit',
      'time_log': 'Zeiten',
      'bank_id': 'Bank',
      'expense_category_id': 'Ausgabenkategorie ID',
      'expense_category': 'Ausgabenkategorie',
      'invoice_currency_id': 'Rechnungs-Währungs-ID',
      'tax_name1': 'Steuersatz Name 1',
      'tax_name2': 'Steuersatz Name 2',
      'tax_name3': 'Steuersatz Name 3',
      'transaction_id': 'Transaktions ID',
      'color_theme': 'Farbthema',
    },
    'el': {
      'import_data': 'Εισαγωγή Δεδομένων',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Επαγγελματική',
      'enterprise': 'Enterprise',
      'last_updated': 'Τελευταία ενημέρωση',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Όνομα Επαφής',
      'contact_last_name': 'Επώνυμο Επαφής',
      'order': 'Order',
      'unassigned': 'Δεν έχει ανατεθεί',
      'partial_value':
          'Πρέπει να είναι μεγαλύτερο του μηδενός και μικρότερο από το σύνολο.',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Ενεργοποίηση',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Ορίστε τον υποτομέα ή εμφάνιστε το τιμολόγιο στη δική σας ιστοσελίδα',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Ασφάλεια',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Εγγραφές',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Επεξεργασία Συνδρομής',
      'created_subscription': 'Επιτυχής δημιουργία συνδρομής',
      'updated_subscription': 'Επιτυχής ενημέρωση συνδρομής',
      'archived_subscription': 'Επιτυχής αρχειοθέτηση συνδρομής',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Επαναποστολή Πρόσκλησης',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Επεστράφη',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Σκανάρετε το barcode με μία :link συμβατή εφαρμογή.',
      'enabled_two_factor':
          'Επιτυχής ενεργοποίηση Αυθεντικοποίησης Δύο Σημείων',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Αυθεντικοποίηση δύο σημείων',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Επεστραμένη Πληρωμή',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'Τρέχον Τετράμηνο',
      'last_quarter': 'Τελευταίο Τετράμηνο',
      'to_update_run': 'Για ενημέρωση εκτελέστε',
      'convert_to_invoice': 'Μετατροπή σε Τιμολόγιο',
      'registration_url': 'URL Εγγραφής',
      'invoice_project': 'Τιμολόγηση Project',
      'invoice_task': 'Τιμολόγηση Εργασίας',
      'invoice_expense': 'Τιμολόγηση Δαπάνης',
      'search_payment_term': 'Αναζήτηση 1 Όρου Πληρωμής',
      'search_payment_terms': 'Αναζήτηση :count Όρων Πληρωμής',
      'save_and_preview': 'Αποθήκευση και Προεπισκόπηση',
      'save_and_email': 'Αποθήκευση και Αποστολή Email',
      'supported_events': 'Υποστηριζόμενα Γεγονότα',
      'converted_amount': 'Μετατρεπόμενο Ποσό',
      'converted_balance': 'Υπόλοιπο από Μετατροπή',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Έχει Αποσταλεί',
      'default_documents': 'Προεπιλεγμένα Κείμενα',
      'document_upload': 'Μεταφόρτωση Εγγράφου',
      'document_upload_help':
          'Ενεργοποίησε τη δυνατότητα οι πελάτες να μεταφορτώνουν έγγραφα',
      'expense_total': 'Συνολική Δαπάνη',
      'enter_taxes': 'Εισαγετε Φόρους',
      'by_rate': 'Με Ποσοστό',
      'by_amount': 'Με Ποσό',
      'enter_amount': 'Εισάγετε Ποσό',
      'before_taxes': 'Προ Φόρων',
      'after_taxes': 'Μετά Φόρων',
      'color': 'Χρώμα',
      'show': 'Εμφάνισε',
      'hide': 'Απόκρυψη',
      'empty_columns': 'Άδιασε Στήλες',
      'debug_mode_is_enabled':
          'Το περιβάλλον αποσφαλμάτωσης έχει ενεργοποιηθεί',
      'debug_mode_is_enabled_help':
          'Προειδοποίηση: προορίζεται για χρήση σε τοπικά μηχανήματα, μπορεί να οδηγήσει σε διαρροή κωδικών. Πατήστε για να μάθετε περισσότερα.',
      'running_tasks': 'Εκτελούμενες εργασίες',
      'recent_tasks': 'Πρόσφατες Εργασίες',
      'recent_expenses': 'Πρόσφατες Δαπάνες',
      'upcoming_expenses': 'Επερχόμενες Δαπάνες',
      'update_app': 'Ενημέρωση Εφαρμογής',
      'started_import': 'Επιτυχής έναρξη εισαγωγής',
      'duplicate_column_mapping': 'Κλωνοποίηση αντιστοίχισης στηλών',
      'uses_inclusive_taxes': 'Χρησιμοποιούνται Συμπεριλαμβανόμενοι Φόροι',
      'is_amount_discount': 'Είναι Ποσό Έκπτωσης',
      'column': 'Κολόνα',
      'sample': 'Παράδειγμα',
      'map_to': 'Αντιστοίχιση Σε',
      'import': 'Εισαγωγή',
      'first_row_as_column_names': 'Χρήση της πρώτης σειράς ως ονόματα στηλών',
      'select_file': 'Παρακαλώ επιλέξτε ένα αρχείο',
      'no_file_selected': 'Δεν Επιλέχθηκε Αρχείο',
      'csv_file': 'Αρχείο CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Τύπος Εισαγωγής',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Εμφάνιση Αδειών Χρήσης',
      'webhook_url': 'Διεύθυνση URL του Webhook',
      'fullscreen_editor': 'Επεξεργαστής Πλήρους Οθόνης',
      'sidebar_editor': 'Επεξεργαστής Πλάγιας Μπάρας',
      'please_type_to_confirm':
          'Παρακαλούμε πληκρολογήστε \":value\" για επιβεβαίωση',
      'purge': 'Εκκαθάριση',
      'service': 'Υπηρεσία',
      'clone_to': 'Κλωνοποίηση Σε',
      'clone_to_other': 'Κλωνοποίηση σε Άλλο',
      'labels': 'Ετικέτες',
      'add_custom': 'Προσθήκη Προσαρμογής',
      'payment_tax': 'Φόρος Πληρωμής',
      'unpaid': 'Μη εξοφλημένη',
      'white_label': 'Λευκή Ετικέτα',
      'delivery_note': 'Σημείωση Παράδοσης',
      'sent_invoices_are_locked': 'Τα απεσταλμένα τιμολόγια είναι κλειδωμένα',
      'paid_invoices_are_locked': 'Τα εξοφλημένα τιμολόγια είναι κλειδωμένα',
      'source_code': 'Πηγαίος Κώδικας',
      'app_platforms': 'Πλατφόρμες Εφαρμογής',
      'invoice_late': 'Καθυστερημένο Ποσό Τιμολογίου',
      'quote_expired': 'Ληγμένη Προσφορά',
      'partial_due': 'Μερική Πληρωμή',
      'invoice_total': 'Σύνολο Τιμολογίου',
      'quote_total': 'Σύνολο Προσφοράς',
      'credit_total': 'Συνολική Πίστωση',
      'recurring_invoice_total': 'Σύνολο Τιμολογίου',
      'actions': 'Ενέργειες',
      'expense_number': 'Αριθμός Δαπάνης',
      'task_number': 'Αριθμός Εργασίας',
      'project_number': 'Αριθμός Project',
      'project_name': 'Όνομα Project',
      'warning': 'Ειδοποίηση',
      'view_settings': 'Ρυθμίσεις Εμφάνισης',
      'company_disabled_warning':
          'Προειδοποίηση: Αυτή η εταιρία δεν έχει ενεργοποιηθεί ακόμα',
      'late_invoice': 'Καθυστερημένο Τιμολόγιο',
      'expired_quote': 'Προσφορά που έληξε',
      'remind_invoice': 'Υπενθύμιση Τιμολογίου',
      'cvv': 'CVV',
      'client_name': 'Όνομα Πελάτη',
      'client_phone': 'Τηλέφωνο Πελάτη',
      'required_fields': 'Απαιτούμενα Πεδία',
      'calculated_rate': 'Υπολογιζόμενο Κόστος',
      'default_task_rate': 'Προεπιλεγμένο Κόστος Εργασίας',
      'clear_cache': 'Καθαρισμός Προσωρινής Μνήμης',
      'sort_order': 'Σειρά Ταξινόμησης',
      'task_status': 'Κατάσταση',
      'task_statuses': 'Καταστάσεις Εργασίας',
      'new_task_status': 'Κατάσταση Νέας Εργασίας',
      'edit_task_status': 'Επεξεργασία Κατάστασης Εργασίας',
      'created_task_status': 'Επιτυχής δημιουργία κατάστασης εργασίας',
      'updated_task_status': 'Επιτυχής ενημέρωση κατάστασης εργασίας',
      'archived_task_status': 'Επιτυχής αρχειοθέτηση κατάστασης εργασίας',
      'deleted_task_status': 'Επιτυχής διαγραφή κατάστασης εργασίας',
      'removed_task_status': 'Επιτυχής αφαίρεση κατάστασης εργασίας',
      'restored_task_status': 'Επιτυχής ανάκτηση κατάστασης εργασίας',
      'archived_task_statuses':
          'Επιτυχής αρχειοθέτηση :value καταστάσεων εργασίας',
      'deleted_task_statuses': 'Επιτυχής διαγραφή :value καταστάσεων εργασίας',
      'restored_task_statuses': 'Επιτυχής ανάκτηση :value καταστάσεων εργασίας',
      'search_task_status': 'Αναζήτηση 1 Κατάστασης Εργασίας',
      'search_task_statuses': 'Αναζήτηση :count Καταστάσεων Εργασίας',
      'show_tasks_table': 'Εμφάνιση Πίνακα Εργασιών',
      'show_tasks_table_help':
          'Πάντα να εμφανίζεται το τμήμα των εργασιών όταν δημιουργούνται τιμολόγια',
      'invoice_task_timelog': 'Χρονολόγιο Τιμολόγησης Εργασίας',
      'invoice_task_timelog_help':
          'Προσθήκη λεπτομεριών χρόνου στις γραμμές των τιμολογίων',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Έναρξη εργασιών πριν την αποθήκευση',
      'configure_statuses': 'Προσαρμογή Καταστάσεων',
      'task_settings': 'Ρυθμίσεις Εργασίας',
      'configure_categories': 'Προσαρμογή Κατηγοριών',
      'expense_categories': 'Κατηγορίες Δαπάνης',
      'new_expense_category': 'Νέα Κατηγορία Δαπανών',
      'edit_expense_category': 'Επεξεργασία Κατηγορίας Δαπάνης',
      'created_expense_category': 'Επιτυχής δημιουργία κατηγορίας δαπανών',
      'updated_expense_category': 'Επιτυχής ενημέρωση κατηγορίας δαπανών',
      'archived_expense_category': 'Επιτυχής αρχειοθέτηση κατηγορίας δαπανών',
      'deleted_expense_category': 'Επιτυχής διαγραφή κατηγορίας',
      'removed_expense_category': 'Επιτυχής αφαίρεση κατηγορίας δαπάνης',
      'restored_expense_category': 'Επιτυχής επαναφορά κατηγορίας δαπανών',
      'archived_expense_categories':
          'Επιτυχής αρχειοθέτηση :count κατηγορίας δαπανών',
      'deleted_expense_categories':
          'Επιτυχής διαγραφή δαπάνης :value κατηγοριών',
      'restored_expense_categories':
          'Επιτυχής επαναφορά δαπάνης :value κατηγοριών',
      'search_expense_category': 'Αναζήτηση 1 Κατηγορίας Δαπάνης',
      'search_expense_categories': 'Αναζήτηση :count Κατηγορίες Δαπανών',
      'use_available_credits': 'Χρήση Διαθέσιμων Πιστώσεων',
      'show_option': 'Εμφάνιση Επιλογής',
      'negative_payment_error':
          'Το ποσό πίστωσης δεν μπορεί να υπερβαίνει το ποσό πληρωμής',
      'view_changes': 'Προβολή Αλλαγών',
      'force_update': 'Εξαναγκασμένη Ενημέρωση',
      'force_update_help':
          'Εκτελείτε την τελευταία έκδοση αλλά μπορεί να υπάρχουν διορθώσεις σε αναμονή.',
      'mark_paid_help': 'Εντοπισμός της δαπάνης που πληρώθηκε',
      'should_be_invoiced': 'Πρέπει να τιμολογηθεί',
      'should_be_invoiced_help': 'Ενεργοποίηση της δαπάνης που θα τιμολογηθεί',
      'add_documents_to_invoice_help': 'Κάνε τα έγγραφα εμφανίσιμα',
      'convert_currency_help': 'Ορισμός Ισοτιμίας Ανταλλαγής',
      'expense_settings': 'Ρυθμίσεις Δαπάνης',
      'clone_to_recurring': 'Κλωνοποίηση σε Επαναλαμβανόμενο',
      'crypto': 'Κρύπτο',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Πληρωμή',
      'user_field': 'Πεδίο Χρήστη',
      'variables': 'Μεταβλητές',
      'show_password': 'Εμφάνιση Κωδικού Πρόσβασης',
      'hide_password': 'Απόκρυψη Κωδικού Πρόσβασης',
      'copy_error': 'Σφάλμα Αντιγραγής',
      'capture_card': 'Κάρτα Σύλληψης',
      'auto_bill_enabled': 'Αυτόματη Χρέωση Ενεργοποιήθηκε',
      'total_taxes': 'Συνολικοί Φόροι',
      'line_taxes': 'Φόροι Γραμμής',
      'total_fields': 'Συνολικά Πεδία',
      'stopped_recurring_invoice':
          'Επιτυχής διακοπή επαναλαμβανόμενου τιμολογίου',
      'started_recurring_invoice':
          'Επιτυχής έναρξη επαναλαμβανόμενου τιμολογίου',
      'resumed_recurring_invoice':
          'Επιτυχής επανεκκίνηση επαναλαμβανόμενου τιμολογίου',
      'gateway_refund': 'Επιστροφή Χρημάτων μέσω Πύλης Πληρωμής (Gateway)',
      'gateway_refund_help':
          'Εκτελέστε την επιστροφή νρημάτων μέσω Πύλης Πληρωμής (Gateway)',
      'due_date_days': 'Ημερομηνία Ολοκλήρωσης',
      'paused': 'Σε παύση',
      'mark_active': 'Σήμανση ως Ενεργό',
      'day_count': 'Ημέρα :count',
      'first_day_of_the_month': 'Πρώτη Μέρα του Μήνα',
      'last_day_of_the_month': 'Τελευταία Μέρα του Μήνα',
      'use_payment_terms': 'Χρησιμοποίηση Όρων Πληρωμής',
      'endless': 'Συνεχής',
      'next_send_date': 'Επόμενη Ημερομηνία Αποστολής',
      'remaining_cycles': 'Εναπομείναντες Κύκλοι',
      'recurring_invoice': 'Επαναλαμβανόμενο Τιμολόγιο',
      'recurring_invoices': 'Επαναλαμβανόμενα Τιμολόγια',
      'new_recurring_invoice': 'Νέο Επαναλαμβανόμενο Τιμολόγιο',
      'edit_recurring_invoice': 'Επεξεργασία Επαναλαμβανόμενων Τιμολογίων',
      'created_recurring_invoice':
          'Επιτυχής δημιουργία επαναλαμβανόμενου τιμολογίου',
      'updated_recurring_invoice':
          'Επιτυχής ενημέρωση επαναλαμβανόμενου τιμολογίου',
      'archived_recurring_invoice':
          'Επιτυχής αρχειοθέτηση επαναλαμβανόμενου τιμολογίου',
      'deleted_recurring_invoice':
          'Επιτυχής διαγραφή επαναλαμβανόμενου τιμολογίου',
      'removed_recurring_invoice':
          'Επιτυχής αφαίρεση επαναλαμβανόμενου τιμολογίου',
      'restored_recurring_invoice':
          'Επιτυχής επαναφορά επαναλαμβανόμενου τιμολογίου',
      'archived_recurring_invoices':
          'Επιτυχής αρχειοθέτηση :value επαναλαμβανόμενων τιμολογίων',
      'deleted_recurring_invoices':
          'Επιτυχής διαγραφή :value επαναλαμβανόμενων τιμολογίων',
      'restored_recurring_invoices':
          'Επιτυχής επαναφορά :value επαναλαμβανόμενων τιμολογίων',
      'search_recurring_invoice': 'Αναζήτηση 1 Επαναλαμβανόμενου Τιμολογίου',
      'search_recurring_invoices':
          'Αναζήτηση :count Επαναλαμβανόμενων Τιμολογίων',
      'send_date': 'Ημερομηνία Αποστολής',
      'auto_bill_on': 'Αυτόματη Χρέωση στις',
      'minimum_under_payment_amount': 'Ελάχιστο Ποσό Υποπληρωμής',
      'profit': 'Κέρδος',
      'line_item': 'Προϊόν Γραμμής',
      'allow_over_payment': 'Επιτρέψτε Υπερπληρωμή',
      'allow_over_payment_help':
          'Υποστήριξη της επιπλεόν πληρωμής για να δέχεστε φιλοδορήματα',
      'allow_under_payment': 'Επιτρέψτε Υποπληρωμή',
      'allow_under_payment_help':
          'Υποστήριξη εξόφλησης κατ\' ελάχιστο του μερικού ποσού',
      'test_mode': 'Περιβάλλον Τεστ',
      'opened': 'Ανοίχθηκε',
      'payment_reconciliation_failure': 'Αποτυχία Συμβιβασμού',
      'payment_reconciliation_success': 'Επιτυχία Συμβιβασμού',
      'gateway_success': 'Επιτυχία Πύλης Πληρωμής',
      'gateway_failure': 'Αποτυχία Πύλης Πληρωμής',
      'gateway_error': 'Σφάλμα Πύλης Πληρωμής',
      'email_send': 'Email απεστάλη',
      'email_retry_queue': 'Ουρά Επαναποστολής Email',
      'failure': 'Αποτυχία',
      'quota_exceeded': 'Υπέρβαση Ορίου',
      'upstream_failure': 'Αποτυχία Ροής',
      'system_logs': 'Αρχείο Καταγραφής Συστήματος',
      'view_portal': 'Προβολή portal',
      'copy_link': 'Αντιγραφή Συνδέσμου',
      'token_billing': 'Αποθήκευση στοιχείων κάρτας',
      'welcome_to_invoice_ninja': 'Καλωσήρθατε στο Invoice Ninja',
      'always': 'Πάντα',
      'optin': 'Συμμετοχή',
      'optout': 'Μη Συμμετοχή',
      'label': 'Ετικέτα',
      'client_number': 'Αριθμός Πελάτη',
      'auto_convert': 'Αυτόματη Μετατροπή',
      'company_name': 'Όνομα Εταιρείας',
      'reminder1_sent': 'Υπενθύμιση 1 Απεστάλη',
      'reminder2_sent': 'Υπενθύμιση 2 Απεστάλη',
      'reminder3_sent': 'Υπενθύμιση 3 Απεστάλη',
      'reminder_last_sent': 'Τελευταία Υπενθύμιση Απεστάλη',
      'pdf_page_info': 'Σελίδα :current από :total',
      'emailed_invoices': 'Επιτυχής αποστολή τιμολογίων',
      'emailed_quotes': 'Επιτυχής αποστολή προσφορών',
      'emailed_credits': 'Επιτυχής αποστολή ΄πιστώσεων με email',
      'gateway': 'Πύλη πληρωμής (Gateway)',
      'view_in_stripe': 'Προβολή στο Stripe',
      'rows_per_page': 'Γραμμές ανά Σελίδα',
      'hours': 'Ώρες',
      'statement': 'Δήλωση',
      'taxes': 'Φόροι',
      'surcharge': 'Επιβάρυνση',
      'apply_payment': 'Πληρωμή',
      'apply': 'Εφαρμογή',
      'unapplied': 'Ανεφάρμοστο',
      'select_label': 'Επιλογή Ετικέτας',
      'custom_labels': 'Προσαρμοσμένες Ετικέτες',
      'record_type': 'Τύπος Εγγραφής',
      'record_name': 'Όνομασία Εγγραφής',
      'file_type': 'Τύπος Αρχείου',
      'height': 'Ύψος',
      'width': 'Πλάτος',
      'to': 'Προς',
      'health_check': 'Έλεγχος Υγείας',
      'payment_type_id': 'Τύπος Πληρωμής',
      'last_login_at': 'Τελευταία Είσοδος στις',
      'company_key': 'Κλειδί Εταιρείας',
      'storefront': 'Βιτρίνα καταστήματος',
      'storefront_help':
          'Ενεργοποίηση εφαρμογών τρίτων για τη δημιουργία τιμολογίων',
      'client_created': 'Πελάτης Δημιουργήθηκε',
      'online_payment_email': 'Email Online Πληρωμών',
      'manual_payment_email': 'Email Χειροκίνητων Πληρωμών',
      'completed': 'Ολοκληρώθηκε',
      'gross': 'Μεικτό',
      'net_amount': 'Καθαρό Ποσό',
      'net_balance': 'Καθαρό Υπόλοιπο',
      'client_settings': 'Ρυθμίσεις Πελάτη',
      'selected_invoices': 'Επιλεγμένα Τιμολόγια',
      'selected_payments': 'Επιλεγμένες Πληρωμές',
      'selected_quotes': 'Επιλεγμένες Προσφορές',
      'selected_tasks': 'Επιλεγμένες Εργασίες',
      'selected_expenses': 'Επιλεγμένες Δαπάνες',
      'upcoming_invoices': 'Προσεχή Τιμολόγια',
      'past_due_invoices': 'Ληγμένα Τιμολόγια',
      'recent_payments': 'Πρόσφατες Πληρωμές',
      'upcoming_quotes': 'Προσεχείς Προσφορές',
      'expired_quotes': 'Προσφορές που έληξαν',
      'create_client': 'Δημιουργία Πελάτη',
      'create_invoice': 'Δημιουργία Τιμολογίου',
      'create_quote': 'Δημιουργία Προσφοράς',
      'create_payment': 'Δημιουργία Πληρωμής',
      'create_vendor': 'Δημιουργία προμηθευτή',
      'update_quote': 'Ενημέρωση Προσφοράς',
      'delete_quote': 'Διαγραφή Προσφοράς',
      'update_invoice': 'Ενημέρωση Τιμολογίου',
      'delete_invoice': 'Διαγραφή Τιμολογίου',
      'update_client': 'Ενημέρωση Πελάτη',
      'delete_client': 'Διαγραφή Πελάτη',
      'delete_payment': 'Διαγραφή Πληρωμής',
      'update_vendor': 'Ενημέρωση Προμηθευτή',
      'delete_vendor': 'Διαγραφή Προμηθευτή',
      'create_expense': 'Δημιουργία Δαπάνης',
      'update_expense': 'Ενημέρωση Δαπάνης',
      'delete_expense': 'Διαγραφή Δαπάνης',
      'create_task': 'Δημιουργία Εργασίας',
      'update_task': 'Ενημέρωση Εργασίας',
      'delete_task': 'Διαγραφή Εργασίας',
      'approve_quote': 'Αποδοχή Προσφοράς',
      'off': 'Κλειστό',
      'when_paid': 'Οταν Πληρωθεί',
      'expires_on': 'Λήγει την',
      'free': 'Δωρεάν',
      'plan': 'Πλάνο',
      'show_sidebar': 'Εμφάνιση Πλευρικής Μπάρας',
      'hide_sidebar': 'Απόκρυψη Πλευρικής Μπάρας',
      'event_type': 'Τύπος Γεγονότος',
      'target_url': 'Στόχος',
      'copy': 'Αντιγραφή',
      'must_be_online':
          'Παρακαλούμε επανεκκινήστε την εφαρμογή μόλις συνδεθείτε στο internet',
      'crons_not_enabled': 'Πρέπει να ενεργοποιήσετε τα crons',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Αναζήτηση :count Webhooks',
      'search_webhook': 'Αναζήτηση 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Νέο Webhook',
      'edit_webhook': 'Επεξεργασία Webhook',
      'created_webhook': 'Επιτυχής δημιουργία webhook',
      'updated_webhook': 'Επιτυχής ενημέρωση webhook',
      'archived_webhook': 'Επιτυχής αρχειοθέτηση webhook',
      'deleted_webhook': 'Επιτυχής διαγραφή webhook',
      'removed_webhook': 'Επιτυχής αφαίρεση webhook',
      'restored_webhook': 'Επιτυχής ανάκτηση webhook',
      'archived_webhooks': 'Επιτυχής αρχειοθέτηση :value webhooks',
      'deleted_webhooks': 'Επιτυχής διαγραφή :value webhooks',
      'removed_webhooks': 'Επιτυχής αφαίρεση :value webhooks',
      'restored_webhooks': 'Επιτυχής ανάκτηση :value webhooks',
      'api_tokens': 'Διακριτικά API',
      'api_docs': 'Κείμενα API',
      'search_tokens': 'Αναζήτηση :count Διακριτικών',
      'search_token': 'Αναζήτηση 1 Διακριτικού',
      'token': 'Διακριτικό',
      'tokens': 'Διακριτικά',
      'new_token': 'Νέο Διακριτικό',
      'edit_token': 'Επεξεργασία Διακριτικού',
      'created_token': 'Επιτυχής δημιουργία διακριτικού',
      'updated_token': 'Επιτυχής ενημέρωση διακριτικού',
      'archived_token': 'Επιτυχής αρχειοθέτηση διακριτικού',
      'deleted_token': 'Επιτυχής διαγραφή διακριτικού',
      'removed_token': 'Επιτυχής αφαίρεση διακριτικού',
      'restored_token': 'Επιτυχής ανάκτηση διακριτικού',
      'archived_tokens': 'Επιτυχής αρχειοθέτηση :value διακριτικών',
      'deleted_tokens': 'Επιτυχής διαγραφή :value διακριτικών',
      'restored_tokens': 'Επιτυχής ανάκτηση :value διακριτικών',
      'client_registration': 'Εγγραφή Πελάτη',
      'client_registration_help':
          'Ενεργοποίηση αυτοεγγραφής πελατών στο portal',
      'customize_and_preview': 'Προσαρμογή & Προεπισκόπηση',
      'email_invoice': 'Αποστολή Τιμολογίου με email',
      'email_quote': 'Αποστολή Προσφοράς',
      'email_credit': 'Αποστολή Πίστωσης με email',
      'email_payment': 'Αποστολή πληρωμής με Email',
      'client_email_not_set':
          'Ο πελάτης δεν έχει καταχωρημένη μία διεύθυνση email',
      'ledger': 'Καθολικό',
      'view_pdf': 'Προβολή PDF',
      'all_records': 'Όλες οι εγγραφές',
      'owned_by_user': 'Ιδιοκτησία του χρήστη',
      'credit_remaining': 'Υπολειπόμενη Πίστωση',
      'contact_name': 'Όνομα Επαφής',
      'use_default': 'Χρήση προεπιλογής',
      'reminder_endless': 'Συνεχής Υπενθύμιση',
      'number_of_days': 'Αριθμός ημερών',
      'configure_payment_terms': 'Προσαρμογή Όρων Πληρωμής',
      'payment_term': 'Όρος Πληρωμής',
      'new_payment_term': 'Νέος Όρος Πληρωμής',
      'edit_payment_term': 'Επεξεργασία Όρου Πληρωμής',
      'created_payment_term': 'Επιτυχής δημιουργία όρων πληρωμής',
      'updated_payment_term': 'Επιτυχής επικαιροποίηση όρων πληρωμής',
      'archived_payment_term': 'Επιτυχής αποθήκευση όρου πληρωμής',
      'deleted_payment_term': 'Επιτυχής διαγραφή όρου πληρωμής',
      'removed_payment_term': 'Επιτυχής αφαίρεση όρου πληρωμής',
      'restored_payment_term': 'Επιτυχής ανάκτηση όρου πληρωμής',
      'archived_payment_terms': 'Επιτυχής αρχειοθέτηση :value όρων πληρωμής',
      'deleted_payment_terms': 'Επιτυχής διαγραφή :value όρων πληρωμής',
      'restored_payment_terms': 'Επιτυχής ανάκτηση :value όρων πληρωμής',
      'email_sign_in': 'Είσοδος με email',
      'change': 'Αλλαγή',
      'change_to_mobile_layout': 'Αλλαγή σε εμφάνιση κινητής συσκευής;',
      'change_to_desktop_layout': 'Αλλαγή σε εμφάνιση Desktop συσκευής;',
      'send_from_gmail': 'Αποστολή μέσω Gmail',
      'reversed': 'Αντιστράφηκε',
      'cancelled': 'Ακυρωμένη',
      'credit_amount': 'Ποσό Πίστωσης',
      'quote_amount': 'Ποσό Προσφοράς',
      'hosted': 'Φιλοξενούμενη',
      'selfhosted': 'Ιδίας Φιλοξενίας',
      'exclusive': 'Δεν συμπεριλαμβάνεται',
      'inclusive': 'Συμπεριλαμβάνεται',
      'hide_menu': 'Απόκρυψη Μενού',
      'show_menu': 'Εμφάνιση Μενού',
      'partially_refunded': 'Μερική Επιστροφή Χρημάτων',
      'search_documents': 'ΑναζήτησηΕγγράφων',
      'search_designs': 'Αναζήτηση Σχεδίων',
      'search_invoices': 'Αναζήτηση Τιμολογίων',
      'search_clients': 'Αναζήτηση Πελατών',
      'search_products': 'Αναζήτηση Προϊόντων',
      'search_quotes': 'Αναζήτηση Προσφορών',
      'search_credits': 'Αναζήτηση Πιστώσεων',
      'search_vendors': 'Αναζήτηση Προμηθευτών',
      'search_users': 'Αναζήτηση Χρηστών',
      'search_tax_rates': 'Αναζήτηση Ποσοστών Φόρου',
      'search_tasks': 'Αναζήτηση Εργασιών',
      'search_settings': 'Αναζήτηση Ρυθμίσεων',
      'search_projects': 'Αναζήτηση Projects',
      'search_expenses': 'Αναζήτηση Δαπανών',
      'search_payments': 'Αναζήτηση Πληρωμών',
      'search_groups': 'Αναζήτηση Γκρουπ',
      'search_company': 'Αναζήτηση Εταιριών',
      'search_document': 'Αναζήτηση 1 Εγγράφου',
      'search_design': 'Αναζήτηση 1 Σχεδίου',
      'search_invoice': 'Αναζήτηση 1 Τιμολογίου',
      'search_client': 'Αναζήτηση 1 Πελάτη',
      'search_product': 'Αναζήτηση 1 Προϊόντος',
      'search_quote': 'Αναζήτηση 1 Προσφοράς',
      'search_credit': 'Αναζήτηση 1 Πίστωσης',
      'search_vendor': 'Αναζήτηση 1 Προμηθευτή',
      'search_user': 'Αναζήτηση 1 Χρήστη',
      'search_tax_rate': 'Αναζήτηση 1 Ποσοστού Φόρου',
      'search_task': 'Αναζήτηση 1 Εργασίας',
      'search_project': 'Αναζήτηση 1 Project',
      'search_expense': 'Αναζήτηση 1 Δαπάνης',
      'search_payment': 'Αναζήτηση 1 Πληρωμής',
      'search_group': 'Αναζήτηση 1 Γκρουπ',
      'refund_payment': 'Επιστροφή Πληρωμής',
      'cancelled_invoice': 'Επιτυχής ακύρωση τιμολογίου',
      'cancelled_invoices': 'Επιτυχής ακύρωση τιμολογίων',
      'reversed_invoice': 'Επιτυχής αντιστροφή τιμολογίου',
      'reversed_invoices': 'Επιτυχής αντιστροφή τιμολογίων',
      'reverse': 'Αντιστροφή',
      'full_name': 'Πλήρες Όνομα',
      'city_state_postal': 'Πόλη/Νομός/Τ.Κ.',
      'postal_city_state': 'ΤΚ/Πόλη/Περιοχή',
      'custom1': 'Πρώτη Προσαρμογή',
      'custom2': 'Δεύτερη Προσαρμογή',
      'custom3': 'Τρίτη Προσαρμογή',
      'custom4': 'Τέταρτη Προσαρμογή',
      'optional': 'Προαιρετικό',
      'license': 'Άδεια Χρήσης',
      'purge_data': 'Εκκαθάριση Δεδομένων',
      'purge_successful': 'Επιτυχής εκκαθάριση δεδομένων επιχείρησης',
      'purge_data_message':
          'Προσοχή: Αυτό θα σβήσει όλα σας τα δεδομένα, χωρίς δυνατότητα αναίρεσης.',
      'invoice_balance': 'Ισοζύγιο Τιμολογίων',
      'age_group_0': '0 - 30 Ημέρες',
      'age_group_30': '30 - 60 Ημέρες',
      'age_group_60': '60 - 90 Ημέρες',
      'age_group_90': '90 - 120 Ημέρες',
      'age_group_120': '120+ Ημέρες',
      'refresh': 'Ανανέωση',
      'saved_design': 'Επιτυχής αποθήκευση σχεδιασμού',
      'client_details': 'Λεπτομέρειες Πελάτη',
      'company_address': 'Διεύθυνση Εταιρείας',
      'invoice_details': 'Στοιχεία Τιμολογίου',
      'quote_details': 'Λεπτομέρειες Προσφοράς',
      'credit_details': 'Λεπτομέρειες Πίστωσης',
      'product_columns': 'Στήλες Προϊόντος',
      'task_columns': 'Στήλες Εργασιών',
      'add_field': 'Προσθήκη Πεδίου',
      'all_events': 'Όλα τα Γεγονότα',
      'permissions': 'Δικαιώματα',
      'none': 'Κανένα',
      'owned': 'Κατέχεται',
      'payment_success': 'Επιτυχία Πληρωμής',
      'payment_failure': 'Αποτυχία Πληρωμής',
      'invoice_sent': ':count τιμολόγιο στάλθηκε',
      'quote_sent': 'Προσφορά στάλθηκε',
      'credit_sent': 'Πίστωση στάλθηκε',
      'invoice_viewed': 'Τιμολόγιο εμφανίστηκε',
      'quote_viewed': 'Προσφορά εμφανίστηκε',
      'credit_viewed': 'Πίστωση εμφανίστηκε',
      'quote_approved': 'Προσφορά έγινε αποδεκτή',
      'receive_all_notifications': 'Αποδοχή όλων των Ειδοποιήσεων',
      'purchase_license': 'Προμήθεια Άδειας Χρήσης',
      'apply_license': 'Εφαρμογή Άδειας Χρήσης',
      'cancel_account': 'Διαγραφή Λογαριασμού',
      'cancel_account_message':
          'Προσοχή: Αυτό θα σβήσει το λογαριασμό σας, χωρίς δυνατότητα αναίρεσης.',
      'delete_company': 'Διαγραφή Επιχείρησης',
      'delete_company_message':
          'Προειδοποίηση: Αυτό θα διαγράψει οριστικά την επιχείρηση, χωρίς αναίρεση.',
      'enabled_modules': 'Ενεργοποιημένες Ενότητες',
      'converted_quote': 'Επιτυχής μετατροπή προσφοράς',
      'credit_design': 'Σχεδιασμός Πιστώσεων',
      'includes': 'Περιλαμβανόμενα',
      'header': 'Επικεφαλίδα',
      'load_design': 'Φόρτωση Σχεδιασμού',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Προσαρμοσμένοι Σχέδια',
      'designs': 'Σχέδια',
      'new_design': 'Νέο σχλεδιο',
      'edit_design': 'Επεξεργασία Σχεδίου',
      'created_design': 'Επιτυχής δημιουργία σχεδιασμού',
      'updated_design': 'Επιτυχής ενημέρωση σχεδιασμού',
      'archived_design': 'Επιτυχής αρχειοθέτηση σχεδιασμού',
      'deleted_design': 'Επιτυχής διαγραφή σχεδιασμού',
      'removed_design': 'Επιτυχής αφαίρεση σχεδιασμού',
      'restored_design': 'Επιτυχής ανάκτηση σχεδιασμού',
      'archived_designs': 'Επιτυχής αρχειοθέτηση :value σχεδίων',
      'deleted_designs': 'Επιτυχής διαγραφή :value σχεδίων',
      'restored_designs': 'Επιτυχής ανάκτηση :value σχεδίων',
      'proposals': 'Προτάσεις',
      'tickets': 'Αιτήματα υποστήριξης',
      'recurring_quotes': 'Επαναλαμβανόμενες Προσφορές',
      'recurring_tasks': 'Επαναλαμβανόμενες Εργασίες',
      'recurring_expenses': 'Επαναλαμβανόμενες Δαπάνες',
      'account_management': 'Διαχείριση Λογαριασμού',
      'credit_date': 'Ημ/νία Πίστωσης',
      'credit': 'Πίστωση',
      'credits': 'Πιστώσεις',
      'new_credit': 'Εισάγετε Πίστωση',
      'edit_credit': 'Επεξεργασία Πίστωσης',
      'created_credit': 'Επιτυχής δημιουργία πίστωσης',
      'updated_credit': 'Επιτυχής ενημέρωση πίστωσης',
      'archived_credit': 'Επιτυχής αρχειοθέτηση πίστωσης',
      'deleted_credit': 'Επιτυχής διαγραφή πίστωσης',
      'removed_credit': 'Επιτυχής αφίαρεση πίστωσης',
      'restored_credit': 'Επιτυχής ανάκτηση πίστωσης',
      'archived_credits': 'Επιτυχής αρχειοθέτηση :count πιστώσεων',
      'deleted_credits': 'Επιτυχής διαγραφή :count πιστώσεων',
      'restored_credits': 'Επιτυχής ανάκτηση :value πιστώσεων',
      'current_version': 'Τρέχουσα έκδοση',
      'latest_version': 'Τελευταία Έκδοση',
      'update_now': 'Ενημέρωση Τώρα',
      'a_new_version_is_available':
          'Υπάρχει διαθέσιμη νεότερη έκδοση της εφαρμογής web',
      'update_available': 'Διαθέσιμη ενημέρωση',
      'app_updated': 'Η ενημέρωση οκοκληρώθηκε με επιτυχία',
      'learn_more': 'Μάθετε περισσότερα',
      'integrations': 'Ενσωματώσεις',
      'tracking_id': 'Κωδικός παρακολούθησης',
      'slack_webhook_url': 'Διεύθυνση URL του Webhook για το Slack',
      'credit_footer': 'Υποσέλιδο Πίστωσης',
      'credit_terms': 'Όροι Πίστωσης',
      'new_company': 'Νέα Εταιρεία',
      'added_company': 'Επιτυχής προσθήκη επιχείρησης',
      'company1': 'Προσαρμοσμένη εταιρεία 1',
      'company2': 'Προσαρμοσμένη εταιρεία 2',
      'company3': 'Προσαρμοσμένη εταιρεία 3',
      'company4': 'Προσαρμοσμένη εταιρεία 4',
      'product1': 'Προσαρμοσμένο Προιόν 1',
      'product2': 'Προσαρμοσμένο Προιόν 2',
      'product3': 'Προσαρμοσμένο Προιόν 3',
      'product4': 'Προσαρμοσμένο Προιόν 4',
      'client1': 'Προσαρμοσμένος Πελάτης 1',
      'client2': 'Προσαρμοσμένος Πελάτης 2',
      'client3': 'Προσαρμοσμένος Πελάτης 3',
      'client4': 'Προσαρμοσμένος Πελάτης 4',
      'contact1': 'Προσαρμοσμένη Επαφή 1',
      'contact2': 'Προσαρμοσμένη Επαφή 2',
      'contact3': 'Προσαρμοσμένη Επαφή 3',
      'contact4': 'Προσαρμοσμένη Επαφή 4',
      'task1': 'Προσαρμοσμένη Εργασία 1',
      'task2': 'Προσαρμοσμένη Εργασία 2',
      'task3': 'Προσαρμοσμένη Εργασία 3',
      'task4': 'Προσαρμοσμένη Εργασία 4',
      'project1': 'Προσαρμοσμένο Εργο 1',
      'project2': 'Προσαρμοσμένο Εργο 2',
      'project3': 'Προσαρμοσμένο Εργο 3',
      'project4': 'Προσαρμοσμένο Εργο 4',
      'expense1': 'Προσαρμοσμένες Δαπάνες 1',
      'expense2': 'Προσαρμοσμένες Δαπάνες 2',
      'expense3': 'Προσαρμοσμένες Δαπάνες 3',
      'expense4': 'Προσαρμοσμένες Δαπάνες 4',
      'vendor1': 'Προσαρμοσμένος Προμηθευτής 1',
      'vendor2': 'Προσαρμοσμένος Προμηθευτής 2',
      'vendor3': 'Προσαρμοσμένος Προμηθευτής 3',
      'vendor4': 'Προσαρμοσμένος Προμηθευτής 4',
      'invoice1': 'Προσαρμοσμένο Τιμολόγιο 1',
      'invoice2': 'Προσαρμοσμένο Τιμολόγιο 2',
      'invoice3': 'Προσαρμοσμένο Τιμολόγιο 3',
      'invoice4': 'Προσαρμοσμένο Τιμολόγιο 4',
      'payment1': 'Προσαρμοσμένη Πληρωμή 1',
      'payment2': 'Προσαρμοσμένη Πληρωμή 2',
      'payment3': 'Προσαρμοσμένη Πληρωμή 3',
      'payment4': 'Προσαρμοσμένη Πληρωμή 4',
      'surcharge1': 'Προσαρμοσμένη Προσαύξηση 1',
      'surcharge2': 'Προσαρμοσμένη Προσαύξηση 2',
      'surcharge3': 'Προσαρμοσμένη Προσαύξηση 3',
      'surcharge4': 'Προσαρμοσμένη Προσαύξηση 4',
      'group1': 'Προσαρμοσένη Ομάδα 1',
      'group2': 'Προσαρμοσένη Ομάδα 2',
      'group3': 'Προσαρμοσένη Ομάδα 3',
      'group4': 'Προσαρμοσένη Ομάδα 4',
      'reset': 'Επαναφορά',
      'number': 'Αριθμός',
      'export': 'Εξαγωγή',
      'chart': 'Διάγραμμα',
      'count': 'Μέτρηση',
      'totals': 'Σύνολα',
      'blank': 'Κενό',
      'day': 'Ημέρα',
      'month': 'Μήνας',
      'year': 'Έτος',
      'subgroup': 'Υποομάδα',
      'is_active': 'Είναι ενεργό',
      'group_by': 'Ομαδοποίηση με',
      'credit_balance': 'Υπόλοιπο Πίστωσης',
      'contact_last_login': 'Τελευταία είσοδος επαφής',
      'contact_full_name': 'Πλήρες ονοματεπώνυμο επαφής',
      'contact_phone': 'Τηλέφωνο Επικοινωνίας',
      'contact_custom_value1': 'Προσαρμοσμένη Τιμή Επαφής 1',
      'contact_custom_value2': 'Προσαρμοσμένη Τιμή Επαφής 2',
      'contact_custom_value3': 'Προσαρμοσμένη Τιμή Επαφής 3',
      'contact_custom_value4': 'Προσαρμοσμένη Τιμή Επαφής 4',
      'shipping_address1': 'Οδός Αποστολής',
      'shipping_address2': 'Διαμέρισμα Αποστολής',
      'shipping_city': 'Πόλη Αποστολής',
      'shipping_state': 'Περιφέρεια Αποστολής',
      'shipping_postal_code': 'Ταχυδρομικός Κώδικας Αποστολής',
      'shipping_country': 'Χώρα Αποστολής',
      'billing_address1': 'Οδός Χρέωσης',
      'billing_address2': 'Διαμέρισμα Χρέωσης',
      'billing_city': 'Πόλη Χρέωσης',
      'billing_state': 'Περιφέρεια Χρέωσης',
      'billing_postal_code': 'Ταχυδρομικός Κωδικός Χρέωσης',
      'billing_country': 'Χώρα Χρέωσης',
      'client_id': 'Id Πελάτη',
      'assigned_to': 'Ανατέθηκε σε',
      'created_by': 'Δημιουργήθηκε από :name',
      'assigned_to_id': 'Ορίστηκε σε Id',
      'created_by_id': 'Δημιουργήθηκε απο Id',
      'add_column': 'Προσθήκη στήλης',
      'edit_columns': 'Επεξεργασία στηλών',
      'columns': 'Στήλες',
      'aging': 'Γήρανση',
      'profit_and_loss': 'Κέρδος και Ζημιά',
      'reports': 'Αναφορές',
      'report': 'Αναφορά',
      'add_company': 'Προσθήκη Εταιρείας',
      'unpaid_invoice': 'Μη Εξοφλημένο Τιμολόγιο',
      'paid_invoice': 'Εξοφλημένο Τιμολόγιο',
      'unapproved_quote': 'Μη Εγκεκριμένη Προσφορά',
      'help': 'Βοήθεια',
      'refund': 'Επιστροφή χρημάτων',
      'refund_date': 'Ημερομηνία επιστροφής χρημάτων',
      'filtered_by': 'Φιλτράρισμα με',
      'contact_email': 'Email Επικοινωνίας',
      'multiselect': 'Πολλαπλή επιλογή',
      'entity_state': 'Περιοχή',
      'verify_password': 'Επαλήθευση Κωδικού',
      'applied': 'Εγινε εφαρμογή',
      'include_recent_errors':
          'Συμπερίληψη πρόσφατων σφαλμάτων απο αρχεία καταγραφής',
      'your_message_has_been_received':
          'Εχουμε λάβει το μήνυμά σας και θα σας απαντήσουμε σύντομα.',
      'message': 'Μήνυμα',
      'from': 'Από',
      'show_product_details': 'Εμφάνιση Λεπτομερειών Προιόντος',
      'show_product_details_help':
          'Συμπερίληψη της περιγραφής και κόστους τιμής στη λίστα προιόντος',
      'pdf_min_requirements': 'Ο κωδικοποιητής PDF απαιτεί :version',
      'adjust_fee_percent': 'Προσαρμογή του ποσοστού του τέλους',
      'adjust_fee_percent_help':
          'Τροποποίηση του ποσοστού του λογαριασμού για τέλος',
      'configure_settings': 'Προσαρμογή Ρυθμίσεων',
      'support_forum': 'φόρουμ υποστήριξης',
      'about': 'Περί',
      'documentation': 'Τεκμηρίωση',
      'contact_us': 'Επικοινωνήστε μαζί μας',
      'subtotal': 'Μερικό Σύνολο',
      'line_total': 'Αξία',
      'item': 'Προϊόν',
      'credit_email': 'Πιστωτικό μήνυμα ηλεκτρονικού ταχυδρομείου',
      'iframe_url': 'Ιστοσελίδα',
      'domain_url': 'Σύνδεσμος URL',
      'password_is_too_short': 'Ο κωδικός πρόσβασης είναι πολύ μικρός',
      'password_is_too_easy':
          'Ο κωδικός πρόσβασης πρέπει να περιέχει ένα κεφαλαίο χαρακτήρα και έναν αριθμό',
      'client_portal_tasks': 'Εργασίες πύλης πελάτη',
      'client_portal_dashboard': 'Πίνακας ελέγχου πύλης πελατών',
      'please_enter_a_value': 'Παρακαλούμε ορίστε μια τιμή',
      'deleted_logo': 'Επιτυχής διαγραφή λογότυπου',
      'yes': 'Ναι',
      'no': 'Όχι',
      'generate_number': 'Δημιουργία Αριθμού',
      'when_saved': 'Οταν αποθηκευτεί',
      'when_sent': 'Οταν αποσταλλεί',
      'select_company': 'Επιλέξτε Εταιρεία',
      'float': 'Float',
      'collapse': 'Συρρίκνωση',
      'show_or_hide': 'Εμφάνιση/απόκρυψη',
      'menu_sidebar': 'Πλευρικό Μενού',
      'history_sidebar': 'Μενού Πλευρικού Ιστορικού',
      'tablet': 'Τάμπλετ',
      'mobile': 'Κινητό',
      'desktop': 'Σταθερός υπολογιστής',
      'layout': 'Εμφάνιση',
      'view': 'Προβολή',
      'module': 'Ενότητα',
      'first_custom': 'Πρώτη Προσαρμογή',
      'second_custom': 'Δεύτερη Προσαρμογή',
      'third_custom': 'Τρίτη Προσαρμογή',
      'show_cost': 'Εμφάνιση Κόστους',
      'show_product_cost': 'Εμφάνιση Κόστους Προιόντος',
      'show_cost_help':
          'Εμφάνιση του πεδίου κόστους προϊόντος για να είναι δυνατή η εύρεση του κέρδους',
      'show_product_quantity': 'Εμφάνισε την Ποσότητα Προϊόντος',
      'show_product_quantity_help':
          'Εμφάνιση του πεδίου ποσότητας προϊόντος, αλλιώς ορισμός σε ένα',
      'show_invoice_quantity': 'Εμφάνισε την Ποσότητα Τιμολογίου',
      'show_invoice_quantity_help':
          'Εμφάνιση του πεδίου ποσότητας γραμμής, αλλιώς ορισμός σε ένα',
      'show_product_discount': 'Εμφάνιση Έκπτωσης Προιόντος',
      'show_product_discount_help': 'Εμφάνιση έκπτωσης του πεδίου γραμμής',
      'default_quantity': 'Προεπιλεγμένη Ποσότητα',
      'default_quantity_help': 'Αυτόματος ορισμός της ποσότητας γραμμής σε ένα',
      'one_tax_rate': 'Ένα Ποσοστό Φόρου',
      'two_tax_rates': 'Δύο Ποσοστά Φόρων',
      'three_tax_rates': 'Τρία Ποσοστά Φόρων',
      'default_tax_rate': 'Προεπιλεγμένο Ποσοστό Φόρου',
      'user': 'Χρήστης',
      'invoice_tax': 'Φόρος Τιμολογίου',
      'line_item_tax': 'Ποσοστό Φόρου Γραμμής',
      'inclusive_taxes': 'Συμπεριλαμβανόμενοι Φόροι',
      'invoice_tax_rates': 'Φόροι Τιμολογίου',
      'item_tax_rates': 'Ποσοστά Φόρων Προϊόντος',
      'no_client_selected': 'Παρακαλώ επιλέξτε πελάτη',
      'configure_rates': 'Προσαρμογή ποσοστών',
      'configure_gateways': 'Παραμετροποίηση Πυλών Πληρωμών (Gateways)',
      'tax_settings': 'Ρυθμίσεις Φόρων',
      'tax_settings_rates': 'Ποσοστά Φόρων',
      'accent_color': 'Χρώμα Τονισμού',
      'switch': 'Εναλλαγή',
      'comma_sparated_list': 'Λίστα διαχωριζόμενη με κόμματα',
      'options': 'Επιλογές',
      'single_line_text': 'Κείμενο μονής γραμμής',
      'multi_line_text': 'Κείμενο πολλαπλών γραμμών',
      'dropdown': 'Πτυσώμενο',
      'field_type': 'Τύπος Πεδίου',
      'recover_password_email_sent':
          'Ένα email ανάκτησης κωδικού έχει αποσταλεί',
      'submit': 'Υποβολή',
      'recover_password': 'Ανάκτηση του κωδικού  πρόσβασής σας',
      'late_fees': 'Καθυστερούμενα Τέλη',
      'credit_number': 'Αριθμός Πίστωσης',
      'payment_number': 'Αριθμός Πληρωμής',
      'late_fee_amount': 'Ποσό Τέλους Καθυστερημένης Εξόφλησης',
      'late_fee_percent': 'Ποσοστό Τέλους Καθυστερημένης Εξόφλησης',
      'schedule': 'Προγραμμάτισε',
      'before_due_date': 'Πριν την ημερομηνία πληρωμής',
      'after_due_date': 'Μετά την ημερομηνία πληρωμής',
      'after_invoice_date': 'Μετά την ημερομηνία τιμολογίου',
      'days': 'Ημέρες',
      'invoice_email': 'Email Τιμολογίων',
      'payment_email': 'Email Πληρωμών',
      'partial_payment': 'Μερική Πληρωμή',
      'payment_partial': 'Μερική Πληρωμή',
      'partial_payment_email': 'Email Μερικής Πληρωμής',
      'quote_email': 'Email Προσφορών',
      'endless_reminder': 'Συνεχής Υπενθύμιση',
      'filtered_by_user': 'Φιλτράρισμα από το Χρήστη',
      'administrator': 'Διαχειριστής',
      'administrator_help':
          'Επιτρέψτε στο χρήστη να διαχειρίζεται χρήστες, να αλλάζει ρυθμίσεις και να τροποποιεί όλες τις εγγραφές',
      'user_management': 'Διαχειριση Χρηστών',
      'users': 'Χρήστες',
      'new_user': 'Νέος Χρήστης',
      'edit_user': 'Επεξεργασία Χρήστη',
      'created_user': 'Επιτυχής δημιουργία χρήστη',
      'updated_user': 'Επιτυχής ενημέρωση χρήστη',
      'archived_user': 'Επιτυχής αρχειοθέτηση χρήστη',
      'deleted_user': 'Επιτυχής διαγραφή χρήστη',
      'removed_user': 'Επιτυχής αφαίρεση χρήστη',
      'restored_user': 'Επιτυχής ανάκτηση χρήστη',
      'archived_users': 'Επιτυχής αρχειοθέτηση :value χρηστών',
      'deleted_users': 'Επιτυχής διαγραφή :value χρηστών',
      'removed_users': 'Επιτυχής αφαίρεση :value χρηστών',
      'restored_users': 'Επιτυχής ανάκτηση :value χρηστών',
      'general_settings': 'Γενικές Ρυθμίσεις',
      'invoice_options': 'Επιλογές Τιμολογίου',
      'hide_paid_to_date': 'Απόκρυψη Εξοφλημένου Ποσού',
      'hide_paid_to_date_help':
          'Εμφάνιση πεδίου \"Εξοφλημένο Ποσό\" μόνο στο παραστατικό όταν ληφθεί μια πληρωμή.',
      'invoice_embed_documents': 'Ενσωματωμένα Έγγραφα',
      'invoice_embed_documents_help':
          'Συμπεριλάβετε τις συνημμένες εικόνες στο τιμολόγιο',
      'all_pages_header': 'Εμφάνιση Κεφαλίδας',
      'all_pages_footer': 'Εμφάνιση Υποσέλιδου',
      'first_page': 'Πρώτη σελίδα',
      'all_pages': 'Προσθήκη Σελίδας',
      'last_page': 'Τελευταία σελίδα',
      'primary_font': 'Κύρια Γραμματοσειρά',
      'secondary_font': 'Δευτερεύουσα Γραμματοσειρά',
      'primary_color': 'Κύριο Χρώμα',
      'secondary_color': 'Δευτερεύον Χρώμα',
      'page_size': 'Μέγεθος Σελίδας',
      'font_size': 'Μέγεθος Γραμμάτων',
      'quote_design': 'Σχεδιασμός Προσφοράς',
      'invoice_fields': 'Πεδία Τιμολογίου',
      'product_fields': 'Πεδία Προϊόντος',
      'invoice_terms': 'Όροι Τιμολογίου',
      'invoice_footer': 'Υποσέλιδο Τιμολογίου',
      'quote_terms': 'Όροι Προσφοράς',
      'quote_footer': 'Υποσέλιδο Προσφοράς',
      'auto_email_invoice': 'Αυτόματο Email',
      'auto_email_invoice_help':
          'Αυτόματη αποστολή επαναλαμβανόμενων τιμολογίων με email όταν δημιουργηθούν.',
      'auto_archive_invoice': 'Αυτόματη Αρχειοθέτηση',
      'auto_archive_invoice_help':
          'Αυτόματη αρχειοθέτηση τιμολογίων όταν εξοφληθούν.',
      'auto_archive_quote': 'Αυτόματη Αρχειοθέτηση',
      'auto_archive_quote_help':
          'Αυτόματη αρχειοθέτηση προσφορών όταν μετατραπούν.',
      'auto_convert_quote': 'Αυτόματη Μετατροπή',
      'auto_convert_quote_help':
          'Αυτόματη μετατροπή της προσφοράς σε τιμολόγιο μόλις γίνει αποδεκτή από τον πελάτη.',
      'workflow_settings': 'Ρυθμίσεις Ροής Εργασιών',
      'freq_daily': 'Ημερήσιο',
      'freq_weekly': 'Εβδομάδα',
      'freq_two_weeks': 'Δύο εβδομάδες',
      'freq_four_weeks': 'Τέσσερις εβδομάδες',
      'freq_monthly': 'Μήνας',
      'freq_two_months': 'Δύο μήνες',
      'freq_three_months': 'Τρεις μήνες',
      'freq_four_months': 'Τέσσερις μήνες',
      'freq_six_months': 'Έξι μήνες',
      'freq_annually': 'Έτος',
      'freq_two_years': 'Δύο χρόνια',
      'freq_three_years': 'Τρία Χρόνια',
      'never': 'Ποτέ',
      'company': 'Εταιρεία',
      'generated_numbers': 'Δημιουργημένοι Αριθμοί',
      'charge_taxes': 'Χρέωση φόρων',
      'next_reset': 'Επόμενη επανεκκίνηση',
      'reset_counter': 'Επανεκκίνηση Μετρητή',
      'recurring_prefix': 'Επαναλαμβανόμενο Πρόθεμα',
      'number_padding': 'Περιθώριο Αρίθμησης',
      'general': 'Γενικός',
      'surcharge_field': 'Πεδίο Επιβάρυνσης',
      'company_field': 'Πεδίο Εταιρείας',
      'company_value': 'Αξία Εταιρίας',
      'credit_field': 'Πεδίο Πίστωσης',
      'invoice_field': 'Πεδίο Τιμολογίου',
      'invoice_surcharge': 'Επιβάρυνση Τιμολογίου',
      'client_field': 'Πεδίο Πελάτη',
      'product_field': 'Πεδίο Προϊόντος',
      'payment_field': 'Πεδίο Πληρωμής',
      'contact_field': 'Πεδίο Επαφής',
      'vendor_field': 'Πεδίο Προμηθευτή',
      'expense_field': 'Πεδίο Δαπάνης',
      'project_field': 'Πεδίο Project',
      'task_field': 'Πεδίο Εργασίας',
      'group_field': 'Πεδίο Γκρουπ',
      'number_counter': 'Μετρητής Αρίθμησης',
      'prefix': 'Πρόθεμα',
      'number_pattern': 'Μοτίβο Αρίθμησης',
      'messages': 'Μηνύματα',
      'custom_css': 'Προσαρμοσμένο CSS',
      'custom_javascript': 'Προσαρμοσμένη JavaScript',
      'signature_on_pdf': 'Εμφάνισε στο PDF',
      'signature_on_pdf_help':
          'Εμφάνισε την υπογραφή του πελάτη στο PDF του τιμολογίου/προσφοράς.',
      'show_accept_invoice_terms': 'Κουτάκι Όρων Τιμολογίου',
      'show_accept_invoice_terms_help':
          'Απαίτηση από τον πελάτη να αποδεχθεί τους όρους του τιμολογίου',
      'show_accept_quote_terms': 'Κουτάκι Όρων Προσφοράς',
      'show_accept_quote_terms_help':
          'Απαίτηση από τον πελάτη να αποδεχθεί τους όρους της προσφοράς',
      'require_invoice_signature': 'Υπογραφή Τιμολογίου',
      'require_invoice_signature_help':
          'Απαίτηση από τον πελάτη να συμπληρώσει την υπογραφή του.',
      'require_quote_signature': 'Υπογραφή Προσφοράς',
      'enable_portal_password': 'Προστασία Τιμολογίων με Κωδικό Πρόσβασης',
      'enable_portal_password_help':
          'Επιτρέπει τον καθορισμό κωδικού πρόσβασης για κάθε επαφή. Αν έχει καθοριστεί κωδικός, η επαφή θα υποχρεούται να καταχωρήσει κωδικό πριν την προβολή των τιμολογίων.',
      'authorization': 'Εξουσιοδότηση',
      'subdomain': 'Υποτομέας',
      'domain': 'Domain',
      'portal_mode': 'Περιβάλλον Portal',
      'email_signature': 'Με εκτίμηση,',
      'enable_email_markup_help':
          'Κάντε τη διαδικασία πληρωμής πιο εύκολη για τους πελάτες σας προσθέτοντας σήμανση από το schema.org στα emails σας.',
      'plain': 'Απλό',
      'light': 'Ανοιχτό',
      'dark': 'Σκούρο',
      'email_design': 'Σχεδίαση Email',
      'attach_pdf': 'Επισύναψε PDF',
      'attach_documents': 'Επισύναψη Εγγράφων',
      'attach_ubl': 'Επισύναψη UBL',
      'email_style': 'Στυλ Email',
      'enable_email_markup': 'Ενεργοποίηση Σημανσης',
      'reply_to_email': 'Email Απάντησης',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Email ιδιαίτερης κοινοποίησης',
      'processed': 'Επεξεργάσθηκε',
      'credit_card': 'Πιστωτική Κάρτα',
      'bank_transfer': 'Τραπεζικό Έμβασμα',
      'priority': 'Προτεραιότητα',
      'fee_amount': 'Ποσό Τέλους',
      'fee_percent': 'Ποσοστό Τέλους',
      'fee_cap': 'Ανώτατο Όριο Τέλους',
      'limits_and_fees': 'Όρια/Τέλη',
      'enable_min': 'Ενεργοποίηση ελάχιστου',
      'enable_max': 'Ενεργοποίηση μέγιστου',
      'min_limit': 'Ελάχιστο: :min',
      'max_limit': 'Μέγιστο: :max',
      'min': 'Ελάχιστο',
      'max': 'Μέγιστο',
      'accepted_card_logos': 'Λογότυπα Αποδεκτών Καρτών',
      'credentials': 'Στοιχεία εισόδου',
      'update_address': 'Ενημέρωση Διεύθυνσης',
      'update_address_help':
          'Ενημέρωση της διεύθυνσης του πελάτη με τα παρεχόμενα στοιχεία',
      'rate': 'Ποσοστό',
      'tax_rate': 'Ποσοστό Φόρου',
      'new_tax_rate': 'Νέο Ποσοστό Φόρου',
      'edit_tax_rate': 'Επεξεργασία ποσοστού φόρου',
      'created_tax_rate': 'Επιτυχής δημιουργία ποσοστού φόρου',
      'updated_tax_rate': 'Επιτυχής ενημέρωση ποσοστού φόρου',
      'archived_tax_rate': 'Επιτυχής αρχειοθέτηση ποσοστού φόρου',
      'deleted_tax_rate': 'Επιτυχής διαγραφή ποσοστού φόρου',
      'restored_tax_rate': 'Επιτυχής ανάκτηση ποσοστού φόρου',
      'archived_tax_rates': 'Επιτυχής αρχειοθέτηση :value ποσοστών φόρου',
      'deleted_tax_rates': 'Επιτυχής διαγραφή :value ποσοστών φόρου',
      'restored_tax_rates': 'Επιτυχής ανάκτηση :value ποσοστών φόρου',
      'fill_products': 'Αυτόματη συμπλήρωση προϊόντων',
      'fill_products_help':
          'Επιλέγοντας ένα προϊόν, αυτόματα θα συμπληρωθεί η περιγραφή και η αξία',
      'update_products': 'Αυτόματη ενημέρωση προϊόντων',
      'update_products_help':
          'Ενημερώνοντας ένα τιμολόγιο, αυτόματα θα ενημερωθεί και η βιβλιοθήκη προϊόντων',
      'convert_products': 'Μετατροπή Τιμών Προϊόντων',
      'convert_products_help':
          'Αυτόματη μετατροπή τιμών προϊόντων στο νόμισμα συναλλαγών του πελάτη',
      'fees': 'Τέλη',
      'limits': 'Όρια',
      'provider': 'Provider',
      'company_gateway': 'Πύλη Πληρωμών (Gateway)',
      'company_gateways': 'Πύλες Πληρωμών (Gateways)',
      'new_company_gateway': 'Νέα Πύλη πληρωμής (Gateway)',
      'edit_company_gateway': 'Επεξεργασία Πύλης Πληρωμών (Gateway)',
      'created_company_gateway': 'Επιτυχής δημιουργία πύλης πληρωμών (Gateway)',
      'updated_company_gateway': 'Επιτυχής ενημέρωση πύλης πληρωμών (Gateway)',
      'archived_company_gateway':
          'Επιτυχής αρχειοθέτηση πύλης πληρωμών (Gateway)',
      'deleted_company_gateway': 'Επιτυχής διαγραφή πύλης πληρωμών (Gateway)',
      'restored_company_gateway': 'Επιτυχής επαναφορά πύλης πληρωμών (Gateway)',
      'archived_company_gateways':
          'Επιτυχής αρχειοθέτηση :value πυλών πληρωμών (gateways)',
      'deleted_company_gateways':
          'Επιτυχής διαγραφή :value πυλών πληρωμών (gateways)',
      'restored_company_gateways':
          'Επιτυχής ανάκτηση :value πυλών πληρωμών (gateways)',
      'continue_editing': 'Συνεχίστε την Επεξεργασία',
      'discard_changes': 'Απόρριψη Αλλαγών',
      'default_value': 'Προεπιλεγμένη τιμή',
      'disabled': 'Απενεργοποιημένο',
      'currency_format': 'Μορφή Νομίσματος',
      'first_day_of_the_week': 'Πρώτη Μέρα της Εβδομάδας',
      'first_month_of_the_year': 'Πρώτος Μήνας του Έτους',
      'sunday': 'Κυριακή',
      'monday': 'Δευτέρα',
      'tuesday': 'Τρίτη',
      'wednesday': 'Τετάρτη',
      'thursday': 'Πέμπτη',
      'friday': 'Παρασκευή',
      'saturday': 'Σάββατο',
      'january': 'Ιανουάριος',
      'february': 'Φεβρουάριος',
      'march': 'Μάρτιος',
      'april': 'Απρίλιος',
      'may': 'Μάιος',
      'june': 'Ιούνιος',
      'july': 'Ιούλιος',
      'august': 'Αύγουστος',
      'september': 'Σεπτέμβριος',
      'october': 'Οκτώβριος',
      'november': 'Νοέμβριος',
      'december': 'Δεκέμβριος',
      'symbol': 'Σύμβολο',
      'ocde': 'Κωδικός',
      'date_format': 'Μορφή Ημερομηνίας',
      'datetime_format': 'Μορφή Ημερομηνίας/Ώρας',
      'military_time': '24ωρη εμφάνιση Ώρας',
      'military_time_help': '24ωρη εμφάνιση Ώρας',
      'send_reminders': 'Αποστολή Υπενθυμίσεων',
      'timezone': 'Ζώνη ώρας',
      'filtered_by_project': 'Φιλτράρισμα ανά Project',
      'filtered_by_group': 'Φιλτράρισμα ανά Γκρουπ',
      'filtered_by_invoice': 'Φιλτράρισμα ανά Τιμολόγιο',
      'filtered_by_client': 'Φιλτράρισμα ανά Πελάτη',
      'filtered_by_vendor': 'Φιλτράρισμα ανά Προμηθευτή',
      'group_settings': 'Ρυθμίσεις Γρουπ',
      'group': 'Ομάδα',
      'groups': 'Γρουπ',
      'new_group': 'Νέο Γκρουπ',
      'edit_group': 'Επεξεργασία Γκρουπ',
      'created_group': 'Επιτυχής δημιουργία γκρουπ',
      'updated_group': 'Επιτυχής ενημέρωση γκρουπ',
      'archived_groups': 'Επιτυχής αρχειοθέτηση :value γκρουπς',
      'deleted_groups': 'Επιτυχής διαγραφή :value γκρουπς',
      'restored_groups': 'Επιτυχής ανάκτηση :value γκρουπς',
      'upload_logo': 'Μεταφόρτωση Λογοτύπου',
      'uploaded_logo': 'Επιτυχής μεταφόρτωση λογοτύπου',
      'logo': 'Λογότυπο',
      'saved_settings': 'Επιτυχής αποθήκευση ρυθμίσεων',
      'product_settings': 'Ρυθμίσεις Προϊόντων',
      'device_settings': 'Ρυθμίσεις Συσκευής',
      'defaults': 'Προεπιλογές',
      'basic_settings': 'Βασικές Ρυθμίσεις',
      'advanced_settings': 'Ρυθμίσεις για Προχωρημένους',
      'company_details': 'Στοιχεία Εταιρείας',
      'user_details': 'Στοιχεία Χρήστη',
      'localization': 'Τοπικές Ρυθμίσεις',
      'online_payments': 'Πληρωμές Online',
      'tax_rates': 'Ποσοστά Φόρων',
      'notifications': 'Ειδοποιήσεις',
      'import_export': 'Εισαγωγή | Εξαγωγή',
      'custom_fields': 'Προσαρμοσμένα Πεδία',
      'invoice_design': 'ΣχέδιοΤιμολογίου',
      'buy_now_buttons': 'Κουμπιά Αγορά Τώρα',
      'email_settings': 'Ρυθμίσεις Email',
      'templates_and_reminders': 'Πρότυπα & Υπενθυμίσεις',
      'credit_cards_and_banks': 'Πιστωτικές Κάρτες & Τράπεζες',
      'data_visualizations': 'Απεικονίσεις Δεδομένων',
      'price': 'Τιμή',
      'email_sign_up': 'Εγγραφή μέσω Email',
      'google_sign_up': 'Εγγραφή μέσω Google',
      'thank_you_for_your_purchase': 'Ευχαριστούμε για την αγορά σας!',
      'redeem': 'Εξαργύρωσε',
      'back': 'Πίσω',
      'past_purchases': 'Παρελθόντες Αγορές',
      'annual_subscription': 'Ετη΄σια Συνδρομή',
      'pro_plan': 'Επαγγελματικό Πλάνο',
      'enterprise_plan': 'Εταιρικό Πλάνο',
      'count_users': ':count χρήστες',
      'upgrade': 'Αναβάθμιση',
      'please_enter_a_first_name': 'Παρακαλούμε εισάγετε μικρό όνομα',
      'please_enter_a_last_name': 'Παρακαλούμε εισάγετε επώνυμο',
      'please_agree_to_terms_and_privacy':
          'Παρακαλούμε συμφωνήστε με τους όρους χρήσης και την πολιτική απορρήτου για να δημιουργήσετε ένα λογαριασμό.',
      'i_agree_to_the': 'Συμφωνώ με το',
      'terms_of_service_link': 'όροι της υπηρεσίας',
      'privacy_policy_link': 'πολιτική απορρήτου',
      'terms_of_service': 'Όροι της Υπηρεσίας',
      'privacy_policy': 'Πολιτική Απορρήτου',
      'sign_up': 'Εγγραφή',
      'account_login': 'Είσοδος στο Λογαριασμό',
      'view_website': 'Εμφάνιση Ιστοσελίδας',
      'create_account': 'Δημιουργία Λογαριασμού',
      'email_login': 'Είσοδος με Email',
      'create_new': 'Δημιουργία Νέου',
      'no_record_selected': 'Δεν έχουν επιλεγεί πεδία.',
      'error_unsaved_changes': 'Παρακαλούμε σώστε ή ακυρώστε τις αλλαγές σας.',
      'download': 'Κατέβασμα',
      'requires_an_enterprise_plan': 'Απαιτεί ένα επαγγελματικό πλάνο',
      'take_picture': 'Λήψη Φωτογραφίας',
      'upload_file': 'Μεταφόρτωση Αρχείου',
      'document': 'Έγγραφο',
      'documents': 'Έγγραφα',
      'new_document': 'Νέο έγγραφο',
      'edit_document': 'Επεξεργασία Εγγράφου',
      'uploaded_document': 'Επιτυχής μεταφόρτωση αρχείου',
      'updated_document': 'Επιτυχής ενημέρωση αρχείου',
      'archived_document': 'Επιτυχής αρχειοθέτηση αρχείου',
      'deleted_document': 'Επιτυχής διαγραφή αρχείου',
      'restored_document': 'Επιτυχής επαναφορά αρχείου',
      'archived_documents': 'Επιτυχής αρχειοθέτηση :value εγγράφων',
      'deleted_documents': 'Επιτυχής διαγραφή :value εγγράφων',
      'restored_documents': 'Επιτυχής ανάκτηση :value εγγράφων',
      'no_history': 'Δεν υπάρχει ιστορικό',
      'expense_date': 'Ημερομηνία Δαπάνης',
      'pending': 'Εκκρεμής',
      'expense_status_1': 'Καταγεγραμμένο',
      'expense_status_2': 'Σε αναμονή',
      'expense_status_3': 'Τιμολογημένο',
      'converted': 'Μετατράπηκε',
      'add_documents_to_invoice': 'Προσθέστε έγγραφα στο τιμολόγιο',
      'exchange_rate': 'Ισοτιμία Ανταλλαγής',
      'convert_currency': 'Μετατροπή νομίσματος',
      'mark_paid': 'Όρισε ως Πληρωμένα',
      'category': 'Κατηγορία',
      'address': 'Διεύθυνση',
      'new_vendor': 'Νέος Προμηθευτής',
      'created_vendor': 'Επιτυχής δημιουργία προμηθευτή',
      'updated_vendor': 'Επιτυχής ενημέρωση προμηθευτή',
      'archived_vendor': 'Επιτυχής αρχειοθέτηση προμηθευτή',
      'deleted_vendor': 'Επιτυχής διαγραφή προμηθευτή',
      'restored_vendor': 'Ο Προμηθευτής ανακτήθηκε με επιτυχία',
      'archived_vendors': 'Επιτυχής αρχειοθέτηση :count προμηθευτών',
      'deleted_vendors': 'Επιτυχής διαγραφή :count προμηθευτών',
      'restored_vendors': 'Επιτυχής ανάκτηση :value προμηθευτών',
      'new_expense': 'Καταχώρηση Δαπάνης',
      'created_expense': 'Επιτυχής δημιουργία δαπάνης',
      'updated_expense': 'Επιτυχής ενημέρωση δαπάνης',
      'archived_expense': 'Επιτυχής αρχειοθέτηση δαπάνης',
      'deleted_expense': 'Επιτυχής διαγραφή δαπάνης',
      'restored_expense': 'Επιτυχής επαναφορά δαπάνης',
      'archived_expenses': 'Επιτυχής αρχειοθέτηση δαπανών',
      'deleted_expenses': 'Επιτυχής διαγραφή δαπανών',
      'restored_expenses': 'Επιτυχής ανάκτηση :value δαπανών',
      'copy_shipping': 'Αντιγραφή Αποστολής',
      'copy_billing': 'Αντιγραφή Χρέωσης',
      'design': 'Σχεδίαση',
      'failed_to_find_record': 'Αποτυχία ανεύρεσης εγγραφής',
      'invoiced': 'Τιμολογημένα',
      'logged': 'Εισηγμένο',
      'running': 'Εκτελείται',
      'resume': 'Συνέχισε',
      'task_errors': 'Παρακαλώ διορθώστε τυχόν επικαλυπτόμενες ώρες',
      'start': 'Έναρξη',
      'stop': 'Λήξη',
      'started_task': 'Επιτυχής έναρξη εργασίας',
      'stopped_task': 'Επιτυχής διακοπή εργασίας',
      'resumed_task': 'Επιτυχής επανέναρξη εργασίας',
      'now': 'Τώρα',
      'auto_start_tasks': 'Αυτόματη Έναρξη Εργασιών',
      'timer': 'Μετρητής',
      'manual': 'Χειροκίνητο',
      'budgeted': 'Προϋπολογισμένο',
      'start_time': 'Ώρα Έναρξης',
      'end_time': 'Ώρα Λήξης',
      'date': 'Ημερομηνία',
      'times': 'Φορές',
      'duration': 'Διάρκεια',
      'new_task': 'Νέα Εργασία',
      'created_task': 'Επιτυχής δημιουργία εργασίας',
      'updated_task': 'Επιτυχής ενημέρωση εργασίας',
      'archived_task': 'Επιτυχής αρχειοθέτηση εργασίας',
      'deleted_task': 'Επιτυχής διαγραφή εργασίας',
      'restored_task': 'Επιτυχής ανάκτηση εργασίας',
      'archived_tasks': 'Επιτυχής αρχειοθέτηση :count εργασιών',
      'deleted_tasks': 'Επιτυχής διαγραφή :count εργασιών',
      'restored_tasks': 'Επιτυχής ανάκτηση :value εργασιών',
      'please_enter_a_name': 'Παρακαλούμε εισάγετε ένα όνομα',
      'budgeted_hours': 'Χρεώσιμες Ώρες',
      'created_project': 'Επιτυχής δημιουργία project',
      'updated_project': 'Επιτυχής ενημέρωση project',
      'archived_project': 'Επιτυχής αρχειοθέτηση project',
      'deleted_project': 'Επιτυχής διαγραφή project',
      'restored_project': 'Επιτυχής ανάκτηση project',
      'archived_projects': 'Επιτυχής αρχειοθέτηση :count projects',
      'deleted_projects': 'Επιτυχής διαγραφή :count projects',
      'restored_projects': 'Επιτυχής ανάκτηση :value projects',
      'new_project': 'Νέο Project',
      'thank_you_for_using_our_app':
          'Ευχαριστούμε που χρησιμοποιήσατε την εφαρμογή μας!',
      'if_you_like_it': 'Εάν σας αρέσει παρακαλούμε',
      'click_here': 'πατήστε εδώ',
      'click_here_capital': 'Πατήστε εδώ',
      'to_rate_it': 'για να το αξιολογήσετε.',
      'average': 'Μέσος όρος',
      'unapproved': 'Μη εγκεκριμένη',
      'authenticate_to_change_setting':
          'Παρακαλούμε αυθεντικοποιήστε για να αλλάξετε αυτή τη ρύθμιση',
      'locked': 'Κλειδωμένη',
      'authenticate': 'Αυθεντικοποιήστε',
      'please_authenticate': 'Παρακαλούμε αυθεντικοποιήστε',
      'biometric_authentication': 'Βιομετρικη αυθεντικοποίηση',
      'footer': 'Υποσέλιδο',
      'compare': 'Σύγκρινε',
      'hosted_login': 'Εισαγωγή σε φιλοξενούμενη έκδοση',
      'selfhost_login': 'Εισαγωγή σε αυτο-φιλοξενούμενη έκδοση',
      'google_sign_in': 'Είσοδος μέσω Google',
      'today': 'Σήμερα',
      'custom_range': 'Προσαρμοσμένο Εύρος',
      'date_range': 'Εύρος Ημερομηνιών',
      'current': 'Τωρινή',
      'previous': 'Προηγούμενη',
      'current_period': 'Τωρινή Περίοδος',
      'comparison_period': 'Περίοδος Σύγκρισης',
      'previous_period': 'Προηγούμενη Περίοδος',
      'previous_year': 'Προηγούμενος Χρόνος',
      'compare_to': 'Σύγκριση με',
      'last7_days': 'Προηγούμενες 7 ημέρες',
      'last_week': 'Προηγούμενη Εβδομάδα',
      'last30_days': 'Τελευταίες 30 Ημέρες',
      'this_month': 'Αυτός ο Μήνας',
      'last_month': 'Προηγούμενος Μήνας',
      'this_year': 'Τρέχον Χρόνος',
      'last_year': 'Προηγούμενος Χρόνος',
      'custom': 'Προσαρμοσμένο',
      'clone_to_invoice': 'Κλωνοποίηση σε Τιμολόγιο',
      'clone_to_quote': 'Κλωνοποίηση σε Προσφορά',
      'clone_to_credit': 'Κλωνοποίηση σε Πίστωση',
      'view_invoice': 'Προβολή Τιμολογίου',
      'convert': 'Μετατροπή',
      'more': 'Περισσότερα',
      'edit_client': 'Επεξεργασία Πελάτη',
      'edit_product': 'Επεξεργασία Προϊόντος',
      'edit_invoice': 'Επεξεργασία Τιμολογίου',
      'edit_quote': 'Επεξεργασία Προσφοράς',
      'edit_payment': 'Επεξεργασία Πληρωμής',
      'edit_task': 'Επεξεργασία Εργασίας',
      'edit_expense': 'Επεξεργασία Δαπάνης',
      'edit_vendor': 'Επεξεργασία Προμηθευτή',
      'edit_project': 'Επεξεργασία Project',
      'edit_recurring_expense': 'Επεξεργασία Επαναλαμβανόμενης Δαπάνης',
      'edit_recurring_quote': 'Επεξεργασία Επαναλαμβανόμενων Προσφορών',
      'billing_address': 'Διεύθυνση Τιμολόγησης',
      'shipping_address': 'Διεύθυνση Αποστολής',
      'total_revenue': 'Συνολικά Έσοδα',
      'average_invoice': 'Μέσος Όρος Τιμολογίων',
      'outstanding': 'Εκκρεμή',
      'invoices_sent': ':count τιμολόγια στάλθηκαν',
      'active_clients': 'ενεργοί πελάτες',
      'close': 'Κλείσιμο',
      'email': 'Email',
      'password': 'Κωδικός Πρόσβασης',
      'url': 'URL',
      'secret': 'Κρυφό',
      'name': 'Επωνυμία',
      'logout': 'Αποσύνδεση',
      'login': 'Είσοδος',
      'filter': 'Φίλτρο',
      'sort': 'Ταξινόμηση',
      'search': 'Αναζήτηση',
      'active': 'Ενεργός',
      'archived': 'Αρχειοθετημένο',
      'deleted': 'Διεγραμμένο',
      'dashboard': 'Πίνακας ελέγχου',
      'archive': 'Αρχειοθέτηση',
      'delete': 'Διαγραφή',
      'restore': 'Ανάκτηση',
      'refresh_complete': 'Ανανέωση Ολοκληρώθηκε',
      'please_enter_your_email': 'Παρακαλώ εισάγετε το email σας',
      'please_enter_your_password':
          'Παρακαλώ εισάγετε τον κωδικό πρόσβασής σας',
      'please_enter_your_url': 'Παρακαλώ εισάγετε το URL σας',
      'please_enter_a_product_key': 'Παρακαλώ εισάγετε το κλειδί προϊόντος σας',
      'ascending': 'Αύξουσα σειρά',
      'descending': 'Φθίνουσα σειρά',
      'save': 'Αποθήκευση',
      'an_error_occurred': 'Εμφανίστηκε ένα σφάλμα.',
      'paid_to_date': 'Εξοφλημένο Ποσό',
      'balance_due': 'Ολικό Σύνολο',
      'balance': 'Υπόλοιπο',
      'overview': 'Επισκόπηση',
      'details': 'Στοιχεία',
      'phone': 'Τηλέφωνο',
      'website': 'Ιστοσελίδα',
      'vat_number': 'ΑΦΜ',
      'id_number': 'Αριθμός ID',
      'create': 'Δημιουργία',
      'copied_to_clipboard': 'Αντιγράφτηκε :value στο πρόχειρο',
      'error': 'Σφάλμα',
      'could_not_launch': 'Αδύνατη η εκτέλεση',
      'contacts': 'Επαφές',
      'additional': 'Επιπρόσθετο',
      'first_name': 'Όνομα',
      'last_name': 'Επώνυμο',
      'add_contact': 'Προσθήκη επαφής',
      'are_you_sure': 'Είστε σίγουροι;',
      'cancel': 'Άκυρο',
      'ok': 'Ok',
      'remove': 'Διαγραφή',
      'email_is_invalid': 'Το Email είναι εσφαλμένο',
      'product': 'Προϊόν',
      'products': 'Προϊόντα',
      'new_product': 'Νέο Προϊόν',
      'created_product': 'Επιτυχής δημιουργία προϊόντος',
      'updated_product': 'Επιτυχής ενημέρωση προϊόντος',
      'archived_product': 'Επιτυχής αρχειοθέτηση προϊόντος',
      'deleted_product': 'Επιτυχής διαγραφή προϊόντος',
      'restored_product': 'Επιτυχής ανάκτηση προϊόντος',
      'archived_products': 'Επιτυχής αρχειοθέτηση :count προϊόντων',
      'deleted_products': 'Επιτυχής διαγραφή :count προϊόντων',
      'restored_products': 'Επιτυχής ανάκτηση :value προϊόντων',
      'product_key': 'Προϊόν',
      'notes': 'Σημειώσεις',
      'cost': 'Κόστος',
      'client': 'Πελάτης',
      'clients': 'Πελάτες',
      'new_client': 'Νέος Πελάτης',
      'created_client': 'Επιτυχής δημιουργία πελάτη',
      'updated_client': 'Επιτυχής ενημέρωση πελάτη',
      'archived_client': 'Επιτυχής αρχειοθέτηση πελάτη',
      'archived_clients': 'Επιτυχής αρχειοθέτηση :count πελατών',
      'deleted_client': 'Επιτυχής διαγραφή πελάτη',
      'deleted_clients': 'Επιτυχής διαγραφή :count πελατών',
      'restored_client': 'Επιτυχής ανάκτηση πελάτη',
      'restored_clients': 'Επιτυχής ανάκτηση :value πελατών',
      'address1': 'Οδός',
      'address2': 'Διαμέρισμα',
      'city': 'Πόλη',
      'state': 'Νομός',
      'postal_code': 'Ταχ. Κώδικας',
      'country': 'Χώρα',
      'invoice': 'Τιμολόγιο',
      'invoices': 'Τιμολόγια',
      'new_invoice': 'Νέο Τιμολόγιο',
      'created_invoice': 'Επιτυχής δημιουργία τιμολογίου',
      'updated_invoice': 'Επιτυχής ενημέρωση τιμολογίου',
      'archived_invoice': 'Επιτυχής αρχειοθέτηση τιμολογίου',
      'deleted_invoice': 'Επιτυχής διαγραφή τιμολογίου',
      'restored_invoice': 'Επιτυχής ανάκτηση τιμολογίου',
      'archived_invoices': 'Επιτυχής αρχειοθέτηση :count τιμολογίων',
      'deleted_invoices': 'Επιτυχής διαγραφή :count τιμολογίων',
      'restored_invoices': 'Επιτυχής ανάκτηση :value τιμολογίων',
      'emailed_invoice': 'Επιτυχής αποστολή τιμολογίου',
      'emailed_payment': 'Επιτυχής αποστολή πληρωμής με Email',
      'amount': 'Ποσό',
      'invoice_number': 'Αριθμός Τιμολογίου',
      'invoice_date': 'Ημερομηνία Τιμολογίου',
      'discount': 'Έκπτωση',
      'po_number': 'Αριθμός Παραγγελίας',
      'terms': 'Όροι',
      'public_notes': 'Δημόσιες Σημειώσεις',
      'private_notes': 'Προσωπικές Σημειώσεις',
      'frequency': 'Συχνότητα',
      'start_date': 'Ημ/νία Έναρξης',
      'end_date': 'Ημ/νία Λήξης',
      'quote_number': 'Αριθμός Προσφοράς',
      'quote_date': 'Ημ/νία Προσφοράς',
      'valid_until': 'Έγκυρο Έως',
      'items': 'Προϊόντα',
      'partial_deposit': 'Μερικό/Κατάθεση',
      'description': 'Περιγραφή',
      'unit_cost': 'Τιμή Μονάδας',
      'quantity': 'Ποσότητα',
      'add_item': 'Προσθήκη Προϊόντος',
      'contact': 'Επαφή',
      'work_phone': 'Τηλέφωνο',
      'total_amount': 'Συνολικό Ποσό',
      'pdf': 'PDF',
      'due_date': 'Ημερομηνία Ολοκλήρωσης',
      'partial_due_date': 'Ημερομηνία Μερικής Πληρωμής',
      'status': 'Κατάσταση',
      'invoice_status_id': 'Κατάσταση Τιμολογίου',
      'quote_status': 'Κατάσταση Προσφοράς',
      'click_plus_to_add_item': 'Πιέστε το + για την προσθήκη ενός προϊόντος',
      'click_plus_to_add_time': 'Πιέστε το + για να προσθέσετε χρόνο',
      'count_selected': ':count επιλέχθηκε',
      'total': 'Σύνολο',
      'percent': 'Ποσοστό',
      'edit': 'Επεξεργασία',
      'dismiss': 'Απέρριψε',
      'please_select_a_date': 'Παρακαλώ επιλέξτε ημερομηνία',
      'please_select_a_client': 'Παρακαλώ επιλέξτε πελάτη',
      'please_select_an_invoice': 'Παρακαλώ επιλέξτε ένα τιμολόγιο',
      'task_rate': 'Κόστος Εργασίας',
      'settings': 'Ρυθμίσεις',
      'language': 'Γλώσσα',
      'currency': 'Νόμισμα',
      'created_at': 'Ημ/νία Δημιουργίας',
      'created_on': 'Δημιουργήθηκε στις',
      'updated_at': 'Ενημερώθηκε',
      'tax': 'Φόρος',
      'please_enter_an_invoice_number':
          'Παρακαλώ εισάγετε ένα αριθμό τιμολογίου',
      'please_enter_a_quote_number': 'Παρακαλώ εισάγετε ένα αριθμό προσφοράς',
      'past_due': 'Ληγμένα',
      'draft': 'Πρόχειρο',
      'sent': 'Απεσταλμένα',
      'viewed': 'Εμφανισμένα',
      'approved': 'Αποδεκτή',
      'partial': 'Μερικό/Κατάθεση',
      'paid': 'Πληρωμένα',
      'mark_sent': 'Σήμανση ως Απεσταλμένο',
      'marked_invoice_as_sent': 'Επιτυχής ορισμός τιμολογίου ως απεσταλμένο',
      'marked_invoice_as_paid': 'Επιτυχής ορισμός τιμολογίου ως απεσταλμένο',
      'marked_invoices_as_sent': 'Επιτυχής ορισμός τιμολογίων ως απεσταλμένα',
      'marked_invoices_as_paid': 'Επιτυχής ορισμός τιμολογίων ως απεσταλμένα',
      'done': 'Έτοιμο',
      'please_enter_a_client_or_contact_name':
          'Παρακαλώ εισάγετε ένα πελάτη ή το όνομα μίας επαφής',
      'dark_mode': 'Σκοτεινό Περιβάλλον',
      'restart_app_to_apply_change':
          'Επανεκκινήστε την εφαρμογή για να εφαρμόσετε την αλλαγή',
      'refresh_data': 'Ανανέωση Δεδομένων',
      'blank_contact': 'Κενή Επαφή',
      'activity': 'Δραστηριότητα',
      'no_records_found': 'Δεν βρέθηκαν εγγραφές',
      'clone': 'Κλωνοποίηση',
      'loading': 'Φόρτωση',
      'industry': 'Βιομηχανία',
      'size': 'Μέγεθος',
      'payment_terms': 'Όροι Πληρωμής',
      'payment_date': 'Ημ/νία Πληρωμής',
      'payment_status': 'Κατάσταση Πληρωμής',
      'payment_status_1': 'Εκκρεμής',
      'payment_status_2': 'Σε λήξη',
      'payment_status_3': 'Απέτυχε',
      'payment_status_4': 'Ολοκληρώθηκε',
      'payment_status_5': 'Μερική επιστροφή χρημάτων',
      'payment_status_6': 'Επιστροφή χρημάτων',
      'payment_status_-1': 'Ανεφάρμοστο',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Καθαρό',
      'client_portal': 'Portal Πελάτη',
      'show_tasks': 'Εμφάνιση εργασιών',
      'email_reminders': 'Email Υπενθύμίσεις',
      'enabled': 'Ενεργοποίηση',
      'recipients': 'Παραλήπτες',
      'initial_email': 'Αρχικό Email',
      'first_reminder': 'Πρώτη Υπενθύμιση',
      'second_reminder': 'Δεύτερη Υπενθύμιση',
      'third_reminder': 'Τρίτη Υπενθύμιση',
      'reminder1': 'Πρώτη Υπενθύμιση',
      'reminder2': 'Δεύτερη Υπενθύμιση',
      'reminder3': 'Τρίτη Υπενθύμιση',
      'template': 'Πρότυπο',
      'send': 'Αποστολή',
      'subject': 'Θέμα',
      'body': 'Κείμενο',
      'send_email': 'Αποστολή Email',
      'email_receipt': 'Αποστολή απόδειξης πληρωμής στον πελάτη',
      'auto_billing': 'Αυτόματη Χρέωση',
      'button': 'Κουμπί',
      'preview': 'Προεπισκόπηση',
      'customize': 'Προσαρμογή',
      'history': 'Ιστορικό',
      'payment': 'Πληρωμή',
      'payments': 'Πληρωμές',
      'refunded': 'Επιστροφή χρημάτων',
      'payment_type': 'Τύπος Πληρωμής',
      'transaction_reference': 'Κωδικός Συναλλαγής',
      'enter_payment': 'Καταχώρηση πληρωμής',
      'new_payment': 'Εισάγετε πληρωμή',
      'created_payment': 'Επιτυχής δημιουργία πληρωμής',
      'updated_payment': 'Επιτυχής ενημέρωση πληρωμής',
      'archived_payment': 'Επιτυχής αρχειοθέτηση πληρωμής',
      'deleted_payment': 'Επιτυχής διαγραφή πληρωμής',
      'restored_payment': 'Επιτυχής ανάκτηση πληρωμής',
      'archived_payments': 'Επιτυχής αρχειοθέτηση :count πληρωμών',
      'deleted_payments': 'Επιτυχής διαγραφή :count πληρωμών',
      'restored_payments': 'Επιτυχής ανάκτηση :value πληρωμών',
      'quote': 'Προσφορά',
      'quotes': 'Προσφορές',
      'new_quote': 'Νέα Προσφορά',
      'created_quote': 'Επιτυχής δημιουργία προσφοράς',
      'updated_quote': 'Επιτυχής ενημέρωση προσφοράς',
      'archived_quote': 'Επιτυχής αρχειοθέτηση προσφοράς',
      'deleted_quote': 'Επιτυχής διαγραφή προσφοράς',
      'restored_quote': 'Επιτυχής ανάκτηση προσφοράς',
      'archived_quotes': 'Επιτυχής αρχειοθέτηση :count προσφορών',
      'deleted_quotes': 'Επιτυχής διαγραφή :count προσφορών',
      'restored_quotes': 'Επιτυχής ανάκτηση :value προσφορών',
      'expense': 'Δαπάνη',
      'expenses': 'Δαπάνες',
      'vendor': 'Προμηθευτής',
      'vendors': 'Προμηθευτές',
      'task': 'Εργασία',
      'tasks': 'Εργασίες',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': 'Ο χρήστης :user δημιούργησε τον πελάτη :client',
      'activity_2': 'Ο χρήστης :user αρχειοθέτησε τον πελάτη :client',
      'activity_3': 'Ο χρήστης :user διέγραψε τον πελάτη :client',
      'activity_4': 'Ο χρήστης :user δημιούργησε το τιμολόγιο :invoice',
      'activity_5': 'Ο χρήστης :user ενημέρωσε το τιμολόγιο :invoice',
      'activity_6':
          'Ο χρήστης :user έστειλε με email το τιμολόγιο :invoice για τον πελάτη :client στην επαφή :contact',
      'activity_7':
          'Η επαφή :contact είδε το τιμολόγιο :invoice για τον πελάτη :client',
      'activity_8': 'Ο χρήστης :user αρχειοθέτησε το τιμολόγιο :invoice',
      'activity_9': 'Ο χρήστης :user διέγραψε το τιμολόγιο :invoice',
      'activity_10':
          'Η επαφή :contact καταχώρησε την πληρωμή ποσού :payment_amount για το τιμολόγιο :invoice για τον πελάτη :client',
      'activity_11': 'Ο χρήστης :user ενημέρωσε την πληρωμή :payment',
      'activity_12': 'Ο χρήστης :user αρχειοθέτησε την πληρωμή :payment',
      'activity_13': 'Ο χρήστης :user διέγραψε την πληρωμή :payment',
      'activity_14': 'Ο χρήστης :user καταχώρησε την πίστωση :credit',
      'activity_15': 'Ο χρήστης :user ενημέρωσε την πίστωση :credit',
      'activity_16': 'Ο χρήστης :user αρχειοθέτησε την πίστωση :credit',
      'activity_17': 'Ο χρήστης :user διέγραψε την πίστωση :credit',
      'activity_18': 'Ο χρήστης :user δημιουργησε την προσφορά :quote',
      'activity_19': 'Ο χρήστης :user ενημέρωσε την προσφορά :quote',
      'activity_20':
          'Ο χρήστης :user έστειλε με email την προσφορά :quote για τον πελάτη :client στην επαφή :contact',
      'activity_21': 'Η επαφή :contact είδε την προσφορά :quote',
      'activity_22': 'Ο χρήστης :user αρχειοθέτησε την προσφορά :quote',
      'activity_23': 'Ο χρήστης :user διέγραψε την προσφορά :quote',
      'activity_24': 'Ο χρήστης :user επανέφερε την προσφορά :quote',
      'activity_25': 'Ο χρήστης :user επανέφερε το τιμολόγιο :invoice',
      'activity_26': 'Ο χρήστης :user επανέφερε τον πελάτη :client',
      'activity_27': 'Ο χρήστης :user επανέφερε την πληρωμή :payment',
      'activity_28': 'Ο χρήστης :user επανέφερε την πίστωση :credit',
      'activity_29':
          'Η επαφή :contact αποδέχτηκε την προσφορά :quote για τον πελάτη :client',
      'activity_30': 'Ο χρήστης :user δημιούργησε τον προμηθευτή :vendor',
      'activity_31': 'Ο χρήστης :user αρχειοθέτησε τον προμηθευτή :vendor',
      'activity_32': 'Ο χρήστης :user διέγραψε τον προμηθευτή :vendor',
      'activity_33': 'Ο χρήστης :user επανέφερε τον προμηθευτή :vendor',
      'activity_34': 'Ο χρήστης :user δημιούργησε τη δαπάνη :expense',
      'activity_35': 'Ο χρήστης :user αρχειοθέτησε τη δαπάνη :expense',
      'activity_36': 'Ο χρήστης :user διέγραψε τη δαπάνη :expense',
      'activity_37': 'Ο χρήστης :user επανέφερε τη δαπάνη :expense',
      'activity_39': ':user ακύρωσε :payment_amount πληρωμής :payment',
      'activity_40':
          ':user επέστρεψε :adjustment μιας :payment_amount πληρωμής :payment',
      'activity_41': ':payment_amount πληρωμής (:payment) απέτυχε',
      'activity_42': 'Ο χρήστης :user δημιούργησε την εργασία :task',
      'activity_43': 'Ο χρήστης :user ενημέρωσε την εργασία :task',
      'activity_44': 'Ο χρήστης :user αρχειοθέτησε την εργασία :task',
      'activity_45': 'Ο χρήστης :user διέγραψε την εργασία :task',
      'activity_46': 'Ο χρήστης :user επανέφερε την εργασία :task',
      'activity_47': 'Ο χρήστης :user ενημέρωσε τη δαπάνη :expense',
      'activity_48': 'Ο χρήστης :user ενημέρωσε το αίτημα υποστήριξης :ticket',
      'activity_49': 'Ο χρήστης :user έκλεισε το αίτημα υποστήριξης :ticket',
      'activity_50': 'Ο χρήστης :user συνένωσε το αίτημα υποστήριξης :ticket',
      'activity_51':
          'Ο χρήστης :user διαίρεσε στα δύο το αίτημα υποστήριξης :ticket',
      'activity_52':
          'Η επαφή :contact δημιούργησε το αίτημα υποστήριξης :ticket',
      'activity_53':
          'Η επαφή :contact επαναδημιούργησε το αίτημα υποστήριξης :ticket',
      'activity_54':
          'Ο χρήστης :user επαναδημιούργησε το αίτημα υποστήριξης :ticket',
      'activity_55': 'Η επαφή :contact απάντησε στο αίτημα υποστήριξης :ticket',
      'activity_56': 'Ο χρήστης :user είδε το αίτημα υποστήριξης :ticket',
      'activity_57':
          'Το σύστημα απέτυχε να στείλει με email το τιμολόγιο :invoice',
      'activity_58': 'Ο χρήστης :user αντίστρεψε το τιμολόγιο :invoice',
      'activity_59': 'Ο χρήστης :user ακύρωσε το τιμολόγιο :invoice',
      'activity_60': 'Η επαφή :contact είδε την προσφορά :quote',
      'activity_61': 'Ο χρήστης :user ενημέρωσε τον πελάτη :client',
      'activity_62': 'Ο χρήστης :user ενημέρωσε τον προμηθευτή :vendor',
      'activity_63':
          'Ο χρήστης :user έστειλε με email πρώτη ειδοποίηση για το τιμολόγιο :invoice στην επαφή :contact',
      'activity_64':
          'Ο χρήστης :user έστειλε με email δεύτερη ειδοποίηση για το τιμολόγιο :invoice στην επαφή :contact',
      'activity_65':
          'Ο χρήστης :user έστειλε με email τρίτη ειδοποίηση για το τιμολόγιο :invoice στην επαφή :contact',
      'activity_66':
          'Ο χρήστης :user έστειλε με email ατέρμονη ειδοποίηση για το τιμολόγιο :invoice στην επαφή :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Κωδικός Πρόσβασης μίας Φοράς',
      'emailed_quote': 'Επιτυχής αποστολή προσφοράς',
      'emailed_credit': 'Επιτυχής αποστολή πίστωσης με email',
      'marked_quote_as_sent': 'Επιτυχής ορισμός προσφοράς ως απεσταλμένη',
      'marked_credit_as_sent': 'Επιτυχής ορισμός πίστωσης ως απεσταλμένη',
      'expired': 'Ληγμένα',
      'all': 'Όλα',
      'select': 'Επιλογή',
      'long_press_multiselect': 'Πολλαπλή επιλογή με Παρατεταμένη πίεση',
      'custom_value1': 'Προσαρμοσμένη Τιμή',
      'custom_value2': 'Προσαρμοσμένη Τιμή',
      'custom_value3': 'Προσαρμοσμένη Τιμή 3',
      'custom_value4': 'Προσαρμοσμένη Τιμή 4',
      'email_style_custom': 'Προσαρμοσμένο Στυλ Email',
      'custom_message_dashboard': 'Προσαρμοσμένο Μήνυμα Πίνακα Διαχείρισης',
      'custom_message_unpaid_invoice':
          'Προσαρμοσμένο Μήνυμα Ανεξόφλητου Τιμολογίου',
      'custom_message_paid_invoice':
          'Προσαρμοσμένο Μήνυμα Εξοφλημένου Τιμολογίου',
      'custom_message_unapproved_quote':
          'Προσαρμοσμένο Μήνυμα Μη Εγκεκριμένη Προσφοράς',
      'lock_invoices': 'Κλείδωμα Τιμολογίων',
      'translations': 'Μεταφράσεις',
      'task_number_pattern': 'Μοτίβο Αρίθμησης Εργασίας',
      'task_number_counter': 'Μετρητής Αρίθμησης Εργασίας',
      'expense_number_pattern': 'Μοτίβο Αρίθμησης Δαπάνης',
      'expense_number_counter': 'Μετρητής Αρίθμησης Δαπάνης',
      'vendor_number_pattern': 'Μοτίβο Αρίθμησης Προμηθευτή',
      'vendor_number_counter': 'Μετρητής Αρίθμησης Προμηθευτή',
      'ticket_number_pattern': 'Μοτίβο Αρίθμησης Αιτήματος Βοήθειας',
      'ticket_number_counter': 'Μετρητής Αρίθμησης Αιτήματος Βοήθειας',
      'payment_number_pattern': 'Μοτίβο Αρίθμησης Πληρωμής',
      'payment_number_counter': 'Μετρητής Αρίθμησης Πληρωμής',
      'invoice_number_pattern': 'Μοτίβο Αρίθμησης Τιμολογίου',
      'invoice_number_counter': 'Αρίθμηση Τιμολογίου',
      'quote_number_pattern': 'Μοτίβο Αρίθμησης Προσφοράς',
      'quote_number_counter': 'Αρίθμηση Προσφοράς',
      'client_number_pattern': 'Μοντέλο αριθμών πιστωτικού',
      'client_number_counter': 'Μετρητής Αριθμών πιστωτικών',
      'credit_number_pattern': 'Μοντέλο πιστωτικού αριθμού',
      'credit_number_counter': 'Μετρητής Αριθμών πιστωτικών',
      'reset_counter_date': 'Μηδενισμός Μετρητή Ημερομηνίας',
      'counter_padding': 'Αντισταθμιστής',
      'shared_invoice_quote_counter':
          'Κοινόχρηστο παράθυρο παραγγελίας τιμολογίου',
      'default_tax_name_1': 'Προεπιλεγμένη ονομασία φορολογικού συντελεστή 1',
      'default_tax_rate_1': 'Προεπιλεγμένος φορολογικός συντελεστής 1',
      'default_tax_name_2': 'Προεπιλεγμένη ονομασία φορολογικού συντελεστή 2',
      'default_tax_rate_2': 'Προεπιλεγμένος φορολογικός συντελεστής 2',
      'default_tax_name_3': 'Προεπιλεγμένη ονομασία φορολογικού συντελεστή 3',
      'default_tax_rate_3': 'Προεπιλεγμένος φορολογικός συντελεστής 3',
      'email_subject_invoice': 'Θέμα τιμολογίου με ηλεκτρονικό ταχυδρομείο',
      'email_subject_quote': 'Θέμα ηλεκτρονικού ταχυδρομείου',
      'email_subject_payment': 'Θέμα Πληρωμής με ηλεκτρονικό ταχυδρομείο',
      'email_subject_payment_partial': 'Θέμα Email μερικής πληρωμής',
      'show_table': 'Εμφάνιση Πίνακα',
      'show_list': 'Εμφάνιση Λίστας',
      'client_city': 'Πόλη Πελάτη',
      'client_state': 'Κράτος Πελάτη',
      'client_country': 'Χώρα Πελάτη',
      'client_is_active': 'Ο Πελάτης είναι Ενεργός',
      'client_balance': 'Ισοζύγιο Πελατών',
      'client_address1': 'Οδός Πελάτη',
      'client_address2': 'Διαμέρισμα Πελάτη',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Οδός Αποστολής Πελάτη',
      'client_shipping_address2': 'Διαμέρισμα Αποστολής Πελάτη',
      'type': 'Τύπος',
      'invoice_amount': 'Ποσό Τιμολογίου',
      'invoice_due_date': 'Ημερομηνία Ολοκλήρωσης',
      'tax_rate1': 'Φορολογικός Συντελεστής 1',
      'tax_rate2': 'Φορολογικός Συντελεστής 2',
      'tax_rate3': 'Φορολογικός Συντελεστής 3',
      'auto_bill': 'Αυτόματη Χρέωση',
      'archived_at': 'Αρχειοθετήθηκε στις',
      'has_expenses': 'Εχει έξοδα',
      'custom_taxes1': 'Προσαρμοσμένη Φορολόγηση 1',
      'custom_taxes2': 'Προσαρμοσμένη Φορολόγηση 2',
      'custom_taxes3': 'Προσαρμοσμένη Φορολόγηση 3',
      'custom_taxes4': 'Προσαρμοσμένη Φορολόγηση 4',
      'custom_surcharge1': 'Προσαρμοσμένη Προσαύξηση 1',
      'custom_surcharge2': 'Προσαρμοσμένη Προσαύξηση 2',
      'custom_surcharge3': 'Προσαρμοσμένη Προσαύξηση 3',
      'custom_surcharge4': 'Προσαρμοσμένη Προσαύξηση 4',
      'is_deleted': 'Εχει διαγραφεί',
      'vendor_city': 'Πόλη Προμηθευτή',
      'vendor_state': 'Κράτος Προμηθευτή',
      'vendor_country': 'Χώρα Προμηθευτή',
      'is_approved': 'Είναι Αποδεκτή',
      'tax_name': 'Ονομασία Φόρου',
      'tax_amount': 'Ποσό Φόρου',
      'tax_paid': 'Πληρωμένος Φόρος',
      'payment_amount': 'Ποσό Πληρωμής',
      'age': 'Ηλικία',
      'is_running': 'Εκτελείται',
      'time_log': 'Αρχείο Καταγραφής Χρόνου',
      'bank_id': 'Τράπεζα',
      'expense_category_id': 'Κωδικός Κατηγορίας Δαπάνης',
      'expense_category': 'Κατηγορία Δαπάνης',
      'invoice_currency_id': 'Κωδικός Νομίσματος Τιμολογίου',
      'tax_name1': 'Ονομασία Φόρου 1',
      'tax_name2': 'Ονομασία Φόρου 2',
      'tax_name3': 'Ονομασία Φόρου 2',
      'transaction_id': 'Κωδικός Συναλλαγής',
      'color_theme': 'Color Theme',
    },
    'it': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Ultimo aggiornamento',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Non assegnato',
      'partial_value': 'Deve essere maggiore di zero ed inferiore al totale',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Abilita',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Imposta il sottodominio o visualizza la fattura sul tuo sito web.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Codice Promo',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sicurezza',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Abbonamento',
      'subscriptions': 'Abbonamenti',
      'new_subscription': 'Nuovo Abbonamento',
      'edit_subscription': 'Modifica Abbonamento',
      'created_subscription': 'Abbonamento creato con successo',
      'updated_subscription': 'Abbonamento aggiornato con successo',
      'archived_subscription': 'Abbonamento archiviato con successo',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Cerca 1 Abbonamento',
      'search_subscriptions': 'Cerca :count Abbonamenti',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Scadenza Password',
      'shared_invoice_credit_counter': 'Contatore di fatture/crediti condiviso',
      'use_last_email': 'Use last email',
      'activate_company': 'Attiva azienda',
      'activate_company_help':
          'Abilitare le e-mail, le fatture ricorrenti e le notifiche',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Si prega di impostare prima una password',
      'changing_phone_disables_two_factor':
          'Attenzione: Cambiare il numero di telefono disabiliterà l\'autenticazione a due fattori',
      'help_translate': 'Contribuisci alla traduzione',
      'please_select_a_country': 'Selezionare un paese',
      'resend_invite': 'Re-invia invito',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Si prega di fornire un numero di telefono cellulare per abilitare l\'autenticazione a due fattori',
      'send_sms': 'Send SMS',
      'sms_code': 'Codice SMS',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Autenticazione a due fattori',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Richiedi una password per il login Social',
      'stay_logged_in': 'Rimani autenticato',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Impostazioni di Sicurezza',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Pagamento Rimborsato',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Converti a Fattura',
      'registration_url': 'URL di registrazione',
      'invoice_project': 'Fattura progetto',
      'invoice_task': 'Fattura l\'attività',
      'invoice_expense': 'Fattura Spesa',
      'search_payment_term': 'Ricerca 1 Termine di pagamento',
      'search_payment_terms': 'Cerca :count termini di pagamento',
      'save_and_preview': 'Salva e mostra anteprima',
      'save_and_email': 'Salva e invia e-mail',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'È inviato',
      'default_documents': 'Documenti di default',
      'document_upload': 'Caricamento Documenti',
      'document_upload_help': 'Permettere ai clienti di caricare documenti',
      'expense_total': 'Totale Spese',
      'enter_taxes': 'Inserire tasse',
      'by_rate': 'Per tasso',
      'by_amount': 'Per ammontare',
      'enter_amount': 'Inserire importo',
      'before_taxes': 'Prima delle tasse',
      'after_taxes': 'Dopo le tasse',
      'color': 'Colore',
      'show': 'Mostra',
      'hide': 'Nascondi',
      'empty_columns': 'Colonne vuote',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Attività in corso',
      'recent_tasks': 'Attività Recenti',
      'recent_expenses': 'Spese Recenti',
      'upcoming_expenses': 'Spese in arrivo',
      'update_app': 'Aggiorna App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Usa tasse inclusive',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Colonna',
      'sample': 'Esempio',
      'map_to': 'Map To',
      'import': 'Importa',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Seleziona un file, per favore',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Seleziona file CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Visualizza le licenze',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Editor a schermo intero',
      'sidebar_editor': 'Editor barra laterale',
      'please_type_to_confirm': 'Digita \":value\" per confermare',
      'purge': 'Purge',
      'service': 'Servizio',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clona su altro',
      'labels': 'Etichette',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Non pagata',
      'white_label': 'White Label',
      'delivery_note': 'Nota di consegna',
      'sent_invoices_are_locked': 'Le fatture inviate sono bloccate',
      'paid_invoices_are_locked': 'Le fatture pagate sono bloccate',
      'source_code': 'Codice Sorgente',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Fattura in ritardo',
      'quote_expired': 'Preventivo scaduto',
      'partial_due': 'Da versare (parziale)',
      'invoice_total': 'Totale Fattura',
      'quote_total': 'Totale Preventivo',
      'credit_total': 'Credito Totale',
      'recurring_invoice_total': 'Totale fattura',
      'actions': 'Azioni',
      'expense_number': 'Numero spesa',
      'task_number': 'Numero attività',
      'project_number': 'Numero progetto',
      'project_name': 'Nome progetto',
      'warning': 'Attenzione',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Attenzione: questa azienda non è ancora stata attivata',
      'late_invoice': 'Fattura in ritardo',
      'expired_quote': 'Preventivo scaduto',
      'remind_invoice': 'Ricorda fattura',
      'cvv': 'CVV',
      'client_name': 'Nome Cliente',
      'client_phone': 'Telefono cliente',
      'required_fields': 'Campi richiesti',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Tasso predefinito di attività',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Stato',
      'task_statuses': 'Stati attività',
      'new_task_status': 'Nuovo stato attività',
      'edit_task_status': 'Modifica stato attività',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Stato dell\'attività aggiornato con successo',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Mostra sempre la sezione delle attività quando si creano le fatture',
      'invoice_task_timelog': 'Timelog delle attività di fatturazione',
      'invoice_task_timelog_help':
          'Aggiungere i dettagli sull\'orario alle voci della fattura',
      'invoice_task_datelog': 'Datelog delle attività di fatturazione',
      'invoice_task_datelog_help':
          'Aggiungi i dettagli della data alle voci della fattura',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Impostazioni attività',
      'configure_categories': 'Configura Categorie',
      'expense_categories': 'Categorie di Spesa',
      'new_expense_category': 'Nuova Categoria di Spesa',
      'edit_expense_category': 'Modifica Categoria di Spesa',
      'created_expense_category': 'Categoria spese creata con successo',
      'updated_expense_category': 'Categoria spese aggiornata con successo',
      'archived_expense_category': 'Categoria spese archiviata con successo',
      'deleted_expense_category': 'Categoria eliminata con successo',
      'removed_expense_category': 'Categoria di spesa rimossa con successo',
      'restored_expense_category': 'Categoria spese ripristinata con successo',
      'archived_expense_categories':
          'Archiviato con successo :count categorie di spesa',
      'deleted_expense_categories':
          'Eliminate con successo :value categorie di spesa',
      'restored_expense_categories':
          'Ripristinate con successo :value categorie di spesa',
      'search_expense_category': 'Cerca 1 categoria di spesa',
      'search_expense_categories': 'Cerca :count categorie di spesa',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Mostra opzione',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'Vedi modifiche',
      'force_update': 'Forza aggiornamento',
      'force_update_help':
          'Stai eseguendo l\'ultima versione, ma potrebbero essere disponibili dei fix in attesa.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Deve essere fatturata',
      'should_be_invoiced_help': 'Permettere la fatturazione della spesa',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Imposta un tasso di cambio',
      'expense_settings': 'Impostazioni Spese',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variabili',
      'show_password': 'Mostra Password',
      'hide_password': 'Nascondi Password',
      'copy_error': 'Copia Errore',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Fattura automatica abilitata',
      'total_taxes': 'Totale Tasse',
      'line_taxes': 'Riga tasse',
      'total_fields': 'Campi Totale',
      'stopped_recurring_invoice': 'Fermata con successo la fattura ricorrente',
      'started_recurring_invoice': 'Fattura ricorrente avviata con successo',
      'resumed_recurring_invoice': 'Fattura ricorrente ripresa con successo',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Pausato',
      'mark_active': 'Segna come attivo',
      'day_count': 'Giorno :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Usa termini di pagamento',
      'endless': 'Endless',
      'next_send_date': 'Prossima data di invio',
      'remaining_cycles': 'Cicli restanti',
      'recurring_invoice': 'Fattura Ricorrente',
      'recurring_invoices': 'Fatture Ricorrenti',
      'new_recurring_invoice': 'Nuova Fattura Ricorrente',
      'edit_recurring_invoice': 'Modifica Fattura Ricorrente',
      'created_recurring_invoice': 'Fattura ricorrente creata con successo',
      'updated_recurring_invoice': 'Fattura ricorrente aggiornata con successo',
      'archived_recurring_invoice':
          'Fattura ricorrente archiviata con successo',
      'deleted_recurring_invoice': 'Fattura ricorrente eliminata con successo',
      'removed_recurring_invoice': 'Fattura ricorrente rimossa con successo',
      'restored_recurring_invoice':
          'Fattura ricorrente ripristinata con successo',
      'archived_recurring_invoices':
          'Archiviato con successo :value fatture ricorrenti',
      'deleted_recurring_invoices':
          'Cancellato con successo :value fatture ricorrenti',
      'restored_recurring_invoices':
          'Ripristinato con successo :value fatture ricorrenti',
      'search_recurring_invoice': 'Cerca 1 fattura ricorrente',
      'search_recurring_invoices': 'Cerca :count Fatture ricorrenti',
      'send_date': 'Data di invio',
      'auto_bill_on': 'Fattura automatica attiva',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Utile',
      'line_item': 'Riga articolo',
      'allow_over_payment': 'Consenti pagamento in eccesso',
      'allow_over_payment_help':
          'Supporta il pagamento di un extra per accettare le mance',
      'allow_under_payment': 'Consenti pagamento ridotto',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Aperto',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'Registri di sistema',
      'view_portal': 'Visualizza il portale',
      'copy_link': 'Copia Collegamento',
      'token_billing': 'Salva carta di credito',
      'welcome_to_invoice_ninja': 'Benvenuti a Invoice Ninja',
      'always': 'Sempre',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Etichetta',
      'client_number': 'Numero Cliente',
      'auto_convert': 'Auto Convert',
      'company_name': 'Nome Azienda',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Pagina :current di :total',
      'emailed_invoices': 'Fatture inviate con successo',
      'emailed_quotes': 'Preventivi inviati con successo',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Piattaforma',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Righe per pagina',
      'hours': 'Ore',
      'statement': 'Statement',
      'taxes': 'Tasse',
      'surcharge': 'Sovrapprezzo',
      'apply_payment': 'Applica pagamento',
      'apply': 'Applica',
      'unapplied': 'Unapplied',
      'select_label': 'Seleziona etichetta',
      'custom_labels': 'Etichette Personalizzate',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Altezza',
      'width': 'Larghezza',
      'to': 'a',
      'health_check': 'Health Check',
      'payment_type_id': 'Tipo di Pagamento',
      'last_login_at': 'Last Login At',
      'company_key': 'Chiave azienda',
      'storefront': 'Storefront',
      'storefront_help': 'Permetti alle app di terze parti di creare fatture',
      'client_created': 'Cliente creato',
      'online_payment_email': 'Email di pagamento online',
      'manual_payment_email': 'Email di pagamento manuale',
      'completed': 'Completato',
      'gross': 'Lordo',
      'net_amount': 'Cifra al netto',
      'net_balance': 'Bilancio Netto',
      'client_settings': 'Impostazioni Cliente',
      'selected_invoices': 'Fatture Selezionate',
      'selected_payments': 'Pagamenti selezionati',
      'selected_quotes': 'Preventivi Selezionati',
      'selected_tasks': 'Attività Selezionate',
      'selected_expenses': 'Spese Selezionate',
      'upcoming_invoices': 'Prossime fatture',
      'past_due_invoices': 'Fatture scadute',
      'recent_payments': 'Pagamenti recenti',
      'upcoming_quotes': 'Preventivi in scadenza',
      'expired_quotes': 'Preventivi Scaduti',
      'create_client': 'Crea nuovo cliente',
      'create_invoice': 'Crea Fattura',
      'create_quote': 'Crea Preventivo',
      'create_payment': 'Crea pagamento',
      'create_vendor': 'Crea fornitore',
      'update_quote': 'Aggiorna Preventivo',
      'delete_quote': 'Cancella Preventivo',
      'update_invoice': 'Aggiorna Fattura',
      'delete_invoice': 'Elimina Fattura',
      'update_client': 'Aggiorna Cliente',
      'delete_client': 'Elimina cliente',
      'delete_payment': 'Elimina pagamento',
      'update_vendor': 'Aggiorna Fornitore',
      'delete_vendor': 'Cancella Fornitore',
      'create_expense': 'Crea Spesa',
      'update_expense': 'Aggiorna Spesa',
      'delete_expense': 'Cancella Spesa',
      'create_task': 'Crea un\'attività',
      'update_task': 'Aggiorna Attività',
      'delete_task': 'Cancella l\'attività',
      'approve_quote': 'Aggiorna Preventivo',
      'off': 'Off',
      'when_paid': 'Quando Pagato',
      'expires_on': 'Scade il',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Mostra Barra Laterale',
      'hide_sidebar': 'Nascondi Barra Laterale',
      'event_type': 'Tipo Evento',
      'target_url': 'Target',
      'copy': 'Copia',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Token',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Token',
      'new_token': 'New Token',
      'edit_token': 'Modifica token',
      'created_token': 'Token creato correttamente',
      'updated_token': 'Token aggiornato correttamente',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Token eliminato correttamente',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Registazione cliente',
      'client_registration_help':
          'Permetti al cliente di registrarsi da solo nel portale',
      'customize_and_preview': 'Personalizza & Anterpima',
      'email_invoice': 'Invia Fattura',
      'email_quote': 'Invia Preventivo via Email',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Pagamento',
      'client_email_not_set': 'Il cliente non ha un indirizzo email impostato',
      'ledger': 'Ledger',
      'view_pdf': 'Vedi PDF',
      'all_records': 'Tutti i dati',
      'owned_by_user': 'Posseduto da utente',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Nome Contatto',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configura termini di pagamento',
      'payment_term': 'Termini di pagamento',
      'new_payment_term': 'Nuovi termini di pagamento',
      'edit_payment_term': 'Modifica termini di pagamento',
      'created_payment_term': 'Termini di pagamento creato con successo',
      'updated_payment_term': 'Termini di pagamento aggiornato con successo',
      'archived_payment_term': 'Termini di pagamento archiviato con successo',
      'deleted_payment_term': 'Termini di pagamento cancellati con successo',
      'removed_payment_term': 'Termini di pagamento rimossi con successo',
      'restored_payment_term': 'Termini di pagamento ripristinati con successo',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Importo Credito',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Esclusiva',
      'inclusive': 'Inclusiva',
      'hide_menu': 'Nascondi menu',
      'show_menu': 'Mostra menu',
      'partially_refunded': 'Parzialmente rimborsato',
      'search_documents': 'Cerca Documenti',
      'search_designs': 'Search Designs',
      'search_invoices': 'Cerca Fatture',
      'search_clients': 'Cerca Clienti',
      'search_products': 'Cerca Prodotti',
      'search_quotes': 'Cerca Preventivi',
      'search_credits': 'Cerca crediti',
      'search_vendors': 'Cerca fornitori',
      'search_users': 'Cerca utenti',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Cerca attività',
      'search_settings': 'Cerca Impostazioni',
      'search_projects': 'Cerca prodotti',
      'search_expenses': 'Cerca spese',
      'search_payments': 'Cerca pagamenti',
      'search_groups': 'Cerca gruppi',
      'search_company': 'Cerca Azienda',
      'search_document': 'Cerca 1 documento',
      'search_design': 'Cerca 1 stile',
      'search_invoice': 'Cerca 1 fattura',
      'search_client': 'Cerca 1 cliente',
      'search_product': 'Cerca 1 prodotto',
      'search_quote': 'Cerca 1 preventivo',
      'search_credit': 'Cerca 1 credito',
      'search_vendor': 'Cerca 1 Fornitore',
      'search_user': 'Cerca 1 utente',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Cerca 1 attività',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Cerca 1 pagamento',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Rimborsa Pagamento',
      'cancelled_invoice': 'Fattura annullata con successo',
      'cancelled_invoices': 'Fatture annullate con successo',
      'reversed_invoice': 'Fattura stornata con successo',
      'reversed_invoices': 'Fatture stornate con successo',
      'reverse': 'Reverse',
      'full_name': 'Nome Completo',
      'city_state_postal': 'Città/Stato/CAP',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Opzionale',
      'license': 'Licenza',
      'purge_data': 'Cancella dati permanentemente',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Saldo della fattura',
      'age_group_0': '0 - 30 Giorni',
      'age_group_30': '30 - 60 Giorni',
      'age_group_60': '60 - 90 Giorni',
      'age_group_90': '90 - 120 Giorni',
      'age_group_120': '120+ Giorni',
      'refresh': 'Aggiorna',
      'saved_design': 'Successfully saved design',
      'client_details': 'Dettagli Cliente',
      'company_address': 'Indirizzo azienda',
      'invoice_details': 'Dettagli fattura',
      'quote_details': 'Dettagli Preventivo',
      'credit_details': 'Dettagli Credito',
      'product_columns': 'Colonne Prodotto',
      'task_columns': 'Colonne attività',
      'add_field': 'Aggiungi campo',
      'all_events': 'Tutti gli eventi',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Posseduto',
      'payment_success': 'Pagamento riuscito',
      'payment_failure': 'Errore di pagamento',
      'invoice_sent': ':count fattura inviata',
      'quote_sent': 'Preventivo inviato',
      'credit_sent': 'Credito inviato',
      'invoice_viewed': 'Fattura visualizzata',
      'quote_viewed': 'Preventivo visualizzato',
      'credit_viewed': 'Credito visualizzato',
      'quote_approved': 'Preventivo approvato',
      'receive_all_notifications': 'Ricevi tutte le notifiche',
      'purchase_license': 'Acquista licenza',
      'apply_license': 'Applica Licenza',
      'cancel_account': 'Elimina l\'account',
      'cancel_account_message':
          'Attenzione: Questo eliminerà permanentemente il tuo account, non si potrà più tornare indietro.',
      'delete_company': 'Elimina azienda',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Moduli abilitati',
      'converted_quote': 'Preventivo convertito con successo',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Carica Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Stili',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposte',
      'tickets': 'Tickets',
      'recurring_quotes': 'Preventivi Ricorrenti',
      'recurring_tasks': 'Attività ricorrenti',
      'recurring_expenses': 'Spese Ricorrenti',
      'account_management': 'Gestione Account',
      'credit_date': 'Data Credito',
      'credit': 'Credito',
      'credits': 'Crediti',
      'new_credit': 'Inserisci il credito',
      'edit_credit': 'Modifica Credito',
      'created_credit': 'Credito creato con successo',
      'updated_credit': 'Credito aggiornato con successo',
      'archived_credit': 'Credito archiviato con successo',
      'deleted_credit': 'Credito eliminato con successo',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Credito ripristinato con successo',
      'archived_credits': ':count crediti archiviati con successo',
      'deleted_credits': ':count crediti eliminati con successo',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versione attuale',
      'latest_version': 'Latest Version',
      'update_now': 'Aggiorna ora',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Aggiornamento disponibile',
      'app_updated': 'Aggiornamento completato con successo',
      'learn_more': 'Scopri di più',
      'integrations': 'Integrazioni',
      'tracking_id': 'Id di tracciamento',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Termini del Credito',
      'new_company': 'Nuova azienda',
      'added_company': 'Azienda aggiunta con successo',
      'company1': 'Azienda Personalizzata 1',
      'company2': 'Azienda Personalizzata 2',
      'company3': 'Azienda Personalizzata 3',
      'company4': 'Azienda Personalizzata 4',
      'product1': 'Prodotto personalizzato 1',
      'product2': 'Prodotto personalizzato 2',
      'product3': 'Prodotto personalizzato 3',
      'product4': 'Prodotto personalizzato 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Attività personalizzata 1',
      'task2': 'Attività personalizzata 2',
      'task3': 'Attività personalizzata 3',
      'task4': 'Attività personalizzata 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Fornitore Personalizzato 1',
      'vendor2': 'Fornitore Personalizzato 2',
      'vendor3': 'Fornitore Personalizzato 3',
      'vendor4': 'Fornitore Personalizzato 4',
      'invoice1': 'Fattura Personalizzata 1',
      'invoice2': 'Fattura Personalizzata 2',
      'invoice3': 'Fattura Personalizzata 3',
      'invoice4': 'Fattura Personalizzata 4',
      'payment1': 'Pagamento personalizzato 1',
      'payment2': 'Pagamento personalizzato 2',
      'payment3': 'Pagamento personalizzato 3',
      'payment4': 'Pagamento personalizzato 4',
      'surcharge1': 'Sovrattassa personalizzata 1',
      'surcharge2': 'Sovrattassa personalizzata 2',
      'surcharge3': 'Sovrattassa personalizzata 3',
      'surcharge4': 'Sovrattassa personalizzata 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Numero',
      'export': 'Esporta',
      'chart': 'Grafico',
      'count': 'Count',
      'totals': 'Totali',
      'blank': 'Vuoto',
      'day': 'GIorno',
      'month': 'Mese',
      'year': 'Anno',
      'subgroup': 'Sottogruppo',
      'is_active': 'È attivo',
      'group_by': 'Raggruppa per',
      'credit_balance': 'Saldo Credito',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Telefono Contatto',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Via di spedizione',
      'shipping_address2': 'Piano/Appartamento di spedizione',
      'shipping_city': 'Città di spedizione',
      'shipping_state': 'Provincia di spedizione',
      'shipping_postal_code': 'Codice Postale di spedizione',
      'shipping_country': 'Paese di spedizione',
      'billing_address1': 'Via di fatturazione',
      'billing_address2': 'App/scala di fatturazione',
      'billing_city': 'Città di fatturazione',
      'billing_state': 'Stato/Provincia di fatturazione',
      'billing_postal_code': 'Codice postale di fatturazione',
      'billing_country': 'Paese fatturazione',
      'client_id': 'Id Cliente',
      'assigned_to': 'Assegnato a',
      'created_by': 'Creato da :name',
      'assigned_to_id': 'Assegnato all\'ID',
      'created_by_id': 'Creato dall\'ID',
      'add_column': 'Aggiungi Colonna',
      'edit_columns': 'Modifica Colonne',
      'columns': 'Colonne',
      'aging': 'Aging',
      'profit_and_loss': 'Utile e Perdite',
      'reports': 'Rapporti',
      'report': 'Report',
      'add_company': 'Aggiungi azienda',
      'unpaid_invoice': 'Fattura non pagata',
      'paid_invoice': 'Fattura pagata',
      'unapproved_quote': 'Preventivi non approvati',
      'help': 'Aiuto',
      'refund': 'Rimborso',
      'refund_date': 'Data di rimborso',
      'filtered_by': 'Filtrato per',
      'contact_email': 'Contact Email',
      'multiselect': 'Multi-selezione',
      'entity_state': 'State',
      'verify_password': 'Verifica Password',
      'applied': 'Applicato',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Messaggio',
      'from': 'Da',
      'show_product_details': 'Mostra i dettagli del prodotto',
      'show_product_details_help':
          'Includi la descrizione ed il costo nel menu a tendina del prodotto',
      'pdf_min_requirements': 'Il generatore di PDF richiede :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configura Impostazioni',
      'support_forum': 'Forum di supporto',
      'about': 'Info',
      'documentation': 'Documentazione',
      'contact_us': 'Contattaci',
      'subtotal': 'Subtotale',
      'line_total': 'Totale Riga',
      'item': 'Articolo',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'URL dominio',
      'password_is_too_short': 'La password è troppo corta',
      'password_is_too_easy':
          'La password deve contenere una maiuscola ed un numero',
      'client_portal_tasks': 'Attività del portale clienti',
      'client_portal_dashboard': 'Pannello di Controllo Portale Clienti',
      'please_enter_a_value': 'Per favore inserisci un valore',
      'deleted_logo': 'Logo eliminato con successo',
      'yes': 'Si',
      'no': 'No',
      'generate_number': 'Genera numero',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Seleziona azienda',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Mostra/nascondi',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'Barra laterale storico',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Vedi',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Mostra Costo',
      'show_product_cost': 'Mostra costo prodotto',
      'show_cost_help':
          'Mostra un campo costo prodotto per tracciare il markup/profitto',
      'show_product_quantity': 'Mostra quantità prodotto',
      'show_product_quantity_help':
          'Mostra un campo quantità prodotto, altrimenti imposta di default a 1',
      'show_invoice_quantity': 'Mostra quantità fattura',
      'show_invoice_quantity_help':
          'Mostra un campo per la quantità degli articoli sulla riga, altrimenti imposta a uno',
      'show_product_discount': 'Mostra sconto prodotto',
      'show_product_discount_help':
          'Mostra un campo sconto articolo sulla riga',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Imposta automaticamente la quantità dell\'elemento nella riga ad uno',
      'one_tax_rate': 'Una aliquota',
      'two_tax_rates': 'Due aliquote',
      'three_tax_rates': 'Tre aliquote',
      'default_tax_rate': 'Aliquota di default',
      'user': 'User',
      'invoice_tax': 'Tassa fattura',
      'line_item_tax': 'Riga imposta articolo',
      'inclusive_taxes': 'Tasse inclusive',
      'invoice_tax_rates': 'Aliquote della fattura',
      'item_tax_rates': 'Tassi d\'imposta articolo',
      'no_client_selected': 'Per favore seleziona un cliente',
      'configure_rates': 'Configura aliquote',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Impostazioni tasse',
      'tax_settings_rates': 'Aliquote Fiscali',
      'accent_color': 'Accent Color',
      'switch': 'Cambia',
      'comma_sparated_list': 'Lista separata da virgole',
      'options': 'Opzioni',
      'single_line_text': 'Testo a riga singola',
      'multi_line_text': 'Testo multi-riga',
      'dropdown': 'Menu a discesa',
      'field_type': 'Tipo di campo',
      'recover_password_email_sent':
          'Una mail di recupero password è stata inviata',
      'submit': 'Invia',
      'recover_password': 'Recupera password',
      'late_fees': 'Late Fees',
      'credit_number': 'Numero Credito',
      'payment_number': 'Numero di pagamento',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Prima della data di scadenza',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'Dopo la data della fattura',
      'days': 'Giorni',
      'invoice_email': 'Email Fattura',
      'payment_email': 'Email Pagamento',
      'partial_payment': 'Pagamento parziale',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Email di pagamento parziale',
      'quote_email': 'Email Preventivo',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtrato per utente',
      'administrator': 'Administrator',
      'administrator_help':
          'Permettere all\'utente di gestire gli utenti, cambiare le impostazioni e modificare tutti i record',
      'user_management': 'Gestione utente',
      'users': 'Utenti',
      'new_user': 'New User',
      'edit_user': 'Modifca Utente',
      'created_user': 'Utente creato con successo',
      'updated_user': 'Utente aggiornato con successo',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Utente eliminato con successo',
      'removed_user': 'Utente rimosso con successo',
      'restored_user': 'Utente ripristinato con successo',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Impostazioni generali',
      'invoice_options': 'Opzioni Fattura',
      'hide_paid_to_date': 'Nascondi la data di pagamento',
      'hide_paid_to_date_help':
          'Visualizza l\'area \"Pagato alla data\" sulle fatture solo dopo aver ricevuto un pagamento.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Includi immagini allegate alla fattura.',
      'all_pages_header': 'Mostra l\'Intestazione nel',
      'all_pages_footer': 'Visualizza Piè di Pagina nel',
      'first_page': 'Prima pagina',
      'all_pages': 'Tutte le pagine',
      'last_page': 'Ultima pagina',
      'primary_font': 'Font primario',
      'secondary_font': 'Font secondario',
      'primary_color': 'Colore primario',
      'secondary_color': 'Colore secondario',
      'page_size': 'Dimensione Pagina',
      'font_size': 'Dimensione Font',
      'quote_design': 'Stile Preventivo',
      'invoice_fields': 'Campi Fattura',
      'product_fields': 'Campi Prodotto',
      'invoice_terms': 'Termini della fattura',
      'invoice_footer': 'Piè di Pagina Fattura',
      'quote_terms': 'Termini del preventivo',
      'quote_footer': 'Piè di Pagina Preventivi',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Invia automaticamente per email le fatture ricorrenti quando vengono create.',
      'auto_archive_invoice': 'Auto Archiviazione',
      'auto_archive_invoice_help':
          'Archivia automaticamente le fatture quando vengono pagate.',
      'auto_archive_quote': 'Auto Archiviazione',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Conversione automatica',
      'auto_convert_quote_help':
          'Converti automaticamente un preventivo in una fattura se approvato da un cliente.',
      'workflow_settings': 'Impostazioni Flusso di Lavoro',
      'freq_daily': 'Giornaliero',
      'freq_weekly': 'Settimanale',
      'freq_two_weeks': 'Due settimane',
      'freq_four_weeks': 'Quattro settimane',
      'freq_monthly': 'Mensile',
      'freq_two_months': 'Due mesi',
      'freq_three_months': 'Tre Mesi',
      'freq_four_months': 'Quattro mesi',
      'freq_six_months': 'Sei Mesi',
      'freq_annually': 'Annuale',
      'freq_two_years': 'Due anni',
      'freq_three_years': 'Tre anni',
      'never': 'Never',
      'company': 'Azienda',
      'generated_numbers': 'Generazione Numeri',
      'charge_taxes': 'Applica Tasse',
      'next_reset': 'Prossimo reset',
      'reset_counter': 'Resetta contatori',
      'recurring_prefix': 'Prefisso Ricorrente',
      'number_padding': 'Number Padding',
      'general': 'Generale',
      'surcharge_field': 'Campo Sovrattassa',
      'company_field': 'Campo azienda',
      'company_value': 'Valore azienda',
      'credit_field': 'Campo Credito',
      'invoice_field': 'Campo fattura',
      'invoice_surcharge': 'Sovrapprezzo Fattura',
      'client_field': 'Campo Cliente',
      'product_field': 'Campo Prodotto',
      'payment_field': 'Campo Pagamento',
      'contact_field': 'Campo Contatti',
      'vendor_field': 'Campo Fornitore',
      'expense_field': 'Campo Spese',
      'project_field': 'Campo Progetto',
      'task_field': 'Campo attività',
      'group_field': 'Campo Gruppo',
      'number_counter': 'Number Counter',
      'prefix': 'Prefisso',
      'number_pattern': 'Pattern numerico',
      'messages': 'Messaggi',
      'custom_css': 'CSS Personalizzato',
      'custom_javascript': 'Javascript personalizzato',
      'signature_on_pdf': 'Mostra su PDF',
      'signature_on_pdf_help':
          'Mostra la firma del cliente sul PDF della fattura/preventivo.',
      'show_accept_invoice_terms':
          'Casella di controllo termini di servizio fatture',
      'show_accept_invoice_terms_help':
          'Rendi obbligatoria l\'accettazione dei termini della fattura.',
      'show_accept_quote_terms':
          'Casella di controllo termini di servizio preventivi',
      'show_accept_quote_terms_help':
          'Rendi obbligatoria l\'accettazione dei termini del preventivo.',
      'require_invoice_signature': 'Firma Fattura',
      'require_invoice_signature_help':
          'Richiedi al cliente di firmare la fattura.',
      'require_quote_signature': 'Firma Bozza',
      'enable_portal_password': 'Fatture Protette da Password',
      'enable_portal_password_help':
          'Permette di impostare una password per ogni contatto. Se una password è impostata, al contatto sarà richiesto di inserire una password prima di visualizzare le fatture.',
      'authorization': 'Autorizzazione',
      'subdomain': 'Sottodominio',
      'domain': 'Dominio',
      'portal_mode': 'Modalità portale',
      'email_signature': 'Distinti saluti,',
      'enable_email_markup_help':
          'Rendi più facile per i tuoi clienti pagarti aggiungendo il markup schema.org alle tue e-mail.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Allega PDF',
      'attach_documents': 'Allega documenti',
      'attach_ubl': 'Allega UBL',
      'email_style': 'Stile Email',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Indirizzo di Risposta mail',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processato',
      'credit_card': 'Carta di Credito',
      'bank_transfer': 'Bonifico Bancario',
      'priority': 'Priorità',
      'fee_amount': 'Importo della tassa',
      'fee_percent': 'Tassa Percentuale',
      'fee_cap': 'Tassa massima',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Attiva minimo',
      'enable_max': 'Attiva massimo',
      'min_limit': 'Minimo :min',
      'max_limit': 'Massimo :max',
      'min': 'Min',
      'max': 'ax',
      'accepted_card_logos': 'Loghi carte accettate',
      'credentials': 'Credenziali',
      'update_address': 'Aggiorna indirizzo',
      'update_address_help':
          'Aggiorna l\'indirizzo del cliente con i dettagli forniti',
      'rate': 'Aliquota',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Modifica aliquota fiscale',
      'created_tax_rate': 'Aliquota fiscale creata',
      'updated_tax_rate': 'Aliquota fiscale aggiornata',
      'archived_tax_rate': 'Successfully archived the tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Riempimento automatico prodotti',
      'fill_products_help':
          'Selezionare un prodotto farà automaticamente inserire la descrizione ed il costo',
      'update_products': 'Aggiorna automaticamente i prodotti',
      'update_products_help':
          'Aggiornare una fatura farà automaticamente aggiornare i prodotti',
      'convert_products': 'Converti prodotti',
      'convert_products_help':
          'Converti automaticamenti i prezzi dei prodotti nella valuta del cliente',
      'fees': 'Commissioni',
      'limits': 'Limiti',
      'provider': 'Provider',
      'company_gateway': 'Piattaforma di Pagamento',
      'company_gateways': 'Piattaforme di Pagamento',
      'new_company_gateway': 'Nuova Piattaforma',
      'edit_company_gateway': 'Modifica Piattaforma',
      'created_company_gateway': 'Piattaforma creata con successo',
      'updated_company_gateway': 'Piattaforma aggiornata con successo',
      'archived_company_gateway': 'Piattaforma archiviata con successo',
      'deleted_company_gateway': 'Piattaforma eliminata con successo',
      'restored_company_gateway': 'Piattaforma ripristinata con successo',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continua la modifica',
      'discard_changes': 'Scarta modifiche',
      'default_value': 'Valore di default',
      'disabled': 'Disabled',
      'currency_format': 'Formato moneta',
      'first_day_of_the_week': 'Primo giorno della settimana',
      'first_month_of_the_year': 'Primo mese dell\'anno',
      'sunday': 'Domenica',
      'monday': 'Lunedì',
      'tuesday': 'Martedì',
      'wednesday': 'Mercoledì',
      'thursday': 'Giovedì',
      'friday': 'Venerdì',
      'saturday': 'Sabato',
      'january': 'Gennaio',
      'february': 'Febbraio',
      'march': 'Marzo',
      'april': 'Aprile',
      'may': 'Maggio',
      'june': 'Giugno',
      'july': 'Luglio',
      'august': 'Agosto',
      'september': 'Settembre',
      'october': 'Ottobre',
      'november': 'Novembre',
      'december': 'Dicembre',
      'symbol': 'Simbolo',
      'ocde': 'Codice',
      'date_format': 'Formato data',
      'datetime_format': 'Formato data e ora',
      'military_time': '24 ore',
      'military_time_help': 'Formato 24 ore',
      'send_reminders': 'Send Reminders',
      'timezone': 'Fuso Orario',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtrato per gruppo',
      'filtered_by_invoice': 'Filtrare per fattura',
      'filtered_by_client': 'Filtrato per cliente',
      'filtered_by_vendor': 'Filtrato per fornitore',
      'group_settings': 'Impostazioni gruppo',
      'group': 'Gruppo',
      'groups': 'Gruppi',
      'new_group': 'Nuovo gruppo',
      'edit_group': 'Modifica gruppo',
      'created_group': 'Gruppo creato con successo',
      'updated_group': 'Gruppo aggiornato con successo',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Carica logo',
      'uploaded_logo': 'Logo caricato con successo',
      'logo': 'Logo',
      'saved_settings': 'Impostazioni salvate con successo',
      'product_settings': 'Preferenze Prodotti',
      'device_settings': 'Impostazioni dispositivo',
      'defaults': 'Predefiniti',
      'basic_settings': 'Impostazioni Base',
      'advanced_settings': 'Impostazioni Avanzate',
      'company_details': 'Dettagli Azienda',
      'user_details': 'Dettagli Utente',
      'localization': 'Linguaggio',
      'online_payments': 'Pagamenti Online',
      'tax_rates': 'Aliquote Fiscali',
      'notifications': 'Notifiche',
      'import_export': 'Importa/Esporta',
      'custom_fields': 'Campi Personalizzabili',
      'invoice_design': 'Stile Fattura',
      'buy_now_buttons': 'Puslanti Compra Ora',
      'email_settings': 'Impostazioni email',
      'templates_and_reminders': 'Modelli & Promemoria',
      'credit_cards_and_banks': 'Carte di credito & banche',
      'data_visualizations': 'Visualizzazioni dei dati',
      'price': 'Prezzo',
      'email_sign_up': 'Registrati via Email',
      'google_sign_up': 'Registrati con Google',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Indietro',
      'past_purchases': 'Acquisti passati',
      'annual_subscription': 'Abbonamento Annuale',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count utenti',
      'upgrade': 'Aggiorna',
      'please_enter_a_first_name': 'Si prega di inserire un nome',
      'please_enter_a_last_name': 'Si prega di inserire un cognome',
      'please_agree_to_terms_and_privacy':
          'Si prega di accettare i termini di servizio e della privacy per creare un account.',
      'i_agree_to_the': 'Accetto la',
      'terms_of_service_link': 'Termini di servizio',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Termini di Servizio',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Registrati',
      'account_login': 'Login account',
      'view_website': 'Visualizza sito web',
      'create_account': 'Crea un account',
      'email_login': 'Login email',
      'create_new': 'Crea Nuovo',
      'no_record_selected': 'Nessun dato selezionato',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Scarica',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Fai foto',
      'upload_file': 'Carica file',
      'document': 'Documento',
      'documents': 'Documenti',
      'new_document': 'Nuovo documento',
      'edit_document': 'Modifica documento',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Nessuno Storico',
      'expense_date': 'Data Spesa',
      'pending': 'In attesa',
      'expense_status_1': 'Registrato',
      'expense_status_2': 'In attesa',
      'expense_status_3': 'Fatturata',
      'converted': 'Convertito',
      'add_documents_to_invoice': 'Aggiungere documenti a fattura',
      'exchange_rate': 'Tasso di Cambio',
      'convert_currency': 'Converti valuta',
      'mark_paid': 'Segna come Pagata',
      'category': 'Categoria',
      'address': 'Indirizzo',
      'new_vendor': 'Nuovo Fornitore',
      'created_vendor': 'Fornitore creato con successo',
      'updated_vendor': 'Fornitore aggiornato con successo',
      'archived_vendor': 'Fornitore archiviato con successo',
      'deleted_vendor': 'Fornitore eliminato con successo',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': ':count fornitori archiviati con successo',
      'deleted_vendors': ':count fornitori eliminati con successo',
      'restored_vendors': 'Ripristinati con successo :value fornitori',
      'new_expense': 'Inserisci spesa',
      'created_expense': 'Spesa creata con successo',
      'updated_expense': 'Spesa aggiornata con successo',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Successfully deleted expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copia Spedizione',
      'copy_billing': 'Copia Fatturazione',
      'design': 'Stile',
      'failed_to_find_record': 'Impossibile trovare dati',
      'invoiced': 'Fatturato',
      'logged': 'Registrato',
      'running': 'In corso',
      'resume': 'Riprendi',
      'task_errors':
          'Si prega di correggere eventuali tempi di sovrapposizione',
      'start': 'Inizia',
      'stop': 'Ferma',
      'started_task': 'Attività iniziata con successo',
      'stopped_task': 'Attività arrestata con successo',
      'resumed_task': 'Attività ripresa con sucesso',
      'now': 'Adesso',
      'auto_start_tasks': 'Partenza automaticha delle attività',
      'timer': 'Timer',
      'manual': 'Manuale',
      'budgeted': 'Preventivato',
      'start_time': 'Tempo di inizio',
      'end_time': 'Tempo di fine',
      'date': 'Data',
      'times': 'Tempi',
      'duration': 'Durata',
      'new_task': 'Nuova Attività',
      'created_task': 'Attività creata con successo',
      'updated_task': 'Attività aggiornata con successo',
      'archived_task': 'Attività archiviata con successo',
      'deleted_task': 'Attività cancellata con successo',
      'restored_task': 'Attività ripristinata con successo',
      'archived_tasks': ':count attività archiviate correttamente',
      'deleted_tasks': ':count attività eliminate correttamente',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Si prega di inserire un nome',
      'budgeted_hours': 'Ore preventivate',
      'created_project': 'Progetto creato con successo',
      'updated_project': 'Progetto aggiornato con successo',
      'archived_project': 'Progetto archiviato con successo',
      'deleted_project': 'Progetto eliminato con successo',
      'restored_project': 'Progetto ripristinato con successo',
      'archived_projects': ':count progetti archiviati con successo',
      'deleted_projects': ':count progetti eliminati con successo',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nuovo Progetto',
      'thank_you_for_using_our_app': 'Grazie di avere scelto la nostra app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'clicca qui',
      'click_here_capital': 'Clicca qui',
      'to_rate_it': 'to rate it.',
      'average': 'Media',
      'unapproved': 'non approvato',
      'authenticate_to_change_setting':
          'Si prega di autenticarsi per cambiare questa impostazione',
      'locked': 'Bloccato',
      'authenticate': 'Autentica',
      'please_authenticate': 'Si prega di autenticarsi',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Piè di Pagina',
      'compare': 'Compara',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Oggi',
      'custom_range': 'Intervallo personalizzato',
      'date_range': 'Intervallo di Tempo',
      'current': 'Corrente',
      'previous': 'Precedente',
      'current_period': 'Periodo corrente',
      'comparison_period': 'Periodo di comparazione',
      'previous_period': 'Periodo precedente',
      'previous_year': 'Anno precedente',
      'compare_to': 'Compara a',
      'last7_days': 'Ultimi 7 giorni',
      'last_week': 'L\'ultima settimana',
      'last30_days': 'Ultimi 30 giorni',
      'this_month': 'Questo mese',
      'last_month': 'Mese scorso',
      'this_year': 'Quest\'anno',
      'last_year': 'Anno scorso',
      'custom': 'Personalizzato',
      'clone_to_invoice': 'Clona a fattura',
      'clone_to_quote': 'Clona su preventivo',
      'clone_to_credit': 'Clona su credito',
      'view_invoice': 'Vedi Fattura',
      'convert': 'Convertire',
      'more': 'Altro',
      'edit_client': 'Modifica Cliente',
      'edit_product': 'Modifica Prodotto',
      'edit_invoice': 'Modifica Fattura',
      'edit_quote': 'Modifica Preventivo',
      'edit_payment': 'Modifica pagamento',
      'edit_task': 'Modifica l\'attività',
      'edit_expense': 'Modifica Spesa',
      'edit_vendor': 'Modifica Fornitore',
      'edit_project': 'Modifica Progetto',
      'edit_recurring_expense': 'Modifica Spesa Ricorrente',
      'edit_recurring_quote': 'Modifica Preventivo Ricorrente',
      'billing_address': 'Indirizzo di fatturazione',
      'shipping_address': 'Indirizzo di spedizione',
      'total_revenue': 'Ricavo totale',
      'average_invoice': 'Fattura media',
      'outstanding': 'Inevase',
      'invoices_sent': ':count fatture inviate',
      'active_clients': 'clienti attivi',
      'close': 'Close',
      'email': 'Email',
      'password': 'Password',
      'url': 'URL',
      'secret': 'Segreta',
      'name': 'Nome',
      'logout': 'Esci',
      'login': 'Login',
      'filter': 'Filtra',
      'sort': 'Ordina',
      'search': 'Cerca',
      'active': 'Attivo',
      'archived': 'Archived',
      'deleted': 'Eliminato',
      'dashboard': 'Pannello di Controllo',
      'archive': 'Archivia',
      'delete': 'Elimina',
      'restore': 'Ripristina',
      'refresh_complete': 'Aggiornamento completi',
      'please_enter_your_email': 'Inserisci la tua email',
      'please_enter_your_password': 'Si prega di inserire la password',
      'please_enter_your_url': 'Inserisci il tuo URL',
      'please_enter_a_product_key': 'Inserire una chiave prodotto',
      'ascending': 'Crescente',
      'descending': 'Decrescente',
      'save': 'Salva',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Pagato a oggi',
      'balance_due': 'Totale da Pagare',
      'balance': 'Bilancio',
      'overview': 'Panoramica',
      'details': 'Dettagli',
      'phone': 'Telefono',
      'website': 'Sito web',
      'vat_number': 'Partita IVA',
      'id_number': 'Codice Fiscale',
      'create': 'Crea',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Errore',
      'could_not_launch': 'Esecuzione non riuscita',
      'contacts': 'Contatti',
      'additional': 'Aggiuntivo',
      'first_name': 'Nome',
      'last_name': 'Cognome',
      'add_contact': 'Aggiungi contatto',
      'are_you_sure': 'Sei sicuro?',
      'cancel': 'Annulla',
      'ok': 'Ok',
      'remove': 'Elimina',
      'email_is_invalid': 'Email non valida',
      'product': 'Prodotto',
      'products': 'Prodotti',
      'new_product': 'Nuovo Prodotto',
      'created_product': 'Prodotto creato con successo',
      'updated_product': 'Prodotto aggiornato con successo',
      'archived_product': 'Prodotto archiviato con successo',
      'deleted_product': 'Prodotto eliminato con successo',
      'restored_product': 'Prodotto ripristinato con successo',
      'archived_products': 'Archiviati con successo :count prodotti',
      'deleted_products': ':count prodotti eliminati con successo',
      'restored_products': 'Ripristinati con successo :value prodotti',
      'product_key': 'Prodotto',
      'notes': 'Note',
      'cost': 'Cost',
      'client': 'Cliente',
      'clients': 'Clienti',
      'new_client': 'Nuovo Cliente',
      'created_client': 'Cliente creato con successo',
      'updated_client': 'Cliente aggiornato con successo',
      'archived_client': 'Cliente archiviato con successo',
      'archived_clients': ':count clienti archiviati con successo',
      'deleted_client': 'Cliente eliminato con successo',
      'deleted_clients': ':count clienti eliminati con successo',
      'restored_client': 'Cliente ripristinato con successo',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Via',
      'address2': 'Appartamento/Piano',
      'city': 'Città',
      'state': 'Stato/Provincia',
      'postal_code': 'Codice postale',
      'country': 'Paese',
      'invoice': 'Fattura',
      'invoices': 'Fatture',
      'new_invoice': 'Nuova Fattura',
      'created_invoice': 'Fattura creata con successo',
      'updated_invoice': 'Fattura aggiornata con successo',
      'archived_invoice': 'Fattura archiviata con successo',
      'deleted_invoice': 'Fattura eliminata con successo',
      'restored_invoice': 'Fattura ripristinata con successo',
      'archived_invoices': ':count fatture archiviate con successo',
      'deleted_invoices': ':count fatture eliminate con successo',
      'restored_invoices': 'Ripristinato con successo :value fatture',
      'emailed_invoice': 'Fattura inviata con successo',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Importo',
      'invoice_number': 'Numero Fattura',
      'invoice_date': 'Data Fattura',
      'discount': 'Sconto',
      'po_number': 'Numero d\'ordine d\'acquisto',
      'terms': 'Termini',
      'public_notes': 'Note Pubbliche (Descrizione in fattura)',
      'private_notes': 'Note Personali',
      'frequency': 'Frequenza',
      'start_date': 'Data Inizio',
      'end_date': 'Data Fine',
      'quote_number': 'Numero Preventivo',
      'quote_date': 'Data Preventivo',
      'valid_until': 'Valido fino a',
      'items': 'Articoli',
      'partial_deposit': 'Parziale/Deposito',
      'description': 'Descrizione',
      'unit_cost': 'Costo Unitario',
      'quantity': 'Quantità',
      'add_item': 'Aggiungi Articolo',
      'contact': 'Contatto',
      'work_phone': 'Telefono',
      'total_amount': 'Ammontare Totale',
      'pdf': 'PDF',
      'due_date': 'Scadenza',
      'partial_due_date': 'Partial Due Date',
      'status': 'Stato',
      'invoice_status_id': 'Stato della fattura',
      'quote_status': 'Stato preventivo',
      'click_plus_to_add_item': 'Clicca su + per aggiungere un articolo',
      'click_plus_to_add_time': 'Premi + per aggiungere tempo',
      'count_selected': ':count selected',
      'total': 'Totale',
      'percent': 'Percentuale',
      'edit': 'Modifica',
      'dismiss': 'Chiudi',
      'please_select_a_date': 'Selezionate una data per favore',
      'please_select_a_client': 'Seleziona un cliente',
      'please_select_an_invoice': 'Selezionate una fattura per favore',
      'task_rate': 'Tariffa per le  attività',
      'settings': 'Impostazioni',
      'language': 'Linguaggio',
      'currency': 'Valuta',
      'created_at': 'Data Creazione',
      'created_on': 'Creato il',
      'updated_at': 'Aggiornato',
      'tax': 'Tassa',
      'please_enter_an_invoice_number':
          'Si prega di inserire un numero di fattura',
      'please_enter_a_quote_number': 'Inserisci un numero preventivo',
      'past_due': 'Scaduta',
      'draft': 'Bozza',
      'sent': 'Inviato',
      'viewed': 'Visto',
      'approved': 'Approvato',
      'partial': 'Parziale/Deposito',
      'paid': 'Pagata',
      'mark_sent': 'Contrassegna come inviato',
      'marked_invoice_as_sent':
          'Fattura contrassegnata con successo come inviata',
      'marked_invoice_as_paid':
          'Fattura contrassegnata con successo come inviata',
      'marked_invoices_as_sent':
          'Fatture contrassegnate con successo come inviate',
      'marked_invoices_as_paid':
          'Fatture contrassegnate con successo come inviate',
      'done': 'Fatto',
      'please_enter_a_client_or_contact_name':
          'Si prega di inserire un cliente o nome del contatto',
      'dark_mode': 'Modalità scura',
      'restart_app_to_apply_change':
          'Riavviare la app per applicare il cambiamento',
      'refresh_data': 'Aggiorna dati',
      'blank_contact': 'Svuota Contatti',
      'activity': 'Attività',
      'no_records_found': 'Nessun dato trovato',
      'clone': 'Clona',
      'loading': 'Caricando',
      'industry': 'Industria',
      'size': 'Dimensione',
      'payment_terms': 'Termini di pagamento',
      'payment_date': 'Data Pagamento',
      'payment_status': 'Stato del pagamento',
      'payment_status_1': 'In attesa',
      'payment_status_2': 'Annullato',
      'payment_status_3': 'Fallito',
      'payment_status_4': 'Completato',
      'payment_status_5': 'Parzialmente rimborsato',
      'payment_status_6': 'Rimborsato',
      'payment_status_-1': 'Non applicato',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Netto',
      'client_portal': 'Portale Clienti',
      'show_tasks': 'Mostra attività',
      'email_reminders': 'Email Reminders',
      'enabled': 'Abilitato',
      'recipients': 'Destinatari',
      'initial_email': 'Email iniziale',
      'first_reminder': 'Primo Promemoria',
      'second_reminder': 'Secondo Promemoria',
      'third_reminder': 'Terzo Promemoria',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Modello',
      'send': 'Invia',
      'subject': 'Oggetto',
      'body': 'Corpo',
      'send_email': 'Invia Email',
      'email_receipt': 'Invia ricevuta di pagamento al cliente',
      'auto_billing': 'Fatturazione automatica',
      'button': 'Pulsante',
      'preview': 'Preview',
      'customize': 'Personalizza',
      'history': 'Storico',
      'payment': 'Pagamento',
      'payments': 'Pagamenti',
      'refunded': 'Rimborsato',
      'payment_type': 'Tipo di Pagamento',
      'transaction_reference': 'Riferimento Transazione',
      'enter_payment': 'Inserisci Pagamento',
      'new_payment': 'Inserisci il pagamento',
      'created_payment': 'Pagamento creato con successo',
      'updated_payment': 'Pagamento aggiornato con successo',
      'archived_payment': 'Pagamento archiviato con successo',
      'deleted_payment': 'Pagamenti eliminati con successo',
      'restored_payment': 'Pagamento ripristinato con successo',
      'archived_payments': ':count pagamenti archiviati con successo',
      'deleted_payments': ':count pagamenti eliminati con successo',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Preventivo',
      'quotes': 'Preventivi',
      'new_quote': 'Nuovo Preventivo',
      'created_quote': 'Preventivo creato con successo',
      'updated_quote': 'Preventivo aggiornato con successo',
      'archived_quote': 'Preventivo archiviato con successo',
      'deleted_quote': 'Preventivo cancellato con successo',
      'restored_quote': 'Preventivo ripristinato con successo',
      'archived_quotes': 'Sono stati archiviati :count preventivi con successo',
      'deleted_quotes': 'Sono stati cancellati :count preventivi con successo',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Spesa',
      'expenses': 'Spese',
      'vendor': 'Fornitore',
      'vendors': 'Fornitori',
      'task': 'Attività',
      'tasks': 'Attività',
      'project': 'Progetto',
      'projects': 'Progetti',
      'activity_1': ':user ha creato il cliente :client',
      'activity_2': ':user ha archiviato il cliente :client',
      'activity_3': ':user ha cancellato il cliente :client',
      'activity_4': ':user ha creato la fattura :invoice',
      'activity_5': ':user ha aggiornato la fattura :invoice',
      'activity_6':
          ':user ha inviato per email la fattura :invoice per:client a :contact',
      'activity_7': ':contact ha visualizzato la fattura :invoice per :client',
      'activity_8': ':user ha archiviato la fattura :invoice',
      'activity_9': ':user ha cancellato la fattura :invoice',
      'activity_10':
          ':contact ha registrato il pagamento :payment di :payment_amount sulla fattura :invoice per :client',
      'activity_11': ':user ha aggiornato il pagamento :payment',
      'activity_12': ':user ha archiviato il pagamento :payment',
      'activity_13': ':user ha cancellato il pagamento :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20':
          ':user ha inviato per email il preventivo :quote per :client a :contact',
      'activity_21': ':contact ha visto il preventivo :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user ha ripristinato il cliente :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact ha approvato il preventivo :quote per :client',
      'activity_30': 'L\'utente :user ha creato il fornitore :vendor',
      'activity_31': 'L\'utente :user ha archiviato il fornitore :vendor',
      'activity_32': 'L\'utente :user ha eliminato il fornitore :vendor',
      'activity_33': 'L\'utente :user ha ripristinato il fornitore :vendor',
      'activity_34': 'L\'utente :user ha creato la spesa :expense',
      'activity_35': 'L\'utente :user ha archiviato la spesa :expense',
      'activity_36': 'L\'utente :user ha eliminato la spesa :expense',
      'activity_37': 'L\'utente :user ha ripristinato la spesa :expense',
      'activity_39':
          ':user ha annullato un pagamento :payment da :payment_amount',
      'activity_40':
          ':user ha rimborsato :adjustment di un pagamento :payment da :payment_amount',
      'activity_41': 'pagamento di :payment_amount (:payment) fallito',
      'activity_42': 'L\'utente :user ha creato l\'attività :task',
      'activity_43': 'L\'utente :user ha aggiornato l\'attività :task',
      'activity_44': 'L\'utente :user ha archiviato l\'attività :task',
      'activity_45': 'L\'utente :user ha eliminato l\'attività :task',
      'activity_46': 'L\'utente :user ha ripristinato l\'attività :task',
      'activity_47': 'L\'utente :user ha aggiornato la spesa :expense',
      'activity_48': ':user ha aggiornato il ticket :ticket',
      'activity_49': ':user ha chiuso il ticket :ticket',
      'activity_50': ':user ha unito il ticket :ticket',
      'activity_51': ':user ha separato il ticket :ticket',
      'activity_52': ':contact ha aperto il ticket :ticket',
      'activity_53': ':contact ha riaperto il ticket :ticket',
      'activity_54': ':user ha riaperto il ticket :ticket',
      'activity_55': ':contact ha risposto al ticket :ticket',
      'activity_56': ':user ha visualizzato il ticket :ticket',
      'activity_57':
          'Il sistema non è riuscito a inviare la fattura :invoice via e-mail',
      'activity_58': ':user ha stornato la fattura :invoice',
      'activity_59': ':user ha cancellato la fattura :invoice',
      'activity_60': ':contact ha visualizzato il preventivo :quote',
      'activity_61': ':user ha aggiornato il cliente :client',
      'activity_62': ':user ha aggiornato il fornitore :vendor',
      'activity_63':
          ':user ha inviato il primo promemoria per la fattura :invoice a :contact',
      'activity_64':
          ':user ha inviato un secondo promemoria per la fattura :invoice a :contact',
      'activity_65':
          ':user ha inviato il terzo promemoria per la fattura :invoice a :contact',
      'activity_66':
          ':user ha inviato un promemoria ricorrente per la fattura :invoice a :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Password a uso singolo',
      'emailed_quote': 'Preventivo inviato con successo',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Scaduto',
      'all': 'Tutti',
      'select': 'Seleziona',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Valore Personalizzato',
      'custom_value2': 'Valore Personalizzato',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard':
          'Messaggio Pannello di Controllo Personalizzato',
      'custom_message_unpaid_invoice':
          'Messaggio personalizzato su fattura non pagata',
      'custom_message_paid_invoice': 'Messaggio personalizzato fattura pagata',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Blocca fatture',
      'translations': 'Traduzioni',
      'task_number_pattern': 'Pattern numero attività',
      'task_number_counter': 'Contatore numero attività',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Pattern numerazione fatture',
      'invoice_number_counter': 'Contatore numerazione fatture',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Contatore numerazione preventivi',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Contatore condiviso fatture/preventivi',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Oggetto della fattura e-mail',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Oggetto e-mail pagamento parziale',
      'show_table': 'Mostra Tabella',
      'show_list': 'Mostra Lista',
      'client_city': 'Città cliente',
      'client_state': 'Stato cliente',
      'client_country': 'Paese cliente',
      'client_is_active': 'Il cliente è attivo',
      'client_balance': 'Bilancio cliente',
      'client_address1': 'Via del cliente',
      'client_address2': 'Appartamento/Scala del cliente',
      'vendor_address1': 'Via Fornitore',
      'vendor_address2': 'Scala/Appartamento Fornitore',
      'client_shipping_address1': 'Via spedizione cliente',
      'client_shipping_address2': 'Appartametno/Scala spedizione cliente',
      'type': 'Tipo',
      'invoice_amount': 'Importo Fattura',
      'invoice_due_date': 'Scadenza fattura',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Fatturazione automatica',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Tasse Personalizzate 1',
      'custom_taxes2': 'Tasse Personalizzate 2',
      'custom_taxes3': 'Tasse Personalizzate 3',
      'custom_taxes4': 'Tasse Personalizzate 4',
      'custom_surcharge1': 'Sovrattassa personalizzata 1',
      'custom_surcharge2': 'Sovrattassa personalizzata 2',
      'custom_surcharge3': 'Sovrattassa personalizzata 3',
      'custom_surcharge4': 'Sovrattassa personalizzata 4',
      'is_deleted': 'È cancellato',
      'vendor_city': 'Città Fornitore',
      'vendor_state': 'Stato Fornitore',
      'vendor_country': 'Paese fornitore',
      'is_approved': 'È approvato',
      'tax_name': 'Nome Tassa',
      'tax_amount': 'Tax Amount',
      'tax_paid': '`Tassa pagata',
      'payment_amount': 'Importo Pagamento',
      'age': 'Età',
      'is_running': 'Is Running',
      'time_log': 'Log temporale',
      'bank_id': 'Banca',
      'expense_category_id': 'ID della categoria di spesa',
      'expense_category': 'Categoria Spesa',
      'invoice_currency_id': 'ID Valuta Fattura',
      'tax_name1': 'Nome tassa 1',
      'tax_name2': 'Nome tassa 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'ID transazione',
      'color_theme': 'Color Theme',
    },
    'ja': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': '有効',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help': 'あなた自身のWebサイトで請求書を表示するために、サブドメインを設定',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': '2要素認証が有効化されました',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': '2要素認証',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email': 'メールアドレス確認用メールが送信されました',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': '請求書に変換',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Invoice Task',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': '隠す',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'カラム',
      'sample': 'サンプル',
      'map_to': 'Map To',
      'import': 'インポート',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'ファイルを選択してください。',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSVファイル',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': '未払い',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': '送信済請求書はロックされました',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': '請求合計',
      'quote_total': '見積金額合計',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': '経費番号',
      'task_number': 'タスク番号',
      'project_number': 'プロジェクト番号',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': '顧客名',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': '強制的にアップデート',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Recurring Invoice',
      'recurring_invoices': '繰り返しの請求書',
      'new_recurring_invoice': 'New Recurring Invoice',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': '繰り返しの請求書をアーカイブしました。',
      'deleted_recurring_invoice': '繰り返しの請求書を削除しました。',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'リンクをコピー',
      'token_billing': 'Save card details',
      'welcome_to_invoice_ninja': 'Invoice Ninja へようこそ',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': '顧客番号',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'リマインダー1送信済',
      'reminder2_sent': 'リマインダー2送信済',
      'reminder3_sent': 'リマインダー3送信済',
      'reminder_last_sent': '最終リマインダー送信済',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hours',
      'statement': 'Statement',
      'taxes': '税',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'To',
      'health_check': 'Health Check',
      'payment_type_id': '入金方法',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': '顧客を登録しました',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': '完了',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': '次回請求',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': '最近の入金',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': '請求を新規作成',
      'create_quote': '見積書を新規作成',
      'create_payment': 'Create Payment',
      'create_vendor': '新しい納入業者',
      'update_quote': 'Update Quote',
      'delete_quote': '見積書を削除',
      'update_invoice': 'Update Invoice',
      'delete_invoice': '請求書を削除',
      'update_client': 'Update Client',
      'delete_client': '顧客を削除',
      'delete_payment': '入金を削除',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Delete Vendor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'タスクを新規作成',
      'update_task': 'Update Task',
      'delete_task': 'タスクを削除',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'フリー',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'APIトークン',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'トークン',
      'tokens': 'トークン',
      'new_token': 'New Token',
      'edit_token': 'トークンを編集',
      'created_token': 'トークンを追加しました。',
      'updated_token': 'トークンを更新しました。',
      'archived_token': 'トークンをアーカイブしました。',
      'deleted_token': 'トークンを削除しました。',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': '請求書をメールする',
      'email_quote': '見積書をメール',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'メールアドレスでログイン',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': '前受金総額',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': '文書の検索',
      'search_designs': 'Search Designs',
      'search_invoices': '請求書の検索',
      'search_clients': '顧客の検索',
      'search_products': '商品の検索',
      'search_quotes': '見積書の検索',
      'search_credits': 'Search Credits',
      'search_vendors': '納入業者の検索',
      'search_users': 'ユーザーの検索',
      'search_tax_rates': '税率の検索',
      'search_tasks': 'タスクの検索',
      'search_settings': '設定の検索',
      'search_projects': 'プロジェクトの検索',
      'search_expenses': '経費の検索',
      'search_payments': 'Search Payments',
      'search_groups': 'グループの検索',
      'search_company': '会社の検索',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count invoice sent',
      'quote_sent': '見積書送信済',
      'credit_sent': '前受金送信済',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'アカウントのキャンセル',
      'cancel_account_message': '注意: あなたのアカウントを完全に削除します。削除の取り消しは出来ません。',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'ヘッダ',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': '繰り返しの見積書',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': '前受金受日付',
      'credit': '前受金',
      'credits': '前受金',
      'new_credit': '前受金を登録',
      'edit_credit': 'Edit Credit',
      'created_credit': '前受金を登録しました。',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': '前受金をアーカイブしました。',
      'deleted_credit': '前受金を削除しました。',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': '前受金を復旧しました',
      'archived_credits': ':count 件の前受金をアーカイブしました。',
      'deleted_credits': ':count 件の前受金を削除しました。',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': '現在のバージョン',
      'latest_version': 'Latest Version',
      'update_now': '今すぐアップデート',
      'a_new_version_is_available': 'Webアプリの新しいバージョンが利用可能です',
      'update_available': 'アップデートが利用可能です',
      'app_updated': 'Update successfully completed',
      'learn_more': '詳細を見る',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': '新しい会社',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'リセット',
      'number': '番号',
      'export': 'エクスポート',
      'chart': 'チャート',
      'count': 'Count',
      'totals': 'Totals',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Group by',
      'credit_balance': '前受金残高',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'レポート',
      'report': 'レポート',
      'add_company': '会社を追加',
      'unpaid_invoice': '未払の請求書',
      'paid_invoice': '支払済の請求書',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'ヘルプ',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'メッセージ',
      'from': 'From',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Documentation',
      'contact_us': 'Contact Us',
      'subtotal': '小計',
      'line_total': 'Line Total',
      'item': 'アイテム',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'はい',
      'no': 'いいえ',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'ユーザ',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': '税の設定',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'パスワードの再設定',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'スケジュール',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': '請求書メール',
      'payment_email': '支払いメール',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': '見積書メール',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'ユーザ管理',
      'users': 'ユーザー',
      'new_user': '新しいユーザ',
      'edit_user': 'ユーザの編集',
      'created_user': 'Successfully created user',
      'updated_user': 'ユーザを更新しました',
      'archived_user': 'ユーザをアーカイブしました。',
      'deleted_user': 'ユーザを削除しました',
      'removed_user': 'Successfully removed user',
      'restored_user': 'ユーザをリストアしました。',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': '一般設定',
      'invoice_options': '請求書オプション',
      'hide_paid_to_date': 'Hide Paid to Date',
      'hide_paid_to_date_help':
          'Only display the \"Paid to Date\" area on your invoices once a payment has been received.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show header on',
      'all_pages_footer': 'Show footer on',
      'first_page': '最初のページ',
      'all_pages': '全てのページ',
      'last_page': '最後のページ',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'プライマリ・カラー',
      'secondary_color': 'セカンダリ・カラー',
      'page_size': 'Page Size',
      'font_size': 'フォントサイズ',
      'quote_design': 'Quote Design',
      'invoice_fields': '請求書をフィールド',
      'product_fields': 'Product Fields',
      'invoice_terms': '請求条件',
      'invoice_footer': '請求書フッター',
      'quote_terms': 'Quote Terms',
      'quote_footer': '見積書フッタ',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help': '支払い完了時に請求書を自動的にアーカイブする',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Two weeks',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Charge taxes',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'プレフィックス',
      'number_pattern': 'Number Pattern',
      'messages': 'メッセージ',
      'custom_css': 'カスタムCSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'サブドメイン',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'どうぞよろしくお願いいたします。',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'プレーン',
      'light': 'ライト',
      'dark': 'ダーク',
      'email_design': 'Eメール デザイン',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'マークアップを許可する',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': '住所を更新',
      'update_address_help': 'Update client\'s address with provided details',
      'rate': '率',
      'tax_rate': '税率',
      'new_tax_rate': '新しい税率',
      'edit_tax_rate': '税率を編集',
      'created_tax_rate': '税率を作成しました',
      'updated_tax_rate': '税率を更新しました。',
      'archived_tax_rate': '税率をアーカイブしました。',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-fill products',
      'fill_products_help':
          'Selecting a product will automatically fill in the description and cost',
      'update_products': '商品の自動更新',
      'update_products_help':
          'Updating an invoice will automatically update the product library',
      'convert_products': '商品の変換',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': '通貨フォーマット',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': '日曜日',
      'monday': '月曜日',
      'tuesday': '火曜日',
      'wednesday': '水曜日',
      'thursday': '木曜日',
      'friday': '金曜日',
      'saturday': '土曜日',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Hour Time',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'ロゴ',
      'saved_settings': 'Successfully saved settings',
      'product_settings': '商品設定',
      'device_settings': 'Device Settings',
      'defaults': 'デフォルト',
      'basic_settings': 'Basic Settings',
      'advanced_settings': '詳細設定',
      'company_details': '企業情報',
      'user_details': 'ユーザの詳細',
      'localization': '地域設定',
      'online_payments': 'オンライン入金',
      'tax_rates': '税率',
      'notifications': '通知',
      'import_export': 'インポート | エクスポート | キャンセル',
      'custom_fields': 'カスタムフィールド',
      'invoice_design': '請求書デザイン',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Eメール設定',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'ビジュアルデータ',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': '利用規約',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'サインアップ',
      'account_login': 'Account Login',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'ダウンロード',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': '保留',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Converted',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': '支払済にする',
      'category': 'Category',
      'address': '住所',
      'new_vendor': 'New Vendor',
      'created_vendor': 'ベンダーを登録しました。',
      'updated_vendor': 'ベンダーを更新しました。',
      'archived_vendor': 'ベンダーをアーカイブしました。',
      'deleted_vendor': 'ベンダーを削除しました。',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': ':count 件の納入業者をアーカイブしました。',
      'deleted_vendors': ':count 件の納入業者を削除しました。',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Enter Expense',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': '経費のアーカイブに成功しました',
      'deleted_expenses': '経費の削除に成功しました',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Invoiced',
      'logged': 'Logged',
      'running': 'Running',
      'resume': 'Resume',
      'task_errors': 'Please correct any overlapping times',
      'start': 'スタート',
      'stop': 'ストップ',
      'started_task': 'Successfully started task',
      'stopped_task': 'タスクを停止しました。',
      'resumed_task': 'Successfully resumed task',
      'now': 'Now',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'タイマー',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': '開始時間',
      'end_time': '終了時間',
      'date': '日付',
      'times': 'Times',
      'duration': 'Duration',
      'new_task': '新しいタスク',
      'created_task': 'タスクが登録されました。',
      'updated_task': 'タスクが更新されました。',
      'archived_task': 'タスクをアーカイブしました。',
      'deleted_task': 'タスクを削除しました。',
      'restored_task': 'タスクをリストアしました。',
      'archived_tasks': ':count件のタスクをアーカイブしました。',
      'deleted_tasks': ':count件のタスクを削除しました。',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'プロジェクトを登録しました。',
      'updated_project': 'プロジェクトを更新しました。',
      'archived_project': 'プロジェクトをアーカイブしました。',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'プロジェクトをリストアしました。',
      'archived_projects': ':count 件のプロジェクトをアーカイブしました。',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': '新しいプロジェクト',
      'thank_you_for_using_our_app': '弊社のAppをご利用頂き誠にありがとうございます。',
      'if_you_like_it': 'If you like it please',
      'click_here': 'こちらをクリック',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': '平均',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'フッタ',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'カスタム',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': '請求書を表示',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': '顧客を編集',
      'edit_product': '商品を編集',
      'edit_invoice': '請求を編集',
      'edit_quote': '見積書を編集',
      'edit_payment': '支払いを編集',
      'edit_task': 'タスクを更新',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Edit Vendor',
      'edit_project': 'プロジェクトを編集',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': '請求先住所',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Total Revenue',
      'average_invoice': '請求の平均',
      'outstanding': 'Outstanding',
      'invoices_sent': ':count invoices sent',
      'active_clients': '有効な顧客',
      'close': '閉じる',
      'email': 'Eメール',
      'password': 'パスワード',
      'url': 'URL',
      'secret': 'Secret',
      'name': '名前',
      'logout': 'ログアウト',
      'login': 'ログイン',
      'filter': 'フィルター',
      'sort': 'Sort',
      'search': '検索',
      'active': '有効',
      'archived': 'Archived',
      'deleted': 'Deleted',
      'dashboard': 'ダッシュボード',
      'archive': 'アーカイブ',
      'delete': '削除',
      'restore': 'リストア',
      'refresh_complete': '更新完了',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': '保存',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': '現在までの支払済額',
      'balance_due': 'Balance Due',
      'balance': 'バランス',
      'overview': '概要',
      'details': '詳細',
      'phone': '電話',
      'website': 'WEBサイト',
      'vat_number': 'VATナンバー',
      'id_number': 'IDナンバー',
      'create': '作成',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'コンタクト',
      'additional': 'Additional',
      'first_name': '名',
      'last_name': '姓',
      'add_contact': '連絡先の追加',
      'are_you_sure': 'よろしいですか？',
      'cancel': 'キャンセル',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'メールアドレスが無効です',
      'product': '商品',
      'products': '商品',
      'new_product': '新しい商品',
      'created_product': '商品を登録しました。',
      'updated_product': '商品を更新しました。',
      'archived_product': '商品をアーカイブしました。',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': ':count 個の商品をアーカイブしました。',
      'deleted_products': ':count 個の商品を削除しました。',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'ノート',
      'cost': 'Cost',
      'client': '顧客',
      'clients': '顧客',
      'new_client': '新しい顧客',
      'created_client': '顧客を登録しました。',
      'updated_client': '顧客を更新しました。',
      'archived_client': '顧客をアーカイブしました。',
      'archived_clients': ':count 件の顧客をアーカイブしました。',
      'deleted_client': '顧客を削除しました。',
      'deleted_clients': ':count 県の顧客を削除しました。',
      'restored_client': '顧客をリストアしました。',
      'restored_clients': 'Successfully restored :value clients',
      'address1': '番地',
      'address2': '建物',
      'city': '市区町村',
      'state': '都道府県',
      'postal_code': '郵便番号',
      'country': '国',
      'invoice': '請求書',
      'invoices': '請求書',
      'new_invoice': '新しい請求書',
      'created_invoice': '請求書を登録しました。',
      'updated_invoice': '請求書を更新しました。',
      'archived_invoice': '請求書をアーカイブしました。',
      'deleted_invoice': '請求書を削除しました。',
      'restored_invoice': '請求書をリストアしました。',
      'archived_invoices': ':count 件の請求書をアーカイブしました。',
      'deleted_invoices': ':count 件の請求書を削除しました。',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': '請求書をメールしました。',
      'emailed_payment': 'Successfully emailed payment',
      'amount': '金額',
      'invoice_number': '請求書番号',
      'invoice_date': '請求日',
      'discount': '値引き',
      'po_number': 'PO番号',
      'terms': '条項',
      'public_notes': 'Public Notes',
      'private_notes': '非公開備考',
      'frequency': '頻度',
      'start_date': '開始日',
      'end_date': '終了日',
      'quote_number': '見積書番号',
      'quote_date': '見積日',
      'valid_until': 'Valid Until',
      'items': 'アイテム',
      'partial_deposit': 'Partial/Deposit',
      'description': '説明',
      'unit_cost': '単価',
      'quantity': '数量',
      'add_item': 'アイテムを追加',
      'contact': 'コンタクト',
      'work_phone': '電話番号',
      'total_amount': '合計金額',
      'pdf': 'PDF',
      'due_date': '支払日',
      'partial_due_date': 'Partial Due Date',
      'status': 'ステータス',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': '合計',
      'percent': 'Percent',
      'edit': '編集',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': '設定',
      'language': 'Language',
      'currency': '通貨',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': '税',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': '送信済',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Partial/Deposit',
      'paid': '支払済',
      'mark_sent': '送付済みにする',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': '完了',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'ダークモード',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'アクティビティ',
      'no_records_found': 'No records found',
      'clone': '複製',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': '支払条件',
      'payment_date': '支払日',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': '完了',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': '顧客ポータル',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'サブジェクト',
      'body': '本文',
      'send_email': 'メールを送信',
      'email_receipt': 'Email payment receipt to the client',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'カスタマイズ',
      'history': '履歴',
      'payment': 'Payment',
      'payments': '入金',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Transaction Reference',
      'enter_payment': '入金を登録',
      'new_payment': '入金を登録',
      'created_payment': '入金を登録しました。',
      'updated_payment': '支払いを更新しました',
      'archived_payment': '入金をアーカイブしました。',
      'deleted_payment': '入金を削除しました。',
      'restored_payment': 'Successfully restored payment',
      'archived_payments': ':count 件の入金をアーカイブしました。',
      'deleted_payments': ':count 件の入金を削除しました。',
      'restored_payments': 'Successfully restored :value payments',
      'quote': '見積書',
      'quotes': '見積書',
      'new_quote': '新しい見積書',
      'created_quote': '見積書を新規作成しました。',
      'updated_quote': '見積書を更新しました。',
      'archived_quote': '見積書をアーカイブしました。',
      'deleted_quote': '見積書を削除しました。',
      'restored_quote': '見積書をリストアしました。',
      'archived_quotes': ':count件の見積書をアーカイブしました。',
      'deleted_quotes': ':count件の見積書を削除しました。',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Expense',
      'expenses': '経費',
      'vendor': 'Vendor',
      'vendors': '納入業者',
      'task': 'Task',
      'tasks': 'タスク',
      'project': 'プロジェクト',
      'projects': 'プロジェクト',
      'activity_1': ':user は 顧客 :client を作成しました。',
      'activity_2': ':user は 顧客 :client をアーカイブしました。',
      'activity_3': ':user は 顧客 :client を削除しました。',
      'activity_4': ':user は 請求書 :invoice を作成しました。',
      'activity_5': ':user は 請求書 :invoice をアーカイブしました。',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user は 請求書 :invoice をアーカイブしました。',
      'activity_9': ':user は 請求書 :invoice をアーカイブしました。',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user updated payment :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored client :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user created expense :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'ワンタイムパスワード',
      'emailed_quote': '見積書をメールしました。',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': '選択',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': '請求書をロック',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': '請求書番号カウンター',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': '請求書番号カウンター',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'テーブルを表示',
      'show_list': 'リストを表示',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': '支払期日',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Bill',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': '税名称',
      'tax_amount': 'Tax Amount',
      'tax_paid': '税金支払済',
      'payment_amount': '入金額',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': '銀行',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'lt': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Įgalinti',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sauga',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Sėkmingai įjungta Dviejų-Lygių Autentifikacija',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Dviejų-Lygių Autentifikacija',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Grąžinti mokėjimai',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convert to Invoice',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Išrašyti sąskaitą',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Slėpti',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Stulpelis',
      'sample': 'Pavyzdys',
      'map_to': 'Map To',
      'import': 'Importuoti',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Pasirinkite failą',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Pasirinkti CSV failą',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Dalinis',
      'invoice_total': 'Suma Viso',
      'quote_total': 'Sąmatos viso',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Kliento Vardas',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Būtina sąskaita faktūra',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'Paypal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Debeto sąskaita',
      'recurring_invoices': 'Debeto sąskaitos',
      'new_recurring_invoice': 'Nauja debeto sąskaita',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Rodyti tinklapį',
      'copy_link': 'Copy Link',
      'token_billing': 'Save card details',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Visada',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Įmonės pavadinimas',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Puslapis :current iš :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Valandos',
      'statement': 'Statement',
      'taxes': 'Mokesčiai',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Pirkėjas',
      'health_check': 'Health Check',
      'payment_type_id': 'Mokėjimo tipas',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Naujos sąskaitos',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Naujausi mokėjimai',
      'upcoming_quotes': 'Upcoming Quotes',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': 'Sukurti sąskaitą',
      'create_quote': 'Sukurti sąmatą',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Delete Quote',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Ištrinti sąskaitą',
      'update_client': 'Update Client',
      'delete_client': 'Trinti klientą',
      'delete_payment': 'Ištrinti mokėjimą',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Trinti',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Sukurti darbą',
      'update_task': 'Update Task',
      'delete_task': 'Trinti',
      'approve_quote': 'Approve Quote',
      'off': 'Išj.',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Free',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Ieškoti :count Webhooks',
      'search_webhook': 'Ieškoti 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Ieškoti :count raktų',
      'search_token': 'Ieškoti 1 raktą',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Edit Token',
      'created_token': 'Successfully created token',
      'updated_token': 'Successfully updated token',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Successfully deleted token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Išsiųsti sąskaitą el. paštu',
      'email_quote': 'Email Quote',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kredito suma',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Dalinai grąžinta',
      'search_documents': 'Ieškoti dokumentų',
      'search_designs': 'Ieškoti dizaino',
      'search_invoices': 'Ieškoti sąskaitų-faktūrų',
      'search_clients': 'Ieškoti klientų',
      'search_products': 'Ieškoti prekių',
      'search_quotes': 'Ieškoti pasiūlymų',
      'search_credits': 'Ieškoti grąžinimų',
      'search_vendors': 'Ieškoti tiekėjų',
      'search_users': 'Ieškoti vartotojų',
      'search_tax_rates': 'Ieškoti mokesčių tarifų',
      'search_tasks': 'Ieškoti užduočių',
      'search_settings': 'Ieškoti nustatymų',
      'search_projects': 'Ieškoti projektų',
      'search_expenses': 'Ieškoti išlaidų',
      'search_payments': 'Ieškoti mokėjimų',
      'search_groups': 'Ieškoti grupių',
      'search_company': 'Ieškoti įmonių',
      'search_document': 'Ieškoti 1 dokumentą',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Grąžinti',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count sąskaita išsiųsta',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Cancel Account',
      'cancel_account_message':
          'Warning: This will permanently delete your account, there is no undo.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Viršus',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Pasiūlymai',
      'tickets': 'Tickets',
      'recurring_quotes': 'Pasikartojančios sąmatos',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Išrašymo data',
      'credit': 'Kreditas',
      'credits': 'Kreditai',
      'new_credit': 'Įvesti kreditą',
      'edit_credit': 'Redaguoti Kreditą',
      'created_credit': 'Successfully created credit',
      'updated_credit': 'Sėkmingai atnaujintas kreditas',
      'archived_credit': 'Successfully archived credit',
      'deleted_credit': 'Successfully deleted credit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Successfully restored credit',
      'archived_credits': 'Successfully archived :count credits',
      'deleted_credits': 'Successfully deleted :count credits',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Dabartinė versija',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Plačiau',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Kredito Poraštė',
      'credit_terms': 'Credit Terms',
      'new_company': 'New Company',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Pasirinktinis Klientas 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Iš naujo',
      'number': 'Number',
      'export': 'Export',
      'chart': 'Diagrama',
      'count': 'Count',
      'totals': 'Viso',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Grupuoti pagal',
      'credit_balance': 'Kredito balansas',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Kliento Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Ataskaitos',
      'report': 'Ataskaita',
      'add_company': 'Add Company',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Nepatvirtinti pasiūlymai',
      'help': 'Pagalba',
      'refund': 'Pinigų grąžinimas',
      'refund_date': 'Grąžinimo data',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Žinutė',
      'from': 'Pardavėjas',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'palaikymo forumas',
      'about': 'About',
      'documentation': 'Documentation',
      'contact_us': 'Contact Us',
      'subtotal': 'Tarpinė suma',
      'line_total': 'Suma',
      'item': 'Prekė/Paslauga',
      'credit_email': 'Credit Email',
      'iframe_url': 'Tinklapis',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Slaptažodyje turi būti didžioji raidė ir skaičius',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Taip',
      'no': 'Ne',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Prašome pasirinkti klientą',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Tax Settings',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Perjungti',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Atkurti slaptažodį',
      'late_fees': 'Late Fees',
      'credit_number': 'Kredito Numeris',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Grafikas',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Invoice Email',
      'payment_email': 'Payment Email',
      'partial_payment': 'Dalinis Apmokėjimas',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Dalino Apmokėjimo El. paštas',
      'quote_email': 'Quote Email',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administratorius',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'User Management',
      'users': 'Vartotojai',
      'new_user': 'New User',
      'edit_user': 'Edit User',
      'created_user': 'Successfully created user',
      'updated_user': 'Successfully updated user',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Successfully deleted user',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Successfully restored user',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'General Settings',
      'invoice_options': 'Invoice Options',
      'hide_paid_to_date': 'Hide paid to date',
      'hide_paid_to_date_help':
          'Only display the \"Paid to Date\" area on your invoices once a payment has been received.',
      'invoice_embed_documents': 'Įkelti dokumentai',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show header on',
      'all_pages_footer': 'Show footer on',
      'first_page': 'first page',
      'all_pages': 'all pages',
      'last_page': 'last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primary Color',
      'secondary_color': 'Secondary Color',
      'page_size': 'Page Size',
      'font_size': 'Font Size',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Invoice Fields',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Sąskaitos sąlygos',
      'invoice_footer': 'Invoice footer',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Automatiškai Konvertuoti',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Kasdien',
      'freq_weekly': 'Kas savaitę',
      'freq_two_weeks': 'Dvi savaitės',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Kas mėnesį',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Charge taxes',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Priešdėlis',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Individualizuotas CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Subdomain',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Linkiu geros dienos,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Tamsu',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Kreditinė kortelė',
      'bank_transfer': 'Pavedimu',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Update Address',
      'update_address_help': 'Update client\'s address with provided details',
      'rate': 'Įkainis',
      'tax_rate': 'Tax Rate',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Edit tax rate',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Successfully archived the tax rate',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-fill products',
      'fill_products_help':
          'Selecting a product will automatically fill in the description and cost',
      'update_products': 'Auto-update products',
      'update_products_help':
          'Updating an invoice will automatically update the product library',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Sekmadienis',
      'monday': 'Pirmadienis',
      'tuesday': 'Antradienis',
      'wednesday': 'Trečiadienis',
      'thursday': 'Ketvirtadienis',
      'friday': 'Penktadienis',
      'saturday': 'Šeštadienis',
      'january': 'Sausis',
      'february': 'Vasaris',
      'march': 'Kovas',
      'april': 'Balandis',
      'may': 'Gegužė',
      'june': 'Birželis',
      'july': 'Liepa',
      'august': 'Rugpjūtis',
      'september': 'Rugsėjis',
      'october': 'Spalis',
      'november': 'Lapkritis',
      'december': 'Gruodis',
      'symbol': 'Simbolis',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 val. formatas',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logotipas',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Product Settings',
      'device_settings': 'Device Settings',
      'defaults': 'Numatyti',
      'basic_settings': 'Basic Settings',
      'advanced_settings': 'Advanced Settings',
      'company_details': 'Imonės informacija',
      'user_details': 'User Details',
      'localization': 'Lokalizacija',
      'online_payments': 'Online mokėjimai',
      'tax_rates': 'Mokesčių įkainiai',
      'notifications': 'Pranešimai',
      'import_export': 'Importas/Eksportas',
      'custom_fields': 'Custom fields',
      'invoice_design': 'Invoice Design',
      'buy_now_buttons': 'Pirkti dabar mygtukas',
      'email_settings': 'Email nustatymai',
      'templates_and_reminders': 'Templates & Reminders',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Data Visualizations',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Terms of Service',
      'privacy_policy': 'Privatumo politika',
      'sign_up': 'Prisijunk',
      'account_login': 'Jungtis',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Atsiųsti',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Dokumentai',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'Laukia patvirtinimo',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Converted',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Valiutos kursas',
      'convert_currency': 'Konvertuoti valiutą',
      'mark_paid': 'Mark Paid',
      'category': 'Kategorija',
      'address': 'Adresas',
      'new_vendor': 'Naujas tiekėjas',
      'created_vendor': 'Sukurtas tiekėjas',
      'updated_vendor': 'Atnaujintas tiekėjas',
      'archived_vendor': 'Sėkmingai suarchyvuoti tiekėjai',
      'deleted_vendor': 'Sėkmingai ištrintas tiekėjas',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Sėkmingai suarchyvuoti :count tiekėjai',
      'deleted_vendors': 'Ištrinta :count tiekėjų',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Enter Expense',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Successfully deleted expenses',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Invoiced',
      'logged': 'Logged',
      'running': 'Vykdomas',
      'resume': 'Tęsti',
      'task_errors': 'Please correct any overlapping times',
      'start': 'Pradėti',
      'stop': 'Stabdyti',
      'started_task': 'Successfully started task',
      'stopped_task': 'Successfully stopped task',
      'resumed_task': 'Successfully resumed task',
      'now': 'Dabar',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Chronometras',
      'manual': 'Nurodyti',
      'budgeted': 'Budgeted',
      'start_time': 'Pradžia',
      'end_time': 'Pabaiga',
      'date': 'Data',
      'times': 'Laikas',
      'duration': 'Trukmė',
      'new_task': 'Naujas darbas',
      'created_task': 'Sukurtas darbas',
      'updated_task': 'Atnaujintas darbas',
      'archived_task': 'Successfully archived task',
      'deleted_task': 'Successfully deleted task',
      'restored_task': 'Successfully restored task',
      'archived_tasks': 'Successfully archived :count tasks',
      'deleted_tasks': 'Successfully deleted :count tasks',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'Jei jums patiko prašome',
      'click_here': 'spausti čia',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Vidurkis',
      'unapproved': 'Nepatvirtinta',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Apačia',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Dabartinis',
      'previous': 'Previous',
      'current_period': 'Dabartinis periodas',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Kurti',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Rodyti sąskaitą',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Redaguoti',
      'edit_product': 'Edit Product',
      'edit_invoice': 'Redaguoti',
      'edit_quote': 'Keisti sąmatą',
      'edit_payment': 'Edit Payment',
      'edit_task': 'Keisti',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Keisti',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Billing address',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Iš viso pajamų',
      'average_invoice': 'Sąskaitų vidurkis',
      'outstanding': 'Neapmokėta',
      'invoices_sent': ':count išsiųstos sąskaitos faktūros',
      'active_clients': 'aktyvūs klientai',
      'close': 'Uždaryti',
      'email': 'El. paštas',
      'password': 'Slaptažodis',
      'url': 'URL',
      'secret': 'Slaptas žodis',
      'name': 'Pavadinimas',
      'logout': 'Log Out',
      'login': 'Login',
      'filter': 'Filtras',
      'sort': 'Sort',
      'search': 'Paieška',
      'active': 'Aktyvus',
      'archived': 'Archived',
      'deleted': 'Deleted',
      'dashboard': 'Darbastalis',
      'archive': 'Archyvas',
      'delete': 'Trinti',
      'restore': 'Atkurti',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Saugoti',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Apmokėta',
      'balance_due': 'Suma Viso',
      'balance': 'Balansas',
      'overview': 'ApžvalgaApžvalga',
      'details': 'Informacija',
      'phone': 'Telefonas',
      'website': 'Internetinis puslapis',
      'vat_number': 'PVM kodas',
      'id_number': 'Įmonės kodas',
      'create': 'Kurti',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontaktinė informacija',
      'additional': 'Additional',
      'first_name': 'Vardas',
      'last_name': 'Pavardė',
      'add_contact': 'Pridėti kontaktą',
      'are_you_sure': 'Ar tikrai?',
      'cancel': 'Atšaukti',
      'ok': 'Ok',
      'remove': 'Trinti',
      'email_is_invalid': 'Email is invalid',
      'product': 'Product',
      'products': 'Prekės',
      'new_product': 'New Product',
      'created_product': 'Successfully created product',
      'updated_product': 'Successfully updated product',
      'archived_product': 'Successfully archived product',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Prekė',
      'notes': 'Notes',
      'cost': 'Cost',
      'client': 'Klientas',
      'clients': 'Klientai',
      'new_client': 'Naujas klientas',
      'created_client': 'Klientas sukurtas',
      'updated_client': 'Successfully updated client',
      'archived_client': 'Successfully archived client',
      'archived_clients': 'Successfully archived :count clients',
      'deleted_client': 'Successfully deleted client',
      'deleted_clients': 'Successfully deleted :count clients',
      'restored_client': 'Successfully restored client',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Gatvė',
      'address2': 'Adresas 2',
      'city': 'Miestas',
      'state': 'Apskritis',
      'postal_code': 'Pašto kodas',
      'country': 'Country',
      'invoice': 'Sąskaita faktūra',
      'invoices': 'Sąskaitos',
      'new_invoice': 'Nauja sąskaita',
      'created_invoice': 'Successfully created invoice',
      'updated_invoice': 'Successfully updated invoice',
      'archived_invoice': 'Successfully archived invoice',
      'deleted_invoice': 'Successfully deleted invoice',
      'restored_invoice': 'Successfully restored invoice',
      'archived_invoices': 'Successfully archived :count invoices',
      'deleted_invoices': 'Successfully deleted :count invoices',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Successfully emailed invoice',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Suma',
      'invoice_number': 'Sąskaitos numeris',
      'invoice_date': 'Išrašymo data',
      'discount': 'Nuolaida',
      'po_number': 'Užsakymo numeris',
      'terms': 'Sąlygos',
      'public_notes': 'Viešos pastabos',
      'private_notes': 'Privatūs užrašai',
      'frequency': 'Periodas',
      'start_date': 'Pradžia',
      'end_date': 'Pabaiga',
      'quote_number': 'Sąmatos numeris',
      'quote_date': 'Sąmatos data',
      'valid_until': 'Galioja iki',
      'items': 'Prekės/Paslaugos',
      'partial_deposit': 'Dalinis/Avansas',
      'description': 'Aprašymas',
      'unit_cost': 'Vnt. kaina',
      'quantity': 'Kiekis',
      'add_item': 'Add Item',
      'contact': 'Kontaktai',
      'work_phone': 'Telefonas',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Apmokėti iki',
      'partial_due_date': 'Dalimis Iki Datos',
      'status': 'Būklė',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Viso',
      'percent': 'Percent',
      'edit': 'Edit',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Nustatymai',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Mokestis',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Išsiųsta',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Dalinis/Avansas',
      'paid': 'Apmokėta',
      'mark_sent': 'Mark sent',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Baigta',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Tamsusis Rėžimas',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Įvykiai',
      'no_records_found': 'Nerasta įrašų',
      'clone': 'Clone',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Atsiskaitymo sąlygos',
      'payment_date': 'Mokėjimo data',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Dalinis grąžinimas',
      'payment_status_6': 'Grąžinta',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Client Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Įjungti',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'First Reminder',
      'second_reminder': 'Second Reminder',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Šablonas',
      'send': 'Send',
      'subject': 'Tema',
      'body': 'Žinutė',
      'send_email': 'Siųsti el. laišką',
      'email_receipt': 'Email payment receipt to the client',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Customize',
      'history': 'Istorija',
      'payment': 'Payment',
      'payments': 'Mokėjimai',
      'refunded': 'Grąžinta',
      'payment_type': 'Mokėjimo tipas',
      'transaction_reference': 'Tranzakcijos numeris',
      'enter_payment': 'Įvesti apmokėjimą',
      'new_payment': 'Naujas mokėjimas',
      'created_payment': 'Successfully created payment',
      'updated_payment': 'Mokėjimas atnaujintas',
      'archived_payment': 'Successfully archived payment',
      'deleted_payment': 'Successfully deleted payment',
      'restored_payment': 'Successfully restored payment',
      'archived_payments': 'Successfully archived :count payments',
      'deleted_payments': 'Successfully deleted :count payments',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Sąmata',
      'quotes': 'Sąmatos',
      'new_quote': 'Nauja sąmata',
      'created_quote': 'Successfully created quote',
      'updated_quote': 'Successfully updated quote',
      'archived_quote': 'Successfully archived quote',
      'deleted_quote': 'Successfully deleted quote',
      'restored_quote': 'Successfully restored quote',
      'archived_quotes': 'Successfully archived :count quotes',
      'deleted_quotes': 'Successfully deleted :count quotes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Išlaidos',
      'expenses': 'Išlaidos',
      'vendor': 'Tiekėjas',
      'vendors': 'Tiekėjai',
      'task': 'Task',
      'tasks': 'Darbai',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user sukūrė klientą :client',
      'activity_2': ':user archived client :client',
      'activity_3': ':user deleted client :client',
      'activity_4': ':user sukurta sąskaita :invoice',
      'activity_5': ':user updated invoice :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user archived invoice :invoice',
      'activity_9': ':user deleted invoice :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user atnaujino mokėjimą :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored client :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user sukurta sąskaita :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount mokėjimas (:payment) nepavyko',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Vienkartinis Slaptažodis',
      'emailed_quote': 'Successfully emailed quote',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'Visi',
      'select': 'Pasirinkite',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote':
          'Tinkintas nepatvirtinto pasiūlymo pranešimas',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Invoice Number Counter',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Quote Number Counter',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'El. pašt. Dalino Apmokėjimo Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Sąskaitos suma',
      'invoice_due_date': 'Terminas',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Automatinis mokėjimas',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Mokėjimo suma',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Laiko Registras',
      'bank_id': 'bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'mk_MK': {
      'import_data': 'Внеси податоци',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Про',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Контакт Име',
      'contact_last_name': 'Контакт Презиме',
      'order': 'Order',
      'unassigned': 'Неназначено',
      'partial_value': 'Мора да биде поголемо од нула а помало од вкупно',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Вклучи',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Поставете го поддоменот или прикажете ја фактурата на вашата веб страна.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Осигурување',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Претплати',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Измени претплата',
      'created_subscription': 'Успешно креирана претплата',
      'updated_subscription': 'Успешно ажурирана претплата',
      'archived_subscription': 'Успешно архивирана претплата',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Повторно испраќање на покана',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Отфрлено',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Скенирај го бар кодот со :link компатибилна апликација.',
      'enabled_two_factor':
          'Успешно овозможена автентификација преку два фактори',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Автентификација преку два фактори',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Рефундирано плаќање',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Последен квартал',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Конвертирај во фактура',
      'registration_url': 'Registration URL',
      'invoice_project': 'Фактурирај проект',
      'invoice_task': 'Задача на фактура',
      'invoice_expense': 'Фактурирај трошок',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Конвертиран износ',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Стандардни документи',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Сокриј',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Колона',
      'sample': 'Примерок',
      'map_to': 'Map To',
      'import': 'Внеси',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Ве молиме изберете датотека',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV датотека',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Услуга',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Неплатено',
      'white_label': 'White Label',
      'delivery_note': 'Забелешка за испорака',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Делумен долг',
      'invoice_total': 'Вкупно по фактура',
      'quote_total': 'Вкупно понуди',
      'credit_total': 'Вкупно кредит',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Предупредување',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Име на клиент',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Успешно ажуриран статус на задача',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Категории на трошоци',
      'new_expense_category': 'Категорија на нов трошок',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Успешно креирана категорија на трошоци',
      'updated_expense_category': 'Успешно ажурирана категорија на трошоци',
      'archived_expense_category': 'Успешно архивирана категорија на трошоци',
      'deleted_expense_category': 'Успешно бришење на категорија',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Успешно повратена категорија на трошоци',
      'archived_expense_categories':
          'Успешно архивирани :count категории на трошоци',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Треба  да биде фактурирано',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Обележи активно',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Повторувачка фактура',
      'recurring_invoices': 'Фактури што се повторуваат',
      'new_recurring_invoice': 'Нова повторувачка фактура',
      'edit_recurring_invoice': 'Измени Повторувачка Фактура',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice':
          'Успешно архивирање на повторувачка фактура',
      'deleted_recurring_invoice': 'Успешно бришење на повторувачка фактура',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Успешно повратена повторувачка фактура',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Профит',
      'line_item': 'Ставка на линија',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Отворено',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Прегледај портал',
      'copy_link': 'Copy Link',
      'token_billing': 'Зачувај детали на картичка',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Секогаш',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Назнака',
      'client_number': 'Број на клиент',
      'auto_convert': 'Auto Convert',
      'company_name': 'Име на компанија',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Успешно пратени по е-пошта фактури',
      'emailed_quotes': 'Успешно пратени по е-пошта понуди',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Платен портал',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Часови',
      'statement': 'Исказ',
      'taxes': 'Даноци',
      'surcharge': 'Доплата',
      'apply_payment': 'Apply Payment',
      'apply': 'Примени',
      'unapplied': 'Unapplied',
      'select_label': 'Избери назнака',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'До',
      'health_check': 'Health Check',
      'payment_type_id': 'Начин на плаќање',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Недоспеани Фактури',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Неодамнешни плаќања',
      'upcoming_quotes': 'Претстојни понуди',
      'expired_quotes': 'Истечени понуди',
      'create_client': 'Креирај клиент',
      'create_invoice': 'Креирај Фактура',
      'create_quote': 'Креирај понуда',
      'create_payment': 'Create Payment',
      'create_vendor': 'Креирај продавач',
      'update_quote': 'Update Quote',
      'delete_quote': 'Избриши понуда',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Избриши Фактура',
      'update_client': 'Update Client',
      'delete_client': 'Избриши Клиент',
      'delete_payment': 'Избриши Плаќање',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Избриши продавач',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Избриши трошок',
      'create_task': 'Креирај задача',
      'update_task': 'Update Task',
      'delete_task': 'Избриши задача',
      'approve_quote': 'Approve Quote',
      'off': 'Исклучено',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Бесплатно',
      'plan': 'План',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Цел',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API токени',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Токен',
      'tokens': 'Токени',
      'new_token': 'New Token',
      'edit_token': 'Измени токен',
      'created_token': 'Успешно креирање на токен',
      'updated_token': 'Успешно ажурирање на токен',
      'archived_token': 'Успешно архивирање на токен',
      'deleted_token': 'Успешно бришење на токен',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Прати Фактура по е-пошта',
      'email_quote': 'Прати понуда по ел. пошта',
      'email_credit': 'Email Credit',
      'email_payment': 'Прати плаќање по е-пошта',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Име на контакт',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Измени термин на плаќање',
      'created_payment_term': 'Успешно креирање на услов за плаќање',
      'updated_payment_term': 'Успешно ажурирање на услов на плаќање',
      'archived_payment_term': 'Успешно архивирање на услов за плаќање',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Износ на кредит',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Ексклузивно',
      'inclusive': 'Инклузивно',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Рефундирај плаќање',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Целосно име',
      'city_state_postal': 'Град/Држава/Поштенски број',
      'postal_city_state': 'Поштенски број/Град/Држава',
      'custom1': 'Прво прилагодено',
      'custom2': 'Второ прилагодено',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Прочисти податоци',
      'purge_successful': 'Успешно прочистени податоци за компаанија',
      'purge_data_message':
          'Предупредување: Ова трајно ќе ги избрише вашите податоци, нема враќање назад.',
      'invoice_balance': 'Состојба по Фактура',
      'age_group_0': '0 - 30 дена',
      'age_group_30': '30 - 60 дена',
      'age_group_60': '60 - 90 дена',
      'age_group_90': '90 - 120 дена',
      'age_group_120': '120+ дена',
      'refresh': 'Освежи',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Детали за фактура',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Дозволи',
      'none': 'Нема',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count испратена фактура',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Примени лиценца',
      'cancel_account': 'Избриши сметка',
      'cancel_account_message':
          'Предупредување: Ова трајно ќе ја избрише вашата сметка, нема враќање.',
      'delete_company': 'Избриши компанија',
      'delete_company_message':
          'Предупредување: Ова трајно ќе ја избрише вашата компанија, нема враќање назад.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Заглавје',
      'load_design': 'Вчитај дизајн',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Предлози',
      'tickets': 'Tickets',
      'recurring_quotes': 'Повторувачки понуди',
      'recurring_tasks': 'Повторувачки Задачи',
      'recurring_expenses': 'Повторувачки трошоци',
      'account_management': 'Менаџирање на сметка',
      'credit_date': 'Датум на кредит',
      'credit': 'Кредит',
      'credits': 'Кредити',
      'new_credit': 'Внеси Кредит',
      'edit_credit': 'Измени кредит',
      'created_credit': 'Успешно креирање на кредит',
      'updated_credit': 'Успешно ажурирање на кредит',
      'archived_credit': 'Успешно архивирање на кредит',
      'deleted_credit': 'Успешно бришење на кредит',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Успешно повратување на кредит',
      'archived_credits': 'Успешно архивирање на :count кредити',
      'deleted_credits': 'Успешно бришење на :count кредити',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Сегашна верзија',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Повеќе',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Нова компанија',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Ресетирај',
      'number': 'Number',
      'export': 'Експортирај',
      'chart': 'Графикон',
      'count': 'Count',
      'totals': 'Вкупно',
      'blank': 'Бланко',
      'day': 'Ден',
      'month': 'Месец',
      'year': 'Година',
      'subgroup': 'Подгрупа',
      'is_active': 'Is Active',
      'group_by': 'Групирај по',
      'credit_balance': 'Состојба на кредит',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Телефон за контакт',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Улица за достава',
      'shipping_address2': 'Апартман за достава',
      'shipping_city': 'Град за достава',
      'shipping_state': 'Држава/Провинција за достава',
      'shipping_postal_code': 'Поштенски број за достава',
      'shipping_country': 'Држава за достава',
      'billing_address1': 'Улица за наплата',
      'billing_address2': 'Апартман за наплата',
      'billing_city': 'Град за наплата',
      'billing_state': 'Држава/Провинција за наплата',
      'billing_postal_code': 'Поштенски број за наплата',
      'billing_country': 'Држава за наплата',
      'client_id': 'Идентификација на клиент',
      'assigned_to': 'Assigned to',
      'created_by': 'Креирано по :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Колони',
      'aging': 'Застарено',
      'profit_and_loss': 'Профит и загуба',
      'reports': 'Извештаи',
      'report': 'Извештај',
      'add_company': 'Додај компанија',
      'unpaid_invoice': 'Неплатена фактура',
      'paid_invoice': 'Платена фактура',
      'unapproved_quote': 'Неодобрена понуда',
      'help': 'Помош',
      'refund': 'Рефундирај',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Е-пошта за контакт',
      'multiselect': 'Multiselect',
      'entity_state': 'Состојба',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Порака',
      'from': 'Од',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'Форум за поддршка',
      'about': 'About',
      'documentation': 'Документација',
      'contact_us': 'Контактирајте не\'',
      'subtotal': 'Вкупно без данок',
      'line_total': 'Вкупно',
      'item': 'Ставка',
      'credit_email': 'Credit Email',
      'iframe_url': 'Веб страна',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Да',
      'no': 'Не',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Мобилен',
      'desktop': 'Десктоп',
      'layout': 'Layout',
      'view': 'Преглед',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Корисник',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Ве молиме изберете клиент',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Поставки за данок',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Префрли',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Опции',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Поднеси',
      'recover_password': 'Поврати ја твојата лозинка',
      'late_fees': 'Late Fees',
      'credit_number': 'Број на кредит',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Износ на провизија за задоцнување',
      'late_fee_percent': 'Процент на провизија за задоцнување',
      'schedule': 'Распоред',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Денови',
      'invoice_email': 'Мејл за фактура',
      'payment_email': 'Мејл за плаќање',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Мејл за понуда',
      'endless_reminder': 'Бескраен потсетник',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Администратор',
      'administrator_help':
          'Дозвола за корисникот да менаџира со корисниците, да ги менува поставките и да ги модифицира сите записи',
      'user_management': 'Управување со корисник',
      'users': 'Корисници',
      'new_user': 'Нов корисник',
      'edit_user': 'Измени корисник',
      'created_user': 'Successfully created user',
      'updated_user': 'Успешно ажурирање на корисник',
      'archived_user': 'Успешно архивирање на корисник',
      'deleted_user': 'Успешно бришење на корисник',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Успешно повратување на корисник',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Општи поставки',
      'invoice_options': 'Поставки за фактура',
      'hide_paid_to_date': 'Сокриј Платено до датум',
      'hide_paid_to_date_help':
          'Прикажи \"Платено до датум\" на фактурите откако ќе биде примено плаќањето.',
      'invoice_embed_documents': 'Вметни документи',
      'invoice_embed_documents_help':
          'Вклучи ги прикачените слики во фактурата.',
      'all_pages_header': 'Прикажи заглавје на',
      'all_pages_footer': 'Прикажи футер на',
      'first_page': 'Прва страна',
      'all_pages': 'Сите страни',
      'last_page': 'Последна страна',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Примарна боја',
      'secondary_color': 'Секундарна боја',
      'page_size': 'Големина на страна',
      'font_size': 'Големина на фонт',
      'quote_design': 'Дизајн на понуда',
      'invoice_fields': 'Полиња за фактура',
      'product_fields': 'Полиња на продукт',
      'invoice_terms': 'Услови по фактура',
      'invoice_footer': 'Футер на фактура',
      'quote_terms': 'Услови на понуда',
      'quote_footer': 'Футер на понуда',
      'auto_email_invoice': 'Автоматска е-пошта',
      'auto_email_invoice_help':
          'Автоматски испрати повторувачки фактури по е-пошта кога ќе бидат креирани.',
      'auto_archive_invoice': 'Автоматско архивирање',
      'auto_archive_invoice_help':
          'Автоматски архивирај фактури кога ќе бидат платени.',
      'auto_archive_quote': 'Автоматско архивирање',
      'auto_archive_quote_help':
          'Автоматски архивирај фактури кога ќе бидат конвертирани.',
      'auto_convert_quote': 'Автоматско конвертирање',
      'auto_convert_quote_help':
          'Автоматски конвертирај понуда во фактура кога истата ќе биде одобрена од клиентот.',
      'workflow_settings': 'Подесувања на текот на работа',
      'freq_daily': 'Дневно',
      'freq_weekly': 'Неделно',
      'freq_two_weeks': 'Две недели',
      'freq_four_weeks': 'Четири недели',
      'freq_monthly': 'Месечно',
      'freq_two_months': 'Два месеци',
      'freq_three_months': 'Три месеци',
      'freq_four_months': 'Четири месеци',
      'freq_six_months': 'Шест месеци',
      'freq_annually': 'Годишно',
      'freq_two_years': 'Две години',
      'freq_three_years': 'Three Years',
      'never': 'Никогаш',
      'company': 'Компанија',
      'generated_numbers': 'Генерирани броеви',
      'charge_taxes': 'Наплати даноци',
      'next_reset': 'Следно ресетирање',
      'reset_counter': 'Ресетирај бројач',
      'recurring_prefix': 'Повторувачки префикс',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Поле за компанија',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Поле за фактура',
      'invoice_surcharge': 'Доплата за фактура',
      'client_field': 'Поле за клиент',
      'product_field': 'Поле за продукт',
      'payment_field': 'Payment Field',
      'contact_field': 'Поле за контакт',
      'vendor_field': 'Поле за продавач',
      'expense_field': 'Поле за трошок',
      'project_field': 'Поле за проект',
      'task_field': 'Поле за задача',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Префикс',
      'number_pattern': 'Number Pattern',
      'messages': 'Пораки',
      'custom_css': 'Прилагоден CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Прикажи на PDF',
      'signature_on_pdf_help':
          'Прикажи го потписот на клиентот на PDF фактура/понуда.',
      'show_accept_invoice_terms': 'Поле за избор на услови за фактура',
      'show_accept_invoice_terms_help':
          'Побарај од клиентот да потврди дека ги прифаќа условите на фактурата.',
      'show_accept_quote_terms': 'Поле за избор на услови за понуда',
      'show_accept_quote_terms_help':
          'Побарај од клиентот да потврди дека ги прифаќа условите на понудата.',
      'require_invoice_signature': 'Потпис на фактура',
      'require_invoice_signature_help':
          'Побарај од клиентот да обезбеди потпис.',
      'require_quote_signature': 'Потпис на понуда',
      'enable_portal_password': 'Фактури заштитени со лозинка',
      'enable_portal_password_help':
          'Ви дозволува поставување на лозинка за секој контакт. Ако поставите лозинка. контактот ќе мора да ја внесе лозинката пред да ги прегледа фактурите.',
      'authorization': 'Овластување',
      'subdomain': 'Поддомен',
      'domain': 'Домен',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Со почит,',
      'enable_email_markup_help':
          'Направете го полесно плаќањето за вашите клиенти со додавање на schema.org обележје на вашите е-пошти',
      'plain': 'Обично',
      'light': 'Светло',
      'dark': 'Темно',
      'email_design': 'Дизајн на е-пошта',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Овозможи обележување',
      'reply_to_email': 'Одговори-на е-пошта',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC е-пошта',
      'processed': 'Processed',
      'credit_card': 'Кредитна картичка',
      'bank_transfer': 'Банкарски трансфер',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Овозможи мин.',
      'enable_max': 'Овозможи макс.',
      'min_limit': 'Мин: :min',
      'max_limit': 'Макс: :max',
      'min': 'Мин',
      'max': 'Макс',
      'accepted_card_logos': 'Прифатени логоа на картичка',
      'credentials': 'Credentials',
      'update_address': 'Ажурирај адреса',
      'update_address_help':
          'Ажурирај ја адресата на клиентот со обезбедените детали',
      'rate': 'Стапка',
      'tax_rate': 'Даночна стапка',
      'new_tax_rate': 'Нова стапка на данок',
      'edit_tax_rate': 'Измени стапка на данок',
      'created_tax_rate': 'Успешно креирана стапка на данок',
      'updated_tax_rate': 'Успешно ажурирана стапка на данок',
      'archived_tax_rate': 'Успешно архивирана стапка на данок',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Автоматско пополнување на продукти',
      'fill_products_help':
          'Избирањето на продукт автоматски ќе ги  исполни полињата за опис и цена',
      'update_products': 'Автоматско ажурирање на продукти',
      'update_products_help':
          'Ажурирањето на факура автоматски ќе ја  ажурира библиотеката на продукти ',
      'convert_products': 'Конвертирај продукти',
      'convert_products_help':
          'Автоматски конвертирај ги цените на продуктите по валутите на клиентите',
      'fees': 'Надоместоци',
      'limits': 'Ограничувања',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Отфрли промени',
      'default_value': 'Default value',
      'disabled': 'Оневозможено',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Недела',
      'monday': 'Понеделник',
      'tuesday': 'Вторник',
      'wednesday': 'Среда',
      'thursday': 'Четврток',
      'friday': 'Петок',
      'saturday': 'Сабота',
      'january': 'Јануари',
      'february': 'Февруари',
      'march': 'Март',
      'april': 'Април',
      'may': 'Мај',
      'june': 'Јуни',
      'july': 'Јули',
      'august': 'Август',
      'september': 'Септември',
      'october': 'Октомври',
      'november': 'Ноември',
      'december': 'Декември',
      'symbol': 'Симбол',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': 'Време од 24 часа',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Група',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Лого',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Поставки за продукт',
      'device_settings': 'Device Settings',
      'defaults': 'Стандарди',
      'basic_settings': 'Основни поставки',
      'advanced_settings': 'Напредни подесувања',
      'company_details': 'Податоци за компанијата',
      'user_details': 'Детали за корисникот',
      'localization': 'Локализација',
      'online_payments': 'Онлајн плаќања',
      'tax_rates': 'Даночни стапки',
      'notifications': 'Известувања',
      'import_export': 'Увоз | Извоз',
      'custom_fields': 'Прилагодливи полиња',
      'invoice_design': 'Дизајн на фактура',
      'buy_now_buttons': 'Купи сега копчиња',
      'email_settings': 'Поставки за е-пошта',
      'templates_and_reminders': 'Шаблони и потсетници',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Визуализација на податоци',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Услови на користење',
      'privacy_policy': 'Полиса за приватност',
      'sign_up': 'Најавување',
      'account_login': 'Најавување на сметка',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Креирај сега',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Преземи',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Документ',
      'documents': 'Документи',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Датум на трошок',
      'pending': 'Во тек',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Конвертирано',
      'add_documents_to_invoice': 'Додај документи на фактура',
      'exchange_rate': 'Девизен курс',
      'convert_currency': 'Конвертирај валута',
      'mark_paid': 'Обележи платено',
      'category': 'Категорија',
      'address': 'Адреса',
      'new_vendor': 'Нов продавач',
      'created_vendor': 'Успешно креирање на продавач',
      'updated_vendor': 'Успешно ажурирање на продавач',
      'archived_vendor': 'Успешно архивирање на продавач',
      'deleted_vendor': 'Успешно бришење на продавач',
      'restored_vendor': 'Успешно повраќање на продавач',
      'archived_vendors': 'Успешно архивирање на :count продавачи',
      'deleted_vendors': 'Успешно бришење на :count продавачи',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Внеси трошок',
      'created_expense': 'Успешно креирање на трошок',
      'updated_expense': 'Успешно ажурирање на трошок',
      'archived_expense': 'Успешно архивирање на трошок',
      'deleted_expense': 'Успешно бришење на трошок',
      'restored_expense': 'Успешно повраќање на трошок',
      'archived_expenses': 'Успешно архивирање на трошоци',
      'deleted_expenses': 'Успешно бришење на трошоци',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Копирај достава',
      'copy_billing': 'Копирај наплата',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Фактурирано',
      'logged': 'Најавено',
      'running': 'Во тек',
      'resume': 'Продолжи',
      'task_errors': 'Ве молиме корегирајте времињата што се преклопуваат',
      'start': 'Почеток',
      'stop': 'Сопри',
      'started_task': 'Успешно започната задача',
      'stopped_task': 'Успешно сопирање на задача',
      'resumed_task': 'Успешно повратување на задача',
      'now': 'Сега',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Тајмер',
      'manual': 'Упатство',
      'budgeted': 'Budgeted',
      'start_time': 'Време за почеток',
      'end_time': 'Измени време',
      'date': 'Датум',
      'times': 'Последователност',
      'duration': 'Времетраење',
      'new_task': 'Нова задача',
      'created_task': 'Успешно креирана задача',
      'updated_task': 'Успешно ажурирана задача',
      'archived_task': 'Успешно архивирање на задача',
      'deleted_task': 'Успешно бришење на задача',
      'restored_task': 'Успешно повратување на задача',
      'archived_tasks': 'Успешно архивирани :count задачи',
      'deleted_tasks': 'Успешно избришани :count задачи',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Буџетирани часови',
      'created_project': 'Успешно креирање на проект',
      'updated_project': 'Успешно ажурирање на проект',
      'archived_project': 'Успешно архивирање на проект',
      'deleted_project': 'Успешно бришење на проект',
      'restored_project': 'Успешно повратување на проект',
      'archived_projects': 'Успешно архивирани :count проекти',
      'deleted_projects': 'Успешно избришани :count проекти',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Нов проект',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'кликни тука',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Футер',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Прилагоден опсег',
      'date_range': 'Опсег на датуми',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Овој месец',
      'last_month': 'Претходен месец',
      'this_year': 'Оваа година',
      'last_year': 'Претходната година',
      'custom': 'Прилагодено',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Преглед на фактура',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Измени клиент',
      'edit_product': 'Измени продукт',
      'edit_invoice': 'Измени Фактура',
      'edit_quote': 'Измени понуда',
      'edit_payment': 'Измени плаќање',
      'edit_task': 'Измени задача',
      'edit_expense': 'Измени трошок',
      'edit_vendor': 'Измени продавач',
      'edit_project': 'Измени проект',
      'edit_recurring_expense': 'Измени повторувачки трошок',
      'edit_recurring_quote': 'Измени Повторувачка Понуда',
      'billing_address': 'Адреса на фактурирање',
      'shipping_address': 'Адреса за достава',
      'total_revenue': 'Вкупен приход',
      'average_invoice': 'Просечна Фактура',
      'outstanding': 'Ненаплатено',
      'invoices_sent': ':count испратени фактури',
      'active_clients': 'Активни Клиенти',
      'close': 'Затвори',
      'email': 'Е-пошта',
      'password': 'Лозинка',
      'url': 'URL',
      'secret': 'Тајно',
      'name': 'Име',
      'logout': 'Одјава',
      'login': 'Најава',
      'filter': 'Филтер',
      'sort': 'Подреди',
      'search': 'Пребарување',
      'active': 'Активен',
      'archived': 'Архивирано',
      'deleted': 'Избришано',
      'dashboard': 'Контролна табла',
      'archive': 'Архивирај',
      'delete': 'Избриши',
      'restore': 'Поврати',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Ве молиме внесете ја вашата е-пошта',
      'please_enter_your_password': 'Ве молиме внесете ја вашата лозинка',
      'please_enter_your_url': 'Ве молиме внесете ја вашата URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Растечки',
      'descending': 'Опаѓачки',
      'save': 'Зачувај',
      'an_error_occurred': 'Настана грешка',
      'paid_to_date': 'Платено до денес',
      'balance_due': 'Вкупно за плаќање',
      'balance': 'Состојба',
      'overview': 'Overview',
      'details': 'Детали',
      'phone': 'Телефон',
      'website': 'Веб Страна',
      'vat_number': 'ДДВ број',
      'id_number': 'Идентификациски број',
      'create': 'Креирај',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Грешка',
      'could_not_launch': 'Could not launch',
      'contacts': 'Контакти',
      'additional': 'Additional',
      'first_name': 'Име',
      'last_name': 'Презиме',
      'add_contact': 'Додади контакт',
      'are_you_sure': 'Дали сте сигурни?',
      'cancel': 'Откажи',
      'ok': 'Ok',
      'remove': 'Отстрани',
      'email_is_invalid': 'Email is invalid',
      'product': 'Продукт',
      'products': 'Продукти',
      'new_product': 'Нов продукт',
      'created_product': 'Успешно креирање на продукт',
      'updated_product': 'Успешно ажурирање на продукт',
      'archived_product': 'Успешно архивирање на продукт',
      'deleted_product': 'Успешно бришење на продукт',
      'restored_product': 'Успешно повратување на продукт',
      'archived_products': 'Успешно архивирање на :count продукти',
      'deleted_products': 'Успешно бришење на :count продукти',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Продукт',
      'notes': 'Забелешки',
      'cost': 'Цена',
      'client': 'Клиент',
      'clients': 'Клиенти',
      'new_client': 'Нов Клиент',
      'created_client': 'Успешно креирање на клиент',
      'updated_client': 'Успешно ажурирање на клиент',
      'archived_client': 'Успешно архивирање на клиент',
      'archived_clients': 'Успешно архивирање на :count клиенти',
      'deleted_client': 'Успешно бришење на клиент',
      'deleted_clients': 'Успешно бришење на :count клиенти',
      'restored_client': 'Успешно повратување на клиент',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Улица',
      'address2': 'Број',
      'city': 'Град',
      'state': 'Општина',
      'postal_code': 'Поштенски број',
      'country': 'Држава',
      'invoice': 'Фактура',
      'invoices': 'Фактури',
      'new_invoice': 'Нова Фактура',
      'created_invoice': 'Успешно креирана фактура',
      'updated_invoice': 'Успешно ажурирана фактура',
      'archived_invoice': 'Успешно архивирана фактура',
      'deleted_invoice': 'Успешно избришана Фактура',
      'restored_invoice': 'Успешно повратување на фактура',
      'archived_invoices': 'Успешно архивирани :count  Фактури',
      'deleted_invoices': 'Успешно избришани :count фактури',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Успешно пратена фактура по е-пошта',
      'emailed_payment': 'Успешно пратено плаќање по е-пошта',
      'amount': 'Количина',
      'invoice_number': 'Број на фактура',
      'invoice_date': 'Датаум на фактура',
      'discount': 'Попуст',
      'po_number': 'Број на нарачка',
      'terms': 'Услови',
      'public_notes': 'Јавни забелешки',
      'private_notes': 'Забелешки',
      'frequency': 'Фреквентност',
      'start_date': 'Почетен датум',
      'end_date': 'Краен датум',
      'quote_number': 'Број на понуда',
      'quote_date': 'Датум на понуда',
      'valid_until': 'Валидно до',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Опис',
      'unit_cost': 'Цена на единица',
      'quantity': 'Количина',
      'add_item': 'Add Item',
      'contact': 'Контакт',
      'work_phone': 'Телефон',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Датум на доспевање',
      'partial_due_date': 'Делумен датум на достасување',
      'status': 'Статус',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Вкупно',
      'percent': 'Процент',
      'edit': 'Измени',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Стапка на задача',
      'settings': 'Подесувања',
      'language': 'Language',
      'currency': 'Валута',
      'created_at': 'Датум на креирање',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Данок',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Минато достасување',
      'draft': 'Нацрт',
      'sent': 'Испратено',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Делумно/Депозит',
      'paid': 'Платено',
      'mark_sent': 'Белегот е пратен',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Завршено',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Темен режим',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Активност',
      'no_records_found': 'No records found',
      'clone': 'Клонирај',
      'loading': 'Вчитување',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Услови на плаќање',
      'payment_date': 'Датум на плаќање',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Портал на клиент',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Овозможено',
      'recipients': 'Приматели',
      'initial_email': 'Почетна е-пошта',
      'first_reminder': 'Прв потсетник',
      'second_reminder': 'Втор потсетник',
      'third_reminder': 'Трет потсетник',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Шаблон',
      'send': 'Send',
      'subject': 'Предмет',
      'body': 'Конструкција',
      'send_email': 'Испрати емаил',
      'email_receipt': 'Прати потврда за плаќање на е-пошта до клиентот',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Преглед',
      'customize': 'Прилагоди',
      'history': 'Историја',
      'payment': 'Плаќање',
      'payments': 'Плаќања',
      'refunded': 'Refunded',
      'payment_type': 'Тип на плаќање',
      'transaction_reference': 'Трансакциска референца',
      'enter_payment': 'Внеси уплата',
      'new_payment': 'Внеси Плаќање',
      'created_payment': 'Успешно креирано плаќање',
      'updated_payment': 'Успешно ажурирано плаќање',
      'archived_payment': 'Успешно архивирано плаќање',
      'deleted_payment': 'Успешно бришење на плаќање',
      'restored_payment': 'Успешно повратување на плаќање',
      'archived_payments': 'Успешно архивирани :count плаќања',
      'deleted_payments': 'Успешно бришење на :count плаќања',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Понуда',
      'quotes': 'Понуди',
      'new_quote': 'Нова понуда',
      'created_quote': 'Успешно креирана понуда',
      'updated_quote': 'Успешно ажурирана понуда',
      'archived_quote': 'Успешно архивирана понуда',
      'deleted_quote': 'Успешно избришана понуда',
      'restored_quote': 'Успешно повратување на понуда',
      'archived_quotes': 'Успешно архивирани :count понуди',
      'deleted_quotes': 'Успешно избришани :count понуди',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Трошок',
      'expenses': 'Трошоци',
      'vendor': 'Продавач',
      'vendors': 'Продавачи',
      'task': 'Задача',
      'tasks': 'Задачи',
      'project': 'Проект',
      'projects': 'Проекти',
      'activity_1': ':user го креираше клиентот :client',
      'activity_2': ':user го архивираше клиентот :client',
      'activity_3': ':user го избриша клиентот :client',
      'activity_4': ':user ја креираше фактурата :invoice',
      'activity_5': ':user ја ажурираше фактурата :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user ја архивира фактурата :invoice',
      'activity_9': ':user ја избриша фактурата :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user го ажурира плаќањето :payment',
      'activity_12': ':user го архивира плаќањето :payment',
      'activity_13': ':user го избриша плаќањето :payment',
      'activity_14': ':user внесе :credit кредит',
      'activity_15': ':user ажурира :credit кредит',
      'activity_16': ':user архивира :credit кредит',
      'activity_17': ':user избриша :credit кредит',
      'activity_18': ':user ја креира понудата :quote',
      'activity_19': ':user ја ажурира понудата :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact ја виде понудата :quote',
      'activity_22': ':user ја архивира понудата :quote',
      'activity_23': ':user ја избриша понудата :quote',
      'activity_24': ':user ја поврати понудата :quote',
      'activity_25': ':user ја поврати фактурата :invoice',
      'activity_26': ':user го поврати клиентот :client',
      'activity_27': ':user го поврати плаќањето :payment',
      'activity_28': ':user го поврати :credit кредитот',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user го креира продавачот :vendor',
      'activity_31': ':user го архивира продавачот :vendor',
      'activity_32': ':user го избриша продавачот :vendor',
      'activity_33': ':user го поврати продавачот :vendor',
      'activity_34': ':user го креира трошокот :expense',
      'activity_35': ':user го архивира трошокот :expense',
      'activity_36': ':user го избриша трошокот :expense',
      'activity_37': ':user го поврати трошокот :expense',
      'activity_39': ':user го откажа :payment_amount плаќањето :payment',
      'activity_40':
          ':user го рефундира :adjustment на :payment_amount плаќање :payment',
      'activity_41': ':payment_amount плаќање (:payment) е неуспешно',
      'activity_42': ':user ја креира задачата :task',
      'activity_43': ':user ажурира задачата :task',
      'activity_44': ':user ја архивира задачата :task',
      'activity_45': ':user ја избриша задачата :task',
      'activity_46': ':user ја поврати задачата :task',
      'activity_47': ':user го ажурира трошокот :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Еднократна лозинка',
      'emailed_quote': 'Успешно пратена понуда по ел. пошта',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Истечено',
      'all': 'Сите',
      'select': 'Избери',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Подесена вредност',
      'custom_value2': 'Подесена вредност',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Бројач на фактури',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Бројач на понуди',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Состојба на клиент',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Тип',
      'invoice_amount': 'Износ на фактура',
      'invoice_due_date': 'Датум на достасување',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Автоматска наплата',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Име на данок',
      'tax_amount': 'Износ на данок',
      'tax_paid': 'Платен данок',
      'payment_amount': 'Износ на плаќање',
      'age': 'Возраст',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Банка',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Категорија на трошок',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'nb_NO': {
      'import_data': 'Import Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Kontakts fornavn',
      'contact_last_name': 'Etternavn',
      'order': 'Order',
      'unassigned': 'Ikke tilordnet',
      'partial_value': 'Må være større enn null og mindre enn totalen',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Aktiver',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Sett subdomenet eller vis fakturaen på ditt eget nettsted.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Sikkerhet',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Abonnement',
      'subscriptions': 'Abonnementer',
      'new_subscription': 'Nytt Abonnement ',
      'edit_subscription': 'Rediger Abonnement',
      'created_subscription': 'Abonnement opprettet',
      'updated_subscription': 'Oppdaterte abonnement',
      'archived_subscription': 'Abonnement arkivert',
      'deleted_subscription': 'Abonnement Slettet',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Underdomene ikke tilgjengelig',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Send invitasjon på nytt',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Aktiverte To-faktor-autentisering',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'To-faktor-autentisering',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Venligst bekreft din epost adresse',
      'refunded_payment': 'Refundert betaling',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'For og oppdatere kjør',
      'convert_to_invoice': 'Konverter til en faktura',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Fakturer Oppgave',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Standard-dokumenter',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Skjul',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Advarsel: en ment for bruk på lokal installasjon, passord er ikke sikkert. Klikk her for og vite mer.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolonne',
      'sample': 'Eksempel',
      'map_to': 'Map To',
      'import': 'Importer',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Vennligst velg en fil',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Velg CSV-fil',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook-URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Ubetalt',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Totalbeløp',
      'quote_total': 'Tilbud totalt',
      'credit_total': 'Total kreditt',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Advarsel',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Kundenavn',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Utgiftskategorier',
      'new_expense_category': 'Ny Utgiftskategori',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Utgiftskategori ble opprettet',
      'updated_expense_category': 'Oppdaterte utgiftskategori',
      'archived_expense_category': 'Utgiftskategori ble arkivert',
      'deleted_expense_category': 'Slettet kategori',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Utgiftskategori ble gjenopprettet',
      'archived_expense_categories': ':count utgiftskategorier ble arkivert',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Sett Aktiv',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Gjentakende Faktura',
      'recurring_invoices': 'Gjentakende Fakturaer',
      'new_recurring_invoice': 'Ny Gjentakende Faktura',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Suksessfullt arkivert gjentakende faktura',
      'deleted_recurring_invoice': 'Suksessfullt slettet gjentakende faktura',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice':
          'Suksessfullt gjenopprettet gjentakende faktura',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Fortjeneste',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logger',
      'view_portal': 'Vis Portal',
      'copy_link': 'Kopier Lenke',
      'token_billing': 'Lagre kortdetaljer',
      'welcome_to_invoice_ninja': 'Velkommen til Invoice Ninja',
      'always': 'Alltid',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Kundenummer',
      'auto_convert': 'Auto Convert',
      'company_name': 'Firmanavn',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'E-postfakturaer sendt',
      'emailed_quotes': 'Tilbud sendt som e-post',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'timer',
      'statement': 'Erklæring',
      'taxes': 'Skatter',
      'surcharge': 'Tilleggsgebyr',
      'apply_payment': 'Apply Payment',
      'apply': 'Bruk',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Til',
      'health_check': 'Health Check',
      'payment_type_id': 'Betalingsmetode',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Forestående Fakturaer',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nylige Betalinger',
      'upcoming_quotes': 'Oppkommende Tilbud',
      'expired_quotes': 'Utløpte Tilbud',
      'create_client': 'Create Client',
      'create_invoice': 'Opprett faktura',
      'create_quote': 'Lag tilbud',
      'create_payment': 'Create Payment',
      'create_vendor': 'Opprett leverandør',
      'update_quote': 'Update Quote',
      'delete_quote': 'Slett tilbud',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Slett faktura',
      'update_client': 'Update Client',
      'delete_client': 'Slett kunde',
      'delete_payment': 'Slett betaling',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Slett Leverandør',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Opprett Oppgave',
      'update_task': 'Update Task',
      'delete_task': 'Slett Oppgave',
      'approve_quote': 'Approve Quote',
      'off': 'Av',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Gratis',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API-tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Rediger Token',
      'created_token': 'Opprettet token',
      'updated_token': 'Oppdaterte token',
      'archived_token': 'Suksessfullt arkivert token',
      'deleted_token': 'Suksessfullt slettet token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'E-postfaktura',
      'email_quote': 'Send tilbudet som E-post',
      'email_credit': 'Email Credit',
      'email_payment': 'E-postbetaling',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Kontakt navn',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kreditbeløp',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Ekslusiv',
      'inclusive': 'Inklusiv',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refunder betaling',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Fullt Navn',
      'city_state_postal': 'By/Fylke/Postnummer',
      'postal_city_state': 'Postnr./Sted/Fylke',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Fjern data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Advarsel: Dette sletter alle dine data permanent, og kan ikke gjennopprettes.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Dager',
      'age_group_30': '30 - 60 Dager',
      'age_group_60': '60 - 90 Dager',
      'age_group_90': '90 - 120 Dager',
      'age_group_120': 'Mer enn 120 dager',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Fakturadetaljer',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count faktura sendt',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'aktiver lisens',
      'cancel_account': 'Kanseler Konto',
      'cancel_account_message':
          'Advarsel: Dette vil permanent slette kontoen din, du kan ikke angre.',
      'delete_company': 'Slett Firma',
      'delete_company_message':
          'Advarsel: Dette vil permanent slette ditt firma, dette kan ikke gjennopprettes.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Header',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Forslag',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Gjentakende Utgifter',
      'account_management': 'Kontoadministrasjon',
      'credit_date': 'Kreditdato',
      'credit': 'Kredit',
      'credits': 'Krediter',
      'new_credit': 'Oppgi Kredit',
      'edit_credit': 'Rediger Kredit',
      'created_credit': 'Kredit opprettet',
      'updated_credit': 'Kredit oppdatert',
      'archived_credit': 'Kredit arkivert',
      'deleted_credit': 'Kredit slettet',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Suksessfullt gjenopprettet kredit',
      'archived_credits': 'Arkiverte :count krediter',
      'deleted_credits': 'Slettet :count krediter',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Nåværende versjon',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Lær mer',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nytt Firma',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Nullstill',
      'number': 'Number',
      'export': 'Eksporter',
      'chart': 'Diagram',
      'count': 'Count',
      'totals': 'Totaler',
      'blank': 'Tom',
      'day': 'Dag',
      'month': 'Måned',
      'year': 'År',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Grupper etter',
      'credit_balance': 'Kreditsaldo',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Fult navn',
      'contact_phone': 'Kontakt Telefon',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Leverings adresse',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Leverings adresse postnummer',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Fakturaadresse',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Fakturering By',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Fakturering Postnummer',
      'billing_country': 'Fakturering Land',
      'client_id': 'Kunde-ID',
      'assigned_to': 'Assigned to',
      'created_by': 'Laget av :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Kolonner',
      'aging': 'Aging',
      'profit_and_loss': 'Fortjeneste og Tap',
      'reports': 'Rapporter',
      'report': 'Rapport',
      'add_company': 'Legg til Firma',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Hjelp',
      'refund': 'Refunder',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Kontakt Epost',
      'multiselect': 'Multiselect',
      'entity_state': 'Tilstand',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Beskjed',
      'from': 'Fra',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Dokumentasjon',
      'contact_us': 'Kontakt Oss',
      'subtotal': 'Totalbeløp',
      'line_total': 'Sum',
      'item': 'Produkt',
      'credit_email': 'Credit Email',
      'iframe_url': 'Nettside',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'User',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Vennligst velg en klient',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Skatteinnstillinger',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Valg',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Send',
      'recover_password': 'Gjenopprett ditt passord',
      'late_fees': 'Late Fees',
      'credit_number': 'Kreditnummer',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Planlegg',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dager',
      'invoice_email': 'Faktura-e-post',
      'payment_email': 'Betalings-e-post',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Tilbuds-e-post',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Brukerhåndtering',
      'users': 'Brukere',
      'new_user': 'Ny Bruker',
      'edit_user': 'Endre bruker',
      'created_user': 'Successfully created user',
      'updated_user': 'Bruker oppdatert',
      'archived_user': 'Suksessfullt arkivert bruker',
      'deleted_user': 'Bruker slettet',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Suksessfullt gjenopprettet bruker',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Systeminnstillinger',
      'invoice_options': 'Faktura alternativer',
      'hide_paid_to_date': 'Skjul delbetalinger',
      'hide_paid_to_date_help':
          'Bare vis delbetalinger om det har forekommet en delbetaling.',
      'invoice_embed_documents': 'Embed Dokumenter',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show header on',
      'all_pages_footer': 'Show footer on',
      'first_page': 'Første side',
      'all_pages': 'Alle sider',
      'last_page': 'Siste side',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primærfarge',
      'secondary_color': 'Sekundær farge',
      'page_size': 'Page Size',
      'font_size': 'Skriftstørrelse',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Faktura felt',
      'product_fields': 'Produktfelter',
      'invoice_terms': 'Vilkår for fakturaen',
      'invoice_footer': 'Faktura Bunntekst',
      'quote_terms': 'Tilbuds Vilkår',
      'quote_footer': 'Tilbud Bunntekst',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Konverter',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daglig',
      'freq_weekly': 'Ukentlig',
      'freq_two_weeks': 'To uker',
      'freq_four_weeks': 'Fire uker',
      'freq_monthly': 'Månedlig',
      'freq_two_months': 'To måneder',
      'freq_three_months': 'Tre måneder',
      'freq_four_months': 'Fire måneder',
      'freq_six_months': 'Seks måneder',
      'freq_annually': 'Årlig',
      'freq_two_years': 'To år',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Company',
      'generated_numbers': 'Genererte Nummere',
      'charge_taxes': 'Inkluder skatt',
      'next_reset': 'Neste Nullstilling',
      'reset_counter': 'Nullstill Teller',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefiks',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Egendefinert CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Faktura-signatur',
      'require_invoice_signature_help': 'Krever klients signatur.',
      'require_quote_signature': 'Tilbuds-signatur',
      'enable_portal_password': 'Passord-beskytt fakturaer',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Autorisasjon',
      'subdomain': 'Subdomene',
      'domain': 'Domene',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Med vennlig hilsen,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Svar til Epost',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC E-post',
      'processed': 'Processed',
      'credit_card': 'Betalingskort',
      'bank_transfer': 'Bankoverføring',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Aktiver min',
      'enable_max': 'Aktiver maks',
      'min_limit': 'Min: :min',
      'max_limit': 'Maks: :max',
      'min': 'Min',
      'max': 'Maks',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Oppdater Adresse',
      'update_address_help': 'Oppdater kundens adresse med oppgitte detaljer',
      'rate': 'Sats',
      'tax_rate': 'Skattesats',
      'new_tax_rate': 'Ny Skattesats',
      'edit_tax_rate': 'Rediger skattesats',
      'created_tax_rate': 'Suksessfullt opprettet skattesats',
      'updated_tax_rate': 'Suksessfullt oppdatert skattesats',
      'archived_tax_rate': 'Suksessfullt arkivert skattesatsen',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Automatisk-utfyll produkter',
      'fill_products_help':
          'Valg av produkt vil automatisk fylle ut beskrivelse og kostnaden',
      'update_products': 'Automatisk oppdater produkter',
      'update_products_help':
          'Å endre en faktura vil automatisk oppdatere produktbilioteket',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Avgifter',
      'limits': 'Begrensninger',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Søndag',
      'monday': 'Mandag',
      'tuesday': 'Tirsdag',
      'wednesday': 'Onsdag',
      'thursday': 'Torsdag',
      'friday': 'Fredag',
      'saturday': 'Lørdag',
      'january': 'Januar',
      'february': 'Februar',
      'march': 'Mars',
      'april': 'April',
      'may': 'Mai',
      'june': 'Juni',
      'july': 'Juli',
      'august': 'August',
      'september': 'September',
      'october': 'Oktober',
      'november': 'November',
      'december': 'Desember',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Timers Tid',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Produkt-innstillinger',
      'device_settings': 'Device Settings',
      'defaults': 'Standarder',
      'basic_settings': 'Grunnleggende Innstillinger',
      'advanced_settings': 'Avanserte innstillinger',
      'company_details': 'Firmainformasjon',
      'user_details': 'Brukerdetaljer',
      'localization': 'Regioninnstillinger',
      'online_payments': 'Nettbetalinger',
      'tax_rates': 'Skattesatser',
      'notifications': 'Varsler',
      'import_export': 'Import | Eksport',
      'custom_fields': 'Egendefinerte felt',
      'invoice_design': 'Fakturadesign',
      'buy_now_buttons': 'Betal Nå-knapper',
      'email_settings': 'E-post-innstillinger',
      'templates_and_reminders': 'Design & Påminnelser',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Datavisualiseringer',
      'price': 'Pris',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'vilkår for bruk',
      'privacy_policy': 'Personvernregler',
      'sign_up': 'Registrer deg',
      'account_login': 'Kontoinnlogging',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Lag ny',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Last ned',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Dokument',
      'documents': 'Dokumenter',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Utgiftsdato',
      'pending': 'Avventer',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Konvertert',
      'add_documents_to_invoice': 'Legg ved dokumenter til faktura',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Merk som betalt',
      'category': 'Kategori',
      'address': 'Adresse',
      'new_vendor': 'Ny Leverandør',
      'created_vendor': 'Opprettet leverandør',
      'updated_vendor': 'Oppdaterte leverandør',
      'archived_vendor': 'Arkiverte leverandør',
      'deleted_vendor': 'Slettet leverandør',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Arkiverte :count leverandører',
      'deleted_vendors': 'Slettet :count leverandører',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Angi utgift',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Successfully deleted expense',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Arkiverte utgifter',
      'deleted_expenses': 'Slettet utgifter',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Fakturert',
      'logged': 'Logget',
      'running': 'Løpende',
      'resume': 'Gjenoppta',
      'task_errors': 'Vennligst rett alle overlappende tider',
      'start': 'Start',
      'stop': 'Stopp',
      'started_task': 'Successfully started task',
      'stopped_task': 'Suksessfullt stoppet oppgave',
      'resumed_task': 'Successfully resumed task',
      'now': 'Nå',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Tidtaker',
      'manual': 'Manuell',
      'budgeted': 'Budgeted',
      'start_time': 'Starttid',
      'end_time': 'Sluttid',
      'date': 'Dato',
      'times': 'Tider',
      'duration': 'Varighet',
      'new_task': 'Ny Oppgave',
      'created_task': 'Suksessfullt opprettet oppgave',
      'updated_task': 'Suksessfullt oppdatert oppgave',
      'archived_task': 'Arkiverte oppgave',
      'deleted_task': 'Slettet oppgave',
      'restored_task': 'Gjenopprettet oppgave',
      'archived_tasks': 'Arkiverte :count oppgaver',
      'deleted_tasks': 'Slettet :count oppgaver',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Opprettet prosjekt',
      'updated_project': 'Oppdaterte prosjekt',
      'archived_project': 'Arkiverte prosjekt',
      'deleted_project': 'Slettet prosjekt',
      'restored_project': 'Gjenopprettet prosjekt',
      'archived_projects': 'Arkiverte :count prosjekter',
      'deleted_projects': 'Slettet :count prosjekter',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nytt Prosjekt',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'klikk her',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Låst',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Bunntekst',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Tilpass Utvalg',
      'date_range': 'Datoperiode',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Denne måneden',
      'last_month': 'Siste måned',
      'this_year': 'Dette Året',
      'last_year': 'Siste år',
      'custom': 'Egendefiner',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Se faktura',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Rediger Kunde',
      'edit_product': 'Endre produkt',
      'edit_invoice': 'Rediger faktura',
      'edit_quote': 'Endre tilbud',
      'edit_payment': 'Rediger Betaling',
      'edit_task': 'Rediger Oppgave',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Rediger Leverandør',
      'edit_project': 'Rediger Prosjekt',
      'edit_recurring_expense': 'Rediger Gjentakende Utgift',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Fakturerings Adresse',
      'shipping_address': 'Leveringsadresse',
      'total_revenue': 'Sum omsetning',
      'average_invoice': 'Gjennomsnittlige fakturaer',
      'outstanding': 'Utestående',
      'invoices_sent': ':count fakturaer sendt',
      'active_clients': 'aktive kunder',
      'close': 'Lukk',
      'email': 'E-post',
      'password': 'Passord',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Navn',
      'logout': 'Logg ut',
      'login': 'Logg inn',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Søk',
      'active': 'Aktiv',
      'archived': 'Arkivert',
      'deleted': 'Slettet',
      'dashboard': 'Skrivebord',
      'archive': 'Arkiv',
      'delete': 'Slett',
      'restore': 'Gjenopprette',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Stigende',
      'descending': 'Synkende',
      'save': 'Lagre',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Betalt til Dato',
      'balance_due': 'Gjenstående',
      'balance': 'Balanse',
      'overview': 'Overview',
      'details': 'Detaljer',
      'phone': 'Telefon',
      'website': 'Nettside',
      'vat_number': 'MVA-nummer',
      'id_number': 'Id nummer',
      'create': 'Lag',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakter',
      'additional': 'Additional',
      'first_name': 'Fornavn',
      'last_name': 'Etternavn',
      'add_contact': 'Legg til kontakt',
      'are_you_sure': 'Er du sikker?',
      'cancel': 'Avbryt',
      'ok': 'Ok',
      'remove': 'Fjern',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produkter',
      'new_product': 'Nytt Produkt',
      'created_product': 'Produkt lagret',
      'updated_product': 'Produkt oppdatert',
      'archived_product': 'Produkt arkivert',
      'deleted_product': 'Slettet produkt',
      'restored_product': 'Gjenopprettet produkt',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Slettet :count produkter',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produkt',
      'notes': 'Notater',
      'cost': 'Kostnad',
      'client': 'Kunde',
      'clients': 'Kunder',
      'new_client': 'Ny Kunde',
      'created_client': 'Opprettet kunde',
      'updated_client': 'Oppdaterte kunde',
      'archived_client': 'Arkiverte kunde',
      'archived_clients': 'Arkiverte :count kunder',
      'deleted_client': 'Slettet kunde',
      'deleted_clients': 'Slettet :count kunder',
      'restored_client': 'Gjenopprettet kunde',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Gate',
      'address2': 'Husnummer',
      'city': 'By',
      'state': 'Fylke',
      'postal_code': 'Postnummer',
      'country': 'Country',
      'invoice': 'Faktura',
      'invoices': 'Fakturaer',
      'new_invoice': 'Ny faktura',
      'created_invoice': 'Faktura opprettet',
      'updated_invoice': 'Faktura oppdatert',
      'archived_invoice': 'Faktura arkivert',
      'deleted_invoice': 'Faktura slettet',
      'restored_invoice': 'Suksessfullt gjenopprettet faktura',
      'archived_invoices': 'Fakturaer arkivert',
      'deleted_invoices': 'Slettet :count fakturaer',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'E-postfaktura sendt',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Beløp',
      'invoice_number': 'Fakturanummer',
      'invoice_date': 'Faktureringsdato',
      'discount': 'Rabatter:',
      'po_number': 'Ordrenummer',
      'terms': 'Vilkår',
      'public_notes': 'Offentlig notater',
      'private_notes': 'Private notater',
      'frequency': 'Frekvens',
      'start_date': 'Startdato',
      'end_date': 'Sluttdato',
      'quote_number': 'Tilbudsnummer',
      'quote_date': 'Tilbudsdato',
      'valid_until': 'Gyldig til',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Beskrivelse',
      'unit_cost': 'Stykkpris',
      'quantity': 'Antall',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefon (arbeid)',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Forfallsdato',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Totalt',
      'percent': 'Prosent',
      'edit': 'Endre',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Oppgavesats',
      'settings': 'Innstillinger',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Dato Opprettet',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Skatt',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Forfalt',
      'draft': 'Kladd',
      'sent': 'Sendt',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Delvis/Depositum',
      'paid': 'Betalt',
      'mark_sent': 'Merk som Sendt',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Ferdig',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Mørk Modus',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Aktivitet',
      'no_records_found': 'No records found',
      'clone': 'Kopier',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Betalingsvilkår',
      'payment_date': 'Betalingsdato',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Kundeportal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Aktivert',
      'recipients': 'Mottakere',
      'initial_email': 'Første E-post',
      'first_reminder': 'Første Påminnelse',
      'second_reminder': 'Andre Påminnelse',
      'third_reminder': 'Tredje Påminnelse',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Mal',
      'send': 'Send',
      'subject': 'Emne',
      'body': 'Body',
      'send_email': 'Send e-post',
      'email_receipt': 'Send betalingskvittering som e-post til kunden',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Tilpass',
      'history': 'Historie',
      'payment': 'Betaling',
      'payments': 'Betalinger',
      'refunded': 'Refunded',
      'payment_type': 'Betalingsmetode',
      'transaction_reference': 'Transaksjonsreferanse',
      'enter_payment': 'Oppgi betaling',
      'new_payment': 'Oppgi Betaling',
      'created_payment': 'Betaling opprettet',
      'updated_payment': 'Suksessfullt oppdatert betaling',
      'archived_payment': 'Betaling arkivert',
      'deleted_payment': 'Betaling slettet',
      'restored_payment': 'Suksessfullt gjenopprettet betaling',
      'archived_payments': 'Arkiverte :count betalinger',
      'deleted_payments': 'Slettet :count betalinger',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Pristilbud',
      'quotes': 'Pristilbud',
      'new_quote': 'Nytt tilbud',
      'created_quote': 'Tilbud opprettet',
      'updated_quote': 'Tilbud oppdatert',
      'archived_quote': 'Tilbud arkivert',
      'deleted_quote': 'Tilbud slettet',
      'restored_quote': 'Suksessfullt gjenopprettet tilbud',
      'archived_quotes': 'Arkiverte :count tilbud',
      'deleted_quotes': 'Slettet :count tilbud',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Utgift',
      'expenses': 'Utgifter',
      'vendor': 'Leverandør',
      'vendors': 'Leverandører',
      'task': 'Oppgave',
      'tasks': 'Oppgaver',
      'project': 'Prosjekt',
      'projects': 'Prosjekter',
      'activity_1': ':user opprettet kunde :client',
      'activity_2': ':user arkiverte kunde :client',
      'activity_3': ':user slettet kunde :client',
      'activity_4': ':user opprettet faktura :invoice',
      'activity_5': ':user oppdaterte faktura :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user arkiverte faktura :invoice',
      'activity_9': ':user slettet faktura :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user oppdaterte betaling :payment',
      'activity_12': ':user arkiverte betaling :payment',
      'activity_13': ':user slettet betaling :payment',
      'activity_14': ':user la inn :credit kredit',
      'activity_15': ':user oppdaterte :credit kredit',
      'activity_16': ':user arkiverte :credit kredit',
      'activity_17': ':user slettet :credit kredit',
      'activity_18': ':user opprettet tilbud :quote',
      'activity_19': ':user oppdaterte tilbud :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viste tilbud :quote',
      'activity_22': ':user arkiverte tilbud :quote',
      'activity_23': ':user slettet tilbud :quote',
      'activity_24': ':user gjenopprettet tilbud :quote',
      'activity_25': ':user gjenopprettet faktura :invoice',
      'activity_26': ':user gjenopprettet kunde :client',
      'activity_27': ':user gjenopprettet betaling :payment',
      'activity_28': ':user gjenopprettet :credit kredit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user opprettet leverandør :vendor',
      'activity_31': ':user arkiverte leverandør :vendor',
      'activity_32': ':user slettet leverandør :vendor',
      'activity_33': ':user gjenopprettet leverandør :vendor',
      'activity_34': ':user opprettet utgift :expense',
      'activity_35': ':user arkiverte utgift :expense',
      'activity_36': ':user slettet utgift :expense',
      'activity_37': ':user gjenopprettet utgift :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user opprettet oppgave :task',
      'activity_43': ':user oppdaterte oppgave :task',
      'activity_44': ':user arkiverte oppgave :task',
      'activity_45': ':user slettet oppgave :task',
      'activity_46': ':user gjenopprettet oppgave :task',
      'activity_47': ':user oppdaterte utgift :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Engangs Passord',
      'emailed_quote': 'Tilbud sendt som e-post',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'Alle',
      'select': 'Velg',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Fakturanummer-teller',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Tilbudsnummer-teller',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Type',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Forfallsdato',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Fakturer',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Skattenavn',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Beløp',
      'age': 'Alder',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Utgiftskategori',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'pl': {
      'import_data': 'Importuj dane',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Nieprzypisano',
      'partial_value': 'Must be greater than zero and less than the total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Aktywuj',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Ustaw subdomenę lub wyświetl fakturę na swojej stronie.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Bezpieczeństwo',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Ponów zaproszenie',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Zwrócono płatność',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'Ten kwartał',
      'last_quarter': 'Poprzedni kwartał',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Konwertuj do faktury',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Fakturuj zadanie',
      'invoice_expense': 'Faktura na wydatek',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Kwota przeliczona',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Domyślne dokumenty',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Ukryj',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolumna',
      'sample': 'Przykład',
      'map_to': 'Map To',
      'import': 'Importuj',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Wybierz plik',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Plik CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Nie zapłacono',
      'white_label': 'White Label',
      'delivery_note': 'Dowód dostawy',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Zaliczka',
      'invoice_total': 'Faktura ogółem',
      'quote_total': 'Suma oferty',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Ostrzeżenie',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'Kod CVV',
      'client_name': 'Nazwa klienta',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Kategorie wydatków',
      'new_expense_category': 'Nowa kategoria wydatków',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Kategoria wydatków została utworzona',
      'updated_expense_category': 'Kategoria wydatków została zaktualizowana',
      'archived_expense_category': 'Kategoria wydatków została zarchiwizowana',
      'deleted_expense_category': 'Usunięto kategorię',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Przywrócono kategorię wydatków',
      'archived_expense_categories': 'Zarchiwizowana :count kategorii wydatków',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Utwórz fakturę',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Oznacz jako aktywną',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Odnawialna faktura',
      'recurring_invoices': 'Faktury odnawialne',
      'new_recurring_invoice': 'Nowa faktura odnawialna',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Odnawialna faktura została zarchiwizowana',
      'deleted_recurring_invoice': 'Odnawialna faktura została usunięta.',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Odnawialna faktura została przywrócona',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Zysk',
      'line_item': 'Element na liście',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Zobacz portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Zapisz dane karty',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Zawsze',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Numer klienta',
      'auto_convert': 'Auto Convert',
      'company_name': 'Nazwa firmy',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Wysyłka maili powiodła się',
      'emailed_quotes': 'Wysyłka ofert powiodła się',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Dostawca płatności',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Godziny',
      'statement': 'Wyciąg',
      'taxes': 'Podatki',
      'surcharge': 'Dopłata',
      'apply_payment': 'Apply Payment',
      'apply': 'Zastosuj',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Do',
      'health_check': 'Health Check',
      'payment_type_id': 'Typ płatności',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Zakończone',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Nadchodzące faktury',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Ostatnie płatności',
      'upcoming_quotes': 'Nadchodzące oferty',
      'expired_quotes': 'Wygasłe oferty',
      'create_client': 'Create Client',
      'create_invoice': 'Utwórz Fakturę',
      'create_quote': 'Stwórz ofertę',
      'create_payment': 'Create Payment',
      'create_vendor': 'Utwórz dostawcę',
      'update_quote': 'Update Quote',
      'delete_quote': 'Usuń ofertę',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Usuń fakturę',
      'update_client': 'Update Client',
      'delete_client': 'Usuń klienta',
      'delete_payment': 'Usuń płatność',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Usuń dostawcę',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Usuń wydatek',
      'create_task': 'Stwórz zadanie',
      'update_task': 'Update Task',
      'delete_task': 'Usuń zadanie',
      'approve_quote': 'Approve Quote',
      'off': 'Wyłaczono',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Darmowe',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'Tokeny API',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokeny',
      'new_token': 'New Token',
      'edit_token': 'Edytuj token',
      'created_token': 'Token został utworzony',
      'updated_token': 'Token został zaktualizowany',
      'archived_token': 'Token został zarchiwizowany',
      'deleted_token': 'Token został usunięty',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Wyślij fakturę',
      'email_quote': 'Wyślij ofertę',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'Wyświetl PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Nazwa kontaktu',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edytuj warunki płatności',
      'created_payment_term': 'Utworzono termin płatności',
      'updated_payment_term': 'Zaktualizowano termin płatności',
      'archived_payment_term': 'Zarchiwizowano termin płatności',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Zaloguj się przez email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kwota kredytu',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Doliczanie do kwoty',
      'inclusive': 'Wliczanie w kwotę',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Szukaj w firmie',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Zwrot płatności',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Pełna nazwa',
      'city_state_postal': 'Miasto/województwo/kod pocztowy',
      'postal_city_state': 'Kod pocztowy/Miasto/Województwo',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 dni',
      'age_group_30': '30 - 60 dni',
      'age_group_60': '60 - 90 dni',
      'age_group_90': '90 - 120 dni',
      'age_group_120': 'ponad 120 dni',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Szczegóły faktury',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Uprawnienia',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count wysłana faktura',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Zastosuj licencję',
      'cancel_account': 'Anuluj konto',
      'cancel_account_message':
          'Ostrzeżenie: Nie można cofnąć tej operacji, wszystkie twoje dane zostaną usunięte.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Nagłówek',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Propozycje',
      'tickets': 'Tickets',
      'recurring_quotes': 'Powtarzalne wyceny',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Zarządzanie kontem',
      'credit_date': 'Data kredytu',
      'credit': 'Kredyt',
      'credits': 'Kredyty',
      'new_credit': 'Wprowadź kredyt',
      'edit_credit': 'Edytuj kredyt',
      'created_credit': 'Kredyt został utworzony',
      'updated_credit': 'Zaktualizowano kredyt',
      'archived_credit': 'Kredyt zarchiwizowano',
      'deleted_credit': 'Kredyt został usunięty',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Kredyt został przywrócony',
      'archived_credits': 'Zarchiwizowano :count kredyty/kredytów',
      'deleted_credits': 'Usunięto :count kredyty/kredytów',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Aktualna wersja',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Więcej informacji',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nowa firma',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reset',
      'number': 'Number',
      'export': 'Eksport',
      'chart': 'Wykres',
      'count': 'Count',
      'totals': 'Suma',
      'blank': 'Puste',
      'day': 'Dzień',
      'month': 'Miesiąc',
      'year': 'Rok',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Grupuj według',
      'credit_balance': 'Saldo kredytowe',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Numer telefonu kontaktu',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Numer klienta',
      'assigned_to': 'Assigned to',
      'created_by': 'Utworzono przez :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Kolumny',
      'aging': 'Odkładanie',
      'profit_and_loss': 'Zysk i strata',
      'reports': 'Raporty',
      'report': 'Raport',
      'add_company': 'Dodaj firmę',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Pomoc',
      'refund': 'Zwrot',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Email kontaktowy',
      'multiselect': 'Multiselect',
      'entity_state': 'Stan',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Wiadomość',
      'from': 'Od',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'forum wsparcia',
      'about': 'About',
      'documentation': 'Dokumentacja',
      'contact_us': 'Skontaktuj się z nami',
      'subtotal': 'Suma wartości netto',
      'line_total': 'Wartość',
      'item': 'Pozycja',
      'credit_email': 'Credit Email',
      'iframe_url': 'Strona internetowa',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Tak',
      'no': 'Nie',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Podgląd',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Użytkownik',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Wybierz klienta',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Ustawienia podatków',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Zmień',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Wyślij',
      'recover_password': 'Odzyskaj swoje hasło',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Zaplanuj',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dni',
      'invoice_email': 'Email faktury',
      'payment_email': 'Email płatności',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Email oferty',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Zezwól użytkownikowi na zarządzanie użytkownikami, edytowanie ustawień oraz wszystkich danych.',
      'user_management': 'Zarządzanie użytkownikami',
      'users': 'Użytkownicy',
      'new_user': 'Nowy użytkownik',
      'edit_user': 'Edytuj użytkownika',
      'created_user': 'Successfully created user',
      'updated_user': 'Użytkownik został zaktualizowany',
      'archived_user': 'Użytkownik został zarchiwizowany',
      'deleted_user': 'Użytkownik został usunięty',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Użytkownik został przywrócony',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Ustawienia ogólne',
      'invoice_options': 'Opcje faktury',
      'hide_paid_to_date': 'Ukryj pole \"Zapłacono dotychczas\"',
      'hide_paid_to_date_help':
          'Wyświetlaj \"Zapłacono dotychczas\" tylko przy tych fakturach, do których otrzymano płatność.',
      'invoice_embed_documents': 'Załączniki',
      'invoice_embed_documents_help': 'Wstaw do faktury załączniki graficzne.',
      'all_pages_header': 'Pokaż nagłówek na',
      'all_pages_footer': 'Pokaż stopkę na',
      'first_page': 'Pierwsza strona',
      'all_pages': 'Wszystkie strony',
      'last_page': 'Ostatnia strona',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Główny kolor',
      'secondary_color': 'Dodatkowy kolor',
      'page_size': 'Rozmiar strony',
      'font_size': 'Rozmiar fonta',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Pola faktury',
      'product_fields': 'Pola produktów',
      'invoice_terms': 'Warunki do faktury',
      'invoice_footer': 'Stopka faktury',
      'quote_terms': 'Warunki oferty',
      'quote_footer': 'Stopka oferty',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Automatycznie konwertuj',
      'auto_convert_quote_help':
          'Utwórz automatycznie fakturę z oferty zaakceptowanej przez klienta.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Codziennie',
      'freq_weekly': 'Co tydzień',
      'freq_two_weeks': 'Co dwa tygodnie',
      'freq_four_weeks': 'Co cztery tygodnie',
      'freq_monthly': 'Co miesiąc',
      'freq_two_months': 'Dwa miesiące',
      'freq_three_months': 'Co trzy miesiące',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Co sześć miesięcy',
      'freq_annually': 'Co rok',
      'freq_two_years': 'Dwa lata',
      'freq_three_years': 'Three Years',
      'never': 'Nigdy',
      'company': 'Company',
      'generated_numbers': 'Wygenerowane numery',
      'charge_taxes': 'Obciąż podatkami',
      'next_reset': 'Następny reset',
      'reset_counter': 'Zresetuj licznik',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefiks',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Własny CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Przycisk wyboru do warunków faktury',
      'show_accept_invoice_terms_help':
          'Wymagaj od klienta potwierdzenia, że akceptuje warunki faktury.',
      'show_accept_quote_terms': 'Przycisk wyboru do warunków oferty',
      'show_accept_quote_terms_help':
          'Wymagaj od klienta potwierdzenia, że akceptuje warunki oferty.',
      'require_invoice_signature': 'Podpis na fakurze',
      'require_invoice_signature_help': 'Wymagaj od klienta podpisania faktury',
      'require_quote_signature': 'Podpis na ofercie',
      'enable_portal_password': 'Faktury chronione hasłem',
      'enable_portal_password_help':
          'Zezwala na utworzenie haseł dla każdego kontaktu. Jeśli hasło zostanie ustanowione, użytkownik będzie musiał podać hasło, aby przeglądać faktury.',
      'authorization': 'Autoryzacja',
      'subdomain': 'Subdomena',
      'domain': 'Domena',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Z wyrazami szacunku,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Zwykły',
      'light': 'Jasny',
      'dark': 'Ciemny',
      'email_design': 'Motyw email',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Aktywuj Markup',
      'reply_to_email': 'Odpowiedz do:',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'UDW Email',
      'processed': 'Processed',
      'credit_card': 'Karta Kredytowa',
      'bank_transfer': 'Przelew bankowy',
      'priority': 'Priorytet',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Aktywuj min',
      'enable_max': 'Aktywuj max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Aktualizuj adres',
      'update_address_help':
          'Zaktualizuj dane adresowe klienta na podstawie dostarczonych informacji',
      'rate': 'Stawka',
      'tax_rate': 'Stawka podatkowa',
      'new_tax_rate': 'Nowa stawka podatkowa',
      'edit_tax_rate': 'Edytuj stawkę podatkową',
      'created_tax_rate': 'Successfully created tax rate',
      'updated_tax_rate': 'Successfully updated tax rate',
      'archived_tax_rate': 'Zarchiwizowano stawkę podatkową',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Automatycznie uzupełniaj produkty',
      'fill_products_help':
          'Wybieranie produktu automatycznie uzupełni opis i kwotę',
      'update_products': 'Automatycznie aktualizuj produkty',
      'update_products_help':
          'Zaktualizowanie faktury automatycznie uaktualni produkt w bibliotece produktów',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatycznie zamieniaj ceny produktu na walutę klienta',
      'fees': 'Opłaty',
      'limits': 'Limity',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Wyłączono',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Niedziela',
      'monday': 'Poniedziałek',
      'tuesday': 'Wtorek',
      'wednesday': 'Środa',
      'thursday': 'Czwartek',
      'friday': 'Piątek',
      'saturday': 'Sobota',
      'january': 'Styczeń',
      'february': 'Luty',
      'march': 'Marzec',
      'april': 'Kwiecień',
      'may': 'Maj',
      'june': 'Czerwiec',
      'july': 'Lipiec',
      'august': 'Sierpień',
      'september': 'Wrzesień',
      'october': 'Październik',
      'november': 'Listopad',
      'december': 'Grudzień',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 godzinny czas',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Grupuj',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Prześlij logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Ustawienia produktu',
      'device_settings': 'Ustawienia urządzenia',
      'defaults': 'Domyślne',
      'basic_settings': 'Ustawienia podstawowe',
      'advanced_settings': 'Ustawienia zaawansowane',
      'company_details': 'Dane firmy',
      'user_details': 'Dane użytkownika',
      'localization': 'Lokalizacja',
      'online_payments': 'Płatności online',
      'tax_rates': 'Stawki podatkowe',
      'notifications': 'Powiadomienia',
      'import_export': 'Import | Eksport danych',
      'custom_fields': 'Dostosowane pola',
      'invoice_design': 'Motyw faktury',
      'buy_now_buttons': 'Przyciski Kup Teraz',
      'email_settings': 'Ustawienia e-mail',
      'templates_and_reminders': 'Szablony i przypomnienia',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Wizualizacje danych',
      'price': 'Cena',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Warunki korzystania z Serwisu',
      'privacy_policy': 'Polityka prywatności',
      'sign_up': 'Zapisz się',
      'account_login': 'Logowanie',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Dodaj nowy/nową',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Pobierz',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Zrób zdjęcie',
      'upload_file': 'Upload File',
      'document': 'Dokument',
      'documents': 'Dokumenty',
      'new_document': 'Nowy dokument',
      'edit_document': 'Edytuj dokument',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Data obciążenia',
      'pending': 'Oczekuję',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Skonwertowano',
      'add_documents_to_invoice': 'Dodaj dokumenty do faktury',
      'exchange_rate': 'Kurs wymiany',
      'convert_currency': 'Konwertuj walutę',
      'mark_paid': 'Oznacz jako zapłaconą',
      'category': 'Kategoria',
      'address': 'Adres',
      'new_vendor': 'Nowy dostawca',
      'created_vendor': 'Dostawca został utworzony',
      'updated_vendor': 'Zaktualizowano dostawcę',
      'archived_vendor': 'Dostawca został zarchiwizowany',
      'deleted_vendor': 'Dostawca został usunięty',
      'restored_vendor': 'Dostawca został przywrócony',
      'archived_vendors': 'Zarchiwizowano :count dostawców',
      'deleted_vendors': 'Usunięto :count dostawców',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Dodaj wydatek',
      'created_expense': 'Wydatek został utworzony',
      'updated_expense': 'Wydatek został zaktualizowany',
      'archived_expense': 'Wydatki zostały zarchiwizowane',
      'deleted_expense': 'Wydatki zostały usunięte',
      'restored_expense': 'Wydatek został przywrócony',
      'archived_expenses': 'Wydatki zostały zarchiwizowane',
      'deleted_expenses': 'Wydatki zostały usunięte',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Zafakturowano',
      'logged': 'Zapisano',
      'running': 'W trakcie',
      'resume': 'Wznów',
      'task_errors': 'Proszę skoryguj nakładające się czasy',
      'start': 'Rozpocznij',
      'stop': 'Zatrzymaj',
      'started_task': 'Successfully started task',
      'stopped_task': 'Zakończono wykonywanie zadania',
      'resumed_task': 'Successfully resumed task',
      'now': 'Teraz',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Odliczanie czasu',
      'manual': 'Wprowadź ręcznie',
      'budgeted': 'Budgeted',
      'start_time': 'Czas rozpoczęcia',
      'end_time': 'Zakończono',
      'date': 'Data',
      'times': 'Razy/Okresy',
      'duration': 'Czas trwania',
      'new_task': 'Nowe zadanie',
      'created_task': 'Pomyślnie utworzono zadanie',
      'updated_task': 'Pomyślnie zaktualizowano zadanie',
      'archived_task': 'Zadania zostało zarchiwizowane',
      'deleted_task': 'Usunięto zadanie',
      'restored_task': 'Zadanie zostało przywrócone',
      'archived_tasks': 'Zarchiwizowano :count zadania/zadań',
      'deleted_tasks': 'Usunięto :count zadania/zadań',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Utworzono projekt',
      'updated_project': 'Zaktualizowano projekt',
      'archived_project': 'Zarchiwizowano projekt',
      'deleted_project': 'Usunięto projekt',
      'restored_project': 'Przywrócono projekt',
      'archived_projects': 'Zarchiwizowano :count projektów',
      'deleted_projects': 'Usunięto :count projekty/projektów',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nowy projekt',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'kliknij tutaj',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Średnia',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Stopka',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Zaloguj się przez Google',
      'today': 'Today',
      'custom_range': 'Określony okres',
      'date_range': 'Zakres czasowy',
      'current': 'Obecny',
      'previous': 'Poprzedni',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Ten miesiąc',
      'last_month': 'Ostatni miesiąc',
      'this_year': 'Ten rok',
      'last_year': 'Ostatni rok',
      'custom': 'Dostosowanie',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Zobacz fakturę',
      'convert': 'Convert',
      'more': 'Więcej',
      'edit_client': 'Edytuj klienta',
      'edit_product': 'Edytuj produkt',
      'edit_invoice': 'Edytuj fakturę',
      'edit_quote': 'Edytuj ofertę',
      'edit_payment': 'Edytuj płatność',
      'edit_task': 'Edytuj zadanie',
      'edit_expense': 'Edytuj wydatek',
      'edit_vendor': 'Edytuj dostawcę',
      'edit_project': 'Edytuj projekt',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Adres rozliczeniowy',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Całkowity dochód',
      'average_invoice': 'Średnia wartość',
      'outstanding': 'Zaległości',
      'invoices_sent': ':count wysłanych faktur',
      'active_clients': 'aktywni klienci',
      'close': 'Zamknij',
      'email': 'Email',
      'password': 'Hasło',
      'url': 'URL',
      'secret': 'Tajny',
      'name': 'Nazwa',
      'logout': 'Wyloguj się',
      'login': 'Zaloguj',
      'filter': 'Filtruj',
      'sort': 'Sortuj',
      'search': 'Szukaj',
      'active': 'Aktywny',
      'archived': 'Zarchiwizowano',
      'deleted': 'Usunięte',
      'dashboard': 'Pulpit',
      'archive': 'Archiwum',
      'delete': 'Usuń',
      'restore': 'Przywróć',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Zapisz',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Zapłacono dotychczas',
      'balance_due': 'Do zapłaty',
      'balance': 'Saldo',
      'overview': 'Podsumowanie',
      'details': 'Szczegóły',
      'phone': 'Telefon',
      'website': 'Strona internetowa',
      'vat_number': 'Numer NIP',
      'id_number': 'REGON',
      'create': 'Utwórz',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakty',
      'additional': 'Additional',
      'first_name': 'Imię',
      'last_name': 'Nazwisko',
      'add_contact': 'Dodaj kontakt',
      'are_you_sure': 'Jesteś pewny?',
      'cancel': 'Anuluj',
      'ok': 'Ok',
      'remove': 'Usuń',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produkty',
      'new_product': 'Nowy produkt',
      'created_product': 'Produkt został utworzony',
      'updated_product': 'Produkt został zaktualizowany',
      'archived_product': 'Produkt został zarchiwizowany',
      'deleted_product': 'Usunięto produkt',
      'restored_product': 'Przywrócono produkt',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Usunięto :count produktów',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produkt',
      'notes': 'Notatki',
      'cost': 'Koszt',
      'client': 'Klient',
      'clients': 'Klienci',
      'new_client': 'Nowy klient',
      'created_client': 'Klient został utworzony',
      'updated_client': 'Klient został zaktualizowany',
      'archived_client': 'Klient został zarchiwizowany',
      'archived_clients': 'Zarchiwizowano :count klientów',
      'deleted_client': 'Klient został usunięty',
      'deleted_clients': 'Usunięto :count klientów',
      'restored_client': 'Klient został przywrócony',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Ulica',
      'address2': 'Nr',
      'city': 'Miasto',
      'state': 'Województwo',
      'postal_code': 'Kod pocztowy',
      'country': 'Kraj',
      'invoice': 'Faktura',
      'invoices': 'Faktury',
      'new_invoice': 'Nowa faktura',
      'created_invoice': 'Faktura została utworzona',
      'updated_invoice': 'Faktura została zaktualizowana',
      'archived_invoice': 'Faktura została zarchiwizowana',
      'deleted_invoice': 'Faktura została usunięta',
      'restored_invoice': 'Faktura została przywrócona',
      'archived_invoices': 'Zarchiwizowano :count faktury',
      'deleted_invoices': 'Usunięto :count faktury',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Faktura została wysłana',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Kwota',
      'invoice_number': 'Numer Faktury',
      'invoice_date': 'Data Faktury',
      'discount': 'Rabat',
      'po_number': 'Numer zamówienia',
      'terms': 'Warunki',
      'public_notes': 'Notatki publiczne',
      'private_notes': 'Prywatne notatki',
      'frequency': 'Częstotliwość',
      'start_date': 'Początkowa data',
      'end_date': 'Końcowa data',
      'quote_number': 'Numer oferty',
      'quote_date': 'Data oferty',
      'valid_until': 'Ważny do',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Opis towaru / usługi',
      'unit_cost': 'Cena j. netto',
      'quantity': 'Ilość',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefon służbowy',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Termin',
      'partial_due_date': 'Partial Due Date',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Suma',
      'percent': 'Procent',
      'edit': 'Edytuj',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Stawka zadania',
      'settings': 'Ustawienia',
      'language': 'Language',
      'currency': 'Waluta',
      'created_at': 'Data utworzenia',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Podatek',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Po terminie',
      'draft': 'Wersja robocza',
      'sent': 'Wysłane',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Zaliczka/Opł.część',
      'paid': 'Zapłacone',
      'mark_sent': 'Oznacz jako wysłane',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Gotowe',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Tryb ciemny',
      'restart_app_to_apply_change':
          'Uruchom ponownie aplikację, aby zastosować zmianę',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Dziennik aktywności',
      'no_records_found': 'No records found',
      'clone': 'Klonuj',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Rozmiar',
      'payment_terms': 'Warunki płatnicze',
      'payment_date': 'Data płatności',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal klienta',
      'show_tasks': 'Pokaż zadania',
      'email_reminders': 'Email Reminders',
      'enabled': 'Aktywny',
      'recipients': 'Odbiorcy',
      'initial_email': 'Początkowy email',
      'first_reminder': 'Pierwsze przypomnienie',
      'second_reminder': 'Drugie przypomnienie',
      'third_reminder': 'Trzecie przypomnienie',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Szablon',
      'send': 'Send',
      'subject': 'Temat',
      'body': 'Treść',
      'send_email': 'Wyślij email',
      'email_receipt': 'Wyślij potwierdzenie zapłaty do klienta',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Dostosuj',
      'history': 'Historia',
      'payment': 'Płatność',
      'payments': 'Płatności',
      'refunded': 'Refunded',
      'payment_type': 'Typ płatności',
      'transaction_reference': 'Numer referencyjny transakcji',
      'enter_payment': 'Wprowadź płatność',
      'new_payment': 'Nowa płatność',
      'created_payment': 'Płatność została utworzona',
      'updated_payment': 'Płatność została zaktualizowana',
      'archived_payment': 'Płatność zostałą zarchiwizowana',
      'deleted_payment': 'Płatność została usunięta',
      'restored_payment': 'Płatność została przywrócona',
      'archived_payments': 'Zarchiwizowano :count płatności',
      'deleted_payments': 'Usunięto :count płatności',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Oferta',
      'quotes': 'Oferty',
      'new_quote': 'Nowa oferta',
      'created_quote': 'Oferta została utworzona',
      'updated_quote': 'Oferta została zaktualizowana',
      'archived_quote': 'Oferta została zarchiwizowana',
      'deleted_quote': 'Oferta została usunięta',
      'restored_quote': 'Oferta została przywrócona',
      'archived_quotes': 'Zarchiwizowano :count ofert',
      'deleted_quotes': 'Usunięto :count ofert',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Wydatek',
      'expenses': 'Wydatki',
      'vendor': 'Dostawca',
      'vendors': 'Dostawcy',
      'task': 'Zadanie',
      'tasks': 'Zadania',
      'project': 'Projekt',
      'projects': 'Projekty',
      'activity_1': ':user stworzył klienta :client',
      'activity_2': ':user zarchiwizował klienta :client',
      'activity_3': ':user usunął klienta :client',
      'activity_4': ':user stworzył fakturę :invoice',
      'activity_5': ':user zaktualizował fakturę :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user zarchiwizował fakturę :invoice',
      'activity_9': ':user usunął fakturę :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user zaktualizował płatność :payment',
      'activity_12': ':user zarchiwizował płatność :payment',
      'activity_13': ':user usunął płatność :payment',
      'activity_14': ':user wprowadził kredyt :credit',
      'activity_15': ':user zaktualizował kredyt :credit',
      'activity_16': ':user zarchiwizował kredyt :credit',
      'activity_17': ':user usunął kredyt :credit',
      'activity_18': ':user stworzył ofertę :quote',
      'activity_19': ':user zakatualizował ofertę :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact wyświetlił ofertę :quote',
      'activity_22': ':user zarchiwizował ofertę :quote',
      'activity_23': ':user usunął ofertę :quote',
      'activity_24': ':user przywrócił ofertę :quote',
      'activity_25': ':user przywrócił fakturę :invoice',
      'activity_26': ':user przywrócił klienta :client',
      'activity_27': ':user przywrócił płatność :payment',
      'activity_28': ':user przywrócił kredyt :credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user utworzył dostawcę :vendor',
      'activity_31': ':user zarchiwizował dostawcę :vendor',
      'activity_32': ':user usunął dostawcę :vendor',
      'activity_33': ':user przywrócił dostawcę :vendor',
      'activity_34': ':user utworzył wydatek :expense',
      'activity_35': ':user zarchiwizował wydatek :expense',
      'activity_36': ':user usunął wydatek :expense',
      'activity_37': ':user przywrócił wydatek :expense',
      'activity_39': ':user anulował płatność na :payment_amount nr. :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': 'płatność :payment_amount (:payment) nieudana',
      'activity_42': ':user stworzył zadanie :task',
      'activity_43': ':user zaktualizował zadanie :task',
      'activity_44': ':user zarchiwizował zadanie :task',
      'activity_45': ':user usunął zadanie :task',
      'activity_46': ':user przywrócił zadanie :task',
      'activity_47': ':user zaktualizował wydatek :expense',
      'activity_48': ':user zaktualizował zgłoszenie :ticket',
      'activity_49': ':user zamknął zgłoszenie :ticket',
      'activity_50': ':user połączył zgłoszenie :ticket',
      'activity_51': ':user rozdzielił zgłoszenie :ticket',
      'activity_52': ':contact otworzył zgłoszenie :ticket',
      'activity_53': ':contact otworzył ponownie zgłoszenie :ticket',
      'activity_54': ':user  otworzył zgłoszenie :ticket ponownie ',
      'activity_55': ':contact odpowiedział w zgłoszeniu :ticket',
      'activity_56': ':user oglądał zgłoszenie :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Oferta została wysłana',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Wygasło',
      'all': 'Wszystko',
      'select': 'Wybierz',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Licznik numeru faktury',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Licznik numeru oferty',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Typ',
      'invoice_amount': 'Kwota faktury',
      'invoice_due_date': 'Termin Płatności',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Płatność Automatyczna',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Nazwa podatku',
      'tax_amount': 'Podatek',
      'tax_paid': 'Podatek zapłacony',
      'payment_amount': 'Kwota płatności',
      'age': 'Wiek',
      'is_running': 'Is Running',
      'time_log': 'Rejestr czasu',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Kategoria wydatku',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'pt_BR': {
      'import_data': 'Importar Dados',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Última Atualização',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Primeiro Nome do Contato',
      'contact_last_name': 'Último Nome do Contato',
      'order': 'Order',
      'unassigned': 'Não Designado',
      'partial_value': 'Precisa ser maior que zero e menor que o total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Habilitar',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Definir o subdomínio ou mostrar a fatura em seu próprio website',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Segurança',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Assinaturas',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Ediar Assinatura',
      'created_subscription': 'Assinatura criada com sucesso',
      'updated_subscription': 'Assinatura atualizada com sucesso',
      'archived_subscription': 'Assinatura arquivada com sucesso',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Reenviar Convite',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Devolvido',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Eccaneie o código de barras com um app compatível com :link',
      'enabled_two_factor':
          'Ativação de Autenticação em 2 Fatores realizada com sucesso.',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Autenticação em 2 Fatores',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Pagamento Reembolsado',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Último Quadrimestre',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Converter em Fatura',
      'registration_url': 'Registration URL',
      'invoice_project': 'Faturar Projeto',
      'invoice_task': 'Faturar Tarefa',
      'invoice_expense': 'Faturar Despesa',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Eventos com Suporte',
      'converted_amount': 'Quantia Convertida',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Documentos Padrão',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Ocultar',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Coluna',
      'sample': 'Amostra',
      'map_to': 'Map To',
      'import': 'Importar',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Selecione um arquivo',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Arquivo CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Serviço',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Não Pago',
      'white_label': 'White Label',
      'delivery_note': 'Nota de Entrega',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Vencimento Parcial',
      'invoice_total': 'Total da Fatura',
      'quote_total': 'Total do Orçamento',
      'credit_total': 'Total do Crédito',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Aviso',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Nome do Cliente',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Status da tarefa atualizado com sucesso',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Categorias de Despesas',
      'new_expense_category': 'Nova Categoria de Despesas',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Categoria de despesas criada com sucesso',
      'updated_expense_category':
          'Categoria de despesas atualizada com sucesso',
      'archived_expense_category':
          'Categoria de despesas arquivada com sucesso',
      'deleted_expense_category': 'Categoria excluída com sucesso',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category':
          'Categoria de despesas restaurada com sucesso',
      'archived_expense_categories':
          ':count categorias de despesas arquivadas com sucesso',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'Ver alterações',
      'force_update': 'Forçar atualização',
      'force_update_help':
          'Você está executando a versão mais recente, mas pode haver correções pendentes disponíveis.',
      'mark_paid_help': 'Acompanhe se a despesa foi paga',
      'should_be_invoiced': 'Deverá ser Faturada',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Marcar como Ativo',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Fatura Recorrente',
      'recurring_invoices': 'Faturas Recorrentes',
      'new_recurring_invoice': 'Nova Fatura Recorrente',
      'edit_recurring_invoice': 'Editar Fatura Recorrente',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Fatura Recorrente arquivada com sucesso',
      'deleted_recurring_invoice': 'Fatura recorrente excluída com sucesso',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Fatura Recorrente restaurada com sucesso',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Lucro',
      'line_item': 'Item de linha',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Aberto',
      'payment_reconciliation_failure': 'Falha de reconciliação',
      'payment_reconciliation_success': 'Sucesso de Reconciliação',
      'gateway_success': 'Sucesso do Portal',
      'gateway_failure': 'Falha do Portal',
      'gateway_error': 'Erro do Portal',
      'email_send': 'Email Enviado',
      'email_retry_queue': 'Fila de Repetição de Email',
      'failure': 'Falha',
      'quota_exceeded': 'Cota excedida',
      'upstream_failure': 'Falha Upstream',
      'system_logs': 'Logs de Sistema',
      'view_portal': 'Visualizar portal',
      'copy_link': 'Link de cópia',
      'token_billing': 'Salvar detalhes do cartão',
      'welcome_to_invoice_ninja': 'Bem-vindo ao Invoice Ninja',
      'always': 'Sempre',
      'optin': 'Autorizar',
      'optout': 'Desautorizar',
      'label': 'Rótulo',
      'client_number': 'Número do Cliente',
      'auto_convert': 'Auto Conversão',
      'company_name': 'Nome da Empresa',
      'reminder1_sent': 'Lembrete 1 Enviado',
      'reminder2_sent': 'Lembrete 2 Enviado',
      'reminder3_sent': 'Lembrete 3 Enviado',
      'reminder_last_sent': 'Último Lembrete Enviado',
      'pdf_page_info': 'Página: atual de: total',
      'emailed_invoices': 'Faturas enviadas por email com sucesso',
      'emailed_quotes': 'Orçamentos enviados por email com sucesso',
      'emailed_credits': 'Créditos enviados por e-mail com sucesso',
      'gateway': 'Gateway',
      'view_in_stripe': 'Ver em Listra',
      'rows_per_page': 'Linhas por Página',
      'hours': 'Horas',
      'statement': 'Declaração',
      'taxes': 'Impostos',
      'surcharge': 'Sobretaxa',
      'apply_payment': 'Aplicar Pagamento',
      'apply': 'Aplicar',
      'unapplied': 'Não Aplicado',
      'select_label': 'Selecione o Rótulo',
      'custom_labels': 'Etiquetas Personalizadas',
      'record_type': 'Tipo de Registro',
      'record_name': 'Nome do Registro',
      'file_type': 'Tipo de Arquivo',
      'height': 'Altura',
      'width': 'Largura',
      'to': 'Para',
      'health_check': 'Exame de saúde',
      'payment_type_id': 'Tipo de Pagamento',
      'last_login_at': 'Último login em',
      'company_key': 'Chave da Empresa',
      'storefront': 'Vitrine',
      'storefront_help': 'Habilite aplicativos de terceiros para criar faturas',
      'client_created': 'Cliente Criado',
      'online_payment_email': 'Email de pagamento online',
      'manual_payment_email': 'Email de pagamento manual',
      'completed': 'Completado',
      'gross': 'Bruto',
      'net_amount': 'Valor líquido',
      'net_balance': 'Saldo Líquido',
      'client_settings': 'Configurações do cliente',
      'selected_invoices': 'Faturas Selecionadas',
      'selected_payments': 'Pagamentos Selecionados',
      'selected_quotes': 'Cotações Selecionadas',
      'selected_tasks': 'Tarefas Selecionadas',
      'selected_expenses': 'Despesas Selecionadas',
      'upcoming_invoices': 'Próximas Faturas',
      'past_due_invoices': 'Faturas Vencidas',
      'recent_payments': 'Pagamentos Recentes',
      'upcoming_quotes': 'Próximos Orçamentos',
      'expired_quotes': 'Orçamentos Expirados',
      'create_client': 'Criar Cliente',
      'create_invoice': 'Criar Fatura',
      'create_quote': 'Criar Orçamento',
      'create_payment': 'Criar Pagamento',
      'create_vendor': 'Criar fornecedor',
      'update_quote': 'Atualizar Cotação',
      'delete_quote': 'Excluir Orçamento',
      'update_invoice': 'Atualizar Fatura',
      'delete_invoice': 'Excluir Fatura',
      'update_client': 'Atualizar Cliente',
      'delete_client': 'Excluir Cliente',
      'delete_payment': 'Excluir Pagamento',
      'update_vendor': 'Atualizar Fornecedor',
      'delete_vendor': 'Excluir Fornecedor',
      'create_expense': 'Criar Despesa',
      'update_expense': 'Atualizar Despesa',
      'delete_expense': 'Excluir Despesa',
      'create_task': 'Criar Tarefa',
      'update_task': 'Atualizar Tarefa',
      'delete_task': 'Excluir Tarefa',
      'approve_quote': 'Aprovar Cotação',
      'off': 'Desligado',
      'when_paid': 'Quando Pago',
      'expires_on': 'Expira em',
      'free': 'Gratuito',
      'plan': 'Plano',
      'show_sidebar': 'Exibir Barra Lateral',
      'hide_sidebar': 'Ocultar Barra Lateral',
      'event_type': 'Tipo de Evento',
      'target_url': 'Alvo',
      'copy': 'Cópia',
      'must_be_online':
          'Reinicie o aplicativo assim que estiver conectado à internet',
      'crons_not_enabled': 'Os crons precisam ser habilitados',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Pesquisar: contar Webhooks',
      'search_webhook': 'Pesquisar 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Nova Webhook',
      'edit_webhook': 'Editar Webhook',
      'created_webhook': 'Webhook Criada com Sucesso',
      'updated_webhook': 'Webhook Atualizada com Sucesso',
      'archived_webhook': 'Webhook Arquivada com Sucesso',
      'deleted_webhook': 'Webhook Excluída com Sucesso',
      'removed_webhook': 'Webhook Removida com Sucesso',
      'restored_webhook': 'Webhook Restaurada com Sucesso',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'Tokens de API',
      'api_docs': 'API Docs',
      'search_tokens': 'Pesquisar: contar Tokens',
      'search_token': 'Pesquisar 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'Novo Token',
      'edit_token': 'Editar Token',
      'created_token': 'Token criado com sucesso',
      'updated_token': 'Token atualizado com sucesso',
      'archived_token': 'Token arquivado com sucesso',
      'deleted_token': 'Token excluído com sucesso',
      'removed_token': 'Token Removido com Sucesso',
      'restored_token': 'Token Restaurado com Sucesso',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Registro de cliente',
      'client_registration_help':
          'Permitir que os clientes se auto-registrem no portal',
      'customize_and_preview': 'Personalizar & Visualizar',
      'email_invoice': 'Enviar Fatura por Email',
      'email_quote': 'Enviar Orçamento por Email',
      'email_credit': 'Crédito de Email',
      'email_payment': 'Pagamento por Email',
      'client_email_not_set':
          'O cliente não tem um endereço de e-mail definido',
      'ledger': 'Ledger',
      'view_pdf': 'Ver PDF',
      'all_records': 'Todos os registros',
      'owned_by_user': 'Propriedade do usuário',
      'credit_remaining': 'Crédito Restante',
      'contact_name': 'Nome do Contato',
      'use_default': 'Use o padrão',
      'reminder_endless': 'Lembrete contínuo',
      'number_of_days': 'Número de dias',
      'configure_payment_terms': 'Configurar as condições de pagamento',
      'payment_term': 'Condição de Pagamento',
      'new_payment_term': 'Novo Condição de Pagamento',
      'edit_payment_term': 'Editar Condição de Pagamento',
      'created_payment_term': 'Condições de pagamento criadas com sucesso',
      'updated_payment_term': 'Condições de pagamento atualizadas com sucesso',
      'archived_payment_term': 'Condições de pagamento arquivadas com sucesso',
      'deleted_payment_term': 'Condição de pagamento excluídas com sucesso',
      'removed_payment_term': 'Condição de pagamento removida com sucesso',
      'restored_payment_term': 'Condição de pagamento restaurado com sucesso',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Entrar com email',
      'change': 'Mudar',
      'change_to_mobile_layout': 'Mudar para o layout móvel?',
      'change_to_desktop_layout': 'Mudar para o layout da área de trabalho?',
      'send_from_gmail': 'Enviar do Gmail',
      'reversed': 'Invertido',
      'cancelled': 'Cancelado',
      'credit_amount': 'Quantia de Crédito',
      'quote_amount': 'Valor da cotação',
      'hosted': 'Hospedado',
      'selfhosted': 'Auto-hospedado',
      'exclusive': 'Exclusivo',
      'inclusive': 'Inclusivo',
      'hide_menu': 'Ocultar Menu',
      'show_menu': 'Exibir Menu',
      'partially_refunded': 'Parcialmente Reembolsado',
      'search_documents': 'Pesquisar Documentos',
      'search_designs': 'Pesquisar Designs',
      'search_invoices': 'Pesquisar Faturas',
      'search_clients': 'Pesquisar Clientes',
      'search_products': 'Pesquisar Produtos',
      'search_quotes': 'Pesquisar Cotações',
      'search_credits': 'Pesquisar Créditos',
      'search_vendors': 'Pesquisar Fornecedores',
      'search_users': 'Pesquisar Usuários',
      'search_tax_rates': 'Pesquisar taxas de impostos',
      'search_tasks': 'Pesquisar Tarefas',
      'search_settings': 'Pesquisar Configurações',
      'search_projects': 'Pesquisar Projetos',
      'search_expenses': 'Pesquisar Despesas',
      'search_payments': 'Pesquisar Pagamentos',
      'search_groups': 'Pesquisar Grupos',
      'search_company': 'Pesquisar Empresa',
      'search_document': 'Pesquisar 1 Documento',
      'search_design': 'Pesquisar 1 Design',
      'search_invoice': 'Pesquisar 1 Fatura',
      'search_client': 'Pesquisar 1 Cliente',
      'search_product': 'Pesquisar 1 Produto',
      'search_quote': 'Pesquisar 1 Cotação',
      'search_credit': 'Pesquisar 1 Crédito',
      'search_vendor': 'Pesquisar 1 Fornecedor',
      'search_user': 'Pesquisar 1 Usuário',
      'search_tax_rate': 'Pesquisar 1 Taxa de Imposto',
      'search_task': 'Pesquisar 1 Tarefa',
      'search_project': 'Pesquisar 1 Projeto',
      'search_expense': 'Pesquisar 1 Despesa',
      'search_payment': 'Pesquisar 1 Pagamento',
      'search_group': 'Pesquisar 1 Grupo',
      'refund_payment': 'Reembolsar Pagamento',
      'cancelled_invoice': 'Fatura Cancelada com Sucesso',
      'cancelled_invoices': 'Faturas Canceladas com Sucesso',
      'reversed_invoice': 'Fatura Revertida com Sucesso',
      'reversed_invoices': 'Faturas Revertidas com Sucesso',
      'reverse': 'Reverter',
      'full_name': 'Nome Completo',
      'city_state_postal': 'Cidade/Estado/CEP',
      'postal_city_state': 'CEP/Cidade/Estado',
      'custom1': 'Primeiro Personalizado',
      'custom2': 'Segundo Personalizado',
      'custom3': 'Terceiro Personalizado',
      'custom4': 'Quarto Personalizado',
      'optional': 'Opcional',
      'license': 'Licença',
      'purge_data': 'Limpar Dados',
      'purge_successful': 'Dados da empresa limpos com sucesso',
      'purge_data_message':
          'Aviso: Isto irá apagar seus dados permanentemente, não há como defazer esta ação.',
      'invoice_balance': 'Saldo da fatura',
      'age_group_0': '0 - 30 Dias',
      'age_group_30': '30 - 60 Dias',
      'age_group_60': '60 - 90 Dias',
      'age_group_90': '90 - 120 Dias',
      'age_group_120': '120+ Dias',
      'refresh': 'Atualizar',
      'saved_design': 'Design salvo com sucesso',
      'client_details': 'Detalhes do cliente',
      'company_address': 'Endereço da companhia',
      'invoice_details': 'Detalhes da Fatura',
      'quote_details': 'Detalhes da cotação',
      'credit_details': 'Detalhes de crédito',
      'product_columns': 'Colunas de Produto',
      'task_columns': 'Colunas de Tarefas',
      'add_field': 'Adicionar campo',
      'all_events': 'Todos os eventos',
      'permissions': 'Permissões',
      'none': 'Nenhum',
      'owned': 'Owned',
      'payment_success': 'Pagamento realizado com sucesso',
      'payment_failure': 'Falha de Pagamento',
      'invoice_sent': ':count fatura enviada',
      'quote_sent': 'Cotação enviada',
      'credit_sent': 'Crédito Enviado',
      'invoice_viewed': 'Fatura visualizada',
      'quote_viewed': 'Cotação visualizada',
      'credit_viewed': 'Crédito visualizado',
      'quote_approved': 'Cotação aprovada',
      'receive_all_notifications': 'Receber todas as notificações',
      'purchase_license': 'Comprar licença',
      'apply_license': 'Aplicar Licença',
      'cancel_account': 'Excluir Conta',
      'cancel_account_message':
          'Aviso: Isso excluirá permanentemente sua conta, não há como desfazer esta ação.',
      'delete_company': 'Excluir Empresa',
      'delete_company_message':
          'Aviso: Isto irá excluir permanentemente sua empresa, não há como desfazer esta ação.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Cotação convertida com sucesso',
      'credit_design': 'Design de Crédito',
      'includes': 'Inclui',
      'header': 'Cabeçalho',
      'load_design': 'Carregar Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Designs personalizados',
      'designs': 'Designs',
      'new_design': 'Novo Design',
      'edit_design': 'Editar Design',
      'created_design': 'Design criado com sucesso',
      'updated_design': 'Design atualizado com sucesso',
      'archived_design': 'Design arquivado com sucesso',
      'deleted_design': 'Design excluído com sucesso',
      'removed_design': 'Design removido com sucesso',
      'restored_design': 'Design restaurado com sucesso',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Propostas',
      'tickets': 'Tickets',
      'recurring_quotes': 'Orçamentos Recorrentes',
      'recurring_tasks': 'Tarefas Recorrentes',
      'recurring_expenses': 'Despesas Recorrentes',
      'account_management': 'Gerenciamento da Conta',
      'credit_date': 'Data do Crédito',
      'credit': 'Crédito',
      'credits': 'Créditos',
      'new_credit': 'Adicionar Crédito',
      'edit_credit': 'Editar Crédito',
      'created_credit': 'Crédito criado com sucesso',
      'updated_credit': 'Crédito atualizado com sucesso',
      'archived_credit': 'Crédito arquivado com sucesso',
      'deleted_credit': 'Crédito excluído com sucesso',
      'removed_credit': 'Crédito removido com sucesso',
      'restored_credit': 'Crédito restaurado com sucesso',
      'archived_credits': ':count créditos arquivados com sucesso',
      'deleted_credits': ':count créditos excluídos com sucesso',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versão Atual',
      'latest_version': 'Última versão',
      'update_now': 'Atualize agora',
      'a_new_version_is_available':
          'Uma nova versão do aplicativo da web está disponível',
      'update_available': 'Atualização disponível',
      'app_updated': 'Atualização completada com sucesso',
      'learn_more': 'Saiba mais',
      'integrations': 'Integrações',
      'tracking_id': 'Id de rastreamento',
      'slack_webhook_url': 'URL Webhook do Slack',
      'credit_footer': 'Rodapé do Crédito',
      'credit_terms': 'Termos do Crédito',
      'new_company': 'Nova Empresa',
      'added_company': 'Empresa adicionada com sucesso',
      'company1': 'Companhia 1 Personalizada',
      'company2': 'Companhia 2 Personalizada',
      'company3': 'Companhia 3 Personalizada',
      'company4': 'Companhia 4 Personalizada',
      'product1': 'Produto 1 Personalizado',
      'product2': 'Produto 2 Personalizado',
      'product3': 'Produto 3 Personalizado',
      'product4': 'Produto 4 Personalizado',
      'client1': 'Cliente 1 Personalizado',
      'client2': 'Cliente 2 Personalizado',
      'client3': 'Cliente 3 Personalizado',
      'client4': 'Cliente 4 Personalizado',
      'contact1': 'Contato 1 Personalizado',
      'contact2': 'Contato 2 Personalizado',
      'contact3': 'Contato 3 Personalizado',
      'contact4': 'Contato 4 Personalizado',
      'task1': 'Tarefa 1 Personalizada',
      'task2': 'Tarefa 2 Personalizada',
      'task3': 'Tarefa 3 Personalizada',
      'task4': 'Tarefa 4 Personalizada',
      'project1': 'Projeto 1 Personalizado',
      'project2': 'Projeto 2 Personalizado',
      'project3': 'Projeto 3 Personalizado',
      'project4': 'Projeto 4 Personalizado',
      'expense1': 'Despesa 1 Personalizada',
      'expense2': 'Despesa 2 Personalizada',
      'expense3': 'Despesa 3 Personalizada',
      'expense4': 'Despesa 4 Personalizada',
      'vendor1': 'Vendedor 1 Personalizado',
      'vendor2': 'Vendedor 2 Personalizado',
      'vendor3': 'Vendedor 3 Personalizado',
      'vendor4': 'Vendedor 4 Personalizado',
      'invoice1': 'Fatura 1 Personalizada',
      'invoice2': 'Fatura 2 Personalizada',
      'invoice3': 'Fatura 3 Personalizada',
      'invoice4': 'Fatura 4 Personalizada',
      'payment1': 'Pagamento 1 Personalizado',
      'payment2': 'Pagamento 2 Personalizado',
      'payment3': 'Pagamento 3 Personalizado',
      'payment4': 'Pagamento 4 Personalizado',
      'surcharge1': 'Sobretaxa Personalizada 1',
      'surcharge2': 'Sobretaxa Personalizada 2',
      'surcharge3': 'Sobretaxa Personalizada 3',
      'surcharge4': 'Sobretaxa Personalizada 4',
      'group1': 'Grupo 1 Personalizado',
      'group2': 'Grupo 2 Personalizado',
      'group3': 'Grupo 3 Personalizado',
      'group4': 'Grupo 4 Personalizado',
      'reset': 'Redefinir',
      'number': 'Número',
      'export': 'Exportar',
      'chart': 'Gráfico',
      'count': 'Contagem',
      'totals': 'Totais',
      'blank': 'Vazio',
      'day': 'Dia',
      'month': 'Mês',
      'year': 'Ano',
      'subgroup': 'Subgrupo',
      'is_active': 'Ativo',
      'group_by': 'Agrupado por',
      'credit_balance': 'Saldo do Crédito',
      'contact_last_login': 'Último Login do Contato',
      'contact_full_name': 'Nome Completo do Contato',
      'contact_phone': 'Telefone de Contato',
      'contact_custom_value1': 'Valor personalizado do contato 1',
      'contact_custom_value2': 'Valor personalizado do contato 2',
      'contact_custom_value3': 'Valor personalizado do contato 3',
      'contact_custom_value4': 'Valor personalizado do contato 4',
      'shipping_address1': 'Rua de envio',
      'shipping_address2': 'Complemento de envio',
      'shipping_city': 'Cidade de envio',
      'shipping_state': 'Estado/Província de envio',
      'shipping_postal_code': 'CEP de envio',
      'shipping_country': 'País de envio',
      'billing_address1': 'Rua de cobrança',
      'billing_address2': 'Complemento de cobrança',
      'billing_city': 'Cidade de cobrança',
      'billing_state': 'Estado/Província de cobrança',
      'billing_postal_code': 'CEP de cobrança',
      'billing_country': 'País de cobrança',
      'client_id': 'Cód Cliente',
      'assigned_to': 'Atribuído para',
      'created_by': 'Criado por :name',
      'assigned_to_id': 'Atribuído ao ID',
      'created_by_id': 'Criado pelo ID',
      'add_column': 'Adicionar Coluna',
      'edit_columns': 'Editar Colunas',
      'columns': 'Colunas',
      'aging': 'Envelhecimento',
      'profit_and_loss': 'Lucro e Prejuízo',
      'reports': 'Relatórios',
      'report': 'Relatório',
      'add_company': 'Adicionar Empresa',
      'unpaid_invoice': 'Fatura não Paga',
      'paid_invoice': 'Fatura Paga',
      'unapproved_quote': 'Orçamento não Aprovado',
      'help': 'Ajuda',
      'refund': 'Reembolsar',
      'refund_date': 'Data de Reembolso',
      'filtered_by': 'Filtrado por',
      'contact_email': 'Email de Contato',
      'multiselect': 'Seleção múltipla',
      'entity_state': 'Estado',
      'verify_password': 'Verificar Senha',
      'applied': 'Aplicado',
      'include_recent_errors': 'Inclui erros recentes dos logs',
      'your_message_has_been_received':
          'Recebemos sua mensagem e tentaremos responder rapidamente.',
      'message': 'Mensagem',
      'from': 'De',
      'show_product_details': 'Mostrar Detalhes do Produto',
      'show_product_details_help':
          'Inclua a descrição e o custo na lista suspensa do produto',
      'pdf_min_requirements':
          'A renderização de PDF precisa da versão :version',
      'adjust_fee_percent': 'Ajustar Porcentagem da Multa',
      'adjust_fee_percent_help': 'Ajustar o percentual de taxa a contabilizar',
      'configure_settings': 'Configurações Gerais',
      'support_forum': 'fórum de suporte',
      'about': 'Sobre',
      'documentation': 'Documentação',
      'contact_us': 'Contate-nos',
      'subtotal': 'Subtotal',
      'line_total': 'Total da Linha',
      'item': 'Item',
      'credit_email': 'E-mail de Crédito',
      'iframe_url': 'Website',
      'domain_url': 'URL do Domínio',
      'password_is_too_short': 'A senha é muito curta',
      'password_is_too_easy':
          'A senha deve conter um caractere maiúsculo e um número',
      'client_portal_tasks': 'Tarefas do Portal do Cliente',
      'client_portal_dashboard': 'Painel do Portal do Cliente',
      'please_enter_a_value': 'Por favor digite um valor',
      'deleted_logo': 'Logo removido com sucesso',
      'yes': 'Sim',
      'no': 'Não',
      'generate_number': 'Gerar Número',
      'when_saved': 'Quando Salvo',
      'when_sent': 'Quando Enviado',
      'select_company': 'Selecionar Empresa',
      'float': 'Flutuante',
      'collapse': 'Fechar',
      'show_or_hide': 'Exibir/esconder',
      'menu_sidebar': 'Menu da Barra Lateral',
      'history_sidebar': 'Barra Lateral de Histórico',
      'tablet': 'Tablet',
      'mobile': 'Móvel',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Visualizar',
      'module': 'Módulo',
      'first_custom': 'Primeiro Personalizado',
      'second_custom': 'Segundo Personalizado',
      'third_custom': 'Terceiro Personalizado',
      'show_cost': 'Mostrar Custo',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Exibir um campo de custo do produto para rastrear a marcação/lucro',
      'show_product_quantity': 'Mostrar Quantidade do Produto',
      'show_product_quantity_help':
          'Mostrar um campo de quantidade de produto, caso contrário o padrão de um',
      'show_invoice_quantity': 'Mostrar quantidade da fatura',
      'show_invoice_quantity_help':
          'Exibir um campo de quantidade de item de linha, caso contrário, o padrão é um',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Quantidade Padrão',
      'default_quantity_help':
          'Defina automaticamente a quantidade do item de linha para um',
      'one_tax_rate': 'Uma taxa de imposto',
      'two_tax_rates': 'Duas taxas de impostos',
      'three_tax_rates': 'Três taxas de impostos',
      'default_tax_rate': 'Taxa de imposto padrão',
      'user': 'Usuário',
      'invoice_tax': 'Imposto da Fatura',
      'line_item_tax': 'Imposto da Linha do Item',
      'inclusive_taxes': 'Impostos Inclusos',
      'invoice_tax_rates': 'Tarifa do Imposto da Fatura',
      'item_tax_rates': 'Tarifa do Imposto do Item',
      'no_client_selected': 'Selecione um cliente',
      'configure_rates': 'Configurar tarifas',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Configurações de Impostos',
      'tax_settings_rates': 'Tarifas de Impostos',
      'accent_color': 'Cor de destaque',
      'switch': 'Mudar',
      'comma_sparated_list': 'Lista separada por vírgulas',
      'options': 'Opções',
      'single_line_text': 'Texto de linha única',
      'multi_line_text': 'Texto multilinha',
      'dropdown': 'Dropdown',
      'field_type': 'Tipo de Campo',
      'recover_password_email_sent':
          'Foi enviado um e-mail de recuperação de senha',
      'submit': 'Enviar',
      'recover_password': 'Recupere sua senha',
      'late_fees': 'Taxas atrasadas',
      'credit_number': 'Número do Crédito',
      'payment_number': 'Pagamento Número',
      'late_fee_amount': 'Quantia da Multa',
      'late_fee_percent': 'Percentual de Multa',
      'schedule': 'Agendamento',
      'before_due_date': 'Até a data de vencimento',
      'after_due_date': 'Depois da data de vencimento',
      'after_invoice_date': 'Até a data da fatura',
      'days': 'Dias',
      'invoice_email': 'Email de Fatura',
      'payment_email': 'Email de Pagamento',
      'partial_payment': 'Pagamento parcial',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Email de pagamento parcial',
      'quote_email': 'Email de Orçamento',
      'endless_reminder': 'Lembrete contínuo',
      'filtered_by_user': 'Filtrado por Usuário',
      'administrator': 'Administrador',
      'administrator_help':
          'Permite ao usuário gerenciar usuários, mudar configurações e modificar todos os registros',
      'user_management': 'Gerenciamento de Usuários',
      'users': 'Usuários',
      'new_user': 'Novo Usuário',
      'edit_user': 'Editar Usuário',
      'created_user': 'Usuário criado com sucesso',
      'updated_user': 'Usuário atualizado com sucesso',
      'archived_user': 'Usuário arquivado com sucesso',
      'deleted_user': 'Usuário excluído com sucesso',
      'removed_user': 'Usuário removido com sucesso',
      'restored_user': 'Usuário restaurado com sucesso',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Configurações Gerais',
      'invoice_options': 'Opções da Fatura',
      'hide_paid_to_date': 'Ocultar Pago até Hoje',
      'hide_paid_to_date_help':
          'Apenas mostrar \"Pago até a Data\" em suas faturas uma vez que o pagamento for recebido.',
      'invoice_embed_documents': 'Embutir Documentos',
      'invoice_embed_documents_help': 'Incluir imagens anexas na fatura.',
      'all_pages_header': 'Exibir Cabeçalho em',
      'all_pages_footer': 'Exibir Rodapé em',
      'first_page': 'Primeira página',
      'all_pages': 'Todas as páginas',
      'last_page': 'Última página',
      'primary_font': 'Fonte Primária',
      'secondary_font': 'Fonte Secundária',
      'primary_color': 'Cor Primária',
      'secondary_color': 'Cor Secundaria',
      'page_size': 'Tamanho da Página',
      'font_size': 'Tamanho da Fonte',
      'quote_design': 'Design do Orçamento',
      'invoice_fields': 'Campos da Fatura',
      'product_fields': 'Campos de Produtos',
      'invoice_terms': 'Condições da Fatura',
      'invoice_footer': 'Rodapé da Fatura',
      'quote_terms': 'Condições do Orçamento',
      'quote_footer': 'Rodapé do Orçamento',
      'auto_email_invoice': 'Email Automático',
      'auto_email_invoice_help':
          'Enviar faturas recorrentes por email automaticamente quando forem criadas.',
      'auto_archive_invoice': 'Arquivar Automaticamente',
      'auto_archive_invoice_help':
          'Arquivar automaticamente faturas quando forem pagas.',
      'auto_archive_quote': 'Arquivar Automaticamente',
      'auto_archive_quote_help':
          'Arquivar automaticamente orçamentos quando forem convertidos.',
      'auto_convert_quote': 'Auto Conversão',
      'auto_convert_quote_help':
          'Converter automaticamente um orçamento quando for aprovado pelo cliente.',
      'workflow_settings': 'Configurações de Fluxo de Trabalho',
      'freq_daily': 'Diariamente',
      'freq_weekly': 'Semanalmente',
      'freq_two_weeks': '2 semanas',
      'freq_four_weeks': '4 semanas',
      'freq_monthly': 'Mensalmente',
      'freq_two_months': 'Dois meses',
      'freq_three_months': '3 meses',
      'freq_four_months': '4 meses',
      'freq_six_months': '6 meses',
      'freq_annually': 'Anualmente',
      'freq_two_years': '2 anos',
      'freq_three_years': 'Três Anos',
      'never': 'Nunca',
      'company': 'Empresa',
      'generated_numbers': 'Números Gerados',
      'charge_taxes': 'Cobrar impostos',
      'next_reset': 'Próximo Reset',
      'reset_counter': 'Reiniciar Contador',
      'recurring_prefix': 'Prefixo da Recorrência',
      'number_padding': 'Preenchimento de número',
      'general': 'Geral',
      'surcharge_field': 'Campo de Sobretaxa',
      'company_field': 'Campo da Empresa',
      'company_value': 'Valor da Empresa',
      'credit_field': 'Campo de Crédito',
      'invoice_field': 'Campo da Fatura',
      'invoice_surcharge': 'Sobretaxa de Fatura',
      'client_field': 'Campo do Cliente',
      'product_field': 'Campo do Produto',
      'payment_field': 'Campo de Pagamento',
      'contact_field': 'Campo do Contato',
      'vendor_field': 'Campo do Fornecedor',
      'expense_field': 'Campo da Despesa',
      'project_field': 'Campo do Projeto',
      'task_field': 'Campo da Tarefa',
      'group_field': 'Campo de Grupo',
      'number_counter': 'Contador Numérico',
      'prefix': 'Prefixo',
      'number_pattern': 'Padrão de Numeração',
      'messages': 'Mensagens',
      'custom_css': 'CSS Personalizado',
      'custom_javascript': 'JavaScript Personalizado',
      'signature_on_pdf': 'Exibir em PDF',
      'signature_on_pdf_help':
          'Exibir a assinatura do cliente no PDF da fatura/orçamento.',
      'show_accept_invoice_terms': 'Checkbox para Condições de Fatura',
      'show_accept_invoice_terms_help':
          'Exigir que o cliente confirme que aceita as condições da fatura.',
      'show_accept_quote_terms': 'Checkbox de Condições do Orçamento',
      'show_accept_quote_terms_help':
          'Exigir que cliente confirme que aceita as Condições do Orçamento',
      'require_invoice_signature': 'Assinatura de Fatura',
      'require_invoice_signature_help':
          'Exigir que o cliente providencie sua assinatura',
      'require_quote_signature': 'Assinatura de Orçamento',
      'enable_portal_password': 'Proteger Faturas com Senha',
      'enable_portal_password_help':
          'Permite definir uma senha para cada contato. Se uma senha for definida, o contato deverá informar uma senha antes de visualizar faturas.',
      'authorization': 'Autorização',
      'subdomain': 'Subdomínio',
      'domain': 'Domínio',
      'portal_mode': 'Modo Portal',
      'email_signature': 'Atenciosamente,',
      'enable_email_markup_help':
          'Tornar mais fácil para os seus clientes efetuarem seus pagamentos acrescentando marcações schema.org a seus emails.',
      'plain': 'Plano',
      'light': 'Claro',
      'dark': 'Escuro',
      'email_design': 'Design do Email',
      'attach_pdf': 'Anexar PDF',
      'attach_documents': 'Anexar Documentos',
      'attach_ubl': 'Anexar UBL',
      'email_style': 'Estilo do E-mail',
      'enable_email_markup': 'Habilitar Marcação',
      'reply_to_email': 'Email para Resposta',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Email CCO',
      'processed': 'Processado',
      'credit_card': 'Cartão de Crédito',
      'bank_transfer': 'Transferência Bancária',
      'priority': 'Prioridade',
      'fee_amount': 'Valor da Multa',
      'fee_percent': 'Porcentagem da Multa',
      'fee_cap': 'Taxa máxima',
      'limits_and_fees': 'Limites/Multas',
      'enable_min': 'Habilitar mín',
      'enable_max': 'Habilitar máx',
      'min_limit': 'Mín: :min',
      'max_limit': 'Máx: :max',
      'min': 'Min',
      'max': 'Máx',
      'accepted_card_logos': 'Logos de Cartões Aceitos',
      'credentials': 'Credenciais',
      'update_address': 'Atualizar Endereço',
      'update_address_help':
          'Atualizar o endereço do cliente com os dados fornecidos',
      'rate': 'Taxa',
      'tax_rate': 'Taxa do Imposto',
      'new_tax_rate': 'Nova Taxa de Imposto',
      'edit_tax_rate': 'Editar Taxa do Imposto',
      'created_tax_rate': 'Taxa de imposto criada com sucesso',
      'updated_tax_rate': 'Taxa de imposto atualizada com sucesso',
      'archived_tax_rate': 'Taxa de imposto arquivada com sucesso',
      'deleted_tax_rate': 'Taxa de imposto excluída com sucesso',
      'restored_tax_rate': 'Taxa de imposto restaurada com sucesso',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-preencher produtos',
      'fill_products_help':
          'Ao selecionar um produto sua descrição e preço serão automaticamente preenchidos',
      'update_products': 'Atualização automática dos produtos',
      'update_products_help':
          'Atualizar uma fatura irá automaticamenteatualizar a biblioteca de produtos',
      'convert_products': 'Converter Produtos',
      'convert_products_help':
          'Converter automaticamente preços de produtos para a moeda do cliente',
      'fees': 'Taxas',
      'limits': 'Limites',
      'provider': 'Provedor',
      'company_gateway': 'Gateway de Pagamento',
      'company_gateways': 'Gateways de Pagamento',
      'new_company_gateway': 'Novo Gateway',
      'edit_company_gateway': 'Editar Gateway',
      'created_company_gateway': 'Gateway criado com sucesso',
      'updated_company_gateway': 'Gateway atualizado com sucesso',
      'archived_company_gateway': 'Gateway arquivado com sucesso',
      'deleted_company_gateway': 'Gateway excluído com sucesso',
      'restored_company_gateway': 'Gateway restaurado com sucesso',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continuar Editando',
      'discard_changes': 'Descartar Mudanças',
      'default_value': 'Valor padrão',
      'disabled': 'Desabilitado',
      'currency_format': 'Formato de Moeda',
      'first_day_of_the_week': 'Primeiro dia da Semana',
      'first_month_of_the_year': 'Primeiro Mês do Ano',
      'sunday': 'Domingo',
      'monday': 'Segunda-Feira',
      'tuesday': 'Terça-Feira',
      'wednesday': 'Quarta-Feira',
      'thursday': 'Quinta-Feira',
      'friday': 'Sexta-Feira',
      'saturday': 'Sábado',
      'january': 'Janeiro',
      'february': 'Fevereiro',
      'march': 'Março',
      'april': 'Abril',
      'may': 'Maio',
      'june': 'Junho',
      'july': 'Julho',
      'august': 'Agosto',
      'september': 'Setembro',
      'october': 'Outubro',
      'november': 'Novembro',
      'december': 'Dezembro',
      'symbol': 'Símbolo',
      'ocde': 'Código',
      'date_format': 'Formato de Data',
      'datetime_format': 'Formato de Data/Hora',
      'military_time': 'Formato de Tempo 24h',
      'military_time_help': 'Formato de Hora 24h',
      'send_reminders': 'Enviar Lembretes',
      'timezone': 'Fuso Horário',
      'filtered_by_project': 'Filtrado por Projeto',
      'filtered_by_group': 'Filtrado por Grupo',
      'filtered_by_invoice': 'Filtrado por Fatura',
      'filtered_by_client': 'Filtrado por Cliente',
      'filtered_by_vendor': 'Filtrado por Vendedor',
      'group_settings': 'Configurações de Grupos',
      'group': 'Grupo',
      'groups': 'Grupos',
      'new_group': 'Novo Grupo',
      'edit_group': 'Editar Grupo',
      'created_group': 'Grupo criado com sucesso',
      'updated_group': 'Grupo atualizado com sucesso',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Carregar Logo',
      'uploaded_logo': 'Logo carregado com sucesso',
      'logo': 'Logo',
      'saved_settings': 'Configurações salvas com sucesso',
      'product_settings': 'Configurações de Produtos',
      'device_settings': 'Configurações do Dispositivo',
      'defaults': 'Padrões',
      'basic_settings': 'Configurações Básicas',
      'advanced_settings': 'Configurações Avançadas',
      'company_details': 'Detalhes da Empresa',
      'user_details': 'Detalhes do Usuário',
      'localization': 'Localização',
      'online_payments': 'Pagamentos Online',
      'tax_rates': 'Taxas de Impostos',
      'notifications': 'Notificações',
      'import_export': 'Importar | Exportar',
      'custom_fields': 'Campos Personalizados',
      'invoice_design': 'Design da Fatura',
      'buy_now_buttons': 'Botões Compre Já',
      'email_settings': 'Configurações de Email',
      'templates_and_reminders': 'Modelos e Lembretes',
      'credit_cards_and_banks': 'Cartões de Crédito & Bancos',
      'data_visualizations': 'Visualização de Dados',
      'price': 'Preço',
      'email_sign_up': 'Inscrição de Email',
      'google_sign_up': 'Inscrição no Google',
      'thank_you_for_your_purchase': 'Obrigado por sua compra!',
      'redeem': 'Resgatar',
      'back': 'Voltar',
      'past_purchases': 'Compras Passadas',
      'annual_subscription': 'Assinatura Anual',
      'pro_plan': 'Plano Pro',
      'enterprise_plan': 'Plano Empresarial',
      'count_users': ':count usuários',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Por favor digite o primeiro nome',
      'please_enter_a_last_name': 'Por favor digite o sobrenome',
      'please_agree_to_terms_and_privacy':
          'Por favor, aceite os termos de serviço e política de privacidade para criar uma conta.',
      'i_agree_to_the': 'Aceito os',
      'terms_of_service_link': 'termos do serviço',
      'privacy_policy_link': 'política de privacidade',
      'terms_of_service': 'Condições do Serviço',
      'privacy_policy': 'Política de Privacidade',
      'sign_up': 'Cadastro',
      'account_login': 'Login na Conta',
      'view_website': 'Ver o Website',
      'create_account': 'Criar Conta',
      'email_login': 'E-mail de Login',
      'create_new': 'Criar Novo',
      'no_record_selected': 'Nenhum registro selecionado',
      'error_unsaved_changes': 'Por favor, salve ou cancele suas alterações',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Necessita um plano empresarial',
      'take_picture': 'Tire uma Foto',
      'upload_file': 'Enviar Arquivo',
      'document': 'Documento',
      'documents': 'Documentos',
      'new_document': 'Novo Documento',
      'edit_document': 'Editar Documento',
      'uploaded_document': 'Documento enviado com sucesso',
      'updated_document': 'Documento atualizado com sucesso',
      'archived_document': 'Documento arquivado com sucesso',
      'deleted_document': 'Documento apagado com sucesso',
      'restored_document': 'Documento recuperado com sucesso',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Sem Histórico',
      'expense_date': 'Data da Despesa',
      'pending': 'Pendente',
      'expense_status_1': 'Autenticado',
      'expense_status_2': 'Pendente',
      'expense_status_3': 'Faturado',
      'converted': 'Convertido',
      'add_documents_to_invoice': 'Adicionar documentos à fatura',
      'exchange_rate': 'Taxa de Câmbio',
      'convert_currency': 'Converter moeda',
      'mark_paid': 'Marcar como Pago',
      'category': 'Categoria',
      'address': 'Endereço',
      'new_vendor': 'Novo Fornecedor',
      'created_vendor': 'Fornecedor criado com sucesso',
      'updated_vendor': 'Fornecedor atualizado com sucesso',
      'archived_vendor': 'Fornecedor arquivado com sucesso',
      'deleted_vendor': 'Fornecedor excluído com sucesso',
      'restored_vendor': 'Fornecedor restaurado com sucesso',
      'archived_vendors': ':count fornecedores arquivados com sucesso',
      'deleted_vendors': ':count fornecedores excluídos com sucesso',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Informar Despesa',
      'created_expense': 'Despesa criada com sucesso',
      'updated_expense': 'Despesa atualizada com sucesso',
      'archived_expense': 'Despesa arquivada com sucesso',
      'deleted_expense': 'Despesa excluída com sucesso',
      'restored_expense': 'Despesa restaurada com sucesso',
      'archived_expenses': 'Despesas arquivadas com sucesso',
      'deleted_expenses': 'Despesas excluídas com sucesso',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copiar Envio',
      'copy_billing': 'Copiar Cobrança',
      'design': 'Design',
      'failed_to_find_record': 'Falha ao procurar registro',
      'invoiced': 'Faturado',
      'logged': 'Logado',
      'running': 'Executando',
      'resume': 'Retomar',
      'task_errors': 'Por favor corrija quaisquer tempos sobrepostos',
      'start': 'Iniciar',
      'stop': 'Parar',
      'started_task': 'Tarefa iniciada com sucesso',
      'stopped_task': 'Tarefa interrompida com sucesso',
      'resumed_task': 'Tarefa continuada com sucesso',
      'now': 'Agora',
      'auto_start_tasks': 'Iniciar Tarefas Automaticamente',
      'timer': 'Timer',
      'manual': 'Manual',
      'budgeted': 'Orçado',
      'start_time': 'Horário de Início',
      'end_time': 'Horário Final',
      'date': 'Data',
      'times': 'Vezes',
      'duration': 'Duração',
      'new_task': 'Nova Tarefa',
      'created_task': 'Tarefa criada com sucesso',
      'updated_task': 'Tarefa atualizada com sucesso',
      'archived_task': 'Tarefa arquivada com sucesso',
      'deleted_task': 'Tarefa excluída com sucesso',
      'restored_task': 'Tarefa restaurada com sucesso',
      'archived_tasks': ':count tarefas arquivadas com sucesso',
      'deleted_tasks': ':count tarefas excluídas com sucesso',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Por favor digite um nome',
      'budgeted_hours': 'Horas Orçadas',
      'created_project': 'Projeto criado com sucesso',
      'updated_project': 'Projeto atualizado com sucesso',
      'archived_project': 'Projeto arquivado com sucesso',
      'deleted_project': 'Projeto excluído com sucesso',
      'restored_project': 'Projeto restaurado com sucesso',
      'archived_projects': ':count projetos arquivados com sucesso',
      'deleted_projects': ':count projetos excluídos com sucesso',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Novo Projeto',
      'thank_you_for_using_our_app': 'Obrigado por usar nosso app!',
      'if_you_like_it': 'Se você desejar por favor',
      'click_here': 'clique aqui',
      'click_here_capital': 'Clique aqui',
      'to_rate_it': 'para dar uma nota.',
      'average': 'Médio',
      'unapproved': 'Não Aprovado',
      'authenticate_to_change_setting':
          'Por favor autentique-se para modificar esta configuração',
      'locked': 'Travado',
      'authenticate': 'Autenticar',
      'please_authenticate': 'Por favor autentique-se',
      'biometric_authentication': 'Autenticação Biométrica',
      'footer': 'Rodapé',
      'compare': 'Comparar',
      'hosted_login': 'Login Hospedado',
      'selfhost_login': 'Login Auto-Hospedado',
      'google_sign_in': 'Entrar com o Google',
      'today': 'Hoje',
      'custom_range': 'Período Personalizado',
      'date_range': 'Período',
      'current': 'Atual',
      'previous': 'Anterior',
      'current_period': 'Período Atual',
      'comparison_period': 'Período de Comparação',
      'previous_period': 'Período Anterior',
      'previous_year': 'Ano Anterior',
      'compare_to': 'Comparar com',
      'last7_days': 'Últimos 7 Dias',
      'last_week': 'Última Semana',
      'last30_days': 'Últimos 30 Dias',
      'this_month': 'Este Mês',
      'last_month': 'Último Mês',
      'this_year': 'Este Ano',
      'last_year': 'Último Ano',
      'custom': 'Personalizado',
      'clone_to_invoice': 'Clonar para Fatura',
      'clone_to_quote': 'Clonar ao Orçamento',
      'clone_to_credit': 'Clone para crédito',
      'view_invoice': 'Visualizar fatura',
      'convert': 'Converter',
      'more': 'Mais',
      'edit_client': 'Editar Cliente',
      'edit_product': 'Editar Produto',
      'edit_invoice': 'Editar Fatura',
      'edit_quote': 'Editar Orçamento',
      'edit_payment': 'Editar Pagamento',
      'edit_task': 'Editar Tarefa',
      'edit_expense': 'Editar Despesa',
      'edit_vendor': 'Editar Fornecedor',
      'edit_project': 'Editar Projeto',
      'edit_recurring_expense': 'Editar Despesa Recorrente',
      'edit_recurring_quote': 'Editar Orçamento Recorrente',
      'billing_address': 'Endereço de Cobrança',
      'shipping_address': 'Endereço de envio',
      'total_revenue': 'Faturamento Total',
      'average_invoice': 'Média por Fatura',
      'outstanding': 'Em Aberto',
      'invoices_sent': ':count faturas enviadas',
      'active_clients': 'clientes ativos',
      'close': 'Fechar',
      'email': 'Email',
      'password': 'Senha',
      'url': 'URL',
      'secret': 'Segredo',
      'name': 'Nome',
      'logout': 'Sair',
      'login': 'Login',
      'filter': 'Filtrar',
      'sort': 'Ordenar',
      'search': 'Pesquisar',
      'active': 'Ativo',
      'archived': 'Arquivado',
      'deleted': 'Excluído',
      'dashboard': 'Painel',
      'archive': 'Arquivar',
      'delete': 'Excluir',
      'restore': 'Restaurar',
      'refresh_complete': 'Refresh Completo',
      'please_enter_your_email': 'Por favor digite seu email',
      'please_enter_your_password': 'Por favor digite sua senha',
      'please_enter_your_url': 'Por favor digite sua URL',
      'please_enter_a_product_key': 'Por favor digite uma chave de produto',
      'ascending': 'Ascendente',
      'descending': 'Descendente',
      'save': 'Salvar',
      'an_error_occurred': 'Um erro ocorreu',
      'paid_to_date': 'Pago até Hoje',
      'balance_due': 'Saldo Devedor',
      'balance': 'Saldo',
      'overview': 'Resumo',
      'details': 'Detalhes',
      'phone': 'Telefone',
      'website': 'Website',
      'vat_number': 'Inscrição Municipal',
      'id_number': 'CNPJ',
      'create': 'Criar',
      'copied_to_clipboard': ':value copiado para a área de transferência',
      'error': 'Erro',
      'could_not_launch': 'Não foi possível iniciar',
      'contacts': 'Contatos',
      'additional': 'Adicional',
      'first_name': 'Nome',
      'last_name': 'Sobrenome',
      'add_contact': 'Adicionar contato',
      'are_you_sure': 'Você tem certeza?',
      'cancel': 'Cancelar',
      'ok': 'Ok',
      'remove': 'Remover',
      'email_is_invalid': 'Email é inválido',
      'product': 'Produto',
      'products': 'Produtos',
      'new_product': 'Novo Produto',
      'created_product': 'Produto criado com sucesso',
      'updated_product': 'Produto atualizado com sucesso',
      'archived_product': 'Produto arquivado com sucesso',
      'deleted_product': 'Produto excluído com sucesso',
      'restored_product': 'Produto restaurado com sucesso',
      'archived_products': ':count produtos arquivados com sucesso',
      'deleted_products': ':count produtos excluídos com sucesso',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produto',
      'notes': 'Notas',
      'cost': 'Custo',
      'client': 'Cliente',
      'clients': 'Clientes',
      'new_client': 'Novo Cliente',
      'created_client': 'Cliente criado com sucesso',
      'updated_client': 'Cliente atualizado com sucesso',
      'archived_client': 'Cliente arquivado com sucesso',
      'archived_clients': ':count clientes arquivados com sucesso',
      'deleted_client': 'Cliente excluído com sucesso',
      'deleted_clients': ':count clientes excluídos com sucesso',
      'restored_client': 'Cliente restaurado com sucesso',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Rua',
      'address2': 'Complemento',
      'city': 'Cidade',
      'state': 'Estado',
      'postal_code': 'CEP',
      'country': 'País',
      'invoice': 'Fatura',
      'invoices': 'Faturas',
      'new_invoice': 'Nova Fatura',
      'created_invoice': 'Fatura criada com sucesso',
      'updated_invoice': 'Fatura atualizada com sucesso',
      'archived_invoice': 'Fatura arquivada com sucesso',
      'deleted_invoice': 'Fatura excluída com sucesso',
      'restored_invoice': 'Fatura restaurada com sucesso',
      'archived_invoices': ':count faturas arquivadas com sucesso',
      'deleted_invoices': ':count faturas excluídas com sucesso',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Fatura enviada por email com sucesso',
      'emailed_payment': 'Pagamento enviado por email com sucesso',
      'amount': 'Quantia',
      'invoice_number': 'Número da Fatura',
      'invoice_date': 'Data da Fatura',
      'discount': 'Desconto',
      'po_number': 'Nº Ordem de Serviço',
      'terms': 'Condições',
      'public_notes': 'Notas Públicas',
      'private_notes': 'Notas Privadas',
      'frequency': 'Frequência',
      'start_date': 'Data Inicial',
      'end_date': 'Data Final',
      'quote_number': 'Número do Orçamento',
      'quote_date': 'Data do Orçamento',
      'valid_until': 'Válido Até',
      'items': 'Itens',
      'partial_deposit': 'Parcial/Depósito',
      'description': 'Descrição',
      'unit_cost': 'Preço Unitário',
      'quantity': 'Quantidade',
      'add_item': 'Adicionar Item',
      'contact': 'Contato',
      'work_phone': 'Telefone',
      'total_amount': 'Quantia Total',
      'pdf': 'PDF',
      'due_date': 'Data de Vencimento',
      'partial_due_date': 'Data de Vencimento Parcial',
      'status': 'Status',
      'invoice_status_id': 'Status da Fatura',
      'quote_status': 'Status do Orçamento',
      'click_plus_to_add_item': 'Clique + para adicionar um item',
      'click_plus_to_add_time': 'Clique + para adicionar tempo',
      'count_selected': ':count selecionados',
      'total': 'Total',
      'percent': 'Porcento',
      'edit': 'Editar',
      'dismiss': 'Dispensar',
      'please_select_a_date': 'Por favor digite uma data',
      'please_select_a_client': 'Por favor selecione um cliente',
      'please_select_an_invoice': 'Por favor escolha uma fatura',
      'task_rate': 'Taxa de Tarefas',
      'settings': 'Configurações',
      'language': 'Idioma',
      'currency': 'Moeda',
      'created_at': 'Data de Criação',
      'created_on': 'Criado em',
      'updated_at': 'Atualizado',
      'tax': 'Imposto',
      'please_enter_an_invoice_number': 'Por favor digite um número de fatura',
      'please_enter_a_quote_number': 'Por favor digite um número de orçamento',
      'past_due': 'Vencido',
      'draft': 'Rascunho',
      'sent': 'Enviado',
      'viewed': 'Visualizado',
      'approved': 'Aprovado',
      'partial': 'Depósito / Parcial',
      'paid': 'Pago',
      'mark_sent': 'Marcar como Enviada',
      'marked_invoice_as_sent': 'Fatura marcada como enviada com sucesso',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Faturas marcadas como enviadas com sucesso',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Concluído',
      'please_enter_a_client_or_contact_name':
          'Por favor digite um cliente ou nome de contato',
      'dark_mode': 'Modo Escuro',
      'restart_app_to_apply_change': 'Reinicie o app para aplicar a mudança',
      'refresh_data': 'Atualizar Dados',
      'blank_contact': 'Contato Vazio',
      'activity': 'Atividade',
      'no_records_found': 'Nenhum registro encontrado',
      'clone': 'Clonar',
      'loading': 'Carregando',
      'industry': 'Indústria',
      'size': 'Tamanho',
      'payment_terms': 'Condição de Pagamento',
      'payment_date': 'Data do Pagamento',
      'payment_status': 'Status do Pagamento',
      'payment_status_1': 'Pendente',
      'payment_status_2': 'Anulado',
      'payment_status_3': 'Falhou',
      'payment_status_4': 'Completado',
      'payment_status_5': 'Parcialmente Reembolsado',
      'payment_status_6': 'Reembolsado',
      'payment_status_-1': 'Não Aplicado',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Vencimento',
      'client_portal': 'Portal do Cliente',
      'show_tasks': 'Exibir tarefas',
      'email_reminders': 'Lembretes de Email',
      'enabled': 'Habilitado',
      'recipients': 'Destinatários',
      'initial_email': 'Email Inicial',
      'first_reminder': 'Primeiro Lembrete',
      'second_reminder': 'Segundo Lembrete',
      'third_reminder': 'Terceiro Lembrete',
      'reminder1': 'Primeiro Lembrete',
      'reminder2': 'Segundo Lembrete',
      'reminder3': 'Terceiro Lembrete',
      'template': 'Modelo',
      'send': 'Enviar',
      'subject': 'Assunto',
      'body': 'Corpo',
      'send_email': 'Enviar Email',
      'email_receipt': 'Enviar recibo de pagamento ao cliente por email',
      'auto_billing': 'Cobrança automática',
      'button': 'Botão',
      'preview': 'Preview',
      'customize': 'Personalizar',
      'history': 'Histórico',
      'payment': 'Pagamento',
      'payments': 'Pagamentos',
      'refunded': 'Reembolsado',
      'payment_type': 'Tipo de Pagamento',
      'transaction_reference': 'Referência da Transação',
      'enter_payment': 'Informar Pagamento',
      'new_payment': 'Adicionar Pagamento',
      'created_payment': 'Pagamento criado com sucesso',
      'updated_payment': 'Pagamento atualizado com sucesso',
      'archived_payment': 'Pagamento arquivado com sucesso',
      'deleted_payment': 'Pagamento excluído com sucesso',
      'restored_payment': 'Pagamento restaurado com sucesso',
      'archived_payments': ':count pagamentos arquivados com sucesso',
      'deleted_payments': ':count pagamentos excluídos com sucesso',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Orçamento',
      'quotes': 'Orçamentos',
      'new_quote': 'Novo Orçamento',
      'created_quote': 'Orçamento criado com sucesso',
      'updated_quote': 'Orçamento atualizado com sucesso',
      'archived_quote': 'Orçamento aquivado com sucesso',
      'deleted_quote': 'Orçamento excluído com sucesso',
      'restored_quote': 'Orçamento restaurado com sucesso',
      'archived_quotes': ':count orçamentos arquivados com sucesso',
      'deleted_quotes': ':count orçamentos excluídos com sucesso',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Despesa',
      'expenses': 'Despesas',
      'vendor': 'Fornecedor',
      'vendors': 'Fornecedores',
      'task': 'Tarefa',
      'tasks': 'Tarefas',
      'project': 'Projeto',
      'projects': 'Projetos',
      'activity_1': ':user criou o cliente :client',
      'activity_2': ':user arquivou o cliente :client',
      'activity_3': ':user excluiu o cliente :client',
      'activity_4': ':user criou a fatura :invoice',
      'activity_5': ':user atualizou a fatura :invoice',
      'activity_6': ':user enviou a fatura :invoice para :client do :contact',
      'activity_7': ':contact viu a fatura :invoice para o :client',
      'activity_8': ':user arquivou a fatura :invoice',
      'activity_9': ':user excluiu a fatura :invoice',
      'activity_10':
          ':contact efetuou o pagamento :payment de :payment_amount da fatura :invoice do cliente :client',
      'activity_11': ':user atualizou o pagamento :payment',
      'activity_12': ':user arquivou o pagamento :payment',
      'activity_13': ':user excluiu o pagamento :payment',
      'activity_14': ':user adicionou crédito :credit',
      'activity_15': ':user atualizou crédito :credit',
      'activity_16': ':user arquivou o crédito de :credit',
      'activity_17': ':user excluiu crédito :credit',
      'activity_18': ':user criou o orçamento :quote',
      'activity_19': ':user atualizou o orçamento :quote',
      'activity_20':
          ':user enviou o orçamento :quote do cliente :client para o contato :contact',
      'activity_21': ':contact visualizou o orçamento :quote',
      'activity_22': ':user arquivou o orçamento :quote',
      'activity_23': ':user excluiu o orçamento :quote',
      'activity_24': ':user restaurou o orçamento :quote',
      'activity_25': ':user restaurou a fatura :invoice',
      'activity_26': ':user restaurou o cliente :client',
      'activity_27': ':user restaurou o pagamento :payment',
      'activity_28': ':user restaurou o crédito :credit',
      'activity_29':
          ':contact aprovou o orçamento :quote para o cliente :client',
      'activity_30': ':user criou o fornecedor :vendor',
      'activity_31': ':user arquivou o fornecedor :vendor',
      'activity_32': ':user excluiu :vendor',
      'activity_33': ':user restaurou o fornecedor :vendor',
      'activity_34': ':user criou a despesa :expense',
      'activity_35': ':user arquivou a despesa :expense',
      'activity_36': ':user excluiu a despesa :expense',
      'activity_37': ':user restaurou a despesa :expense',
      'activity_39':
          ':user cancelou um pagamento de :payment_amount em :payment',
      'activity_40':
          ':user reembolsou :adjustment de um pagamento :payment_amount em :payment',
      'activity_41': 'Pagamento :payment_amount (:payment) falhou',
      'activity_42': ':user criou a tarefa :task',
      'activity_43': ':user atualizou a tarefa :task',
      'activity_44': ':user arquivou a tarefa :task',
      'activity_45': ':user excluiu a tarefa :task',
      'activity_46': ':user restaurou a tarefa :task',
      'activity_47': ':user atualizou a despesa :expense',
      'activity_48': ':user atualizou o ticket :ticket',
      'activity_49': ':user fechou o ticket :ticket',
      'activity_50': ':user uniu o ticket :ticket',
      'activity_51': ':user dividiu o ticket :ticket',
      'activity_52': ':contact abriu o ticket :ticket',
      'activity_53': ':contact reabriu o ticket :ticket',
      'activity_54': ':user reabriu o ticket :ticket',
      'activity_55': ':contact respondeu o ticket :ticket',
      'activity_56': ':user visualizou o ticket :ticket',
      'activity_57': 'O sistema falhou ao enviar a fatura :invoice',
      'activity_58': ': fatura revertida pelo usuário: fatura',
      'activity_59': ': fatura cancelada pelo usuário: fatura',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Senha One-Time (OTP)',
      'emailed_quote': 'Orçamento enviado por email com sucesso',
      'emailed_credit': 'Crédito enviado com sucesso',
      'marked_quote_as_sent': 'Orçamento marcado como enviado com sucesso',
      'marked_credit_as_sent': 'Crédito marcado com sucesso como enviado',
      'expired': 'Expirado',
      'all': 'Todos',
      'select': 'Selecionar',
      'long_press_multiselect': 'Seleção múltipla de longa pressão',
      'custom_value1': 'Valor Personalizado',
      'custom_value2': 'Valor Personalizado',
      'custom_value3': 'Valor Personalizado 3',
      'custom_value4': 'Valor Personalizado 4',
      'email_style_custom': 'Estilo de E-mail Personalizado',
      'custom_message_dashboard': 'Mensagem de Painel Personalizada',
      'custom_message_unpaid_invoice':
          'Mensagem Personalizada de Fatura Atrasada',
      'custom_message_paid_invoice': 'Mensagem Personalizada de Fatura Paga',
      'custom_message_unapproved_quote':
          'Mensagem Personalizada de Orçamento Não Aprovado',
      'lock_invoices': 'Bloquear Faturas',
      'translations': 'Traduções',
      'task_number_pattern': 'Padrão de Numeração de Tarefa',
      'task_number_counter': 'Contador Numérico de Tarefas',
      'expense_number_pattern': 'Padrão de Numeração de Despesa',
      'expense_number_counter': 'Contador Numérico de Despesas',
      'vendor_number_pattern': 'Padrão de Numeração de Vendedor',
      'vendor_number_counter': 'Contador Numérico de Vendedores',
      'ticket_number_pattern': 'Padrão de Numeração de Ticket',
      'ticket_number_counter': 'Contador Numérico de Tickets',
      'payment_number_pattern': 'Padrão de Numeração de Pagamento',
      'payment_number_counter': 'Contador Numérico de Pagamentos',
      'invoice_number_pattern': 'Padrão de Numeração de Fatura',
      'invoice_number_counter': 'Contador Numérico de Faturas',
      'quote_number_pattern': 'Padrão de Numeração de Orçamento',
      'quote_number_counter': 'Contador Numérico de Orçamentos',
      'client_number_pattern': 'Padrão de Numeração de Crédito',
      'client_number_counter': 'Contador Numérico de Créditos',
      'credit_number_pattern': 'Padrão de Numeração de Crédito',
      'credit_number_counter': 'Contador Numérico de Créditos',
      'reset_counter_date': 'Reiniciar Data do Contador',
      'counter_padding': 'Padrão do Contador',
      'shared_invoice_quote_counter':
          'Contador de cotação de fatura compartilhada',
      'default_tax_name_1': 'Nome fiscal padrão 1',
      'default_tax_rate_1': 'Taxa de imposto padrão 1',
      'default_tax_name_2': 'Nome fiscal padrão 2',
      'default_tax_rate_2': 'Taxa de imposto padrão 2',
      'default_tax_name_3': 'Nome fiscal padrão 3',
      'default_tax_rate_3': 'Taxa de imposto padrão 3',
      'email_subject_invoice': 'Assunto do E-mail de Fatura',
      'email_subject_quote': 'Assunto do E-mail de Orçamento',
      'email_subject_payment': 'Assunto do E-mail de Pagamento',
      'email_subject_payment_partial': 'Assunto de pagamento parcial por email',
      'show_table': 'Exibir Tabelas',
      'show_list': 'Exibir Lista',
      'client_city': 'Cidade do Cliente',
      'client_state': 'Estado do Cliente',
      'client_country': 'País do Cliente',
      'client_is_active': 'Cliente Ativo',
      'client_balance': 'Balanço do Cliente',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tipo',
      'invoice_amount': 'Quantia da Fatura',
      'invoice_due_date': 'Data de Vencimento',
      'tax_rate1': 'Taxa de imposto 1',
      'tax_rate2': 'Taxa de imposto 2',
      'tax_rate3': 'Taxa de imposto 3',
      'auto_bill': 'Cobrança Automática',
      'archived_at': 'Arquivado em',
      'has_expenses': 'Tem despesas',
      'custom_taxes1': 'Impostos personalizados 1',
      'custom_taxes2': 'Impostos personalizados 2',
      'custom_taxes3': 'Impostos personalizados 3',
      'custom_taxes4': 'Impostos personalizados 4',
      'custom_surcharge1': 'Sobretaxa Personalizada 1',
      'custom_surcharge2': 'Sobretaxa Personalizada 2',
      'custom_surcharge3': 'Sobretaxa Personalizada 3',
      'custom_surcharge4': 'Sobretaxa Personalizada 4',
      'is_deleted': 'Excluído',
      'vendor_city': 'Cidade do Vendedor',
      'vendor_state': 'Estado do Vendedor',
      'vendor_country': 'País do Vendedor',
      'is_approved': 'Está aprovado',
      'tax_name': 'Nome do Imposto',
      'tax_amount': 'Quantia de Impostos',
      'tax_paid': 'Impostos pagos',
      'payment_amount': 'Quantia de Pagamento',
      'age': 'Idade',
      'is_running': 'Is Running',
      'time_log': 'Log de Tempo',
      'bank_id': 'Banco',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Categoria de Despesa',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'pt_PT': {
      'import_data': 'Importar Dados',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Não atribuído',
      'partial_value': 'Deve ser maior que zero e menor que o total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Ativar',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Indique o subdomínio ou mostre a nota de pag. no seu site.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Segurança',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Reenviar convite',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Pagamento Reembolsado',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Converter em Nota de Pag.',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Faturar Tarefa',
      'invoice_expense': 'Nota de Pagamento da Despesa',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Ocultar',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Coluna',
      'sample': 'Exemplo',
      'map_to': 'Map To',
      'import': 'Importar',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Por favor selecionar um arquivo',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Ficheiro CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Não pago',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Vencimento Parcial',
      'invoice_total': 'Total da Nota de Pag.',
      'quote_total': 'Total do Orçamento',
      'credit_total': 'Total em crédito',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Aviso',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Client Name',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Categorias de Despesas',
      'new_expense_category': 'Nova Categoria de Despesas',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Categoria de despesa criada com sucesso',
      'updated_expense_category': 'Categoria de despesa atualizada com sucesso',
      'archived_expense_category': 'Categoria de despesa arquivada com sucesso',
      'deleted_expense_category': 'Categoria apagada com sucesso',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category':
          'Categoria de despesa restaurada com sucesso',
      'archived_expense_categories':
          ':count categorias de despesa arquivadas com sucesso',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Deve ser faturada',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Ativar',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Nota de Pagamento Recorrente',
      'recurring_invoices': 'Notas de Pagamento Recorrentes',
      'new_recurring_invoice': 'Nova Nota de Pagamento Recorrente',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Nota de Pagamento Recorrente arquivada',
      'deleted_recurring_invoice': 'Nota de Pagamento Recorrente removida',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Nota de Pagamento Recorrente restaurada',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Lucro',
      'line_item': 'Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Guardar detalhes do cartão',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Sempre',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Nº Cliente',
      'auto_convert': 'Auto Convert',
      'company_name': 'Nome da Empresa',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Notas de pag. enviadas com sucesso',
      'emailed_quotes': 'Orçamentos enviados com sucesso',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Hours',
      'statement': 'Declaração',
      'taxes': 'Impostos',
      'surcharge': 'Sobretaxa',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Para',
      'health_check': 'Health Check',
      'payment_type_id': 'Tipo de pagamento',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Próximas Notas de Pag.',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Pagamentos Recentes',
      'upcoming_quotes': 'Próximos Orçamentos',
      'expired_quotes': 'Orçamentos Expirados',
      'create_client': 'Create Client',
      'create_invoice': 'Criar Nota Pag.',
      'create_quote': 'Criar Orçamento',
      'create_payment': 'Create Payment',
      'create_vendor': 'Criar fornecedor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Apagar Orçamento',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Apagar Nota Pag.',
      'update_client': 'Update Client',
      'delete_client': 'Apagar Cliente',
      'delete_payment': 'Apagar Pagamento',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Apagar Fornecedor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Apagar Despesa',
      'create_task': 'Criar Tarefa',
      'update_task': 'Update Task',
      'delete_task': 'Apagar Tarefa',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Grátis',
      'plan': 'Plano',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Editat Token',
      'created_token': 'Token criado',
      'updated_token': 'Token atualizado',
      'archived_token': 'Token arquivado',
      'deleted_token': 'Token apagado',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Enviar Nota Pag.',
      'email_quote': 'Enviar Orçamento',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Nome do Contacto',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Editar Termo de Pagamento',
      'created_payment_term': 'Criado termo de pagamento com sucesso',
      'updated_payment_term': 'Atualizado termo de pagamento com sucesso',
      'archived_payment_term': 'Arquivado termo de pagamento com sucesso',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Valor do Crédito',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Reembolsar Pagamento',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Nome completo',
      'city_state_postal': 'Cidade/Distrito/C. Postal',
      'postal_city_state': 'Código-Postal/Cidade/Distrito',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purgar dados',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message': 'Aviso: apagará todos os seus dados.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Dias',
      'age_group_30': '30 - 60 Dias',
      'age_group_60': '60 - 90 Dias',
      'age_group_90': '90 - 120 Dias',
      'age_group_120': '120+ Dias',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Detalhes da nota de pag.',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissões',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count nota de pag. enviada',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Aplicar Linceça',
      'cancel_account': 'Cancelar Conta',
      'cancel_account_message':
          'Aviso: Irá apagar permanentemente a sua conta.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Cabeçalho',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Gerir Conta',
      'credit_date': 'Data do Crédito',
      'credit': 'Crédito',
      'credits': 'Créditos',
      'new_credit': 'Introduzir Crédito',
      'edit_credit': 'Editar Crédito',
      'created_credit': 'Crédito criado com sucesso',
      'updated_credit': 'Crédito atualizado com sucesso',
      'archived_credit': 'Crédito arquivado com sucesso',
      'deleted_credit': 'Crédito apagado com sucesso',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Crédito restaurado',
      'archived_credits': ':count créditos arquivados com sucesso',
      'deleted_credits': ':count créditos apagados com sucesso',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versão Atual',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Saber mais',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nova Empresa',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Redefinir',
      'number': 'Number',
      'export': 'Exportar',
      'chart': 'Gráfico',
      'count': 'Count',
      'totals': 'Totais',
      'blank': 'Vazio',
      'day': 'Dia',
      'month': 'Mês',
      'year': 'Ano',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Agrupado por',
      'credit_balance': 'Balanço do Crédito',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contato telefónico',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Criado por :nome',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Colunas',
      'aging': 'Vencidas',
      'profit_and_loss': 'Lucro e prejuízo',
      'reports': 'Relatórios',
      'report': 'Relatório',
      'add_company': 'Adicionar Empresa',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Ajuda',
      'refund': 'Reembolsar',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Email',
      'multiselect': 'Multiselect',
      'entity_state': 'Estado',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Mensagem',
      'from': 'De',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'Documentação',
      'contact_us': 'Contacte-nos',
      'subtotal': 'Subtotal',
      'line_total': 'Total',
      'item': 'Item',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Sim',
      'no': 'Não',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Visualizar',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Utilizador',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Por favor selecione um cliente',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Definições de Impostos',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Alterar',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submeter',
      'recover_password': 'Recuperar palavra-passe',
      'late_fees': 'Late Fees',
      'credit_number': 'Nota de rédito número',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Agendamento',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dias',
      'invoice_email': 'E-mail para Notas de Pag.',
      'payment_email': 'E-mail para Pagamentos',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'E-mail para Orçamentos',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrador',
      'administrator_help':
          'Permite ao utilizador gerir utilizadores, alterar definições e modificar registos.',
      'user_management': 'Gerir utilizadores',
      'users': 'Utilizadores',
      'new_user': 'Novo Utilizador',
      'edit_user': 'Editar Utilizador',
      'created_user': 'Successfully created user',
      'updated_user': 'Utilizador atualizado',
      'archived_user': 'Utilizador arquivado',
      'deleted_user': 'Utilizador apagado',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Utilizador restaurado',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Definições Gerais',
      'invoice_options': 'Opções da Nota Pag.',
      'hide_paid_to_date': 'Ocultar data de pagamento',
      'hide_paid_to_date_help':
          'Apenas mostrar a \"Data de Pagamento\" quanto o pagamento tiver sido efetuado.',
      'invoice_embed_documents': 'Documentos Embutidos',
      'invoice_embed_documents_help':
          'Incluir imagens anexadas na nota de pagamento.',
      'all_pages_header': 'Mostrar cabeçalho ativo',
      'all_pages_footer': 'Mostrar rodapé ativo',
      'first_page': 'primeira página',
      'all_pages': 'todas as páginas',
      'last_page': 'última página',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Cor Principal',
      'secondary_color': 'Cor Secundaria',
      'page_size': 'Tamanho da Página',
      'font_size': 'Tamanho do Texto',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Campos da Nota Pag.',
      'product_fields': 'Campos do produto',
      'invoice_terms': 'Condições da Nota de Pagamento',
      'invoice_footer': 'Rodapé da Nota Pag.',
      'quote_terms': 'Condições do Orçamento',
      'quote_footer': 'Rodapé do Orçamento',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Converter automaticamente um orçamento quando for aprovado pelo cliente.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Semanal',
      'freq_two_weeks': '2 semanas',
      'freq_four_weeks': '4 semanas',
      'freq_monthly': 'Mensal',
      'freq_two_months': 'Dois meses',
      'freq_three_months': 'Trimestral',
      'freq_four_months': 'Four months',
      'freq_six_months': 'Semestral',
      'freq_annually': 'Anual',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Nunca',
      'company': 'Company',
      'generated_numbers': 'Números gerados',
      'charge_taxes': 'Impostos',
      'next_reset': 'Próxima redefinição',
      'reset_counter': 'Redefinir contador',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefixo',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'CSS Personalizado',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Checkbox para Termos da Nota de Pagamento',
      'show_accept_invoice_terms_help':
          'Requer que o cliente confirme que aceita os termos da nota de pagamento.',
      'show_accept_quote_terms': 'Checkbox para Termos do Orçamento',
      'show_accept_quote_terms_help':
          'Requer que o cliente confirme que aceita os termos do orçamento.',
      'require_invoice_signature': 'Assinatura da Nota de Pagamento',
      'require_invoice_signature_help':
          'Requer que o cliente introduza a sua assinatura.',
      'require_quote_signature': 'Assinatura de Orçamento',
      'enable_portal_password': 'Proteger notas de pag. com palavra-passe',
      'enable_portal_password_help':
          'Permite definir uma palavra-passe para cada contacto. Se uma palavra-passe for definida, o contacto deverá introduzir a palavra-passe antes de visualizar a nota de pagamento.',
      'authorization': 'Autorização',
      'subdomain': 'Subdomínio',
      'domain': 'Domínio',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Atenciosamente,',
      'enable_email_markup_help':
          'Tornar mais fácil para os seus clientes efetuarem os pagamentos, acrescentando marcação schema.org a seus e-mails.',
      'plain': 'Plano',
      'light': 'Claro',
      'dark': 'Escuro',
      'email_design': 'Template de E-mail',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Ativar Marcação',
      'reply_to_email': 'Email de resposta',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Email BCC',
      'processed': 'Processed',
      'credit_card': 'Cartão de Crédito',
      'bank_transfer': 'Transferência Bancária',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Ativar min',
      'enable_max': 'Ativar max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Atualizar Morada',
      'update_address_help': 'Atualizar morada do cliente',
      'rate': 'Valor',
      'tax_rate': 'Imposto',
      'new_tax_rate': 'Novo Imposto',
      'edit_tax_rate': 'Editar Imposto',
      'created_tax_rate': 'Imposto Adicionado',
      'updated_tax_rate': 'Imposto Atualizado',
      'archived_tax_rate': 'Imposto Arquivado',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Sugerir produtos',
      'fill_products_help':
          'Selecionando o produto descrição e preço serão preenchidos automaticamente',
      'update_products': 'Atualização automática dos produtos',
      'update_products_help':
          'Atualizando na nota de pagamento o produto também será atualizado',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Taxas',
      'limits': 'Limites',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Desativado',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Domingo',
      'monday': 'Segunda-Feira',
      'tuesday': 'Terça-Feira',
      'wednesday': 'Quarta-Feira',
      'thursday': 'Quinta-Feira',
      'friday': 'Sexta-Feira',
      'saturday': 'Sábado',
      'january': 'Janeiro',
      'february': 'Fevereiro',
      'march': 'Março',
      'april': 'Abril',
      'may': 'Maio',
      'june': 'Junho',
      'july': 'Julho',
      'august': 'Agosto',
      'september': 'Setembro',
      'october': 'Outubro',
      'november': 'Novembro',
      'december': 'Dezembro',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24h',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Definições de Produtos',
      'device_settings': 'Device Settings',
      'defaults': 'Padrões',
      'basic_settings': 'Definições Básicas',
      'advanced_settings': 'Definições Avançadas',
      'company_details': 'Detalhes da Empresa',
      'user_details': 'Detalhes do Utilizador',
      'localization': 'Localização',
      'online_payments': 'Pagamentos Online',
      'tax_rates': 'Impostos',
      'notifications': 'Notificações',
      'import_export': 'Importar/Exportar',
      'custom_fields': 'Campos Personalizados',
      'invoice_design': 'Design das Nota Pag.',
      'buy_now_buttons': 'Botões Comprar Agora',
      'email_settings': 'Definições de E-mail',
      'templates_and_reminders': 'Modelos & Lembretes',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Visualização de Dados',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Condições do Serviço',
      'privacy_policy': 'Política de Privacidade',
      'sign_up': 'Registar',
      'account_login': 'Iniciar sessão',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Criar Nova',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Download',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documentos',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Data da Despesa',
      'pending': 'Pendente',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Convertido',
      'add_documents_to_invoice': 'Adicionar documento à nota de pag.',
      'exchange_rate': 'Taxa de Câmbio',
      'convert_currency': 'Converter moeda',
      'mark_paid': 'Marcar como Pago',
      'category': 'Categoria',
      'address': 'Morada',
      'new_vendor': 'Novo Fornecedor',
      'created_vendor': 'Fornecedor criado com sucesso',
      'updated_vendor': 'Fornecedor atualizado com sucesso',
      'archived_vendor': 'Fornecedor arquivado com sucesso',
      'deleted_vendor': 'Fornecedor removido com sucesso',
      'restored_vendor': 'Fornecedor restarurado com sucesso',
      'archived_vendors': ':count fornecedores arquivados com sucesso',
      'deleted_vendors': ':count fornecedores removidos com sucesso',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Introduzir Despesa',
      'created_expense': 'Despesa criada com sucesso',
      'updated_expense': 'Despesa atualizada com sucesso',
      'archived_expense': 'Despesa arquivada com sucesso',
      'deleted_expense': 'Despesa excluída com sucesso',
      'restored_expense': 'Despesa restaurada com sucesso',
      'archived_expenses': 'Despesas arquivadas com sucesso',
      'deleted_expenses': 'Despesas excluídas com sucesso',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Faturado',
      'logged': 'Em aberto',
      'running': 'Em execução',
      'resume': 'Retormar',
      'task_errors': 'Corrija os tempos sobrepostos',
      'start': 'Iniciar',
      'stop': 'Parar',
      'started_task': 'Successfully started task',
      'stopped_task': 'Tarefa interrompida',
      'resumed_task': 'Successfully resumed task',
      'now': 'Agora',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Temporizador',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Início',
      'end_time': 'Final',
      'date': 'Data',
      'times': 'Tempo',
      'duration': 'Duração',
      'new_task': 'Nova Tarefa',
      'created_task': 'Tarefa criada',
      'updated_task': 'Tarefa atualizada',
      'archived_task': 'Tarefa arquivada',
      'deleted_task': 'Tarefa apagada',
      'restored_task': 'Tarefa restaurada',
      'archived_tasks': ':count Tarefas arquivadas',
      'deleted_tasks': ':count Tarefas apagadas',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Projeto criado com sucesso',
      'updated_project': 'Projeto atualizado com sucesso',
      'archived_project': 'Projeto arquivado com sucesso',
      'deleted_project': 'Projeto apagado com sucesso',
      'restored_project': 'Projeto restaurado com sucesso',
      'archived_projects': ':count projetos arquivados com sucesso',
      'deleted_projects': ':count projectos apagadas com sucesso',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Novo Projeto',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'clique aqui',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Rodapé',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Intervalo Personalizado',
      'date_range': 'Interevalo de Datas',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Este Mês',
      'last_month': 'Último Mês',
      'this_year': 'Este ano',
      'last_year': 'Último Ano',
      'custom': 'Personalizado',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Visualizar nota de pag.',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Editar Cliente',
      'edit_product': 'Editar Prodruto',
      'edit_invoice': 'Editar Nota Pag.',
      'edit_quote': 'Editar Orçamento',
      'edit_payment': 'Editar Pagamento',
      'edit_task': 'Editar Tarefa',
      'edit_expense': 'Editar Despesa',
      'edit_vendor': 'Editar Fornecedor',
      'edit_project': 'Editar Projeto',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Morada de faturação',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Total faturado',
      'average_invoice': 'Média por Nota de Pag.',
      'outstanding': 'Em Aberto',
      'invoices_sent': ':count notas de pag. enviadas',
      'active_clients': 'Clientes ativos',
      'close': 'Fechar',
      'email': 'E-mail',
      'password': 'Palavra-passe',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Nome',
      'logout': 'Sair',
      'login': 'Iniciar sessão',
      'filter': 'Filtrar',
      'sort': 'Sort',
      'search': 'Pesquisa',
      'active': 'Ativo',
      'archived': 'Arquivado',
      'deleted': 'Apagado',
      'dashboard': 'Dashboard',
      'archive': 'Arquivar',
      'delete': 'Apagar',
      'restore': 'Restaurar',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Guardar',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Pago até à data',
      'balance_due': 'Valor',
      'balance': 'Saldo',
      'overview': 'Overview',
      'details': 'Detalhes',
      'phone': 'Telefone',
      'website': 'Website',
      'vat_number': 'NIF',
      'id_number': 'ID Number',
      'create': 'Criar',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Contatos',
      'additional': 'Additional',
      'first_name': 'Primeiro Nome',
      'last_name': 'Último Nome',
      'add_contact': 'Adicionar contato',
      'are_you_sure': 'Tem a certeza?',
      'cancel': 'Cancelar',
      'ok': 'Ok',
      'remove': 'Remover',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produto',
      'products': 'Produtos',
      'new_product': 'Novo Produto',
      'created_product': 'Produto criado',
      'updated_product': 'Produto atualizado',
      'archived_product': 'Produto arquivado',
      'deleted_product': 'Producto apagado com sucesso',
      'restored_product': 'Produto restaurado com sucesso',
      'archived_products': ':count Produtos arquivados com sucesso',
      'deleted_products': ':count produtos apagados com sucesso',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produto',
      'notes': 'Observações',
      'cost': 'Custo',
      'client': 'Cliente',
      'clients': 'Clientes',
      'new_client': 'Novo Cliente',
      'created_client': 'Cliente criado com sucesso',
      'updated_client': 'Cliente atualizado com sucesso',
      'archived_client': 'Cliente arquivado com sucesso',
      'archived_clients': ':count clientes arquivados com sucesso',
      'deleted_client': 'Clientes removidos com sucesso',
      'deleted_clients': ':count clientes removidos com sucesso',
      'restored_client': 'Cliente restaurado',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Rua',
      'address2': 'Complemento',
      'city': 'Cidade',
      'state': 'Distrito/Província',
      'postal_code': 'Código Postal',
      'country': 'País',
      'invoice': 'Nota de Pagamento',
      'invoices': 'Notas Pag.',
      'new_invoice': 'Nova Nota Pag.',
      'created_invoice': 'Nota de Pagamento criada com sucesso',
      'updated_invoice': 'Nota de Pagamento atualizada com sucesso',
      'archived_invoice': 'Nota de Pagamento arquivado com sucesso',
      'deleted_invoice': 'Nota de Pagamento apagados com sucesso',
      'restored_invoice': 'Nota de Pagamento restaurada',
      'archived_invoices': ':count nota de pagamentos arquivados com sucesso',
      'deleted_invoices': ':count nota de pagamentos apagados com sucesso',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Nota de Pagamento enviada por e-mail com sucesso',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Valor',
      'invoice_number': 'Número NP',
      'invoice_date': 'Data da NP',
      'discount': 'Desconto',
      'po_number': 'Núm. Ordem de Serviço',
      'terms': 'Condições',
      'public_notes': 'Notas Públicas',
      'private_notes': 'Notas Privadas',
      'frequency': 'Frequência',
      'start_date': 'Data Inicial',
      'end_date': 'Data Final',
      'quote_number': 'Número do Orçamento',
      'quote_date': 'Data do Orçamento',
      'valid_until': 'Válido até',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Descrição',
      'unit_cost': 'Custo Unitário',
      'quantity': 'Quantidade',
      'add_item': 'Add Item',
      'contact': 'Contato',
      'work_phone': 'Telefone',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Data de Vencimento',
      'partial_due_date': 'Partial Due Date',
      'status': 'Estado',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Percentagem',
      'edit': 'Editar',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Definições',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Imposto',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Rascunho',
      'sent': 'Sent',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Depósito/Parcial',
      'paid': 'Pago',
      'mark_sent': 'Marcar como Enviada',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Feito',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Modo Escuro',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Atividade',
      'no_records_found': 'No records found',
      'clone': 'Clonar',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Condições de Pagamento',
      'payment_date': 'Data do Pagamento',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal do Cliente',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Ativo',
      'recipients': 'Destinatários',
      'initial_email': 'Email inicial',
      'first_reminder': 'Primeiro Lembrete',
      'second_reminder': 'Segundo Lembrete',
      'third_reminder': 'Terceiro Lembrete',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Assunto',
      'body': 'Conteúdo',
      'send_email': 'Enviar email',
      'email_receipt': 'E-mail para envio do recibo de pagamento',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Pré-visualizar',
      'customize': 'Personalizar',
      'history': 'Histórico',
      'payment': 'Pagamento',
      'payments': 'Pagamentos',
      'refunded': 'Refunded',
      'payment_type': 'Tipo de Pagamento',
      'transaction_reference': 'Referência da Transação',
      'enter_payment': 'Introduzir Pag.',
      'new_payment': 'Introduzir Pagamento',
      'created_payment': 'Pagamento criado com sucesso',
      'updated_payment': 'Pagamento atualizado',
      'archived_payment': 'Pagamento arquivado com sucesso',
      'deleted_payment': 'Pagamento apagado com sucesso',
      'restored_payment': 'Pagamento restaurado',
      'archived_payments': ':count pagamentos arquivados com sucesso',
      'deleted_payments': ':count pagamentos apagados com sucesso',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Orçamento',
      'quotes': 'Orçamentos',
      'new_quote': 'Novo Orçamento',
      'created_quote': 'Orçamento criado',
      'updated_quote': 'Orçamento atualizado',
      'archived_quote': 'Orçamento aquivado',
      'deleted_quote': 'Orçamento apagado',
      'restored_quote': 'Orçamento restaurado',
      'archived_quotes': ':count Orçamento(s) arquivado(s)',
      'deleted_quotes': ':count Orçamento(s) apagados(s)',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Despesa',
      'expenses': 'Despesas',
      'vendor': 'Fornecedor',
      'vendors': 'Fornecedor',
      'task': 'Tarefa',
      'tasks': 'Tarefas',
      'project': 'Projeto',
      'projects': 'Projetos',
      'activity_1': ':user criou o cliente :client',
      'activity_2': ':user arquivou o cliente :client',
      'activity_3': ':user removeu o cliente :client',
      'activity_4': ':user criou a nota de pagamento :invoice',
      'activity_5': ':user atualizou a nota de pagamento :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user arquivou a nota de pagamento :invoice',
      'activity_9': ':user removeu a nota de pagamento :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user atualizou o pagamento :payment',
      'activity_12': ':user arquivou o pagamento :payment',
      'activity_13': ':user removeu o pagamento :payment',
      'activity_14': ':user adicionou crédito :credit',
      'activity_15': ':user atualizou crédito :credit',
      'activity_16': ':user arquivou crédito :credit',
      'activity_17': ':user removeu crédito :credit',
      'activity_18': ':user adicionou o orçamento :quote',
      'activity_19': ':user atualizou  o orçamento :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact visualizou o orçamento :quote',
      'activity_22': ':user arquivou o orçamento :quote',
      'activity_23': ':user removeu o orçamento :quote',
      'activity_24': ':user restaurou o orçamento :quote',
      'activity_25': ':user restaurou a nota de pagamento :invoice',
      'activity_26': ':user restaurou o cliente :client',
      'activity_27': ':user restaurou o pagamento :payment',
      'activity_28': ':user restaurou o crédito :credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user criou o fornecedor :vendor',
      'activity_31': ':user arquivou o fornecedor :vendor',
      'activity_32': ':user apagou o fornecedor :vendor',
      'activity_33': ':user restaurou o fornecedor :vendor',
      'activity_34': ':user criou a despesa :expense',
      'activity_35': ':user arquivou a despesa :expense',
      'activity_36': ':user apagou a despesa :expense',
      'activity_37': ':user restaurou a despesa :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': 'pagamento (:payment)  de :payment_amount falhou',
      'activity_42': ':user criou a tarefa :task',
      'activity_43': ':user atualizou a tarefa :task',
      'activity_44': ':user arquivou a tarefa :task',
      'activity_45': ':user apagou a tarefa :task',
      'activity_46': ':user restaurou a tarefa :task',
      'activity_47': ':user atualizou a despesa :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Orçamento enviado',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expirada',
      'all': 'Todos',
      'select': 'Selecionar',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Numeração das Notas de Pagamento',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Numeração dos Orçamentos',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tipo',
      'invoice_amount': 'Total da Nota de Pagamento',
      'invoice_due_date': 'Data de vencimento',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Cobrança Automática',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Nome do Imposto',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Valor do Pagamento',
      'age': 'Idade',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Banco',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Categoria de Despesas',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'ro': {
      'import_data': 'Importă Date',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Trebuie să fie mai mare ca zero și mai mic ca totalul',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Activeaza',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Transformă în Factură',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Făcturează task',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Ascunde',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Coloana',
      'sample': 'Exemplar',
      'map_to': 'Map To',
      'import': 'Importa',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Alege un fisier',
      'no_file_selected': 'No File Selected',
      'csv_file': 'fisier CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Total factura',
      'quote_total': 'Total Proforma',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Nume Client',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Factura Recurenta',
      'recurring_invoices': 'Facturi Recurente',
      'new_recurring_invoice': 'Adauga Factura Recurenta',
      'edit_recurring_invoice': 'Editare factura recurenta',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Factură recurentă arhivată cu succes',
      'deleted_recurring_invoice': 'Factură recurentă ștearsă cu succes',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Factură recurentă restaurată cu succes',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Salvează datele cardului',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Ore',
      'statement': 'Extras',
      'taxes': 'Taxe',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Către',
      'health_check': 'Health Check',
      'payment_type_id': 'Tip plata',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Facturi urmatoare',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Plati recente',
      'upcoming_quotes': 'Proforme următoare',
      'expired_quotes': 'Proforme expirate',
      'create_client': 'Create Client',
      'create_invoice': 'Creaza factura',
      'create_quote': 'Creaza Proforma',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Sterge Proforma',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Sterge factura',
      'update_client': 'Update Client',
      'delete_client': 'Sterge client',
      'delete_payment': 'Sterge plata',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Șterge Furnizor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Delete Expense',
      'create_task': 'Crează Task',
      'update_task': 'Update Task',
      'delete_task': 'Șterge Task',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Gratis',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'Token API',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Token-uri',
      'new_token': 'New Token',
      'edit_token': 'Modifica token',
      'created_token': 'Token creat',
      'updated_token': 'Actualizează token',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Token șters',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Trimite email',
      'email_quote': 'Trimite Proforma',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Valoare credit',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count factură trimisă',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Aplică Licență',
      'cancel_account': 'Șterge cont',
      'cancel_account_message':
          'ATENȚIE: Toate datele vor fi șterse definitiv, nu se pot recupera.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Antet',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Proforme Recurente',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Administrare cont',
      'credit_date': 'Data Creditului',
      'credit': 'Credit',
      'credits': 'Credite',
      'new_credit': 'Adaugă Credit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Credit adăugat cu succes',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Credit arhivat cu succes',
      'deleted_credit': 'Credit șters',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Credit restaurat',
      'archived_credits': ':count credite au fost arhivate cu succes',
      'deleted_credits': ':count șters',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versiunea Curentă',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Afla mai mult',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Firmă nouă',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Resetează',
      'number': 'Number',
      'export': 'Exportă',
      'chart': 'Grafic',
      'count': 'Count',
      'totals': 'Total',
      'blank': 'Blank',
      'day': 'Zi',
      'month': 'Lună',
      'year': 'An',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Grupează după',
      'credit_balance': 'Soldul Creditului',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Asignare lui',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit și Pierdere',
      'reports': 'Reports',
      'report': 'Raport',
      'add_company': 'Adaugă Firmă',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Ajutor',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Mesaj',
      'from': 'De la',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'forum suport',
      'about': 'About',
      'documentation': 'Documentație',
      'contact_us': 'Contact Us',
      'subtotal': 'Subtotal',
      'line_total': 'Total linie',
      'item': 'Element',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Da',
      'no': 'Nu',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Vezi',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Utilizator',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Alege un client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Setari Taxe',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Recuperează parola',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Email Factură',
      'payment_email': 'Email Plată',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Email Ofertă',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Utilizatori',
      'users': 'Utilizatori',
      'new_user': 'New User',
      'edit_user': 'Modifică Utilizator',
      'created_user': 'Successfully created user',
      'updated_user': 'Utilizator actualizat',
      'archived_user': 'Arhivare utilizator cu succes',
      'deleted_user': 'Utilizator șters',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Utilizator restaurat',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Optiuni Generale',
      'invoice_options': 'Opțiuni Factură',
      'hide_paid_to_date': 'Ascunde câmpul \"Plătit până la\"',
      'hide_paid_to_date_help':
          'Afișează \"Plătit pana la\" decât când plata a fost efectuată.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'Prima pagină',
      'all_pages': 'Toate paginile',
      'last_page': 'Ultima pagină',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Culoare Principală',
      'secondary_color': 'Culoare Secundară',
      'page_size': 'Dimensiune Pagină',
      'font_size': 'Dimensiune Font',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Câmpuri Factură',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Termeni facturare',
      'invoice_footer': 'Subsol Factură',
      'quote_terms': 'Termeni Proformă',
      'quote_footer': 'Subsol Proformă',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Zilnic',
      'freq_weekly': 'Săptămânal',
      'freq_two_weeks': 'Două Săptămâni',
      'freq_four_weeks': 'Patru Săptămâni',
      'freq_monthly': 'Lunar',
      'freq_two_months': 'Două Luni',
      'freq_three_months': 'Trei Luni',
      'freq_four_months': 'Patru Luni',
      'freq_six_months': 'Șase Luni',
      'freq_annually': 'Anual',
      'freq_two_years': 'Doi Ani',
      'freq_three_years': 'Three Years',
      'never': 'Niciodată',
      'company': 'Company',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Taxe',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Editeaza CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Subdomeniu',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'În legătură cu,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Deschisa',
      'dark': 'Intunecata',
      'email_design': 'Design Email',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Card de Credit',
      'bank_transfer': 'Transfer Bancar',
      'priority': 'Prioritate',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Actualizează Adresa',
      'update_address_help':
          'Actualizează adresa clientului cu detaliile trimise',
      'rate': 'Valoare',
      'tax_rate': 'Valoare Taxă',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Editează valoare taxă',
      'created_tax_rate': 'Valoare taxă creată cu succes',
      'updated_tax_rate': 'Valoare taxă actualizată cu succes',
      'archived_tax_rate': 'Valoare taxă arhivată cu succes',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Completează automat produsele',
      'fill_products_help':
          'Alegând un produs  descrierea și prețul vor fi completate automat',
      'update_products': 'Actualizare automată a produselor',
      'update_products_help':
          'Actualizând o factură se va actualiza si librăria de produse',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Dezactivat',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Duminică',
      'monday': 'Luni',
      'tuesday': 'Marți',
      'wednesday': 'Miercuri',
      'thursday': 'Joi',
      'friday': 'Vineri',
      'saturday': 'Sâmbătă',
      'january': 'Ianuarie',
      'february': 'Februarie',
      'march': 'Martie',
      'april': 'Aprilie',
      'may': 'Mai',
      'june': 'Iunie',
      'july': 'Iulie',
      'august': 'August',
      'september': 'Septembrie',
      'october': 'Octombrie',
      'november': 'Noiembrie',
      'december': 'Decembrie',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': 'Format 24 Ore',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Opțiuni Produs',
      'device_settings': 'Device Settings',
      'defaults': 'Implicit',
      'basic_settings': 'Opțiuni de bază',
      'advanced_settings': 'Opțiuni avansate',
      'company_details': 'Detalii companie',
      'user_details': 'Detalii utilizator',
      'localization': 'Localizare',
      'online_payments': 'Plati online',
      'tax_rates': 'Valori taxa',
      'notifications': 'Notificări',
      'import_export': 'Import | Export',
      'custom_fields': 'Câmpuri personalizate',
      'invoice_design': 'Design factură',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Setari email',
      'templates_and_reminders': 'Șabloane & Notificări',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Vizualizare Date',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Termenii Serviciului',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Inscrie-te',
      'account_login': 'Autentificare',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Descarca',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Expense Date',
      'pending': 'În așteptare',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Transformă',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Curs Valutar',
      'convert_currency': 'Transformă moneda',
      'mark_paid': 'Mark Paid',
      'category': 'Category',
      'address': 'Adresă',
      'new_vendor': 'Furnizor Nou',
      'created_vendor': 'Furnizor creat cu succes',
      'updated_vendor': 'Furnizor actualizat cu succes',
      'archived_vendor': 'Furnizor arhivat cu succes',
      'deleted_vendor': 'Furnizor șters cu succes',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': ':count furnizori arhivați cu succes',
      'deleted_vendors': ':count furnizori ștersi cu succes',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Introdu Cheltuială',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Successfully updated expense',
      'archived_expense': 'Cheltuială arhivată cu succes',
      'deleted_expense': 'Cheltuială ștearsă cu succes',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Cheltuieli arhivate cu succes',
      'deleted_expenses': 'Cheltuieli șterse cu succes',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Facturat',
      'logged': 'Înregistrat',
      'running': 'În derulare',
      'resume': 'Continuă',
      'task_errors': 'Te rog corectează suprapunerea timpilor',
      'start': 'Start',
      'stop': 'Stop',
      'started_task': 'Successfully started task',
      'stopped_task': 'Task oprit',
      'resumed_task': 'Successfully resumed task',
      'now': 'Acum',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Cronometru',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Timp pornire',
      'end_time': 'Timp încheiere',
      'date': 'Data',
      'times': 'Times',
      'duration': 'Durată',
      'new_task': 'Task nou',
      'created_task': 'Task creat',
      'updated_task': 'Task actualizat',
      'archived_task': 'Task arhivat',
      'deleted_task': 'Task șters',
      'restored_task': 'Task restaurat',
      'archived_tasks': 'Arhivat :count task-uri',
      'deleted_tasks': 'Șters :count task-uri',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Proiect nou',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'apasă aici',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Subsol',
      'compare': 'Compară',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Astăzi',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Curent',
      'previous': 'Anterior',
      'current_period': 'Perioada Curentă',
      'comparison_period': 'Perioada Comparării',
      'previous_period': 'Perioada Anterioară',
      'previous_year': 'Anul Anterior',
      'compare_to': 'Compară cu',
      'last7_days': 'Ultimele 7 Zile',
      'last_week': 'Săptămâna Trecută',
      'last30_days': 'Ultimele 30 Zile',
      'this_month': 'Luna curentă',
      'last_month': 'Luna trecută',
      'this_year': 'Anul Curent',
      'last_year': 'Anul Trecut',
      'custom': 'Personalizat',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Vizualizare Factură',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Modifica client',
      'edit_product': 'Modifica produs',
      'edit_invoice': 'Modifica factura',
      'edit_quote': 'Modifica Proforma',
      'edit_payment': 'Modifică Plata',
      'edit_task': 'Modifică Task',
      'edit_expense': 'Edit Expense',
      'edit_vendor': 'Editează Furnizor',
      'edit_project': 'Editează Proiect',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Editare oferta recurenta',
      'billing_address': 'Adresă de facturare',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Venituri Totale',
      'average_invoice': 'Medie facturi',
      'outstanding': 'Restante',
      'invoices_sent': ':count facturi trimise',
      'active_clients': 'clienti activi',
      'close': 'Inchide',
      'email': 'Email',
      'password': 'Parola',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Nume',
      'logout': 'Deconectare',
      'login': 'Autentificare',
      'filter': 'Filtreaza',
      'sort': 'Sort',
      'search': 'Cauta',
      'active': 'Activ',
      'archived': 'Arhivat',
      'deleted': 'Șters',
      'dashboard': 'Panou Control',
      'archive': 'Arhiva',
      'delete': 'Sterge',
      'restore': 'Restaurează',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Salveaza',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Plătit Pâna Acum',
      'balance_due': 'Total De Plată',
      'balance': 'Balanta',
      'overview': 'Overview',
      'details': 'Detalii',
      'phone': 'Telefon',
      'website': 'Site web',
      'vat_number': 'C.I.F.',
      'id_number': 'Nr. Reg. Com.',
      'create': 'Creaza',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Contacte',
      'additional': 'Additional',
      'first_name': 'Prenume',
      'last_name': 'Nume',
      'add_contact': 'Adauga contact',
      'are_you_sure': 'Sigur?',
      'cancel': 'Renunta',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produs',
      'products': 'Produse',
      'new_product': 'New Product',
      'created_product': 'Produs creat cu succes',
      'updated_product': 'Produs actualizat cu succes',
      'archived_product': 'Produs arhivat cu succes',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produs',
      'notes': 'Notițe',
      'cost': 'Cost',
      'client': 'Client',
      'clients': 'Clienti',
      'new_client': 'Client nou',
      'created_client': 'S-a creat clientul cu succes',
      'updated_client': 'Client actualizat cu succes.',
      'archived_client': 'Client arhivat cu succes.',
      'archived_clients': ':count clienti arhivat cu succes.',
      'deleted_client': 'Client sters cu succes.',
      'deleted_clients': ':count clienti stersi cu succes.',
      'restored_client': 'Client restaurat',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Strada',
      'address2': 'Apartament',
      'city': 'Localitate',
      'state': 'Județ/Sector',
      'postal_code': 'Cod poștal',
      'country': 'Tara',
      'invoice': 'Factură',
      'invoices': 'Facturi',
      'new_invoice': 'Factura noua',
      'created_invoice': 'Factura creata cu succes.',
      'updated_invoice': 'Factura actualiazata cu succes.',
      'archived_invoice': 'Factura arhivata cu succes.',
      'deleted_invoice': 'Factura stearsa cu succes.',
      'restored_invoice': 'Factură restaurată',
      'archived_invoices': ':count facturi arhivate cu succes.',
      'deleted_invoices': ':count facturi sterse cu succes',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Factura trimisa pe email cu succes',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Valoare',
      'invoice_number': 'Număr factură',
      'invoice_date': 'Data factură',
      'discount': 'Discount',
      'po_number': 'Ordin de cumpărare nr',
      'terms': 'Termeni',
      'public_notes': 'Public Notes',
      'private_notes': 'Note particulare',
      'frequency': 'Frecventa',
      'start_date': 'Data inceput',
      'end_date': 'Data sfirsit',
      'quote_number': 'Numar Proforma',
      'quote_date': 'Data Proforma',
      'valid_until': 'Valabil până la',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Descriere',
      'unit_cost': 'Preț unitar',
      'quantity': 'Cantitate',
      'add_item': 'Add Item',
      'contact': 'Contact',
      'work_phone': 'Telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Scadența',
      'partial_due_date': 'Partial Due Date',
      'status': 'Stare',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Status oferta',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Percent',
      'edit': 'Modifica',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Te rugam selecteaza o oferta',
      'task_rate': 'Task Rate',
      'settings': 'Setari',
      'language': 'Language',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Taxă',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Trimis',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Parțial/Depunere',
      'paid': 'Plătit',
      'mark_sent': 'Marchează ca trimis',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Gata',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Mod întunecat',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Reactualizeaza datele',
      'blank_contact': 'Contact gol',
      'activity': 'Activitate',
      'no_records_found': 'Nu exista inregistrari',
      'clone': 'Multiplică',
      'loading': 'Loading',
      'industry': 'Industrie',
      'size': 'Size',
      'payment_terms': 'Termeni de plată',
      'payment_date': 'Data platii',
      'payment_status': 'Status plata',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal Client',
      'show_tasks': 'Arata sarcini',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'Prima Notificare',
      'second_reminder': 'A Doua Notificare',
      'third_reminder': 'A Treia Notificare',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Șablon',
      'send': 'Trimite',
      'subject': 'Subiect',
      'body': 'Mesaj',
      'send_email': 'Trimite Email',
      'email_receipt': 'Trimite pe email dovada plății',
      'auto_billing': 'Auto billing',
      'button': 'Buton',
      'preview': 'Previzualizare',
      'customize': 'Personalizeaza',
      'history': 'Istoric',
      'payment': 'Plata',
      'payments': 'Plati',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Referinta tranzactie',
      'enter_payment': 'Introdu plata',
      'new_payment': 'Introdu plata',
      'created_payment': 'Plata creata cu succes.',
      'updated_payment': 'Plată actualizată',
      'archived_payment': 'Plata arhivata cu succes',
      'deleted_payment': 'Plata stearsa cu succes.',
      'restored_payment': 'Plată restaurată',
      'archived_payments': ':count plati arhivate cu succes',
      'deleted_payments': ':count plati sterse cu succes.',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Proforma',
      'quotes': 'Proforme',
      'new_quote': 'Proforma Nou',
      'created_quote': 'Proformă creată cu succes',
      'updated_quote': 'Proformă actualizată cu succes',
      'archived_quote': 'Proformă arhivată cu succes',
      'deleted_quote': 'Proformă ștearsă',
      'restored_quote': 'Proformă restaurată',
      'archived_quotes': ':count proforme arhivate cu succes',
      'deleted_quotes': ':count proforme șterse cu succes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Cheltuială',
      'expenses': 'Cheltuieli',
      'vendor': 'Furnizor',
      'vendors': 'Furnizori',
      'task': 'Task',
      'tasks': 'Task-uri',
      'project': 'Proiect',
      'projects': 'Proiecte',
      'activity_1': ':user a creat clientul :client',
      'activity_2': ':user a arhivat clientul :client',
      'activity_3': ':user a șters clientul :client',
      'activity_4': ':user a creat factura :invoice',
      'activity_5': ':user a actualizat factura :invoice',
      'activity_6':
          ':user a trimis pe email factura :invoice pentru :client la :contact',
      'activity_7': ':contact a vizualizat factura :invoice pentru :client',
      'activity_8': ':user a arhivat factura :invoice',
      'activity_9': ':user a șters factura :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user a actualizat plata :payment',
      'activity_12': ':user a arhivat plata :payment',
      'activity_13': ':user a șters plata :payment',
      'activity_14': ':user a încărcat :credit credite',
      'activity_15': ':user a actualizat :credit credite',
      'activity_16': ':user a arhivat :credit credite',
      'activity_17': ':user a șters :credit credite',
      'activity_18': ':user a creat proforma :quote',
      'activity_19': ':user a actualizat proforma :quote',
      'activity_20':
          ':user a trimis pe email proforma :quote pentru :client la :contact',
      'activity_21': ':contact a vizualizat proforma :quote',
      'activity_22': ':user a arhivat proforma :quote',
      'activity_23': ':user a șters proforma :quote',
      'activity_24': ':user a restaurat proforma :quote',
      'activity_25': ':user a restaurat factura :invoice',
      'activity_26': ':user a restaurat clientul :client',
      'activity_27': ':user a restaurat plata :payment',
      'activity_28': ':user a restaurat :credit credite',
      'activity_29': ':contact a aprobat proforma :quote pentru :client',
      'activity_30': ':user a creat furnizorul :vendor',
      'activity_31': ':user a arhivat furnizorul :vendor',
      'activity_32': ':user a șters furnizorul :vendor',
      'activity_33': ':user a restaurat furnizorul :vendor',
      'activity_34': ':user a creat cheltuiala :expense',
      'activity_35': ':user a arhivat cheltuiala :expense',
      'activity_36': ':user a șters cheltuiala :expense',
      'activity_37': ':user a restaurat cheltuiala :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Proformă trimisă cu succes',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Selecteaza',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Plajă număr factură',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Plajă număr proformă',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tip',
      'invoice_amount': 'Valoare Factură',
      'invoice_due_date': 'Data Scadenta',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto Facturare',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Valoare plata',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Log Timp',
      'bank_id': 'Banca',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'ru_RU': {
      'import_data': 'Импорт данных',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Последнее обновление',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Должно быть больше нуля и меньше, чем всего',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Включить',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Послать приглашение еще раз',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Конвертировать в счёт',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Включить в счет',
      'invoice_expense': 'Invoice Expense',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Документы по умолчанию',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Скрыть',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Столбец',
      'sample': 'Пример',
      'map_to': 'Map To',
      'import': 'Импорт',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Укажите файл',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV-файл',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Услуга',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Неоплачено',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Итого счёта',
      'quote_total': 'Всего',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Имя клиента',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Категория Затрат',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Категория Затрат успешно обновлена',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Should be invoiced',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Повторяющийся счет',
      'recurring_invoices': 'Повторяющиеся счета',
      'new_recurring_invoice': 'Создать повторяющийся счет',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Successfully archived recurring invoice',
      'deleted_recurring_invoice': 'Successfully deleted recurring invoice',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Successfully restored recurring invoice',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Сохранить данные карты',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Шлюз',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Часы',
      'statement': 'Statement',
      'taxes': 'Налоги',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Выбрать ярлык',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Кому',
      'health_check': 'Health Check',
      'payment_type_id': 'Способ оплаты',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Текущие счета',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Последние платежи',
      'upcoming_quotes': 'Входящие Прайс-листы',
      'expired_quotes': 'Expired Quotes',
      'create_client': 'Create Client',
      'create_invoice': 'Создать счёт',
      'create_quote': 'Создать котировку',
      'create_payment': 'Create Payment',
      'create_vendor': 'Создать поставщика',
      'update_quote': 'Update Quote',
      'delete_quote': 'Удалить котировку',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Удалить счёт',
      'update_client': 'Update Client',
      'delete_client': 'Удалить клиента',
      'delete_payment': 'Удалить оплату',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Delete Vendor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Удалить затраты',
      'create_task': 'Создать задание',
      'update_task': 'Update Task',
      'delete_task': 'Удалить задание',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Бесплатно',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Цель',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API токены',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Права',
      'tokens': 'Токены',
      'new_token': 'New Token',
      'edit_token': 'Изменить права',
      'created_token': 'Токен создан',
      'updated_token': 'Токен обновлён',
      'archived_token': 'Successfully archived token',
      'deleted_token': 'Токен успешно удалён',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Отправить счёт',
      'email_quote': 'Отправить прайс-лист',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Сумма кредита',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Полное имя',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Индекс/Город/Страна',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Информация о компании успешно удалена',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0-30 дней',
      'age_group_30': '30-60 дней',
      'age_group_60': '60-90 дней',
      'age_group_90': '90-120 дней',
      'age_group_120': '120+ дней',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Детали счёта',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count счет отправлен',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Удалить аккаунт',
      'cancel_account_message':
          'Внимание: это приведет к безвозвратному удалению вашего аккаунта.',
      'delete_company': 'Удалить компанию',
      'delete_company_message':
          'Внимание: это приведет к безвозвратному удалению вашего аккаунта.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Заголовок',
      'load_design': 'Загрузить шаблон',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Коммерческие предложения',
      'tickets': 'Тикеты',
      'recurring_quotes': 'Повторяющиеся Прайс-листы',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Дата кредита',
      'credit': 'Кредит',
      'credits': 'Кредиты',
      'new_credit': 'Добавить кредит',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Кредит успешно создан',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Кредит успешно архивирован',
      'deleted_credit': 'Кредит успешно удален',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Кредит восстановлен',
      'archived_credits': 'Успешно архивировано :count кредита(ов)',
      'deleted_credits': 'Успешно удалено :count кредита(ов)',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Текущая версия',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Узнать больше',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Новая компания',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Сбросить',
      'number': 'Number',
      'export': 'Экспорт',
      'chart': 'График',
      'count': 'Count',
      'totals': 'Итого',
      'blank': 'Blank',
      'day': 'День',
      'month': 'Месяц',
      'year': 'Год',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Группировать',
      'credit_balance': 'Баланс кредита',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Контактный номер',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Назначен',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Столбцы',
      'aging': 'Aging',
      'profit_and_loss': 'Прибыли и убытки',
      'reports': 'Reports',
      'report': 'Отчет',
      'add_company': 'Добавить компанию',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Оплаченные счета',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Помощь',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Почта для связи',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Сообщение',
      'from': 'От',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'Форум поддержки',
      'about': 'About',
      'documentation': 'Документация',
      'contact_us': 'Свяжитесь с нами',
      'subtotal': 'Промежуточный итог',
      'line_total': 'Всего',
      'item': 'Название',
      'credit_email': 'Credit Email',
      'iframe_url': 'Веб-сайт',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Yes',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Рабочий стол',
      'layout': 'Layout',
      'view': 'View',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Пользователь',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Настройки налогов',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Настройки',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Восстановить пароль',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Сумма пени',
      'late_fee_percent': 'Процент пени',
      'schedule': 'Schedule',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Дни',
      'invoice_email': 'Письмо',
      'payment_email': 'Платежный адрес эл. почты',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Quote Email',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Управление пользователями',
      'users': 'Пользователи',
      'new_user': 'New User',
      'edit_user': 'Редактировать',
      'created_user': 'Successfully created user',
      'updated_user': 'Профиль пользователя успешно обновлён',
      'archived_user': 'Successfully archived user',
      'deleted_user': 'Пользователь удалён',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Пользователь успешно восстановлен',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Общие настройки',
      'invoice_options': 'Настройки счета',
      'hide_paid_to_date': 'Скрыть \"Оплатить до\"',
      'hide_paid_to_date_help':
          'Показывать только «Оплатить до» в ваших счетах после получения платежа.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Основной цвет',
      'secondary_color': 'Второстепенный цвет',
      'page_size': 'Page Size',
      'font_size': 'Размер шрифта',
      'quote_design': 'Шаблон котировок',
      'invoice_fields': 'Поля счёта',
      'product_fields': 'Поля товара/услуги',
      'invoice_terms': 'Условия счёта',
      'invoice_footer': 'Нижний колонтитул счета',
      'quote_terms': 'Quote Terms',
      'quote_footer': 'Quote Footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'Еженедельно',
      'freq_two_weeks': 'Две недели',
      'freq_four_weeks': 'Четыре недели',
      'freq_monthly': 'Ежемесячно',
      'freq_two_months': 'Два месяца',
      'freq_three_months': 'Три месяца',
      'freq_four_months': 'Четыре месяца',
      'freq_six_months': 'Полгода',
      'freq_annually': 'Annually',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Компания',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Расчёт налогов',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Повторяющийся префикс',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Поле Компании',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Поле Счёта',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Поле товара/услуги',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Поле Поставщика',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Префикс',
      'number_pattern': 'Number Pattern',
      'messages': 'Сообщения',
      'custom_css': 'Пользовательские CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Поддомен',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'С Уважением,',
      'enable_email_markup_help':
          'Make it easier for your clients to pay you by adding schema.org markup to your emails.',
      'plain': 'Plain',
      'light': 'Light',
      'dark': 'Dark',
      'email_design': 'Email Design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Enable Markup',
      'reply_to_email': 'Ответить на сообщение',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Приоритет',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Обновить адрес',
      'update_address_help':
          'Обновить адрес клиента предоставленными сведениями',
      'rate': 'Ставка',
      'tax_rate': 'Налоговая ставка',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Изменить налоговую ставку',
      'created_tax_rate': 'Налоговая ставка создана',
      'updated_tax_rate': 'Налоговая ставка обновлена',
      'archived_tax_rate': 'Налоговая ставка добавлена в архив',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Автозаполнение товаров/услуг',
      'fill_products_help':
          'Выбор товара/услуги автоматически заполнит описание и стоимость',
      'update_products': 'Автообновление товаров/услуг',
      'update_products_help':
          'Обновление счета автоматически обновит библиотеку товаров/услуг',
      'convert_products': 'Конвертировать товар/услугу',
      'convert_products_help':
          'Автоматически конвертировать цену продукта в валюту клиента',
      'fees': 'Платы',
      'limits': 'Лимиты',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Disabled',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Воскресенье',
      'monday': 'Понедельник',
      'tuesday': 'Вторник',
      'wednesday': 'Среда',
      'thursday': 'Четверг',
      'friday': 'Пятница',
      'saturday': 'Суббота',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24-часовой формат',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Группа',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Логотип',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Настройки товара/услуги',
      'device_settings': 'Device Settings',
      'defaults': 'По умолчанию',
      'basic_settings': 'Основные настройки',
      'advanced_settings': 'Расширенные настройки',
      'company_details': 'Детали компании',
      'user_details': 'Данные пользователя',
      'localization': 'Локализация',
      'online_payments': 'Онлайн платежи',
      'tax_rates': 'Налоговые ставки',
      'notifications': 'Оповещения',
      'import_export': 'Импорт | Экспорт',
      'custom_fields': 'Настраиваемые поля',
      'invoice_design': 'Дизайн счёта',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Настройки эл. почты',
      'templates_and_reminders': 'Шаблоны и Напоминания',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Представление данных',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Условия использования',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Зарегистрироваться',
      'account_login': 'Логин',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Скачать',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Документы',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Дата Затрат',
      'pending': 'Ожидающий',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Преобразован',
      'add_documents_to_invoice': 'Добавить документы в Счёт',
      'exchange_rate': 'Exchange Rate',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Отметить оплаченным',
      'category': 'Категория',
      'address': 'Адрес',
      'new_vendor': 'Новый поставщик',
      'created_vendor': 'Поставщик успешно создан',
      'updated_vendor': 'Поставщик успешно обновлён',
      'archived_vendor': 'Поставщик успешно архивирован',
      'deleted_vendor': 'Поставщик успешно удалён',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Успешно архивировано :count поставщика(ов)',
      'deleted_vendors': 'Успешно удалено :count поставщика(ов)',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Добавить затраты',
      'created_expense': 'Successfully created expense',
      'updated_expense': 'Затраты успешно обновлены',
      'archived_expense': 'Successfully archived expense',
      'deleted_expense': 'Затраты успешно удалены',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Successfully archived expenses',
      'deleted_expenses': 'Затраты успешно удалены',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Выставлен счёт',
      'logged': 'Учтено',
      'running': 'Выполняется',
      'resume': 'Назад',
      'task_errors': 'Пожалуйста, исправьте перекрывающиеся интервалы',
      'start': 'Старт',
      'stop': 'Стоп',
      'started_task': 'Successfully started task',
      'stopped_task': 'Задание успешно остановлено',
      'resumed_task': 'Задание успешно возобновлено',
      'now': 'Сейчас',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Таймер',
      'manual': 'Руководство',
      'budgeted': 'Budgeted',
      'start_time': 'Время начала',
      'end_time': 'Время завершения',
      'date': 'Дата',
      'times': 'Время',
      'duration': 'Длительность',
      'new_task': 'Новое задание',
      'created_task': 'Задание успешно создано',
      'updated_task': 'Задание успешно обновлено',
      'archived_task': 'Задание перенесено в архив',
      'deleted_task': 'Задание удалено',
      'restored_task': 'Задание успешно восстановлено',
      'archived_tasks': 'Перенесено в архив :count заданий',
      'deleted_tasks': 'Удалено :count заданий',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Спасибо что используете наше приложение!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'нажмите сюда',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Неподтверждённые',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Заблокировано',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Нижний колонтитул',
      'compare': 'Сравнить',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Сегодня',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Текущий',
      'previous': 'Предыдущий',
      'current_period': 'Текущий период',
      'comparison_period': 'Период сравнения',
      'previous_period': 'Прошлый период',
      'previous_year': 'Преддуший год',
      'compare_to': 'Сравнить с',
      'last7_days': 'Последние 7 дней',
      'last_week': 'За последнюю неделю',
      'last30_days': 'Последние 30 дней',
      'this_month': 'Текущий месяц',
      'last_month': 'Прошлый месяц',
      'this_year': 'This Year',
      'last_year': 'Прошлый год',
      'custom': 'Настроить',
      'clone_to_invoice': 'Добавить в счёт',
      'clone_to_quote': 'Добавить в Прайс-лист',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Посмотреть счет',
      'convert': 'Конвертирован',
      'more': 'More',
      'edit_client': 'Изменить клиента',
      'edit_product': 'Редактировать товар/услугу',
      'edit_invoice': 'Изменить счёт',
      'edit_quote': 'Изменить котировку',
      'edit_payment': 'Редактировать оплату',
      'edit_task': 'Редактировать задание',
      'edit_expense': 'Редактировать затраты',
      'edit_vendor': 'Редактировать Поставщика',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Адрес для выставления счетов',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Совокупный доход',
      'average_invoice': 'Средний счёт',
      'outstanding': 'Исходящие',
      'invoices_sent': ':count счетов отправлено',
      'active_clients': 'активные клиенты',
      'close': 'Закрыть',
      'email': 'Эл. почта',
      'password': 'Пароль',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Название',
      'logout': 'Выйти',
      'login': 'Логин',
      'filter': 'Фильтр',
      'sort': 'Сортировка',
      'search': 'Искать',
      'active': 'Активный',
      'archived': 'Архив',
      'deleted': 'Удален',
      'dashboard': 'Главная',
      'archive': 'Архивировать',
      'delete': 'Удалить',
      'restore': 'Восстановить',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Пожалуйста, введите код товара/услуги',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Сохранить',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Оплачено',
      'balance_due': 'Неоплачено',
      'balance': 'К оплате',
      'overview': 'Overview',
      'details': 'Детали',
      'phone': 'Телефон',
      'website': 'Веб-сайт',
      'vat_number': 'регистрационный номер плательщика НДС',
      'id_number': 'Идентификационный номер',
      'create': 'Создать',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Ошибка',
      'could_not_launch': 'Could not launch',
      'contacts': 'Контакты',
      'additional': 'Additional',
      'first_name': 'Имя',
      'last_name': 'Фамилия',
      'add_contact': 'Добавить контакт',
      'are_you_sure': 'Вы уверены?',
      'cancel': 'Отмена',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Товар/услуга',
      'products': 'Товары/Услуги',
      'new_product': 'Новый товар/услуга',
      'created_product': 'Товар/услуга успешно создан',
      'updated_product': 'Информация о товаре/услуге успешно обновлёна',
      'archived_product': 'Товар/услуга перенесен в архив',
      'deleted_product': 'Товар/услуга успешно удалены',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Товар/услуга',
      'notes': 'Заметки',
      'cost': 'Cost',
      'client': 'Клиент',
      'clients': 'Клиенты',
      'new_client': 'Новый клиент',
      'created_client': 'Клиент успешно создан',
      'updated_client': 'Клиент успешно обновлён',
      'archived_client': 'Клиент успешно архивирован',
      'archived_clients': 'Успешно архивировано :count клиента(ов)',
      'deleted_client': 'Клиент успешно удалён',
      'deleted_clients': 'Успешно удалено :count клиента(ов)',
      'restored_client': 'Клиент восстановлен',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Улица',
      'address2': 'Дом/Офис',
      'city': 'Город',
      'state': 'Район',
      'postal_code': 'Почтовый код',
      'country': 'Страна',
      'invoice': 'Счёт',
      'invoices': 'Счета',
      'new_invoice': 'Новый счёт',
      'created_invoice': 'Счет успешно создан',
      'updated_invoice': 'Счет успешно обновлён',
      'archived_invoice': 'Счет успешно архивирован',
      'deleted_invoice': 'Счет успешно удалён',
      'restored_invoice': 'Счёт успешно восстановлен',
      'archived_invoices': 'Успешно архивировано :count счта(ов)',
      'deleted_invoices': 'Успешно удалено :count счта(ов)',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Счет успешно отправлен по почте',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Всего',
      'invoice_number': 'Номер счёта',
      'invoice_date': 'Дата счёта',
      'discount': 'Скидка',
      'po_number': 'Номер платежного поручения',
      'terms': 'Условия',
      'public_notes': 'Публичные заметки',
      'private_notes': 'Персональные заметки',
      'frequency': 'Периодичность',
      'start_date': 'Дата начала',
      'end_date': 'Дата завершения',
      'quote_number': 'Номер котировки',
      'quote_date': 'Дата котировки',
      'valid_until': 'Действителен до',
      'items': 'Items',
      'partial_deposit': 'Частичная оплата',
      'description': 'Описание',
      'unit_cost': 'Стоимость за единицу',
      'quantity': 'Количество',
      'add_item': 'Add Item',
      'contact': 'Контакт',
      'work_phone': 'Телефон',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Срок оплаты',
      'partial_due_date': 'Partial Due Date',
      'status': 'Статус',
      'invoice_status_id': 'Статус Счёта',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Всего',
      'percent': 'Процент',
      'edit': 'Редактировать',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Пожалуйста, выбирите дату',
      'please_select_a_client': 'Пожалуйста, выбирите клиента',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Настройки',
      'language': 'Язык',
      'currency': 'Валюта',
      'created_at': 'Дата создания',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Налог',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Черновик',
      'sent': 'Отправить',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Частичная оплата',
      'paid': 'Оплачено',
      'mark_sent': 'Отметить отправленным',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Готово',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Темная тема',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Обновить Данные',
      'blank_contact': 'Blank Contact',
      'activity': 'Активность',
      'no_records_found': 'No records found',
      'clone': 'Копировать',
      'loading': 'Загружается',
      'industry': 'Industry',
      'size': 'Размер',
      'payment_terms': 'Условия оплаты',
      'payment_date': 'Дата платежа',
      'payment_status': 'Статус Платежа',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Client Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Получатели',
      'initial_email': 'Initial Email',
      'first_reminder': 'Первое напоминание',
      'second_reminder': 'Второе напоминание',
      'third_reminder': 'Третье напоминание',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Шаблон',
      'send': 'Send',
      'subject': 'Тема',
      'body': 'Содержание',
      'send_email': 'Отправить эл.почту',
      'email_receipt': 'Отправлять клиенту квитанцию об оплате по эл.почте',
      'auto_billing': 'Auto billing',
      'button': 'Кнопка',
      'preview': 'Preview',
      'customize': 'Настроить',
      'history': 'История',
      'payment': 'Платёж',
      'payments': 'Оплаты',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Референс транзакции',
      'enter_payment': 'Добавить оплату',
      'new_payment': 'Добавить оплату',
      'created_payment': 'Платёж успешно создан',
      'updated_payment': 'Платёж успешно обновлён',
      'archived_payment': 'Платёж успешно архивирован',
      'deleted_payment': 'Платёж успешно удалён',
      'restored_payment': 'Платёж восстановлен',
      'archived_payments': 'Успешно архивировано :count платежа(ей)',
      'deleted_payments': 'Успешно удалено :count платежа(ей)',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Котировка',
      'quotes': 'Прайс-листы',
      'new_quote': 'Новая котировка',
      'created_quote': 'Прайс-лист успешно создан',
      'updated_quote': 'Прайс-лист успешно обновлён',
      'archived_quote': 'Прайс-лист перемещен в архив',
      'deleted_quote': 'Котировка успешно удалена',
      'restored_quote': 'Successfully restored quote',
      'archived_quotes': 'Перенесено в архив :count котировок',
      'deleted_quotes': 'Successfully deleted :count quotes',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Затрата',
      'expenses': 'Затраты',
      'vendor': 'Vendor',
      'vendors': 'Поставщики',
      'task': 'Task',
      'tasks': 'Задание',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user  Создал Клиента :client',
      'activity_2': ':user archived client :client',
      'activity_3': ':user deleted client :client',
      'activity_4': ':user Создал Счёт :invoice',
      'activity_5': ':user Обновил счёт :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user archived invoice :invoice',
      'activity_9': ':user deleted invoice :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user updated payment :payment',
      'activity_12': ':user archived payment :payment',
      'activity_13': ':user deleted payment :payment',
      'activity_14': ':user entered :credit credit',
      'activity_15': ':user updated :credit credit',
      'activity_16': ':user archived :credit credit',
      'activity_17': ':user deleted :credit credit',
      'activity_18': ':user created quote :quote',
      'activity_19': ':user updated quote :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact viewed quote :quote',
      'activity_22': ':user archived quote :quote',
      'activity_23': ':user deleted quote :quote',
      'activity_24': ':user restored quote :quote',
      'activity_25': ':user restored invoice :invoice',
      'activity_26': ':user restored client :client',
      'activity_27': ':user restored payment :payment',
      'activity_28': ':user restored :credit credit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user created expense :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user Удалил Затраты :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user Обновил Затраты :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Прайс-лист успешно отправлен',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Выбрать',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Счетчик номера счёта',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Quote Number Counter',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Тип',
      'invoice_amount': 'Invoice Amount',
      'invoice_due_date': 'Срок оплаты',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Авто-счет',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Tax Name',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Сумма платежа',
      'age': 'Возраст',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Категория затрат',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'sr_RS': {
      'import_data': 'Uvezi podatke',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Poslednja izmena',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Ime kontakta',
      'contact_last_name': 'Prezime kontakta',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Mora biti veće od nula i manje od sume',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Omogući',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Pretplata',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Izmeni Pretplatu',
      'created_subscription': 'Uspešno kreirana pretplata',
      'updated_subscription': 'Uspešno osvežena pretplata',
      'archived_subscription': 'Uspešno arhivirana pretplata',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Odbijene',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Očitajte bar kod uz pomoć :link kompatibilne aplikacije.',
      'enabled_two_factor': 'Dvostepena autorizacija uspešno aktivirana',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Dvostepena autorizacija',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Konvertuj u račun',
      'registration_url': 'Registration URL',
      'invoice_project': 'Naplati Projekat',
      'invoice_task': 'Fakturiši zadatak',
      'invoice_expense': 'Trošak računa',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Konvertovani iznos',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Podrazumevani dokumenti',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Sakrij',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolona',
      'sample': 'Uzorak',
      'map_to': 'Map To',
      'import': 'Uvoz',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Mollim odaberite datoteku',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV datoteka',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Usluga',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Otpremnica',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Račun sveukupno',
      'quote_total': 'Ukupno predračuna',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Upozorenje',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Ime klijenta',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Uspešno ažuriran status zadatka',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'Nova kategorija troškova',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Uspešno vraćena kategorija troškova',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Treba biti fakturisan',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Ponavljajući račun',
      'recurring_invoices': 'Ponavljajući računi',
      'new_recurring_invoice': 'Novi ponavljajući račun',
      'edit_recurring_invoice': 'Izmeni ponavljajući račun',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Uspešno arhiviran redovni račun',
      'deleted_recurring_invoice': 'Uspešno obrisan redovni račun',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Uspešno obnovljen redovni račun',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Otvorene',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Pogledaj portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Izmeni detalje kartice',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Oznaka',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Company Name',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Kanal',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Sati',
      'statement': 'Statement',
      'taxes': 'Porezi',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Prihvati',
      'unapplied': 'Unapplied',
      'select_label': 'Selektuj oznaku',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Prima',
      'health_check': 'Health Check',
      'payment_type_id': 'Tip uplate',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Dolazni računi',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nedavne uplate',
      'upcoming_quotes': 'Nadolazeći predračuni',
      'expired_quotes': 'Istekli predračuni',
      'create_client': 'Kreiraj klijenta',
      'create_invoice': 'Kreiraj račun',
      'create_quote': 'Kreiraj predračun',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Obriši predračun',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Obriši račun',
      'update_client': 'Update Client',
      'delete_client': 'Obriši klijenta',
      'delete_payment': 'Obriši uplatu',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Obriši dobavljača',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Obriši trošak',
      'create_task': 'Kreiraj zadatak',
      'update_task': 'Update Task',
      'delete_task': 'Obriši zadatak',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Slobodan',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Cilj',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API tokeni',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokeni',
      'new_token': 'New Token',
      'edit_token': 'Izmeni token',
      'created_token': 'Uspešno kreiran token',
      'updated_token': 'Uspešno ažuriran token',
      'archived_token': 'Uspešno arhiviran token',
      'deleted_token': 'Uspešno obrisan token',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Pošalji e-poštom',
      'email_quote': 'Šalji predračun e-poštom',
      'email_credit': 'Email Credit',
      'email_payment': 'Pošalji uplatu ePoštom',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Uredi uslove plaćanja',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Iznos kredita',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'Prvo prilagođeno',
      'custom2': 'Drugo prilagođeno',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Uspešno očišćeni podatci kompanije',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Osveži',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'Nema',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count račun poslat',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Izbriši nalog',
      'cancel_account_message':
          'Upozorenje: Ovo će trajno izbrisati vaš nalog.',
      'delete_company': 'Izbriši kompaniju',
      'delete_company_message':
          'Upozorenje: Ovo će potpuno obrisati podatke o Vašooj komplaniji, nema mogućnosti povratka podataka.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Zaglavlje',
      'load_design': 'Učitaj Dizajn',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Ponude',
      'tickets': 'Tiketi',
      'recurring_quotes': 'Ponavljajuća ponuda',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Redovni troškovi',
      'account_management': 'Account Management',
      'credit_date': 'Datum kredita',
      'credit': 'Kredit',
      'credits': 'Krediti',
      'new_credit': 'Unesi kredit',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Uspešno kreiran kredit',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Uspešno arhiviran kredit',
      'deleted_credit': 'Uspešno obrisan kredit',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Uspešno vraćen kredit',
      'archived_credits': 'Uspešno arhivirano :count kredita',
      'deleted_credits': 'Uspešno obrisano :count kredita',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Trenutna verzija',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Više informacija',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Novo preduzeće',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Resetuj',
      'number': 'Number',
      'export': 'Izvoz',
      'chart': 'Karte',
      'count': 'Count',
      'totals': 'Totali',
      'blank': 'Blank',
      'day': 'Dan',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Podgrupa',
      'is_active': 'Is Active',
      'group_by': 'Grupiši po',
      'credit_balance': 'Stanje kredita',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Adresa za slanje ulica',
      'shipping_address2': 'Adresa za slanje stan/apartman',
      'shipping_city': 'Adresa za slanje grad',
      'shipping_state': 'Adresa za slanje - Provincija/Pokrajina',
      'shipping_postal_code': 'Adresa za slanje poštanski broj',
      'shipping_country': 'Adresa za slanje država',
      'billing_address1': 'Adresa naplate - Ulica',
      'billing_address2': 'Adresa računa - Stan/Spartman',
      'billing_city': 'Adresa naplate - Grad',
      'billing_state': 'Adresa naplate - Provincija/Pokrajina',
      'billing_postal_code': 'Adresa naplate - Poštanski broj',
      'billing_country': 'Adresa naplate - Država',
      'client_id': 'Client Id',
      'assigned_to': 'Dodeljeno za',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Izveštaji',
      'add_company': 'Dodaj preduzeće',
      'unpaid_invoice': 'Neplaćeni računi',
      'paid_invoice': 'Plaćeni računi',
      'unapproved_quote': 'Ne odobrene ponude',
      'help': 'Pomoć',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filter po',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'State',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Poruka',
      'from': 'Šalje',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'forum podrške',
      'about': 'About',
      'documentation': 'Dokumentacija',
      'contact_us': 'Contact Us',
      'subtotal': 'Sveukupno',
      'line_total': 'Ukupno',
      'item': 'Stavka',
      'credit_email': 'Credit Email',
      'iframe_url': 'Sajt',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Lozinka je prekratka',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Da',
      'no': 'Ne',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobilni',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Pregled',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Korisnik',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Molimo odaberite klijenta',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Podešavanja poreza',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Opcije',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Povratite vašu lozinku',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Iznos honorara',
      'late_fee_percent': 'Procenat honorara',
      'schedule': 'Raspored',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dani',
      'invoice_email': 'E-pošta računa',
      'payment_email': 'E-pošta uplate',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'E-pošta ponuda',
      'endless_reminder': 'Beskrajni podsetnik',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Upravljanje korisnicima',
      'users': 'Korisnici',
      'new_user': 'New User',
      'edit_user': 'Uredi korisnika',
      'created_user': 'Successfully created user',
      'updated_user': 'Korisnik je uspešno ažuriran',
      'archived_user': 'Uspešno arhiviran korisnik',
      'deleted_user': 'Korisnik je uspešno obrisan',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Uspešno obnovljen korisnik',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Opšte postavke',
      'invoice_options': 'Opcije računa',
      'hide_paid_to_date': 'Sakrij datum plaćanja',
      'hide_paid_to_date_help':
          'Prikažite \"Datum plaćanja\" na računima, onda kada je uplata primljena.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Prikaži zaglavlje na',
      'all_pages_footer': 'Prikaži podnožje na',
      'first_page': 'First page',
      'all_pages': 'All pages',
      'last_page': 'Last page',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primarna boja',
      'secondary_color': 'Sekundarna boja',
      'page_size': 'Page Size',
      'font_size': 'Veličina fonta',
      'quote_design': 'Dizajn ponude',
      'invoice_fields': 'Polja računa',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Uslovi računa',
      'invoice_footer': 'Podnožje računa',
      'quote_terms': 'Uslovi predračuna',
      'quote_footer': 'Podnožje ponude',
      'auto_email_invoice': 'Automatsko slanje ePošte',
      'auto_email_invoice_help':
          'Automatski pošalji ponavljajuće račune u momentu kreiranja.',
      'auto_archive_invoice': 'AAutomatsko arhiviranje',
      'auto_archive_invoice_help':
          'Automatski arhiviraj račune kada su plaćeni.',
      'auto_archive_quote': 'Automatsko Arhiviranje',
      'auto_archive_quote_help':
          'Automatski arhiviraj ponude kada su konvertovane.',
      'auto_convert_quote': 'Auto konverzija',
      'auto_convert_quote_help':
          'Automatski konvertujte ponudu u račun nakon što je odobrena od strane klijenta.',
      'workflow_settings': 'Podešavanje toka rada',
      'freq_daily': 'Svakodnevno',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Two weeks',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Monthly',
      'freq_two_months': 'Two months',
      'freq_three_months': 'Three months',
      'freq_four_months': 'Četiri meseca',
      'freq_six_months': 'Six months',
      'freq_annually': 'Annually',
      'freq_two_years': 'Dve godine',
      'freq_three_years': 'Three Years',
      'never': 'Nikada',
      'company': 'Kompanija',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Naplati poreze',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Prefiks koji se ponavlja',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Polje kompanija',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Polje računa',
      'invoice_surcharge': 'Višak Računa',
      'client_field': 'Polje klijenta',
      'product_field': 'Polje proizvod',
      'payment_field': 'Payment Field',
      'contact_field': 'Polje kontakt',
      'vendor_field': 'Polje dobavljača',
      'expense_field': 'Polje troškova',
      'project_field': 'Polje Projekta',
      'task_field': 'Polje zadatak',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefiks',
      'number_pattern': 'Number Pattern',
      'messages': 'Poruke',
      'custom_css': 'Prilagođeni CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Prikaži u PDF-u',
      'signature_on_pdf_help':
          'Prikazite potpis klijenta na PDF-u računa/ponude.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Poddomena',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Sa poštovanjem,',
      'enable_email_markup_help':
          'Olakšajte svojim klijentima plaćanje dodavanjem schema.org markupa vašoj e-pošti.',
      'plain': 'Obično',
      'light': 'Svetlo',
      'dark': 'Tamno',
      'email_design': 'Dizajn e-pošte',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Omogući markup',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Prioritet',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Logoi Prihvaćenih Kartica',
      'credentials': 'Credentials',
      'update_address': 'Ažuriraj adresu',
      'update_address_help':
          'Ažuriraj adresu klijenta uz dostavljenim detaljima',
      'rate': 'Stopa',
      'tax_rate': 'Poreska stopa',
      'new_tax_rate': 'New Tax Rate',
      'edit_tax_rate': 'Uredi poresku stopu',
      'created_tax_rate': 'Uspešno kreirana poreska stopa',
      'updated_tax_rate': 'Uspešno ažurirana poreska stopa',
      'archived_tax_rate': 'Uspešno arhivirana poreska stopa',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Proizvodi sa samoispunom',
      'fill_products_help':
          'Odabir proizvoda će automatski ispuniti opis i cijenu',
      'update_products': 'Proizvodi sa autoažuriranjem',
      'update_products_help':
          'Ažuriranje računa automatski ažurirati registar proizvoda',
      'convert_products': 'Konvertuj proizvode',
      'convert_products_help':
          'Automatski konvertuj cene proizvoda u valutu klijenta',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Otkaži izmene',
      'default_value': 'Default value',
      'disabled': 'Onemogućeno',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'Prvi dan u nedelji',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Nedelja',
      'monday': 'Ponedeljak',
      'tuesday': 'Utorak',
      'wednesday': 'Sreda',
      'thursday': 'Četvrtak',
      'friday': 'Petak',
      'saturday': 'Subota',
      'january': 'January',
      'february': 'February',
      'march': 'March',
      'april': 'April',
      'may': 'May',
      'june': 'June',
      'july': 'July',
      'august': 'August',
      'september': 'September',
      'october': 'October',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 satno vreme',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Grupa',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Postavke proizvoda',
      'device_settings': 'Device Settings',
      'defaults': 'Podrazumevano',
      'basic_settings': 'Osnovna podešavanja',
      'advanced_settings': 'Napredne postavke',
      'company_details': 'Detalji preduzeća',
      'user_details': 'Detalji korisnika',
      'localization': 'Lokalizacija',
      'online_payments': 'Online uplate',
      'tax_rates': 'Porezne stope',
      'notifications': 'Obaveštenja',
      'import_export': 'Uvoz i Izvoz',
      'custom_fields': 'Prilagođena polja',
      'invoice_design': 'Dizajn računa',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Postavke e-pošte',
      'templates_and_reminders': 'Šabloni & podsetnici',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Vizualizacije podataka',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Hvala Vam na kupovini!',
      'redeem': 'Redeem',
      'back': 'Nazad',
      'past_purchases': 'Ranije kupovine',
      'annual_subscription': 'Godišnja pretplata',
      'pro_plan': 'Pro plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count korisnika',
      'upgrade': 'Nadogradi',
      'please_enter_a_first_name': 'Unesite ime',
      'please_enter_a_last_name': 'Unesite prezime',
      'please_agree_to_terms_and_privacy':
          'Molimo Vas prihvatite uslove korišćenja i politiku privatnosti da biste registrovali korisnički nalog.',
      'i_agree_to_the': 'Slažem se sa',
      'terms_of_service_link': 'uslovima korišćenja',
      'privacy_policy_link': 'politikom privatnosti',
      'terms_of_service': 'Uslovi korištenja usluge',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Prijava',
      'account_login': 'Korisnička prijava',
      'view_website': 'Poseti web sajt',
      'create_account': 'Registruj nalog',
      'email_login': 'Prijavite se emailom',
      'create_new': 'Kreiraj novi',
      'no_record_selected': 'Nijedan zapis nije odabran',
      'error_unsaved_changes': 'Molimo Vas sačuvajte ili otkažite izmene',
      'download': 'Preuzmi',
      'requires_an_enterprise_plan': 'Zahteva Enterprise plan',
      'take_picture': 'Fotografiši',
      'upload_file': 'Pošalji fajl',
      'document': 'Dokument',
      'documents': 'Dokumenti',
      'new_document': 'Novi dokument',
      'edit_document': 'Izmeni dokument',
      'uploaded_document': 'Uspešno poslat dokument',
      'updated_document': 'Uspešno ažuriran dokument',
      'archived_document': 'Uspešno arhiviran dokument',
      'deleted_document': 'Uspešno obrisan dokument',
      'restored_document': 'Uspešno vraćen dokument',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'Nema istorije',
      'expense_date': 'Datum troška',
      'pending': 'Na čekanju',
      'expense_status_1': 'Prijavljen',
      'expense_status_2': 'Na čekanju',
      'expense_status_3': 'Fakturisano',
      'converted': 'Konvertovano',
      'add_documents_to_invoice': 'Dodaj dokumente uz Račun',
      'exchange_rate': 'Kurs',
      'convert_currency': 'Konvertuj valutu',
      'mark_paid': 'Mark Paid',
      'category': 'Category',
      'address': 'Adresa',
      'new_vendor': 'Novi dobavljač',
      'created_vendor': 'Uspešno kreiran dobavljač',
      'updated_vendor': 'Uspešno ažuriran dobavljač',
      'archived_vendor': 'Uspešno arhiviran dobavljač',
      'deleted_vendor': 'Uspešno obrisan dobavljač',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': 'Uspešno arhivirano :count dobavljača',
      'deleted_vendors': 'Uspešno obrisano :count dobavljača',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Unesi trošak',
      'created_expense': 'Uspešno kreiran trošak',
      'updated_expense': 'Uspešno ažuriran trošak',
      'archived_expense': 'Uspešno arhiviran trošak',
      'deleted_expense': 'Uspešno obrisan trošak',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Uspešno arhivirani troškovi',
      'deleted_expenses': 'Uspešno obrisan trošak',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Kopiraj adresu za slanje',
      'copy_billing': 'Kopiraj adresu za naplatu',
      'design': 'Dizajn',
      'failed_to_find_record': 'Zapis nije pronađen',
      'invoiced': 'Fakturisano',
      'logged': 'Logovano',
      'running': 'Pokrenuto',
      'resume': 'Nastavi',
      'task_errors': 'Molimo korigujte vremena koja se poklapaju',
      'start': 'Početak',
      'stop': 'Završetak',
      'started_task': 'Uspešno pokrenut zadatak',
      'stopped_task': 'Uspešno završen zadatak',
      'resumed_task': 'Uspešno nastavljen zadatak',
      'now': 'Sada',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Štoperica',
      'manual': 'Ručno',
      'budgeted': 'Budgeted',
      'start_time': 'Početno vreme',
      'end_time': 'Vreme završetka',
      'date': 'Datum',
      'times': 'Vremena',
      'duration': 'Trajanje',
      'new_task': 'Novi zadatak',
      'created_task': 'Uspešno kreiran zadatak',
      'updated_task': 'Uspešno ažuriran zadatak',
      'archived_task': 'Uspešno arhiviran zadatak',
      'deleted_task': 'Uspešno obrisan zadatak',
      'restored_task': 'Uspešno obnovljen zadatak',
      'archived_tasks': 'Uspešno arhivirano :count zadataka',
      'deleted_tasks': 'Uspešno obrisano :count zadataka',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Unesite Vaše ime',
      'budgeted_hours': 'Budžetirani sati',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Uspešno vraćen projekat',
      'archived_projects': 'Uspešno arhivirano :count projekata',
      'deleted_projects': 'Uspešno obrisano :count projekata',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Hvala Vam što koristite našu aplikaciju!',
      'if_you_like_it': 'Ako Vam se dopada molimo Vas',
      'click_here': 'kliknite ovde',
      'click_here_capital': 'Click here',
      'to_rate_it': 'da je ocenite.',
      'average': 'Prosek',
      'unapproved': 'Neodobreno',
      'authenticate_to_change_setting':
          'Molimo Vas auotorizujte se da biste promenili ovu opciju',
      'locked': 'Zaključano',
      'authenticate': 'Autorizuj se',
      'please_authenticate': 'Molimo Vas da se autorizujete',
      'biometric_authentication': 'Biometrijska autorizacija',
      'footer': 'Podnožje',
      'compare': 'Uporedi',
      'hosted_login': 'Hostovan login',
      'selfhost_login': 'Samohostovan login',
      'google_sign_in': 'Login with Google',
      'today': 'Danas',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Tekući',
      'previous': 'Prethodni',
      'current_period': 'Tekući period',
      'comparison_period': 'Period za upoređivanje',
      'previous_period': 'Prethodni period',
      'previous_year': 'Sledeći period',
      'compare_to': 'Uporedi sa',
      'last7_days': 'Poslednjih 7 dana',
      'last_week': 'Poslednja sedmica',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Prilagođeno',
      'clone_to_invoice': 'Kloniraj u račun',
      'clone_to_quote': 'Kloniraj u ponudu',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Pregled računa',
      'convert': 'Konvertuj',
      'more': 'Više',
      'edit_client': 'Uredi klijenta',
      'edit_product': 'Uredi proizvod',
      'edit_invoice': 'Uredi račun',
      'edit_quote': 'Uredi predračun',
      'edit_payment': 'Izmeni uplatu',
      'edit_task': 'Uredi zadatak',
      'edit_expense': 'Izmeni trošak',
      'edit_vendor': 'Uredi dobavljača',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Izmena redovnih troškova',
      'edit_recurring_quote': 'Izmeni ponavljajuću ponudu',
      'billing_address': 'Adresa računa',
      'shipping_address': 'Adresa za slanje',
      'total_revenue': 'Ukupni prihod',
      'average_invoice': 'Prosečni račun',
      'outstanding': 'Nenaplaćeno',
      'invoices_sent': ':count račun poslat',
      'active_clients': 'aktivni klijenti',
      'close': 'Zatvori',
      'email': 'E-pošta',
      'password': 'Lozinka',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Ime',
      'logout': 'Odjava',
      'login': 'Prijava',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Pretraga',
      'active': 'Aktivan',
      'archived': 'Arhivirano',
      'deleted': 'Obrisano',
      'dashboard': 'Kontrolna tabla',
      'archive': 'Arhiva',
      'delete': 'Obriši',
      'restore': 'Vrati',
      'refresh_complete': 'Osvežavanje završeno',
      'please_enter_your_email': 'Unesite adresu e-pošte',
      'please_enter_your_password': 'Unesite lozinku',
      'please_enter_your_url': 'Unesite web adresu',
      'please_enter_a_product_key': 'Unesite šifru proizvoda',
      'ascending': 'Rastuće',
      'descending': 'Opadajuće',
      'save': 'Snimi',
      'an_error_occurred': 'Desila se greška',
      'paid_to_date': 'Plaćeno na vreme',
      'balance_due': 'Stanje duga',
      'balance': 'Stanje',
      'overview': 'Pregled',
      'details': 'Detalji',
      'phone': 'Telefon',
      'website': 'Web adresa',
      'vat_number': 'PIB',
      'id_number': 'Matični broj',
      'create': 'Kreiraj',
      'copied_to_clipboard': 'Sadržaj :value kopiran u klipbord',
      'error': 'Greška',
      'could_not_launch': 'Nije moguće pokrenuti',
      'contacts': 'Kontakti',
      'additional': 'Dodatno',
      'first_name': 'Ime',
      'last_name': 'Prezime',
      'add_contact': 'Dodaj kontakt',
      'are_you_sure': 'Da li ste sigurni?',
      'cancel': 'Odustani',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Adresa e-pošte nije validna',
      'product': 'Proizvod',
      'products': 'Proizvodi',
      'new_product': 'New Product',
      'created_product': 'Proizvod je uspešno kreiran',
      'updated_product': 'Proizvod je uspešno ažuriran',
      'archived_product': 'Proizvod je uspešno arhiviran',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Uspešno vraćen proizvod',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Product',
      'notes': 'Beleške',
      'cost': 'Cost',
      'client': 'Klijent',
      'clients': 'Klijenti',
      'new_client': 'Novi klijent',
      'created_client': 'Klijent je uspešno kreiran',
      'updated_client': 'Uspešno ažuriranje klijenta',
      'archived_client': 'Uspešno arhiviran klijent',
      'archived_clients': 'Uspešno arhivirano :count klijenata',
      'deleted_client': 'Uspešno obrisan klijent',
      'deleted_clients': 'Uspešno obrisano :count klijenata',
      'restored_client': 'Uspešno vraćen klijent',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Ulica',
      'address2': 'Sprat/soba',
      'city': 'Grad',
      'state': 'Okrug',
      'postal_code': 'Poštanski broj',
      'country': 'Zemlja',
      'invoice': 'Račun',
      'invoices': 'Računi',
      'new_invoice': 'Novi račun',
      'created_invoice': 'Uspešno kreiran račun',
      'updated_invoice': 'Uspešno ažuriran račun',
      'archived_invoice': 'Uspešno arhiviran račun',
      'deleted_invoice': 'Uspešno obrisan račun',
      'restored_invoice': 'Uspešno vraćen račun',
      'archived_invoices': 'Uspešno arhivirano :count računa',
      'deleted_invoices': 'Uspešno obrisano :count računa',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Račun uspešno poslat e-poštom',
      'emailed_payment': 'Uplata uspešno poslata putem elektronske pošte',
      'amount': 'Iznos',
      'invoice_number': 'Broj računa',
      'invoice_date': 'Datum računa',
      'discount': 'Popust',
      'po_number': 'Broj narudžbe',
      'terms': 'Uslovi',
      'public_notes': 'Javne beleške',
      'private_notes': 'Privatne beleške',
      'frequency': 'Frekvencija',
      'start_date': 'Početni datum',
      'end_date': 'Završni datum',
      'quote_number': 'Broj ponude',
      'quote_date': 'Datum ponude',
      'valid_until': 'Vredi do',
      'items': 'Stavke',
      'partial_deposit': 'Avans/Depozit',
      'description': 'Opis',
      'unit_cost': 'Jedinična cena',
      'quantity': 'Količina',
      'add_item': 'Dodaj stavku',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Ukupan iznos',
      'pdf': 'PDF',
      'due_date': 'Datum dospeća',
      'partial_due_date': 'Datum dospeća avansa',
      'status': 'Status',
      'invoice_status_id': 'Status računa',
      'quote_status': 'Status ponude',
      'click_plus_to_add_item': 'Kliknite + za dodavanje stavke',
      'click_plus_to_add_time': 'Kliknite + za dodavanje vremena',
      'count_selected': ':count selektovano',
      'total': 'Sveukupno',
      'percent': 'Percent',
      'edit': 'Uredi',
      'dismiss': 'Odbaci',
      'please_select_a_date': 'Izaberite datum',
      'please_select_a_client': 'Izaberite klijenta',
      'please_select_an_invoice': 'Izaberite račun',
      'task_rate': 'Stopa zadatka',
      'settings': 'Postavke',
      'language': 'Jezik',
      'currency': 'Valuta',
      'created_at': 'Datum kreiranja',
      'created_on': 'Created On',
      'updated_at': 'Ažurirano',
      'tax': 'Porez',
      'please_enter_an_invoice_number': 'Unesite broj računa',
      'please_enter_a_quote_number': 'Unesite broj ponude',
      'past_due': 'Van valute',
      'draft': 'Draft',
      'sent': 'Poslato',
      'viewed': 'Pregledano',
      'approved': 'Odobreno',
      'partial': 'Partial/Deposit',
      'paid': 'Plaćeno',
      'mark_sent': 'Označi kao poslato',
      'marked_invoice_as_sent': 'Račun uspešno obeležen kao poslat',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Završeno',
      'please_enter_a_client_or_contact_name':
          'Unesite klijenta ili ime kontakta',
      'dark_mode': 'Tamni prikaz',
      'restart_app_to_apply_change':
          'Restartuje aplikaciju za aktiviranje izmene',
      'refresh_data': 'Osveži podatke',
      'blank_contact': 'Prazan kontakt',
      'activity': 'Aktivnost',
      'no_records_found': 'Nema zapisa',
      'clone': 'Kloniraj',
      'loading': 'Učitavanje',
      'industry': 'Delatnost',
      'size': 'Veličina',
      'payment_terms': 'Uslovi plaćanja',
      'payment_date': 'Datum uplate',
      'payment_status': 'Status plaćanja',
      'payment_status_1': 'Na čekanju',
      'payment_status_2': 'Storno',
      'payment_status_3': 'Neuspešno',
      'payment_status_4': 'Završeno',
      'payment_status_5': 'Delimično povraćeno',
      'payment_status_6': 'Povraćeno',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Čisto',
      'client_portal': 'Klijentski portal',
      'show_tasks': 'Prikaži zadatke',
      'email_reminders': 'Podsetnici e-poštom',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'Prvi podsetnik',
      'second_reminder': 'Drugi podsetnik',
      'third_reminder': 'Treći podsetnik',
      'reminder1': 'Prvi podsetnik',
      'reminder2': 'Drugi podsetnik',
      'reminder3': 'Treći podsetnik',
      'template': 'Šablon',
      'send': 'Pošalji',
      'subject': 'Naslov',
      'body': 'Telo',
      'send_email': 'Send Email',
      'email_receipt': 'Pošalji e-poštom račun klijentu',
      'auto_billing': 'Automatski račun',
      'button': 'Dugme',
      'preview': 'Preview',
      'customize': 'Prilagodi',
      'history': 'Istorija',
      'payment': 'Uplata',
      'payments': 'Uplate',
      'refunded': 'Povraćeno',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Referenca transakcije',
      'enter_payment': 'Unesi uplatu',
      'new_payment': 'Unesi plaćanje',
      'created_payment': 'Uspešno kreirana uplata',
      'updated_payment': 'Uspešno ažurirana uplata',
      'archived_payment': 'Uspešno arhivirana uplata',
      'deleted_payment': 'Uspešno obrisana uplata',
      'restored_payment': 'Uspešno vraćena uplata',
      'archived_payments': 'Uspešno arhivirana :count uplata',
      'deleted_payments': 'Uspešno obrisano :count uplata',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Ponuda',
      'quotes': 'Ponude',
      'new_quote': 'Novi predračun',
      'created_quote': 'Predračun je uspešno kreiran',
      'updated_quote': 'Predračun je uspešno ažuriran',
      'archived_quote': 'Predračun je uspešno arhiviran',
      'deleted_quote': 'Predračun uspešno obrisan',
      'restored_quote': 'Uspešno vraćen predračun',
      'archived_quotes': 'Uspešno arhivirano :count predračuna',
      'deleted_quotes': 'Uspešno obrisano :count predračuna',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Trošak',
      'expenses': 'Troškovi',
      'vendor': 'Dobavljač',
      'vendors': 'Dobavljači',
      'task': 'Task',
      'tasks': 'Zadaci',
      'project': 'Project',
      'projects': 'Projekti',
      'activity_1': ':user kreirao klijenta :client',
      'activity_2': ':user arhivirao klijenta :client',
      'activity_3': ':user obrisao klijenta :client',
      'activity_4': ':user kreirao račun :invoice',
      'activity_5': ':user ažurirao račun :invoice',
      'activity_6':
          ':user je poslao račun :invoice za :client kontaktu :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user arhivirao račun :invoice',
      'activity_9': ':user obrisao račun :invoce',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user ažurirao uplatu :payment',
      'activity_12': ':user ahivirao uplatu :payment',
      'activity_13': ':user obrisao uplatu :payment',
      'activity_14': ':user upisao :credit kredit',
      'activity_15': ':user ažurirao :credit kredit',
      'activity_16': ':user arhivirao :credit kredit',
      'activity_17': ':user obrisao :credit kredit',
      'activity_18': ':user kreirao predračun :quote',
      'activity_19': ':user ažurirao predračun :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact pregledao predračun :quote',
      'activity_22': ':user arhivirao predračun :quote',
      'activity_23': ':user obrisao predračun :quote',
      'activity_24': ':user obnovio predračun :quote',
      'activity_25': ':user obnovio račun :invoice',
      'activity_26': ':user obnovio klijenta :client',
      'activity_27': ':user obnovio uplatu :payment',
      'activity_28': ':user obnovio :credit kredit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user created vendor :vendor',
      'activity_31': ':user archived vendor :vendor',
      'activity_32': ':user deleted vendor :vendor',
      'activity_33': ':user restored vendor :vendor',
      'activity_34': ':user kreirao trošak :expense',
      'activity_35': ':user archived expense :expense',
      'activity_36': ':user deleted expense :expense',
      'activity_37': ':user restored expense :expense',
      'activity_39': ':user je otkazao :payment_amount plaćanje :payment',
      'activity_40':
          ':user vratio :adjustment od :payment_amount plaćanja :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user created task :task',
      'activity_43': ':user updated task :task',
      'activity_44': ':user archived task :task',
      'activity_45': ':user deleted task :task',
      'activity_46': ':user restored task :task',
      'activity_47': ':user updated expense :expense',
      'activity_48': ':user ažurirao tiket :ticket',
      'activity_49': ':user zatvorio tiket :ticket',
      'activity_50': ':user spojio tiket :ticket',
      'activity_51': ':user podelio tiket :ticket',
      'activity_52': ':contact otvorio tiket :ticket',
      'activity_53': ':contact obnovio tiket :ticket',
      'activity_54': ':user obnovio tiket :ticket',
      'activity_55': ':contact odgovorio na tiket :ticket',
      'activity_56': ':user pogledao tiket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Jednokratna lozinka',
      'emailed_quote': 'Predračun je uspešno poslan e-poštom',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Ponuda uspešno obeležena kao poslata',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Odaberi',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Prilagođena Vrednost',
      'custom_value2': 'Prilagođena Vrednost',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Brojač računa',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Brojač ponuda',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tip',
      'invoice_amount': 'Iznos računa',
      'invoice_due_date': 'Datum valute',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto naplata',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Naziv poreske stope',
      'tax_amount': 'Iznos poreza',
      'tax_paid': 'Porez Plaćen',
      'payment_amount': 'Iznos uplate',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Vremenski zapisi',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'sl': {
      'import_data': 'Uvozi podatke',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Ime kontakta',
      'contact_last_name': 'Priimek konakta',
      'order': 'Order',
      'unassigned': 'Ne-dodeljen',
      'partial_value': 'Mora biti večje od nič in manjše od celotnega zneska',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Omogoči',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Nastavite pod domeno ali prikažite račun na vaši spletni strani.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Varnost',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Naročnina',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Uredi naročnino',
      'created_subscription': 'Naročnina uspešno ustvarjena',
      'updated_subscription': 'Naročnina uspešno posodobljena',
      'archived_subscription': 'Naročnina uspešno arhivirana',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Znova pošlji vabilo',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Skenirajte barkodo s aplikacijo kot na primer :link. Spodaj vnesite prvo generirano geslo za enkratno rabo.',
      'enabled_two_factor': 'Dvostopenjska avtentikacija je omogočena',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Dvostopenjska avtentikacija',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Vrnjeno plačilo',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Prejšnje četrtletje',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Pretvori v račun',
      'registration_url': 'Registration URL',
      'invoice_project': 'Fakturiraj projekt',
      'invoice_task': 'Fakturiraj opravilo',
      'invoice_expense': 'Strošek računa',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Pretvorjeni znesek',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Privzeti dokumenti',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Skrij',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Stolpec',
      'sample': 'Vzorec',
      'map_to': 'Map To',
      'import': 'Uvozi',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Prosim izberi datoteko',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV datoteka',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Storitev',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Neplačano',
      'white_label': 'White Label',
      'delivery_note': 'Dobavnica',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Delno plačilo do',
      'invoice_total': 'Znesek',
      'quote_total': 'Znesek predračuna',
      'credit_total': 'Dobropis skupaj',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Opozorilo',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Ime stranke',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Kategorije stroškov',
      'new_expense_category': 'Nova katergorija stroškov',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Kategorija stroškov uspešno ustvarjena',
      'updated_expense_category': 'Kategorija stroškov uspešno nadgrajena',
      'archived_expense_category': 'Kategorija stroškov uspešno arhivirana',
      'deleted_expense_category': 'Kategorija uspešno odstranjena',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Kategorija stroškov uspešno obnovljena',
      'archived_expense_categories':
          'Kategorija stroškov :count uspešno arhivirana',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Bo fakturiran',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Označi kot Aktivno',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Ponavljajoči račun',
      'recurring_invoices': 'Ponavljajoči računi',
      'new_recurring_invoice': 'Nov ponavljajoči račun',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Ponavljajoči račun uspešno arhiviran',
      'deleted_recurring_invoice': 'Ponavljajoči račun uspešno odstranjen',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Ponavljajoči račun uspešno obnovljen',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Vrstična postavka',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Poglej portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Shrani podatke kartice',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Vedno',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Oznaka',
      'client_number': 'Št. stranke',
      'auto_convert': 'Auto Convert',
      'company_name': 'Naziv podjetja',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Račun uspešno poslan',
      'emailed_quotes': 'Uspešno poslani predračuni',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Prehod',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Ur',
      'statement': 'Izpisek',
      'taxes': 'Davki',
      'surcharge': 'Doplačilo',
      'apply_payment': 'Apply Payment',
      'apply': 'Potrdi',
      'unapplied': 'Unapplied',
      'select_label': 'Izberi oznako',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Do',
      'health_check': 'Health Check',
      'payment_type_id': 'Način plačila',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Prihajajoči računi',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nedavna plačila',
      'upcoming_quotes': 'Prihajajoči predračuni',
      'expired_quotes': 'Potekli predračuni',
      'create_client': 'Ustvari stranko',
      'create_invoice': 'Ustvari račun',
      'create_quote': 'Ustvari predračun',
      'create_payment': 'Create Payment',
      'create_vendor': 'Ustvari prodajalca',
      'update_quote': 'Update Quote',
      'delete_quote': 'Odstrani ponubdo',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Zbriši račun',
      'update_client': 'Update Client',
      'delete_client': 'Odstrani stranko',
      'delete_payment': 'Odstrani plačilo',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Odstrani prodajalca',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Odstrani strošek',
      'create_task': 'Vnesi opravilo',
      'update_task': 'Update Task',
      'delete_task': 'Odstrani opravilo',
      'approve_quote': 'Approve Quote',
      'off': 'Izklopljeno',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Brezplačno',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Cilj',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API žetoni',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Žeton',
      'tokens': 'Žetoni',
      'new_token': 'New Token',
      'edit_token': 'Uredi žeton',
      'created_token': 'Žeton uspešno ustvarjen',
      'updated_token': 'Žeton uspešno posodobljen',
      'archived_token': 'Žeton uspešno arhiviran',
      'deleted_token': 'Žeton uspešno odstranjen',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Pošlji račun na e-pošto',
      'email_quote': 'Pošlji predračun',
      'email_credit': 'Email Credit',
      'email_payment': 'Pošlji plačilo po elektronki pošti',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Kontaktno ime',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Uredi plačilni pogoj',
      'created_payment_term': 'Plačilni pogoji uspešno ustvarjeni',
      'updated_payment_term': 'Plačilni pogoji uspešno posodobljeni',
      'archived_payment_term': 'Plačilni pogoji uspešno arhivirani',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Znesek dobropisa',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Ekskluzivno',
      'inclusive': 'Vključeno',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Vračilo plačila',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Polno ime',
      'city_state_postal': 'Mesto/Država/Pošta',
      'postal_city_state': 'Pošta/Mesto/Država',
      'custom1': 'Prvi po meri',
      'custom2': 'Drugi po meri',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Izprazni podatke',
      'purge_successful': 'Podatki podjetja uspešno odstranjeni',
      'purge_data_message':
          'Opozorilo: Vaši podatki bodo trajno zbrisani. Razveljavitev kasneje ni mogoča.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Dni',
      'age_group_30': '30 - 60 Dni',
      'age_group_60': '60 - 90 Dni',
      'age_group_90': '90 - 120 Dni',
      'age_group_120': '120+ dni',
      'refresh': 'Osveži',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Detalji računa',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Pravice',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count račun poslan',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Potrdi licenco',
      'cancel_account': 'Odstani račun',
      'cancel_account_message':
          'Opozorilo: Vaš račun bo trajno zbrisan. Razveljavitev ni mogoča.',
      'delete_company': 'Izbriši podjetje',
      'delete_company_message':
          'Opozorilo: Vaše podjetne bo trajno zbrisano. Razveljavitev ni mogoča.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Glava',
      'load_design': 'Noloži obliko',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Ponudbe',
      'tickets': 'Tickets',
      'recurring_quotes': 'Ponavljajoči predračuni',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Ponavaljajoči stroški',
      'account_management': 'Upravljanje računa',
      'credit_date': 'Datum dobropisa',
      'credit': 'Dobropis',
      'credits': 'Dobropisi',
      'new_credit': 'Vnesi dobropis',
      'edit_credit': 'Uredi dobropis',
      'created_credit': 'Dobropis uspešno ustvarjen',
      'updated_credit': 'Uspešno posodobljen dobropis',
      'archived_credit': 'Dobropis uspešno arhiviran',
      'deleted_credit': 'Dobropis uspešno odstranjen',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Dobropis uspešno obnovljen',
      'archived_credits': 'Število uspešno arhiviranih dobropisov: :count',
      'deleted_credits': 'Število uspešno odstranjenih dobropisov: :count',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Trenutna različica',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Izvedi več',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Novo podjetje',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Ponastavi',
      'number': 'Number',
      'export': 'Izvoz',
      'chart': 'Grafikon',
      'count': 'Count',
      'totals': 'Vsote',
      'blank': 'Prazno',
      'day': 'Dan',
      'month': 'Mesec',
      'year': 'Leto',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Združi v skupino',
      'credit_balance': 'Saldo dobropisa',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Kontaktni telefon',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Ulica (za dostavo)',
      'shipping_address2': 'Hišna št./stanovanje (za dostavo)',
      'shipping_city': 'Mesto (za dostavo)',
      'shipping_state': 'Regija/pokrajina (za dostavo)',
      'shipping_postal_code': 'Poštna št. (za dostavo)',
      'shipping_country': 'Država (za dostavo)',
      'billing_address1': 'Ulica (za račun)',
      'billing_address2': 'Hišna št./Stanovanje (za račun)',
      'billing_city': 'Mesto (za račun)',
      'billing_state': 'Regija/pokrajina (za račun)',
      'billing_postal_code': 'Poštna št. (za račun)',
      'billing_country': 'Države (za račun)',
      'client_id': 'Id stranke',
      'assigned_to': 'Assigned to',
      'created_by': 'Ustvaril :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Stolpci',
      'aging': 'Staranje',
      'profit_and_loss': 'Profit in izguba',
      'reports': 'Poročila',
      'report': 'Poročilo',
      'add_company': 'Dodaj podjetje',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Nepotrjen predračun',
      'help': 'Pomoč',
      'refund': 'Vračilo',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Kontaktna e-pošta',
      'multiselect': 'Multiselect',
      'entity_state': 'Stanje',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Sporočilo',
      'from': 'Od',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'forum za podporo',
      'about': 'About',
      'documentation': 'Dokumentacija',
      'contact_us': 'Kontakt',
      'subtotal': 'Neto',
      'line_total': 'Skupaj',
      'item': 'Postavka',
      'credit_email': 'Credit Email',
      'iframe_url': 'Spletna stran',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Da',
      'no': 'Ne',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Ogled',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Uporabnik',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Prosim izberite stranko',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Davčne dastavitve',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Proklop',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Možnosti',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Oddaj',
      'recover_password': 'Obnovite vaše geslo',
      'late_fees': 'Late Fees',
      'credit_number': 'Št. dobropisa',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Vrednost zamudnih obresti',
      'late_fee_percent': 'Odstotek za zamudne obresti',
      'schedule': 'Urnik',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dnevi',
      'invoice_email': 'Račun',
      'payment_email': 'Potrdilo',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Predračun',
      'endless_reminder': 'Periodičen opomin',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Upravljalec',
      'administrator_help':
          'Dovoli uporabniku da upravlja z uporabniki, nastavitvami in vsemi zapisi',
      'user_management': 'Uporabniki',
      'users': 'Uporabniki',
      'new_user': 'Nov uporabnik',
      'edit_user': 'Uredi uporabnika',
      'created_user': 'Successfully created user',
      'updated_user': 'Uporabnik uspešno posodobljen',
      'archived_user': 'Uporabnik uspešno arhiviran',
      'deleted_user': 'Uporabnik uspešno odstranjen',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Uporabnik uspešno obnovljen',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Splošne nastavitve',
      'invoice_options': 'Možnosti računa',
      'hide_paid_to_date': 'Skrij datum plačila',
      'hide_paid_to_date_help':
          'Prikaži le \"Plačano\" polje v računu, nakar je bilo plačilo prejeto.',
      'invoice_embed_documents': 'Omogočeni dokumenti',
      'invoice_embed_documents_help': 'V računu vključi pripete slike.',
      'all_pages_header': 'Prikaži glavo na',
      'all_pages_footer': 'Prikaži nogo na',
      'first_page': 'Prva stran',
      'all_pages': 'Vse strani',
      'last_page': 'Zadnja stran',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Osnovna barva',
      'secondary_color': 'Sekundarna barva',
      'page_size': 'Velikost strani',
      'font_size': 'Velikost pisave',
      'quote_design': 'Predloga predračuna',
      'invoice_fields': 'Polja računa',
      'product_fields': 'Polja izdelka',
      'invoice_terms': 'Pogoji računa',
      'invoice_footer': 'Noga računa',
      'quote_terms': 'Pogoji predračuna',
      'quote_footer': 'Noga predračuna',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help': 'Samodejno arhiviraj predračune po pretvorbi.',
      'auto_convert_quote': 'Samodejna Pretvorba',
      'auto_convert_quote_help':
          'Samodejno pretvori predračun v račun, ki ga stranka potrdi.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Dnevno',
      'freq_weekly': 'Tedensko',
      'freq_two_weeks': 'Dva tedna',
      'freq_four_weeks': 'Štiri tedni',
      'freq_monthly': 'Mesečno',
      'freq_two_months': 'Dva meseca',
      'freq_three_months': 'Trije meseci',
      'freq_four_months': 'Na štiri mesece',
      'freq_six_months': 'Šest mesecev',
      'freq_annually': 'Letno',
      'freq_two_years': 'Na dve leti',
      'freq_three_years': 'Three Years',
      'never': 'Nikoli',
      'company': 'Company',
      'generated_numbers': 'Ustvarjene številke',
      'charge_taxes': 'Zaračunaj davke',
      'next_reset': 'Naslednja ponastavitev',
      'reset_counter': 'Ponastavi števec',
      'recurring_prefix': 'Predpona ponavljajočih',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Polje izdelka',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Predpona',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'CSS po meri',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Prikaži na PDF',
      'signature_on_pdf_help':
          'Prikaži podpis stranke na PDF računu/predračunu.',
      'show_accept_invoice_terms': 'Potrditev pogojev računa',
      'show_accept_invoice_terms_help':
          'Stranka mora potrditi strinjanje s pogoji na računu.',
      'show_accept_quote_terms': 'Potrditev pogojev predračuna',
      'show_accept_quote_terms_help':
          'Stranka mora potrditi strinjanje s pogoji na predračunu.',
      'require_invoice_signature': 'Podpis računa',
      'require_invoice_signature_help':
          'Zahteva od stranke, da zagotovi svoj podpis.',
      'require_quote_signature': 'Podpis predračuna',
      'enable_portal_password': 'Zaščiti račune z geslom',
      'enable_portal_password_help':
          'Omogoča da nastavite geslo za vsako osebo. Če je geslo nastavljeno, ga bo uporabnik moral vnesti pred ogledom računa.',
      'authorization': 'Overovitev',
      'subdomain': 'Poddomena',
      'domain': 'Domena',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Lep pozdrav,',
      'enable_email_markup_help':
          'Olajšajte strankam plačevanje z dodajanjem schema.org označb v vašo e-pošto.',
      'plain': 'Navadno',
      'light': 'Svetlo',
      'dark': 'Temno',
      'email_design': 'Stil e-pošte',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Omogoči označbe.',
      'reply_to_email': 'Reply-To',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC',
      'processed': 'Processed',
      'credit_card': 'Kreditna kartica',
      'bank_transfer': 'Bančno nakazilo',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Omogoči minimalno',
      'enable_max': 'Omogoči maximalno',
      'min_limit': 'Minimalno: :min',
      'max_limit': 'Maksimalno: :max',
      'min': 'Minimalno',
      'max': 'Maksimalno',
      'accepted_card_logos': 'Prikazani logotipi katric',
      'credentials': 'Credentials',
      'update_address': 'Posodobi naslov',
      'update_address_help': 'Posodobi naslov stranke z predloženimi podatki',
      'rate': 'Cena',
      'tax_rate': 'Davčna stopnja',
      'new_tax_rate': 'Nova davčna stopnja',
      'edit_tax_rate': 'Uredi davčno stopnjo',
      'created_tax_rate': 'Davčna stopnja uspešno ustvarjena',
      'updated_tax_rate': 'Davčna stopnja uspešno posodobljena',
      'archived_tax_rate': 'Davčna stopnja uspešno arhivirana',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Samodejno vnesi izdelke',
      'fill_products_help': 'Izbira izdelka bo samodejno vnesla opis in ceno',
      'update_products': 'Samodejno posodobi izdelke',
      'update_products_help':
          'Posodobitev računa bo samodejno posodobila knjižnico izdelkov',
      'convert_products': 'Pretvori izdelke',
      'convert_products_help':
          'Samodejno pretvori cene izdelkov v valuto stranke',
      'fees': 'Provizije',
      'limits': 'Omejitve',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Zavrzi spremembe',
      'default_value': 'Default value',
      'disabled': 'Onemogočeno',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Nedelja',
      'monday': 'Ponedeljek',
      'tuesday': 'Torek',
      'wednesday': 'Sreda',
      'thursday': 'Četrtek',
      'friday': 'Petek',
      'saturday': 'Sobota',
      'january': 'Januar',
      'february': 'Februar',
      'march': 'Marec',
      'april': 'April',
      'may': 'Maj',
      'june': 'Junij',
      'july': 'Julij',
      'august': 'August',
      'september': 'September',
      'october': 'Oktober',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 urni čas',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logotip',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'izdelka',
      'device_settings': 'Device Settings',
      'defaults': 'Privzeto',
      'basic_settings': 'Osnovne nastavitve',
      'advanced_settings': 'Napredne nastavitve',
      'company_details': 'Podatki podjetja',
      'user_details': 'Podrobnosti uporabnika',
      'localization': 'Lokalizacija',
      'online_payments': 'Spletna plačila',
      'tax_rates': 'Davčne stopnje',
      'notifications': 'Obvestila',
      'import_export': 'Uvoz | Izvoz',
      'custom_fields': 'Polja po meri',
      'invoice_design': 'Izgled računa',
      'buy_now_buttons': 'Gumbi za takojšnji nakup',
      'email_settings': 'Nastavitve e-pošte',
      'templates_and_reminders': 'Predloge in opomini',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Vizualizacija podatkov',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Pogoji uporabe',
      'privacy_policy': 'Pravilnik o zasebnosti',
      'sign_up': 'Prijavi se',
      'account_login': 'Prijava v račun',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Ustvari',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Prenesi',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Dokument',
      'documents': 'Dokumenti',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Datum stroška',
      'pending': 'V teku',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Pretvorjeno',
      'add_documents_to_invoice': 'Pripni datoteke',
      'exchange_rate': 'Menjalni tečaj',
      'convert_currency': 'Pretvori valuto',
      'mark_paid': 'Označi kot plačano',
      'category': 'Kategorija',
      'address': 'Naslov',
      'new_vendor': 'Nov prodajalec',
      'created_vendor': 'Prodajalec uspešno ustvarjen',
      'updated_vendor': 'Prodajalec uspešno posodobljen',
      'archived_vendor': 'Prodajalec uspešno arhiviran',
      'deleted_vendor': 'Prodajalec uspešno odstranjen',
      'restored_vendor': 'Prodajalec uspešno obnovljen',
      'archived_vendors':
          'Število uspešno arhiviranih prodajalcev: :count clients',
      'deleted_vendors': 'Število uspešno odstranjenih prodajalcev: :count',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Vnesi strošek',
      'created_expense': 'Strošek uspešno vnešen',
      'updated_expense': 'Strošek uspešno posodobljen',
      'archived_expense': 'Strošek uspešno arhiviran',
      'deleted_expense': 'Strošek uspešno odstranjen',
      'restored_expense': 'Strošek uspešno obnovljen',
      'archived_expenses': 'Stroški uspešno arhivirani',
      'deleted_expenses': 'Stroški uspešno odstranjeni',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Kopiraj naslov za dostavo',
      'copy_billing': 'Kopiraj naslov za račun',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Fakturirano',
      'logged': 'Prijavljeno',
      'running': 'V teku',
      'resume': 'Nadaljuj',
      'task_errors': 'Prosim popravite prekrivajoče časove',
      'start': 'Začetek',
      'stop': 'Končaj',
      'started_task': 'Opravilo uspešno pričeto',
      'stopped_task': 'Opravilo uspešno ustavljeno',
      'resumed_task': 'Opravilo uspešno ponovno zagnano',
      'now': 'Zdaj',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Merilec časa',
      'manual': 'Ročno',
      'budgeted': 'Budgeted',
      'start_time': 'Začetek',
      'end_time': 'Čas zaključka',
      'date': 'Datum',
      'times': 'Čas',
      'duration': 'Trajanje',
      'new_task': 'Novo opravilo',
      'created_task': 'Opravilo uspešno ustvarjeno',
      'updated_task': 'Opravilo uspešno posodobljeno',
      'archived_task': 'Opravilo uspešno arhivirano',
      'deleted_task': 'Opravilo uspešno odstranjeno',
      'restored_task': 'Opravilo uspešno obnovljeno',
      'archived_tasks': 'Število uspešno odstranjenih opravil: :count',
      'deleted_tasks': 'Število uspešno odstranjenih opravil: :count tasks',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Predvidene ure',
      'created_project': 'Projekt uspešno ustvarjen',
      'updated_project': 'Projekt uspešno posodobljen',
      'archived_project': 'Projekt uspešno arhiviran',
      'deleted_project': 'Projekt uspešno odstranjen',
      'restored_project': 'Projekt uspešno obnovljen',
      'archived_projects': 'Število uspešno arhiviranih projektov: :count',
      'deleted_projects': 'Število uspešno odstranjenih projektov: :count',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Now projekt',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'klikni tukaj',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Noga',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Obseg po meri',
      'date_range': 'Časovno obdobje',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Ta mesec',
      'last_month': 'Zadnji mesec',
      'this_year': 'To leto',
      'last_year': 'Zadnje leto',
      'custom': 'Po meri',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Kopiraj v predračun',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Ogled računa',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Uredi stranko',
      'edit_product': 'Uredi izdelek',
      'edit_invoice': 'Uredi račun',
      'edit_quote': 'Uredi predračun',
      'edit_payment': 'Uredi plačilo',
      'edit_task': 'Uredi opravilo',
      'edit_expense': 'Uredi strošek',
      'edit_vendor': 'Uredi prodajalca',
      'edit_project': 'Uredi projekt',
      'edit_recurring_expense': 'Uredi ponavaljajoč strošek',
      'edit_recurring_quote': 'Uredi ponavaljajoč predračun',
      'billing_address': 'Naslov za pošiljanje računa',
      'shipping_address': 'Naslov za dostavo',
      'total_revenue': 'Skupni prihodki',
      'average_invoice': 'Povprečen račun',
      'outstanding': 'Odprte postavke',
      'invoices_sent': ':count računi poslani',
      'active_clients': 'aktivne stranke',
      'close': 'Zapri',
      'email': 'E-pošta',
      'password': 'Geslo',
      'url': 'URL',
      'secret': 'Skrivnost',
      'name': 'Ime',
      'logout': 'Odjava',
      'login': 'Prijava',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Išči',
      'active': 'Aktivno',
      'archived': 'Arhivirano',
      'deleted': 'Odstranjeno',
      'dashboard': 'Nadzorna plošča',
      'archive': 'Arhiv',
      'delete': 'Odstrani',
      'restore': 'Obnovitev',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Prosim vnesi ključ izdelka',
      'ascending': 'Naraščajoče',
      'descending': 'Padajoče',
      'save': 'Shrani',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Že plačano',
      'balance_due': 'Za plačilo',
      'balance': 'Saldo',
      'overview': 'Overview',
      'details': 'Podrobnosti',
      'phone': 'Telefon',
      'website': 'Spletišče',
      'vat_number': 'Davčna št.',
      'id_number': 'ID št.',
      'create': 'Ustvari',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakti',
      'additional': 'Additional',
      'first_name': 'Ime',
      'last_name': 'Priimek',
      'add_contact': 'Dodaj kontakt',
      'are_you_sure': 'Ali ste prepričani?',
      'cancel': 'Prekliči',
      'ok': 'Ok',
      'remove': 'Odstrani',
      'email_is_invalid': 'Email is invalid',
      'product': 'Izdelek',
      'products': 'Izdelki',
      'new_product': 'Nov izdelek',
      'created_product': 'Izdelek uspešno ustvarjen',
      'updated_product': 'Izdelek uspešno posodobljen',
      'archived_product': 'Izdelek uspešno arhiviran',
      'deleted_product': 'Izdelek uspešno odstranjen',
      'restored_product': 'Izdelek uspešno obnovljen',
      'archived_products': 'Število uspešno arhiviranih izdelkov: :count',
      'deleted_products': 'Število uspešno odstranjenih izdelkov: :count',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Izdelki',
      'notes': 'Opis',
      'cost': 'Cena',
      'client': 'Stranka',
      'clients': 'Stranke',
      'new_client': 'Nova stranka',
      'created_client': 'Stranka uspešno ustvarjena',
      'updated_client': 'Uspešno posodobljena stranka',
      'archived_client': 'Stranka uspešno arhivirana',
      'archived_clients': 'Število uspešno arhiviranih strank: :count clients',
      'deleted_client': 'Stranka uspešno odstranjena',
      'deleted_clients': 'Število uspešno odstranjenih strank: :count',
      'restored_client': 'Stranka uspešno obnovljena',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Ulica',
      'address2': 'Hišna št./Stanovanje',
      'city': 'Mesto',
      'state': 'Regija/pokrajina',
      'postal_code': 'Poštna št.',
      'country': 'Država',
      'invoice': 'Račun',
      'invoices': 'Računi',
      'new_invoice': 'Nov račun',
      'created_invoice': 'Račun uspešno ustvarjen',
      'updated_invoice': 'Račun uspešno posodobljen',
      'archived_invoice': 'Račun uspešno arhiviran',
      'deleted_invoice': 'Račun uspešno odstranjen',
      'restored_invoice': 'Račun uspešno obnovljen',
      'archived_invoices':
          'Število uspešno arhiviranih računov: :count invoices',
      'deleted_invoices':
          'Število uspešno odstranjenih ponudb: :count invoices',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Račun uspešno poslan',
      'emailed_payment': 'Plačilo poslano po elektronski pošti',
      'amount': 'Znesek',
      'invoice_number': 'Št. računa',
      'invoice_date': 'Datum računa',
      'discount': 'Popust',
      'po_number': 'Št. naročilnice',
      'terms': 'Pogoji',
      'public_notes': 'Javni zaznamki',
      'private_notes': 'Zasebni zaznamki',
      'frequency': 'Pogostost',
      'start_date': 'Datum začetka',
      'end_date': 'Datum zapadlost',
      'quote_number': 'Št. predračuna',
      'quote_date': 'Datum predračuna',
      'valid_until': 'Veljavnost',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Opis',
      'unit_cost': 'Cena',
      'quantity': 'Količina',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Službeni telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Rok plačila',
      'partial_due_date': 'Delno plačilo do datuma',
      'status': 'Stanje',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Stanje predračuna',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Skupaj',
      'percent': 'Odstotek',
      'edit': 'Uredi',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Urna postavka',
      'settings': 'Nastavitve',
      'language': 'Language',
      'currency': 'Valuta',
      'created_at': 'Ustvarjen dne',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'DDV',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Prosim vnesi številko predračuna',
      'past_due': 'Zapadlo',
      'draft': 'Osnutek',
      'sent': 'Poslano',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Delno plačilo/polog',
      'paid': 'Plačano',
      'mark_sent': 'Označi kot poslano',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Končano',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Temen način',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Dejavnost',
      'no_records_found': 'No records found',
      'clone': 'Kloniraj',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Plačilni pogoji',
      'payment_date': 'Datum plačila',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal za stranke',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Omogočeno',
      'recipients': 'Prejemniki',
      'initial_email': 'Prva e-pošta',
      'first_reminder': 'Prvi opomin',
      'second_reminder': 'Drugi opomin',
      'third_reminder': 'Tretji opomin',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Predloga',
      'send': 'Send',
      'subject': 'Naslov',
      'body': 'Vsebina',
      'send_email': 'Pošlji e-pošto',
      'email_receipt': 'Pošlji račun stranki',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Predogled',
      'customize': 'Prilagodi po meri',
      'history': 'Zgodovina',
      'payment': 'Plačilo',
      'payments': 'Plačila',
      'refunded': 'Refunded',
      'payment_type': 'Način plačila',
      'transaction_reference': 'Referenca transakcije',
      'enter_payment': 'Vnesi plačilo',
      'new_payment': 'Vnesi plačilo',
      'created_payment': 'Plačilo uspešno ustvarjeno',
      'updated_payment': 'Plačilo uspešno posodobljeno',
      'archived_payment': 'Plačilo uspešno arhivirano',
      'deleted_payment': 'Plačilo uspešno odstranjeno',
      'restored_payment': 'Plačilo uspešno obnovljeno',
      'archived_payments': 'Število uspešno arhiviranih plačil: :count',
      'deleted_payments': 'Število uspešno odstranjenih plačil: :count',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Predračun',
      'quotes': 'Predračuni',
      'new_quote': 'Nov predračun',
      'created_quote': 'Predračun uspešno ustvarjen',
      'updated_quote': 'Predračun uspešno posodobljen',
      'archived_quote': 'Predračun uspešno arhiviran',
      'deleted_quote': 'Predračun uspešno odstranjen',
      'restored_quote': 'Predračun uspešno obnovljen',
      'archived_quotes': 'Število uspešno arhiviranih predračunov:',
      'deleted_quotes': 'Število uspešno odstranjenih predračunov: :count',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Strošek',
      'expenses': 'Stroški',
      'vendor': 'Prodajalec',
      'vendors': 'Prodajalci',
      'task': 'Opravilo',
      'tasks': 'Opravila',
      'project': 'Projekt',
      'projects': 'Projekti',
      'activity_1': ':user je ustvaril stranko :client',
      'activity_2': ':user je arhiviraj stranko :client',
      'activity_3': ':user je odstranil stranko :client',
      'activity_4': ':user je ustvaril račun :invoice',
      'activity_5': ':user je posodobil račun :invoice',
      'activity_6': ':user je račun :invoice za :client poslal osebi :contact',
      'activity_7': ':contact si je ogledal račun :invoice za :client',
      'activity_8': ':user je arhiviral račun :invoice',
      'activity_9': ':user je odstranil račun :invoice',
      'activity_10':
          ':contact je vnesel plačilo :payment v znesku :payment_amount na računu :invoice za :client',
      'activity_11': ':user je posodobil plačilo :payment',
      'activity_12': ':user je arhiviral plačilo :payment',
      'activity_13': ':user je odstranil :payment',
      'activity_14': ':user je vnesel :credit dobropis',
      'activity_15': ':user je posodobil :credit dobropis',
      'activity_16': ':user je arhiviral :credit dobropis',
      'activity_17': ':user je odstranil :credit dobropis',
      'activity_18': ':user je ustvaril predračun :quote',
      'activity_19': ':user je posodobil predračun :quote',
      'activity_20':
          ':user je predračun :quote za :client poslal osebi :contact',
      'activity_21': ':contact je pogledal predračun :quote',
      'activity_22': ':user je arhiviral predračun :quote',
      'activity_23': ':user je odstranil predračun :quote',
      'activity_24': ':user je obnovil predračun :quote',
      'activity_25': ':user je obnovil račun :invoice',
      'activity_26': ':user je obnovil stranko :client',
      'activity_27': ':user je obnovil plačilo :payment',
      'activity_28': ':user je obnovil dobropis :credit',
      'activity_29': ':contact je potrdil predračun :quote za :client',
      'activity_30': ':user je ustvaril prodajalca :vendor',
      'activity_31': ':user je arhiviral prodajalca :vendor',
      'activity_32': ':user je odstranil prodajalca :vendor',
      'activity_33': ':user je obnovil prodajalca :vendor',
      'activity_34': ':user je vnesel strošek :expense',
      'activity_35': ':user je arhiviral strošek :expense',
      'activity_36': ':user je izbrisal strošek :expense',
      'activity_37': ':user je obnovil strošek :expense',
      'activity_39':
          ':user je preklical plačilo :payment v znesku :payment_amount',
      'activity_40':
          ':user je vrnil :adjustment od plačila :payment v znesku :payment_amount',
      'activity_41': ':payment_amount plačilo (:payment) ni uspelo',
      'activity_42': ':user je vnesel opravilo :task',
      'activity_43': ':user je posodobil opravilo :task',
      'activity_44': ':user je arhiviral opravilo :task',
      'activity_45': ':user je izbrisal opravilo :task',
      'activity_46': ':user je obnovil opravilo :task',
      'activity_47': ':user je posodobil opravilo :expense',
      'activity_48': ':user je posodobil zahtevek :ticker',
      'activity_49': ':user je zaprl zahtevek :ticket',
      'activity_50': ':user je združil zahtevek :ticket',
      'activity_51': ':user je razdružil zahtevek :ticket',
      'activity_52': ':contact je odprl zahtevek :ticket',
      'activity_53': ':contact je ponovno odprl zahtevek :ticket',
      'activity_54': ':user je ponovno odprl zahtevek :ticket',
      'activity_55': ':contact je odgovoril na zahtevek :ticket',
      'activity_56': ':user si je ogledal zahtevek :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Geslo za enkratno uporabo',
      'emailed_quote': 'Predračun uspešno poslan',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Predračun označen kot poslan',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Poteklo',
      'all': 'Vse',
      'select': 'Izberi',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Vrednost po meri',
      'custom_value2': 'Vrednost po meri',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Števec za račun',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Števec številke predračuna',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tip',
      'invoice_amount': 'Znesek računa',
      'invoice_due_date': 'Veljavnost',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Samodejno plačilo',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Ime davčne stopnje',
      'tax_amount': 'Znesek davka',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Znesek plačila',
      'age': 'Starost',
      'is_running': 'Is Running',
      'time_log': 'Časovni Dnevnik',
      'bank_id': 'Banka',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Kategorija stroškov',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'es': {
      'import_data': 'Importar datos',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Primer Nombre de Contacto',
      'contact_last_name': 'Apellido de Contacto',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Debe ser mayor que cero y menor que el total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Activar',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Asigne el suubdominio o mostrar la factura en su propio sitio web.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Suscripciones',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Editar suscripción',
      'created_subscription': 'Suscripción creada correctamente',
      'updated_subscription': 'Suscripción actualizada correctamente',
      'archived_subscription': 'Suscripción correctamente archivada',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Reenviar Invitación',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Escanee el código de barras con una aplicación compatible con :link',
      'enabled_two_factor':
          'Autenticación de Dos Factores habilitada con éxito',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Autenticación de Dos Factores',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Último Trimestre',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Convertir a Factura',
      'registration_url': 'URL de Registro',
      'invoice_project': 'Facturar proyecto',
      'invoice_task': 'Tarea de Factura',
      'invoice_expense': 'Facturar Gasto',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Eventos Soportados',
      'converted_amount': 'Cantidad Convertida',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Documentos por defecto',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Ocultar',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Columna',
      'sample': 'Ejemplo',
      'map_to': 'Map To',
      'import': 'Importar',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Por favor selecciona un archivo',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Seleccionar archivo CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Servicio',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Sin Pagar',
      'white_label': 'White Label',
      'delivery_note': 'Nota de Entrega',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Total Facturado',
      'quote_total': 'Total cotizado',
      'credit_total': 'Crédito Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Nombre del Cliente',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Expense Categories',
      'new_expense_category': 'New Expense Category',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Categoría actualizada con éxito',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Debe ser Facturado',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Marcar como Activo',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Factura Recurrente',
      'recurring_invoices': 'Facturas Recurrentes',
      'new_recurring_invoice': 'Nueva Factura Recurrente',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Factura periódica archivada',
      'deleted_recurring_invoice': 'Factura periódica borrada',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'Factura periódica restaurada',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Ganancia',
      'line_item': 'Item de Linea',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Abierto',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Guardar detalles de la tarjeta',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Etiqueta',
      'client_number': 'Cliente Número',
      'auto_convert': 'Auto Convert',
      'company_name': 'Nombre de Empresa',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Facturas enviadas por correo electrónico con éxito.',
      'emailed_quotes':
          'Cotizaciones enviadas por correo electrónico con éxito.',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Pasarela de Pagos',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Horas',
      'statement': 'Estado De Cuenta',
      'taxes': 'Impuestos',
      'surcharge': 'Sobrecargo',
      'apply_payment': 'Apply Payment',
      'apply': 'Aplicar',
      'unapplied': 'Unapplied',
      'select_label': 'Seleccionar Etiqueta',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Para',
      'health_check': 'Health Check',
      'payment_type_id': 'Tipo de pago',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Próximas Facturas',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Pagos Recientes',
      'upcoming_quotes': 'Próximas Cotizaciones',
      'expired_quotes': 'Cotizaciones Vencidas',
      'create_client': 'Crear Cliente',
      'create_invoice': 'Crear Factura',
      'create_quote': 'Crear Cotización',
      'create_payment': 'Create Payment',
      'create_vendor': 'Crear Proveedor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Eliminar Cotización',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Eliminar Factura',
      'update_client': 'Update Client',
      'delete_client': 'Eliminar Cliente',
      'delete_payment': 'Eliminar Pago',
      'update_vendor': 'Actualizar Proveedor',
      'delete_vendor': 'Borrar Proveedor',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Borrar Gasto',
      'create_task': 'Crear Tarea',
      'update_task': 'Update Task',
      'delete_task': 'Eliminar Tarea',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Gratis',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Objetivo',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Editar Token',
      'created_token': 'Token creado con éxito',
      'updated_token': 'Token actualizado con éxito',
      'archived_token': 'Token archivado',
      'deleted_token': 'Token eliminado con éxito',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Registro de Cliente',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Enviar factura por correo',
      'email_quote': 'Enviar Cotización',
      'email_credit': 'Email Credit',
      'email_payment': 'Enviar Pago por Correo Electrónico',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Editar el Términos de Pago',
      'created_payment_term': 'Término de pago creado con éxito',
      'updated_payment_term': 'Término de pago actualizado con éxito',
      'archived_payment_term': 'Término de pago archivado con éxito',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Cantidad de Crédito',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusivo',
      'inclusive': 'Inclusivo',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Buscar Proveedor',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Buscar 1 Proveedor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Nombre Completo',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Código Postal/Ciudad/Estado',
      'custom1': 'Primero Personalizado',
      'custom2': 'Segundo Personalizado',
      'custom3': 'Tercero Personalizado',
      'custom4': 'Cuarto Personalizado',
      'optional': 'Opcional',
      'license': 'Licencia',
      'purge_data': 'Purgar Datos',
      'purge_successful': 'Datos de la empresa purgados con éxito',
      'purge_data_message':
          'Advertencia: Esto borrará definitivamente tus datos, no hay de deshacerlo.',
      'invoice_balance': 'Balance de la Factura',
      'age_group_0': '0 - 30 Días',
      'age_group_30': '30 - 60 Días',
      'age_group_60': '60 - 90 Días',
      'age_group_90': '90 - 120 Días',
      'age_group_120': '120+ Días',
      'refresh': 'Refrescar',
      'saved_design': 'Diseño guardado con éxito',
      'client_details': 'Detalles del Cliente',
      'company_address': 'Dirección de la Empresa',
      'invoice_details': 'Detalles de la Factura',
      'quote_details': 'Detalles de la Cotización',
      'credit_details': 'Detalles del Crédito',
      'product_columns': 'Columna de Productos',
      'task_columns': 'Columna de Tareas',
      'add_field': 'Agregar Campos',
      'all_events': 'Todos los Eventos',
      'permissions': 'Permissions',
      'none': 'Ninguno',
      'owned': 'Propiedad',
      'payment_success': 'Pago Exitóso',
      'payment_failure': 'Fallos con el Pago',
      'invoice_sent': ':count factura enviada',
      'quote_sent': 'Cotización Enviada',
      'credit_sent': 'Crédito Enviado',
      'invoice_viewed': 'Factura Vista',
      'quote_viewed': 'Crédito Visto',
      'credit_viewed': 'Crédito Visto',
      'quote_approved': 'Cotización Aprobada',
      'receive_all_notifications': 'Recibir Todas Las Notificaciones',
      'purchase_license': 'Comprar Licencia',
      'apply_license': 'Activar Licencia',
      'cancel_account': 'Cancelar Cuenta',
      'cancel_account_message':
          'AVISO: Esta acción eliminará tu cuenta de forma permanente.',
      'delete_company': 'Eliminar Empresa',
      'delete_company_message':
          'Advertencia: Esto eliminará su empresa, no hay manera de deshacerlo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Cotización convertida con éxito',
      'credit_design': 'Diseño de Créditos',
      'includes': 'Incluir',
      'header': 'Encabezado',
      'load_design': 'Cargar Diseño',
      'css_framework': 'Framework de CSS',
      'custom_designs': 'Diseños Personalizados',
      'designs': 'Diseños',
      'new_design': 'Nuevo Diseño',
      'edit_design': 'Editar Diseño',
      'created_design': 'Diseño creado con éxito',
      'updated_design': 'Diseño actualizado con éxito',
      'archived_design': 'Diseño archivado con éxito',
      'deleted_design': 'Diseño eliminado con éxito',
      'removed_design': 'Diseño removido con éxito',
      'restored_design': 'Diseño restaurado con éxito',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Propuestas',
      'tickets': 'Tickets',
      'recurring_quotes': 'Cotizaciones Recurrentes',
      'recurring_tasks': 'Tareas Recurrentes',
      'recurring_expenses': 'Gastos Recurrentes',
      'account_management': 'Account Management',
      'credit_date': 'Fecha de Crédito',
      'credit': 'Crédito',
      'credits': 'Créditos',
      'new_credit': 'Ingresa el Crédito',
      'edit_credit': 'Editar Crédito',
      'created_credit': 'Crédito creado con éxito',
      'updated_credit': 'Crédito actualizado con éxito',
      'archived_credit': 'Crédito archivado con éxito',
      'deleted_credit': 'Créditos eliminados con éxito',
      'removed_credit': 'Crédito removido con éxito',
      'restored_credit': 'Crédito restaurado con éxito',
      'archived_credits': ':count creditos archivados con éxito',
      'deleted_credits': ':count creditos eliminados con éxito',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Versión Actual',
      'latest_version': 'Últiima Versión',
      'update_now': 'Actualizarse Ahora',
      'a_new_version_is_available':
          'Una nueva versión de la aplicación está disponible',
      'update_available': 'Actualización Disponible',
      'app_updated': 'Actualización completada con éxito',
      'learn_more': 'Saber más',
      'integrations': 'Integraciones',
      'tracking_id': 'Id de Rastreo',
      'slack_webhook_url': 'URL del Webhook de Slack',
      'credit_footer': 'Pie de Página del Crédito',
      'credit_terms': 'Términos del Crédito',
      'new_company': 'Nueva Empresa',
      'added_company': 'Empresa agregada con éxito',
      'company1': 'Empresa Personalizada 1',
      'company2': 'Empresa Personalizada 2',
      'company3': 'Empresa Personalizada 3',
      'company4': 'Empresa Personalizada 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Reiniciar',
      'number': 'Number',
      'export': 'Exportar',
      'chart': 'Gráfica',
      'count': 'Count',
      'totals': 'Totales',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Mes',
      'year': 'Año',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Agrupar por',
      'credit_balance': 'Saldo de Crédito',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Teléfono de Contacto',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Calle de Envío',
      'shipping_address2': 'Apto/Suite de Envío',
      'shipping_city': 'Ciudad de Envío',
      'shipping_state': 'Estado/Provincia de Envío',
      'shipping_postal_code': 'Código Postal de Envío',
      'shipping_country': 'País de Envío',
      'billing_address1': 'Calle de Facturación',
      'billing_address2': 'Apto/Suite de Facturación',
      'billing_city': 'Ciudad de Facturación',
      'billing_state': 'Estado/Provincia de Facturación',
      'billing_postal_code': 'Código Postal de Facturación',
      'billing_country': 'País de Facturación',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Creado por :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columnas',
      'aging': 'Envejecimiento',
      'profit_and_loss': 'Ganancias y Pérdidas',
      'reports': 'Informes',
      'report': 'Reporte',
      'add_company': 'Agregar Empresa',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Ayuda',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Correo de Contacto',
      'multiselect': 'Multiselect',
      'entity_state': 'Estado',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Mensaje',
      'from': 'De',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'foro de soporte',
      'about': 'About',
      'documentation': 'Documentación',
      'contact_us': 'Contáctenos',
      'subtotal': 'Subtotal',
      'line_total': 'Total',
      'item': 'Concepto',
      'credit_email': 'Credit Email',
      'iframe_url': 'Sitio Web',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Si',
      'no': 'No',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Seleccionar Empresa',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Ver',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Usuario',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Configuración de Impuestos',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Opciones',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Enviar',
      'recover_password': 'Recuperar contraseña',
      'late_fees': 'Late Fees',
      'credit_number': 'Número de Crédito',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Valor Tarifa por Tardanza',
      'late_fee_percent': 'Porcentaje Tarifa por Tardanza',
      'schedule': 'Programar',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Días',
      'invoice_email': 'Correo de Factura',
      'payment_email': 'Correo de Pago',
      'partial_payment': 'Pago Parcial',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Correo Electrónico de Pago Parcial',
      'quote_email': 'Correo de Cotizacion',
      'endless_reminder': 'Recordatorio sin fin',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrador',
      'administrator_help':
          'Permitir que administre usuarios, cambie configuraciones y modifique cualquier registro',
      'user_management': 'Gestión de Usuarios',
      'users': 'Usuarios',
      'new_user': 'Nuevo Usuario',
      'edit_user': 'Editar Usario',
      'created_user': 'Successfully created user',
      'updated_user': 'Usario actualizado con éxito',
      'archived_user': 'Usuario archivado',
      'deleted_user': 'Usario eliminado con éxito',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Usuario restaurado con éxito',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Configuración General',
      'invoice_options': 'Opciones de Factura',
      'hide_paid_to_date': 'Ocultar Valor Pagado a la Fecha',
      'hide_paid_to_date_help':
          'Solo mostrar la opción “Pagado a la fecha” en sus facturas cuando se ha recibido un pago.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Mostrar encabezado',
      'all_pages_footer': 'Mostrar pie',
      'first_page': 'Primera página',
      'all_pages': 'Todas las páginas',
      'last_page': 'Última página',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Color Primario',
      'secondary_color': 'Color Secundario',
      'page_size': 'Page Size',
      'font_size': 'Tamaño de Letra',
      'quote_design': 'Diseño de Cotización',
      'invoice_fields': 'Campos de Factura',
      'product_fields': 'Campos de Producto',
      'invoice_terms': 'Términos de Facturación',
      'invoice_footer': 'Pie de págia de la factura',
      'quote_terms': 'Terminos de Cotización',
      'quote_footer': 'Pie de la Cotización',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convertir',
      'auto_convert_quote_help':
          'Convierte un presupuesto en factura automaticamente cuando los aprueba el cliente.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Diario',
      'freq_weekly': 'Weekly',
      'freq_two_weeks': 'Two weeks',
      'freq_four_weeks': 'Four weeks',
      'freq_monthly': 'Mensual',
      'freq_two_months': 'Dos meses',
      'freq_three_months': 'Tres meses',
      'freq_four_months': 'Cuatro meses',
      'freq_six_months': 'Seis meses',
      'freq_annually': 'Annually',
      'freq_two_years': 'Dos años',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Empresa',
      'generated_numbers': 'Números Generados',
      'charge_taxes': 'Cargar Impuestos',
      'next_reset': 'Siguiente Reinicio',
      'reset_counter': 'Reiniciar Contador',
      'recurring_prefix': 'Prefijo Recurrente',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Campo de Empresa',
      'company_value': 'Valor de Empresa',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Campo Proveedor',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefijo',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'CSS Personalizado',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Ver en PDF',
      'signature_on_pdf_help':
          'Mostrar la firma del cliente en los PDF de facturas/presupuestos.',
      'show_accept_invoice_terms': 'Casilla de los Términos de la Factura',
      'show_accept_invoice_terms_help':
          'Requerir que el cliente confirme que aceptó los términos de la factura.',
      'show_accept_quote_terms': 'Casilla de los Términos de la Cotización',
      'show_accept_quote_terms_help':
          'Requerir que el cliente confirme que aceptó los términos de la cotización.',
      'require_invoice_signature': 'Firma de la Facturra',
      'require_invoice_signature_help':
          'Requerir que el cliente provea su firma.',
      'require_quote_signature': 'Firma de la Cotización',
      'enable_portal_password': 'Proteger Facturas con Contraseña',
      'enable_portal_password_help':
          'Permite establecer una contraseña para cada contacto. Si una contraseña es establecida, se le será solicitada al contacto para acceder a sus facturas.',
      'authorization': 'Autorización',
      'subdomain': 'Subdominio',
      'domain': 'Dominio',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Un cordial saludo,',
      'enable_email_markup_help':
          'Haga que sea fácil para sus clientes que paguen mediante la adición de marcas \"schema.org\" a sus correos electrónicos.',
      'plain': 'Plano',
      'light': 'Claro',
      'dark': 'Oscuro',
      'email_design': 'Diseño de Correo',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Habilitar Markup',
      'reply_to_email': 'Correo de Respuesta',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Correo para Copia Oculta BCC',
      'processed': 'Processed',
      'credit_card': 'Credit Card',
      'bank_transfer': 'Bank Transfer',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Logos de Tarjetas Aceptadas',
      'credentials': 'Credentials',
      'update_address': 'Actualizar Dirección',
      'update_address_help':
          'Actualiza la dirección del cliente con los detalles proporcionados',
      'rate': 'Tasas',
      'tax_rate': 'Tasa de Impuesto',
      'new_tax_rate': 'Nueva Tasa de Impuesto',
      'edit_tax_rate': 'Editar tasa de impuesto',
      'created_tax_rate': 'Tasa de impuesto creada con éxito',
      'updated_tax_rate': 'Tasa de impuesto actualizada con éxito',
      'archived_tax_rate': 'Tasa de impuesto archivada con éxito',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-rellenar productos',
      'fill_products_help':
          'Seleccionar un producto automáticamente configurará la descripción y coste',
      'update_products': 'Auto-actualizar productos',
      'update_products_help':
          'Actualizar una factura automáticamente actualizará los productos',
      'convert_products': 'Convertir productos',
      'convert_products_help':
          'Convertir automáticamente precios de los productos a la moneda del cliente',
      'fees': 'Tarifas',
      'limits': 'Límites',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Descartar Cambios',
      'default_value': 'Default value',
      'disabled': 'Deshabilitado',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Domingo',
      'monday': 'Lunes',
      'tuesday': 'Martes',
      'wednesday': 'Miércoles',
      'thursday': 'Jueves',
      'friday': 'Viernes',
      'saturday': 'Sábado',
      'january': 'Enero',
      'february': 'Febrero',
      'march': 'Marzo',
      'april': 'Abril',
      'may': 'Mayo',
      'june': 'Junio',
      'july': 'Julio',
      'august': 'Agosto',
      'september': 'Septiembre',
      'october': 'Octubre',
      'november': 'Noviembre',
      'december': 'Diciembre',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': 'Tiempo 24 Horas',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtro por Proveedor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Configuración del Producto',
      'device_settings': 'Device Settings',
      'defaults': 'Valores por Defecto',
      'basic_settings': 'Configuración Básica',
      'advanced_settings': 'Configuración Avanzada',
      'company_details': 'Detalles de la Empresa',
      'user_details': 'Detalles de Usuario',
      'localization': 'Localización',
      'online_payments': 'Pagos Online',
      'tax_rates': 'Tasas de Impuesto',
      'notifications': 'Notificaciones',
      'import_export': 'Importar/Exportar',
      'custom_fields': 'Campos personalizados',
      'invoice_design': 'Diseño de factura',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'Configuración del Correo Electrónico',
      'templates_and_reminders': 'Plantillas & Recordatorios',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Visualización de Datos',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Términos de Servicio',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Registrarse',
      'account_login': 'Iniciar Sesión',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Descargar',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Documento',
      'documents': 'Documents',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Fecha del Gasto',
      'pending': 'Pendiente',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Convertido',
      'add_documents_to_invoice': 'Agregar documentos a la factura',
      'exchange_rate': 'Tipo de Cambio',
      'convert_currency': 'Convertir moneda',
      'mark_paid': 'Marcar como Pagado',
      'category': 'Category',
      'address': 'Dirección',
      'new_vendor': 'Nuevo Proveedor',
      'created_vendor': 'Proveedor creado con éxito',
      'updated_vendor': 'Proveedor actualizado con éxito',
      'archived_vendor': 'Proveedor archivado con éxito',
      'deleted_vendor': 'Proveedor eliminado con éxito',
      'restored_vendor': 'Proveedor recuperado con éxito',
      'archived_vendors': ':count proveedores actualizados con éxito',
      'deleted_vendors': ':count proveedores actualizados con éxito',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Ingrese el Gasto',
      'created_expense': 'Gasto creado correctamente',
      'updated_expense': 'Gasto actualizado correctamente',
      'archived_expense': 'Gasto archivado correctamente',
      'deleted_expense': 'Gasto borrado correctamente',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Gastos archivados correctamente',
      'deleted_expenses': 'Gastos borrados correctamente',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copiar envío',
      'copy_billing': 'Copiar facturación',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Facturado',
      'logged': 'Registrado',
      'running': 'Ejecutando',
      'resume': 'Continuar',
      'task_errors':
          'Por favor corrija cualquier tiempo que se sobreponga con otro',
      'start': 'Iniciar',
      'stop': 'Detener',
      'started_task': 'Tarea iniciada con éxito',
      'stopped_task': 'Tarea detenida con éxito',
      'resumed_task': 'Tarea reanudada con éxito',
      'now': 'Ahora',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Temporizador',
      'manual': 'Manual',
      'budgeted': 'Budgeted',
      'start_time': 'Tiempo de Inicio',
      'end_time': 'Tiempo Final',
      'date': 'Fecha',
      'times': 'Tiempos',
      'duration': 'Duración',
      'new_task': 'Nueva Tarea',
      'created_task': 'Tarea creada con éxito',
      'updated_task': 'Tarea actualizada con éxito',
      'archived_task': 'Tarea archivada con éxito',
      'deleted_task': 'Tarea eliminada con éxito',
      'restored_task': 'Tarea restaurada con éxito',
      'archived_tasks': ':count tareas archivadas con éxito',
      'deleted_tasks': ':count tareas eliminadas con éxito',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Proyecto creado con éxito',
      'updated_project': 'Proyecto actualizado con éxito',
      'archived_project': 'Proyecto archivado con éxito',
      'deleted_project': 'Proyecto eliminado con éxito',
      'restored_project': 'Proyecto restaurado con éxito',
      'archived_projects': 'Archivados con éxito :count proyectos',
      'deleted_projects': 'Eliminados con éxito :count proyectos',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nuevo Proyecto',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'haz clic aquí',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Pie de Página',
      'compare': 'Comparar',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Rango Personalizado',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Periodo de Comparación',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Comparar con',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Este Mes',
      'last_month': 'Mes Anterior',
      'this_year': 'Este Año',
      'last_year': 'Año Anterior',
      'custom': 'Personalizado',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clonar como Crédito',
      'view_invoice': 'Ver Factura',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Editar Cliente',
      'edit_product': 'Editar Producto',
      'edit_invoice': 'Editar Factura',
      'edit_quote': 'Editar Cotización',
      'edit_payment': 'Editar Pago',
      'edit_task': 'Editar Tarea',
      'edit_expense': 'Editar Gasto',
      'edit_vendor': 'Editar Proveedor',
      'edit_project': 'Editar Proyecto',
      'edit_recurring_expense': 'Editar Gasto Recurrente',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Dirección de facturación',
      'shipping_address': 'Dirección de Envío',
      'total_revenue': 'Ingresos Totales',
      'average_invoice': 'Promedio de Facturación',
      'outstanding': 'Pendiente de Cobro',
      'invoices_sent': ':count facturas enviadas',
      'active_clients': 'Clientes Activos',
      'close': 'Cerrar',
      'email': 'Correo Electrónico',
      'password': 'Contraseña',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Nombre',
      'logout': 'Cerrar sesión',
      'login': 'Iniciar Sesión',
      'filter': 'Filtrar',
      'sort': 'Sort',
      'search': 'Búsqueda',
      'active': 'Activo',
      'archived': 'Archivado',
      'deleted': 'Eliminado',
      'dashboard': 'Inicio',
      'archive': 'Archivar',
      'delete': 'Eliminar',
      'restore': 'Restaurar',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascendente',
      'descending': 'Descendente',
      'save': 'Guardar',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Pagado',
      'balance_due': 'Pendiente',
      'balance': 'Saldo',
      'overview': 'Overview',
      'details': 'Detalles',
      'phone': 'Teléfono',
      'website': 'Sitio Web',
      'vat_number': 'CIF/NIF',
      'id_number': 'ID Number',
      'create': 'Crear',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Contactos',
      'additional': 'Additional',
      'first_name': 'Nombres',
      'last_name': 'Apellidos',
      'add_contact': 'Añadir contacto',
      'are_you_sure': '¿Estás Seguro?',
      'cancel': 'Cancelar',
      'ok': 'Ok',
      'remove': 'Remove',
      'email_is_invalid': 'Email is invalid',
      'product': 'Producto',
      'products': 'Productos',
      'new_product': 'Nuevo Producto',
      'created_product': 'Producto creado con éxito',
      'updated_product': 'Producto actualizado con éxito',
      'archived_product': 'Producto archivado con éxito',
      'deleted_product': 'Producto actualizado con éxito',
      'restored_product': 'Producto restaurado con éxito',
      'archived_products': ':count productos archivados con éxito',
      'deleted_products': 'Eliminados con éxito :count productos',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Producto',
      'notes': 'Notas',
      'cost': 'Costo',
      'client': 'Cliente',
      'clients': 'Clientes',
      'new_client': 'Nuevo Cliente',
      'created_client': 'cliente creado con éxito',
      'updated_client': 'Cliente actualizado con éxito',
      'archived_client': 'Cliente archivado con éxito',
      'archived_clients': ':count clientes archivados con éxito',
      'deleted_client': 'Cliente eliminado con éxito',
      'deleted_clients': ':count clientes eliminados con éxito',
      'restored_client': 'Cliente restaurado con éxito',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Calle',
      'address2': 'Bloq/Pta',
      'city': 'Ciudad',
      'state': 'Región/Provincia',
      'postal_code': 'Código Postal',
      'country': 'País',
      'invoice': 'Factura',
      'invoices': 'Facturas',
      'new_invoice': 'Nueva Factura',
      'created_invoice': 'Factura creada con éxito',
      'updated_invoice': 'Factura actualizada con éxito',
      'archived_invoice': 'Factura archivada con éxito',
      'deleted_invoice': 'Factura eliminada con éxito',
      'restored_invoice': 'Factura restaurada con éxito',
      'archived_invoices': ':count facturas archivados con éxito',
      'deleted_invoices': ':count facturas eliminadas con éxito',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Factura enviada con éxito',
      'emailed_payment': 'Pago enviado por correo con éxito',
      'amount': 'Cantidad',
      'invoice_number': 'Número de Factura',
      'invoice_date': 'Fecha de Factura',
      'discount': 'Descuento',
      'po_number': 'Número de Orden',
      'terms': 'Términos',
      'public_notes': 'Notas',
      'private_notes': 'Notas Privadas',
      'frequency': 'Frequencia',
      'start_date': 'Fecha de Inicio',
      'end_date': 'Fecha de Finalización',
      'quote_number': 'Numero de cotización',
      'quote_date': 'Fecha cotización',
      'valid_until': 'Válida Hasta',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Descripción',
      'unit_cost': 'Coste unitario',
      'quantity': 'Cantidad',
      'add_item': 'Add Item',
      'contact': 'Contacto',
      'work_phone': 'Teléfono',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Fecha de Pago',
      'partial_due_date': 'Fecha de Vencimiento Parcial',
      'status': 'Estado',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Total',
      'percent': 'Porciento',
      'edit': 'Editar',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Tasa de Tarea',
      'settings': 'Configuración',
      'language': 'Language',
      'currency': 'Moneda',
      'created_at': 'Fecha de Creación',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Impuesto',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Vencido',
      'draft': 'Borrador',
      'sent': 'Enviado',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Parcial/Depósito',
      'paid': 'Pagado',
      'mark_sent': 'Marcar como enviado',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Hecho',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Modo Oscuro',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Actividad',
      'no_records_found': 'No records found',
      'clone': 'Clon',
      'loading': 'Cargando',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Términos de Pago',
      'payment_date': 'Fecha de Pago',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal de Cliente',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Remitentes',
      'initial_email': 'Email Inicial',
      'first_reminder': 'Primer Recordatorio',
      'second_reminder': 'Segundo Recordatorio',
      'third_reminder': 'Tercer Recordatorio',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Plantilla',
      'send': 'Send',
      'subject': 'Asunto',
      'body': 'Mensaje',
      'send_email': 'Enviar email',
      'email_receipt':
          'Enviar por correo electrónico el recibo de pago al cliente',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Preview',
      'customize': 'Personalizar',
      'history': 'Historial',
      'payment': 'pago',
      'payments': 'Pagos',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'Referencia de Transacción',
      'enter_payment': 'Agregar Pago',
      'new_payment': 'Ingresa el Pago',
      'created_payment': 'Pago creado con éxito',
      'updated_payment': 'Pago actualizado con éxito',
      'archived_payment': 'Pago archivado con éxito',
      'deleted_payment': 'Pago eliminado con éxito',
      'restored_payment': 'Pago restaurado con éxito',
      'archived_payments': ':count pagos archivados con éxito',
      'deleted_payments': ':count pagos eliminados con éxito',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Cotización',
      'quotes': 'Cotizaciones',
      'new_quote': 'Nueva cotización',
      'created_quote': 'Cotización creada con éxito',
      'updated_quote': 'Cotización actualizada con éxito',
      'archived_quote': 'Cotización archivada con éxito',
      'deleted_quote': 'Cotizaciónes eliminadas con éxito',
      'restored_quote': 'Cotización restaurada con éxito',
      'archived_quotes': ':count cotizaciones archivadas con exito',
      'deleted_quotes': ':count cotizaciones eliminadas con exito',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Gasto',
      'expenses': 'Gastos',
      'vendor': 'Proveedor',
      'vendors': 'Proveedores',
      'task': 'Task',
      'tasks': 'Tareas',
      'project': 'Proyecto',
      'projects': 'Proyectos',
      'activity_1': ':user creó el cliente :client',
      'activity_2': ':user archivó el cliente :client',
      'activity_3': ':user eliminó el cliente :client',
      'activity_4': ':user creó la factura :invoice',
      'activity_5': ':user actualizó la factura :invoice',
      'activity_6':
          ':user envió por correo electrónico la factura :invoice para el cliente :client a :contact',
      'activity_7': ':contact vió la factura :invoice del cliente :client',
      'activity_8': ':user archivó la factura :invoice',
      'activity_9': ':user eliminó la factura :invoice',
      'activity_10':
          ':contact ingresó el pago :payment por el valor :payment_amount en la factura :invoice del cliente :client',
      'activity_11': ':user actualizó el pago :payment',
      'activity_12': ':user archivó el pago :payment',
      'activity_13': ':user eliminó el pago :payment',
      'activity_14': ':user ingresó :credit créditos',
      'activity_15': ':user actualizó :credit créditos',
      'activity_16': ':user archivó :credit créditos',
      'activity_17': ':user eliminó :credit créditos',
      'activity_18': ':user creó la cotización :quote',
      'activity_19': ':user actualizó la cotización :quote',
      'activity_20':
          ':user envió por correo electrónico la cotización :quote a :contact',
      'activity_21': ':contact vió la cotización :quote',
      'activity_22': ':user archivó la cotización :quote',
      'activity_23': ':user eliminó la cotización :quote',
      'activity_24': ':user restauró la cotización :quote',
      'activity_25': ':user restauró factura :invoice',
      'activity_26': ':user restauró el cliente :client',
      'activity_27': ':user restauró el pago :payment',
      'activity_28': ':user restauró :credit créditos',
      'activity_29':
          ':contact aprovó la cotización :quote para el cliente :client',
      'activity_30': ':user creó al vendedor :vendor',
      'activity_31': ':user archivó al vendedor :vendor',
      'activity_32': ':user eliminó al vendedor :vendor',
      'activity_33': ':user restauró al vendedor :vendor',
      'activity_34': ':user creó expense :expense',
      'activity_35': ':user archivó el gasto :expense',
      'activity_36': ':user eliminó el gasto :expense',
      'activity_37': ':user restauró el gasto :expense',
      'activity_39': ':usaer canceló :payment_amount pago :payment',
      'activity_40':
          ':user reembolsó :adjustment de un pago de :payment_amount :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user creó la tarea :task',
      'activity_43': ':user actualizó la tarea :task',
      'activity_44': ':user archivó la tarea :task',
      'activity_45': ':user eliminó la tarea :task',
      'activity_46': ':user restauró la tarea :task',
      'activity_47': ':user actrulizó el gasto :expense',
      'activity_48': ':user actualizó el ticket :ticket',
      'activity_49': ':user cerró el ticket :ticket',
      'activity_50': ':user fusionó el ticket :ticket',
      'activity_51': ':user dividió el ticket :ticket',
      'activity_52': ':contact abrió el ticket :ticket',
      'activity_53': ':contact volvió a abrir el ticket :ticket',
      'activity_54': ':user volvió a abrir el ticket :ticket',
      'activity_55': ':contact respondió el ticket :ticket',
      'activity_56': ':user vió el ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Contraseña de una sola vez',
      'emailed_quote': 'Cotización enviada con éxito',
      'emailed_credit': 'Crédito enviado por correo electrónico con éxito',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Crédito marcado como enviado con éxito',
      'expired': 'Vencida',
      'all': 'All',
      'select': 'Seleccionar',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Valor Personalizado',
      'custom_value2': 'Valor Personalizado',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Numeración de facturación',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Numeración de Cotizaciones',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial':
          'Asunto del correo electrónico de pago parcial',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tipo',
      'invoice_amount': 'Importe de Factura',
      'invoice_due_date': 'Fecha de Vencimiento',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Cobro Automático',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Está Eliminado',
      'vendor_city': 'Ciudad del Proveedor',
      'vendor_state': 'Estado del Proveedor',
      'vendor_country': 'País del Proveedor',
      'is_approved': 'Está Aprobado',
      'tax_name': 'Nombre de Impuesto',
      'tax_amount': 'Suma de Impuestos',
      'tax_paid': 'Impuestos pagados',
      'payment_amount': 'Valor del Pago',
      'age': 'Edad',
      'is_running': 'Is Running',
      'time_log': 'Registro de Tiempo',
      'bank_id': 'banco',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Categoría de Gastos',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'es_ES': {
      'import_data': 'Importar datos',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Última Actualización',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Nombre del Contacto',
      'contact_last_name': 'Apellido del contacto',
      'order': 'Order',
      'unassigned': 'Sin asignar',
      'partial_value': 'Debe ser mayor que 0 y menos que el Total',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Activar',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Asigne el subdominio o mostrar la factura en su propio sitio web.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Código promocional',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Seguridad',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscripción',
      'subscriptions': 'Suscripciones',
      'new_subscription': 'Nueva Subscripción',
      'edit_subscription': 'Editar Suscripción',
      'created_subscription': 'Suscripción creada correctamente',
      'updated_subscription': 'Suscripción actualizada correctamente',
      'archived_subscription': 'Suscripción archivada correctamente',
      'deleted_subscription': 'Subscripción borrada correctamente',
      'removed_subscription': 'Subscripción eliminada correctamente',
      'restored_subscription': 'Subscripción restaurada correctamente',
      'search_subscription': 'Buscar 1 Subscripción',
      'search_subscriptions': 'Buscar :count Subscripciones',
      'subdomain_is_not_available': 'El subdominio no está disponible',
      'connect_gmail': 'Conectar Gmail',
      'disconnect_gmail': 'Desconectar Gmail',
      'connected_gmail': 'Gmail conectado correctamente',
      'disconnected_gmail': 'Gmail desconectado correctamente',
      'update_fail_help':
          'Cámbios en el código pueden estar bloqueando la actualización, puedes ejecutar este comando para descartar los cambios:',
      'client_id_number': 'Número ID Cliente',
      'count_minutes': ':count Minutos',
      'password_timeout': 'Caducidad de Contraseña',
      'shared_invoice_credit_counter': 'Contador de Factura/Crédito Compartido',
      'use_last_email': 'Usar último email',
      'activate_company': 'Activar Compañía',
      'activate_company_help':
          'Activar emails, facturas recurrentes y notificaciones',
      'an_error_occurred_try_again':
          'Ha ocurrido un error, por favor inténtalo de nuevo',
      'please_first_set_a_password':
          'Por favor, primero establezca una contraseña',
      'changing_phone_disables_two_factor':
          'Atención: Cambiar el número de teléfono desactivará autenticación en 2 pasos',
      'help_translate': 'Ayuda a Traducir',
      'please_select_a_country': 'Por favor, indica un país',
      'resend_invite': 'Reenviar Invitación',
      'disabled_two_factor':
          'Autenticación en 2 pasos desactivada correctamente',
      'connected_google': 'Cuenta conectada correctamente',
      'disconnected_google': 'Cuenta desconectada correctamente',
      'delivered': 'Entregado',
      'bounced': 'Rebotados',
      'spam': 'Spam',
      'view_docs': 'Ver Documentos',
      'enter_phone_to_enable_two_factor':
          'Por favor, facilita un número de teléfono para activar la autenticación en dos pasos',
      'send_sms': 'Enviar SMS',
      'sms_code': 'Código SMS',
      'two_factor_setup_help':
          'Escanea el codigo de barras con una :link aplicacion compatible',
      'enabled_two_factor':
          'Autenticacion en dos pasos habilitada correctamente',
      'connect_google': 'Conectar Google',
      'disconnect_google': 'Desconectar Google',
      'enable_two_factor': 'Autenticacion en dos pasos',
      'disable_two_factor': 'Desactivar Autenticación en 2 Pasos',
      'require_password_with_social_login':
          'Requerir contraseña con Social Login',
      'stay_logged_in': 'Permanecer Conectado',
      'session_about_to_expire': 'Atención: Tu sesión está a punto de expirar',
      'count_hours': ':count Horas',
      'count_day': '1 Día',
      'count_days': ':count Días',
      'web_session_timeout': 'Tiempo de finalización de la sesión Web',
      'security_settings': 'Opciones de Seguridad',
      'resend_email': 'Reenviar Email',
      'confirm_your_email_address': 'Por favor, confirma tu dirección de email',
      'refunded_payment': 'Pago Reembolsado',
      'partially_unapplied': 'Parcialmente sin aplicar',
      'select_a_gmail_user':
          'Por favor, selecciona un usuario autenticado con Gmail',
      'list_long_press': 'Pulsación Larga en Lista',
      'show_actions': 'Mostrar Acciones',
      'start_multiselect': 'Iniciar Multiselección',
      'email_sent_to_confirm_email':
          'Un email ha sido enviado para confirmar la dirección de correo',
      'counter_pattern_error':
          'Para usar :client_counter por favor, añade o bien :client_number o :client_id_number para evitar conflictos',
      'this_quarter': 'Trimestre Actual',
      'last_quarter': 'Trimestre Anterior',
      'to_update_run': 'Para actualizar ejecute',
      'convert_to_invoice': 'Convertir a Factura',
      'registration_url': 'URL de registro',
      'invoice_project': 'Facturar Proyecto',
      'invoice_task': 'Facturar tarea',
      'invoice_expense': 'Facturar Gasto',
      'search_payment_term': 'Buscar 1 Término de Pago',
      'search_payment_terms': 'Buscar :count Términos de Pago',
      'save_and_preview': 'Guardar y Previsualizar',
      'save_and_email': 'Guardar y Enviar',
      'supported_events': 'Eventos Soportados',
      'converted_amount': 'Cuenta convertida',
      'converted_balance': 'Balance Convertido',
      'converted_paid_to_date': 'Pagado a la fecha convertido',
      'converted_credit_balance': 'Saldo de crédito convertido',
      'converted_total': 'Total convertido',
      'is_sent': 'Enviada',
      'default_documents': 'Documents por defecto',
      'document_upload': 'Subir Documento',
      'document_upload_help': 'Activar la subida de documentos de los clientes',
      'expense_total': 'Gasto Total',
      'enter_taxes': 'Introducir Impuestos',
      'by_rate': 'Por Tarifa',
      'by_amount': 'Por Cantidad',
      'enter_amount': 'Introduce Cantidad',
      'before_taxes': 'Antes de Impuestos',
      'after_taxes': 'Después de Impuestos',
      'color': 'Color',
      'show': 'Mostrar',
      'hide': 'Ocultar',
      'empty_columns': 'Vaciar Columnas',
      'debug_mode_is_enabled': 'Modo de depuración activo',
      'debug_mode_is_enabled_help':
          'Atención: sólo está destinado para usarse en maquinas locales, puede filtrar credenciales. Pulsa para saber más.',
      'running_tasks': 'Tareas en Proceso',
      'recent_tasks': 'Tareas Recientes',
      'recent_expenses': 'Gastos Recientes',
      'upcoming_expenses': 'Próximos Gastos',
      'update_app': 'Actualizar App',
      'started_import': 'Importación iniciada correctamente',
      'duplicate_column_mapping': 'Mapeo de columnas duplicado',
      'uses_inclusive_taxes': 'Usar Impuestos Inclusivos',
      'is_amount_discount': 'Es cantidad de descuento',
      'column': 'Columna',
      'sample': 'Ejemplo',
      'map_to': 'Mapear a',
      'import': 'Importar',
      'first_row_as_column_names': 'Usar primera fila como nombres de columna',
      'select_file': 'Seleccionar archivo',
      'no_file_selected': 'No hay archivos seleccionados',
      'csv_file': 'Seleccionar archivo CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Contabilidad',
      'required_files_missing': 'Por favor facilita todos los CSVs.',
      'import_type': 'Importar Tipo',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'Ver Licencias',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Editor a Pantalla Completa',
      'sidebar_editor': 'Editor de Barra Lateral',
      'please_type_to_confirm': 'Por favor, escribe \":value\" para confirmar',
      'purge': 'Purgar',
      'service': 'Servicio',
      'clone_to': 'Clonar a',
      'clone_to_other': 'Clonar a otra',
      'labels': 'Etiquetas',
      'add_custom': 'Añadir Personalizado',
      'payment_tax': 'Impuesto de Pago',
      'unpaid': 'Impagado',
      'white_label': 'Marca Blanca',
      'delivery_note': 'Nota para el envio',
      'sent_invoices_are_locked': 'Las facturas enviadas están bloqueadas',
      'paid_invoices_are_locked': 'Las facturas pagadas están bloqueadas',
      'source_code': 'Código Fuente',
      'app_platforms': 'Añadir Plataformas',
      'invoice_late': 'Atraso de Factura',
      'quote_expired': 'Presupuesto Expirado',
      'partial_due': 'Adelanto',
      'invoice_total': 'Total Facturado',
      'quote_total': 'Total Presupuestado',
      'credit_total': 'Crédito Total',
      'recurring_invoice_total': 'Total Factura',
      'actions': 'Acciones',
      'expense_number': 'Número de Gasto',
      'task_number': 'Número de Tarea',
      'project_number': 'Número de Proyecto',
      'project_name': 'Nombre de Proyecto',
      'warning': 'Advertencia',
      'view_settings': 'Ver Configuración',
      'company_disabled_warning':
          'Advertencia: esta compañía aún no ha sido activada',
      'late_invoice': 'Factura Atrasada',
      'expired_quote': 'Presupuesto Expirado',
      'remind_invoice': 'Recordar Factura',
      'cvv': 'CVV',
      'client_name': 'Nombre del Cliente',
      'client_phone': 'Teléfono del Cliente',
      'required_fields': 'Campos Requeridos',
      'calculated_rate': 'Tasa Calculada',
      'default_task_rate': 'Tarifa de Tarea por Defecto',
      'clear_cache': 'Borrar Caché',
      'sort_order': 'Orden Clasificación',
      'task_status': 'Estado',
      'task_statuses': 'Estados de Tarea',
      'new_task_status': 'Nuevo Estado de Tarea',
      'edit_task_status': 'Editar Estado de Tarea',
      'created_task_status': 'Estado de tarea creado correctamente',
      'updated_task_status': 'Se actualizó correctamente el estado de la tarea',
      'archived_task_status': 'Estado de tarea archivado correctamente',
      'deleted_task_status': 'Estado de tarea borrado correctamente',
      'removed_task_status': 'Estado de tarea eliminado correctamente',
      'restored_task_status': 'Estado de tarea restaurado correctamente',
      'archived_task_statuses':
          ':value estados de tarea archivados correctamente',
      'deleted_task_statuses': ':value estados de tarea borrados correctamente',
      'restored_task_statuses':
          ':value estados de tarea restaurados correctamente',
      'search_task_status': 'Buscar 1 Estado de Tarea',
      'search_task_statuses': 'Buscar :count Estados de Tarea',
      'show_tasks_table': 'Mostrar Tabla de Tareas',
      'show_tasks_table_help':
          'Mostrar siempre la sección de tareas cuando se creen facturas',
      'invoice_task_timelog': 'Registro de Tiempo de Tarea Facturada',
      'invoice_task_timelog_help':
          'Añadir detalles de tiempo a los artículos de línea de factura',
      'invoice_task_datelog': 'Fecha de Tarea en Factura',
      'invoice_task_datelog_help':
          'Añadir detalles de fecha a los artículos de línea de la factura',
      'auto_start_tasks_help': 'Empezar tareas antes de guardar',
      'configure_statuses': 'Configurar Estados',
      'task_settings': 'Configuración de Tareas',
      'configure_categories': 'Configurar Categorías',
      'expense_categories': 'Categorías de Gasto',
      'new_expense_category': 'Nueva Categoría de Gasto',
      'edit_expense_category': 'Editar Categoría de Gasto',
      'created_expense_category': 'Categoría de gasto creada correctamente',
      'updated_expense_category':
          'Categoría de gasto actualizada correctamente',
      'archived_expense_category': 'Categoría de gasto archivada correctamente',
      'deleted_expense_category': 'Categoría eliminada correctamente',
      'removed_expense_category': 'Categoría de gasto eliminada correctamente',
      'restored_expense_category':
          'Categoría de Gasto restaurada correctamente',
      'archived_expense_categories':
          ':count categorías de gasto actualizados correctamente',
      'deleted_expense_categories':
          ':value categorías de gasto borradas correctamente',
      'restored_expense_categories':
          ':value categorías de gasto restauradas correctamente',
      'search_expense_category': 'Buscar 1 Categoría de Gasto',
      'search_expense_categories': 'Buscar :count Categorías de Gasto',
      'use_available_credits': 'Usar Crédito Disponible',
      'show_option': 'Mostrar Opción',
      'negative_payment_error':
          'La cantidad de crédito no puede exceder la cantidada pagada',
      'view_changes': 'Ver Cambios',
      'force_update': 'Forzar Actualización',
      'force_update_help':
          'Estás usando la última versión, pero puede haber corrección de errores pendientes.',
      'mark_paid_help': 'Seguir que la factura haya sido pagada',
      'should_be_invoiced': 'Debe ser Facturado',
      'should_be_invoiced_help': 'Activar que los gastos sean facturables',
      'add_documents_to_invoice_help': 'Hacer los documentos visibles',
      'convert_currency_help': 'Establecer un tipo de cambio',
      'expense_settings': 'Configuración de Gastos',
      'clone_to_recurring': 'Clonar a Recurrente',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'Campo de Usuario',
      'variables': 'Variables',
      'show_password': 'Mostrar Contraseña',
      'hide_password': 'Ocultar Contraseña',
      'copy_error': 'Copiar Error',
      'capture_card': 'Capturar Tarjeta',
      'auto_bill_enabled': 'Activar Auto Facturación',
      'total_taxes': 'Impuestos Totales',
      'line_taxes': 'Impuestos de Línea',
      'total_fields': 'Campos Totales',
      'stopped_recurring_invoice':
          'Se ha parado la factura recurrente correctamente',
      'started_recurring_invoice':
          'Se ha iniciado la factura recurrente correctamente',
      'resumed_recurring_invoice':
          'Se ha reiniciado la factura recurrente correctamente',
      'gateway_refund': 'Pasarela de Devolución',
      'gateway_refund_help': 'Procesar la devolución con la pasarela de pago',
      'due_date_days': 'Fecha de Vencimiento',
      'paused': 'Pausado',
      'mark_active': 'Marcar como Activo',
      'day_count': 'Día :count',
      'first_day_of_the_month': 'Primer Día del Mes',
      'last_day_of_the_month': 'Último Día del Mes',
      'use_payment_terms': 'Usar Términos de Pago',
      'endless': 'Sin Fín',
      'next_send_date': 'Próxima Fecha de Envío',
      'remaining_cycles': 'Ciclos Pendientes',
      'recurring_invoice': 'Factura Recurrente',
      'recurring_invoices': 'Facturas Recurrentes',
      'new_recurring_invoice': 'Nueva Factura Recurrente',
      'edit_recurring_invoice': 'Editar Factura Recurrente',
      'created_recurring_invoice': 'Factura recurrente creada correctamente',
      'updated_recurring_invoice':
          'Factura recurrente actualizada correctamente',
      'archived_recurring_invoice':
          'Factura recurrente archivada correctamente',
      'deleted_recurring_invoice': 'Factura recurrente borrada correctamente',
      'removed_recurring_invoice': 'Factura recurrente eliminada correctamente',
      'restored_recurring_invoice':
          'Factura recurrente restaurada correctamente',
      'archived_recurring_invoices':
          ':value facturas recurrentes archivadas correctamente',
      'deleted_recurring_invoices':
          ':value facturas recurrentes borradas correctamente',
      'restored_recurring_invoices':
          ':value facturas recurrentes restauradas correctamente',
      'search_recurring_invoice': 'Buscar 1 Factura Recurrente',
      'search_recurring_invoices': 'Buscar :count Facturas Recurrentes',
      'send_date': 'Fecha de Envío',
      'auto_bill_on': 'Facturación Automática Activa',
      'minimum_under_payment_amount': 'Cantidad Mínima de Pago',
      'profit': 'Beneficio',
      'line_item': 'Linea de Concepto',
      'allow_over_payment': 'Permitir Sobrepago',
      'allow_over_payment_help': 'Permitir pagos extra para aceptar propinas',
      'allow_under_payment': 'Permitir Pago de Menos',
      'allow_under_payment_help':
          'Permitir pagar como mínimo la cantidad parcial/depósito',
      'test_mode': 'Modo Test',
      'opened': 'Abiertos',
      'payment_reconciliation_failure': 'Fallo de Conciliación',
      'payment_reconciliation_success': 'Concilicación correcta',
      'gateway_success': 'Éxito de pasarela',
      'gateway_failure': 'Fallo de Pasarela',
      'gateway_error': 'Error de Pasarela',
      'email_send': 'Email Enviado',
      'email_retry_queue': 'Cola de Reenvío de Email',
      'failure': 'Fallo',
      'quota_exceeded': 'Cuota Excedida',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'Registros del Sistema',
      'view_portal': 'Ver portal',
      'copy_link': 'Copiar Enlace',
      'token_billing': 'Guardar datos de la tarjeta',
      'welcome_to_invoice_ninja': 'Bienvenid@ a Invoice Ninja',
      'always': 'Siempre',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Etiqueta',
      'client_number': 'Código de Cliente',
      'auto_convert': 'Auto Convertir',
      'company_name': 'Nombre de la Empresa',
      'reminder1_sent': 'Recordatorio 1, enviado',
      'reminder2_sent': 'Recordatorio 2, enviado',
      'reminder3_sent': 'Recordatorio 3, enviado',
      'reminder_last_sent': 'Último recordatorio enviado',
      'pdf_page_info': 'Página :current de :total',
      'emailed_invoices': 'Facturas enviadas correctamente',
      'emailed_quotes': 'Presupuestos enviados correctamente',
      'emailed_credits': 'Créditos enviados correctamente',
      'gateway': 'Pasarela',
      'view_in_stripe': 'Ver en Stripe',
      'rows_per_page': 'Filas por Página',
      'hours': 'horas',
      'statement': 'Estado de cuenta',
      'taxes': 'Impuestos',
      'surcharge': 'Recargo',
      'apply_payment': 'Aplicar Pago',
      'apply': 'Aplicar',
      'unapplied': 'Sin Aplicar',
      'select_label': 'Seleccionar etiqueta',
      'custom_labels': 'Etiquetas Personalizadas',
      'record_type': 'Tipo de Registro',
      'record_name': 'Nombre de Registro',
      'file_type': 'Tipo de Archivo',
      'height': 'Altura',
      'width': 'Anchura',
      'to': 'Para',
      'health_check': 'Consultar Estado de Sistema',
      'payment_type_id': 'Tipo de Pago',
      'last_login_at': 'Último Acceso el',
      'company_key': 'Clave de empresa',
      'storefront': 'Escaparate',
      'storefront_help': 'Activar apps de terceros para crear facturas',
      'client_created': 'Cliente Creado',
      'online_payment_email': 'Email de Pago Online',
      'manual_payment_email': 'Email de Pago Manual',
      'completed': 'Completado',
      'gross': 'Bruto',
      'net_amount': 'Importe Neto',
      'net_balance': 'Balance Neto',
      'client_settings': 'Configuración de Cliente',
      'selected_invoices': 'Facturas Seleccionadas',
      'selected_payments': 'Pagos Seleccionados',
      'selected_quotes': 'Presupuestos Seleccionados',
      'selected_tasks': 'Tareas Seleccionadas',
      'selected_expenses': 'Gastos Seleccionados',
      'upcoming_invoices': 'Próximas Facturas',
      'past_due_invoices': 'Facturas Fuera de Plazo',
      'recent_payments': 'Pagos recientes',
      'upcoming_quotes': 'Próximos Presupuestos',
      'expired_quotes': 'Presupuestos Expirados',
      'create_client': 'Crear cliente',
      'create_invoice': 'Crear Factura',
      'create_quote': 'Crear Presupuesto',
      'create_payment': 'Crear Pago',
      'create_vendor': 'Crear Proveedor',
      'update_quote': 'Actualizar Presupuesto',
      'delete_quote': 'Eliminar Presupuesto',
      'update_invoice': 'Actualizar Factura',
      'delete_invoice': 'Eliminar Factura',
      'update_client': 'Actualizar Cliente',
      'delete_client': 'Eliminar Cliente',
      'delete_payment': 'Eliminar Pago',
      'update_vendor': 'Actualizar Proveedor',
      'delete_vendor': 'Borrar Proveedor',
      'create_expense': 'Crear Gasto',
      'update_expense': 'Actualizar Gasto',
      'delete_expense': 'Borrar Gasto',
      'create_task': 'Crear Tarea',
      'update_task': 'Actualizar Tarea',
      'delete_task': 'Borrar Tarea',
      'approve_quote': 'Aprobar Presupuesto',
      'off': 'Apagado',
      'when_paid': 'Al Pagar',
      'expires_on': 'Expira el',
      'free': 'Gratuito',
      'plan': 'Plan',
      'show_sidebar': 'Mostrar Barra Lateral',
      'hide_sidebar': 'Ocultar Barra Lateral',
      'event_type': 'Tipo de Evento',
      'target_url': 'objetivo',
      'copy': 'Copiar',
      'must_be_online':
          'Por favor reinicia la app cuando te conectes a internet',
      'crons_not_enabled': 'La tarea cron debe ser activada',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Buscar :count Webhooks',
      'search_webhook': 'Buscar 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'Nuevo Webhook',
      'edit_webhook': 'Editar Webhook',
      'created_webhook': 'Webhook creado correctamente',
      'updated_webhook': 'Webhook actualizado correctamente',
      'archived_webhook': 'Webhook archivado correctamente',
      'deleted_webhook': 'Webhook borrado correctamente',
      'removed_webhook': 'Webhook eliminado correctamente',
      'restored_webhook': 'Webhook restaurado correctamente',
      'archived_webhooks': ':value webhooks archivados correctamente',
      'deleted_webhooks': ':value webhooks borrados correctamente',
      'removed_webhooks': ':value webhooks eliminados correctamente',
      'restored_webhooks': ':value webhooks restaurados correctamente',
      'api_tokens': 'API Tokens',
      'api_docs': 'Documentación de API',
      'search_tokens': 'Buscar :count Tokens',
      'search_token': 'Buscar 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'Nuevo Token',
      'edit_token': 'Editar Token',
      'created_token': 'Token creado correctamente',
      'updated_token': 'Token actualizado correctamente',
      'archived_token': 'Token archivado correctamente',
      'deleted_token': 'Token eliminado correctamente',
      'removed_token': 'Token eliminado correctamente',
      'restored_token': 'Token restaurado correctamente',
      'archived_tokens': ':value tokens archivados correctamente',
      'deleted_tokens': ':value tokens borrados correctamente',
      'restored_tokens': ':value tokens restaurados correctamente',
      'client_registration': 'Registro de Cliente',
      'client_registration_help':
          'Permitir a los clientes auto-registrarse en el portal',
      'customize_and_preview': 'Personalizar y Previsualizar',
      'email_invoice': 'Enviar Factura por EMail',
      'email_quote': 'Enviar Presupuesto',
      'email_credit': 'Enviar Crédito',
      'email_payment': 'Pago por correo electrónico',
      'client_email_not_set':
          'El cliente no tiene establecida una dirección de email',
      'ledger': 'Libro Mayor',
      'view_pdf': 'Ver PDF',
      'all_records': 'Todos los registros',
      'owned_by_user': 'Propiedad del usuario',
      'credit_remaining': 'Crédito Restante',
      'contact_name': 'Nombre del Contacto',
      'use_default': 'Usar por defecto',
      'reminder_endless': 'Recordatorios Sin Fín',
      'number_of_days': 'Número de días',
      'configure_payment_terms': 'Configurar Términos de Pago',
      'payment_term': 'Término de Pago',
      'new_payment_term': 'Nuevo Término de Pago',
      'edit_payment_term': 'Editar el Términos de Pago',
      'created_payment_term': 'Términos de pago creados correctamente',
      'updated_payment_term': 'Términos de pago actualizados correctamente',
      'archived_payment_term': 'Términos de pago archivados correctamente',
      'deleted_payment_term': 'Término de pago borrado correctamente',
      'removed_payment_term': 'Término de pago eliminado correctamente',
      'restored_payment_term': 'Término de pago restaurado correctamente',
      'archived_payment_terms':
          ':value términos de pago archivados correctamente',
      'deleted_payment_terms': ':value téminos de pago borrados correctamente',
      'restored_payment_terms':
          ':value téminos de pago restaurados correctamente',
      'email_sign_in': 'Ingresar con email',
      'change': 'Cambiar',
      'change_to_mobile_layout': '¿Cambiar al formato de móvil?',
      'change_to_desktop_layout': '¿Cambiar al formato de escritorio?',
      'send_from_gmail': 'Enviar desde Gmail',
      'reversed': 'Revertida',
      'cancelled': 'Cancelada',
      'credit_amount': 'Cantidad de Crédito',
      'quote_amount': 'Total de Presupuesto',
      'hosted': 'Hospedado',
      'selfhosted': 'Hospedaje Propio',
      'exclusive': 'Exclusivo',
      'inclusive': 'Inclusivo',
      'hide_menu': 'Ocultar Menú',
      'show_menu': 'Mostrar Menú',
      'partially_refunded': 'Parcialmente Reintegrada',
      'search_documents': 'Buscar Documentos',
      'search_designs': 'Buscar Diseños',
      'search_invoices': 'Buscar Facturas',
      'search_clients': 'Buscar Clientes',
      'search_products': 'Buscar Productos',
      'search_quotes': 'Buscar Presupuestos',
      'search_credits': 'Buscar Créditos',
      'search_vendors': 'Buscar Proveedores',
      'search_users': 'Buscar Usuarios',
      'search_tax_rates': 'Buscar Tipos de Impuesto',
      'search_tasks': 'Buscar Tareas',
      'search_settings': 'Buscar Opciones',
      'search_projects': 'Buscar Proyectos',
      'search_expenses': 'Buscar Gastos',
      'search_payments': 'Buscar Pagos',
      'search_groups': 'Buscar Grupos',
      'search_company': 'Buscar Compañía',
      'search_document': 'Buscar 1 Documento',
      'search_design': 'Buscar 1 Diseño',
      'search_invoice': 'Buscar 1 Factura',
      'search_client': 'Buscar 1 Cliente',
      'search_product': 'Buscar 1 Producto',
      'search_quote': 'Buscar 1 Presupuesto',
      'search_credit': 'Buscar 1 Crédito',
      'search_vendor': 'Buscar 1 Proveedor',
      'search_user': 'Buscar 1 Usuario',
      'search_tax_rate': 'Buscar 1 Tipo de Impuesto',
      'search_task': 'Buscar 1 Tarea',
      'search_project': 'Buscar 1 Proyecto',
      'search_expense': 'Buscar 1 Gasto',
      'search_payment': 'Buscar 1 Pago',
      'search_group': 'Buscar 1 Grupo',
      'refund_payment': 'Reembolsar Pago',
      'cancelled_invoice': 'Factura cancelada correctamente',
      'cancelled_invoices': 'Facturas canceladas correctamente',
      'reversed_invoice': 'Factura revertida correctamente',
      'reversed_invoices': 'Facturas revertidas correctamente',
      'reverse': 'Revertir',
      'full_name': 'Nombre completo',
      'city_state_postal': 'Ciudad / Provincia / C.Postal',
      'postal_city_state': 'C.Postal / Ciudad / Provincia',
      'custom1': 'Primera personalización',
      'custom2': 'Segunda personalización',
      'custom3': 'Tercera Personalización',
      'custom4': 'Cuarta Personalización',
      'optional': 'Opcional',
      'license': 'Licencia',
      'purge_data': 'Purgar Datos',
      'purge_successful': 'Datos de la empresa purgados correctamente',
      'purge_data_message':
          'Advertencia: Esto borrará definitivamente sus datos, no hay deshacer.',
      'invoice_balance': 'Balance de Factura',
      'age_group_0': '0 - 30 Días',
      'age_group_30': '30 - 60 Días',
      'age_group_60': '60 - 90 Días',
      'age_group_90': '90 - 120 Días',
      'age_group_120': '120+ Días',
      'refresh': 'Refrescar',
      'saved_design': 'Diseño guardado correctamente',
      'client_details': 'Detalles de Cliente',
      'company_address': 'Dirección de Compañía',
      'invoice_details': 'Detalles de Factura',
      'quote_details': 'Detalles del Presupuesto',
      'credit_details': 'Detalles de Crédito',
      'product_columns': 'Columnas de Producto',
      'task_columns': 'Columnas de Tarea',
      'add_field': 'Añadir Campo',
      'all_events': 'Todos los Eventos',
      'permissions': 'Permisos',
      'none': 'Ninguno',
      'owned': 'Propietario',
      'payment_success': 'Pago realizado con éxito',
      'payment_failure': 'Fallo de Pago',
      'invoice_sent': 'Factura :count enviada',
      'quote_sent': 'Prespuesto Enviado',
      'credit_sent': 'Crédito Enviado',
      'invoice_viewed': 'Factura Vista',
      'quote_viewed': 'Presupuesto Visto',
      'credit_viewed': 'Crédito Visto',
      'quote_approved': 'Presupuesto Aprobado',
      'receive_all_notifications': 'Recibir Todas las Notificaciones',
      'purchase_license': 'Comprar Licencia',
      'apply_license': 'Renovar licencia',
      'cancel_account': 'Cancelar Cuenta',
      'cancel_account_message':
          'Atención: Esta acción eliminará permanentemente tu cuenta y no se podrá deshacer.',
      'delete_company': 'Borrar Compañía',
      'delete_company_message':
          'Advertencia: esto eliminará definitivamente su empresa, no hay deshacer.',
      'enabled_modules': 'Modulos Activados',
      'converted_quote': 'Presupuesto convertido correctamente',
      'credit_design': 'Diseño de Crédito',
      'includes': 'Incluye',
      'header': 'Cabecera',
      'load_design': 'Cargar diseño',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Diseños Personalizados',
      'designs': 'Diseños',
      'new_design': 'Nuevo Diseño',
      'edit_design': 'Editar Diseño',
      'created_design': 'Diseño creado correctamente',
      'updated_design': 'Diseño actualizado correctamente',
      'archived_design': 'Diseño archivado correctamente',
      'deleted_design': 'Diseño borrado correctamente',
      'removed_design': 'Diseño eliminado correctamente',
      'restored_design': 'Diseño restaurado correctamente',
      'archived_designs': ':value diseños archivados correctamente',
      'deleted_designs': ':value diseños borrados correctamente',
      'restored_designs': ':value diseños restaurados correctamente',
      'proposals': 'Propuestas',
      'tickets': 'Tickets',
      'recurring_quotes': 'Presupuestos Recurrentes',
      'recurring_tasks': 'Tareas Recurrentes',
      'recurring_expenses': 'Gastos Periódicos',
      'account_management': 'Administración de la Cuenta',
      'credit_date': 'Fecha de Crédito',
      'credit': 'Crédito',
      'credits': 'Créditos',
      'new_credit': 'Introducir el Crédito',
      'edit_credit': 'Editar Crédito',
      'created_credit': 'Crédito creado correctamente',
      'updated_credit': 'Crédito actualizado correctamente',
      'archived_credit': 'Crédito archivado correctamente',
      'deleted_credit': 'Créditos eliminados correctamente',
      'removed_credit': 'Crédito eliminado correctamente',
      'restored_credit': 'Crédito restaurado correctamente',
      'archived_credits': ':count créditos archivados correctamente',
      'deleted_credits': ':count créditos eliminados correctamente',
      'restored_credits': ':value créditos restaurados correctamente',
      'current_version': 'Versión Actual',
      'latest_version': 'Última Versión',
      'update_now': 'Actualizar Ahora',
      'a_new_version_is_available':
          'Una nueva versión de la aplicación web está disponible',
      'update_available': 'Actualización Disponible',
      'app_updated': 'Actualización completada correctamente',
      'learn_more': 'Saber más',
      'integrations': 'Integraciones',
      'tracking_id': 'Id seguimiento',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Pie de Página de Crédito',
      'credit_terms': 'Términos de Crédito',
      'new_company': 'Nueva Compañía',
      'added_company': 'Compañía añadida correctamente',
      'company1': 'Compañía Personalizada 1',
      'company2': 'Compañía Personalizada 2',
      'company3': 'Compañía Personalizada 3',
      'company4': 'Compañía Personalizada 4',
      'product1': 'Producto Personalizado 1',
      'product2': 'Producto Personalizado 2',
      'product3': 'Producto Personalizado 3',
      'product4': 'Producto Personalizado 4',
      'client1': 'Cliente Personalizado 1',
      'client2': 'Cliente Personalizado 2',
      'client3': 'Cliente Personalizado 3',
      'client4': 'Cliente Personalizado 4',
      'contact1': 'Contacto Personalizado 1',
      'contact2': 'Contacto Personalizado 2',
      'contact3': 'Contacto Personalizado 3',
      'contact4': 'Contacto Personalizado 4',
      'task1': 'Tarea Personalizada 1',
      'task2': 'Tarea Personalizada 2',
      'task3': 'Tarea Personalizada 3',
      'task4': 'Tarea Personalizada 4',
      'project1': 'Proyecto Personalizado 1',
      'project2': 'Proyecto Personalizado 2',
      'project3': 'Proyecto Personalizado 3',
      'project4': 'Proyecto Personalizado 4',
      'expense1': 'Gasto Personalizado 1',
      'expense2': 'Gasto Personalizado 2',
      'expense3': 'Gasto Personalizado 3',
      'expense4': 'Gasto Personalizado 4',
      'vendor1': 'Proveedor Personalizado 1',
      'vendor2': 'Proveedor Personalizado 2',
      'vendor3': 'Proveedor Personalizado 3',
      'vendor4': 'Proveedor Personalizado 4',
      'invoice1': 'Factura Personalizada 1',
      'invoice2': 'Factura Personalizada 2',
      'invoice3': 'Factura Personalizada 3',
      'invoice4': 'Factura Personalizada 4',
      'payment1': 'Pago Personalizado 1',
      'payment2': 'Pago Personalizado 2',
      'payment3': 'Pago Personalizado 3',
      'payment4': 'Pago Personalizado 4',
      'surcharge1': 'Recargo Personalizado 1',
      'surcharge2': 'Recargo Personalizado 2',
      'surcharge3': 'Recargo Personalizado 3',
      'surcharge4': 'Recargo Personalizado 4',
      'group1': 'Grupo Personalizado 1',
      'group2': 'Grupo Personalizado 2',
      'group3': 'Grupo Personalizado 3',
      'group4': 'Grupo Personalizado 4',
      'reset': 'Restaurar',
      'number': 'Número',
      'export': 'Exportar',
      'chart': 'Gráfica',
      'count': 'Recuento',
      'totals': 'Totales',
      'blank': 'Vacio',
      'day': 'Día',
      'month': 'Mes',
      'year': 'Año',
      'subgroup': 'Subgrupo',
      'is_active': 'Activo',
      'group_by': 'Agrupar por',
      'credit_balance': 'Saldo de Crédito',
      'contact_last_login': 'Último Acceso de Contacto',
      'contact_full_name': 'Nombre Completo de Contacto',
      'contact_phone': 'Teléfono del Contacto',
      'contact_custom_value1': 'Contacto Personalizado 1',
      'contact_custom_value2': 'Contacto Personalizado 2',
      'contact_custom_value3': 'Contacto Personalizado 3',
      'contact_custom_value4': 'Contacto Personalizado 4',
      'shipping_address1': 'Calle de Envio',
      'shipping_address2': 'Piso de Envio',
      'shipping_city': 'Ciudad de Envio',
      'shipping_state': 'Provincia de Envio',
      'shipping_postal_code': 'Cod. Postal de Envio',
      'shipping_country': 'Pais de Envio',
      'billing_address1': 'Calle de Facturacion',
      'billing_address2': 'Piso de Facturacion',
      'billing_city': 'Ciudad de Facturacion',
      'billing_state': 'Provincia de Facturacion',
      'billing_postal_code': 'Cod. Postal de Facturacion',
      'billing_country': 'Pais de Facturacion',
      'client_id': 'Id del cliente',
      'assigned_to': 'Asignado a',
      'created_by': 'Creado por :name',
      'assigned_to_id': 'Asignado a Id',
      'created_by_id': 'Creado por Id',
      'add_column': 'Añadir Columna',
      'edit_columns': 'Editar Columnas',
      'columns': 'Columnas',
      'aging': 'Envejecimiento',
      'profit_and_loss': 'Ganancias y Pérdidas',
      'reports': 'Informes',
      'report': 'Informe',
      'add_company': 'Añadir Compañía',
      'unpaid_invoice': 'Factura Impagada',
      'paid_invoice': 'Factura Pagada',
      'unapproved_quote': 'Presupuesto No Aprobado',
      'help': 'Ayuda',
      'refund': 'Reembolo',
      'refund_date': 'Fecha de Reembolso',
      'filtered_by': 'Filtrado por',
      'contact_email': 'Email del Contacto',
      'multiselect': 'Multiselección',
      'entity_state': 'Estado',
      'verify_password': 'Verificar Contraseña',
      'applied': 'Aplicado',
      'include_recent_errors': 'Incluir errores recientes de los registros',
      'your_message_has_been_received':
          'Hemos recibido tu mensaje e intentaremos responderte cuanto antes.',
      'message': 'Mensaje',
      'from': 'De',
      'show_product_details': 'Mostrar Detalles de Producto',
      'show_product_details_help':
          'Incluir la descripción y el coste en el desplegable del producto',
      'pdf_min_requirements': 'El renderizador de PDF requiere :version',
      'adjust_fee_percent': 'Ajustar Porcentaje de Tarifa',
      'adjust_fee_percent_help':
          'Ajustar el porcentaje para dar cuenta de la tarifa',
      'configure_settings': 'Configurar Opciones',
      'support_forum': 'Foro de soporte',
      'about': 'Acerca de',
      'documentation': 'Documentación',
      'contact_us': 'Contácte con Nosotros',
      'subtotal': 'Subtotal',
      'line_total': 'Total',
      'item': 'Concepto',
      'credit_email': 'Correo electrónico de crédito',
      'iframe_url': 'Website',
      'domain_url': 'URL del Dominio',
      'password_is_too_short': 'La contraseña es demasiado corta',
      'password_is_too_easy':
          'La contraseña debe contener una letra mayúscula y un número',
      'client_portal_tasks': 'Tareas del Portal Cliente',
      'client_portal_dashboard': 'Escritorio del Portal Cliente',
      'please_enter_a_value': 'Por favor, introduzca un valor',
      'deleted_logo': 'Logo borrado correctamente',
      'yes': 'Sí',
      'no': 'No',
      'generate_number': 'Generar Número',
      'when_saved': 'Al Guardar',
      'when_sent': 'Al Enviar',
      'select_company': 'Seleccionar Compañía',
      'float': 'Flotante',
      'collapse': 'Ocultar',
      'show_or_hide': 'Mostrar/Ocultar',
      'menu_sidebar': 'Menú en Barra Lateral',
      'history_sidebar': 'Histórico en Barra Lateral',
      'tablet': 'Tableta',
      'mobile': 'Móvil',
      'desktop': 'Escritorio',
      'layout': 'Diseño',
      'view': 'Ver',
      'module': 'Modulo',
      'first_custom': 'Primera Personalización',
      'second_custom': 'Segunda Personalización',
      'third_custom': 'Tercera Personalización',
      'show_cost': 'Mostrar Coste',
      'show_product_cost': 'Mostrar Coste de Producto',
      'show_cost_help':
          'Mostrar un campo de coste de producto para seguir el margen/beneficio',
      'show_product_quantity': 'Mostrar Cantidad de Productos',
      'show_product_quantity_help':
          'Mostrar un campo de cantidad de productos, de lo contrario predeterminar a uno',
      'show_invoice_quantity': 'Mostrar Cantidad de Factura',
      'show_invoice_quantity_help':
          'Mostrar un campo de cantidad de artículo de línea; de lo contrario, el valor predeterminado es uno',
      'show_product_discount': 'Mostrar Descuento de Producto',
      'show_product_discount_help':
          'Mostrar un campo de descuento en la línea de artículo',
      'default_quantity': 'Cantidad por Defecto',
      'default_quantity_help':
          'Poner la cantidad de artículos automáticamente a uno',
      'one_tax_rate': 'Un Tipo de Impuesto',
      'two_tax_rates': 'Dos Tipos de Impuesto',
      'three_tax_rates': 'Tres Tipos de Impuesto',
      'default_tax_rate': 'Impuesto por Defecto',
      'user': 'Usuario',
      'invoice_tax': 'Impuesto de Factura',
      'line_item_tax': 'Impuesto de Artículo',
      'inclusive_taxes': 'Impuestos Inclusivos',
      'invoice_tax_rates': 'Tipos de Impuesto de Factura',
      'item_tax_rates': 'Tipos de Impuesto de Artículo',
      'no_client_selected': 'Por favor seleccione un cliente',
      'configure_rates': 'Configurar tipos',
      'configure_gateways': 'Configurar Pasarelas',
      'tax_settings': 'Configuración de Impuestos',
      'tax_settings_rates': 'Tipos de Impuesto',
      'accent_color': 'Color de Acento',
      'switch': 'Cambiar',
      'comma_sparated_list': 'Lista separada por comas',
      'options': 'Opciones',
      'single_line_text': 'Texto de una sola línea',
      'multi_line_text': 'Texto de líneas múltiples',
      'dropdown': 'Desplegable',
      'field_type': 'Tipo de Campo',
      'recover_password_email_sent':
          'Se ha enviado un email de recuperación de contraseña',
      'submit': 'Enviar',
      'recover_password': 'Recuperar Contraseña',
      'late_fees': 'Cargos por pagos atrasados',
      'credit_number': 'Código de Crédito',
      'payment_number': 'Nº de Pago',
      'late_fee_amount': 'Cargo por pago atrasado',
      'late_fee_percent': 'Porcentaje por pago atrasado',
      'schedule': 'Programar',
      'before_due_date': 'Antes de la fecha de vencimiento',
      'after_due_date': 'Después de la fecha de vencimiento',
      'after_invoice_date': 'Después de la fecha de la factura',
      'days': 'Días',
      'invoice_email': 'Email de Facturas',
      'payment_email': 'Email de Pagos',
      'partial_payment': 'Pago Parcial',
      'payment_partial': 'Pago Parcial',
      'partial_payment_email': 'Correo electrónico de pago parcial',
      'quote_email': 'Email de Presupuestos',
      'endless_reminder': 'Recordatorio Sin Fín',
      'filtered_by_user': 'Filtrado por usuario',
      'administrator': 'Administrador',
      'administrator_help':
          'Permitir que administre usuarios, cambie configuración y modifique cualquier registro',
      'user_management': 'Administración de Usuarios',
      'users': 'Usuarios',
      'new_user': 'Nuevo Usuario',
      'edit_user': 'Editar Usario',
      'created_user': 'Usuario creado con éxito',
      'updated_user': 'Usario actualizado correctamente',
      'archived_user': 'Usuario archivado correctamente',
      'deleted_user': 'Usario eliminado correctamente',
      'removed_user': 'Usuario eliminado correctamente',
      'restored_user': 'Usuario restaurado correctamente',
      'archived_users': ':value usuarios archivados correctamente',
      'deleted_users': ':value usuarios borrados correctamente',
      'removed_users': ':value usuarios eliminados correctamente',
      'restored_users': ':value usuarios restaurados correctamente',
      'general_settings': 'Configuración General',
      'invoice_options': 'Opciones de Factura',
      'hide_paid_to_date': 'Ocultar el valor Pagado a la Fecha',
      'hide_paid_to_date_help':
          'Solo mostrará el valor Pagado a la Fecha en sus Facturas cuando se ha recibido un Pago.',
      'invoice_embed_documents': 'Documentos anexados',
      'invoice_embed_documents_help': 'Incluye imagenes adjuntas en la factura',
      'all_pages_header': 'Mostrar Cabecera en',
      'all_pages_footer': 'Mostrar Pie en',
      'first_page': 'Primera página',
      'all_pages': 'Todas las páginas',
      'last_page': 'Última página',
      'primary_font': 'Fuente primaria',
      'secondary_font': 'Fuente secundaria',
      'primary_color': 'Color Primario',
      'secondary_color': 'Color Secundario',
      'page_size': 'Tamaño de Pagina',
      'font_size': 'Tamaño de Letra',
      'quote_design': 'Diseños del presupuesto',
      'invoice_fields': 'Campos de Factura',
      'product_fields': 'Campos de Producto',
      'invoice_terms': 'Términos de Facturación',
      'invoice_footer': 'Pie de Página de la Factura',
      'quote_terms': 'Términos del Presupuesto',
      'quote_footer': 'Pie del Presupuesto',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automáticamente enviar por email facturas recurrentes cuando sean creadas.',
      'auto_archive_invoice': 'Auto Archivar',
      'auto_archive_invoice_help':
          'Automáticamente archivar facturas cuando sean pagadas.',
      'auto_archive_quote': 'Auto Archivar',
      'auto_archive_quote_help':
          'Automáticamente archivar presupuestos cuando sean convertidos.',
      'auto_convert_quote': 'Auto Convertir',
      'auto_convert_quote_help':
          'Convertir un Presupuesto en Factura automáticamente cuando lo apruebe el cliente.',
      'workflow_settings': 'Configuración de Flujos',
      'freq_daily': 'Diariamente',
      'freq_weekly': 'Semanal',
      'freq_two_weeks': 'Dos semanas',
      'freq_four_weeks': 'Cuatro semanas',
      'freq_monthly': 'Mensual',
      'freq_two_months': 'Dos meses',
      'freq_three_months': 'Tres meses',
      'freq_four_months': 'Cuatro meses',
      'freq_six_months': 'Seis meses',
      'freq_annually': 'Anual',
      'freq_two_years': 'Dos Años',
      'freq_three_years': 'Tres Años',
      'never': 'Nunca',
      'company': 'Empresa',
      'generated_numbers': 'Números Generados',
      'charge_taxes': 'Cargar Impuestos',
      'next_reset': 'Proximo Reinicio',
      'reset_counter': 'Reiniciar Contador',
      'recurring_prefix': 'Prefijo Recurrente',
      'number_padding': 'Relleno numérico',
      'general': 'General',
      'surcharge_field': 'Campo de recargo',
      'company_field': 'Campo de Empresa',
      'company_value': 'Valor de compañía',
      'credit_field': 'Campo de crédito',
      'invoice_field': 'Campo de Factura',
      'invoice_surcharge': 'Recargo de Factura',
      'client_field': 'Campo de Cliente',
      'product_field': 'Campo de Producto',
      'payment_field': 'Campo de pago',
      'contact_field': 'Campo de Contacto',
      'vendor_field': 'Campo de Proveedor',
      'expense_field': 'Campo de Gasto',
      'project_field': 'Campo de Proyecto',
      'task_field': 'Campo de Tarea',
      'group_field': 'Campo de grupo',
      'number_counter': 'Contador de números',
      'prefix': 'Prefijo',
      'number_pattern': 'Patrón numérico',
      'messages': 'Mensajes',
      'custom_css': 'CSS personalizado',
      'custom_javascript': 'JavaScript Personalizado',
      'signature_on_pdf': 'Mostrar en PDF',
      'signature_on_pdf_help':
          'Mostrar la firma del cliente en el PDF de la factura/presupuesto',
      'show_accept_invoice_terms':
          'Mostrar aceptación de términos de la factura',
      'show_accept_invoice_terms_help':
          'Requerir que el cliente confirme que acepta los términos de la factura.',
      'show_accept_quote_terms':
          'Mostrar aceptación de términos del presupuesto',
      'show_accept_quote_terms_help':
          'Requerir que el cliente confirme que acepta los términos del presupuesto.',
      'require_invoice_signature': 'Firma de la factura',
      'require_invoice_signature_help':
          'Requerir que el cliente proporcione su firma.',
      'require_quote_signature': 'Firma del presupuesto.',
      'enable_portal_password': 'Proteger Facturas con Contraseña',
      'enable_portal_password_help':
          'Habilite para seleccionar una contraseña para cada contacto. Si una contraseña esta especificada, se le será solicitada al contacto para acceder a sus facturas.',
      'authorization': 'Autorización',
      'subdomain': 'Subdominio',
      'domain': 'Dominio',
      'portal_mode': 'Modo portal',
      'email_signature': 'Un cordial saludo,',
      'enable_email_markup_help':
          'Haga que sea fácil para sus clientes que paguen mediante la adición de marcas \"schema.org\" a sus correos electrónicos.',
      'plain': 'Plano',
      'light': 'Claro',
      'dark': 'Oscuro',
      'email_design': 'Diseño de Correo',
      'attach_pdf': 'Adjuntar PDF',
      'attach_documents': 'Adjuntar Documentos',
      'attach_ubl': 'Adjuntar UBL',
      'email_style': 'Estilo de correo electrónico',
      'enable_email_markup': 'Habilitar Markup',
      'reply_to_email': 'Direccion Email de Respuesta',
      'reply_to_name': 'Nombre de Responder a',
      'bcc_email': 'BCC Email',
      'processed': 'Procesado',
      'credit_card': 'Tarjeta de Crédito',
      'bank_transfer': 'Transferencia bancaria',
      'priority': 'Prioridad',
      'fee_amount': 'Importe de la cuota',
      'fee_percent': 'Porcentaje de tarifa',
      'fee_cap': 'Límite de tarifa',
      'limits_and_fees': 'Límites/Tarifas',
      'enable_min': 'Activar Mínimo',
      'enable_max': 'Activar Máximo',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Mínimo',
      'max': 'Máximo',
      'accepted_card_logos': 'Logotipos de tarjetas aceptadas',
      'credentials': 'Credenciales',
      'update_address': 'Actualizar Dirección',
      'update_address_help':
          'Actualizar la direccion del cliente con los datos provistos',
      'rate': 'Precio',
      'tax_rate': 'Impuesto',
      'new_tax_rate': 'Nuevo Impuesto',
      'edit_tax_rate': 'Editar impuesto',
      'created_tax_rate': 'Impuesto creado correctamente',
      'updated_tax_rate': 'Impuesto actualizado correctamente',
      'archived_tax_rate': 'Impuesto archivado correctamente',
      'deleted_tax_rate': 'Impuesto borrado correctamente',
      'restored_tax_rate': 'Impuesto restaurado correctamente',
      'archived_tax_rates': ':value tipos impositivos archivados correctamente',
      'deleted_tax_rates': ':value tipos impositivos borrados correctamente',
      'restored_tax_rates':
          ':value tipos impositivos restaurados correctamente',
      'fill_products': 'Auto-rellenar Productos',
      'fill_products_help':
          'Seleccionar un producto automáticamente configurará la descripción y coste',
      'update_products': 'Auto-actualizar Productos',
      'update_products_help':
          'Actualizar una Factura automáticamente actualizará los Productos',
      'convert_products': 'Convertir Productos',
      'convert_products_help':
          'Convertir automáticamente los precios de los productos a la divisa del cliente',
      'fees': 'Cargos',
      'limits': 'Limites',
      'provider': 'Proveedor',
      'company_gateway': 'Pasarela de pago',
      'company_gateways': 'Pasarelas de pago',
      'new_company_gateway': 'Nueva pasarela',
      'edit_company_gateway': 'Editar pasarela',
      'created_company_gateway': 'Pasarela creada correctamente',
      'updated_company_gateway': 'Pasarela actualizada correctamente',
      'archived_company_gateway': 'Pasarela archivada correctamente',
      'deleted_company_gateway': 'Pasarela borrada correctamente',
      'restored_company_gateway': 'Pasarela restaurada correctamente',
      'archived_company_gateways': ':value pasarelas archivadas correctamente',
      'deleted_company_gateways': ':value pasarelas borradas correctamente',
      'restored_company_gateways': ':value pasarelas restauradas correctamente',
      'continue_editing': 'Seguir editando',
      'discard_changes': 'Descartar los cambios',
      'default_value': 'Valor por defecto',
      'disabled': 'Deshabilitado',
      'currency_format': 'Formato de moneda',
      'first_day_of_the_week': 'Primer día de la semana',
      'first_month_of_the_year': 'Primer mes del año',
      'sunday': 'Domingo',
      'monday': 'Lunes',
      'tuesday': 'Martes',
      'wednesday': 'Miércoles',
      'thursday': 'Jueves',
      'friday': 'Viernes',
      'saturday': 'Sábado',
      'january': 'Enero',
      'february': 'Febrero',
      'march': 'Marzo',
      'april': 'Abril',
      'may': 'Mayo',
      'june': 'Junio',
      'july': 'Julio',
      'august': 'Agosto',
      'september': 'Septiembre',
      'october': 'Octubre',
      'november': 'Noviembre',
      'december': 'Diciembre',
      'symbol': 'Símbolo',
      'ocde': 'Código',
      'date_format': 'Formato de fecha',
      'datetime_format': 'Formato de fecha y hora',
      'military_time': '24 Horas',
      'military_time_help': 'Formato de 24 Horas',
      'send_reminders': 'Enviar recordatorios',
      'timezone': 'Zona horaria',
      'filtered_by_project': 'Fitlrado por Proyecto',
      'filtered_by_group': 'Filtrado por Grupo',
      'filtered_by_invoice': 'Filtrado por Factura',
      'filtered_by_client': 'Filtrado por Cliente',
      'filtered_by_vendor': 'Filtrado por proveedor',
      'group_settings': 'Opciones de Grupo',
      'group': 'Grupo',
      'groups': 'Grupos',
      'new_group': 'Nuevo grupo',
      'edit_group': 'Editar grupo',
      'created_group': 'Grupo creado correctamente',
      'updated_group': 'Grupo actualizado correctamente',
      'archived_groups': ':value grupos archivados correctamente',
      'deleted_groups': ':value grupos borrados correctamente',
      'restored_groups': ':value grupos restaurados correctamente',
      'upload_logo': 'Subir Logo',
      'uploaded_logo': 'Logo subido',
      'logo': 'Logo',
      'saved_settings': 'Ajustes guardados',
      'product_settings': 'Configuración de Producto',
      'device_settings': 'Opciones de dispositivo',
      'defaults': 'Ajustes Predefinidos',
      'basic_settings': 'Configuración Básica',
      'advanced_settings': 'Configuración Avanzada',
      'company_details': 'Detalles de la Empresa',
      'user_details': 'Detalles de Usuario',
      'localization': 'Localización',
      'online_payments': 'Pagos Online',
      'tax_rates': 'Impuestos',
      'notifications': 'Notificaciones',
      'import_export': 'Importar/Exportar',
      'custom_fields': 'Campos Personalizados',
      'invoice_design': 'Diseño de Factura',
      'buy_now_buttons': 'Botones de Comprar Ahora',
      'email_settings': 'Configuración del Correo Electrónico',
      'templates_and_reminders': 'Plantillas & Recordatorios',
      'credit_cards_and_banks': 'Tarjetas de Crédito y Bancos',
      'data_visualizations': 'Visualización de Datos',
      'price': 'Precio',
      'email_sign_up': 'Registrarse con Email',
      'google_sign_up': 'Registrarse con Google',
      'thank_you_for_your_purchase': '¡Gracias por su compra!',
      'redeem': 'Redimir',
      'back': 'Atrás',
      'past_purchases': 'Compras Pasadas',
      'annual_subscription': 'Suscripción anual',
      'pro_plan': 'Plan Pro',
      'enterprise_plan': 'Plan Enterprise',
      'count_users': ':count usuarios',
      'upgrade': 'Mejorar',
      'please_enter_a_first_name': 'Introduce tu nombre',
      'please_enter_a_last_name': 'Introduce tu apellido',
      'please_agree_to_terms_and_privacy':
          'Por favor, acepta los términos de servicio y la política de privacidad para crear una cuenta',
      'i_agree_to_the': 'Estoy de acuerdo con',
      'terms_of_service_link': 'términos de servicio',
      'privacy_policy_link': 'política de privacidad',
      'terms_of_service': 'Términos de servicio',
      'privacy_policy': 'Política de Privacidad',
      'sign_up': 'Registrarse',
      'account_login': 'Inicio de Sesión con su Cuenta',
      'view_website': 'Ver Sitio Web',
      'create_account': 'Crear Cuenta',
      'email_login': 'Iniciar sesión con correo electrónico',
      'create_new': 'Crear Nuevo',
      'no_record_selected': 'No se han seleccionado registros',
      'error_unsaved_changes': 'Guarda o cancela tus cambios',
      'download': 'Descargar',
      'requires_an_enterprise_plan': 'Requiere plan \'enterprise\'',
      'take_picture': 'Tomar foto',
      'upload_file': 'Subir archivo',
      'document': 'Documento',
      'documents': 'Documentos',
      'new_document': 'Nuevo documento',
      'edit_document': 'Editar documento',
      'uploaded_document': 'Documento subido satisfactoriamente',
      'updated_document': 'Documento actualizado satisfactoriamente',
      'archived_document': 'Documento archivado satisfactoriamente',
      'deleted_document': 'Documento borrado satisfactoriamente',
      'restored_document': 'Documento restaurado satisfactoriamente',
      'archived_documents': ':value documentos archivados correctamente',
      'deleted_documents': ':value documentos borrados correctamente',
      'restored_documents': ':value documentos restaurados correctamente',
      'no_history': 'Sin historial',
      'expense_date': 'Fecha',
      'pending': 'Pendiente',
      'expense_status_1': 'Registrado',
      'expense_status_2': 'Pendiente',
      'expense_status_3': 'Facturado',
      'converted': 'Modificada',
      'add_documents_to_invoice': 'Agregar documentos a la factura',
      'exchange_rate': 'Tipo de Cambio',
      'convert_currency': 'Convertir moneda',
      'mark_paid': 'Marcar como pagado',
      'category': 'Categoría',
      'address': 'Dirección',
      'new_vendor': 'Nuevo Proveedor',
      'created_vendor': 'Proveedor creado correctamente',
      'updated_vendor': 'Proveedor actualizado correctamente',
      'archived_vendor': 'Proveedor archivado correctamente',
      'deleted_vendor': 'Proveedor eliminado correctamente',
      'restored_vendor': 'Proveedor restaurado correctamente',
      'archived_vendors': ':count proveedores actualizados correctamente',
      'deleted_vendors': ':count proveedores actualizados correctamente',
      'restored_vendors': ':value proveedores restaurados correctamente',
      'new_expense': 'Nuevo Gasto',
      'created_expense': 'Gasto creado correctamente',
      'updated_expense': 'Gasto actualizado correctamente',
      'archived_expense': 'Gasto archivado correctamente',
      'deleted_expense': 'Gasto borrado correctamente',
      'restored_expense': 'Gasto restaurado correctamente',
      'archived_expenses': 'Gastos archivados correctamente',
      'deleted_expenses': 'Gastos borrados correctamente',
      'restored_expenses': ':value gastos restaurados correctamente',
      'copy_shipping': 'Copiar Envío',
      'copy_billing': 'Copia Facturación',
      'design': 'Diseño',
      'failed_to_find_record': 'Fallo al buscar registro',
      'invoiced': 'Facturado',
      'logged': 'Registrado',
      'running': 'Ejecutando',
      'resume': 'Reanudar',
      'task_errors':
          'Por favor corrija cualquier tiempo que se solape con otro',
      'start': 'Iniciar',
      'stop': 'Parar',
      'started_task': 'Tarea empezada correctamente',
      'stopped_task': 'Tarea parada correctamente',
      'resumed_task': 'La tarea se reanudó correctamente',
      'now': 'Ahora',
      'auto_start_tasks': 'Tareas programadas',
      'timer': 'Temporizador',
      'manual': 'Manual',
      'budgeted': 'Presupuestado',
      'start_time': 'Hora de Inicio',
      'end_time': 'Hora de Fin',
      'date': 'Fecha',
      'times': 'Tiempos',
      'duration': 'Duración',
      'new_task': 'Nueva tarea',
      'created_task': 'Tarea creada correctamente',
      'updated_task': 'Tarea actualizada correctamente',
      'archived_task': 'Tarea archivada correctamente',
      'deleted_task': 'Tarea borrada correctamente',
      'restored_task': 'Tarea restaurada correctamente',
      'archived_tasks': ':count tareas archivadas correctamente',
      'deleted_tasks': ':count tareas borradas correctamente',
      'restored_tasks': ':value tareas restauradas correctamente',
      'please_enter_a_name': 'Por favor introduce un nombre',
      'budgeted_hours': 'Horas Presupuestadas',
      'created_project': 'Proyecto creado correctamente',
      'updated_project': 'Proyecto actualizado correctamente',
      'archived_project': 'Proyecto archivado correctamente',
      'deleted_project': 'Proyecto eliminado correctamente',
      'restored_project': 'Proyecto restaurado correctamente',
      'archived_projects': ':count proyectos archivados correctamente',
      'deleted_projects': ':count proyecto eliminados correctamente',
      'restored_projects': ':value proyectos restaurados correctamente',
      'new_project': 'Nuevo Proyecto',
      'thank_you_for_using_our_app': '¡Gracias por utilizar nuestra app!',
      'if_you_like_it': 'Si te gusta por favor',
      'click_here': 'pulse aqui',
      'click_here_capital': 'Pulsa aquí',
      'to_rate_it': 'para valorar.',
      'average': 'Promedio',
      'unapproved': 'No aprobado',
      'authenticate_to_change_setting':
          'Por favor, autenticarse para cambiar esta configuración',
      'locked': 'Bloqueado',
      'authenticate': 'Autenticación',
      'please_authenticate': 'Por favor, autenticarse',
      'biometric_authentication': 'Autenticación biométrica',
      'footer': 'Pie',
      'compare': 'Comparar',
      'hosted_login': 'Acceso alojado',
      'selfhost_login': 'Acceso auto alojado',
      'google_sign_in': 'Ingresar con Google',
      'today': 'Hoy',
      'custom_range': 'Rango personalizado',
      'date_range': 'Rango de fechas',
      'current': 'Actual',
      'previous': 'Previo',
      'current_period': 'Periodo Actual',
      'comparison_period': 'Periodo de Comparación',
      'previous_period': 'Periodo Anterior',
      'previous_year': 'Año Anterior',
      'compare_to': 'Comparar con',
      'last7_days': 'Últimos 7 días',
      'last_week': 'Última Semana',
      'last30_days': 'Últimos 30 días',
      'this_month': 'Este Mes',
      'last_month': 'Último Mes',
      'this_year': 'Este Año',
      'last_year': 'Último Año',
      'custom': 'Personalizado',
      'clone_to_invoice': 'Clonar a Factura',
      'clone_to_quote': 'Clonar a Presupuesto',
      'clone_to_credit': 'Clonar a Crédito',
      'view_invoice': 'Ver Factura',
      'convert': 'Convertir',
      'more': 'Más',
      'edit_client': 'Editar Cliente',
      'edit_product': 'Editar Producto',
      'edit_invoice': 'Editar Factura',
      'edit_quote': 'Editar Presupuesto',
      'edit_payment': 'Editar Pago',
      'edit_task': 'Editar Tarea',
      'edit_expense': 'Editar Gasto',
      'edit_vendor': 'Editar Proveedor',
      'edit_project': 'Editar Proyecto',
      'edit_recurring_expense': 'Editar Gasto Periódico',
      'edit_recurring_quote': 'Editar Presupuesto Recurrente',
      'billing_address': 'Dirección de Facturación',
      'shipping_address': 'Direccion de Envio',
      'total_revenue': 'Ingresos Totales',
      'average_invoice': 'Promedio de Facturación',
      'outstanding': 'Pendiente de Cobro',
      'invoices_sent': ':count facturas enviadas',
      'active_clients': 'Clientes Activos',
      'close': 'Cerrar',
      'email': 'Email',
      'password': 'Contraseña',
      'url': 'URL',
      'secret': 'Secreto',
      'name': 'Nombre',
      'logout': 'Cerrar sesión',
      'login': 'Iniciar Sesión',
      'filter': 'Filtrar',
      'sort': 'Orden',
      'search': 'Búsqueda',
      'active': 'Activo',
      'archived': 'Archivado',
      'deleted': 'Eliminado',
      'dashboard': 'Inicio',
      'archive': 'Archivar',
      'delete': 'Eliminar',
      'restore': 'Restaurar',
      'refresh_complete': 'Actualización Completa',
      'please_enter_your_email': 'Por favor introduce tu email',
      'please_enter_your_password': 'Por favor introduce tu contraseña',
      'please_enter_your_url': 'Por favor introduce tu URL',
      'please_enter_a_product_key': 'Por favor introduce un código de producto',
      'ascending': 'Ascendente',
      'descending': 'Descendente',
      'save': 'Guardar',
      'an_error_occurred': 'Ha ocurrido un error',
      'paid_to_date': 'Pagado',
      'balance_due': 'Pendiente',
      'balance': 'Saldo',
      'overview': 'Resumen',
      'details': 'Detalles',
      'phone': 'Teléfono',
      'website': 'Página Web',
      'vat_number': 'NIF/CIF',
      'id_number': 'Nº de identificación',
      'create': 'Crear',
      'copied_to_clipboard': ':value copiado al portapapeles',
      'error': 'Error',
      'could_not_launch': 'No se puede abrir',
      'contacts': 'Contactos',
      'additional': 'Adicional',
      'first_name': 'Nombre',
      'last_name': 'Apellidos',
      'add_contact': 'Añadir Contacto',
      'are_you_sure': '¿Está Seguro?',
      'cancel': 'Cancelar',
      'ok': 'Ok',
      'remove': 'Borrar',
      'email_is_invalid': 'El email es inválido',
      'product': 'Producto',
      'products': 'Productos',
      'new_product': 'Nuevo Producto',
      'created_product': 'Producto creado correctamente',
      'updated_product': 'Producto actualizado correctamente',
      'archived_product': 'Producto archivado correctamente',
      'deleted_product': 'Producto eliminado correctamente',
      'restored_product': 'Producto restaurado correctamente',
      'archived_products': ':count productos archivados correctamente',
      'deleted_products': ':count productos eliminados correctamente',
      'restored_products': ':value productos restaurados correctamente',
      'product_key': 'Producto',
      'notes': 'Notas',
      'cost': 'Coste',
      'client': 'Cliente',
      'clients': 'Clientes',
      'new_client': 'Nuevo Cliente',
      'created_client': 'Cliente creado correctamente',
      'updated_client': 'Cliente actualizado correctamente',
      'archived_client': 'Cliente archivado correctamente',
      'archived_clients': ':count clientes archivados correctamente',
      'deleted_client': 'Cliente eliminado correctamente',
      'deleted_clients': ':count clientes eliminados correctamente',
      'restored_client': 'Cliente restaurada correctamente',
      'restored_clients': ':value clientes restaurados correctamente',
      'address1': 'Calle',
      'address2': 'Bloq/Pta',
      'city': 'Ciudad',
      'state': 'Provincia',
      'postal_code': 'Código Postal',
      'country': 'Pais',
      'invoice': 'Factura',
      'invoices': 'Facturas',
      'new_invoice': 'Nueva Factura',
      'created_invoice': 'Factura creada correctamente',
      'updated_invoice': 'Factura actualizada correctamente',
      'archived_invoice': 'Factura archivada correctamente',
      'deleted_invoice': 'Factura eliminada correctamente',
      'restored_invoice': 'Factura restaurada correctamente',
      'archived_invoices': ':count facturas archivadas correctamente',
      'deleted_invoices': ':count facturas eliminadas correctamente',
      'restored_invoices': ':value facturas restauradas correctamente',
      'emailed_invoice': 'Factura enviada correctamente',
      'emailed_payment': 'Pago enviado correctamente por correo electrónico',
      'amount': 'Cantidad',
      'invoice_number': 'Número de Factura',
      'invoice_date': 'Fecha de Factura',
      'discount': 'Descuento',
      'po_number': 'Número de Pedido',
      'terms': 'Términos',
      'public_notes': 'Notas',
      'private_notes': 'Notas Privadas',
      'frequency': 'Frecuencia',
      'start_date': 'Fecha de Inicio',
      'end_date': 'Fecha de Fin',
      'quote_number': 'Número de Presupuesto',
      'quote_date': 'Fecha Presupuesto',
      'valid_until': 'Válido hasta',
      'items': 'Artículos',
      'partial_deposit': 'Parcial/Depósito',
      'description': 'Descripción',
      'unit_cost': 'Precio Unitario',
      'quantity': 'Cantidad',
      'add_item': 'Añadir Artículo',
      'contact': 'Contacto',
      'work_phone': 'Teléfono',
      'total_amount': 'Cantidad Total',
      'pdf': 'PDF',
      'due_date': 'Vencimiento',
      'partial_due_date': 'Fecha de vencimiento parcial',
      'status': 'Estado',
      'invoice_status_id': 'Estado de Factura',
      'quote_status': 'Estado de Presupuesto',
      'click_plus_to_add_item': 'Pulsa + para añadir un artículo',
      'click_plus_to_add_time': 'Pulsa + para añadir tiempo',
      'count_selected': ':count seleccionado',
      'total': 'Total',
      'percent': 'Porcentaje',
      'edit': 'Editar',
      'dismiss': 'Descartar',
      'please_select_a_date': 'Por favor selecciona una fecha',
      'please_select_a_client': 'Por favor selecciona un cliente',
      'please_select_an_invoice': 'Por favor, seleccione una factura',
      'task_rate': 'Tasa de tareas',
      'settings': 'Configuración',
      'language': 'Idioma',
      'currency': 'Divisa',
      'created_at': 'Fecha de Creación',
      'created_on': 'Creado el',
      'updated_at': 'Actualizado',
      'tax': 'Impuesto',
      'please_enter_an_invoice_number':
          'Por favor introduce un número de factura',
      'please_enter_a_quote_number':
          'Por favor introduce un número de presupuesto',
      'past_due': 'Vencido',
      'draft': 'Borrador',
      'sent': 'Enviada',
      'viewed': 'Vistas',
      'approved': 'Aprobados',
      'partial': 'Parcial/Depósito',
      'paid': 'Pagado',
      'mark_sent': 'Marcar como Enviado',
      'marked_invoice_as_sent': 'Factura marcada como enviada correctamente',
      'marked_invoice_as_paid': 'Factura marcada como enviada correctamente',
      'marked_invoices_as_sent':
          'Facturas marcadas como enviadas correctamente',
      'marked_invoices_as_paid':
          'Facturas marcadas como enviadas correctamente',
      'done': 'Hecho',
      'please_enter_a_client_or_contact_name':
          'Por favor introduce un cliente o nombre de contacto',
      'dark_mode': 'Modo Oscuro',
      'restart_app_to_apply_change': 'Reinicia la app para aplicar el cambio',
      'refresh_data': 'Actualizar Datos',
      'blank_contact': 'Contacto Nuevo',
      'activity': 'Actividad',
      'no_records_found': 'No se han encontrado registros',
      'clone': 'Clonar',
      'loading': 'Cargando',
      'industry': 'Sector',
      'size': 'Tamaño',
      'payment_terms': 'Términos de Pago',
      'payment_date': 'Fecha de Pago',
      'payment_status': 'Estado de Pago',
      'payment_status_1': 'Pendiente',
      'payment_status_2': 'Anulado',
      'payment_status_3': 'Fallido',
      'payment_status_4': 'Completado',
      'payment_status_5': 'Parcialmente Reembolsado',
      'payment_status_6': 'Reembolsado',
      'payment_status_-1': 'Sin Aplicar',
      'payment_status_-2': 'Sin aplicar parcialmente',
      'net': 'Neto',
      'client_portal': 'Portal Cliente',
      'show_tasks': 'Mostrar tareas',
      'email_reminders': 'Emails Recordatorios',
      'enabled': 'Habilitado',
      'recipients': 'Destinatarios',
      'initial_email': 'Email Inicial',
      'first_reminder': 'Primer Recordatorio',
      'second_reminder': 'Segundo Recordatorio',
      'third_reminder': 'Tercer Recordatorio',
      'reminder1': 'Primer Recordatorio',
      'reminder2': 'Segundo Recordatorio',
      'reminder3': 'Tercer Recordatorio',
      'template': 'Plantilla',
      'send': 'Enviar',
      'subject': 'Asunto',
      'body': 'Cuerpo',
      'send_email': 'Enviar Email',
      'email_receipt': 'Enviar Recibo de Pago al cliente',
      'auto_billing': 'Auto facturación',
      'button': 'Botón',
      'preview': 'Vista Previa',
      'customize': 'Personalizar',
      'history': 'Historial',
      'payment': 'Pago',
      'payments': 'Pagos',
      'refunded': 'Reembolsado',
      'payment_type': 'Tipo de Pago',
      'transaction_reference': 'Referencia de Transacción',
      'enter_payment': 'Agregar Pago',
      'new_payment': 'Introduzca el Pago',
      'created_payment': 'Pago creado correctamente',
      'updated_payment': 'Pago actualizado correctamente',
      'archived_payment': 'Pago archivado correctamente',
      'deleted_payment': 'Pago eliminado correctamente',
      'restored_payment': 'Pago restaurado correctamente',
      'archived_payments': ':count pagos archivados correctamente',
      'deleted_payments': ':count pagos eliminados correctamente',
      'restored_payments': ':value pagos restaurados correctamente',
      'quote': 'Presupuesto',
      'quotes': 'Presupuestos',
      'new_quote': 'Nuevo Presupuesto',
      'created_quote': 'Presupuesto creado correctamente',
      'updated_quote': 'Presupuesto actualizado correctamente',
      'archived_quote': 'Presupuesto archivado correctamente',
      'deleted_quote': 'Presupuesto eliminado correctamente',
      'restored_quote': 'Presupuesto restaurada correctamente',
      'archived_quotes': ':count Presupuestos archivados correctamente',
      'deleted_quotes': ':count Presupuestos eliminados correctamente',
      'restored_quotes': ':value presupuestos restaurados correctamente',
      'expense': 'Gasto',
      'expenses': 'Gastos',
      'vendor': 'Proveedor',
      'vendors': 'Proveedores',
      'task': 'Tarea',
      'tasks': 'Tareas',
      'project': 'Proyecto',
      'projects': 'Proyectos',
      'activity_1': ':user creó el cliente :client',
      'activity_2': ':user archivó el cliente :client',
      'activity_3': ':user borró el cliente :client',
      'activity_4': ':user archivó la factura :invoice',
      'activity_5': ':user actualizó la factura :invoice',
      'activity_6':
          ':user ha enviado por mail la factura :invoice de :client a :contact',
      'activity_7': ':contact ha visto la factura :invoice: de :client',
      'activity_8': ':user archivó la factura :invoice',
      'activity_9': ':user borró la factura :invoice',
      'activity_10':
          ':contact ingresó el pago :payment por importe de :payment_amount en la factura Nº :invoice de :client',
      'activity_11': ':user actualizó el Pago :payment',
      'activity_12': ':user archivó el pago :payment',
      'activity_13': ':user borró el pago :payment',
      'activity_14': ':user introdujo :credit crédito',
      'activity_15': ':user actualizó :credit crédito',
      'activity_16': ':user archivó :credit crédito',
      'activity_17': ':user deleted :credit crédito',
      'activity_18': ':user borró el presupuesto :quote',
      'activity_19': ':user actualizó el presupuesto :quote',
      'activity_20': ':user envió presupuesto :quote para :client a :contact',
      'activity_21': ':contact vió el presupuesto :quote',
      'activity_22': ':user archivó el presupuesto :quote',
      'activity_23': ':user borró el presupuesto :quote',
      'activity_24': ':user restauró el presupuesto :quote',
      'activity_25': ':user restauró la factura :invoice',
      'activity_26': ':user restauró el cliente :client',
      'activity_27': ':user restauró el pago :payment',
      'activity_28': ':user restauró :credit crédito',
      'activity_29': ':contact ha aprovado el presupuesto :quote para :client',
      'activity_30': ':user creó al vendedor :vendor',
      'activity_31': ':user archivó al vendedor :vendor',
      'activity_32': ':user eliminó al vendedor :vendor',
      'activity_33': ':user restauró al vendedor :vendor',
      'activity_34': ':user creó el gasto :expense',
      'activity_35': ':user archivó el gasto :expense',
      'activity_36': ':user eliminó el gasto :expense',
      'activity_37': ':user restauró el gasto :expense',
      'activity_39': ':user cancelo :payment_amount del pago :payment',
      'activity_40':
          ':user reembolsó :adjustment de :payment_amount del pago :payment',
      'activity_41': 'Fallo el pago de :payment_amount para  (:payment)',
      'activity_42': ':user creó la tarea :task',
      'activity_43': ':user actualizó la tarea :task',
      'activity_44': ':user archivó la tarea :task',
      'activity_45': ':user eliminó la tarea :task',
      'activity_46': ':user restauró la tarea :task',
      'activity_47': ':user actualizó el gasto :expense',
      'activity_48': ':user actualizó el ticket :ticket',
      'activity_49': ':user cerró el ticket :ticket',
      'activity_50': ':user unió el ticket :ticket',
      'activity_51': ':user dividió el ticket :ticket',
      'activity_52': ':contact abrió el ticket :ticket',
      'activity_53': ':contact reabrió el ticket :ticket',
      'activity_54': ':user reabrió el ticket :ticket',
      'activity_55': ':contact respondió el ticket :ticket',
      'activity_56': ':user vio el ticket :ticket',
      'activity_57': 'El sistema falló al enviar la factura :invoice',
      'activity_58': ':user revirtió la factura :invoice',
      'activity_59': ':user canceló la factura :invoice',
      'activity_60': ':contact vió el presupuesto :quote',
      'activity_61': ':user actualizó el cliente :cliente',
      'activity_62': ':user actualizó el proveedor :vendor',
      'activity_63':
          ':user envió por email el primer recordatorio de la factura :invoice a :contact',
      'activity_64':
          ':user envió por email el segundo recordatorio de la factura :invoice a :contact',
      'activity_65':
          ':user envió por email el tercer recordatorio de la factura :invoice a :contact',
      'activity_66':
          ':user envió por email el recordatorio sin fín de la factura :invoice a :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Password de un solo uso',
      'emailed_quote': 'Presupuesto enviado correctamente',
      'emailed_credit': 'Crédito enviado correctamente',
      'marked_quote_as_sent': 'Presupuesto marcado como enviado correctamente',
      'marked_credit_as_sent': 'Marcar crédito como enviado',
      'expired': 'Expirada',
      'all': 'Todo',
      'select': 'Seleccionar',
      'long_press_multiselect': 'Multiselección en pulsación prolongada',
      'custom_value1': 'Valor Personalizado',
      'custom_value2': 'Valor Personalizado',
      'custom_value3': 'Valor Personalizado 3',
      'custom_value4': 'Valor Personalizado 4',
      'email_style_custom': 'Estilo de Email Personalizado',
      'custom_message_dashboard': 'Mensaje de Escritorio Personalizado',
      'custom_message_unpaid_invoice':
          'Mensaje de Factura Impagada Personalizada',
      'custom_message_paid_invoice': 'Mensaje de Factura Pagada Personalizada',
      'custom_message_unapproved_quote':
          'Mensaje de Presupuesto no Aprobado Personalizado',
      'lock_invoices': 'Bloquear Facturas',
      'translations': 'Traducciones',
      'task_number_pattern': 'Patrón del Número de Tarea',
      'task_number_counter': 'Contador del Número de Tarea',
      'expense_number_pattern': 'Patrón del Número de Gasto',
      'expense_number_counter': 'Contador del Número de Gasto',
      'vendor_number_pattern': 'Patrón del Número de Proveedor',
      'vendor_number_counter': 'Contador del Número de Proveedor',
      'ticket_number_pattern': 'Patrón del Número de Ticket',
      'ticket_number_counter': 'Contador del Número de Ticket',
      'payment_number_pattern': 'Patrón del Número de Pago',
      'payment_number_counter': 'Contador del Número de Pago',
      'invoice_number_pattern': 'Patrón del Número de Factura',
      'invoice_number_counter': 'Contador del Número de Factura',
      'quote_number_pattern': 'Patrón del Número de Presupuesto',
      'quote_number_counter': 'Contador del Número de Presupuesto',
      'client_number_pattern': 'Patrón del Número de Crédito',
      'client_number_counter': 'Contador del Número de Crédito',
      'credit_number_pattern': 'Patrón del Número de Crédito',
      'credit_number_counter': 'Contador del Número de Crédito',
      'reset_counter_date': 'Resetear Fecha del Contador',
      'counter_padding': 'Relleno del Contador',
      'shared_invoice_quote_counter':
          'Compartir la numeración para presupuesto y factura',
      'default_tax_name_1': 'Nombre de Impuesto por Defecto 1',
      'default_tax_rate_1': 'Tasa de Impuesto por Defecto 1',
      'default_tax_name_2': 'Nombre de Impuesto por Defecto 2',
      'default_tax_rate_2': 'Tasa de Impuesto por Defecto 2',
      'default_tax_name_3': 'Nombre de Impuesto por Defecto 3',
      'default_tax_rate_3': 'Tasa de Impuesto por Defecto 3',
      'email_subject_invoice': 'Asunto de Email de Factura',
      'email_subject_quote': 'Asunto de Email de Presupuesto',
      'email_subject_payment': 'Asunto de Email de Pago',
      'email_subject_payment_partial': 'Asunto de Email de Pago Parcial',
      'show_table': 'Mostrar Tabla',
      'show_list': 'Mostrar Lista',
      'client_city': 'Ciudad del Cliente',
      'client_state': 'Provincia del Cliente',
      'client_country': 'País del Cliente',
      'client_is_active': 'El Cliente está Activo',
      'client_balance': 'Balance del Cliente',
      'client_address1': 'Calle del Cliente',
      'client_address2': 'Bloq/Pta del Cliente',
      'vendor_address1': 'Calle de Proveedor',
      'vendor_address2': 'Bloq/Pta del Proveedor',
      'client_shipping_address1': 'Calle de Envío del Cliente',
      'client_shipping_address2': 'Bloq/Pta de Envío del Cliente',
      'type': 'Tipo',
      'invoice_amount': 'Importe de Factura',
      'invoice_due_date': 'Fecha Límite de Pago',
      'tax_rate1': 'Impuesto 1',
      'tax_rate2': 'Impuesto 2',
      'tax_rate3': 'Impuesto 3',
      'auto_bill': 'Facturación Automática',
      'archived_at': 'Archivado el',
      'has_expenses': 'Tiene Gastos',
      'custom_taxes1': 'Impuestos Personalizados 1',
      'custom_taxes2': 'Impuestos Personalizados 2',
      'custom_taxes3': 'Impuestos Personalizados 3',
      'custom_taxes4': 'Impuestos Personalizados 4',
      'custom_surcharge1': 'Recargo Personalizado 1',
      'custom_surcharge2': 'Recargo Personalizado 2',
      'custom_surcharge3': 'Recargo Personalizado 3',
      'custom_surcharge4': 'Recargo Personalizado 4',
      'is_deleted': 'Borrado',
      'vendor_city': 'Ciudad del Proveedor',
      'vendor_state': 'Provincia del Proveedor',
      'vendor_country': 'País del Proveedor',
      'is_approved': 'Aprobada',
      'tax_name': 'Nombre de Impuesto',
      'tax_amount': 'Total Impuestos',
      'tax_paid': 'Impuestos Pagados',
      'payment_amount': 'Valor del Pago',
      'age': 'Edad',
      'is_running': 'Corriendo',
      'time_log': 'Registro Temporal',
      'bank_id': 'Banco',
      'expense_category_id': 'ID de la Categoría de Gasto',
      'expense_category': 'Categoría del Gasto',
      'invoice_currency_id': 'ID de Moneda de Facturación',
      'tax_name1': 'Nombre de Impuesto 1',
      'tax_name2': 'Nombre de Impuesto 2',
      'tax_name3': 'Nombre de Impuesto 3',
      'transaction_id': 'ID de Transacción',
      'color_theme': 'Color del Tema',
    },
    'sv': {
      'import_data': 'Importera Data',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Kontakt Förnamn',
      'contact_last_name': 'Kontakt Efternamn',
      'order': 'Order',
      'unassigned': 'Otilldelad',
      'partial_value': 'Måste vara större än noll och mindre än totalen',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Tillgänglig',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Ställ in subdomänen eller visa fakturorna på din egen hemsida',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Säkerhet',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Prenumerationer',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Ändra prenumeration',
      'created_subscription': 'Skapat prenumerationer utan problem',
      'updated_subscription': 'Uppdaterat prenumerationer utan problem',
      'archived_subscription': 'Arkivera prenumeration utan problem',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Skicka inbjudan igen',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help':
          'Skanna streckkoden med en :link kompatibel app.',
      'enabled_two_factor': 'Aktiverade Två-Vägs autentisering utan problem',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Tvåfaktorsautentisering',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Återbetalat betalning',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Omvandla till faktura',
      'registration_url': 'Registration URL',
      'invoice_project': 'Fakturera projekt',
      'invoice_task': 'Fakturera uppgift',
      'invoice_expense': 'Faktura kostnad',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Konverterad summa',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Standard dokument',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Dölj',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Kolumn',
      'sample': 'Exempel',
      'map_to': 'Map To',
      'import': 'Importera',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Välj fil',
      'no_file_selected': 'No File Selected',
      'csv_file': 'Välj CSV-fil',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Obetald',
      'white_label': 'White Label',
      'delivery_note': 'Följesedel',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Delvis försenad',
      'invoice_total': 'Totalsumma',
      'quote_total': 'Offertsumma',
      'credit_total': 'Kredit Totalt',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Varning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Kundnamn',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Utgifts kategorier',
      'new_expense_category': 'Ny utgifts kategori',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Framgångsrikt skapat kostnadskategori',
      'updated_expense_category': 'Framgångsrikt uppdaterat kostnadskategori',
      'archived_expense_category': 'Framgångsrikt arkiverat kostnadskategori',
      'deleted_expense_category': 'Kategori borttagen',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Framgångsrikt återställt kostnadskategori',
      'archived_expense_categories':
          'Framgångsrikt arkiverat :count kostnadskategori',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Ska detta faktureras',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Markera aktiv',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Återkommande faktura',
      'recurring_invoices': 'Återkommande fakturor',
      'new_recurring_invoice': 'Ny återkommande faktura',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice':
          'Framgångsrikt arkiverat återkommande faktura',
      'deleted_recurring_invoice':
          'Framgångsrikt tagit bort återkommande faktura',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice':
          'Framgångsrikt återställt återkommande faktura',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Förtjänst',
      'line_item': 'Rad',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'Se portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Spara kortinformation',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Alltid',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Rubrik',
      'client_number': 'Kundnummer',
      'auto_convert': 'Auto Convert',
      'company_name': 'Företagsnamn',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'E-postade fakturorna utan problem',
      'emailed_quotes': 'E-postade offerterna utan problem',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Timmar',
      'statement': 'Transaktionsdatum',
      'taxes': 'Moms',
      'surcharge': 'Tilläggsavgift',
      'apply_payment': 'Apply Payment',
      'apply': 'Verkställ',
      'unapplied': 'Unapplied',
      'select_label': 'Välj rubrik',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Till',
      'health_check': 'Health Check',
      'payment_type_id': 'Betalningssätt',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Kommande fakturor',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Nyligen utförda betalningar',
      'upcoming_quotes': 'Kommande Offerter',
      'expired_quotes': 'Utgångna Offerter',
      'create_client': 'Skapa kund',
      'create_invoice': 'Skapa faktura',
      'create_quote': 'Skapa offert',
      'create_payment': 'Create Payment',
      'create_vendor': 'Skapa tillverkare',
      'update_quote': 'Update Quote',
      'delete_quote': 'Ta bort offert',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Ta bort faktura',
      'update_client': 'Update Client',
      'delete_client': 'Radera kund',
      'delete_payment': 'Ta bort betalning',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Ta bort leverantör',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Ta bort kostnad',
      'create_task': 'Skapa uppgift',
      'update_task': 'Update Task',
      'delete_task': 'Radera uppgift',
      'approve_quote': 'Approve Quote',
      'off': 'Av',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Gratis',
      'plan': 'Nivå',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Mål',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'Ändra token',
      'created_token': 'Token skapad',
      'updated_token': 'Token uppdaterad',
      'archived_token': 'Framgångsrikt arkiverat Token',
      'deleted_token': 'Token borttagen',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'E-posta faktura',
      'email_quote': 'E-posta offert',
      'email_credit': 'Email Credit',
      'email_payment': 'Eposta betalning',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Kontakt namn',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Editera betalningsvillkor',
      'created_payment_term': 'Skapade betalningsvillkor utan problem',
      'updated_payment_term': 'Uppdaterade betalningsvillkor utan problem',
      'archived_payment_term': 'Arkiverat betalningsvillkor utan problem',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kreditsumma',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exklusive',
      'inclusive': 'Inklusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Återbetala betalning',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Hela namnet',
      'city_state_postal': 'Stad/Län/Postnummer',
      'postal_city_state': 'Postadress/Stad/Stat',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Rensa uppgifter.',
      'purge_successful': 'Rensade utan problem företags data',
      'purge_data_message':
          'Varning: Detta kommer permanent ta bort din information, det finns ingen återvända.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Dagar',
      'age_group_30': '30 - 60 Dagar',
      'age_group_60': '60 - 90 Dagar',
      'age_group_90': '90 - 120 Dagar',
      'age_group_120': '120+ Dagar',
      'refresh': 'Uppdatera',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Faktura detaljer',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Behörigheter',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count fakturor skickade',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Uppge Licens',
      'cancel_account': 'Avsluta konto',
      'cancel_account_message':
          'Varning: Detta kommer permanent ta bort ditt konto, detta går inte att ångra.',
      'delete_company': 'Ta bort företag',
      'delete_company_message':
          'Varning: Detta kommer permanent ta bort till bolag, det finns ingen återvändo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Rubrik',
      'load_design': 'Ladda design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Förslag',
      'tickets': 'Tickets',
      'recurring_quotes': 'Återkommande offerter',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Återkommande utgifter',
      'account_management': 'Kontohantering',
      'credit_date': 'Kreditdatum',
      'credit': 'Kredit',
      'credits': 'Kreditfakturor',
      'new_credit': 'Ange Kredit',
      'edit_credit': 'Redigera Kreditfaktura',
      'created_credit': 'Kreditfaktura skapad',
      'updated_credit': 'Kreditfaktura uppdaterad',
      'archived_credit': 'Kreditfaktura arkiverad',
      'deleted_credit': 'Kreditfaktura borttagen',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Kreditfaktura återställd',
      'archived_credits': ':count kreditfakturor arkiverade',
      'deleted_credits': ':count kreditfakturor borttagna',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Nuvarande version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Hjälp',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Nytt företag',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Återställa',
      'number': 'Number',
      'export': 'Exportera',
      'chart': 'Lista',
      'count': 'Count',
      'totals': 'Total',
      'blank': 'Blank',
      'day': 'Dag',
      'month': 'Månad',
      'year': 'År',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Gruppera genom',
      'credit_balance': 'Kreditbalans',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Kontakt telefon',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Leverans gata',
      'shipping_address2': 'Leverans våning',
      'shipping_city': 'Leverans stad',
      'shipping_state': 'Leverans län',
      'shipping_postal_code': 'Leverans postnummer',
      'shipping_country': 'Leverans land',
      'billing_address1': 'Fakturerings gata',
      'billing_address2': 'Fakturerings våning',
      'billing_city': 'Fakturerings stad',
      'billing_state': 'Fakturerings län',
      'billing_postal_code': 'Fakturerings postnummer',
      'billing_country': 'Fakturerings land',
      'client_id': 'Klient Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Skapad av :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Kolumner',
      'aging': 'Börjar bli gammal',
      'profit_and_loss': 'Förtjänst och Förlust',
      'reports': 'Rapporter',
      'report': 'Rapport',
      'add_company': 'Lägg till företag',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Hjälp',
      'refund': 'Återbetalning',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Kontakt e-post',
      'multiselect': 'Multiselect',
      'entity_state': 'Tillstånd',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Meddelande',
      'from': 'Från',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'Supportforum',
      'about': 'About',
      'documentation': 'Dokumentation',
      'contact_us': 'Kontakta oss',
      'subtotal': 'Delsumma',
      'line_total': 'Summa',
      'item': 'Artikel',
      'credit_email': 'Credit Email',
      'iframe_url': 'Webbsida',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Ja',
      'no': 'Nej',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Visa',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Användare',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Välj en klient',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Moms inställningar',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Växla',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Val',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Skicka',
      'recover_password': 'Återställ ditt lösenord',
      'late_fees': 'Late Fees',
      'credit_number': 'Kreditnummer',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Förseningsavgifts summa',
      'late_fee_percent': 'Förseningsavgifts procent',
      'schedule': 'Schema',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Dagar',
      'invoice_email': 'Faktura e-post',
      'payment_email': 'Betalnings e-post',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Offert e-post',
      'endless_reminder': 'Oändlig påminnelse',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administratör',
      'administrator_help':
          'Tillåt användare att hantera användare, ändra inställningar och ändra alla värden',
      'user_management': 'Användarhantering',
      'users': 'Användare',
      'new_user': 'Ny användare',
      'edit_user': 'Ändra användare',
      'created_user': 'Successfully created user',
      'updated_user': 'Användare uppdaterad',
      'archived_user': 'Framgångsrikt arkiverat användare',
      'deleted_user': 'Användare borttagen',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Användare återställd',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Generella inställningar',
      'invoice_options': 'Fakturainställningar',
      'hide_paid_to_date': 'Dölj \"Betald till\"',
      'hide_paid_to_date_help':
          'Visa bara \"Betald till\"-sektionen på fakturan när en betalning har mottagits.',
      'invoice_embed_documents': 'Bädda in dokument',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Visa Header på',
      'all_pages_footer': 'Visa Footer på',
      'first_page': 'Första sidan',
      'all_pages': 'Alla sidor',
      'last_page': 'Sista sidan',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Primär färg',
      'secondary_color': 'Sekundär färg',
      'page_size': 'Sidstorlek',
      'font_size': 'Storlek på framsida',
      'quote_design': 'Offert design',
      'invoice_fields': 'Fakturafält',
      'product_fields': 'Produkt fält',
      'invoice_terms': 'Fakturavillkor',
      'invoice_footer': 'Faktura sidfot',
      'quote_terms': 'Offertvillkor',
      'quote_footer': 'Offert footer',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Konvertera',
      'auto_convert_quote_help':
          'Konvertera automatiskt en offert till en faktura när den godkänts av en klient.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Dagligen',
      'freq_weekly': 'Veckovis',
      'freq_two_weeks': 'Två veckor',
      'freq_four_weeks': 'Fyra veckor',
      'freq_monthly': 'Månadsvis',
      'freq_two_months': 'Två månader',
      'freq_three_months': 'Tre månader',
      'freq_four_months': 'Fyra månader',
      'freq_six_months': 'Sex månader',
      'freq_annually': 'Årsvis',
      'freq_two_years': 'Två år',
      'freq_three_years': 'Three Years',
      'never': 'Aldrig',
      'company': 'Företag',
      'generated_numbers': 'Genererade nummer',
      'charge_taxes': 'Inkludera moms',
      'next_reset': 'Nästa återställning',
      'reset_counter': 'Återställ räknare',
      'recurring_prefix': 'Återkommande prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Företagsfält',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Fakturafält',
      'invoice_surcharge': 'Tilläggsavgift till faktura',
      'client_field': 'Klientfält',
      'product_field': 'Produktfält',
      'payment_field': 'Payment Field',
      'contact_field': 'Kontaktfält',
      'vendor_field': 'Leverantörsfält',
      'expense_field': 'Utgiftsfält',
      'project_field': 'Projektfält',
      'task_field': 'Uppgiftsfält',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Prefix',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Anpassad CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Visa på PDF',
      'signature_on_pdf_help': 'Visa kundens signatur på fakturan/offerten.',
      'show_accept_invoice_terms': 'Faktura villkor kryssruta',
      'show_accept_invoice_terms_help':
          'Kräv att klienten accepterar faktura villkoren.',
      'show_accept_quote_terms': 'Offert villkors kryssruta',
      'show_accept_quote_terms_help':
          'Kräv att klienten accepterar offert villkoren.',
      'require_invoice_signature': 'Faktura signatur',
      'require_invoice_signature_help': 'Kräv signatur av klient.',
      'require_quote_signature': 'Offert signatur',
      'enable_portal_password': 'Lösenordsskydda fakturor',
      'enable_portal_password_help':
          'Tillåter dig att sätta ett lösenord för varje kontakt. Om ett lösenord är valt kommer kontakten vara tvungen att skriva in lösenordet innan den kan se fakturan.',
      'authorization': 'Tillstånd',
      'subdomain': 'Underdomän',
      'domain': 'Domän',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Vänliga hälsningar,',
      'enable_email_markup_help':
          'Gör det enklare för dina klienter att betala genom att lägga till schema.org märkning till dina e-post.',
      'plain': 'Vanlig',
      'light': 'Ljus',
      'dark': 'Mörk',
      'email_design': 'E-post design',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'Aktivera märkning',
      'reply_to_email': 'Reply-To E-post',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'Eposta hemlig kopia',
      'processed': 'Processed',
      'credit_card': 'Betalkort',
      'bank_transfer': 'Banköverföring',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Aktivera min',
      'enable_max': 'Aktivera max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepterade kort logos',
      'credentials': 'Credentials',
      'update_address': 'Uppdatera adress',
      'update_address_help':
          'Uppdatera kundens adress med tillhandahållna uppgifter',
      'rate': 'á-pris',
      'tax_rate': 'Skattenivå',
      'new_tax_rate': 'Ny skatte nivå',
      'edit_tax_rate': 'Redigera skattenivå',
      'created_tax_rate': 'Framgångsrikt skapat skattesats',
      'updated_tax_rate': 'Framgångsrikt uppdaterad momssats',
      'archived_tax_rate': 'Framgångsrikt arkiverat skattenivån/momssatsen',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Auto-ifyll produkter',
      'fill_products_help':
          'Välj en produkt för att automatiskt fylla i beskrivning och pris',
      'update_products': 'Auto-uppdaterade produkter',
      'update_products_help':
          'Uppdatera en faktura för att automatiskt uppdatera produktbiblioteket',
      'convert_products': 'Konvertera produkter',
      'convert_products_help':
          'Konvertera automatiskt produkt priser till kundens valuta',
      'fees': 'Avgifter',
      'limits': 'Gränser',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Avbryt ändringar',
      'default_value': 'Default value',
      'disabled': 'Avstängd',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Söndag',
      'monday': 'Måndag',
      'tuesday': 'Tisdag',
      'wednesday': 'Onsdag',
      'thursday': 'Torsdag',
      'friday': 'Fredag',
      'saturday': 'Lördag',
      'january': 'Januari',
      'february': 'Februari',
      'march': 'Mars',
      'april': 'April',
      'may': 'Maj',
      'june': 'Juni',
      'july': 'Juli',
      'august': 'Augusti',
      'september': 'September',
      'october': 'Oktober',
      'november': 'November',
      'december': 'December',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Timmars tid',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logotyp',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Produkt inställningar',
      'device_settings': 'Device Settings',
      'defaults': 'Förinställningar',
      'basic_settings': 'Grundläggande inställningar',
      'advanced_settings': 'Avancerade inställningar',
      'company_details': 'Företagsinfo',
      'user_details': 'Användaruppgifter',
      'localization': 'Språkanpassning',
      'online_payments': 'Onlinebetalningar',
      'tax_rates': 'Momsnivåer',
      'notifications': 'Meddelanden',
      'import_export': 'Importera/Exportera',
      'custom_fields': 'Anpassade fält',
      'invoice_design': 'Fakturadesign',
      'buy_now_buttons': 'Köp Nu knappar',
      'email_settings': 'E-postinställningar',
      'templates_and_reminders': 'Mallar & Påminnelser',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Datavisualisering',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Villkor för tjänsten',
      'privacy_policy': 'Integritetspolicy',
      'sign_up': 'Registrera dig',
      'account_login': 'Inloggning',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Skapa Ny',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'Ladda ner',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Dokument',
      'documents': 'Dokument',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Kostnads datum',
      'pending': 'Pågående',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Konvertera',
      'add_documents_to_invoice': 'Bifoga dokument till fakturan',
      'exchange_rate': 'Växlingskurs',
      'convert_currency': 'Konvertera valuta',
      'mark_paid': 'Markera betald',
      'category': 'Kategori',
      'address': 'Adress',
      'new_vendor': 'Ny leverantör',
      'created_vendor': 'Framgångsrikt skapat leverantör',
      'updated_vendor': 'Framgångsrikt uppdaterat leverantör',
      'archived_vendor': 'Framgångsrikt arkiverat leverantör',
      'deleted_vendor': 'Framgångsrikt raderat leverantör',
      'restored_vendor': 'Lyckades återställa leverantör',
      'archived_vendors': 'Framgångsrikt arkiverat :count leverantörer',
      'deleted_vendors': 'Framgångsrikt raderat :count leverantörer',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Ny Kostnad',
      'created_expense': 'Framgångsrikt skapat kostnad',
      'updated_expense': 'Framgångsrikt uppdaterat kostnad',
      'archived_expense': 'Framgångsrikt arkiverat kostnad',
      'deleted_expense': 'Framgångsrikt tagit bort kostnad',
      'restored_expense': 'Lyckades återställa utgifter',
      'archived_expenses': 'Framgångsrikt arkiverat kostnader',
      'deleted_expenses': 'Framgångsrikt tagit bort kostnader',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Kopiera frakt',
      'copy_billing': 'Kopiera betalning',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Fakturerad',
      'logged': 'Loggat',
      'running': 'Körs',
      'resume': 'Återuppta',
      'task_errors': 'Korrigera överlappande tider',
      'start': 'Start',
      'stop': 'Stoppa',
      'started_task': 'Startat uppgift utan problem',
      'stopped_task': 'Framgångsrikt stoppad uppgift',
      'resumed_task': 'fortsatt uppgiften utan problem',
      'now': 'Nu',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Timer',
      'manual': 'Manuell',
      'budgeted': 'Budgeted',
      'start_time': 'Start-tid',
      'end_time': 'Sluttid',
      'date': 'Datum',
      'times': 'Tider',
      'duration': 'Varaktighet',
      'new_task': 'Ny uppgift',
      'created_task': 'Framgångsrikt skapad uppgift',
      'updated_task': 'Lyckad uppdatering av uppgift',
      'archived_task': 'Framgångsrikt arkiverad uppgift',
      'deleted_task': 'Framgångsrikt raderad uppgift',
      'restored_task': 'Framgångsrikt återställd uppgift',
      'archived_tasks': 'Framgångsrikt arkiverade :count uppgifter',
      'deleted_tasks': 'Framgångsrikt raderade :count uppgifter',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeterade timmar',
      'created_project': 'Projekt skapat',
      'updated_project': 'Projektet uppdaterat',
      'archived_project': 'Projekt arkiverat',
      'deleted_project': 'Projekt borttaget',
      'restored_project': 'Projekt återställt',
      'archived_projects': ':count projekt arkiverade',
      'deleted_projects': ':count projekt borttagna',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'Nytt Projekt',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'klicka här',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Sidfot',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Anpassat intervall',
      'date_range': 'Datumintervall',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'Denna månaden',
      'last_month': 'Senaste månaden',
      'this_year': 'Detta året',
      'last_year': 'Senaste året',
      'custom': 'Utforma',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Visa faktura',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Redigera kund',
      'edit_product': 'Redigera produkt',
      'edit_invoice': 'Redigera faktura',
      'edit_quote': 'Ändra offert',
      'edit_payment': 'Ändra betalning',
      'edit_task': 'Redigera uppgift',
      'edit_expense': 'Redigera kostnad',
      'edit_vendor': 'Ändra leverantör',
      'edit_project': 'Ändra Produkt',
      'edit_recurring_expense': 'Ändra återkommande utgift',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Fakturaadress',
      'shipping_address': 'Leverans adress',
      'total_revenue': 'Totala intäkter',
      'average_invoice': 'Genomsnittlig faktura',
      'outstanding': 'Utestående/Obetalt',
      'invoices_sent': ':count fakturor skickade',
      'active_clients': 'aktiva kunder',
      'close': 'Stäng',
      'email': 'E-post',
      'password': 'Lösenord',
      'url': 'URL',
      'secret': 'Hemlig',
      'name': 'Namn',
      'logout': 'Logga ut',
      'login': 'Logga in',
      'filter': 'Filter',
      'sort': 'Sort',
      'search': 'Sök',
      'active': 'Aktiv',
      'archived': 'Arkiverad',
      'deleted': 'Ta bort',
      'dashboard': 'Översikt',
      'archive': 'Arkiv',
      'delete': 'Ta bort',
      'restore': 'Återställ',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Stigande',
      'descending': 'Fallande',
      'save': 'Spara',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Betalt hittills',
      'balance_due': 'Resterande belopp',
      'balance': 'Balans',
      'overview': 'Overview',
      'details': 'Detaljer',
      'phone': 'Telefon',
      'website': 'Hemsida',
      'vat_number': 'Momsregistreringsnummer',
      'id_number': 'ID-nummer',
      'create': 'Skapa',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'Kontakter',
      'additional': 'Additional',
      'first_name': 'Förnamn',
      'last_name': 'Efternamn',
      'add_contact': 'Lägg till kontakt',
      'are_you_sure': 'Är du säker?',
      'cancel': 'Avbryt',
      'ok': 'Ok',
      'remove': 'Ta bort',
      'email_is_invalid': 'Email is invalid',
      'product': 'Produkt',
      'products': 'Produkter',
      'new_product': 'Ny produkt',
      'created_product': 'Produkt skapad',
      'updated_product': 'Produkt uppdaterad',
      'archived_product': 'Produkt arkiverad',
      'deleted_product': 'Produkt borttagen',
      'restored_product': 'Produkt återställd',
      'archived_products': 'Arkiverade :count produkter utan problem',
      'deleted_products': ':count produkter borttagna',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Produkt',
      'notes': 'Notis',
      'cost': 'Kostnad',
      'client': 'Kund',
      'clients': 'Kunder',
      'new_client': 'Ny kund',
      'created_client': 'Kund skapad',
      'updated_client': 'Kund uppdaterad',
      'archived_client': 'Kund arkiverad',
      'archived_clients': ':count kunder arkiverade',
      'deleted_client': 'kund borttagen',
      'deleted_clients': ':count kunder borttagna',
      'restored_client': 'Kund återställd',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Adress 1',
      'address2': 'Adress 2',
      'city': 'Ort',
      'state': 'Landskap',
      'postal_code': 'Postnummer',
      'country': 'Land',
      'invoice': 'Faktura',
      'invoices': 'Fakturor',
      'new_invoice': 'Ny faktura',
      'created_invoice': 'Faktura skapad',
      'updated_invoice': 'Faktura uppdaterad',
      'archived_invoice': 'Faktura arkiverad',
      'deleted_invoice': 'Faktura borttagen',
      'restored_invoice': 'Faktura återställd',
      'archived_invoices': ':count fakturor arkiverade',
      'deleted_invoices': ':count fakturor borttagna',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Faktura skickad som e-post',
      'emailed_payment': 'Epostade betalningen utan problem',
      'amount': 'Summa',
      'invoice_number': 'Fakturanummer',
      'invoice_date': 'Fakturadatum',
      'discount': 'Rabatt',
      'po_number': 'Referensnummer',
      'terms': 'Villkor',
      'public_notes': 'Publika noteringar',
      'private_notes': 'Privata anteckningar',
      'frequency': 'Frekvens',
      'start_date': 'Startdatum',
      'end_date': 'Slutdatum',
      'quote_number': 'Offertnummer',
      'quote_date': 'Offertdatum',
      'valid_until': 'Giltig till',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Beskrivning',
      'unit_cost': 'Enhetspris',
      'quantity': 'Antal',
      'add_item': 'Add Item',
      'contact': 'Kontakt',
      'work_phone': 'Telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Sista betalningsdatum',
      'partial_due_date': 'Delvis förfallen',
      'status': 'Status',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Totalsumma',
      'percent': 'Procent',
      'edit': 'Ändra',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Uppgifts taxa',
      'settings': 'Inställningar',
      'language': 'Language',
      'currency': 'Valuta',
      'created_at': 'Skapat datum',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Moms',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Förfallen',
      'draft': 'Utkast',
      'sent': 'Skickat',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'delinsättning',
      'paid': 'Betald',
      'mark_sent': 'Markera skickad',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Klar',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Mörkt läge',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Händelse',
      'no_records_found': 'No records found',
      'clone': 'Kopiera',
      'loading': 'Laddar',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Betalningsvillkor',
      'payment_date': 'Betalningsdatum',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Klient Portal',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Aktiverad',
      'recipients': 'Mottagare',
      'initial_email': 'Påbörja epost',
      'first_reminder': 'Första Påminnelse',
      'second_reminder': 'Andra Påminnelse',
      'third_reminder': 'Tredje Påminnelse',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Mall',
      'send': 'Send',
      'subject': 'Subject',
      'body': 'Organisation/Avdelning',
      'send_email': 'Skicka epost',
      'email_receipt': 'E-posta kvitto till kunden',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'Förhandsgranska',
      'customize': 'Skräddarsy',
      'history': 'Historik',
      'payment': 'Betalning',
      'payments': 'Betalningar',
      'refunded': 'Refunded',
      'payment_type': 'Betalningstyp',
      'transaction_reference': 'Transaktion referens',
      'enter_payment': 'Ange betalning',
      'new_payment': 'Ny betalning',
      'created_payment': 'Betalning registrerad',
      'updated_payment': 'Betalning uppdaterad',
      'archived_payment': 'Betalning arkiverad',
      'deleted_payment': 'Betalning borttagen',
      'restored_payment': 'betalning återställd',
      'archived_payments': ':count betalningar arkiverade',
      'deleted_payments': ':count betalningar borttagna',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Offert',
      'quotes': 'Offerter',
      'new_quote': 'Ny offert',
      'created_quote': 'Offert skapad',
      'updated_quote': 'Offert uppdaterad',
      'archived_quote': 'Offert arkiverad',
      'deleted_quote': 'Offert borttagen',
      'restored_quote': 'Offert återställd',
      'archived_quotes': ':count offerter arkiverade',
      'deleted_quotes': ':count offerter borttagna',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Utgift',
      'expenses': 'Utgifter',
      'vendor': 'Leverantör',
      'vendors': 'Leverantörer',
      'task': 'Uppgift',
      'tasks': 'Uppgifter',
      'project': 'Projekt',
      'projects': 'Projekt',
      'activity_1': ':user skapade kund :client',
      'activity_2': ':user arkiverade kund :client',
      'activity_3': ':user raderade kund :client',
      'activity_4': ':user skapade faktura :invoice',
      'activity_5': ':user uppdaterade faktura :invoice',
      'activity_6': ':user mailade faktura :invoice för :client till :contact',
      'activity_7': ':contact visade faktura :invoice för :client',
      'activity_8': ':user arkiverade faktura :invoice',
      'activity_9': ':user raderade faktura :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user uppdaterade betalning :payment',
      'activity_12': ':user arkiverade betalning :payment',
      'activity_13': ':user tog bort betalning :payment',
      'activity_14': ':user skickade in :credit kredit',
      'activity_15': ':user updaterade :credit kredit',
      'activity_16': ':user arkiverade :credit kredit',
      'activity_17': ':user tog bort :credit kredit',
      'activity_18': ':user skapade offert :quote',
      'activity_19': ':user uppdaterade offert :quote',
      'activity_20': ':user mailade offert :quote för :client för :contact',
      'activity_21': ':contact visade offert :quote',
      'activity_22': ':user arkiverade offert :quote',
      'activity_23': ':user tog bort offert :quote',
      'activity_24': ':user återställde offert :quote',
      'activity_25': ':user återställde Faktura :invoice',
      'activity_26': ':user återställde klient :client',
      'activity_27': ':user återställde betalning :payment',
      'activity_28': ':user återställde :credit kredit',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user skapade leverantör :vendor',
      'activity_31': ':user arkiverade leverantör :vendor',
      'activity_32': ':user tog bort leverantör :vendor',
      'activity_33': ':user återställde leverantör :vendor',
      'activity_34': ':user skapade kostnad :expense',
      'activity_35': ':user arkiverade kostnad :expense',
      'activity_36': ':user tog bort kostnad :expense',
      'activity_37': ':user återställde kostnad :expense',
      'activity_39': ':user avbröt en :payment_amount betalning :payment',
      'activity_40':
          ':user återbetalade :adjustment av en :payment_amount betalning :payment',
      'activity_41': ':payment_amount betalning (:payment) misslyckad',
      'activity_42': ':user skapade uppgift :task',
      'activity_43': ':user uppdaterade uppgift :task',
      'activity_44': ':user arkiverade uppgift :task',
      'activity_45': ':user tog bort uppgift :task',
      'activity_46': ':user återställde uppgift :task',
      'activity_47': ':user uppdaterade kostnad :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'Engångs lösenord',
      'emailed_quote': 'Offert e-postad',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Utgått',
      'all': 'Alla',
      'select': 'Välj',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Anpassat värde',
      'custom_value2': 'Anpassat värde',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Fakturaräknare',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Offerträknare',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Typ',
      'invoice_amount': 'Faktura belopp',
      'invoice_due_date': 'Förfallodatum',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Auto debitera',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Skattenamn',
      'tax_amount': 'Moms summa',
      'tax_paid': 'Moms betalad',
      'payment_amount': 'Betald summa',
      'age': 'Ålder',
      'is_running': 'Is Running',
      'time_log': 'Tidslogg',
      'bank_id': 'Bank',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Kostnads kategori',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'th': {
      'import_data': 'นำเข้าข้อมูล',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'ชื่อที่ติดต่อ',
      'contact_last_name': 'นามสกุลที่ติดต่อ',
      'order': 'Order',
      'unassigned': 'ยังไม่ได้มอบหมาย',
      'partial_value': 'ต้องมากกว่าศูนย์และน้อยกว่ายอดรวม',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'เปิดใช้งาน',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'ตั้งค่าโดเมนย่อยหรือแสดงใบแจ้งหนี้ในเว็บไซต์ของคุณเอง.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'ความปลอดภัย',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'ส่งคำเชิญอีกครั้ง',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'การคืนเงิน',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'แปลงเป็นใบแจ้งหนี้',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'งานใบแจ้งหนี้',
      'invoice_expense': 'ค่าใช้จ่ายในใบแจ้งหนี้',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'เอกสารเริ่มต้น',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'ซ่อน',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'คอลัมน์',
      'sample': 'ตัวอย่าง',
      'map_to': 'Map To',
      'import': 'นำเข้า',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'กรุณาเลือกไฟล์',
      'no_file_selected': 'No File Selected',
      'csv_file': 'ไฟล์ CSV',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'การบริการ',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'ยังไม่จ่าย',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'ครบกำหนด',
      'invoice_total': 'ยอดรวมตามใบแจ้งหนี้',
      'quote_total': 'ยอดรวมทั้งหมด',
      'credit_total': 'เครดิตทั้งหมด',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'คำเตือน',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'ชื่อลูกค้า',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'หมวดค่าใช้จ่าย',
      'new_expense_category': 'หมวดค่าใช้จ่ายใหม่',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'สร้างหมวดหมู่ค่าใช้จ่ายเรียบร้อยแล้ว',
      'updated_expense_category': 'อัปเดตหมวดค่าใช้จ่ายเรียบร้อยแล้ว',
      'archived_expense_category': 'หมวดหมู่ค่าใช้จ่ายที่เก็บสำเร็จแล้ว',
      'deleted_expense_category': 'นำออกหมวดหมู่เรียบร้อยแล้ว',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'กู้คืนหมวดหมู่ค่าใช้จ่ายเรียบร้อยแล้ว',
      'archived_expense_categories':
          'จัดเก็บเรียบร้อยแล้ว :count หมวดค่าใช้จ่าย',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'ควรได้รับใบแจ้งหนี้',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'ทำเครื่องหมายว่าใช้งานอยู่',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'ทำใบแจ้งหนี้ซ้ำ',
      'recurring_invoices': 'ทำใบแจ้งหนี้ซ้ำ',
      'new_recurring_invoice': 'ใบแจ้งหนี้ซ้ำใหม่',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'เก็บบันทึกใบแจ้งหนี้ประจำแล้ว',
      'deleted_recurring_invoice': 'ลบใบแจ้งหนี้ประจำแล้ว',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice': 'กู้คืนใบแจ้งหนี้แล้ว',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'กำไร',
      'line_item': 'รายการ',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'ดูพอร์ทัล',
      'copy_link': 'Copy Link',
      'token_billing': 'บันทึกรายละเอียดบัตรเครดิต',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'ตลอดเวลา',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'ป้ายกำกับ',
      'client_number': 'หมายเลขลูกค้า',
      'auto_convert': 'Auto Convert',
      'company_name': 'ชื่อบริษัท',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'ส่งอีเมลใบแจ้งหนี้เรียบร้อยแล้ว',
      'emailed_quotes': 'ส่งอีเมลใบเสนอราคาเรียบร้อยแล้ว',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'เกตเวย์',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'ชั่วโมง',
      'statement': 'รายงาน',
      'taxes': 'ภาษี',
      'surcharge': 'คิดค่าใช้จ่าย',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'เลือกป้ายกำกับ',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'ไปยัง',
      'health_check': 'Health Check',
      'payment_type_id': 'ประเภทการจ่ายเงิน',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'ใบแจ้งหนี้ที่จะเกิดขึ้น',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'การชำระเงินล่าสุด',
      'upcoming_quotes': 'ใบเสนอราคาที่จะเกิดขึ้น',
      'expired_quotes': 'ใบเสนอราคาหมดอายุ',
      'create_client': 'Create Client',
      'create_invoice': 'สร้างใบแจ้งหนี้',
      'create_quote': 'สร้างใบเสนอราคา',
      'create_payment': 'Create Payment',
      'create_vendor': 'สร้างผู้ขาย',
      'update_quote': 'Update Quote',
      'delete_quote': 'ลบใบเสนอราคา',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'ลบใบแจ้งหนี้',
      'update_client': 'Update Client',
      'delete_client': 'ลบลูกค้า',
      'delete_payment': 'ลบการจ่ายเงิน',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'ลบผู้ขาย',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'ลบค่าใช้จ่าย',
      'create_task': 'สร้างงาน',
      'update_task': 'Update Task',
      'delete_task': 'ลบงาน',
      'approve_quote': 'Approve Quote',
      'off': 'ปิด',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'ฟรี',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokens',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokens',
      'new_token': 'New Token',
      'edit_token': 'แก้ไข Token',
      'created_token': 'สร้าง Token เรียบร้อย',
      'updated_token': 'อัปเดท Token เรียบร้อย',
      'archived_token': 'เก็บบันทึก Token แล้ว',
      'deleted_token': 'ลบ Token เรียบร้อย',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'ส่งอีเมล',
      'email_quote': 'อีเมล์ใบเสนอราคา',
      'email_credit': 'Email Credit',
      'email_payment': 'การชำระเงินทางอีเมล',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'ชื่อผู้ติดต่อ',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'แก้ไขข้อกำหนดในการชำระเงิน',
      'created_payment_term': 'สร้างเงื่อนไขการชำระเงินเรียบร้อยแล้ว',
      'updated_payment_term': 'อัปเดตเงื่อนไขการชำระเงินเรียบร้อยแล้ว',
      'archived_payment_term': 'เก็บบันทึกระยะเวลาการชำระเงินเรียบร้อยแล้ว',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'ยอดเครดิต',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'พิเศษ',
      'inclusive': 'รวมทั้ง',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'การชำระเงินคืน',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'ชื่อเต็ม',
      'city_state_postal': 'เมือง / รัฐ / ไปรษณีย์',
      'postal_city_state': 'ไปรษณีย์ / เมือง / รัฐ',
      'custom1': 'กำหนดเองครั้งแรก',
      'custom2': 'กำหนดเองครั้งที่สอง',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'ล้างข้อมูล',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'คำเตือน: การดำเนินการนี้จะลบข้อมูลของคุณอย่างถาวรและไม่สามารถนำคืนกลับมาได้',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 วัน',
      'age_group_30': '30 - 60 วัน',
      'age_group_60': '60 - 90 วัน',
      'age_group_90': '90 - 120 วัน',
      'age_group_120': '120+ วัน',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'รายละเอียดใบแจ้งหนี้',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'สิทธิ์',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count ส่งใบแจ้งหนี้',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'สมัครไลเซนต์',
      'cancel_account': 'ลบบัญชี',
      'cancel_account_message':
          'คำเตือน: การดำเนินการนี้จะลบบัญชีของคุณอย่างถาวรและไม่สามารถนำกลับมาได้',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'ส่วนหัว',
      'load_design': 'โหลดการออกแบบ',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Recurring Quotes',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'ค่าใช้จ่ายที่เกิดขึ้นประจำ',
      'account_management': 'การจัดการบัญชี',
      'credit_date': 'วันที่เครดิต',
      'credit': 'เครดิต',
      'credits': 'เครดิต',
      'new_credit': 'ป้อนเครดิต',
      'edit_credit': 'แก้ไขเครดิต',
      'created_credit': 'สร้างเครดิตเรียบร้อยแล้ว',
      'updated_credit': 'อัปเดตเครดิตแล้ว',
      'archived_credit': 'เก็บเครดิตเรียบร้อยแล้ว',
      'deleted_credit': 'ลบเครดิตเรียบร้อยแล้ว',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'กู้คืนเครดิตเรียบร้อยแล้ว',
      'archived_credits': 'เก็บข้อมูลเรียบร้อยแล้ว :count เครดิต',
      'deleted_credits': 'ลบเรียบร้อยแล้ว :count เครดิต',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'รุ่นปัจจุบัน',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'อ่านต่อ',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'บริษัทใหม่',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'รีเซ็ต',
      'number': 'Number',
      'export': 'ส่งออก',
      'chart': 'แผนภูมิ',
      'count': 'Count',
      'totals': 'ทั้งหมด',
      'blank': 'ว่าง',
      'day': 'วัน',
      'month': 'เดือน',
      'year': 'ปี',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'จัดกลุ่มตาม',
      'credit_balance': 'เครดิตคงเหลือ',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'โทรศัพท์ติดต่อ',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'รหัสลูกค้า',
      'assigned_to': 'Assigned to',
      'created_by': 'สร้างโดย :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'คอลัม',
      'aging': 'อายุลูกหนี้',
      'profit_and_loss': 'กำไรและขาดทุน',
      'reports': 'รายงาน',
      'report': 'รายงาน',
      'add_company': 'เพิ่ม บริษัท',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'ช่วยเหลือ',
      'refund': 'คืนเงิน',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'ติดต่ออีเมล์',
      'multiselect': 'Multiselect',
      'entity_state': 'สถานะ',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'ข้อความ',
      'from': 'จาก',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'support forum',
      'about': 'About',
      'documentation': 'เอกสาร',
      'contact_us': 'ติดต่อเรา',
      'subtotal': 'รวมเงิน',
      'line_total': 'รวมเงิน',
      'item': 'รายการ',
      'credit_email': 'Credit Email',
      'iframe_url': 'Website',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'รหัสผ่านสั้นเกินไป',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'ใช่',
      'no': 'ไม่ใช่',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'ดู',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'ผู้ใช้งาน',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'โปรดเลือกลูกค้า',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'การตั้งค่าภาษี',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'สลับ',
      'comma_sparated_list': 'Comma separated list',
      'options': 'ตัวเลือก',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'ยอมรับ',
      'recover_password': 'กู้คืนรหัสผ่านของคุณ',
      'late_fees': 'Late Fees',
      'credit_number': 'หมายเลขเครดิต',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'ค่าปรับล่าช้าจำนวน',
      'late_fee_percent': 'ค่าปรับล่าช้าเป็นเปอร์เซนต์',
      'schedule': 'ตารางเวลา',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'วัน',
      'invoice_email': 'อีเมลใบแจ้งหนี้',
      'payment_email': 'อีเมลการชำระเงิน',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'อีเมล์ใบเสนอราคา',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'ผู้ดูแลระบบ',
      'administrator_help':
          'อนุญาตให้ผู้ใช้ จัดการผู้ใช้อื่น เปลี่ยนแปลงการตั้งค่าและแก้ไขระเบียนทั้งหมด',
      'user_management': 'การจัดการผู้ใช้',
      'users': 'ผู้ใช้งาน',
      'new_user': 'ผู้ใช้ใหม่',
      'edit_user': 'แก้ไขผู้ใช้',
      'created_user': 'Successfully created user',
      'updated_user': 'อัปเดตผู้ใช้เรียบร้อยแล้ว',
      'archived_user': 'เก็บบันทึกผู้ใช้แล้ว',
      'deleted_user': 'ลบผู้ใช้เรียบร้อยแล้ว',
      'removed_user': 'Successfully removed user',
      'restored_user': 'กู้คืนผู้ใช้เรียบร้อยแล้ว',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'การตั้งค่าทั่วไป',
      'invoice_options': 'ตัวเลือกใบเสนอราคา',
      'hide_paid_to_date': 'ซ่อนการจ่ายเงินไปยังวันที่',
      'hide_paid_to_date_help':
          'แสดงเฉพาะพื้นที่ \"ชำระเงินถึงวันที่\" ในใบแจ้งหนี้ของคุณเมื่อได้รับการชำระเงินแล้ว',
      'invoice_embed_documents': 'ฝังเอกสาร',
      'invoice_embed_documents_help': 'รวมภาพที่แนบมาในใบแจ้งหนี้',
      'all_pages_header': 'แสดงหัวเรื่อง',
      'all_pages_footer': 'แสดงส่วนท้าย',
      'first_page': 'หน้าแรก',
      'all_pages': 'หน้าทั้งหมด',
      'last_page': 'หน้าสุดท้าย',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'สีหลัก',
      'secondary_color': 'สีรอง',
      'page_size': 'ขนาดหน้า',
      'font_size': 'ขนาดตัวอักษร',
      'quote_design': 'ออกแบบใบเสนอราคา',
      'invoice_fields': 'ฟิลด์ใบเสนอราคา',
      'product_fields': 'ข้อมูลสินค้า',
      'invoice_terms': 'ระยะเวลาใบแจ้งหนี้',
      'invoice_footer': 'ส่วนท้ายของใบแจ้งหนี้',
      'quote_terms': 'ระยะเวลาใบเสนอราคา',
      'quote_footer': 'สุดท้ายใบเสนอราคา',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'แปลงอัตโนมัติ',
      'auto_convert_quote_help':
          'แปลงใบเสนอราคาให้เป็นใบแจ้งหนี้โดยอัตโนมัติเมื่อได้รับอนุมัติจากลูกค้า',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Daily',
      'freq_weekly': 'รายสัปดาห์',
      'freq_two_weeks': 'สองสัปดาห์',
      'freq_four_weeks': 'สี่สับดาห์',
      'freq_monthly': 'รายเดือน',
      'freq_two_months': '2 เดือน',
      'freq_three_months': 'สามเดือน',
      'freq_four_months': 'Four months',
      'freq_six_months': 'หกเดือน',
      'freq_annually': 'รายปี',
      'freq_two_years': 'Two years',
      'freq_three_years': 'Three Years',
      'never': 'ไม่เคย',
      'company': 'Company',
      'generated_numbers': 'ตัวเลขที่สร้างขึ้น',
      'charge_taxes': 'ภาษีค่าบริการ',
      'next_reset': 'รีเซ็ตครั้งต่อไป',
      'reset_counter': 'รีเซ็ตตัวนับ',
      'recurring_prefix': 'คำนำหน้า ที่เกิดขึ้นประจำ',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Client Field',
      'product_field': 'Product Field',
      'payment_field': 'Payment Field',
      'contact_field': 'Contact Field',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'คำนำหน้า',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'ปรับแต่ง CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Checkbox เงื่อนไขในใบแจ้งหนี้',
      'show_accept_invoice_terms_help':
          'กำหนดให้ลูกค้ายืนยันว่ายอมรับข้อกำหนดในใบแจ้งหนี้',
      'show_accept_quote_terms': 'Checkbox เงื่อนไขใบเสนอราคา',
      'show_accept_quote_terms_help':
          'กำหนดให้ลูกค้ายืนยันว่ายอมรับเงื่อนไขการเสนอราคา',
      'require_invoice_signature': 'ลายเซ็นของใบแจ้งหนี้',
      'require_invoice_signature_help': 'กำหนดให้ลูกค้าจัดหาลายเซ็น',
      'require_quote_signature': 'ลายมือชื่อใบเสนอราคา',
      'enable_portal_password': 'รหัสผ่านป้องกันใบแจ้งหนี้',
      'enable_portal_password_help':
          'ช่วยให้คุณสามารถตั้งรหัสผ่านสำหรับแต่ละรายชื่อ หากมีการตั้งรหัสผ่านผู้ติดต่อจะต้องป้อนรหัสผ่านก่อนดูใบแจ้งหนี้',
      'authorization': 'การอนุญาต',
      'subdomain': 'Subdomain',
      'domain': 'โดเมน',
      'portal_mode': 'Portal Mode',
      'email_signature': 'ด้วยความเคารพ',
      'enable_email_markup_help':
          'ทำให้ลูกค้าของคุณสามารถจ่ายเงินให้คุณได้ง่ายขึ้นโดยการเพิ่มมาร์กอัป schema.org ลงในอีเมลของคุณ',
      'plain': 'ธรรมดา',
      'light': 'บาง',
      'dark': 'มืด',
      'email_design': 'ออกแบบอีเมล์',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'เปิดใช้งาน Markup',
      'reply_to_email': 'ตอบกลับอีเมล',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'บัตรเครดิต',
      'bank_transfer': 'โอนเงินผ่านธนาคาร',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'เปิดใช้ค่าต่ำสุด',
      'enable_max': 'เปิดใช้ค่าสูงสุด',
      'min_limit': 'ต่ำสุด :min',
      'max_limit': 'สูงสุด :max',
      'min': 'น้อย',
      'max': 'มาก',
      'accepted_card_logos': 'ยอมรับโลโก้ของบัตร',
      'credentials': 'Credentials',
      'update_address': 'อัปเดตที่อยู่',
      'update_address_help': 'อัปเดตที่อยู่ของลูกค้าโดยระบุรายละเอียดไว้',
      'rate': 'อัตรา',
      'tax_rate': 'อัตราภาษี',
      'new_tax_rate': 'อัตราภาษีใหม่',
      'edit_tax_rate': 'แก้ไขอัตราภาษี',
      'created_tax_rate': 'อัปเดตอัตราภาษีเรียบร้อยแล้ว',
      'updated_tax_rate': 'อัปเดตอัตราภาษีเรียบร้อยแล้ว',
      'archived_tax_rate': 'เก็บบันทึกอัตราภาษีเรียบร้อยแล้ว',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'เติมข้อมูลอัตโนมัติ',
      'fill_products_help':
          'การเลือกสินค้า จะเติมคำอธิบายและค่าใช้จ่ายโดยอัตโนมัติ',
      'update_products': 'อัปเดตผลิตภัณฑ์โดยอัตโนมัติ',
      'update_products_help': 'การอัปเดตใบแจ้งหนี้ จะอัปเดตสินค้าโดยอัตโนมัติ',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'ค่าธรรมเนียม',
      'limits': 'จำกัด',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'ปิดการใช้',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'วันอาทิตย์',
      'monday': 'วันจันทร์',
      'tuesday': 'วันอังคาร',
      'wednesday': 'วันพุธ',
      'thursday': 'วันพฤหัสบดี',
      'friday': 'วันศุกร์',
      'saturday': 'วันเสาร์',
      'january': 'มกราคม',
      'february': 'กุมภาพันธ์',
      'march': 'มีนาคม',
      'april': 'เมษายน',
      'may': 'พฤษภาคม',
      'june': 'มิถุนายน',
      'july': 'กรกฎาคม',
      'august': 'สิงหาคม',
      'september': 'กันยายน',
      'october': 'ตุลาคม',
      'november': 'พฤศจิกายน',
      'december': 'ธันวาคม',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 ชั่วโมง',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'www',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'การตั้งค่าสินค้า',
      'device_settings': 'Device Settings',
      'defaults': 'ค่าเริ่มต้น',
      'basic_settings': 'ตั้งค่าพื้นฐาน',
      'advanced_settings': 'ตั้งค่าขั้นสูง',
      'company_details': 'รายละเอียดบริษัท',
      'user_details': 'รายละเอียดผู้ใช้',
      'localization': 'การปรับให้เข้ากับท้องถิ่น',
      'online_payments': 'การจ่ายเงินออนไลน์',
      'tax_rates': 'อัตราภาษี',
      'notifications': 'การแจ้งเตือน',
      'import_export': 'นำเข้า | ส่งออก',
      'custom_fields': 'ฟิลด์ที่กำหนดเอง',
      'invoice_design': 'ออกแบบใบแจ้งหนี้',
      'buy_now_buttons': 'ปุ่มซื้อเดี๋ยวนี้',
      'email_settings': 'ตั้งค่าอีเมล์',
      'templates_and_reminders': 'เทมเพลตและการแจ้งเตือน',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'การแสดงภาพข้อมูล',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'เงื่อนไขการให้บริการ',
      'privacy_policy': 'นโยบายความเป็นส่วนตัว',
      'sign_up': 'ลงทะเบียน',
      'account_login': 'ลงชื่อเข้าใช้',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'สร้างใหม่',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'ดาวน์โหลด',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'เอกสาร:',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'วันที่เบิกจ่าย',
      'pending': 'รอดำเนินการ',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'แปลง',
      'add_documents_to_invoice': 'เพิ่มเอกสารลงในใบแจ้งหนี้',
      'exchange_rate': 'อัตราแลกเปลี่ยน',
      'convert_currency': 'แปลงสกุลเงิน',
      'mark_paid': 'ทำเครื่องว่าจ่ายเงินแล้ว',
      'category': 'แคตตาล็อก',
      'address': 'ที่อยู่',
      'new_vendor': 'ผู้ขายใหม่',
      'created_vendor': 'สร้างผู้ขายสำเร็จแล้ว',
      'updated_vendor': 'อัปเดตผู้ขายสำเร็จแล้ว',
      'archived_vendor': 'การจัดเก็บผู้ขายประสบความสำเร็จ',
      'deleted_vendor': 'ลบผู้ขายเรียบร้อยแล้ว',
      'restored_vendor': 'ผู้ขายที่ได้รับการคืนค่าเรียบร้อยแล้ว',
      'archived_vendors': 'เก็บเข้าคลังเรียบร้อยแล้ว :count ผู้ขาย',
      'deleted_vendors': 'ลบเรียบร้อยแล้ว :count ผู้ขาย',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'ป้อนค่าใช้จ่าย',
      'created_expense': 'สร้างค่าใช้จ่ายสำเร็จ',
      'updated_expense': 'อัปเดตค่าใช้จ่ายสำเร็จ',
      'archived_expense': 'เก็บบันทึกค่าใช้จ่ายสำเร็จ',
      'deleted_expense': 'ลบค่าใช้จ่ายสำเร็จ',
      'restored_expense': 'กู้คืนค่าใช้จ่ายเรียบร้อยแล้ว',
      'archived_expenses': 'เก็บบันทึกค่าใช้จ่ายสำเร็จ',
      'deleted_expenses': 'ลบค่าใช้จ่ายสำเร็จ',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'ออกแบบ',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'ออกใบแจ้งหนี้',
      'logged': 'บันทึกการเข้า',
      'running': 'กำลังทำงาน',
      'resume': 'ทำต่อไป',
      'task_errors': 'โปรดแก้ไขเวลาซ้อนทับกัน',
      'start': 'เริ่ม',
      'stop': 'หยุด',
      'started_task': 'Successfully started task',
      'stopped_task': 'หยุดงานเรียบร้อย',
      'resumed_task': 'ทำงานต่อสำเร็จ',
      'now': 'ตอนนี้',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'จับเวลา',
      'manual': 'คู่มือ',
      'budgeted': 'Budgeted',
      'start_time': 'เวลาเริ่มต้น',
      'end_time': 'เวลาสิ้นสุด',
      'date': 'วันที่',
      'times': 'เวลา',
      'duration': 'ระยะเวลา',
      'new_task': 'งานใหม่',
      'created_task': 'สร้างงานเรียบร้อยแล้ว',
      'updated_task': 'อัปเดตงานสำเร็จแล้ว',
      'archived_task': 'เก็บบันทึกงานเรียบร้อยแล้ว',
      'deleted_task': 'ลบงานเรียบร้อย',
      'restored_task': 'กู้คืนงานเรียบร้อยแล้ว',
      'archived_tasks': 'เก็บเรียบร้อยแล้ว :count งาน',
      'deleted_tasks': 'ลบงานเรียบร้อย :count งาน',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'โปรดระบุชื่อ',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'สร้างโครงการสำเร็จ',
      'updated_project': 'อัปเดตโครงการสำเร็จ',
      'archived_project': 'เก็บบันทึกโครงการสำเร็จ',
      'deleted_project': 'ลบโครงการสำเร็จ',
      'restored_project': 'กู้คืนโครงการสำเร็จ',
      'archived_projects': 'เก็บบันทึก :count โครงการ',
      'deleted_projects': 'ลบโครงการ :count โครงการ',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'โครงการใหม่',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'คลิกที่นี่',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'ค่าเฉลี่ย',
      'unapproved': 'ไม่อนุมัติ',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'ส่วนท้าย',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'ระบุช่วง',
      'date_range': 'ช่วงวันที่',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'เดือนนี้',
      'last_month': 'เดือนล่าสุด',
      'this_year': 'ปีนี้',
      'last_year': 'ปีล่าสุด',
      'custom': 'กำหนดเอง',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'ดูใบแจ้งหนี้',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'แก้ไขลูกค้า',
      'edit_product': 'แก้ไขสินค้า',
      'edit_invoice': 'แก้ไขใบแจ้งหนี้',
      'edit_quote': 'แก้ไขใบเสนอราคา',
      'edit_payment': 'แก้ไขรายการจ่ายเงิน',
      'edit_task': 'แก้ไขงาน',
      'edit_expense': 'แก้ไขค่าใช้จ่าย',
      'edit_vendor': 'แก้ไขผู้ขาย',
      'edit_project': 'แก้ไขโครงการ',
      'edit_recurring_expense': 'แก้ไขค่าใช้จ่ายที่เกิดประจำ',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'ที่อยู่เรียกเก็บเงิน',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'รายได้รวม',
      'average_invoice': 'ใบแจ้งหนี้เฉลี่ย',
      'outstanding': 'โดดเด่น',
      'invoices_sent': ':count ส่งใบแจ้งหนี้',
      'active_clients': 'ลูกค้าที่ใช้งานอยู่',
      'close': 'ปิด',
      'email': 'อีเมล',
      'password': 'รหัสผ่าน',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'ชื่อ',
      'logout': 'ออกจากระบบ',
      'login': 'เข้าสู่ระบบ',
      'filter': 'กรอง',
      'sort': 'Sort',
      'search': 'ค้นหา',
      'active': 'ใช้งานอยู่',
      'archived': 'เก็บถาวร',
      'deleted': 'ลบแล้ว',
      'dashboard': 'แดชบอร์ด',
      'archive': 'เอกสารเก่า',
      'delete': 'ลบ',
      'restore': 'กู้คืน',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'บันทึก',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'ยอดชำระแล้ว',
      'balance_due': 'ยอดคงเหลือ',
      'balance': 'ยอดคงเหลือ',
      'overview': 'Overview',
      'details': 'รายละเอียด',
      'phone': 'โทร.',
      'website': 'เว็บไซต์',
      'vat_number': 'หมายเลขภาษี',
      'id_number': 'หมายเลขประจำตัวประชาชน',
      'create': 'สร้าง',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Error',
      'could_not_launch': 'Could not launch',
      'contacts': 'ผู้ติดต่อ',
      'additional': 'Additional',
      'first_name': 'ชื่อ',
      'last_name': 'นามสกุล',
      'add_contact': 'เพิ่มผู้ติดต่อ',
      'are_you_sure': 'แน่ใจหรือไม่?',
      'cancel': 'ยกเลิก',
      'ok': 'Ok',
      'remove': 'เอาออก',
      'email_is_invalid': 'Email is invalid',
      'product': 'สินค้า',
      'products': 'ผลิตภัณฑ์',
      'new_product': 'สินค้าใหม่',
      'created_product': 'สร้างสินค้าสำเร็จแล้ว',
      'updated_product': 'อัปเดตสินค้าสำเร็จแล้ว',
      'archived_product': 'บันทึกสินค้าสำเร็จ',
      'deleted_product': 'ลบสินค้าแล้ว',
      'restored_product': 'กู้คืนสินค้าแล้ว',
      'archived_products': 'เก็บบันทึก :count สินค้า',
      'deleted_products': 'ลบสินค้า :count สินค้า',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'สินค้า',
      'notes': 'บันทึก',
      'cost': 'ค่าใช้จ่าย',
      'client': 'ลูกค้า',
      'clients': 'ลูกค้า',
      'new_client': 'เพิ่มลูกค้า',
      'created_client': 'สร้างลูกค้าเรียบร้อยแล้ว',
      'updated_client': 'อัปเดตลูกค้าเรียบร้อยแล้ว',
      'archived_client': 'เก็บข้อมูลลูกค้าเรียบร้อยแล้ว',
      'archived_clients': 'เก็บข้อมูลเรียบร้อยแล้ว: นับลูกค้า',
      'deleted_client': 'ลบลูกค้าเรียบร้อยแล้ว',
      'deleted_clients': 'ลบลูกค้าเรียบร้อยแล้ว :count ลูกค้า',
      'restored_client': 'กู้คืนลูกค้าเรียบร้อยแล้ว',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'ถนน',
      'address2': 'อาคาร',
      'city': 'อำเภอ',
      'state': 'จังหวัด',
      'postal_code': 'รหัสไปรษณีย์',
      'country': 'ประเทศ',
      'invoice': 'ใบแจ้งหนี้',
      'invoices': 'ใบแจ้งหนี้',
      'new_invoice': 'เพิ่มใบแจ้งหนี้',
      'created_invoice': 'สร้างใบแจ้งหนี้เรียบร้อยแล้ว',
      'updated_invoice': 'อัปเดตใบแจ้งหนี้สำเร็จแล้ว',
      'archived_invoice': 'เก็บบันทึกใบแจ้งหนี้เรียบร้อยแล้ว',
      'deleted_invoice': 'ลบใบแจ้งหนี้เรียบร้อยแล้ว',
      'restored_invoice': 'กู้คืนใบแจ้งหนี้เรียบร้อยแล้ว',
      'archived_invoices': 'บันทึกเรียบร้อยแล้ว :count ใบแจ้งหนี้',
      'deleted_invoices': 'ลบเรียบร้อยแล้ว :count ใบแจ้งหนี้',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'ส่งใบแจ้งหนี้ทางอีเมลเรียบร้อยแล้ว',
      'emailed_payment': 'ส่งอีเมลเรียบร้อยแล้ว',
      'amount': 'ยอดเงิน',
      'invoice_number': 'เลขที่ใบแจ้งหนี้',
      'invoice_date': 'วันที่แจ้งหนี้',
      'discount': 'ส่วนลด',
      'po_number': 'เลขที่ใบสั่งซื้อ',
      'terms': 'เงื่อนไข',
      'public_notes': 'หมายเหตุแบบเปิด',
      'private_notes': 'หมายเหตุภายใน',
      'frequency': 'ความถี่',
      'start_date': 'วันที่เริ่ม',
      'end_date': 'วันที่สิ้นสุด',
      'quote_number': 'หมายเลขใบเสนอราคา',
      'quote_date': 'วันที่อ้างอิง',
      'valid_until': 'ใช้ได้ถึงวันที่',
      'items': 'Items',
      'partial_deposit': 'Partial/Deposit',
      'description': 'รายละเอียด',
      'unit_cost': 'ราคาต่อหน่วย',
      'quantity': 'จำนวน',
      'add_item': 'Add Item',
      'contact': 'ผู้ติดต่อ',
      'work_phone': 'โทรศัพท์',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'วันถึงกำหนดชำระ',
      'partial_due_date': 'Partial Due Date',
      'status': 'สถานะ',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'คลิ๊กปุ่ม + เพื่อเพิ่มรายการ',
      'count_selected': ':count selected',
      'total': 'ทั้งหมด',
      'percent': 'เปอร์เซนต์',
      'edit': 'แก้ไข',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'การตั้งค่า',
      'language': 'Language',
      'currency': 'สกุลเงิน',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'ภาษี',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'ดราฟ',
      'sent': 'ส่ง',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'บางส่วน / เงินฝาก',
      'paid': 'จ่ายเงิน',
      'mark_sent': 'ทำเครื่องหมายไว้',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'เรียบร้อย',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'โหมดกลางคืน',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'กิจกรรม',
      'no_records_found': 'No records found',
      'clone': 'ทำซ้ำ',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'เงื่อนไขการชำระ',
      'payment_date': 'กำหนดจ่าย',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Portal ลูกค้า',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'เปิด',
      'recipients': 'ผู้รับ',
      'initial_email': 'อีเมลเริ่มต้น',
      'first_reminder': 'คำเตือนครั้งแรก',
      'second_reminder': 'คำเตือนครั้งที่สอง',
      'third_reminder': 'Third Reminder',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'แบบ',
      'send': 'Send',
      'subject': 'เรื่อง',
      'body': 'เนื้อเรื่อง',
      'send_email': 'ส่งอีเมล',
      'email_receipt': 'ใบเสร็จการชำระเงินทางอีเมลให้กับลูกค้า',
      'auto_billing': 'Auto billing',
      'button': 'Button',
      'preview': 'ดูตัวอย่าง',
      'customize': 'ปรับแต่ง',
      'history': 'ประวัติ',
      'payment': 'การจ่ายเงิน',
      'payments': 'การจ่ายเงิน',
      'refunded': 'Refunded',
      'payment_type': 'ประเภทการชำระเงิน',
      'transaction_reference': 'รายการอ้างอิง',
      'enter_payment': 'เพิ่มรายการจ่ายเงิน',
      'new_payment': 'ป้อนการชำระเงิน',
      'created_payment': 'สร้างการชำระเงินเรียบร้อยแล้ว',
      'updated_payment': 'การปรับปรุงรายการจ่ายเงินเสร็จสมบูรณ์',
      'archived_payment': 'เก็บการชำระเงินเรียบร้อยแล้ว',
      'deleted_payment': 'ลบการชำระเงินเรียบร้อยแล้ว',
      'restored_payment': 'กู้คืนการชำระเงินเรียบร้อยแล้ว',
      'archived_payments': 'เก็บข้อมูลเรียบร้อยแล้ว :count การชำระเงิน',
      'deleted_payments': 'ลบเรียบร้อยแล้ว :count การชำระเงิน',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'ใบเสนอราคา',
      'quotes': 'ใบเสนอราคา',
      'new_quote': 'ใบเสนอราคาใหม่',
      'created_quote': 'สร้างใบเสนอราคาเรียบร้อย',
      'updated_quote': 'อัปเดทใบเสนอราคาเรียบร้อย',
      'archived_quote': 'เก็บบันทึกใบเสนอราคาเรียบร้อย',
      'deleted_quote': 'ลบใบเสนอราคาเรียบร้อย',
      'restored_quote': 'กู้คืนใบเสนอราคาเรียบร้อยแล้ว',
      'archived_quotes': 'เก็บบันทึกเรียบร้อย :count ใบเสนอราคา',
      'deleted_quotes': 'ลบเรียบร้อย :count ใบเสนอราคา',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'ค่าใช้จ่าย',
      'expenses': 'ค่าใช้จ่าย',
      'vendor': 'ผู้ขาย',
      'vendors': 'ผู้ขาย',
      'task': 'งาน',
      'tasks': 'งาน',
      'project': 'โครงการ',
      'projects': 'โครงการ',
      'activity_1': ':user ได้สร้างผู้ใช้ :client',
      'activity_2': ':user เก็บบันทึก :client',
      'activity_3': ':user ได้ลบผู้ใช้ :client',
      'activity_4': ':user ได้สร้างใบแจ้งหนี้ :invoice',
      'activity_5': ':user ได้อัปเดทแจ้งหนี้ :invoice',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user บันทึกใบแจ้งหนี้ :invoice',
      'activity_9': ':user ได้ลบใบแจ้งหนี้ :invoice',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user  อัปเดตการชำระเงินที่แล้ว :payment',
      'activity_12': ':user เก็บบันทึกการจ่ายเงิน  :payment',
      'activity_13': ':user ลบการจ่ายเงิน :payment',
      'activity_14': ':user ป้อน :credit เครดิต',
      'activity_15': ':user อัปเดต :credit เครดิต',
      'activity_16': ':user เก็บบันทึก :credit เครดิต',
      'activity_17': ':user ลบแล้ว :credit เครดิต',
      'activity_18': ':user สร้างใบเสนอราคา :quote',
      'activity_19': ';user อัปเดตใบเสนอราคา :quote',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact ดูใบเสนอราคา :quote',
      'activity_22': ':user เก็บบันทึกใบเสนอราคา :quote',
      'activity_23': ':user ลบใบเสนอราคา :quote',
      'activity_24': ':user กู้คืนใบเสนอราคา :quote',
      'activity_25': ':user กู้คืนใบแจ้งหนี้ :invoice',
      'activity_26': ':user กู้คืน ลูกค้า :client',
      'activity_27': ':user กู้คืนการชำระเงิน :payment',
      'activity_28': ':user กู้คืน :credit เครดิต',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user สร้างผู้ขาย :vendor',
      'activity_31': ':user เก็บบันทึกผู้ขาย :vendor',
      'activity_32': ':user ได้ลบผู้ขาย :vendor',
      'activity_33': ':user ได้กู้คืนผู้ขาย :vendor',
      'activity_34': ':user ได้สร้างค่าใช้จ่าย :expense',
      'activity_35': ':user ได้เก็บบันทึกค่าใช้จ่าย :expense',
      'activity_36': ':user ได้ลบค่าใช้จ่าย :expense',
      'activity_37': ':user ได้กู้คืนค่าใช้จ่าย :expense',
      'activity_39': ':user ยกเลิก :payment_amount  การชำระเงิน :payment',
      'activity_40':
          ':usre คืนเงิน :adjustment ของ  :payment_amount การชำระเงิน :payment',
      'activity_41': ':payment_amount จ่ายชำระเงิน (:payment) ล้มเหลว',
      'activity_42': ':user ได้สร้างงาน :task',
      'activity_43': ':user ได้อัปเดตงาน :task',
      'activity_44': ':user ได้บันทึกงาน :task',
      'activity_45': ':user ได้ลบงาน :task',
      'activity_46': ':user ได้กู้คืนงาน :task',
      'activity_47': ':user ได้อัปเดตค่าใช้จ่าย :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'อีเมล์ใบเสนอราคาเรียบร้อย',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'หมดอายุ',
      'all': 'ทั้งหมด',
      'select': 'เลือก',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'การนับจำนวนใบแจ้งหนี้',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'การนับจำนวนใบเสนอราคา',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'ชนิด',
      'invoice_amount': 'จำนวนใบแจ้งหนี้',
      'invoice_due_date': 'วันครบกำหนด',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'บิลอัตโนมัติ',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'ชื่อภาษี',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'ยอดจ่าย',
      'age': 'อายุ',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'ธนาคาร',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'หมวดค่าใช้จ่าย',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
    'tr_TR': {
      'import_data': 'Verileri İçe Aktar',
      'import_settings': 'Import Settings',
      'json_file_missing': 'Please provide the JSON file',
      'json_option_missing': 'Please select to import the settings and/or data',
      'json': 'JSON',
      'no_payment_types_enabled': 'No payment types enabled',
      'wait_for_data': 'Please wait for the data to finish loading',
      'net_total': 'Net Total',
      'has_taxes': 'Has Taxes',
      'import_customers': 'Import Customers',
      'imported_customers': 'Successfully started importing customers',
      'login_success': 'Successful Login',
      'login_failure': 'Failed Login',
      'exported_data':
          'Once the file is ready you\"ll receive an email with a download link',
      'include_deleted_clients': 'Include Deleted Clients',
      'include_deleted_clients_help':
          'Load records belonging to deleted clients',
      'step_1_sign_in': 'Step 1: Sign In',
      'step_2_authorize': 'Step 2: Authorize',
      'account_id': 'Account ID',
      'migration_not_yet_completed': 'The migration has not yet completed',
      'activity_100': ':user created recurring invoice :recurring_invoice',
      'activity_101': ':user updated recurring invoice :recurring_invoice',
      'activity_102': ':user archived recurring invoice :recurring_invoice',
      'activity_103': ':user deleted recurring invoice :recurring_invoice',
      'activity_104': ':user restored recurring invoice :recurring_invoice',
      'show_task_end_date': 'Show Task End Date',
      'show_task_end_date_help': 'Enable specifying the task end date',
      'gateway_setup': 'Gateway Setup',
      'preview_sidebar': 'Preview Sidebar',
      'years_data_shown': 'Years Data Shown',
      'ended_all_sessions': 'Successfully ended all sessions',
      'end_all_sessions': 'End All Sessions',
      'count_session': '1 Session',
      'count_sessions': ':count Sessions',
      'invoice_created': 'Invoice Created',
      'quote_created': 'Quote Created',
      'credit_created': 'Credit Created',
      'pro': 'Pro',
      'enterprise': 'Enterprise',
      'last_updated': 'Last Updated',
      'invoice_item': 'Invoice Item',
      'quote_item': 'Quote Item',
      'contact_first_name': 'Contact First Name',
      'contact_last_name': 'Contact Last Name',
      'order': 'Order',
      'unassigned': 'Unassigned',
      'partial_value': 'Sıfırdan büyük olmalı ve toplamdan daha az olmalı',
      'search_kanban': 'Search Kanban',
      'search_kanbans': 'Search Kanban',
      'kanban': 'Kanban',
      'enable': 'Etkinleştir',
      'move_top': 'Move Top',
      'move_up': 'Move Up',
      'move_down': 'Move Down',
      'move_bottom': 'Move Bottom',
      'subdomain_help':
          'Set the subdomain or display the invoice on your own website.',
      'body_variable_missing':
          'Error: the custom email must include a :body variable',
      'add_body_variable_message': 'Make sure to include a :body variable',
      'view_date_formats': 'View Date Formats',
      'is_viewed': 'Is Viewed',
      'letter': 'Letter',
      'legal': 'Legal',
      'page_layout': 'Page Layout',
      'portrait': 'Portrait',
      'landscape': 'Landscape',
      'owner_upgrade_to_paid_plan':
          'The account owner can upgrade to a paid plan to enable the advanced advanced settings',
      'upgrade_to_paid_plan':
          'Upgrade to a paid plan to enable the advanded settings',
      'invoice_payment_terms': 'Invoice Payment Terms',
      'quote_valid_until': 'Quote Valid Until',
      'no_headers': 'No Headers',
      'add_header': 'Add Header',
      'remove_header': 'Remove Header',
      'return_url': 'Return URL',
      'rest_method': 'REST Method',
      'header_key': 'Header Key',
      'header_value': 'Header Value',
      'recurring_products': 'Recurring Products',
      'promo_code': 'Promo code',
      'promo_discount': 'Promo Discount',
      'allow_cancellation': 'Allow Cancellation',
      'per_seat_enabled': 'Per Seat Enabled',
      'max_seats_limit': 'Max Seats Limit',
      'trial_enabled': 'Trial Enabled',
      'trial_duration': 'Trial Duration',
      'allow_query_overrides': 'Allow Query Overrides',
      'allow_plan_changes': 'Allow Plan Changes',
      'plan_map': 'Plan Map',
      'refund_period': 'Refund Period',
      'webhook_configuration': 'Webhook Configuration',
      'purchase_page': 'Purchase Page',
      'security': 'Security',
      'email_bounced': 'Email Bounced',
      'email_spam_complaint': 'Spam Complaint',
      'email_delivery': 'Email Delivery',
      'webhook_response': 'Webhook Response',
      'pdf_response': 'PDF Response',
      'authentication_failure': 'Authentication Failure',
      'pdf_failed': 'PDF Failed',
      'pdf_success': 'PDF Success',
      'modified': 'Modified',
      'subscription': 'Subscription',
      'subscriptions': 'Subscriptions',
      'new_subscription': 'New Subscription',
      'edit_subscription': 'Edit Subscription',
      'created_subscription': 'Successfully created subscription',
      'updated_subscription': 'Successfully updated subscription',
      'archived_subscription': 'Successfully archived subscription',
      'deleted_subscription': 'Successfully deleted subscription',
      'removed_subscription': 'Successfully removed subscription',
      'restored_subscription': 'Successfully restored subscription',
      'search_subscription': 'Search 1 Subscription',
      'search_subscriptions': 'Search :count Subscriptions',
      'subdomain_is_not_available': 'Subdomain is not available',
      'connect_gmail': 'Connect Gmail',
      'disconnect_gmail': 'Disconnect Gmail',
      'connected_gmail': 'Successfully connected Gmail',
      'disconnected_gmail': 'Successfully disconnected Gmail',
      'update_fail_help':
          'Changes to the codebase may be blocking the update, you can run this command to discard the changes:',
      'client_id_number': 'Client ID Number',
      'count_minutes': ':count Minutes',
      'password_timeout': 'Password Timeout',
      'shared_invoice_credit_counter': 'Shared Invoice/Credit Counter',
      'use_last_email': 'Use last email',
      'activate_company': 'Activate Company',
      'activate_company_help':
          'Enable emails, recurring invoices and notifications',
      'an_error_occurred_try_again': 'An error occurred, please try again',
      'please_first_set_a_password': 'Please first set a password',
      'changing_phone_disables_two_factor':
          'Warning: Changing your phone number will disable 2FA',
      'help_translate': 'Help Translate',
      'please_select_a_country': 'Please select a country',
      'resend_invite': 'Resend Invitation',
      'disabled_two_factor': 'Successfully disabled 2FA',
      'connected_google': 'Successfully connected account',
      'disconnected_google': 'Successfully disconnected account',
      'delivered': 'Delivered',
      'bounced': 'Bounced',
      'spam': 'Spam',
      'view_docs': 'View Docs',
      'enter_phone_to_enable_two_factor':
          'Please provide a mobile phone number to enable two factor authentication',
      'send_sms': 'Send SMS',
      'sms_code': 'SMS Code',
      'two_factor_setup_help': 'Scan the bar code with a :link compatible app.',
      'enabled_two_factor': 'Successfully enabled Two-Factor Authentication',
      'connect_google': 'Connect Google',
      'disconnect_google': 'Disconnect Google',
      'enable_two_factor': 'Two-Factor Authentication',
      'disable_two_factor': 'Disable Two Factor',
      'require_password_with_social_login':
          'Require Password with Social Login',
      'stay_logged_in': 'Stay Logged In',
      'session_about_to_expire': 'Warning: Your session is about to expire',
      'count_hours': ':count Hours',
      'count_day': '1 Day',
      'count_days': ':count Days',
      'web_session_timeout': 'Web Session Timeout',
      'security_settings': 'Security Settings',
      'resend_email': 'Resend Email',
      'confirm_your_email_address': 'Please confirm your email address',
      'refunded_payment': 'Refunded Payment',
      'partially_unapplied': 'Partially Unapplied',
      'select_a_gmail_user': 'Please select a user authenticated with Gmail',
      'list_long_press': 'List Long Press',
      'show_actions': 'Show Actions',
      'start_multiselect': 'Start Multiselect',
      'email_sent_to_confirm_email':
          'An email has been sent to confirm the email address',
      'counter_pattern_error':
          'To use :client_counter please add either :client_number or :client_id_number to prevent conflicts',
      'this_quarter': 'This Quarter',
      'last_quarter': 'Last Quarter',
      'to_update_run': 'To update run',
      'convert_to_invoice': 'Faturaya Dönüştür',
      'registration_url': 'Registration URL',
      'invoice_project': 'Invoice Project',
      'invoice_task': 'Fatura Görevi',
      'invoice_expense': 'Gider Faturası',
      'search_payment_term': 'Search 1 Payment Term',
      'search_payment_terms': 'Search :count Payment Terms',
      'save_and_preview': 'Save and Preview',
      'save_and_email': 'Save and Email',
      'supported_events': 'Supported Events',
      'converted_amount': 'Converted Amount',
      'converted_balance': 'Converted Balance',
      'converted_paid_to_date': 'Converted Paid to Date',
      'converted_credit_balance': 'Converted Credit Balance',
      'converted_total': 'Converted Total',
      'is_sent': 'Is Sent',
      'default_documents': 'Default Documents',
      'document_upload': 'Document Upload',
      'document_upload_help': 'Enable clients to upload documents',
      'expense_total': 'Expense Total',
      'enter_taxes': 'Enter Taxes',
      'by_rate': 'By Rate',
      'by_amount': 'By Amount',
      'enter_amount': 'Enter Amount',
      'before_taxes': 'Before Taxes',
      'after_taxes': 'After Taxes',
      'color': 'Color',
      'show': 'Show',
      'hide': 'Gizle',
      'empty_columns': 'Empty Columns',
      'debug_mode_is_enabled': 'Debug mode is enabled',
      'debug_mode_is_enabled_help':
          'Warning: it is intended for use on local machines, it can leak credentials. Click to learn more.',
      'running_tasks': 'Running Tasks',
      'recent_tasks': 'Recent Tasks',
      'recent_expenses': 'Recent Expenses',
      'upcoming_expenses': 'Upcoming Expenses',
      'update_app': 'Update App',
      'started_import': 'Successfully started import',
      'duplicate_column_mapping': 'Duplicate column mapping',
      'uses_inclusive_taxes': 'Uses Inclusive Taxes',
      'is_amount_discount': 'Is Amount Discount',
      'column': 'Sütun',
      'sample': 'Örnek',
      'map_to': 'Map To',
      'import': 'İçe Aktar',
      'first_row_as_column_names': 'Use first row as column names',
      'select_file': 'Lütfen bir dosya seçin',
      'no_file_selected': 'No File Selected',
      'csv_file': 'CSV dosya',
      'csv': 'CSV',
      'freshbooks': 'FreshBooks',
      'invoice2go': 'Invoice2go',
      'invoicely': 'Invoicely',
      'waveaccounting': 'Wave Accounting',
      'zoho': 'Zoho',
      'accounting': 'Accounting',
      'required_files_missing': 'Please provide all CSVs.',
      'import_type': 'Import Type',
      'html_mode': 'HTML Mode',
      'html_mode_help': 'Preview updates faster but is less accurate',
      'view_licenses': 'View Licenses',
      'webhook_url': 'Webhook URL',
      'fullscreen_editor': 'Fullscreen Editor',
      'sidebar_editor': 'Sidebar Editor',
      'please_type_to_confirm': 'Please type \":value\" to confirm',
      'purge': 'Purge',
      'service': 'Service',
      'clone_to': 'Clone To',
      'clone_to_other': 'Clone to Other',
      'labels': 'Labels',
      'add_custom': 'Add Custom',
      'payment_tax': 'Payment Tax',
      'unpaid': 'Unpaid',
      'white_label': 'White Label',
      'delivery_note': 'Delivery Note',
      'sent_invoices_are_locked': 'Sent invoices are locked',
      'paid_invoices_are_locked': 'Paid invoices are locked',
      'source_code': 'Source Code',
      'app_platforms': 'App Platforms',
      'invoice_late': 'Invoice Late',
      'quote_expired': 'Quote Expired',
      'partial_due': 'Partial Due',
      'invoice_total': 'Fatura Toplam',
      'quote_total': 'Teklif Toplam',
      'credit_total': 'Credit Total',
      'recurring_invoice_total': 'Invoice Total',
      'actions': 'Actions',
      'expense_number': 'Expense Number',
      'task_number': 'Task Number',
      'project_number': 'Project Number',
      'project_name': 'Project Name',
      'warning': 'Warning',
      'view_settings': 'View Settings',
      'company_disabled_warning':
          'Warning: this company has not yet been activated',
      'late_invoice': 'Late Invoice',
      'expired_quote': 'Expired Quote',
      'remind_invoice': 'Remind Invoice',
      'cvv': 'CVV',
      'client_name': 'Müşteri Adı',
      'client_phone': 'Client Phone',
      'required_fields': 'Required Fields',
      'calculated_rate': 'Calculated Rate',
      'default_task_rate': 'Default Task Rate',
      'clear_cache': 'Clear Cache',
      'sort_order': 'Sort Order',
      'task_status': 'Status',
      'task_statuses': 'Task Statuses',
      'new_task_status': 'New Task Status',
      'edit_task_status': 'Edit Task Status',
      'created_task_status': 'Successfully created task status',
      'updated_task_status': 'Successfully update task status',
      'archived_task_status': 'Successfully archived task status',
      'deleted_task_status': 'Successfully deleted task status',
      'removed_task_status': 'Successfully removed task status',
      'restored_task_status': 'Successfully restored task status',
      'archived_task_statuses': 'Successfully archived :value task statuses',
      'deleted_task_statuses': 'Successfully deleted :value task statuses',
      'restored_task_statuses': 'Successfully restored :value task statuses',
      'search_task_status': 'Search 1 Task Status',
      'search_task_statuses': 'Search :count Task Statuses',
      'show_tasks_table': 'Show Tasks Table',
      'show_tasks_table_help':
          'Always show the tasks section when creating invoices',
      'invoice_task_timelog': 'Invoice Task Timelog',
      'invoice_task_timelog_help': 'Add time details to the invoice line items',
      'invoice_task_datelog': 'Invoice Task Datelog',
      'invoice_task_datelog_help': 'Add date details to the invoice line items',
      'auto_start_tasks_help': 'Start tasks before saving',
      'configure_statuses': 'Configure Statuses',
      'task_settings': 'Task Settings',
      'configure_categories': 'Configure Categories',
      'expense_categories': 'Gider Kategorisi',
      'new_expense_category': 'Yeni Gider Kategorisi',
      'edit_expense_category': 'Edit Expense Category',
      'created_expense_category': 'Successfully created expense category',
      'updated_expense_category': 'Successfully updated expense category',
      'archived_expense_category': 'Successfully archived expense category',
      'deleted_expense_category': 'Successfully deleted category',
      'removed_expense_category': 'Successfully removed expense category',
      'restored_expense_category': 'Successfully restored expense category',
      'archived_expense_categories':
          'Successfully archived :count expense category',
      'deleted_expense_categories':
          'Successfully deleted expense :value categories',
      'restored_expense_categories':
          'Successfully restored expense :value categories',
      'search_expense_category': 'Search 1 Expense Category',
      'search_expense_categories': 'Search :count Expense Categories',
      'use_available_credits': 'Use Available Credits',
      'show_option': 'Show Option',
      'negative_payment_error':
          'The credit amount cannot exceed the payment amount',
      'view_changes': 'View Changes',
      'force_update': 'Force Update',
      'force_update_help':
          'You are running the latest version but there may be pending fixes available.',
      'mark_paid_help': 'Track the expense has been paid',
      'should_be_invoiced': 'Faturalanmalı mı',
      'should_be_invoiced_help': 'Enable the expense to be invoiced',
      'add_documents_to_invoice_help': 'Make the documents visible',
      'convert_currency_help': 'Set an exchange rate',
      'expense_settings': 'Expense Settings',
      'clone_to_recurring': 'Clone to Recurring',
      'crypto': 'Crypto',
      'paypal': 'PayPal',
      'alipay': 'Alipay',
      'sofort': 'Sofort',
      'apple_pay': 'Apple/Google Pay',
      'user_field': 'User Field',
      'variables': 'Variables',
      'show_password': 'Show Password',
      'hide_password': 'Hide Password',
      'copy_error': 'Copy Error',
      'capture_card': 'Capture Card',
      'auto_bill_enabled': 'Auto Bill Enabled',
      'total_taxes': 'Total Taxes',
      'line_taxes': 'Line Taxes',
      'total_fields': 'Total Fields',
      'stopped_recurring_invoice': 'Successfully stopped recurring invoice',
      'started_recurring_invoice': 'Successfully started recurring invoice',
      'resumed_recurring_invoice': 'Successfully resumed recurring invoice',
      'gateway_refund': 'Gateway Refund',
      'gateway_refund_help': 'Process the refund with the payment gateway',
      'due_date_days': 'Due Date',
      'paused': 'Paused',
      'mark_active': 'Mark Active',
      'day_count': 'Day :count',
      'first_day_of_the_month': 'First Day of the Month',
      'last_day_of_the_month': 'Last Day of the Month',
      'use_payment_terms': 'Use Payment Terms',
      'endless': 'Endless',
      'next_send_date': 'Next Send Date',
      'remaining_cycles': 'Remaining Cycles',
      'recurring_invoice': 'Tekrarlayan Fatura',
      'recurring_invoices': 'Tekrarlayan Faturalar',
      'new_recurring_invoice': 'Yeni Tekrarlayan Fatura',
      'edit_recurring_invoice': 'Edit Recurring Invoice',
      'created_recurring_invoice': 'Successfully created recurring invoice',
      'updated_recurring_invoice': 'Successfully updated recurring invoice',
      'archived_recurring_invoice': 'Tekrarlayan fatura başarıyla arşivlendi',
      'deleted_recurring_invoice': 'Tekrarlayan fatura başarıyla silindi',
      'removed_recurring_invoice': 'Successfully removed recurring invoice',
      'restored_recurring_invoice':
          'Tekrarlayan fatura başarıyla geri yüklendi',
      'archived_recurring_invoices':
          'Successfully archived recurring :value invoices',
      'deleted_recurring_invoices':
          'Successfully deleted recurring :value invoices',
      'restored_recurring_invoices':
          'Successfully restored recurring :value invoices',
      'search_recurring_invoice': 'Search 1 Recurring Invoice',
      'search_recurring_invoices': 'Search :count Recurring Invoices',
      'send_date': 'Send Date',
      'auto_bill_on': 'Auto Bill On',
      'minimum_under_payment_amount': 'Minimum Under Payment Amount',
      'profit': 'Profit',
      'line_item': 'Line Item',
      'allow_over_payment': 'Allow Over Payment',
      'allow_over_payment_help': 'Support paying extra to accept tips',
      'allow_under_payment': 'Allow Under Payment',
      'allow_under_payment_help':
          'Support paying at minimum the partial/deposit amount',
      'test_mode': 'Test Mode',
      'opened': 'Opened',
      'payment_reconciliation_failure': 'Reconciliation Failure',
      'payment_reconciliation_success': 'Reconciliation Success',
      'gateway_success': 'Gateway Success',
      'gateway_failure': 'Gateway Failure',
      'gateway_error': 'Gateway Error',
      'email_send': 'Email Send',
      'email_retry_queue': 'Email Retry Queue',
      'failure': 'Failure',
      'quota_exceeded': 'Quota Exceeded',
      'upstream_failure': 'Upstream Failure',
      'system_logs': 'System Logs',
      'view_portal': 'View Portal',
      'copy_link': 'Copy Link',
      'token_billing': 'Kart bilgilerini sakla',
      'welcome_to_invoice_ninja': 'Welcome to Invoice Ninja',
      'always': 'Always',
      'optin': 'Opt-In',
      'optout': 'Opt-Out',
      'label': 'Label',
      'client_number': 'Client Number',
      'auto_convert': 'Auto Convert',
      'company_name': 'Şirket Adı',
      'reminder1_sent': 'Reminder 1 Sent',
      'reminder2_sent': 'Reminder 2 Sent',
      'reminder3_sent': 'Reminder 3 Sent',
      'reminder_last_sent': 'Reminder Last Sent',
      'pdf_page_info': 'Page :current of :total',
      'emailed_invoices': 'Successfully emailed invoices',
      'emailed_quotes': 'Successfully emailed quotes',
      'emailed_credits': 'Successfully emailed credits',
      'gateway': 'Gateway',
      'view_in_stripe': 'View in Stripe',
      'rows_per_page': 'Rows Per Page',
      'hours': 'Saat',
      'statement': 'Statement',
      'taxes': 'Vergiler',
      'surcharge': 'Surcharge',
      'apply_payment': 'Apply Payment',
      'apply': 'Apply',
      'unapplied': 'Unapplied',
      'select_label': 'Select Label',
      'custom_labels': 'Custom Labels',
      'record_type': 'Record Type',
      'record_name': 'Record Name',
      'file_type': 'File Type',
      'height': 'Height',
      'width': 'Width',
      'to': 'Kime',
      'health_check': 'Health Check',
      'payment_type_id': 'Ödeme Türü',
      'last_login_at': 'Last Login At',
      'company_key': 'Company Key',
      'storefront': 'Storefront',
      'storefront_help': 'Enable third-party apps to create invoices',
      'client_created': 'Client Created',
      'online_payment_email': 'Online Payment Email',
      'manual_payment_email': 'Manual Payment Email',
      'completed': 'Completed',
      'gross': 'Gross',
      'net_amount': 'Net Amount',
      'net_balance': 'Net Balance',
      'client_settings': 'Client Settings',
      'selected_invoices': 'Selected Invoices',
      'selected_payments': 'Selected Payments',
      'selected_quotes': 'Selected Quotes',
      'selected_tasks': 'Selected Tasks',
      'selected_expenses': 'Selected Expenses',
      'upcoming_invoices': 'Yaklaşan Faturalar',
      'past_due_invoices': 'Past Due Invoices',
      'recent_payments': 'Son Ödemeler',
      'upcoming_quotes': 'Tarihi Yaklaşan Teklifler',
      'expired_quotes': 'Tarihi Dolan Teklifler',
      'create_client': 'Create Client',
      'create_invoice': 'Fatura Oluştur',
      'create_quote': 'Teklif Oluştur',
      'create_payment': 'Create Payment',
      'create_vendor': 'Create vendor',
      'update_quote': 'Update Quote',
      'delete_quote': 'Teklif Sil',
      'update_invoice': 'Update Invoice',
      'delete_invoice': 'Faturayı Sil',
      'update_client': 'Update Client',
      'delete_client': 'Müşteri Sil',
      'delete_payment': 'Ödeme Sil',
      'update_vendor': 'Update Vendor',
      'delete_vendor': 'Tedarikçiyi Sil',
      'create_expense': 'Create Expense',
      'update_expense': 'Update Expense',
      'delete_expense': 'Gider Sil',
      'create_task': 'Görev Oluştur',
      'update_task': 'Update Task',
      'delete_task': 'Görev Sil',
      'approve_quote': 'Approve Quote',
      'off': 'Off',
      'when_paid': 'When Paid',
      'expires_on': 'Expires On',
      'free': 'Ücretsiz',
      'plan': 'Plan',
      'show_sidebar': 'Show Sidebar',
      'hide_sidebar': 'Hide Sidebar',
      'event_type': 'Event Type',
      'target_url': 'Target',
      'copy': 'Copy',
      'must_be_online': 'Please restart the app once connected to the internet',
      'crons_not_enabled': 'The crons need to be enabled',
      'api_webhooks': 'API Webhooks',
      'search_webhooks': 'Search :count Webhooks',
      'search_webhook': 'Search 1 Webhook',
      'webhook': 'Webhook',
      'webhooks': 'Webhooks',
      'new_webhook': 'New Webhook',
      'edit_webhook': 'Edit Webhook',
      'created_webhook': 'Successfully created webhook',
      'updated_webhook': 'Successfully updated webhook',
      'archived_webhook': 'Successfully archived webhook',
      'deleted_webhook': 'Successfully deleted webhook',
      'removed_webhook': 'Successfully removed webhook',
      'restored_webhook': 'Successfully restored webhook',
      'archived_webhooks': 'Successfully archived :value webhooks',
      'deleted_webhooks': 'Successfully deleted :value webhooks',
      'removed_webhooks': 'Successfully removed :value webhooks',
      'restored_webhooks': 'Successfully restored :value webhooks',
      'api_tokens': 'API Tokenları',
      'api_docs': 'API Docs',
      'search_tokens': 'Search :count Tokens',
      'search_token': 'Search 1 Token',
      'token': 'Token',
      'tokens': 'Tokenlar',
      'new_token': 'New Token',
      'edit_token': 'Token düzenle',
      'created_token': 'Token başarıyla oluşturuldu',
      'updated_token': 'Token başarıyla güncellendi',
      'archived_token': 'Token başarıyla arşivlendi',
      'deleted_token': 'Token başarıyla silindi',
      'removed_token': 'Successfully removed token',
      'restored_token': 'Successfully restored token',
      'archived_tokens': 'Successfully archived :value tokens',
      'deleted_tokens': 'Successfully deleted :value tokens',
      'restored_tokens': 'Successfully restored :value tokens',
      'client_registration': 'Client Registration',
      'client_registration_help':
          'Enable clients to self register in the portal',
      'customize_and_preview': 'Customize & Preview',
      'email_invoice': 'Faturayı E-Posta ile gönder',
      'email_quote': 'Teklifi E-Posta ile Gönder',
      'email_credit': 'Email Credit',
      'email_payment': 'Email Payment',
      'client_email_not_set': 'Client does not have an email address set',
      'ledger': 'Ledger',
      'view_pdf': 'View PDF',
      'all_records': 'All records',
      'owned_by_user': 'Owned by user',
      'credit_remaining': 'Credit Remaining',
      'contact_name': 'Contact Name',
      'use_default': 'Use default',
      'reminder_endless': 'Endless Reminders',
      'number_of_days': 'Number of days',
      'configure_payment_terms': 'Configure Payment Terms',
      'payment_term': 'Payment Term',
      'new_payment_term': 'New Payment Term',
      'edit_payment_term': 'Edit Payment Term',
      'created_payment_term': 'Successfully created payment term',
      'updated_payment_term': 'Successfully updated payment term',
      'archived_payment_term': 'Successfully archived payment term',
      'deleted_payment_term': 'Successfully deleted payment term',
      'removed_payment_term': 'Successfully removed payment term',
      'restored_payment_term': 'Successfully restored payment term',
      'archived_payment_terms': 'Successfully archived :value payment terms',
      'deleted_payment_terms': 'Successfully deleted :value payment terms',
      'restored_payment_terms': 'Successfully restored :value payment terms',
      'email_sign_in': 'Login with email',
      'change': 'Change',
      'change_to_mobile_layout': 'Change to the mobile layout?',
      'change_to_desktop_layout': 'Change to the desktop layout?',
      'send_from_gmail': 'Send from Gmail',
      'reversed': 'Reversed',
      'cancelled': 'Cancelled',
      'credit_amount': 'Kredi Tutarı',
      'quote_amount': 'Quote Amount',
      'hosted': 'Hosted',
      'selfhosted': 'Self-Hosted',
      'exclusive': 'Exclusive',
      'inclusive': 'Inclusive',
      'hide_menu': 'Hide Menu',
      'show_menu': 'Show Menu',
      'partially_refunded': 'Partially Refunded',
      'search_documents': 'Search Documents',
      'search_designs': 'Search Designs',
      'search_invoices': 'Search Invoices',
      'search_clients': 'Search Clients',
      'search_products': 'Search Products',
      'search_quotes': 'Search Quotes',
      'search_credits': 'Search Credits',
      'search_vendors': 'Search Vendors',
      'search_users': 'Search Users',
      'search_tax_rates': 'Search Tax Rates',
      'search_tasks': 'Search Tasks',
      'search_settings': 'Search Settings',
      'search_projects': 'Search Projects',
      'search_expenses': 'Search Expenses',
      'search_payments': 'Search Payments',
      'search_groups': 'Search Groups',
      'search_company': 'Search Company',
      'search_document': 'Search 1 Document',
      'search_design': 'Search 1 Design',
      'search_invoice': 'Search 1 Invoice',
      'search_client': 'Search 1 Client',
      'search_product': 'Search 1 Product',
      'search_quote': 'Search 1 Quote',
      'search_credit': 'Search 1 Credit',
      'search_vendor': 'Search 1 Vendor',
      'search_user': 'Search 1 User',
      'search_tax_rate': 'Search 1 Tax Rate',
      'search_task': 'Search 1 Tasks',
      'search_project': 'Search 1 Project',
      'search_expense': 'Search 1 Expense',
      'search_payment': 'Search 1 Payment',
      'search_group': 'Search 1 Group',
      'refund_payment': 'Refund Payment',
      'cancelled_invoice': 'Successfully cancelled invoice',
      'cancelled_invoices': 'Successfully cancelled invoices',
      'reversed_invoice': 'Successfully reversed invoice',
      'reversed_invoices': 'Successfully reversed invoices',
      'reverse': 'Reverse',
      'full_name': 'Full Name',
      'city_state_postal': 'City/State/Postal',
      'postal_city_state': 'Postal/City/State',
      'custom1': 'First Custom',
      'custom2': 'Second Custom',
      'custom3': 'Third Custom',
      'custom4': 'Fourth Custom',
      'optional': 'Optional',
      'license': 'License',
      'purge_data': 'Purge Data',
      'purge_successful': 'Successfully purged company data',
      'purge_data_message':
          'Warning: This will permanently erase your data, there is no undo.',
      'invoice_balance': 'Invoice Balance',
      'age_group_0': '0 - 30 Days',
      'age_group_30': '30 - 60 Days',
      'age_group_60': '60 - 90 Days',
      'age_group_90': '90 - 120 Days',
      'age_group_120': '120+ Days',
      'refresh': 'Refresh',
      'saved_design': 'Successfully saved design',
      'client_details': 'Client Details',
      'company_address': 'Company Address',
      'invoice_details': 'Invoice Details',
      'quote_details': 'Quote Details',
      'credit_details': 'Credit Details',
      'product_columns': 'Product Columns',
      'task_columns': 'Task Columns',
      'add_field': 'Add Field',
      'all_events': 'All Events',
      'permissions': 'Permissions',
      'none': 'None',
      'owned': 'Owned',
      'payment_success': 'Payment Success',
      'payment_failure': 'Payment Failure',
      'invoice_sent': ':count fatura gönderildi',
      'quote_sent': 'Quote Sent',
      'credit_sent': 'Credit Sent',
      'invoice_viewed': 'Invoice Viewed',
      'quote_viewed': 'Quote Viewed',
      'credit_viewed': 'Credit Viewed',
      'quote_approved': 'Quote Approved',
      'receive_all_notifications': 'Receive All Notifications',
      'purchase_license': 'Purchase License',
      'apply_license': 'Apply License',
      'cancel_account': 'Hesabı Sil',
      'cancel_account_message':
          'Warning: This will permanently delete your account, there is no undo.',
      'delete_company': 'Delete Company',
      'delete_company_message':
          'Warning: This will permanently delete your company, there is no undo.',
      'enabled_modules': 'Enabled Modules',
      'converted_quote': 'Successfully converted quote',
      'credit_design': 'Credit Design',
      'includes': 'Includes',
      'header': 'Üstbilgi',
      'load_design': 'Load Design',
      'css_framework': 'CSS Framework',
      'custom_designs': 'Custom Designs',
      'designs': 'Designs',
      'new_design': 'New Design',
      'edit_design': 'Edit Design',
      'created_design': 'Successfully created design',
      'updated_design': 'Successfully updated design',
      'archived_design': 'Successfully archived design',
      'deleted_design': 'Successfully deleted design',
      'removed_design': 'Successfully removed design',
      'restored_design': 'Successfully restored design',
      'archived_designs': 'Successfully archived :value designs',
      'deleted_designs': 'Successfully deleted :value designs',
      'restored_designs': 'Successfully restored :value designs',
      'proposals': 'Proposals',
      'tickets': 'Tickets',
      'recurring_quotes': 'Tekrarlayan Fiyat Teklifleri',
      'recurring_tasks': 'Recurring Tasks',
      'recurring_expenses': 'Recurring Expenses',
      'account_management': 'Account Management',
      'credit_date': 'Kredi Tarihi',
      'credit': 'Kredi',
      'credits': 'Krediler',
      'new_credit': 'Kredi Gir',
      'edit_credit': 'Edit Credit',
      'created_credit': 'Kredi başarıyla oluşturuldu',
      'updated_credit': 'Successfully updated credit',
      'archived_credit': 'Kredi başarıyla arşivlendi',
      'deleted_credit': 'Kredi başarıyla silindi',
      'removed_credit': 'Successfully removed credit',
      'restored_credit': 'Kredi Başarıyla Geri Yüklendi',
      'archived_credits': ':count kredi arşivlendi',
      'deleted_credits': ':count kredi başarıyla silindi',
      'restored_credits': 'Successfully restored :value credits',
      'current_version': 'Mevcut version',
      'latest_version': 'Latest Version',
      'update_now': 'Update Now',
      'a_new_version_is_available': 'A new version of the web app is available',
      'update_available': 'Update Available',
      'app_updated': 'Update successfully completed',
      'learn_more': 'Daha fazla bilgi edin',
      'integrations': 'Integrations',
      'tracking_id': 'Tracking Id',
      'slack_webhook_url': 'Slack Webhook URL',
      'credit_footer': 'Credit Footer',
      'credit_terms': 'Credit Terms',
      'new_company': 'Yeni Firma',
      'added_company': 'Successfully added company',
      'company1': 'Custom Company 1',
      'company2': 'Custom Company 2',
      'company3': 'Custom Company 3',
      'company4': 'Custom Company 4',
      'product1': 'Custom Product 1',
      'product2': 'Custom Product 2',
      'product3': 'Custom Product 3',
      'product4': 'Custom Product 4',
      'client1': 'Custom Client 1',
      'client2': 'Custom Client 2',
      'client3': 'Custom Client 3',
      'client4': 'Custom Client 4',
      'contact1': 'Custom Contact 1',
      'contact2': 'Custom Contact 2',
      'contact3': 'Custom Contact 3',
      'contact4': 'Custom Contact 4',
      'task1': 'Custom Task 1',
      'task2': 'Custom Task 2',
      'task3': 'Custom Task 3',
      'task4': 'Custom Task 4',
      'project1': 'Custom Project 1',
      'project2': 'Custom Project 2',
      'project3': 'Custom Project 3',
      'project4': 'Custom Project 4',
      'expense1': 'Custom Expense 1',
      'expense2': 'Custom Expense 2',
      'expense3': 'Custom Expense 3',
      'expense4': 'Custom Expense 4',
      'vendor1': 'Custom Vendor 1',
      'vendor2': 'Custom Vendor 2',
      'vendor3': 'Custom Vendor 3',
      'vendor4': 'Custom Vendor 4',
      'invoice1': 'Custom Invoice 1',
      'invoice2': 'Custom Invoice 2',
      'invoice3': 'Custom Invoice 3',
      'invoice4': 'Custom Invoice 4',
      'payment1': 'Custom Payment 1',
      'payment2': 'Custom Payment 2',
      'payment3': 'Custom Payment 3',
      'payment4': 'Custom Payment 4',
      'surcharge1': 'Custom Surcharge 1',
      'surcharge2': 'Custom Surcharge 2',
      'surcharge3': 'Custom Surcharge 3',
      'surcharge4': 'Custom Surcharge 4',
      'group1': 'Custom Group 1',
      'group2': 'Custom Group 2',
      'group3': 'Custom Group 3',
      'group4': 'Custom Group 4',
      'reset': 'Sıfırla',
      'number': 'Number',
      'export': 'Dışa Aktar',
      'chart': 'Grafik',
      'count': 'Count',
      'totals': 'Toplamlar',
      'blank': 'Blank',
      'day': 'Day',
      'month': 'Month',
      'year': 'Year',
      'subgroup': 'Subgroup',
      'is_active': 'Is Active',
      'group_by': 'Gruplandır',
      'credit_balance': 'Kredi Bakiyesi',
      'contact_last_login': 'Contact Last Login',
      'contact_full_name': 'Contact Full Name',
      'contact_phone': 'Contact Phone',
      'contact_custom_value1': 'Contact Custom Value 1',
      'contact_custom_value2': 'Contact Custom Value 2',
      'contact_custom_value3': 'Contact Custom Value 3',
      'contact_custom_value4': 'Contact Custom Value 4',
      'shipping_address1': 'Shipping Street',
      'shipping_address2': 'Shipping Apt/Suite',
      'shipping_city': 'Shipping City',
      'shipping_state': 'Shipping State/Province',
      'shipping_postal_code': 'Shipping Postal Code',
      'shipping_country': 'Shipping Country',
      'billing_address1': 'Billing Street',
      'billing_address2': 'Billing Apt/Suite',
      'billing_city': 'Billing City',
      'billing_state': 'Billing State/Province',
      'billing_postal_code': 'Billing Postal Code',
      'billing_country': 'Billing Country',
      'client_id': 'Client Id',
      'assigned_to': 'Assigned to',
      'created_by': 'Created by :name',
      'assigned_to_id': 'Assigned To Id',
      'created_by_id': 'Created By Id',
      'add_column': 'Add Column',
      'edit_columns': 'Edit Columns',
      'columns': 'Columns',
      'aging': 'Aging',
      'profit_and_loss': 'Profit and Loss',
      'reports': 'Reports',
      'report': 'Rapor',
      'add_company': 'Firma Ekle',
      'unpaid_invoice': 'Unpaid Invoice',
      'paid_invoice': 'Paid Invoice',
      'unapproved_quote': 'Unapproved Quote',
      'help': 'Yardım',
      'refund': 'Refund',
      'refund_date': 'Refund Date',
      'filtered_by': 'Filtered by',
      'contact_email': 'Contact Email',
      'multiselect': 'Multiselect',
      'entity_state': 'Durum',
      'verify_password': 'Verify Password',
      'applied': 'Applied',
      'include_recent_errors': 'Include recent errors from the logs',
      'your_message_has_been_received':
          'We have received your message and will try to respond promptly.',
      'message': 'Mesaj',
      'from': 'Kimden',
      'show_product_details': 'Show Product Details',
      'show_product_details_help':
          'Include the description and cost in the product dropdown',
      'pdf_min_requirements': 'The PDF renderer requires :version',
      'adjust_fee_percent': 'Adjust Fee Percent',
      'adjust_fee_percent_help': 'Adjust percent to account for fee',
      'configure_settings': 'Configure Settings',
      'support_forum': 'destek forum',
      'about': 'About',
      'documentation': 'Belgeler',
      'contact_us': 'Contact Us',
      'subtotal': 'Aratoplam',
      'line_total': 'Tutar',
      'item': 'Öğe',
      'credit_email': 'Credit Email',
      'iframe_url': 'Web adresi',
      'domain_url': 'Domain URL',
      'password_is_too_short': 'Password is too short',
      'password_is_too_easy':
          'Password must contain an upper case character and a number',
      'client_portal_tasks': 'Client Portal Tasks',
      'client_portal_dashboard': 'Client Portal Dashboard',
      'please_enter_a_value': 'Please enter a value',
      'deleted_logo': 'Successfully deleted logo',
      'yes': 'Evet',
      'no': 'Hayır',
      'generate_number': 'Generate Number',
      'when_saved': 'When Saved',
      'when_sent': 'When Sent',
      'select_company': 'Select Company',
      'float': 'Float',
      'collapse': 'Collapse',
      'show_or_hide': 'Show/hide',
      'menu_sidebar': 'Menu Sidebar',
      'history_sidebar': 'History Sidebar',
      'tablet': 'Tablet',
      'mobile': 'Mobile',
      'desktop': 'Desktop',
      'layout': 'Layout',
      'view': 'Görüntüle',
      'module': 'Module',
      'first_custom': 'First Custom',
      'second_custom': 'Second Custom',
      'third_custom': 'Third Custom',
      'show_cost': 'Show Cost',
      'show_product_cost': 'Show Product Cost',
      'show_cost_help':
          'Display a product cost field to track the markup/profit',
      'show_product_quantity': 'Show Product Quantity',
      'show_product_quantity_help':
          'Display a product quantity field, otherwise default to one',
      'show_invoice_quantity': 'Show Invoice Quantity',
      'show_invoice_quantity_help':
          'Display a line item quantity field, otherwise default to one',
      'show_product_discount': 'Show Product Discount',
      'show_product_discount_help': 'Display a line item discount field',
      'default_quantity': 'Default Quantity',
      'default_quantity_help':
          'Automatically set the line item quantity to one',
      'one_tax_rate': 'One Tax Rate',
      'two_tax_rates': 'Two Tax Rates',
      'three_tax_rates': 'Three Tax Rates',
      'default_tax_rate': 'Default Tax Rate',
      'user': 'Kullanıcı',
      'invoice_tax': 'Invoice Tax',
      'line_item_tax': 'Line Item Tax',
      'inclusive_taxes': 'Inclusive Taxes',
      'invoice_tax_rates': 'Invoice Tax Rates',
      'item_tax_rates': 'Item Tax Rates',
      'no_client_selected': 'Please select a client',
      'configure_rates': 'Configure rates',
      'configure_gateways': 'Configure Gateways',
      'tax_settings': 'Vergi Ayarları',
      'tax_settings_rates': 'Tax Rates',
      'accent_color': 'Accent Color',
      'switch': 'Switch',
      'comma_sparated_list': 'Comma separated list',
      'options': 'Options',
      'single_line_text': 'Single-line text',
      'multi_line_text': 'Multi-line text',
      'dropdown': 'Dropdown',
      'field_type': 'Field Type',
      'recover_password_email_sent': 'A password recovery email has been sent',
      'submit': 'Submit',
      'recover_password': 'Şifreni kurtar',
      'late_fees': 'Late Fees',
      'credit_number': 'Credit Number',
      'payment_number': 'Payment Number',
      'late_fee_amount': 'Late Fee Amount',
      'late_fee_percent': 'Late Fee Percent',
      'schedule': 'program',
      'before_due_date': 'Before the due date',
      'after_due_date': 'After the due date',
      'after_invoice_date': 'After the invoice date',
      'days': 'Days',
      'invoice_email': 'Fatura E-postası',
      'payment_email': 'Ödeme E-postası',
      'partial_payment': 'Partial Payment',
      'payment_partial': 'Partial Payment',
      'partial_payment_email': 'Partial Payment Email',
      'quote_email': 'Teklif E-postası',
      'endless_reminder': 'Endless Reminder',
      'filtered_by_user': 'Filtered by User',
      'administrator': 'Administrator',
      'administrator_help':
          'Allow user to manage users, change settings and modify all records',
      'user_management': 'Kullanıcı yönetimi',
      'users': 'Kullanıcılar',
      'new_user': 'Yeni Kullanıcı',
      'edit_user': 'Kullanıcı Düzenle',
      'created_user': 'Successfully created user',
      'updated_user': 'Kullanıcı başarıyla güncellendi',
      'archived_user': 'Kullanıcı başarıyla arşivlendi',
      'deleted_user': 'Kullanıcı başarıyla silindi',
      'removed_user': 'Successfully removed user',
      'restored_user': 'Kullanıcı başarıyla geri yüklendi',
      'archived_users': 'Successfully archived :value users',
      'deleted_users': 'Successfully deleted :value users',
      'removed_users': 'Successfully removed :value users',
      'restored_users': 'Successfully restored :value users',
      'general_settings': 'Genel Ayarlar',
      'invoice_options': 'Fatura Seçenekleri',
      'hide_paid_to_date': 'Ödeme Tarihini Gizle',
      'hide_paid_to_date_help':
          'Bir ödeme alındığında yalnızca faturalarınızdaki \"Ödenen Tarihi\" alanını görüntüleyin.',
      'invoice_embed_documents': 'Embed Documents',
      'invoice_embed_documents_help': 'Include attached images in the invoice.',
      'all_pages_header': 'Show Header on',
      'all_pages_footer': 'Show Footer on',
      'first_page': 'İlk sayfa',
      'all_pages': 'Tüm sayfalar',
      'last_page': 'Son sayfa',
      'primary_font': 'Primary Font',
      'secondary_font': 'Secondary Font',
      'primary_color': 'Birincil Renk',
      'secondary_color': 'İkincil Renk',
      'page_size': 'Sayfa Boyutu',
      'font_size': 'Font Boyutu',
      'quote_design': 'Quote Design',
      'invoice_fields': 'Fatura Alanları',
      'product_fields': 'Product Fields',
      'invoice_terms': 'Fatura Şartları',
      'invoice_footer': 'Fatura Altbilgisi',
      'quote_terms': 'Teklif Şartları',
      'quote_footer': 'Teklif Altbilgisi',
      'auto_email_invoice': 'Auto Email',
      'auto_email_invoice_help':
          'Automatically email recurring invoices when they are created.',
      'auto_archive_invoice': 'Auto Archive',
      'auto_archive_invoice_help':
          'Automatically archive invoices when they are paid.',
      'auto_archive_quote': 'Auto Archive',
      'auto_archive_quote_help':
          'Automatically archive quotes when they are converted.',
      'auto_convert_quote': 'Auto Convert',
      'auto_convert_quote_help':
          'Automatically convert a quote to an invoice when approved by a client.',
      'workflow_settings': 'Workflow Settings',
      'freq_daily': 'Günlük',
      'freq_weekly': 'Haftalık',
      'freq_two_weeks': '2 hafta',
      'freq_four_weeks': '4 hafta',
      'freq_monthly': 'Aylık',
      'freq_two_months': 'Two months',
      'freq_three_months': '3 Ay',
      'freq_four_months': '4 Ay',
      'freq_six_months': '6 Ay',
      'freq_annually': 'Yıllık',
      'freq_two_years': '2 Yıl',
      'freq_three_years': 'Three Years',
      'never': 'Never',
      'company': 'Şirket',
      'generated_numbers': 'Generated Numbers',
      'charge_taxes': 'Vergi masrafları',
      'next_reset': 'Next Reset',
      'reset_counter': 'Reset Counter',
      'recurring_prefix': 'Recurring Prefix',
      'number_padding': 'Number Padding',
      'general': 'General',
      'surcharge_field': 'Surcharge Field',
      'company_field': 'Company Field',
      'company_value': 'Company Value',
      'credit_field': 'Credit Field',
      'invoice_field': 'Invoice Field',
      'invoice_surcharge': 'Invoice Surcharge',
      'client_field': 'Müşteri Alanı',
      'product_field': 'Ürün Alanı',
      'payment_field': 'Payment Field',
      'contact_field': 'İletişim Alanı',
      'vendor_field': 'Vendor Field',
      'expense_field': 'Expense Field',
      'project_field': 'Project Field',
      'task_field': 'Task Field',
      'group_field': 'Group Field',
      'number_counter': 'Number Counter',
      'prefix': 'Seri',
      'number_pattern': 'Number Pattern',
      'messages': 'Messages',
      'custom_css': 'Özel CSS',
      'custom_javascript': 'Custom JavaScript',
      'signature_on_pdf': 'Show on PDF',
      'signature_on_pdf_help':
          'Show the client signature on the invoice/quote PDF.',
      'show_accept_invoice_terms': 'Invoice Terms Checkbox',
      'show_accept_invoice_terms_help':
          'Require client to confirm that they accept the invoice terms.',
      'show_accept_quote_terms': 'Quote Terms Checkbox',
      'show_accept_quote_terms_help':
          'Require client to confirm that they accept the quote terms.',
      'require_invoice_signature': 'Invoice Signature',
      'require_invoice_signature_help':
          'Require client to provide their signature.',
      'require_quote_signature': 'Quote Signature',
      'enable_portal_password': 'Password Protect Invoices',
      'enable_portal_password_help':
          'Allows you to set a password for each contact. If a password is set, the contact will be required to enter a password before viewing invoices.',
      'authorization': 'Authorization',
      'subdomain': 'Alt etki alanı',
      'domain': 'Domain',
      'portal_mode': 'Portal Mode',
      'email_signature': 'Saygılarımızla,',
      'enable_email_markup_help':
          'Müşterilerinizin e-postalarınıza schema.org işaretleme ekleyerek ödeme yapmalarını kolaylaştırın.',
      'plain': 'Düz',
      'light': 'Aydınlık',
      'dark': 'Koyu',
      'email_design': 'E-Posta Dizaynı',
      'attach_pdf': 'Attach PDF',
      'attach_documents': 'Attach Documents',
      'attach_ubl': 'Attach UBL',
      'email_style': 'Email Style',
      'enable_email_markup': 'İşaretlemeyi Etkinleştir',
      'reply_to_email': 'Reply-To Email',
      'reply_to_name': 'Reply-To Name',
      'bcc_email': 'BCC Email',
      'processed': 'Processed',
      'credit_card': 'Kredi Kartı',
      'bank_transfer': 'Banka Transferi (EFT/Havale)',
      'priority': 'Priority',
      'fee_amount': 'Fee Amount',
      'fee_percent': 'Fee Percent',
      'fee_cap': 'Fee Cap',
      'limits_and_fees': 'Limits/Fees',
      'enable_min': 'Enable min',
      'enable_max': 'Enable max',
      'min_limit': 'Min: :min',
      'max_limit': 'Max: :max',
      'min': 'Min',
      'max': 'Max',
      'accepted_card_logos': 'Accepted Card Logos',
      'credentials': 'Credentials',
      'update_address': 'Adresi Güncelle',
      'update_address_help':
          'Müşterinin adresini verilen ayrıntılarla güncelleyin',
      'rate': 'Tarife',
      'tax_rate': 'Vergi Oranı',
      'new_tax_rate': 'Yeni Vergi Oranı',
      'edit_tax_rate': 'Vergi oranı düzenle',
      'created_tax_rate': 'Vergi oranı başarıyla oluşturuldu',
      'updated_tax_rate': 'Vergi oranı başarıyla güncellendi',
      'archived_tax_rate': 'Vergi oranı başarıyla arşivlendi',
      'deleted_tax_rate': 'Successfully deleted tax rate',
      'restored_tax_rate': 'Successfully restored tax rate',
      'archived_tax_rates': 'Successfully archived :value tax rates',
      'deleted_tax_rates': 'Successfully deleted :value tax rates',
      'restored_tax_rates': 'Successfully restored :value tax rates',
      'fill_products': 'Otomatik doldurma ürünleri',
      'fill_products_help':
          'Bir ürün seçmek açıklama ve maliyeti otomatik olarak dolduracaktır',
      'update_products': 'Ürünleri otomatik güncelle',
      'update_products_help':
          'Faturayı güncellemek ürün kütüphanesini otomatik olarak dolduracaktır.',
      'convert_products': 'Convert Products',
      'convert_products_help':
          'Automatically convert product prices to the client\'s currency',
      'fees': 'Fees',
      'limits': 'Limits',
      'provider': 'Provider',
      'company_gateway': 'Payment Gateway',
      'company_gateways': 'Payment Gateways',
      'new_company_gateway': 'New Gateway',
      'edit_company_gateway': 'Edit Gateway',
      'created_company_gateway': 'Successfully created gateway',
      'updated_company_gateway': 'Successfully updated gateway',
      'archived_company_gateway': 'Successfully archived gateway',
      'deleted_company_gateway': 'Successfully deleted gateway',
      'restored_company_gateway': 'Successfully restored gateway',
      'archived_company_gateways': 'Successfully archived :value gateways',
      'deleted_company_gateways': 'Successfully deleted :value gateways',
      'restored_company_gateways': 'Successfully restored :value gateways',
      'continue_editing': 'Continue Editing',
      'discard_changes': 'Discard Changes',
      'default_value': 'Default value',
      'disabled': 'Devre Dışı',
      'currency_format': 'Currency Format',
      'first_day_of_the_week': 'First Day of the Week',
      'first_month_of_the_year': 'First Month of the Year',
      'sunday': 'Pazar',
      'monday': 'Pazartesi',
      'tuesday': 'Salı',
      'wednesday': 'Çarşamba',
      'thursday': 'Perşembe',
      'friday': 'Cuma',
      'saturday': 'Cumartesi',
      'january': 'Ocak',
      'february': 'Şubat',
      'march': 'Mart',
      'april': 'Nisan',
      'may': 'Mayıs',
      'june': 'Haziran',
      'july': 'Temmuz',
      'august': 'Ağustos',
      'september': 'Eylül',
      'october': 'Ekim',
      'november': 'Kasım',
      'december': 'Aralık',
      'symbol': 'Symbol',
      'ocde': 'Code',
      'date_format': 'Date Format',
      'datetime_format': 'Datetime Format',
      'military_time': '24 Saat Zaman Biçimi',
      'military_time_help': '24 Hour Display',
      'send_reminders': 'Send Reminders',
      'timezone': 'Timezone',
      'filtered_by_project': 'Filtered by Project',
      'filtered_by_group': 'Filtered by Group',
      'filtered_by_invoice': 'Filtered by Invoice',
      'filtered_by_client': 'Filtered by Client',
      'filtered_by_vendor': 'Filtered by Vendor',
      'group_settings': 'Group Settings',
      'group': 'Group',
      'groups': 'Groups',
      'new_group': 'New Group',
      'edit_group': 'Edit Group',
      'created_group': 'Successfully created group',
      'updated_group': 'Successfully updated group',
      'archived_groups': 'Successfully archived :value groups',
      'deleted_groups': 'Successfully deleted :value groups',
      'restored_groups': 'Successfully restored :value groups',
      'upload_logo': 'Upload Logo',
      'uploaded_logo': 'Successfully uploaded logo',
      'logo': 'Logo',
      'saved_settings': 'Successfully saved settings',
      'product_settings': 'Ürün Ayarları',
      'device_settings': 'Device Settings',
      'defaults': 'Varsayılanlar',
      'basic_settings': 'Temel Ayarlar',
      'advanced_settings': 'Gelişmiş Ayarlar',
      'company_details': 'Şirket Detayları',
      'user_details': 'Kullanıcı Detayları',
      'localization': 'Yerelleştirme',
      'online_payments': 'Çevrimiçi Ödemeler',
      'tax_rates': 'Vergi Oranları',
      'notifications': 'Bildirimler',
      'import_export': 'İçe Aktarım | Dışa Aktarım',
      'custom_fields': 'Özel Alanlar',
      'invoice_design': 'Fatura Dizaynı',
      'buy_now_buttons': 'Buy Now Buttons',
      'email_settings': 'E-posta ayarları',
      'templates_and_reminders': 'Şablonlar & Hatırlatmalar',
      'credit_cards_and_banks': 'Credit Cards & Banks',
      'data_visualizations': 'Veri Görselleştirmeleri',
      'price': 'Price',
      'email_sign_up': 'Email Sign Up',
      'google_sign_up': 'Google Sign Up',
      'thank_you_for_your_purchase': 'Thank you for your purchase!',
      'redeem': 'Redeem',
      'back': 'Back',
      'past_purchases': 'Past Purchases',
      'annual_subscription': 'Annual Subscription',
      'pro_plan': 'Pro Plan',
      'enterprise_plan': 'Enterprise Plan',
      'count_users': ':count users',
      'upgrade': 'Upgrade',
      'please_enter_a_first_name': 'Please enter a first name',
      'please_enter_a_last_name': 'Please enter a last name',
      'please_agree_to_terms_and_privacy':
          'Please agree to the terms of service and privacy policy to create an account.',
      'i_agree_to_the': 'I agree to the',
      'terms_of_service_link': 'terms of service',
      'privacy_policy_link': 'privacy policy',
      'terms_of_service': 'Hizmet Şartları',
      'privacy_policy': 'Privacy Policy',
      'sign_up': 'Kayıt Ol',
      'account_login': 'Hesap girişi',
      'view_website': 'View Website',
      'create_account': 'Create Account',
      'email_login': 'Email Login',
      'create_new': 'Create New',
      'no_record_selected': 'No record selected',
      'error_unsaved_changes': 'Please save or cancel your changes',
      'download': 'İndir',
      'requires_an_enterprise_plan': 'Requires an enterprise plan',
      'take_picture': 'Take Picture',
      'upload_file': 'Upload File',
      'document': 'Document',
      'documents': 'Dokümanlar',
      'new_document': 'New Document',
      'edit_document': 'Edit Document',
      'uploaded_document': 'Successfully uploaded document',
      'updated_document': 'Successfully updated document',
      'archived_document': 'Successfully archived document',
      'deleted_document': 'Successfully deleted document',
      'restored_document': 'Successfully restored document',
      'archived_documents': 'Successfully archived :value documents',
      'deleted_documents': 'Successfully deleted :value documents',
      'restored_documents': 'Successfully restored :value documents',
      'no_history': 'No History',
      'expense_date': 'Gider Tarihi',
      'pending': 'Beklemede',
      'expense_status_1': 'Logged',
      'expense_status_2': 'Pending',
      'expense_status_3': 'Invoiced',
      'converted': 'Dönüştürüldü',
      'add_documents_to_invoice': 'Add documents to invoice',
      'exchange_rate': 'Döviz Kuru',
      'convert_currency': 'Convert currency',
      'mark_paid': 'Mark Paid',
      'category': 'Kategori',
      'address': 'Adres',
      'new_vendor': 'Yeni Tedarikçi',
      'created_vendor': 'Satıcı başarıyla oluşturuldu',
      'updated_vendor': 'Satıcı başarıyla güncellendi',
      'archived_vendor': 'Satıcı başarıyla arşivlendi',
      'deleted_vendor': 'Satıcı başarıyla silindi',
      'restored_vendor': 'Successfully restored vendor',
      'archived_vendors': ':count satıcı başarıyla arşivlendi',
      'deleted_vendors': ':count satıcı başarıyla silindi',
      'restored_vendors': 'Successfully restored :value vendors',
      'new_expense': 'Gider Girişi',
      'created_expense': 'Gider oluşturuldu',
      'updated_expense': 'Gider güncellendi',
      'archived_expense': 'Gider başarıyla arşivlendi',
      'deleted_expense': 'Gider başarıyla silindi',
      'restored_expense': 'Successfully restored expense',
      'archived_expenses': 'Giderler başarıyla arşivlendi',
      'deleted_expenses': 'Giderler başarıyla silindi',
      'restored_expenses': 'Successfully restored :value expenses',
      'copy_shipping': 'Copy Shipping',
      'copy_billing': 'Copy Billing',
      'design': 'Design',
      'failed_to_find_record': 'Failed to find record',
      'invoiced': 'Faturalandı',
      'logged': 'Loglandı',
      'running': 'Çalışıyor',
      'resume': 'Devam Et',
      'task_errors': 'Lütfen örtüşen süreleri düzeltin',
      'start': 'Başlama',
      'stop': 'Bitiş',
      'started_task': 'Successfully started task',
      'stopped_task': 'Görev başarıyla durduruldu',
      'resumed_task': 'Successfully resumed task',
      'now': 'Şimdi',
      'auto_start_tasks': 'Auto Start Tasks',
      'timer': 'Zamanlayıcı',
      'manual': 'Manuel',
      'budgeted': 'Budgeted',
      'start_time': 'Başlangıç Zamanı',
      'end_time': 'Bitiş Zamanı',
      'date': 'Tarih',
      'times': 'Zamanlar',
      'duration': 'Süre',
      'new_task': 'Yeni Görev',
      'created_task': 'Görev başarıyla oluşturuldu',
      'updated_task': 'Görev başarıyla güncellendi',
      'archived_task': 'Görev başarıyla arşivlendi',
      'deleted_task': 'Görev başarıyla silindi',
      'restored_task': 'Görev başarıyla geri yüklendi',
      'archived_tasks': 'Arşivlenen görev sayısı :count',
      'deleted_tasks': 'Silinen görev sayısı :count',
      'restored_tasks': 'Successfully restored :value tasks',
      'please_enter_a_name': 'Please enter a name',
      'budgeted_hours': 'Budgeted Hours',
      'created_project': 'Successfully created project',
      'updated_project': 'Successfully updated project',
      'archived_project': 'Successfully archived project',
      'deleted_project': 'Successfully deleted project',
      'restored_project': 'Successfully restored project',
      'archived_projects': 'Successfully archived :count projects',
      'deleted_projects': 'Successfully deleted :count projects',
      'restored_projects': 'Successfully restored :value projects',
      'new_project': 'New Project',
      'thank_you_for_using_our_app': 'Thank you for using our app!',
      'if_you_like_it': 'If you like it please',
      'click_here': 'buraya tıklayın',
      'click_here_capital': 'Click here',
      'to_rate_it': 'to rate it.',
      'average': 'Average',
      'unapproved': 'Unapproved',
      'authenticate_to_change_setting':
          'Please authenticate to change this setting',
      'locked': 'Locked',
      'authenticate': 'Authenticate',
      'please_authenticate': 'Please authenticate',
      'biometric_authentication': 'Biometric Authentication',
      'footer': 'Altbilgi',
      'compare': 'Compare',
      'hosted_login': 'Hosted Login',
      'selfhost_login': 'Selfhost Login',
      'google_sign_in': 'Login with Google',
      'today': 'Today',
      'custom_range': 'Custom Range',
      'date_range': 'Date Range',
      'current': 'Current',
      'previous': 'Previous',
      'current_period': 'Current Period',
      'comparison_period': 'Comparison Period',
      'previous_period': 'Previous Period',
      'previous_year': 'Previous Year',
      'compare_to': 'Compare to',
      'last7_days': 'Last 7 Days',
      'last_week': 'Last Week',
      'last30_days': 'Last 30 Days',
      'this_month': 'This Month',
      'last_month': 'Last Month',
      'this_year': 'This Year',
      'last_year': 'Last Year',
      'custom': 'Özel',
      'clone_to_invoice': 'Clone to Invoice',
      'clone_to_quote': 'Clone to Quote',
      'clone_to_credit': 'Clone to Credit',
      'view_invoice': 'Fatura Görüntüle',
      'convert': 'Convert',
      'more': 'More',
      'edit_client': 'Müşteri Düzenle',
      'edit_product': 'Ürün Düzenle',
      'edit_invoice': 'Fatura Düzenle',
      'edit_quote': 'Teklif Düzenle',
      'edit_payment': 'Ödeme düzenle',
      'edit_task': 'Görev Düzenle',
      'edit_expense': 'Gideri Düzenle',
      'edit_vendor': 'Tedarikçiyi Düzenle',
      'edit_project': 'Edit Project',
      'edit_recurring_expense': 'Edit Recurring Expense',
      'edit_recurring_quote': 'Edit Recurring Quote',
      'billing_address': 'Fatura Adresi',
      'shipping_address': 'Shipping Address',
      'total_revenue': 'Toplam Gelir',
      'average_invoice': 'Ortalama Fatura',
      'outstanding': 'Ödenmemiş',
      'invoices_sent': ':count fatura gönderildi',
      'active_clients': 'aktif müşteriler',
      'close': 'Kapat',
      'email': 'E-Posta',
      'password': 'Şifre',
      'url': 'URL',
      'secret': 'Secret',
      'name': 'Ünvan',
      'logout': 'Oturumu kapat',
      'login': 'Oturum aç',
      'filter': 'Filtrele',
      'sort': 'Sort',
      'search': 'Arama',
      'active': 'Aktif',
      'archived': 'Arşivlendi',
      'deleted': 'Silindi',
      'dashboard': 'Gösterge Paneli',
      'archive': 'Arşivle',
      'delete': 'Sil',
      'restore': 'Geri yükle',
      'refresh_complete': 'Refresh Complete',
      'please_enter_your_email': 'Please enter your email',
      'please_enter_your_password': 'Please enter your password',
      'please_enter_your_url': 'Please enter your URL',
      'please_enter_a_product_key': 'Please enter a product key',
      'ascending': 'Ascending',
      'descending': 'Descending',
      'save': 'Kaydet',
      'an_error_occurred': 'An error occurred',
      'paid_to_date': 'Ödenen',
      'balance_due': 'Genel Toplam',
      'balance': 'Bakiye',
      'overview': 'Overview',
      'details': 'Detaylar',
      'phone': 'Telefon',
      'website': 'Web adresi',
      'vat_number': 'Vergi Numarası',
      'id_number': 'ID Numarası',
      'create': 'Oluştur',
      'copied_to_clipboard': 'Copied :value to the clipboard',
      'error': 'Hata',
      'could_not_launch': 'Could not launch',
      'contacts': 'Yetkili',
      'additional': 'Additional',
      'first_name': 'Adı',
      'last_name': 'Soyadı',
      'add_contact': 'Yetkili Ekle',
      'are_you_sure': 'Emin misiniz?',
      'cancel': 'İptal',
      'ok': 'Tamam',
      'remove': 'Sil',
      'email_is_invalid': 'E-posta geçersiz',
      'product': 'Ürün',
      'products': 'Ürünler',
      'new_product': 'Yeni Ürün',
      'created_product': 'Ürün başarıyla oluşturuldu',
      'updated_product': 'Ürün başarıyla güncellendi',
      'archived_product': 'Ürün başarıyla arşivlendi',
      'deleted_product': 'Successfully deleted product',
      'restored_product': 'Successfully restored product',
      'archived_products': 'Successfully archived :count products',
      'deleted_products': 'Successfully deleted :count products',
      'restored_products': 'Successfully restored :value products',
      'product_key': 'Ürün',
      'notes': 'Notlar',
      'cost': 'Cost',
      'client': 'Müşteri',
      'clients': 'Müşteriler',
      'new_client': 'Yeni Müşteri',
      'created_client': 'Müşteri başarıyla oluşturuldu',
      'updated_client': 'Müşteri başarıyla güncellendi',
      'archived_client': 'Müşteri başarıyla arşivlendi',
      'archived_clients': ':count müşteri başarıyla arşivlendi',
      'deleted_client': 'Müşteri başarıyla silindi',
      'deleted_clients': ':count müşteri başarıyla silindi',
      'restored_client': 'Müşteri Başarıyla Geri Yüklendi',
      'restored_clients': 'Successfully restored :value clients',
      'address1': 'Adres',
      'address2': 'Adres',
      'city': 'Şehir',
      'state': 'İlçe',
      'postal_code': 'Posta Kodu',
      'country': 'Ülke',
      'invoice': 'Fatura',
      'invoices': 'Faturalar',
      'new_invoice': 'Yeni Fatura',
      'created_invoice': 'Fatura başarıyla oluşturuldu',
      'updated_invoice': 'Fatura başarıyla güncellendi',
      'archived_invoice': 'Fatura başarıyla arşivlendi',
      'deleted_invoice': 'Fatura başarıyla silindi',
      'restored_invoice': 'Fatura Başarıyla Geri Yüklendi',
      'archived_invoices': ':count fatura başarıyla arşivlendi',
      'deleted_invoices': ':count fatura başarıyla silindi',
      'restored_invoices': 'Successfully restored :value invoices',
      'emailed_invoice': 'Fatura başarıyla e-posta ile gönderildi',
      'emailed_payment': 'Successfully emailed payment',
      'amount': 'Tutar',
      'invoice_number': 'Fatura Numarası',
      'invoice_date': 'Fatura Tarihi',
      'discount': 'İskonto',
      'po_number': 'Sipariş No',
      'terms': 'Koşullar',
      'public_notes': 'Açık Notlar',
      'private_notes': 'Özel Notlar',
      'frequency': 'Sıklık',
      'start_date': 'Başlangıç Tarihi',
      'end_date': 'Bitiş Tarihi',
      'quote_number': 'Teklif Numarası',
      'quote_date': 'Teklif Tarihi',
      'valid_until': 'Geçerlilik Tarihi',
      'items': 'Ögeler',
      'partial_deposit': 'Partial/Deposit',
      'description': 'Açıklama',
      'unit_cost': 'Birim Fiyatı',
      'quantity': 'Miktar',
      'add_item': 'Öge Ekle',
      'contact': 'Kişi',
      'work_phone': 'Telefon',
      'total_amount': 'Total Amount',
      'pdf': 'PDF',
      'due_date': 'Ödeme Tarihi',
      'partial_due_date': 'Partial Due Date',
      'status': 'Durum',
      'invoice_status_id': 'Invoice Status',
      'quote_status': 'Quote Status',
      'click_plus_to_add_item': 'Click + to add an item',
      'click_plus_to_add_time': 'Click + to add time',
      'count_selected': ':count selected',
      'total': 'Toplam',
      'percent': 'Percent',
      'edit': 'Düzenle',
      'dismiss': 'Dismiss',
      'please_select_a_date': 'Please select a date',
      'please_select_a_client': 'Please select a client',
      'please_select_an_invoice': 'Please select an invoice',
      'task_rate': 'Task Rate',
      'settings': 'Ayarlar',
      'language': 'Dil',
      'currency': 'Currency',
      'created_at': 'Date Created',
      'created_on': 'Created On',
      'updated_at': 'Updated',
      'tax': 'Vergi',
      'please_enter_an_invoice_number': 'Please enter an invoice number',
      'please_enter_a_quote_number': 'Please enter a quote number',
      'past_due': 'Past Due',
      'draft': 'Draft',
      'sent': 'Gönder',
      'viewed': 'Viewed',
      'approved': 'Approved',
      'partial': 'Kısmi / Mevduat',
      'paid': 'Ödenen',
      'mark_sent': 'Gönderilmiş Olarak İşaretle',
      'marked_invoice_as_sent': 'Successfully marked invoice as sent',
      'marked_invoice_as_paid': 'Successfully marked invoice as paid',
      'marked_invoices_as_sent': 'Successfully marked invoices as sent',
      'marked_invoices_as_paid': 'Successfully marked invoices as paid',
      'done': 'Tamam',
      'please_enter_a_client_or_contact_name':
          'Please enter a client or contact name',
      'dark_mode': 'Karanlık Mod',
      'restart_app_to_apply_change': 'Restart the app to apply the change',
      'refresh_data': 'Refresh Data',
      'blank_contact': 'Blank Contact',
      'activity': 'Aktivite',
      'no_records_found': 'No records found',
      'clone': 'Çoğalt',
      'loading': 'Loading',
      'industry': 'Industry',
      'size': 'Size',
      'payment_terms': 'Ödeme koşulları',
      'payment_date': 'Ödeme Tarihi',
      'payment_status': 'Payment Status',
      'payment_status_1': 'Pending',
      'payment_status_2': 'Voided',
      'payment_status_3': 'Failed',
      'payment_status_4': 'Completed',
      'payment_status_5': 'Partially Refunded',
      'payment_status_6': 'Refunded',
      'payment_status_-1': 'Unapplied',
      'payment_status_-2': 'Partially Unapplied',
      'net': 'Net',
      'client_portal': 'Müşteri Portalı',
      'show_tasks': 'Show tasks',
      'email_reminders': 'Email Reminders',
      'enabled': 'Enabled',
      'recipients': 'Recipients',
      'initial_email': 'Initial Email',
      'first_reminder': 'İlk Hatırlatıcı',
      'second_reminder': 'İkinci Hatırlatıcı',
      'third_reminder': 'Üçüncü Hatırlatıcı',
      'reminder1': 'First Reminder',
      'reminder2': 'Second Reminder',
      'reminder3': 'Third Reminder',
      'template': 'Template',
      'send': 'Send',
      'subject': 'Konu',
      'body': 'Gövde',
      'send_email': 'E-Mail Gönder',
      'email_receipt': 'Ödeme makbuzunu müşteriye e-postayla gönder',
      'auto_billing': 'Auto billing',
      'button': 'Buton',
      'preview': 'Preview',
      'customize': 'Özelleştir',
      'history': 'Geçmiş',
      'payment': 'Ödeme',
      'payments': 'Ödemeler',
      'refunded': 'Refunded',
      'payment_type': 'Payment Type',
      'transaction_reference': 'İşlem Referansı',
      'enter_payment': 'Ödeme Gir',
      'new_payment': 'Ödeme Gir',
      'created_payment': 'Ödeme başarıyla oluşturuldu',
      'updated_payment': 'Ödeme başarıyla güncellendi',
      'archived_payment': 'Ödeme başarıyla arşivlendi',
      'deleted_payment': 'Ödeme başarıyla silindi',
      'restored_payment': 'Ödeme Başarıyla Geri Yüklendi',
      'archived_payments': ':count ödeme arşivlendi',
      'deleted_payments': ':count ödeme silindi',
      'restored_payments': 'Successfully restored :value payments',
      'quote': 'Teklif',
      'quotes': 'Teklifler',
      'new_quote': 'Yeni Teklif',
      'created_quote': 'Teklif başarıyla oluşturuldu',
      'updated_quote': 'Teklif başarıyla güncellendi',
      'archived_quote': 'Teklif başarıyla arşivlendi',
      'deleted_quote': 'Teklif başarıyla silindi',
      'restored_quote': 'Teklif Başarıyla Geri Yüklendi',
      'archived_quotes': ':count teklif başarıyla arşivlendi',
      'deleted_quotes': ':count teklif başarıyla silindi',
      'restored_quotes': 'Successfully restored :value quotes',
      'expense': 'Gider',
      'expenses': 'Giderler',
      'vendor': 'Tedarikçi',
      'vendors': 'Tedarikçiler',
      'task': 'Task',
      'tasks': 'Görevler',
      'project': 'Project',
      'projects': 'Projects',
      'activity_1': ':user :client müşteri hesabını oluşturdu',
      'activity_2': ':user :client müşteri hesabını arşivledi',
      'activity_3': ':user :client müştei hesabını sildi',
      'activity_4': ':user :invoice nolu faturayı oluşturdu',
      'activity_5': ':user :invoice nolu faturayı güncelledi',
      'activity_6': ':user emailed invoice :invoice for :client to :contact',
      'activity_7': ':contact viewed invoice :invoice for :client',
      'activity_8': ':user :invoice nolu faturayı arşivledi',
      'activity_9': ':user :invoice nolu faturayı sildi',
      'activity_10':
          ':contact entered payment :payment for :payment_amount on invoice :invoice for :client',
      'activity_11': ':user :payment tutarlı ödemeyi güncelledi',
      'activity_12': ':user :payment tutarlı ödemeyi arşivledi',
      'activity_13': ':user :payment tutarlı ödemeyi sildi',
      'activity_14': ':user :credit kredi girdi',
      'activity_15': ':user :credit kredi güncelledi',
      'activity_16': ':user :credit kredi arşivledi',
      'activity_17': ':user :credit kredi sildi',
      'activity_18': ':user :quote nolu teklifi oluşturdu',
      'activity_19': ':user :quote nolu teklifi güncelledi',
      'activity_20': ':user emailed quote :quote for :client to :contact',
      'activity_21': ':contact adlı yetkili :quote nolu teklifi görüntüledi',
      'activity_22': ':user :quote nolu teklifi arşivledi',
      'activity_23': ':user :quote nolu teklifi sildi',
      'activity_24': ':user :quote nolu teklifi geri yükledi',
      'activity_25': ':user :invoice nolu faturayı geri yükledi',
      'activity_26': ':user :client müşterisini geri yükledi',
      'activity_27': ':user :payment tutarında ödemeyi geri yükledi',
      'activity_28': ':user :credit kredisini geri yükledi',
      'activity_29': ':contact approved quote :quote for :client',
      'activity_30': ':user :vendor satıcısını oluşturdu',
      'activity_31': ':user :vendor satıcısını arşivledi',
      'activity_32': ':user :vendor satıcısını sildi',
      'activity_33': ':user :vendor satıcısını geri yükledi',
      'activity_34': ':user masraf oluşturdu :expense',
      'activity_35': ':user masraf arşivledi :expense',
      'activity_36': ':user masraf sildi :expense',
      'activity_37': ':user masraf geri yükledi :expense',
      'activity_39': ':user cancelled a :payment_amount payment :payment',
      'activity_40':
          ':user refunded :adjustment of a :payment_amount payment :payment',
      'activity_41': ':payment_amount payment (:payment) failed',
      'activity_42': ':user :task görevini oluşturdu',
      'activity_43': ':user :task görevini güncelledi',
      'activity_44': ':user :task görevini arşivledi',
      'activity_45': ':user :task görevini sildi',
      'activity_46': ':user :task görevini geri yükledi',
      'activity_47': ':user masraf güncelledi :expense',
      'activity_48': ':user updated ticket :ticket',
      'activity_49': ':user closed ticket :ticket',
      'activity_50': ':user merged ticket :ticket',
      'activity_51': ':user split ticket :ticket',
      'activity_52': ':contact opened ticket :ticket',
      'activity_53': ':contact reopened ticket :ticket',
      'activity_54': ':user reopened ticket :ticket',
      'activity_55': ':contact replied ticket :ticket',
      'activity_56': ':user viewed ticket :ticket',
      'activity_57': 'System failed to email invoice :invoice',
      'activity_58': ':user reversed invoice :invoice',
      'activity_59': ':user cancelled invoice :invoice',
      'activity_60': ':contact viewed quote :quote',
      'activity_61': ':user updated client :client',
      'activity_62': ':user updated vendor :vendor',
      'activity_63':
          ':user emailed first reminder for invoice :invoice to :contact',
      'activity_64':
          ':user emailed second reminder for invoice :invoice to :contact',
      'activity_65':
          ':user emailed third reminder for invoice :invoice to :contact',
      'activity_66':
          ':user emailed endless reminder for invoice :invoice to :contact',
      'activity_80': ':user created subscription :subscription',
      'activity_81': ':user updated subscription :subscription',
      'activity_82': ':user archived subscription :subscription',
      'activity_83': ':user deleted subscription :subscription',
      'activity_84': ':user restored subscription :subscription',
      'one_time_password': 'One Time Password',
      'emailed_quote': 'Teklif başarıyla e-posta ile gönderildi',
      'emailed_credit': 'Successfully emailed credit',
      'marked_quote_as_sent': 'Successfully marked quote as sent',
      'marked_credit_as_sent': 'Successfully marked credit as sent',
      'expired': 'Expired',
      'all': 'All',
      'select': 'Seç',
      'long_press_multiselect': 'Long-press Multiselect',
      'custom_value1': 'Custom Value',
      'custom_value2': 'Custom Value',
      'custom_value3': 'Custom Value 3',
      'custom_value4': 'Custom Value 4',
      'email_style_custom': 'Custom Email Style',
      'custom_message_dashboard': 'Custom Dashboard Message',
      'custom_message_unpaid_invoice': 'Custom Unpaid Invoice Message',
      'custom_message_paid_invoice': 'Custom Paid Invoice Message',
      'custom_message_unapproved_quote': 'Custom Unapproved Quote Message',
      'lock_invoices': 'Lock Invoices',
      'translations': 'Translations',
      'task_number_pattern': 'Task Number Pattern',
      'task_number_counter': 'Task Number Counter',
      'expense_number_pattern': 'Expense Number Pattern',
      'expense_number_counter': 'Expense Number Counter',
      'vendor_number_pattern': 'Vendor Number Pattern',
      'vendor_number_counter': 'Vendor Number Counter',
      'ticket_number_pattern': 'Ticket Number Pattern',
      'ticket_number_counter': 'Ticket Number Counter',
      'payment_number_pattern': 'Payment Number Pattern',
      'payment_number_counter': 'Payment Number Counter',
      'invoice_number_pattern': 'Invoice Number Pattern',
      'invoice_number_counter': 'Fatura No Sayacı',
      'quote_number_pattern': 'Quote Number Pattern',
      'quote_number_counter': 'Teklif No Sayacı',
      'client_number_pattern': 'Credit Number Pattern',
      'client_number_counter': 'Credit Number Counter',
      'credit_number_pattern': 'Credit Number Pattern',
      'credit_number_counter': 'Credit Number Counter',
      'reset_counter_date': 'Reset Counter Date',
      'counter_padding': 'Counter Padding',
      'shared_invoice_quote_counter': 'Shared Invoice Quote Counter',
      'default_tax_name_1': 'Default Tax Name 1',
      'default_tax_rate_1': 'Default Tax Rate 1',
      'default_tax_name_2': 'Default Tax Name 2',
      'default_tax_rate_2': 'Default Tax Rate 2',
      'default_tax_name_3': 'Default Tax Name 3',
      'default_tax_rate_3': 'Default Tax Rate 3',
      'email_subject_invoice': 'Email Invoice Subject',
      'email_subject_quote': 'Email Quote Subject',
      'email_subject_payment': 'Email Payment Subject',
      'email_subject_payment_partial': 'Email Partial Payment Subject',
      'show_table': 'Show Table',
      'show_list': 'Show List',
      'client_city': 'Client City',
      'client_state': 'Client State',
      'client_country': 'Client Country',
      'client_is_active': 'Client is Active',
      'client_balance': 'Client Balance',
      'client_address1': 'Client Street',
      'client_address2': 'Client Apt/Suite',
      'vendor_address1': 'Vendor Street',
      'vendor_address2': 'Vendor Apt/Suite',
      'client_shipping_address1': 'Client Shipping Street',
      'client_shipping_address2': 'Client Shipping Apt/Suite',
      'type': 'Tür',
      'invoice_amount': 'Fatura Tutarı',
      'invoice_due_date': 'Vade',
      'tax_rate1': 'Tax Rate 1',
      'tax_rate2': 'Tax Rate 2',
      'tax_rate3': 'Tax Rate 3',
      'auto_bill': 'Otomatik Fatura',
      'archived_at': 'Archived At',
      'has_expenses': 'Has Expenses',
      'custom_taxes1': 'Custom Taxes 1',
      'custom_taxes2': 'Custom Taxes 2',
      'custom_taxes3': 'Custom Taxes 3',
      'custom_taxes4': 'Custom Taxes 4',
      'custom_surcharge1': 'Custom Surcharge 1',
      'custom_surcharge2': 'Custom Surcharge 2',
      'custom_surcharge3': 'Custom Surcharge 3',
      'custom_surcharge4': 'Custom Surcharge 4',
      'is_deleted': 'Is Deleted',
      'vendor_city': 'Vendor City',
      'vendor_state': 'Vendor State',
      'vendor_country': 'Vendor Country',
      'is_approved': 'Is Approved',
      'tax_name': 'Vergi Adı',
      'tax_amount': 'Tax Amount',
      'tax_paid': 'Tax Paid',
      'payment_amount': 'Ödeme Tutarı',
      'age': 'Age',
      'is_running': 'Is Running',
      'time_log': 'Time Log',
      'bank_id': 'Banka',
      'expense_category_id': 'Expense Category ID',
      'expense_category': 'Expense Category',
      'invoice_currency_id': 'Invoice Currency ID',
      'tax_name1': 'Tax Name 1',
      'tax_name2': 'Tax Name 2',
      'tax_name3': 'Tax Name 3',
      'transaction_id': 'Transaction ID',
      'color_theme': 'Color Theme',
    },
  };

  String get createdProject =>
      _localizedValues[localeCode]['created_project'] ?? '';

  String get updatedProject =>
      _localizedValues[localeCode]['updated_project'] ?? '';

  String get archivedProject =>
      _localizedValues[localeCode]['archived_project'] ?? '';

  String get deletedProject =>
      _localizedValues[localeCode]['deleted_project'] ?? '';

  String get restoredProject =>
      _localizedValues[localeCode]['restored_project'] ?? '';

  String get archivedProjects =>
      _localizedValues[localeCode]['archived_projects'] ?? '';

  String get deletedProjects =>
      _localizedValues[localeCode]['deleted_projects'] ?? '';

  String get restoredProjects =>
      _localizedValues[localeCode]['restored_projects'] ?? '';

  String get newProject => _localizedValues[localeCode]['new_project'] ?? '';

  String get thankYouForUsingOurApp =>
      _localizedValues[localeCode]['thank_you_for_using_our_app'] ?? '';

  String get ifYouLikeIt =>
      _localizedValues[localeCode]['if_you_like_it'] ?? '';

  String get clickHere => _localizedValues[localeCode]['click_here'] ?? '';

  String get clickHereCapital =>
      _localizedValues[localeCode]['click_here_capital'] ?? '';

  String get toRateIt => _localizedValues[localeCode]['to_rate_it'] ?? '';

  String get average => _localizedValues[localeCode]['average'] ?? '';

  String get unapproved => _localizedValues[localeCode]['unapproved'] ?? '';

  String get authenticateToChangeSetting =>
      _localizedValues[localeCode]['authenticate_to_change_setting'] ?? '';

  String get locked => _localizedValues[localeCode]['locked'] ?? '';

  String get authenticate => _localizedValues[localeCode]['authenticate'] ?? '';

  String get pleaseAuthenticate =>
      _localizedValues[localeCode]['please_authenticate'] ?? '';

  String get biometricAuthentication =>
      _localizedValues[localeCode]['biometric_authentication'] ?? '';

  String get footer => _localizedValues[localeCode]['footer'] ?? '';

  String get compare => _localizedValues[localeCode]['compare'] ?? '';

  String get hostedLogin => _localizedValues[localeCode]['hosted_login'] ?? '';

  String get selfhostLogin =>
      _localizedValues[localeCode]['selfhost_login'] ?? '';

  String get googleSignIn =>
      _localizedValues[localeCode]['google_sign_in'] ?? '';

  String get today => _localizedValues[localeCode]['today'] ?? '';

  String get customRange => _localizedValues[localeCode]['custom_range'] ?? '';

  String get dateRange => _localizedValues[localeCode]['date_range'] ?? '';

  String get current => _localizedValues[localeCode]['current'] ?? '';

  String get previous => _localizedValues[localeCode]['previous'] ?? '';

  String get currentPeriod =>
      _localizedValues[localeCode]['current_period'] ?? '';

  String get comparisonPeriod =>
      _localizedValues[localeCode]['comparison_period'] ?? '';

  String get previousPeriod =>
      _localizedValues[localeCode]['previous_period'] ?? '';

  String get previousYear =>
      _localizedValues[localeCode]['previous_year'] ?? '';

  String get compareTo => _localizedValues[localeCode]['compare_to'] ?? '';

  String get last7Days => _localizedValues[localeCode]['last7_days'] ?? '';

  String get lastWeek => _localizedValues[localeCode]['last_week'] ?? '';

  String get last30Days => _localizedValues[localeCode]['last30_days'] ?? '';

  String get thisMonth => _localizedValues[localeCode]['this_month'] ?? '';

  String get lastMonth => _localizedValues[localeCode]['last_month'] ?? '';

  String get thisYear => _localizedValues[localeCode]['this_year'] ?? '';

  String get lastYear => _localizedValues[localeCode]['last_year'] ?? '';

  String get custom => _localizedValues[localeCode]['custom'] ?? '';

  String get cloneToInvoice =>
      _localizedValues[localeCode]['clone_to_invoice'] ?? '';

  String get cloneToQuote =>
      _localizedValues[localeCode]['clone_to_quote'] ?? '';

  String get viewInvoice => _localizedValues[localeCode]['view_invoice'] ?? '';

  String get convert => _localizedValues[localeCode]['convert'] ?? '';

  String get more => _localizedValues[localeCode]['more'] ?? '';

  String get editClient => _localizedValues[localeCode]['edit_client'] ?? '';

  String get editProduct => _localizedValues[localeCode]['edit_product'] ?? '';

  String get editInvoice => _localizedValues[localeCode]['edit_invoice'] ?? '';

  String get editQuote => _localizedValues[localeCode]['edit_quote'] ?? '';

  String get editPayment => _localizedValues[localeCode]['edit_payment'] ?? '';

  String get editTask => _localizedValues[localeCode]['edit_task'] ?? '';

  String get editExpense => _localizedValues[localeCode]['edit_expense'] ?? '';

  String get editVendor => _localizedValues[localeCode]['edit_vendor'] ?? '';

  String get editProject => _localizedValues[localeCode]['edit_project'] ?? '';

  String get editCredit => _localizedValues[localeCode]['edit_credit'] ?? '';

  String get editRecurringInvoice =>
      _localizedValues[localeCode]['edit_recurring_invoice'] ?? '';

  String get editRecurringExpense =>
      _localizedValues[localeCode]['edit_recurring_expense'] ?? '';

  String get editRecurringQuote =>
      _localizedValues[localeCode]['edit_recurring_quote'] ?? '';

  String get billingAddress =>
      _localizedValues[localeCode]['billing_address'] ?? '';

  String get shippingAddress =>
      _localizedValues[localeCode]['shipping_address'] ?? '';

  String get totalRevenue =>
      _localizedValues[localeCode]['total_revenue'] ?? '';

  String get averageInvoice =>
      _localizedValues[localeCode]['average_invoice'] ?? '';

  String get outstanding => _localizedValues[localeCode]['outstanding'] ?? '';

  String get invoicesSent =>
      _localizedValues[localeCode]['invoices_sent'] ?? '';

  String get activeClients =>
      _localizedValues[localeCode]['active_clients'] ?? '';

  String get close => _localizedValues[localeCode]['close'] ?? '';

  String get email => _localizedValues[localeCode]['email'] ?? '';

  String get password => _localizedValues[localeCode]['password'] ?? '';

  String get url => _localizedValues[localeCode]['url'] ?? '';

  String get secret => _localizedValues[localeCode]['secret'] ?? '';

  String get name => _localizedValues[localeCode]['name'] ?? '';

  String get logout => _localizedValues[localeCode]['logout'] ?? '';

  String get login => _localizedValues[localeCode]['login'] ?? '';

  String get filter => _localizedValues[localeCode]['filter'] ?? '';

  String get sort => _localizedValues[localeCode]['sort'] ?? '';

  String get search => _localizedValues[localeCode]['search'] ?? '';

  String get active => _localizedValues[localeCode]['active'] ?? '';

  String get archived => _localizedValues[localeCode]['archived'] ?? '';

  String get deleted => _localizedValues[localeCode]['deleted'] ?? '';

  String get dashboard => _localizedValues[localeCode]['dashboard'] ?? '';

  String get archive => _localizedValues[localeCode]['archive'] ?? '';

  String get delete => _localizedValues[localeCode]['delete'] ?? '';

  String get restore => _localizedValues[localeCode]['restore'] ?? '';

  String get refreshComplete =>
      _localizedValues[localeCode]['refresh_complete'] ?? '';

  String get pleaseEnterYourEmail =>
      _localizedValues[localeCode]['please_enter_your_email'] ?? '';

  String get pleaseEnterYourPassword =>
      _localizedValues[localeCode]['please_enter_your_password'] ?? '';

  String get pleaseEnterYourUrl =>
      _localizedValues[localeCode]['please_enter_your_url'] ?? '';

  String get pleaseEnterAProductKey =>
      _localizedValues[localeCode]['please_enter_a_product_key'] ?? '';

  String get pleaseEnterAFirstName =>
      _localizedValues[localeCode]['please_enter_a_first_name'] ?? '';

  String get pleaseEnterALastName =>
      _localizedValues[localeCode]['please_enter_a_last_name'] ?? '';

  String get ascending => _localizedValues[localeCode]['ascending'] ?? '';

  String get descending => _localizedValues[localeCode]['descending'] ?? '';

  String get save => _localizedValues[localeCode]['save'] ?? '';

  String get anErrorOccurred =>
      _localizedValues[localeCode]['an_error_occurred'] ?? '';

  String get paidToDate => _localizedValues[localeCode]['paid_to_date'] ?? '';

  String get balanceDue => _localizedValues[localeCode]['balance_due'] ?? '';

  String get balance => _localizedValues[localeCode]['balance'] ?? '';

  String get overview => _localizedValues[localeCode]['overview'] ?? '';

  String get details => _localizedValues[localeCode]['details'] ?? '';

  String get phone => _localizedValues[localeCode]['phone'] ?? '';

  String get website => _localizedValues[localeCode]['website'] ?? '';

  String get vatNumber => _localizedValues[localeCode]['vat_number'] ?? '';

  String get idNumber => _localizedValues[localeCode]['id_number'] ?? '';

  String get create => _localizedValues[localeCode]['create'] ?? '';

  String get copiedToClipboard =>
      _localizedValues[localeCode]['copied_to_clipboard'] ?? '';

  String get error => _localizedValues[localeCode]['error'] ?? '';

  String get couldNotLaunch =>
      _localizedValues[localeCode]['could_not_launch'] ?? '';

  String get contacts => _localizedValues[localeCode]['contacts'] ?? '';

  String get additional => _localizedValues[localeCode]['additional'] ?? '';

  String get firstName => _localizedValues[localeCode]['first_name'] ?? '';

  String get lastName => _localizedValues[localeCode]['last_name'] ?? '';

  String get addContact => _localizedValues[localeCode]['add_contact'] ?? '';

  String get areYouSure => _localizedValues[localeCode]['are_you_sure'] ?? '';

  String get cancel => _localizedValues[localeCode]['cancel'] ?? '';

  String get ok => _localizedValues[localeCode]['ok'] ?? '';

  String get remove => _localizedValues[localeCode]['remove'] ?? '';

  String get emailIsInvalid =>
      _localizedValues[localeCode]['email_is_invalid'] ?? '';

  String get product => _localizedValues[localeCode]['product'] ?? '';

  String get products => _localizedValues[localeCode]['products'] ?? '';

  String get newProduct => _localizedValues[localeCode]['new_product'] ?? '';

  String get createdProduct =>
      _localizedValues[localeCode]['created_product'] ?? '';

  String get updatedProduct =>
      _localizedValues[localeCode]['updated_product'] ?? '';

  String get archivedProduct =>
      _localizedValues[localeCode]['archived_product'] ?? '';

  String get deletedProduct =>
      _localizedValues[localeCode]['deleted_product'] ?? '';

  String get restoredProduct =>
      _localizedValues[localeCode]['restored_product'] ?? '';

  String get archivedProducts =>
      _localizedValues[localeCode]['archived_products'] ?? '';

  String get deletedProducts =>
      _localizedValues[localeCode]['deleted_products'] ?? '';

  String get restoredProducts =>
      _localizedValues[localeCode]['restored_products'] ?? '';

  String get newVendor => _localizedValues[localeCode]['new_vendor'] ?? '';

  String get createdVendor =>
      _localizedValues[localeCode]['created_vendor'] ?? '';

  String get updatedVendor =>
      _localizedValues[localeCode]['updated_vendor'] ?? '';

  String get archivedVendor =>
      _localizedValues[localeCode]['archived_vendor'] ?? '';

  String get deletedVendor =>
      _localizedValues[localeCode]['deleted_vendor'] ?? '';

  String get restoredVendor =>
      _localizedValues[localeCode]['restored_vendor'] ?? '';

  String get archivedVendors =>
      _localizedValues[localeCode]['archived_vendors'] ?? '';

  String get deletedVendors =>
      _localizedValues[localeCode]['deleted_vendors'] ?? '';

  String get restoredVendors =>
      _localizedValues[localeCode]['restored_vendors'] ?? '';

  String get document => _localizedValues[localeCode]['document'] ?? '';

  String get documents => _localizedValues[localeCode]['documents'] ?? '';

  String get newDocument => _localizedValues[localeCode]['new_document'] ?? '';

  String get editDocument =>
      _localizedValues[localeCode]['edit_document'] ?? '';

  String get uploadedDocument =>
      _localizedValues[localeCode]['uploaded_document'] ?? '';

  String get updatedDocument =>
      _localizedValues[localeCode]['updated_document'] ?? '';

  String get archivedDocument =>
      _localizedValues[localeCode]['archived_document'] ?? '';

  String get deletedDocument =>
      _localizedValues[localeCode]['deleted_document'] ?? '';

  String get restoredDocument =>
      _localizedValues[localeCode]['restored_document'] ?? '';

  String get archivedDocuments =>
      _localizedValues[localeCode]['archived_documents'] ?? '';

  String get deletedDocuments =>
      _localizedValues[localeCode]['deleted_documents'] ?? '';

  String get restoredDocuments =>
      _localizedValues[localeCode]['restored_documents'] ?? '';

  String get newExpense => _localizedValues[localeCode]['new_expense'] ?? '';

  String get createdExpense =>
      _localizedValues[localeCode]['created_expense'] ?? '';

  String get updatedExpense =>
      _localizedValues[localeCode]['updated_expense'] ?? '';

  String get archivedExpense =>
      _localizedValues[localeCode]['archived_expense'] ?? '';

  String get deletedExpense =>
      _localizedValues[localeCode]['deleted_expense'] ?? '';

  String get restoredExpense =>
      _localizedValues[localeCode]['restored_expense'] ?? '';

  String get archivedExpenses =>
      _localizedValues[localeCode]['archived_expenses'] ?? '';

  String get deletedExpenses =>
      _localizedValues[localeCode]['deleted_expenses'] ?? '';

  String get restoredExpenses =>
      _localizedValues[localeCode]['restored_expenses'] ?? '';

  String get productKey => _localizedValues[localeCode]['product_key'] ?? '';

  String get notes => _localizedValues[localeCode]['notes'] ?? '';

  String get cost => _localizedValues[localeCode]['cost'] ?? '';

  String get client => _localizedValues[localeCode]['client'] ?? '';

  String get clients => _localizedValues[localeCode]['clients'] ?? '';

  String get newClient => _localizedValues[localeCode]['new_client'] ?? '';

  String get createdClient =>
      _localizedValues[localeCode]['created_client'] ?? '';

  String get updatedClient =>
      _localizedValues[localeCode]['updated_client'] ?? '';

  String get archivedClient =>
      _localizedValues[localeCode]['archived_client'] ?? '';

  String get deletedClient =>
      _localizedValues[localeCode]['deleted_client'] ?? '';

  String get restoredClient =>
      _localizedValues[localeCode]['restored_client'] ?? '';

  String get archivedClients =>
      _localizedValues[localeCode]['archived_clients'] ?? '';

  String get deletedClients =>
      _localizedValues[localeCode]['deleted_clients'] ?? '';

  String get restoredClients =>
      _localizedValues[localeCode]['restored_clients'] ?? '';

  String get address1 => _localizedValues[localeCode]['address1'] ?? '';

  String get address2 => _localizedValues[localeCode]['address2'] ?? '';

  String get city => _localizedValues[localeCode]['city'] ?? '';

  String get state => _localizedValues[localeCode]['state'] ?? '';

  String get postalCode => _localizedValues[localeCode]['postal_code'] ?? '';

  String get country => _localizedValues[localeCode]['country'] ?? '';

  String get invoice => _localizedValues[localeCode]['invoice'] ?? '';

  String get invoices => _localizedValues[localeCode]['invoices'] ?? '';

  String get newInvoice => _localizedValues[localeCode]['new_invoice'] ?? '';

  String get createdInvoice =>
      _localizedValues[localeCode]['created_invoice'] ?? '';

  String get updatedInvoice =>
      _localizedValues[localeCode]['updated_invoice'] ?? '';

  String get archivedInvoice =>
      _localizedValues[localeCode]['archived_invoice'] ?? '';

  String get deletedInvoice =>
      _localizedValues[localeCode]['deleted_invoice'] ?? '';

  String get restoredInvoice =>
      _localizedValues[localeCode]['restored_invoice'] ?? '';

  String get archivedInvoices =>
      _localizedValues[localeCode]['archived_invoices'] ?? '';

  String get deletedInvoices =>
      _localizedValues[localeCode]['deleted_invoices'] ?? '';

  String get restoredInvoices =>
      _localizedValues[localeCode]['restored_invoices'] ?? '';

  String get emailedInvoice =>
      _localizedValues[localeCode]['emailed_invoice'] ?? '';

  String get emailedPayment =>
      _localizedValues[localeCode]['emailed_payment'] ?? '';

  String get amount => _localizedValues[localeCode]['amount'] ?? '';

  String get invoiceNumber =>
      _localizedValues[localeCode]['invoice_number'] ?? '';

  String get invoiceDate => _localizedValues[localeCode]['invoice_date'] ?? '';

  String get discount => _localizedValues[localeCode]['discount'] ?? '';

  String get poNumber => _localizedValues[localeCode]['po_number'] ?? '';

  String get terms => _localizedValues[localeCode]['terms'] ?? '';

  String get publicNotes => _localizedValues[localeCode]['public_notes'] ?? '';

  String get privateNotes =>
      _localizedValues[localeCode]['private_notes'] ?? '';

  String get frequency => _localizedValues[localeCode]['frequency'] ?? '';

  String get startDate => _localizedValues[localeCode]['start_date'] ?? '';

  String get endDate => _localizedValues[localeCode]['end_date'] ?? '';

  String get quoteNumber => _localizedValues[localeCode]['quote_number'] ?? '';

  String get quoteDate => _localizedValues[localeCode]['quote_date'] ?? '';

  String get validUntil => _localizedValues[localeCode]['valid_until'] ?? '';

  String get items => _localizedValues[localeCode]['items'] ?? '';

  String get partialDeposit =>
      _localizedValues[localeCode]['partial_deposit'] ?? '';

  String get description => _localizedValues[localeCode]['description'] ?? '';

  String get unitCost => _localizedValues[localeCode]['unit_cost'] ?? '';

  String get quantity => _localizedValues[localeCode]['quantity'] ?? '';

  String get addItem => _localizedValues[localeCode]['add_item'] ?? '';

  String get contact => _localizedValues[localeCode]['contact'] ?? '';

  String get workPhone => _localizedValues[localeCode]['work_phone'] ?? '';

  String get totalAmount => _localizedValues[localeCode]['total_amount'] ?? '';

  String get pdf => _localizedValues[localeCode]['pdf'] ?? '';

  String get dueDate => _localizedValues[localeCode]['due_date'] ?? '';

  String get partialDueDate =>
      _localizedValues[localeCode]['partial_due_date'] ?? '';

  String get status => _localizedValues[localeCode]['status'] ?? '';

  String get invoiceStatusId =>
      _localizedValues[localeCode]['invoice_status_id'] ?? '';

  String get quoteStatusId =>
      _localizedValues[localeCode]['quote_status'] ?? '';

  String get clickPlusToAddItem =>
      _localizedValues[localeCode]['click_plus_to_add_item'] ?? '';

  String get clickPlusToAddTime =>
      _localizedValues[localeCode]['click_plus_to_add_time'] ?? '';

  String get countSelected =>
      _localizedValues[localeCode]['count_selected'] ?? '';

  String get total => _localizedValues[localeCode]['total'] ?? '';

  String get totals => _localizedValues[localeCode]['totals'] ?? '';

  String get percent => _localizedValues[localeCode]['percent'] ?? '';

  String get edit => _localizedValues[localeCode]['edit'] ?? '';

  String get dismiss => _localizedValues[localeCode]['dismiss'] ?? '';

  String get pleaseSelectADate =>
      _localizedValues[localeCode]['please_select_a_date'] ?? '';

  String get pleaseSelectAClient =>
      _localizedValues[localeCode]['please_select_a_client'] ?? '';

  String get pleaseSelectAnInvoice =>
      _localizedValues[localeCode]['please_select_an_invoice'] ?? '';

  String get taskRate => _localizedValues[localeCode]['task_rate'] ?? '';

  String get settings => _localizedValues[localeCode]['settings'] ?? '';

  String get language => _localizedValues[localeCode]['language'] ?? '';

  String get currency => _localizedValues[localeCode]['currency'] ?? '';

  String get createdAt => _localizedValues[localeCode]['created_at'] ?? '';

  String get createdOn => _localizedValues[localeCode]['created_on'] ?? '';

  String get updatedAt => _localizedValues[localeCode]['updated_at'] ?? '';

  String get tax => _localizedValues[localeCode]['tax'] ?? '';

  String get pleaseEnterAnInvoiceNumber =>
      _localizedValues[localeCode]['please_enter_an_invoice_number'] ?? '';

  String get pleaseEnterAQuoteNumber =>
      _localizedValues[localeCode]['please_enter_a_quote_number'] ?? '';

  String get pastDue => _localizedValues[localeCode]['past_due'] ?? '';

  String get draft => _localizedValues[localeCode]['draft'] ?? '';

  String get sent => _localizedValues[localeCode]['sent'] ?? '';

  String get viewed => _localizedValues[localeCode]['viewed'] ?? '';

  String get approved => _localizedValues[localeCode]['approved'] ?? '';

  String get partial => _localizedValues[localeCode]['partial'] ?? '';

  String get paid => _localizedValues[localeCode]['paid'] ?? '';

  String get markSent => _localizedValues[localeCode]['mark_sent'] ?? '';

  String get markedInvoiceAsSent =>
      _localizedValues[localeCode]['marked_invoice_as_sent'] ?? '';

  String get markedInvoiceAsPaid =>
      _localizedValues[localeCode]['marked_invoice_as_paid'] ?? '';

  String get markedInvoicesAsSent =>
      _localizedValues[localeCode]['marked_invoices_as_sent'] ?? '';

  String get markedInvoicesAsPaid =>
      _localizedValues[localeCode]['marked_invoices_as_paid'] ?? '';

  String get done => _localizedValues[localeCode]['done'] ?? '';

  String get pleaseEnterAClientOrContactName =>
      _localizedValues[localeCode]['please_enter_a_client_or_contact_name'] ??
      '';

  String get darkMode => _localizedValues[localeCode]['dark_mode'] ?? '';

  String get restartAppToApplyChange =>
      _localizedValues[localeCode]['restart_app_to_apply_change'] ?? '';

  String get refreshData => _localizedValues[localeCode]['refresh_data'] ?? '';

  String get blankContact =>
      _localizedValues[localeCode]['blank_contact'] ?? '';

  String get activity => _localizedValues[localeCode]['activity'] ?? '';

  String get noRecordsFound =>
      _localizedValues[localeCode]['no_records_found'] ?? '';

  String get clone => _localizedValues[localeCode]['clone'] ?? '';

  String get loading => _localizedValues[localeCode]['loading'] ?? '';

  String get industry => _localizedValues[localeCode]['industry'] ?? '';

  String get size => _localizedValues[localeCode]['size'] ?? '';

  String get paymentTerms =>
      _localizedValues[localeCode]['payment_terms'] ?? '';

  String get paymentDate => _localizedValues[localeCode]['payment_date'] ?? '';

  String get paymentStatus =>
      _localizedValues[localeCode]['payment_status'] ?? '';

  String get paymentStatus1 =>
      _localizedValues[localeCode]['payment_status_1'] ?? '';

  String get paymentStatus2 =>
      _localizedValues[localeCode]['payment_status_2'] ?? '';

  String get paymentStatus3 =>
      _localizedValues[localeCode]['payment_status_3'] ?? '';

  String get paymentStatus4 =>
      _localizedValues[localeCode]['payment_status_4'] ?? '';

  String get paymentStatus5 =>
      _localizedValues[localeCode]['payment_status_5'] ?? '';

  String get paymentStatus6 =>
      _localizedValues[localeCode]['payment_status_6'] ?? '';

  String get net => _localizedValues[localeCode]['net'] ?? '';

  String get clientPortal =>
      _localizedValues[localeCode]['client_portal'] ?? '';

  String get showTasks => _localizedValues[localeCode]['show_tasks'] ?? '';

  String get emailReminders =>
      _localizedValues[localeCode]['email_reminders'] ?? '';

  String get enabled => _localizedValues[localeCode]['enabled'] ?? '';

  String get recipients => _localizedValues[localeCode]['recipients'] ?? '';

  String get initialEmail =>
      _localizedValues[localeCode]['initial_email'] ?? '';

  String get firstReminder =>
      _localizedValues[localeCode]['first_reminder'] ?? '';

  String get secondReminder =>
      _localizedValues[localeCode]['second_reminder'] ?? '';

  String get thirdReminder =>
      _localizedValues[localeCode]['third_reminder'] ?? '';

  String get template => _localizedValues[localeCode]['template'] ?? '';

  String get send => _localizedValues[localeCode]['send'] ?? '';

  String get subject => _localizedValues[localeCode]['subject'] ?? '';

  String get body => _localizedValues[localeCode]['body'] ?? '';

  String get sendEmail => _localizedValues[localeCode]['send_email'] ?? '';

  String get emailReceipt =>
      _localizedValues[localeCode]['email_receipt'] ?? '';

  String get autoBilling => _localizedValues[localeCode]['auto_billing'] ?? '';

  String get button => _localizedValues[localeCode]['button'] ?? '';

  String get preview => _localizedValues[localeCode]['preview'] ?? '';

  String get customize => _localizedValues[localeCode]['customize'] ?? '';

  String get history => _localizedValues[localeCode]['history'] ?? '';

  String get payment => _localizedValues[localeCode]['payment'] ?? '';

  String get payments => _localizedValues[localeCode]['payments'] ?? '';

  String get refunded => _localizedValues[localeCode]['refunded'] ?? '';

  String get paymentType => _localizedValues[localeCode]['payment_type'] ?? '';

  String get transactionReference =>
      _localizedValues[localeCode]['transaction_reference'] ?? '';

  String get enterPayment =>
      _localizedValues[localeCode]['enter_payment'] ?? '';

  String get createdPayment =>
      _localizedValues[localeCode]['created_payment'] ?? '';

  String get updatedPayment =>
      _localizedValues[localeCode]['updated_payment'] ?? '';

  String get archivedPayment =>
      _localizedValues[localeCode]['archived_payment'] ?? '';

  String get deletedPayment =>
      _localizedValues[localeCode]['deleted_payment'] ?? '';

  String get restoredPayment =>
      _localizedValues[localeCode]['restored_payment'] ?? '';

  String get archivedPayments =>
      _localizedValues[localeCode]['archived_payments'] ?? '';

  String get deletedPayments =>
      _localizedValues[localeCode]['deleted_payments'] ?? '';

  String get restoredPayments =>
      _localizedValues[localeCode]['restored_payments'] ?? '';

  String get quote => _localizedValues[localeCode]['quote'] ?? '';

  String get quotes => _localizedValues[localeCode]['quotes'] ?? '';

  String get newQuote => _localizedValues[localeCode]['new_quote'] ?? '';

  String get createdQuote =>
      _localizedValues[localeCode]['created_quote'] ?? '';

  String get updatedQuote =>
      _localizedValues[localeCode]['updated_quote'] ?? '';

  String get archivedQuote =>
      _localizedValues[localeCode]['archived_quote'] ?? '';

  String get deletedQuote =>
      _localizedValues[localeCode]['deleted_quote'] ?? '';

  String get restoredQuote =>
      _localizedValues[localeCode]['restored_quote'] ?? '';

  String get archivedQuotes =>
      _localizedValues[localeCode]['archived_quotes'] ?? '';

  String get deletedQuotes =>
      _localizedValues[localeCode]['deleted_quotes'] ?? '';

  String get restoredQuotes =>
      _localizedValues[localeCode]['restored_quotes'] ?? '';

  String get expense => _localizedValues[localeCode]['expense'] ?? '';

  String get expenses => _localizedValues[localeCode]['expenses'] ?? '';

  String get vendor => _localizedValues[localeCode]['vendor'] ?? '';

  String get vendors => _localizedValues[localeCode]['vendors'] ?? '';

  String get task => _localizedValues[localeCode]['task'] ?? '';

  String get tasks => _localizedValues[localeCode]['tasks'] ?? '';

  String get project => _localizedValues[localeCode]['project'] ?? '';

  String get projects => _localizedValues[localeCode]['projects'] ?? '';

  String get oneTimePassword =>
      _localizedValues[localeCode]['one_time_password'] ?? '';

  String get emailedQuote =>
      _localizedValues[localeCode]['emailed_quote'] ?? '';

  String get emailedCredit =>
      _localizedValues[localeCode]['emailed_credit'] ?? '';

  String get markedQuoteAsSent =>
      _localizedValues[localeCode]['marked_quote_as_sent'] ?? '';

  String get markedCreditAsSent =>
      _localizedValues[localeCode]['marked_credit_as_sent'] ?? '';

  String get expired => _localizedValues[localeCode]['expired'] ?? '';

  String get budgetedHours =>
      _localizedValues[localeCode]['budgeted_hours'] ?? '';

  String get pleaseEnterAName =>
      _localizedValues[localeCode]['please_enter_a_name'] ?? '';

  String get createdTask => _localizedValues[localeCode]['created_task'] ?? '';

  String get updatedTask => _localizedValues[localeCode]['updated_task'] ?? '';

  String get archivedTask =>
      _localizedValues[localeCode]['archived_task'] ?? '';

  String get deletedTask => _localizedValues[localeCode]['deleted_task'] ?? '';

  String get restoredTask =>
      _localizedValues[localeCode]['restored_task'] ?? '';

  String get archivedTasks =>
      _localizedValues[localeCode]['archived_tasks'] ?? '';

  String get deletedTasks =>
      _localizedValues[localeCode]['deleted_tasks'] ?? '';

  String get restoredTasks =>
      _localizedValues[localeCode]['restored_tasks'] ?? '';

  String get newTask => _localizedValues[localeCode]['new_task'] ?? '';

  String get duration => _localizedValues[localeCode]['duration'] ?? '';

  String get times => _localizedValues[localeCode]['times'] ?? '';

  String get date => _localizedValues[localeCode]['date'] ?? '';

  String get startTime => _localizedValues[localeCode]['start_time'] ?? '';

  String get endTime => _localizedValues[localeCode]['end_time'] ?? '';

  String get budgeted => _localizedValues[localeCode]['budgeted'] ?? '';

  String get timer => _localizedValues[localeCode]['timer'] ?? '';

  String get manual => _localizedValues[localeCode]['manual'] ?? '';

  String get autoStartTasks =>
      _localizedValues[localeCode]['auto_start_tasks'] ?? '';

  String get autoStartTasksHelp =>
      _localizedValues[localeCode]['auto_start_tasks_help'] ?? '';

  String get now => _localizedValues[localeCode]['now'] ?? '';

  String get startedTask => _localizedValues[localeCode]['started_task'] ?? '';

  String get stoppedTask => _localizedValues[localeCode]['stopped_task'] ?? '';

  String get resumedTask => _localizedValues[localeCode]['resumed_task'] ?? '';

  String get start => _localizedValues[localeCode]['start'] ?? '';

  String get stop => _localizedValues[localeCode]['stop'] ?? '';

  String get taskErrors => _localizedValues[localeCode]['task_errors'] ?? '';

  String get resume => _localizedValues[localeCode]['resume'] ?? '';

  String get running => _localizedValues[localeCode]['running'] ?? '';

  String get invoiced => _localizedValues[localeCode]['invoiced'] ?? '';

  String get logged => _localizedValues[localeCode]['logged'] ?? '';

  String get failedToFindRecord =>
      _localizedValues[localeCode]['failed_to_find_record'] ?? '';

  String get passwordIsTooShort =>
      _localizedValues[localeCode]['password_is_too_short'] ?? '';

  String get design => _localizedValues[localeCode]['design'] ?? '';

  String get copyShipping =>
      _localizedValues[localeCode]['copy_shipping'] ?? '';

  String get copyBilling => _localizedValues[localeCode]['copy_billing'] ?? '';

  String get address => _localizedValues[localeCode]['address'] ?? '';

  String get category => _localizedValues[localeCode]['category'] ?? '';

  String get markPaid => _localizedValues[localeCode]['mark_paid'] ?? '';

  String get convertCurrency =>
      _localizedValues[localeCode]['convert_currency'] ?? '';

  String get exchangeRate =>
      _localizedValues[localeCode]['exchange_rate'] ?? '';

  String get addDocumentsToInvoice =>
      _localizedValues[localeCode]['add_documents_to_invoice'] ?? '';

  String get pending => _localizedValues[localeCode]['pending'] ?? '';

  String get converted => _localizedValues[localeCode]['converted'] ?? '';

  String get expenseStatus1 =>
      _localizedValues[localeCode]['expense_status_1'] ?? '';

  String get expenseStatus2 =>
      _localizedValues[localeCode]['expense_status_2'] ?? '';

  String get expenseStatus3 =>
      _localizedValues[localeCode]['expense_status_3'] ?? '';

  String get expenseDate => _localizedValues[localeCode]['expense_date'] ?? '';

  String get noHistory => _localizedValues[localeCode]['no_history'] ?? '';

  String get takePicture => _localizedValues[localeCode]['take_picture'] ?? '';

  String get selectFile => _localizedValues[localeCode]['select_file'] ?? '';

  String get uploadFile => _localizedValues[localeCode]['upload_file'] ?? '';

  String get requiredFilesMissing =>
      _localizedValues[localeCode]['required_files_missing'] ?? '';

  String get download => _localizedValues[localeCode]['download'] ?? '';

  String get noRecordSelected =>
      _localizedValues[localeCode]['no_record_selected'] ?? '';

  String get requiresAnEnterprisePlan =>
      _localizedValues[localeCode]['requires_an_enterprise_plan'] ?? '';

  String get errorUnsavedChanges =>
      _localizedValues[localeCode]['error_unsaved_changes'] ?? '';

  String get createNew => _localizedValues[localeCode]['create_new'] ?? '';

  String get emailLogin => _localizedValues[localeCode]['email_login'] ?? '';

  String get createAccount =>
      _localizedValues[localeCode]['create_account'] ?? '';

  String get viewWebsite => _localizedValues[localeCode]['view_website'] ?? '';

  String get accountLogin =>
      _localizedValues[localeCode]['account_login'] ?? '';

  String get signUp => _localizedValues[localeCode]['sign_up'] ?? '';

  String get googleSignUp =>
      _localizedValues[localeCode]['google_sign_up'] ?? '';

  String get pleaseAgreeToTermsAndPrivacy =>
      _localizedValues[localeCode]['please_agree_to_terms_and_privacy'] ?? '';

  String get iAgreeToThe =>
      _localizedValues[localeCode]['i_agree_to_the'] ?? '';

  String get termsOfService =>
      _localizedValues[localeCode]['terms_of_service'] ?? '';

  String get privacyPolicy =>
      _localizedValues[localeCode]['privacy_policy'] ?? '';

  String get upgrade => _localizedValues[localeCode]['upgrade'] ?? '';

  String get proPlan => _localizedValues[localeCode]['pro_plan'] ?? '';

  String get enterprisePlan =>
      _localizedValues[localeCode]['enterprise_plan'] ?? '';

  String get countUsers => _localizedValues[localeCode]['count_users'] ?? '';

  String get annualSubscription =>
      _localizedValues[localeCode]['annual_subscription'] ?? '';

  String get pastPurchases =>
      _localizedValues[localeCode]['past_purchases'] ?? '';

  String get back => _localizedValues[localeCode]['back'] ?? '';

  String get redeem => _localizedValues[localeCode]['redeem'] ?? '';

  String get thankYouForYourPurchase =>
      _localizedValues[localeCode]['thank_you_for_your_purchase'] ?? '';

  String get select => _localizedValues[localeCode]['select'] ?? '';

  String get longPressSelectionIsDefault =>
      _localizedValues[localeCode]['long_press_multiselect'] ?? '';

  String get all => _localizedValues[localeCode]['all'] ?? '';

  String get emailSignUp => _localizedValues[localeCode]['email_sign_up'] ?? '';

  String get price => _localizedValues[localeCode]['price'] ?? '';

  String get companyDetails =>
      _localizedValues[localeCode]['company_details'] ?? '';

  String get userDetails => _localizedValues[localeCode]['user_details'] ?? '';

  String get localization => _localizedValues[localeCode]['localization'] ?? '';

  String get onlinePayments =>
      _localizedValues[localeCode]['online_payments'] ?? '';

  String get taxRates => _localizedValues[localeCode]['tax_rates'] ?? '';

  String get notifications =>
      _localizedValues[localeCode]['notifications'] ?? '';

  String get importExport =>
      _localizedValues[localeCode]['import_export'] ?? '';

  String get customFields =>
      _localizedValues[localeCode]['custom_fields'] ?? '';

  String get invoiceDesign =>
      _localizedValues[localeCode]['invoice_design'] ?? '';

  String get buyNowButtons =>
      _localizedValues[localeCode]['buy_now_buttons'] ?? '';

  String get emailSettings =>
      _localizedValues[localeCode]['email_settings'] ?? '';

  String get templatesAndReminders =>
      _localizedValues[localeCode]['templates_and_reminders'] ?? '';

  String get creditCardsAndBanks =>
      _localizedValues[localeCode]['credit_cards_and_banks'] ?? '';

  String get dataVisualizations =>
      _localizedValues[localeCode]['data_visualizations'] ?? '';

  String get basicSettings =>
      _localizedValues[localeCode]['basic_settings'] ?? '';

  String get advancedSettings =>
      _localizedValues[localeCode]['advanced_settings'] ?? '';

  String get defaults => _localizedValues[localeCode]['defaults'] ?? '';

  String get deviceSettings =>
      _localizedValues[localeCode]['device_settings'] ?? '';

  String get productSettings =>
      _localizedValues[localeCode]['product_settings'] ?? '';

  String get savedSettings =>
      _localizedValues[localeCode]['saved_settings'] ?? '';

  String get logo => _localizedValues[localeCode]['logo'] ?? '';

  String get uploadLogo => _localizedValues[localeCode]['upload_logo'] ?? '';

  String get uploadedLogo =>
      _localizedValues[localeCode]['uploaded_logo'] ?? '';

  String get newGroup => _localizedValues[localeCode]['new_group'] ?? '';

  String get createdGroup =>
      _localizedValues[localeCode]['created_group'] ?? '';

  String get updatedGroup =>
      _localizedValues[localeCode]['updated_group'] ?? '';

  String get archivedGroup =>
      _localizedValues[localeCode]['archived_group'] ?? '';

  String get deletedGroup =>
      _localizedValues[localeCode]['deleted_group'] ?? '';

  String get restoredGroup =>
      _localizedValues[localeCode]['restored_group'] ?? '';

  String get archivedGroups =>
      _localizedValues[localeCode]['archived_groups'] ?? '';

  String get deletedGroups =>
      _localizedValues[localeCode]['deleted_groups'] ?? '';

  String get restoredGroups =>
      _localizedValues[localeCode]['restored_groups'] ?? '';

  String get editGroup => _localizedValues[localeCode]['edit_group'] ?? '';

  String get groups => _localizedValues[localeCode]['groups'] ?? '';

  String get groupSettings =>
      _localizedValues[localeCode]['group_settings'] ?? '';

  String get filteredByGroup =>
      _localizedValues[localeCode]['filtered_by_group'] ?? '';

  String get filteredByClient =>
      _localizedValues[localeCode]['filtered_by_client'] ?? '';

  String get filteredByVendor =>
      _localizedValues[localeCode]['filtered_by_vendor'] ?? '';

  String get filteredByInvoice =>
      _localizedValues[localeCode]['filtered_by_invoice'] ?? '';

  String get filteredByProject =>
      _localizedValues[localeCode]['filtered_by_project'] ?? '';

  String get group => _localizedValues[localeCode]['group'] ?? '';

  String get timezone => _localizedValues[localeCode]['timezone'] ?? '';

  String get dateFormat => _localizedValues[localeCode]['date_format'] ?? '';

  String get datetimeFormat =>
      _localizedValues[localeCode]['datetime_format'] ?? '';

  String get militaryTime =>
      _localizedValues[localeCode]['military_time'] ?? '';

  String get militaryTimeHelp =>
      _localizedValues[localeCode]['military_time_help'] ?? '';

  String get sendReminders =>
      _localizedValues[localeCode]['send_reminders'] ?? '';

  String get symbol => _localizedValues[localeCode]['symbol'] ?? '';

  String get code => _localizedValues[localeCode]['ocde'] ?? '';

  String get sunday => _localizedValues[localeCode]['sunday'] ?? '';

  String get monday => _localizedValues[localeCode]['monday'] ?? '';

  String get tuesday => _localizedValues[localeCode]['tuesday'] ?? '';

  String get wednesday => _localizedValues[localeCode]['wednesday'] ?? '';

  String get thursday => _localizedValues[localeCode]['thursday'] ?? '';

  String get friday => _localizedValues[localeCode]['friday'] ?? '';

  String get saturday => _localizedValues[localeCode]['saturday'] ?? '';

  String get january => _localizedValues[localeCode]['january'] ?? '';

  String get february => _localizedValues[localeCode]['february'] ?? '';

  String get march => _localizedValues[localeCode]['march'] ?? '';

  String get april => _localizedValues[localeCode]['april'] ?? '';

  String get may => _localizedValues[localeCode]['may'] ?? '';

  String get june => _localizedValues[localeCode]['june'] ?? '';

  String get july => _localizedValues[localeCode]['july'] ?? '';

  String get august => _localizedValues[localeCode]['august'] ?? '';

  String get september => _localizedValues[localeCode]['september'] ?? '';

  String get october => _localizedValues[localeCode]['october'] ?? '';

  String get november => _localizedValues[localeCode]['november'] ?? '';

  String get december => _localizedValues[localeCode]['december'] ?? '';

  String get firstDayOfTheWeek =>
      _localizedValues[localeCode]['first_day_of_the_week'] ?? '';

  String get firstMonthOfTheYear =>
      _localizedValues[localeCode]['first_month_of_the_year'] ?? '';

  String get currencyFormat =>
      _localizedValues[localeCode]['currency_format'] ?? '';

  String get disabled => _localizedValues[localeCode]['disabled'] ?? '';

  String get defaultValue =>
      _localizedValues[localeCode]['default_value'] ?? '';

  String get discardChanges =>
      _localizedValues[localeCode]['discard_changes'] ?? '';

  String get continueEditing =>
      _localizedValues[localeCode]['continue_editing'] ?? '';

  String get editCompanyGateway =>
      _localizedValues[localeCode]['edit_company_gateway'] ?? '';

  String get newCompanyGateway =>
      _localizedValues[localeCode]['new_company_gateway'] ?? '';

  String get createdCompanyGateway =>
      _localizedValues[localeCode]['created_company_gateway'] ?? '';

  String get updatedCompanyGateway =>
      _localizedValues[localeCode]['updated_company_gateway'] ?? '';

  String get archivedCompanyGateway =>
      _localizedValues[localeCode]['archived_company_gateway'] ?? '';

  String get deletedCompanyGateway =>
      _localizedValues[localeCode]['deleted_company_gateway'] ?? '';

  String get restoredCompanyGateway =>
      _localizedValues[localeCode]['restored_company_gateway'] ?? '';

  String get archivedCompanyGateways =>
      _localizedValues[localeCode]['archived_company_gateways'] ?? '';

  String get deletedCompanyGateways =>
      _localizedValues[localeCode]['deleted_company_gateways'] ?? '';

  String get restoredCompanyGateways =>
      _localizedValues[localeCode]['restored_company_gateways'] ?? '';

  String get companyGateways =>
      _localizedValues[localeCode]['company_gateways'] ?? '';

  String get companyGateway =>
      _localizedValues[localeCode]['company_gateway'] ?? '';

  String get provider => _localizedValues[localeCode]['provider'] ?? '';

  String get fees => _localizedValues[localeCode]['fees'] ?? '';

  String get limits => _localizedValues[localeCode]['limits'] ?? '';

  String get fillProducts =>
      _localizedValues[localeCode]['fill_products'] ?? '';

  String get fillProductsHelp =>
      _localizedValues[localeCode]['fill_products_help'] ?? '';

  String get updateProducts =>
      _localizedValues[localeCode]['update_products'] ?? '';

  String get updateProductsHelp =>
      _localizedValues[localeCode]['update_products_help'] ?? '';

  String get convertProducts =>
      _localizedValues[localeCode]['convert_products'] ?? '';

  String get convertProductsHelp =>
      _localizedValues[localeCode]['convert_products_help'] ?? '';

  String get newTaxRate => _localizedValues[localeCode]['new_tax_rate'] ?? '';

  String get createdTaxRate =>
      _localizedValues[localeCode]['created_tax_rate'] ?? '';

  String get updatedTaxRate =>
      _localizedValues[localeCode]['updated_tax_rate'] ?? '';

  String get archivedTaxRate =>
      _localizedValues[localeCode]['archived_tax_rate'] ?? '';

  String get deletedTaxRate =>
      _localizedValues[localeCode]['deleted_tax_rate'] ?? '';

  String get restoredTaxRate =>
      _localizedValues[localeCode]['restored_tax_rate'] ?? '';

  String get archivedTaxRates =>
      _localizedValues[localeCode]['archived_tax_rates'] ?? '';

  String get deletedTaxRates =>
      _localizedValues[localeCode]['deleted_tax_rates'] ?? '';

  String get restoredTaxRates =>
      _localizedValues[localeCode]['restored_tax_rates'] ?? '';

  String get editTaxRate => _localizedValues[localeCode]['edit_tax_rate'] ?? '';

  String get taxRate => _localizedValues[localeCode]['tax_rate'] ?? '';

  String get rate => _localizedValues[localeCode]['rate'] ?? '';

  String get updateAddress =>
      _localizedValues[localeCode]['update_address'] ?? '';

  String get updateAddressHelp =>
      _localizedValues[localeCode]['update_address_help'] ?? '';

  String get credentials => _localizedValues[localeCode]['credentials'] ?? '';

  String get acceptedCardLogos =>
      _localizedValues[localeCode]['accepted_card_logos'] ?? '';

  String get min => _localizedValues[localeCode]['min'] ?? '';

  String get max => _localizedValues[localeCode]['max'] ?? '';

  String get minLimit => _localizedValues[localeCode]['min_limit'] ?? '';

  String get maxLimit => _localizedValues[localeCode]['max_limit'] ?? '';

  String get enableMin => _localizedValues[localeCode]['enable_min'] ?? '';

  String get enableMax => _localizedValues[localeCode]['enable_max'] ?? '';

  String get limitsAndFees =>
      _localizedValues[localeCode]['limits_and_fees'] ?? '';

  String get feeAmount => _localizedValues[localeCode]['fee_amount'] ?? '';

  String get feePercent => _localizedValues[localeCode]['fee_percent'] ?? '';

  String get feeCap => _localizedValues[localeCode]['fee_cap'] ?? '';

  String get priority => _localizedValues[localeCode]['priority'] ?? '';

  String get creditCard => _localizedValues[localeCode]['credit_card'] ?? '';

  String get bankTransfer =>
      _localizedValues[localeCode]['bank_transfer'] ?? '';

  String get processed => _localizedValues[localeCode]['processed'] ?? '';

  String get replyToEmail =>
      _localizedValues[localeCode]['reply_to_email'] ?? '';

  String get replyToName => _localizedValues[localeCode]['reply_to_name'] ?? '';

  String get bccEmail => _localizedValues[localeCode]['bcc_email'] ?? '';

  String get attachPdf => _localizedValues[localeCode]['attach_pdf'] ?? '';

  String get attachDocuments =>
      _localizedValues[localeCode]['attach_documents'] ?? '';

  String get attachUbl => _localizedValues[localeCode]['attach_ubl'] ?? '';

  String get emailStyle => _localizedValues[localeCode]['email_style'] ?? '';

  String get enableMarkup =>
      _localizedValues[localeCode]['enable_email_markup'] ?? '';

  String get enableMarkupHelp =>
      _localizedValues[localeCode]['enable_email_markup_help'] ?? '';

  String get emailDesign => _localizedValues[localeCode]['email_design'] ?? '';

  String get plain => _localizedValues[localeCode]['plain'] ?? '';

  String get light => _localizedValues[localeCode]['light'] ?? '';

  String get dark => _localizedValues[localeCode]['dark'] ?? '';

  String get emailSignature =>
      _localizedValues[localeCode]['email_signature'] ?? '';

  String get portalMode => _localizedValues[localeCode]['portal_mode'] ?? '';

  String get domain => _localizedValues[localeCode]['domain'] ?? '';

  String get subdomain => _localizedValues[localeCode]['subdomain'] ?? '';

  String get authorization =>
      _localizedValues[localeCode]['authorization'] ?? '';

  String get enablePortalPassword =>
      _localizedValues[localeCode]['enable_portal_password'] ?? '';

  String get enablePortalPasswordHelp =>
      _localizedValues[localeCode]['enable_portal_password_help'] ?? '';

  String get showAcceptInvoiceTerms =>
      _localizedValues[localeCode]['show_accept_invoice_terms'] ?? '';

  String get showAcceptInvoiceTermsHelp =>
      _localizedValues[localeCode]['show_accept_invoice_terms_help'] ?? '';

  String get showAcceptQuoteTerms =>
      _localizedValues[localeCode]['show_accept_quote_terms'] ?? '';

  String get showAcceptQuoteTermsHelp =>
      _localizedValues[localeCode]['show_accept_quote_terms_help'] ?? '';

  String get requireInvoiceSignature =>
      _localizedValues[localeCode]['require_invoice_signature'] ?? '';

  String get requireInvoiceSignatureHelp =>
      _localizedValues[localeCode]['require_invoice_signature_help'] ?? '';

  String get requireQuoteSignature =>
      _localizedValues[localeCode]['require_quote_signature'] ?? '';

  String get signatureOnPdf =>
      _localizedValues[localeCode]['signature_on_pdf'] ?? '';

  String get signatureOnPdfHelp =>
      _localizedValues[localeCode]['signature_on_pdf_help'] ?? '';

  String get customCss => _localizedValues[localeCode]['custom_css'] ?? '';

  String get customJavascript =>
      _localizedValues[localeCode]['custom_javascript'] ?? '';

  String get messages => _localizedValues[localeCode]['messages'] ?? '';

  String get prefix => _localizedValues[localeCode]['prefix'] ?? '';

  String get numberPattern =>
      _localizedValues[localeCode]['number_pattern'] ?? '';

  String get numberCounter =>
      _localizedValues[localeCode]['number_counter'] ?? '';

  String get creditField => _localizedValues[localeCode]['credit_field'] ?? '';

  String get invoiceField =>
      _localizedValues[localeCode]['invoice_field'] ?? '';

  String get clientField => _localizedValues[localeCode]['client_field'] ?? '';

  String get productField =>
      _localizedValues[localeCode]['product_field'] ?? '';

  String get paymentField =>
      _localizedValues[localeCode]['payment_field'] ?? '';

  String get contactField =>
      _localizedValues[localeCode]['contact_field'] ?? '';

  String get vendorField => _localizedValues[localeCode]['vendor_field'] ?? '';

  String get expenseField =>
      _localizedValues[localeCode]['expense_field'] ?? '';

  String get projectField =>
      _localizedValues[localeCode]['project_field'] ?? '';

  String get taskField => _localizedValues[localeCode]['task_field'] ?? '';

  String get groupField => _localizedValues[localeCode]['group_field'] ?? '';

  String get general => _localizedValues[localeCode]['general'] ?? '';

  String get numberPadding =>
      _localizedValues[localeCode]['number_padding'] ?? '';

  String get recurringPrefix =>
      _localizedValues[localeCode]['recurring_prefix'] ?? '';

  String get resetCounter =>
      _localizedValues[localeCode]['reset_counter'] ?? '';

  String get nextReset => _localizedValues[localeCode]['next_reset'] ?? '';

  String get credit => _localizedValues[localeCode]['credit'] ?? '';

  String get credits => _localizedValues[localeCode]['credits'] ?? '';

  String get customSurcharge =>
      _localizedValues[localeCode]['invoice_surcharge'] ?? '';

  String get chargeTaxes => _localizedValues[localeCode]['charge_taxes'] ?? '';

  String get companyField =>
      _localizedValues[localeCode]['company_field'] ?? '';

  String get companyValue =>
      _localizedValues[localeCode]['company_value'] ?? '';

  String get generatedNumbers =>
      _localizedValues[localeCode]['generated_numbers'] ?? '';

  String get company => _localizedValues[localeCode]['company'] ?? '';

  String get surchargeField =>
      _localizedValues[localeCode]['surcharge_field'] ?? '';

  String get never => _localizedValues[localeCode]['never'] ?? '';

  String get freqDaily => _localizedValues[localeCode]['freq_daily'] ?? '';

  String get freqWeekly => _localizedValues[localeCode]['freq_weekly'] ?? '';

  String get freqTwoWeeks =>
      _localizedValues[localeCode]['freq_two_weeks'] ?? '';

  String get freqFourWeeks =>
      _localizedValues[localeCode]['freq_four_weeks'] ?? '';

  String get freqMonthly => _localizedValues[localeCode]['freq_monthly'] ?? '';

  String get freqTwoMonths =>
      _localizedValues[localeCode]['freq_two_months'] ?? '';

  String get freqThreeMonths =>
      _localizedValues[localeCode]['freq_three_months'] ?? '';

  String get freqFourMonths =>
      _localizedValues[localeCode]['freq_four_months'] ?? '';

  String get freqSixMonths =>
      _localizedValues[localeCode]['freq_six_months'] ?? '';

  String get freqAnnually =>
      _localizedValues[localeCode]['freq_annually'] ?? '';

  String get freqTwoYears =>
      _localizedValues[localeCode]['freq_two_years'] ?? '';

  String get freqThreeYears =>
      _localizedValues[localeCode]['freq_three_years'] ?? '';

  String get workflowSettings =>
      _localizedValues[localeCode]['workflow_settings'] ?? '';

  String get autoEmailInvoice =>
      _localizedValues[localeCode]['auto_email_invoice'] ?? '';

  String get autoEmailInvoiceHelp =>
      _localizedValues[localeCode]['auto_email_invoice_help'] ?? '';

  String get autoArchiveInvoice =>
      _localizedValues[localeCode]['auto_archive_invoice'] ?? '';

  String get autoArchiveInvoiceHelp =>
      _localizedValues[localeCode]['auto_archive_invoice_help'] ?? '';

  String get autoArchiveQuote =>
      _localizedValues[localeCode]['auto_archive_quote'] ?? '';

  String get autoArchiveQuoteHelp =>
      _localizedValues[localeCode]['auto_archive_quote_help'] ?? '';

  String get autoConvertQuote =>
      _localizedValues[localeCode]['auto_convert_quote'] ?? '';

  String get autoConvertQuoteHelp =>
      _localizedValues[localeCode]['auto_convert_quote_help'] ?? '';

  String get invoiceTerms =>
      _localizedValues[localeCode]['invoice_terms'] ?? '';

  String get invoiceFooter =>
      _localizedValues[localeCode]['invoice_footer'] ?? '';

  String get quoteTerms => _localizedValues[localeCode]['quote_terms'] ?? '';

  String get quoteFooter => _localizedValues[localeCode]['quote_footer'] ?? '';

  String get invoiceFields =>
      _localizedValues[localeCode]['invoice_fields'] ?? '';

  String get productFields =>
      _localizedValues[localeCode]['product_fields'] ?? '';

  String get quoteDesign => _localizedValues[localeCode]['quote_design'] ?? '';

  String get pageSize => _localizedValues[localeCode]['page_size'] ?? '';

  String get fontSize => _localizedValues[localeCode]['font_size'] ?? '';

  String get primaryColor =>
      _localizedValues[localeCode]['primary_color'] ?? '';

  String get secondaryColor =>
      _localizedValues[localeCode]['secondary_color'] ?? '';

  String get primaryFont => _localizedValues[localeCode]['primary_font'] ?? '';

  String get secondaryFont =>
      _localizedValues[localeCode]['secondary_font'] ?? '';

  String get hidePaidToDate =>
      _localizedValues[localeCode]['hide_paid_to_date'] ?? '';

  String get hidePaidToDateHelp =>
      _localizedValues[localeCode]['hide_paid_to_date_help'] ?? '';

  String get invoiceEmbedDocuments =>
      _localizedValues[localeCode]['invoice_embed_documents'] ?? '';

  String get invoiceEmbedDocumentsHelp =>
      _localizedValues[localeCode]['invoice_embed_documents_help'] ?? '';

  String get allPagesHeader =>
      _localizedValues[localeCode]['all_pages_header'] ?? '';

  String get allPagesFooter =>
      _localizedValues[localeCode]['all_pages_footer'] ?? '';

  String get firstPage => _localizedValues[localeCode]['first_page'] ?? '';

  String get allPages => _localizedValues[localeCode]['all_pages'] ?? '';

  String get lastPage => _localizedValues[localeCode]['last_page'] ?? '';

  String get generalSettings =>
      _localizedValues[localeCode]['general_settings'] ?? '';

  String get invoiceOptions =>
      _localizedValues[localeCode]['invoice_options'] ?? '';

  String get newUser => _localizedValues[localeCode]['new_user'] ?? '';

  String get createdUser => _localizedValues[localeCode]['created_user'] ?? '';

  String get updatedUser => _localizedValues[localeCode]['updated_user'] ?? '';

  String get archivedUser =>
      _localizedValues[localeCode]['archived_user'] ?? '';

  String get deletedUser => _localizedValues[localeCode]['deleted_user'] ?? '';

  String get removedUser => _localizedValues[localeCode]['removed_user'] ?? '';

  String get restoredUser =>
      _localizedValues[localeCode]['restored_user'] ?? '';

  String get archivedUsers =>
      _localizedValues[localeCode]['archived_users'] ?? '';

  String get deletedUsers =>
      _localizedValues[localeCode]['deleted_users'] ?? '';

  String get removedUsers =>
      _localizedValues[localeCode]['removed_users'] ?? '';

  String get restoredUsers =>
      _localizedValues[localeCode]['restored_users'] ?? '';

  String get editUser => _localizedValues[localeCode]['edit_user'] ?? '';

  String get users => _localizedValues[localeCode]['users'] ?? '';

  String get userManagement =>
      _localizedValues[localeCode]['user_management'] ?? '';

  String get administrator =>
      _localizedValues[localeCode]['administrator'] ?? '';

  String get administratorHelp =>
      _localizedValues[localeCode]['administrator_help'] ?? '';

  String get filteredByUser =>
      _localizedValues[localeCode]['filtered_by_user'] ?? '';

  String get endlessReminder =>
      _localizedValues[localeCode]['endless_reminder'] ?? '';

  String get invoiceEmail =>
      _localizedValues[localeCode]['invoice_email'] ?? '';

  String get paymentEmail =>
      _localizedValues[localeCode]['payment_email'] ?? '';

  String get quoteEmail => _localizedValues[localeCode]['quote_email'] ?? '';

  String get days => _localizedValues[localeCode]['days'] ?? '';

  String get beforeDueDate =>
      _localizedValues[localeCode]['before_due_date'] ?? '';

  String get afterDueDate =>
      _localizedValues[localeCode]['after_due_date'] ?? '';

  String get afterInvoiceDate =>
      _localizedValues[localeCode]['after_invoice_date'] ?? '';

  String get schedule => _localizedValues[localeCode]['schedule'] ?? '';

  String get lateFeeAmount =>
      _localizedValues[localeCode]['late_fee_amount'] ?? '';

  String get lateFeePercent =>
      _localizedValues[localeCode]['late_fee_percent'] ?? '';

  String get creditNumber =>
      _localizedValues[localeCode]['credit_number'] ?? '';

  String get paymentNumber =>
      _localizedValues[localeCode]['payment_number'] ?? '';

  String get lateFees => _localizedValues[localeCode]['late_fees'] ?? '';

  String get recoverPassword =>
      _localizedValues[localeCode]['recover_password'] ?? '';

  String get submit => _localizedValues[localeCode]['submit'] ?? '';

  String get recoverPasswordEmailSent =>
      _localizedValues[localeCode]['recover_password_email_sent'] ?? '';

  String get fieldType => _localizedValues[localeCode]['field_type'] ?? '';

  String get singleLineText =>
      _localizedValues[localeCode]['single_line_text'] ?? '';

  String get multiLineText =>
      _localizedValues[localeCode]['multi_line_text'] ?? '';

  String get dropdown => _localizedValues[localeCode]['dropdown'] ?? '';

  String get options => _localizedValues[localeCode]['options'] ?? '';

  String get commaSeparatedList =>
      _localizedValues[localeCode]['comma_sparated_list'] ?? '';

  String get switchLabel => _localizedValues[localeCode]['switch'] ?? '';

  String get accentColor => _localizedValues[localeCode]['accent_color'] ?? '';

  String get taxSettings => _localizedValues[localeCode]['tax_settings'] ?? '';

  String get configureRates =>
      _localizedValues[localeCode]['configure_rates'] ?? '';

  String get configureGateways =>
      _localizedValues[localeCode]['configure_gateways'] ?? '';

  String get taxSettingsRates =>
      _localizedValues[localeCode]['tax_settings_rates'] ?? '';

  String get noClientSelected =>
      _localizedValues[localeCode]['no_client_selected'] ?? '';

  String get invoiceTax => _localizedValues[localeCode]['invoice_tax'] ?? '';

  String get lineItemTax => _localizedValues[localeCode]['line_item_tax'] ?? '';

  String get inclusiveTaxes =>
      _localizedValues[localeCode]['inclusive_taxes'] ?? '';

  String get invoiceTaxRates =>
      _localizedValues[localeCode]['invoice_tax_rates'] ?? '';

  String get itemTaxRates =>
      _localizedValues[localeCode]['item_tax_rates'] ?? '';

  String get user => _localizedValues[localeCode]['user'] ?? '';

  String get defaultTaxRate =>
      _localizedValues[localeCode]['default_tax_rate'] ?? '';

  String get oneTaxRate => _localizedValues[localeCode]['one_tax_rate'] ?? '';

  String get twoTaxRates => _localizedValues[localeCode]['two_tax_rates'] ?? '';

  String get threeTaxRates =>
      _localizedValues[localeCode]['three_tax_rates'] ?? '';

  String get customValue1 =>
      _localizedValues[localeCode]['custom_value1'] ?? '';

  String get customValue2 =>
      _localizedValues[localeCode]['custom_value2'] ?? '';

  String get customValue3 =>
      _localizedValues[localeCode]['custom_value3'] ?? '';

  String get customValue4 =>
      _localizedValues[localeCode]['custom_value4'] ?? '';

  String get emailStyleCustom =>
      _localizedValues[localeCode]['email_style_custom'] ?? '';

  String get customMessageDashboard =>
      _localizedValues[localeCode]['custom_message_dashboard'] ?? '';

  String get customMessageUnpaidInvoice =>
      _localizedValues[localeCode]['custom_message_unpaid_invoice'] ?? '';

  String get customMessagePaidInvoice =>
      _localizedValues[localeCode]['custom_message_paid_invoice'] ?? '';

  String get customMessageUnapprovedQuote =>
      _localizedValues[localeCode]['custom_message_unapproved_quote'] ?? '';

  String get lockInvoices =>
      _localizedValues[localeCode]['lock_invoices'] ?? '';

  String get translations => _localizedValues[localeCode]['translations'] ?? '';

  String get taskNumberPattern =>
      _localizedValues[localeCode]['task_number_pattern'] ?? '';

  String get taskNumberCounter =>
      _localizedValues[localeCode]['task_number_counter'] ?? '';

  String get expenseNumberPattern =>
      _localizedValues[localeCode]['expense_number_pattern'] ?? '';

  String get expenseNumberCounter =>
      _localizedValues[localeCode]['expense_number_counter'] ?? '';

  String get vendorNumberPattern =>
      _localizedValues[localeCode]['vendor_number_pattern'] ?? '';

  String get vendorNumberCounter =>
      _localizedValues[localeCode]['vendor_number_counter'] ?? '';

  String get ticketNumberPattern =>
      _localizedValues[localeCode]['ticket_number_pattern'] ?? '';

  String get ticketNumberCounter =>
      _localizedValues[localeCode]['ticket_number_counter'] ?? '';

  String get paymentNumberPattern =>
      _localizedValues[localeCode]['payment_number_pattern'] ?? '';

  String get paymentNumberCounter =>
      _localizedValues[localeCode]['payment_number_counter'] ?? '';

  String get invoiceNumberPattern =>
      _localizedValues[localeCode]['invoice_number_pattern'] ?? '';

  String get invoiceNumberCounter =>
      _localizedValues[localeCode]['invoice_number_counter'] ?? '';

  String get quoteNumberPattern =>
      _localizedValues[localeCode]['quote_number_pattern'] ?? '';

  String get quoteNumberCounter =>
      _localizedValues[localeCode]['quote_number_counter'] ?? '';

  String get clientNumberPattern =>
      _localizedValues[localeCode]['client_number_pattern'] ?? '';

  String get clientNumberCounter =>
      _localizedValues[localeCode]['client_number_counter'] ?? '';

  String get creditNumberPattern =>
      _localizedValues[localeCode]['credit_number_pattern'] ?? '';

  String get creditNumberCounter =>
      _localizedValues[localeCode]['credit_number_counter'] ?? '';

  String get resetCounterDate =>
      _localizedValues[localeCode]['reset_counter_date'] ?? '';

  String get counterPadding =>
      _localizedValues[localeCode]['counter_padding'] ?? '';

  String get sharedInvoiceQuoteCounter =>
      _localizedValues[localeCode]['shared_invoice_quote_counter'] ?? '';

  String get defaultTaxName1 =>
      _localizedValues[localeCode]['default_tax_name_1'] ?? '';

  String get defaultTaxRate1 =>
      _localizedValues[localeCode]['default_tax_rate_1'] ?? '';

  String get defaultTaxName2 =>
      _localizedValues[localeCode]['default_tax_name_2'] ?? '';

  String get defaultTaxRate2 =>
      _localizedValues[localeCode]['default_tax_rate_2'] ?? '';

  String get defaultTaxName3 =>
      _localizedValues[localeCode]['default_tax_name_3'] ?? '';

  String get defaultTaxRate3 =>
      _localizedValues[localeCode]['default_tax_rate_3'] ?? '';

  String get emailSubjectInvoice =>
      _localizedValues[localeCode]['email_subject_invoice'] ?? '';

  String get emailSubjectQuote =>
      _localizedValues[localeCode]['email_subject_quote'] ?? '';

  String get emailSubjectPayment =>
      _localizedValues[localeCode]['email_subject_payment'] ?? '';

  String get emailSubjectPaymentPartial =>
      _localizedValues[localeCode]['email_subject_payment_partial'] ?? '';

  String get showCost => _localizedValues[localeCode]['show_cost'] ?? '';

  String get showProductCost =>
      _localizedValues[localeCode]['show_product_cost'] ?? '';

  String get showCostHelp =>
      _localizedValues[localeCode]['show_cost_help'] ?? '';

  String get showInvoiceQuantity =>
      _localizedValues[localeCode]['show_invoice_quantity'] ?? '';

  String get showProductQuantityHelp =>
      _localizedValues[localeCode]['show_product_quantity_help'] ?? '';

  String get showProductQuantity =>
      _localizedValues[localeCode]['show_product_quantity'] ?? '';

  String get showInvoiceQuantityHelp =>
      _localizedValues[localeCode]['show_invoice_quantity_help'] ?? '';

  String get showProductDiscount =>
      _localizedValues[localeCode]['show_product_discount'] ?? '';

  String get showProductDiscountHelp =>
      _localizedValues[localeCode]['show_product_discount_help'] ?? '';

  String get defaultQuantity =>
      _localizedValues[localeCode]['default_quantity'] ?? '';

  String get defaultTaskRate =>
      _localizedValues[localeCode]['default_task_rate'] ?? '';

  String get defaultQuantityHelp =>
      _localizedValues[localeCode]['default_quantity_help'] ?? '';

  String get firstCustom => _localizedValues[localeCode]['first_custom'] ?? '';

  String get secondCustom =>
      _localizedValues[localeCode]['second_custom'] ?? '';

  String get thirdCustom => _localizedValues[localeCode]['third_custom'] ?? '';

  String get module => _localizedValues[localeCode]['module'] ?? '';

  String get view => _localizedValues[localeCode]['view'] ?? '';

  String get layout => _localizedValues[localeCode]['layout'] ?? '';

  String get mobile => _localizedValues[localeCode]['mobile'] ?? '';

  String get desktop => _localizedValues[localeCode]['desktop'] ?? '';

  String get tablet => _localizedValues[localeCode]['tablet'] ?? '';

  String get float => _localizedValues[localeCode]['float'] ?? '';

  String get collapse => _localizedValues[localeCode]['collapse'] ?? '';

  String get showOrHide => _localizedValues[localeCode]['show_or_hide'] ?? '';

  String get menuSidebar => _localizedValues[localeCode]['menu_sidebar'] ?? '';

  String get historySidebar =>
      _localizedValues[localeCode]['history_sidebar'] ?? '';

  String get selectCompany =>
      _localizedValues[localeCode]['select_company'] ?? '';

  String get newPayment => _localizedValues[localeCode]['new_payment'] ?? '';

  String get showTable => _localizedValues[localeCode]['show_table'] ?? '';

  String get showList => _localizedValues[localeCode]['show_list'] ?? '';

  String get whenSaved => _localizedValues[localeCode]['when_saved'] ?? '';

  String get whenSent => _localizedValues[localeCode]['when_sent'] ?? '';

  String get generateNumber =>
      _localizedValues[localeCode]['generate_number'] ?? '';

  String get yes => _localizedValues[localeCode]['yes'] ?? '';

  String get no => _localizedValues[localeCode]['no'] ?? '';

  String get deletedLogo => _localizedValues[localeCode]['deleted_logo'] ?? '';

  String get pleaseEnterAValue =>
      _localizedValues[localeCode]['please_enter_a_value'] ?? '';

  String get clientPortalTasks =>
      _localizedValues[localeCode]['client_portal_tasks'] ?? '';

  String get clientPortalDashboard =>
      _localizedValues[localeCode]['client_portal_dashboard'] ?? '';

  String get passwordIsTooEasy =>
      _localizedValues[localeCode]['password_is_too_easy'] ?? '';

  String get iFrameUrl => _localizedValues[localeCode]['iframe_url'] ?? '';

  String get domainUrl => _localizedValues[localeCode]['domain_url'] ?? '';

  String get creditEmail => _localizedValues[localeCode]['credit_email'] ?? '';

  String get item => _localizedValues[localeCode]['item'] ?? '';

  String get lineTotal => _localizedValues[localeCode]['line_total'] ?? '';

  String get calculateSubtotal =>
      _localizedValues[localeCode]['subtotal'] ?? '';

  String get contactUs => _localizedValues[localeCode]['contact_us'] ?? '';

  String get documentation =>
      _localizedValues[localeCode]['documentation'] ?? '';

  String get about => _localizedValues[localeCode]['about'] ?? '';

  String get supportForum =>
      _localizedValues[localeCode]['support_forum'] ?? '';

  String get configureSettings =>
      _localizedValues[localeCode]['configure_settings'] ?? '';

  String get adjustFeePercent =>
      _localizedValues[localeCode]['adjust_fee_percent'] ?? '';

  String get adjustFeePercentHelp =>
      _localizedValues[localeCode]['adjust_fee_percent_help'] ?? '';

  String get pdfMinRequirements =>
      _localizedValues[localeCode]['pdf_min_requirements'] ?? '';

  String get showProductDetails =>
      _localizedValues[localeCode]['show_product_details'] ?? '';

  String get showProductDetailsHelp =>
      _localizedValues[localeCode]['show_product_details_help'] ?? '';

  String get from => _localizedValues[localeCode]['from'] ?? '';

  String get message => _localizedValues[localeCode]['message'] ?? '';

  String get yourMessageHasBeenReceived =>
      _localizedValues[localeCode]['your_message_has_been_received'] ?? '';

  String get includeRecentErrors =>
      _localizedValues[localeCode]['include_recent_errors'] ?? '';

  String get applied => _localizedValues[localeCode]['applied'] ?? '';

  String get verifyPassword =>
      _localizedValues[localeCode]['verify_password'] ?? '';

  String get entityState => _localizedValues[localeCode]['entity_state'] ?? '';

  String get multiselect => _localizedValues[localeCode]['multiselect'] ?? '';

  String get contactEmail =>
      _localizedValues[localeCode]['contact_email'] ?? '';

  String get filteredBy => _localizedValues[localeCode]['filtered_by'] ?? '';

  String get refund => _localizedValues[localeCode]['refund'] ?? '';

  String get refundDate => _localizedValues[localeCode]['refund_date'] ?? '';

  String get help => _localizedValues[localeCode]['help'] ?? '';

  String get unpaidInvoice =>
      _localizedValues[localeCode]['unpaid_invoice'] ?? '';

  String get paidInvoice => _localizedValues[localeCode]['paid_invoice'] ?? '';

  String get unapprovedQuote =>
      _localizedValues[localeCode]['unapproved_quote'] ?? '';

  String get addCompany => _localizedValues[localeCode]['add_company'] ?? '';

  String get reports => _localizedValues[localeCode]['reports'] ?? '';

  String get report => _localizedValues[localeCode]['report'] ?? '';

  String get aging => _localizedValues[localeCode]['aging'] ?? '';

  String get columns => _localizedValues[localeCode]['columns'] ?? '';

  String get profitAndLoss =>
      _localizedValues[localeCode]['profit_and_loss'] ?? '';

  String get editColumns => _localizedValues[localeCode]['edit_columns'] ?? '';

  String get addColumn => _localizedValues[localeCode]['add_column'] ?? '';

  String get assignedTo => _localizedValues[localeCode]['assigned_to'] ?? '';

  String get createdBy => _localizedValues[localeCode]['created_by'] ?? '';

  String get assignedTId =>
      _localizedValues[localeCode]['assigned_to_id'] ?? '';

  String get createdById => _localizedValues[localeCode]['created_by_id'] ?? '';

  String get clientId => _localizedValues[localeCode]['client_id'] ?? '';

  String get shippingAddress1 =>
      _localizedValues[localeCode]['shipping_address1'] ?? '';

  String get shippingAddress2 =>
      _localizedValues[localeCode]['shipping_address2'] ?? '';

  String get shippingCity =>
      _localizedValues[localeCode]['shipping_city'] ?? '';

  String get shippingState =>
      _localizedValues[localeCode]['shipping_state'] ?? '';

  String get shippingPostalCode =>
      _localizedValues[localeCode]['shipping_postal_code'] ?? '';

  String get shippingCountry =>
      _localizedValues[localeCode]['shipping_country'] ?? '';

  String get billingAddress1 =>
      _localizedValues[localeCode]['billing_address1'] ?? '';

  String get billingAddress2 =>
      _localizedValues[localeCode]['billing_address2'] ?? '';

  String get billingCity => _localizedValues[localeCode]['billing_city'] ?? '';

  String get billingState =>
      _localizedValues[localeCode]['billing_state'] ?? '';

  String get billingPostalCode =>
      _localizedValues[localeCode]['billing_postal_code'] ?? '';

  String get billingCountry =>
      _localizedValues[localeCode]['billing_country'] ?? '';

  String get contactFullName =>
      _localizedValues[localeCode]['contact_full_name'] ?? '';

  String get contactPhone =>
      _localizedValues[localeCode]['contact_phone'] ?? '';

  String get contactCustomValue1 =>
      _localizedValues[localeCode]['contact_custom_value1'] ?? '';

  String get contactCustomValue2 =>
      _localizedValues[localeCode]['contact_custom_value2'] ?? '';

  String get contactCustomValue3 =>
      _localizedValues[localeCode]['contact_custom_value3'] ?? '';

  String get contactCustomValue4 =>
      _localizedValues[localeCode]['contact_custom_value4'] ?? '';

  String get creditBalance =>
      _localizedValues[localeCode]['credit_balance'] ?? '';

  String get contactLastLogin =>
      _localizedValues[localeCode]['contact_last_login'] ?? '';

  String get groupBy => _localizedValues[localeCode]['group_by'] ?? '';

  String get isActive => _localizedValues[localeCode]['is_active'] ?? '';

  String get subgroup => _localizedValues[localeCode]['subgroup'] ?? '';

  String get day => _localizedValues[localeCode]['day'] ?? '';

  String get month => _localizedValues[localeCode]['month'] ?? '';

  String get year => _localizedValues[localeCode]['year'] ?? '';

  String get blank => _localizedValues[localeCode]['blank'] ?? '';

  String get count => _localizedValues[localeCode]['count'] ?? '';

  String get chart => _localizedValues[localeCode]['chart'] ?? '';

  String get export => _localizedValues[localeCode]['export'] ?? '';

  String get number => _localizedValues[localeCode]['number'] ?? '';

  String get reset => _localizedValues[localeCode]['reset'] ?? '';

  String get client1 => _localizedValues[localeCode]['client1'] ?? '';

  String get client2 => _localizedValues[localeCode]['client2'] ?? '';

  String get client3 => _localizedValues[localeCode]['client3'] ?? '';

  String get client4 => _localizedValues[localeCode]['client4'] ?? '';

  String get company1 => _localizedValues[localeCode]['company1'] ?? '';

  String get company2 => _localizedValues[localeCode]['company2'] ?? '';

  String get company3 => _localizedValues[localeCode]['company3'] ?? '';

  String get company4 => _localizedValues[localeCode]['company4'] ?? '';

  String get product1 => _localizedValues[localeCode]['product1'] ?? '';

  String get product2 => _localizedValues[localeCode]['product2'] ?? '';

  String get product3 => _localizedValues[localeCode]['product3'] ?? '';

  String get product4 => _localizedValues[localeCode]['product4'] ?? '';

  String get contact1 => _localizedValues[localeCode]['contact1'] ?? '';

  String get contact2 => _localizedValues[localeCode]['contact2'] ?? '';

  String get contact3 => _localizedValues[localeCode]['contact3'] ?? '';

  String get contact4 => _localizedValues[localeCode]['contact4'] ?? '';

  String get task1 => _localizedValues[localeCode]['task1'] ?? '';

  String get task2 => _localizedValues[localeCode]['task2'] ?? '';

  String get task3 => _localizedValues[localeCode]['task3'] ?? '';

  String get task4 => _localizedValues[localeCode]['task4'] ?? '';

  String get project1 => _localizedValues[localeCode]['project1'] ?? '';

  String get project2 => _localizedValues[localeCode]['project2'] ?? '';

  String get project3 => _localizedValues[localeCode]['project3'] ?? '';

  String get project4 => _localizedValues[localeCode]['project4'] ?? '';

  String get expense1 => _localizedValues[localeCode]['expense1'] ?? '';

  String get expense2 => _localizedValues[localeCode]['expense2'] ?? '';

  String get expense3 => _localizedValues[localeCode]['expense3'] ?? '';

  String get expense4 => _localizedValues[localeCode]['expense4'] ?? '';

  String get vendor1 => _localizedValues[localeCode]['vendor1'] ?? '';

  String get vendor2 => _localizedValues[localeCode]['vendor2'] ?? '';

  String get vendor3 => _localizedValues[localeCode]['vendor3'] ?? '';

  String get vendor4 => _localizedValues[localeCode]['vendor4'] ?? '';

  String get invoice1 => _localizedValues[localeCode]['invoice1'] ?? '';

  String get invoice2 => _localizedValues[localeCode]['invoice2'] ?? '';

  String get invoice3 => _localizedValues[localeCode]['invoice3'] ?? '';

  String get invoice4 => _localizedValues[localeCode]['invoice4'] ?? '';

  String get payment1 => _localizedValues[localeCode]['payment1'] ?? '';

  String get payment2 => _localizedValues[localeCode]['payment2'] ?? '';

  String get payment3 => _localizedValues[localeCode]['payment3'] ?? '';

  String get payment4 => _localizedValues[localeCode]['payment4'] ?? '';

  String get surcharge1 => _localizedValues[localeCode]['surcharge1'] ?? '';

  String get surcharge2 => _localizedValues[localeCode]['surcharge2'] ?? '';

  String get surcharge3 => _localizedValues[localeCode]['surcharge3'] ?? '';

  String get surcharge4 => _localizedValues[localeCode]['surcharge4'] ?? '';

  String get group1 => _localizedValues[localeCode]['group1'] ?? '';

  String get group2 => _localizedValues[localeCode]['group2'] ?? '';

  String get group3 => _localizedValues[localeCode]['group3'] ?? '';

  String get group4 => _localizedValues[localeCode]['group4'] ?? '';

  String get addedCompany =>
      _localizedValues[localeCode]['added_company'] ?? '';

  String get newCompany => _localizedValues[localeCode]['new_company'] ?? '';

  String get creditFooter =>
      _localizedValues[localeCode]['credit_footer'] ?? '';

  String get creditTerms => _localizedValues[localeCode]['credit_terms'] ?? '';

  String get slackWebhookUrl =>
      _localizedValues[localeCode]['slack_webhook_url'] ?? '';

  String get trackingId => _localizedValues[localeCode]['tracking_id'] ?? '';

  String get integrations => _localizedValues[localeCode]['integrations'] ?? '';

  String get learnMore => _localizedValues[localeCode]['learn_more'] ?? '';

  String get updateAvailable =>
      _localizedValues[localeCode]['update_available'] ?? '';

  String get aNewVersionIsAvailable =>
      _localizedValues[localeCode]['a_new_version_is_available'] ?? '';

  String get updateNow => _localizedValues[localeCode]['update_now'] ?? '';

  String get currentVersion =>
      _localizedValues[localeCode]['current_version'] ?? '';

  String get latestVersion =>
      _localizedValues[localeCode]['latest_version'] ?? '';

  String get appUpdated => _localizedValues[localeCode]['app_updated'] ?? '';

  String get taskStatus => _localizedValues[localeCode]['task_status'] ?? '';

  String get taskStatuses =>
      _localizedValues[localeCode]['task_statuses'] ?? '';

  String get newTaskStatus =>
      _localizedValues[localeCode]['new_task_status'] ?? '';

  String get createdTaskStatus =>
      _localizedValues[localeCode]['created_task_status'] ?? '';

  String get updatedTaskStatus =>
      _localizedValues[localeCode]['updated_task_status'] ?? '';

  String get archivedTaskStatus =>
      _localizedValues[localeCode]['archived_task_status'] ?? '';

  String get deletedTaskStatus =>
      _localizedValues[localeCode]['deleted_task_status'] ?? '';

  String get restoredTaskStatus =>
      _localizedValues[localeCode]['restored_task_status'] ?? '';

  String get archivedTaskStatuses =>
      _localizedValues[localeCode]['archived_task_statuses'] ?? '';

  String get deletedTaskStatuses =>
      _localizedValues[localeCode]['deleted_task_statuses'] ?? '';

  String get restoredTaskStatuses =>
      _localizedValues[localeCode]['restored_task_statuses'] ?? '';

  String get editTaskStatus =>
      _localizedValues[localeCode]['edit_task_status'] ?? '';

  String get searchTaskStatus =>
      _localizedValues[localeCode]['search_task_status'] ?? '';

  String get expenseTotal =>
      _localizedValues[localeCode]['expense_total'] ?? '';

  String get expenseCategory =>
      _localizedValues[localeCode]['expense_category'] ?? '';

  String get expenseCategories =>
      _localizedValues[localeCode]['expense_categories'] ?? '';

  String get newExpenseCategory =>
      _localizedValues[localeCode]['new_expense_category'] ?? '';

  String get createdExpenseCategory =>
      _localizedValues[localeCode]['created_expense_category'] ?? '';

  String get updatedExpenseCategory =>
      _localizedValues[localeCode]['updated_expense_category'] ?? '';

  String get archivedExpenseCategory =>
      _localizedValues[localeCode]['archived_expense_category'] ?? '';

  String get deletedExpenseCategory =>
      _localizedValues[localeCode]['deleted_expense_category'] ?? '';

  String get restoredExpenseCategory =>
      _localizedValues[localeCode]['restored_expense_category'] ?? '';

  String get archivedExpenseCategories =>
      _localizedValues[localeCode]['archived_expense_categories'] ?? '';

  String get deletedExpenseCategories =>
      _localizedValues[localeCode]['deleted_expense_categories'] ?? '';

  String get restoredExpenseCategories =>
      _localizedValues[localeCode]['restored_expense_categories'] ?? '';

  String get editExpenseCategory =>
      _localizedValues[localeCode]['edit_expense_category'] ?? '';

  String get searchExpenseCategory =>
      _localizedValues[localeCode]['search_expense_category'] ?? '';

  String get recurringInvoice =>
      _localizedValues[localeCode]['recurring_invoice'] ?? '';

  String get recurringInvoices =>
      _localizedValues[localeCode]['recurring_invoices'] ?? '';

  String get newRecurringInvoice =>
      _localizedValues[localeCode]['new_recurring_invoice'] ?? '';

  String get createdRecurringInvoice =>
      _localizedValues[localeCode]['created_recurring_invoice'] ?? '';

  String get updatedRecurringInvoice =>
      _localizedValues[localeCode]['updated_recurring_invoice'] ?? '';

  String get archivedRecurringInvoice =>
      _localizedValues[localeCode]['archived_recurring_invoice'] ?? '';

  String get deletedRecurringInvoice =>
      _localizedValues[localeCode]['deleted_recurring_invoice'] ?? '';

  String get restoredRecurringInvoice =>
      _localizedValues[localeCode]['restored_recurring_invoice'] ?? '';

  String get archivedRecurringInvoices =>
      _localizedValues[localeCode]['archived_recurring_invoices'] ?? '';

  String get deletedRecurringInvoices =>
      _localizedValues[localeCode]['deleted_recurring_invoices'] ?? '';

  String get restoredRecurringInvoices =>
      _localizedValues[localeCode]['restored_recurring_invoices'] ?? '';

  String get stoppedRecurringInvoice =>
      _localizedValues[localeCode]['stopped_recurring_invoice'] ?? '';

  String get startedRecurringInvoice =>
      _localizedValues[localeCode]['started_recurring_invoice'] ?? '';

  String get resumedRecurringInvoice =>
      _localizedValues[localeCode]['resumed_recurring_invoice'] ?? '';

  String get searchRecurringInvoice =>
      _localizedValues[localeCode]['search_recurring_invoice'] ?? '';

  String get webhook => _localizedValues[localeCode]['webhook'] ?? '';

  String get webhooks => _localizedValues[localeCode]['webhooks'] ?? '';

  String get newWebhook => _localizedValues[localeCode]['new_webhook'] ?? '';

  String get createdWebhook =>
      _localizedValues[localeCode]['created_webhook'] ?? '';

  String get updatedWebhook =>
      _localizedValues[localeCode]['updated_webhook'] ?? '';

  String get archivedWebhook =>
      _localizedValues[localeCode]['archived_webhook'] ?? '';

  String get deletedWebhook =>
      _localizedValues[localeCode]['deleted_webhook'] ?? '';

  String get restoredWebhook =>
      _localizedValues[localeCode]['restored_webhook'] ?? '';

  String get archivedWebhooks =>
      _localizedValues[localeCode]['archived_webhooks'] ?? '';

  String get deletedWebhooks =>
      _localizedValues[localeCode]['deleted_webhooks'] ?? '';

  String get restoredWebhooks =>
      _localizedValues[localeCode]['restored_webhooks'] ?? '';

  String get editWebhook => _localizedValues[localeCode]['edit_webhook'] ?? '';

  String get token => _localizedValues[localeCode]['token'] ?? '';

  String get tokens => _localizedValues[localeCode]['tokens'] ?? '';

  String get newToken => _localizedValues[localeCode]['new_token'] ?? '';

  String get createdToken =>
      _localizedValues[localeCode]['created_token'] ?? '';

  String get updatedToken =>
      _localizedValues[localeCode]['updated_token'] ?? '';

  String get archivedToken =>
      _localizedValues[localeCode]['archived_token'] ?? '';

  String get deletedToken =>
      _localizedValues[localeCode]['deleted_token'] ?? '';

  String get restoredToken =>
      _localizedValues[localeCode]['restored_token'] ?? '';

  String get archivedTokens =>
      _localizedValues[localeCode]['archived_tokens'] ?? '';

  String get deletedTokens =>
      _localizedValues[localeCode]['deleted_tokens'] ?? '';

  String get restoredTokens =>
      _localizedValues[localeCode]['restored_tokens'] ?? '';

  String get editToken => _localizedValues[localeCode]['edit_token'] ?? '';

  String get paymentTerm => _localizedValues[localeCode]['payment_term'] ?? '';

  String get newPaymentTerm =>
      _localizedValues[localeCode]['new_payment_term'] ?? '';

  String get createdPaymentTerm =>
      _localizedValues[localeCode]['created_payment_term'] ?? '';

  String get updatedPaymentTerm =>
      _localizedValues[localeCode]['updated_payment_term'] ?? '';

  String get archivedPaymentTerm =>
      _localizedValues[localeCode]['archived_payment_term'] ?? '';

  String get deletedPaymentTerm =>
      _localizedValues[localeCode]['deleted_payment_term'] ?? '';

  String get restoredPaymentTerm =>
      _localizedValues[localeCode]['restored_payment_term'] ?? '';

  String get archivedPaymentTerms =>
      _localizedValues[localeCode]['archived_payment_terms'] ?? '';

  String get deletedPaymentTerms =>
      _localizedValues[localeCode]['deleted_payment_terms'] ?? '';

  String get restoredPaymentTerms =>
      _localizedValues[localeCode]['restored_payment_terms'] ?? '';

  String get editPaymentTerm =>
      _localizedValues[localeCode]['edit_payment_term'] ?? '';

  String get designs => _localizedValues[localeCode]['designs'] ?? '';

  String get newDesign => _localizedValues[localeCode]['new_design'] ?? '';

  String get createdDesign =>
      _localizedValues[localeCode]['created_design'] ?? '';

  String get updatedDesign =>
      _localizedValues[localeCode]['updated_design'] ?? '';

  String get archivedDesign =>
      _localizedValues[localeCode]['archived_design'] ?? '';

  String get deletedDesign =>
      _localizedValues[localeCode]['deleted_design'] ?? '';

  String get restoredDesign =>
      _localizedValues[localeCode]['restored_design'] ?? '';

  String get archivedDesigns =>
      _localizedValues[localeCode]['archived_designs'] ?? '';

  String get deletedDesigns =>
      _localizedValues[localeCode]['deleted_designs'] ?? '';

  String get restoredDesigns =>
      _localizedValues[localeCode]['restored_designs'] ?? '';

  String get editDesign => _localizedValues[localeCode]['edit_design'] ?? '';

  String get newCredit => _localizedValues[localeCode]['new_credit'] ?? '';

  String get createdCredit =>
      _localizedValues[localeCode]['created_credit'] ?? '';

  String get updatedCredit =>
      _localizedValues[localeCode]['updated_credit'] ?? '';

  String get archivedCredit =>
      _localizedValues[localeCode]['archived_credit'] ?? '';

  String get deletedCredit =>
      _localizedValues[localeCode]['deleted_credit'] ?? '';

  String get restoredCredit =>
      _localizedValues[localeCode]['restored_credit'] ?? '';

  String get archivedCredits =>
      _localizedValues[localeCode]['archived_credits'] ?? '';

  String get deletedCredits =>
      _localizedValues[localeCode]['deleted_credits'] ?? '';

  String get restoredCredits =>
      _localizedValues[localeCode]['restored_credits'] ?? '';

  String get creditDate => _localizedValues[localeCode]['credit_date'] ?? '';

  String get accountManagement =>
      _localizedValues[localeCode]['account_management'] ?? '';

  String get proposals => _localizedValues[localeCode]['proposals'] ?? '';

  String get tickets => _localizedValues[localeCode]['tickets'] ?? '';

  String get recurringQuotes =>
      _localizedValues[localeCode]['recurring_quotes'] ?? '';

  String get recurringTasks =>
      _localizedValues[localeCode]['recurring_tasks'] ?? '';

  String get recurringExpenses =>
      _localizedValues[localeCode]['recurring_expenses'] ?? '';

  String get customDesigns =>
      _localizedValues[localeCode]['custom_designs'] ?? '';

  String get cssFramework =>
      _localizedValues[localeCode]['css_framework'] ?? '';

  String get loadDesign => _localizedValues[localeCode]['load_design'] ?? '';

  String get header => _localizedValues[localeCode]['header'] ?? '';

  String get includes => _localizedValues[localeCode]['includes'] ?? '';

  String get creditDesign =>
      _localizedValues[localeCode]['credit_design'] ?? '';

  String get partialPaymentEmail =>
      _localizedValues[localeCode]['partial_payment_email'] ?? '';

  String get partialPayment =>
      _localizedValues[localeCode]['partial_payment'] ?? '';

  String get convertedQuote =>
      _localizedValues[localeCode]['converted_quote'] ?? '';

  String get enabledModules =>
      _localizedValues[localeCode]['enabled_modules'] ?? '';

  String get cancelAccount =>
      _localizedValues[localeCode]['cancel_account'] ?? '';

  String get cancelAccountMessage =>
      _localizedValues[localeCode]['cancel_account_message'] ?? '';

  String get deleteCompany =>
      _localizedValues[localeCode]['delete_company'] ?? '';

  String get deleteCompanyMessage =>
      _localizedValues[localeCode]['delete_company_message'] ?? '';

  String get purchaseLicense =>
      _localizedValues[localeCode]['purchase_license'] ?? '';

  String get applyLicense =>
      _localizedValues[localeCode]['apply_license'] ?? '';

  String get receiveAllNotifications =>
      _localizedValues[localeCode]['receive_all_notifications'] ?? '';

  String get invoiceSent => _localizedValues[localeCode]['invoice_sent'] ?? '';

  String get invoiceViewed =>
      _localizedValues[localeCode]['invoice_viewed'] ?? '';

  String get paymentSuccess =>
      _localizedValues[localeCode]['payment_success'] ?? '';

  String get paymentFailure =>
      _localizedValues[localeCode]['payment_failure'] ?? '';

  String get quoteSent => _localizedValues[localeCode]['quote_sent'] ?? '';

  String get quoteViewed => _localizedValues[localeCode]['quote_viewed'] ?? '';

  String get quoteApproved =>
      _localizedValues[localeCode]['quote_approved'] ?? '';

  String get creditSent => _localizedValues[localeCode]['credit_sent'] ?? '';

  String get creditViewed =>
      _localizedValues[localeCode]['credit_viewed'] ?? '';

  String get none => _localizedValues[localeCode]['none'] ?? '';

  String get owned => _localizedValues[localeCode]['owned'] ?? '';

  String get permissions => _localizedValues[localeCode]['permissions'] ?? '';

  String get allEvents => _localizedValues[localeCode]['all_events'] ?? '';

  String get addField => _localizedValues[localeCode]['add_field'] ?? '';

  String get clientDetails =>
      _localizedValues[localeCode]['client_details'] ?? '';

  String get companyAddress =>
      _localizedValues[localeCode]['company_address'] ?? '';

  String get invoiceDetails =>
      _localizedValues[localeCode]['invoice_details'] ?? '';

  String get quoteDetails =>
      _localizedValues[localeCode]['quote_details'] ?? '';

  String get creditDetails =>
      _localizedValues[localeCode]['credit_details'] ?? '';

  String get productColumns =>
      _localizedValues[localeCode]['product_columns'] ?? '';

  String get taskColumns => _localizedValues[localeCode]['task_columns'] ?? '';

  String get cloneToCredit =>
      _localizedValues[localeCode]['clone_to_credit'] ?? '';

  String get savedDesign => _localizedValues[localeCode]['saved_design'] ?? '';

  String get refresh => _localizedValues[localeCode]['refresh'] ?? '';

  String get clientCity => _localizedValues[localeCode]['client_city'] ?? '';

  String get clientState => _localizedValues[localeCode]['client_state'] ?? '';

  String get clientCountry =>
      _localizedValues[localeCode]['client_country'] ?? '';

  String get clientIsActive =>
      _localizedValues[localeCode]['client_is_active'] ?? '';

  String get clientBalance =>
      _localizedValues[localeCode]['client_balance'] ?? '';

  String get clientAddress1 =>
      _localizedValues[localeCode]['client_address1'] ?? '';

  String get clientAddress2 =>
      _localizedValues[localeCode]['client_address2'] ?? '';

  String get vendorAddress1 =>
      _localizedValues[localeCode]['vendor_address1'] ?? '';

  String get vendorAddress2 =>
      _localizedValues[localeCode]['vendor_address2'] ?? '';

  String get type => _localizedValues[localeCode]['type'] ?? '';

  String get importType => _localizedValues[localeCode]['import_type'] ?? '';

  String get import => _localizedValues[localeCode]['import'] ?? '';

  String get invoiceAmount =>
      _localizedValues[localeCode]['invoice_amount'] ?? '';

  String get invoiceDueDate =>
      _localizedValues[localeCode]['invoice_due_date'] ?? '';

  String get taxRate1 => _localizedValues[localeCode]['tax_rate1'] ?? '';

  String get taxRate2 => _localizedValues[localeCode]['tax_rate2'] ?? '';

  String get taxRate3 => _localizedValues[localeCode]['tax_rate3'] ?? '';

  String get autoBill => _localizedValues[localeCode]['auto_bill'] ?? '';

  String get archivedAt => _localizedValues[localeCode]['archived_at'] ?? '';

  String get hasExpenses => _localizedValues[localeCode]['has_expenses'] ?? '';

  String get customTaxes1 =>
      _localizedValues[localeCode]['custom_taxes1'] ?? '';

  String get customTaxes2 =>
      _localizedValues[localeCode]['custom_taxes2'] ?? '';

  String get customTaxes3 =>
      _localizedValues[localeCode]['custom_taxes3'] ?? '';

  String get customTaxes4 =>
      _localizedValues[localeCode]['custom_taxes4'] ?? '';

  String get customSurcharge1 =>
      _localizedValues[localeCode]['custom_surcharge1'] ?? '';

  String get customSurcharge2 =>
      _localizedValues[localeCode]['custom_surcharge2'] ?? '';

  String get customSurcharge3 =>
      _localizedValues[localeCode]['custom_surcharge3'] ?? '';

  String get customSurcharge4 =>
      _localizedValues[localeCode]['custom_surcharge4'] ?? '';

  String get isDeleted => _localizedValues[localeCode]['is_deleted'] ?? '';

  String get vendorCity => _localizedValues[localeCode]['vendor_city'] ?? '';

  String get vendorState => _localizedValues[localeCode]['vendor_state'] ?? '';

  String get vendorCountry =>
      _localizedValues[localeCode]['vendor_country'] ?? '';

  String get isApproved => _localizedValues[localeCode]['is_approved'] ?? '';

  String get taxName => _localizedValues[localeCode]['tax_name'] ?? '';

  String get taxAmount => _localizedValues[localeCode]['tax_amount'] ?? '';

  String get taxPaid => _localizedValues[localeCode]['tax_paid'] ?? '';

  String get paymentAmount =>
      _localizedValues[localeCode]['payment_amount'] ?? '';

  String get age => _localizedValues[localeCode]['age'] ?? '';

  String get ageGroup0 => _localizedValues[localeCode]['age_group_0'] ?? '';

  String get ageGroup30 => _localizedValues[localeCode]['age_group_30'] ?? '';

  String get ageGroup60 => _localizedValues[localeCode]['age_group_60'] ?? '';

  String get ageGroup90 => _localizedValues[localeCode]['age_group_90'] ?? '';

  String get ageGroup120 => _localizedValues[localeCode]['age_group_120'] ?? '';

  String get invoiceBalance =>
      _localizedValues[localeCode]['invoice_balance'] ?? '';

  String get purgeData => _localizedValues[localeCode]['purge_data'] ?? '';

  String get purgeSuccessful =>
      _localizedValues[localeCode]['purge_successful'] ?? '';

  String get purgeDataMessage =>
      _localizedValues[localeCode]['purge_data_message'] ?? '';

  String get license => _localizedValues[localeCode]['license'] ?? '';

  String get optional => _localizedValues[localeCode]['optional'] ?? '';

  String get custom1 => _localizedValues[localeCode]['custom1'] ?? '';

  String get custom2 => _localizedValues[localeCode]['custom2'] ?? '';

  String get custom3 => _localizedValues[localeCode]['custom3'] ?? '';

  String get custom4 => _localizedValues[localeCode]['custom4'] ?? '';

  String get fullName => _localizedValues[localeCode]['full_name'] ?? '';

  String get cityStatePostal =>
      _localizedValues[localeCode]['city_state_postal'] ?? '';

  String get postalCityState =>
      _localizedValues[localeCode]['postal_city_state'] ?? '';

  String get reverse => _localizedValues[localeCode]['reverse'] ?? '';

  String get cancelledInvoice =>
      _localizedValues[localeCode]['cancelled_invoice'] ?? '';

  String get cancelledInvoices =>
      _localizedValues[localeCode]['cancelled_invoices'] ?? '';

  String get reversedInvoice =>
      _localizedValues[localeCode]['reversed_invoice'] ?? '';

  String get reversedInvoices =>
      _localizedValues[localeCode]['reversed_invoices'] ?? '';

  String get refundPayment =>
      _localizedValues[localeCode]['refund_payment'] ?? '';

  String get searchInvoices =>
      _localizedValues[localeCode]['search_invoices'] ?? '';

  String get searchClients =>
      _localizedValues[localeCode]['search_clients'] ?? '';

  String get searchProducts =>
      _localizedValues[localeCode]['search_products'] ?? '';

  String get searchQuotes =>
      _localizedValues[localeCode]['search_quotes'] ?? '';

  String get searchCredits =>
      _localizedValues[localeCode]['search_credits'] ?? '';

  String get searchVendors =>
      _localizedValues[localeCode]['search_vendors'] ?? '';

  String get searchUsers => _localizedValues[localeCode]['search_users'] ?? '';

  String get searchTaxRates =>
      _localizedValues[localeCode]['search_tax_rates'] ?? '';

  String get searchTasks => _localizedValues[localeCode]['search_tasks'] ?? '';

  String get searchSettings =>
      _localizedValues[localeCode]['search_settings'] ?? '';

  String get searchProjects =>
      _localizedValues[localeCode]['search_projects'] ?? '';

  String get searchExpenses =>
      _localizedValues[localeCode]['search_expenses'] ?? '';

  String get searchPayments =>
      _localizedValues[localeCode]['search_payments'] ?? '';

  String get searchGroups =>
      _localizedValues[localeCode]['search_groups'] ?? '';

  String get searchCompany =>
      _localizedValues[localeCode]['search_company'] ?? '';

  String get searchDocuments =>
      _localizedValues[localeCode]['search_documents'] ?? '';

  String get searchDesigns =>
      _localizedValues[localeCode]['search_designs'] ?? '';

  String get searchInvoice =>
      _localizedValues[localeCode]['search_invoice'] ?? '';

  String get searchClient =>
      _localizedValues[localeCode]['search_client'] ?? '';

  String get searchProduct =>
      _localizedValues[localeCode]['search_product'] ?? '';

  String get searchQuote => _localizedValues[localeCode]['search_quote'] ?? '';

  String get searchCredit =>
      _localizedValues[localeCode]['search_credit'] ?? '';

  String get searchVendor =>
      _localizedValues[localeCode]['search_vendor'] ?? '';

  String get searchUser => _localizedValues[localeCode]['search_user'] ?? '';

  String get searchTaxRate =>
      _localizedValues[localeCode]['search_tax_rate'] ?? '';

  String get searchTask => _localizedValues[localeCode]['search_task'] ?? '';

  String get searchProject =>
      _localizedValues[localeCode]['search_project'] ?? '';

  String get searchExpense =>
      _localizedValues[localeCode]['search_expense'] ?? '';

  String get searchPayment =>
      _localizedValues[localeCode]['search_payment'] ?? '';

  String get searchGroup => _localizedValues[localeCode]['search_group'] ?? '';

  String get searchDocument =>
      _localizedValues[localeCode]['search_document'] ?? '';

  String get searchDesign =>
      _localizedValues[localeCode]['search_design'] ?? '';

  String get searchToken => _localizedValues[localeCode]['search_token'] ?? '';

  String get searchWebhook =>
      _localizedValues[localeCode]['search_webhook'] ?? '';

  String get partiallyRefunded =>
      _localizedValues[localeCode]['partially_refunded'] ?? '';

  String get hideMenu => _localizedValues[localeCode]['hide_menu'] ?? '';

  String get showMenu => _localizedValues[localeCode]['show_menu'] ?? '';

  String get exclusive => _localizedValues[localeCode]['exclusive'] ?? '';

  String get inclusive => _localizedValues[localeCode]['inclusive'] ?? '';

  String get hosted => _localizedValues[localeCode]['hosted'] ?? '';

  String get selfhosted => _localizedValues[localeCode]['selfhosted'] ?? '';

  String get creditAmount =>
      _localizedValues[localeCode]['credit_amount'] ?? '';

  String get quoteAmount => _localizedValues[localeCode]['quote_amount'] ?? '';

  String get reversed => _localizedValues[localeCode]['reversed'] ?? '';

  String get cancelled => _localizedValues[localeCode]['cancelled'] ?? '';

  String get sendFromGmail =>
      _localizedValues[localeCode]['send_from_gmail'] ?? '';

  String get changeToMobileLayout =>
      _localizedValues[localeCode]['change_to_mobile_layout'] ?? '';

  String get changeToDekstopLayout =>
      _localizedValues[localeCode]['change_to_desktop_layout'] ?? '';

  String get change => _localizedValues[localeCode]['change'] ?? '';

  String get emailSignIn => _localizedValues[localeCode]['email_sign_in'] ?? '';

  String get configurePaymentTerms =>
      _localizedValues[localeCode]['configure_payment_terms'] ?? '';

  String get numberOfDays =>
      _localizedValues[localeCode]['number_of_days'] ?? '';

  String get reminderEndless =>
      _localizedValues[localeCode]['reminder_endless'] ?? '';

  String get useDefault => _localizedValues[localeCode]['use_default'] ?? '';

  String get contactName => _localizedValues[localeCode]['contact_name'] ?? '';

  String get creditRemaining =>
      _localizedValues[localeCode]['credit_remaining'] ?? '';

  String get allRecords => _localizedValues[localeCode]['all_records'] ?? '';

  String get ownedByUser => _localizedValues[localeCode]['owned_by_user'] ?? '';

  String get viewPdf => _localizedValues[localeCode]['view_pdf'] ?? '';

  String get ledger => _localizedValues[localeCode]['ledger'] ?? '';

  String get clientEmailNotSet =>
      _localizedValues[localeCode]['client_email_not_set'] ?? '';

  String get emailInvoice =>
      _localizedValues[localeCode]['email_invoice'] ?? '';

  String get emailQuote => _localizedValues[localeCode]['email_quote'] ?? '';

  String get emailCredit => _localizedValues[localeCode]['email_credit'] ?? '';

  String get emailPayment =>
      _localizedValues[localeCode]['email_payment'] ?? '';

  String get customizeAndPreview =>
      _localizedValues[localeCode]['customize_and_preview'] ?? '';

  String get clientRegistration =>
      _localizedValues[localeCode]['client_registration'] ?? '';

  String get clientRegistrationHelp =>
      _localizedValues[localeCode]['client_registration_help'] ?? '';

  String get documentUpload =>
      _localizedValues[localeCode]['document_upload'] ?? '';

  String get documentUploadHelp =>
      _localizedValues[localeCode]['document_upload_help'] ?? '';

  String get subtotal => _localizedValues[localeCode]['subtotal'] ?? '';

  String get searchTokens =>
      _localizedValues[localeCode]['search_tokens'] ?? '';

  String get searchWebhooks =>
      _localizedValues[localeCode]['search_webhooks'] ?? '';

  String get apiTokens => _localizedValues[localeCode]['api_tokens'] ?? '';

  String get apiDocs => _localizedValues[localeCode]['api_docs'] ?? '';

  String get apiWebhooks => _localizedValues[localeCode]['api_webhooks'] ?? '';

  String get cronsNotEnabled =>
      _localizedValues[localeCode]['crons_not_enabled'] ?? '';

  String get mustBeOnline =>
      _localizedValues[localeCode]['must_be_online'] ?? '';

  String get copy => _localizedValues[localeCode]['copy'] ?? '';

  String get targetUrl => _localizedValues[localeCode]['target_url'] ?? '';

  String get eventType => _localizedValues[localeCode]['event_type'] ?? '';

  String get showSidebar => _localizedValues[localeCode]['show_sidebar'] ?? '';

  String get hideSidebar => _localizedValues[localeCode]['hide_sidebar'] ?? '';

  String get plan => _localizedValues[localeCode]['plan'] ?? '';

  String get free => _localizedValues[localeCode]['free'] ?? '';

  String get expiresOn => _localizedValues[localeCode]['expires_on'] ?? '';

  String get off => _localizedValues[localeCode]['off'] ?? '';

  String get whenPaid => _localizedValues[localeCode]['when_paid'] ?? '';

  String get createClient =>
      _localizedValues[localeCode]['create_client'] ?? '';

  String get createInvoice =>
      _localizedValues[localeCode]['create_invoice'] ?? '';

  String get createQuote => _localizedValues[localeCode]['create_quote'] ?? '';

  String get createPayment =>
      _localizedValues[localeCode]['create_payment'] ?? '';

  String get createVendor =>
      _localizedValues[localeCode]['create_vendor'] ?? '';

  String get updateQuote => _localizedValues[localeCode]['update_quote'] ?? '';

  String get deleteQuote => _localizedValues[localeCode]['delete_quote'] ?? '';

  String get updateInvoice =>
      _localizedValues[localeCode]['update_invoice'] ?? '';

  String get deleteInvoice =>
      _localizedValues[localeCode]['delete_invoice'] ?? '';

  String get updateClient =>
      _localizedValues[localeCode]['update_client'] ?? '';

  String get deleteClient =>
      _localizedValues[localeCode]['delete_client'] ?? '';

  String get deletePayment =>
      _localizedValues[localeCode]['delete_payment'] ?? '';

  String get updateVendor =>
      _localizedValues[localeCode]['update_vendor'] ?? '';

  String get deleteVendor =>
      _localizedValues[localeCode]['delete_vendor'] ?? '';

  String get createExpense =>
      _localizedValues[localeCode]['create_expense'] ?? '';

  String get updateExpense =>
      _localizedValues[localeCode]['update_expense'] ?? '';

  String get deleteExpense =>
      _localizedValues[localeCode]['delete_expense'] ?? '';

  String get createTask => _localizedValues[localeCode]['create_task'] ?? '';

  String get updateTask => _localizedValues[localeCode]['update_task'] ?? '';

  String get deleteTask => _localizedValues[localeCode]['delete_task'] ?? '';

  String get approveQuote =>
      _localizedValues[localeCode]['approve_quote'] ?? '';

  String get upcomingInvoices =>
      _localizedValues[localeCode]['upcoming_invoices'] ?? '';

  String get pastDueInvoices =>
      _localizedValues[localeCode]['past_due_invoices'] ?? '';

  String get recentPayments =>
      _localizedValues[localeCode]['recent_payments'] ?? '';

  String get upcomingQuotes =>
      _localizedValues[localeCode]['upcoming_quotes'] ?? '';

  String get expiredQuotes =>
      _localizedValues[localeCode]['expired_quotes'] ?? '';

  String get selectedInvoices =>
      _localizedValues[localeCode]['selected_invoices'] ?? '';

  String get selectedPayments =>
      _localizedValues[localeCode]['selected_payments'] ?? '';

  String get selectedQuotes =>
      _localizedValues[localeCode]['selected_quotes'] ?? '';

  String get selectedTasks =>
      _localizedValues[localeCode]['selected_tasks'] ?? '';

  String get selectedExpenses =>
      _localizedValues[localeCode]['selected_expenses'] ?? '';

  String get clientSettings =>
      _localizedValues[localeCode]['client_settings'] ?? '';

  String get netAmount => _localizedValues[localeCode]['net_amount'] ?? '';

  String get netBalance => _localizedValues[localeCode]['net_balance'] ?? '';

  String get gross => _localizedValues[localeCode]['gross'] ?? '';

  String get completed => _localizedValues[localeCode]['completed'] ?? '';

  String get onlinePaymentEmail =>
      _localizedValues[localeCode]['online_payment_email'] ?? '';

  String get manualPaymentEmail =>
      _localizedValues[localeCode]['manual_payment_email'] ?? '';

  String get clientCreated =>
      _localizedValues[localeCode]['client_created'] ?? '';

  String get storefront => _localizedValues[localeCode]['storefront'] ?? '';

  String get storefrontHelp =>
      _localizedValues[localeCode]['storefront_help'] ?? '';

  String get companyKey => _localizedValues[localeCode]['company_key'] ?? '';

  String get lastLoginAt => _localizedValues[localeCode]['last_login_at'] ?? '';

  String get paymenTypeId =>
      _localizedValues[localeCode]['payment_type_id'] ?? '';

  String get healthCheck => _localizedValues[localeCode]['health_check'] ?? '';

  String get to => _localizedValues[localeCode]['to'] ?? '';

  String get recordType => _localizedValues[localeCode]['record_type'] ?? '';

  String get recordName => _localizedValues[localeCode]['record_name'] ?? '';

  String get fileType => _localizedValues[localeCode]['file_type'] ?? '';

  String get height => _localizedValues[localeCode]['height'] ?? '';

  String get width => _localizedValues[localeCode]['width'] ?? '';

  String get customLabels =>
      _localizedValues[localeCode]['custom_labels'] ?? '';

  String get selectLabel => _localizedValues[localeCode]['select_label'] ?? '';

  String get unapplied => _localizedValues[localeCode]['unapplied'] ?? '';

  String get apply => _localizedValues[localeCode]['apply'] ?? '';

  String get applyPayment =>
      _localizedValues[localeCode]['apply_payment'] ?? '';

  String get surcharge => _localizedValues[localeCode]['surcharge'] ?? '';

  String get hours => _localizedValues[localeCode]['hours'] ?? '';

  String get statement => _localizedValues[localeCode]['statement'] ?? '';

  String get taxes => _localizedValues[localeCode]['taxes'] ?? '';

  String get rowsPerPage => _localizedValues[localeCode]['rows_per_page'] ?? '';

  String get viewInStripe =>
      _localizedValues[localeCode]['view_in_stripe'] ?? '';

  String get gateway => _localizedValues[localeCode]['gateway'] ?? '';

  String get emailedInvoices =>
      _localizedValues[localeCode]['emailed_invoices'] ?? '';

  String get emailedQuotes =>
      _localizedValues[localeCode]['emailed_quotes'] ?? '';

  String get emailedCredits =>
      _localizedValues[localeCode]['emailed_credits'] ?? '';

  String get pdfPageInfo => _localizedValues[localeCode]['pdf_page_info'] ?? '';

  String get reminder1Sent =>
      _localizedValues[localeCode]['reminder1_sent'] ?? '';

  String get reminder2Sent =>
      _localizedValues[localeCode]['reminder2_sent'] ?? '';

  String get reminder3Sent =>
      _localizedValues[localeCode]['reminder3_sent'] ?? '';

  String get reminderLastSent =>
      _localizedValues[localeCode]['reminder_last_sent'] ?? '';

  String get companyName => _localizedValues[localeCode]['company_name'] ?? '';

  String get clientNumber =>
      _localizedValues[localeCode]['client_number'] ?? '';

  String get autoConvert => _localizedValues[localeCode]['auto_convert'] ?? '';

  String get label => _localizedValues[localeCode]['label'] ?? '';

  String get always => _localizedValues[localeCode]['always'] ?? '';

  String get optIn => _localizedValues[localeCode]['optin'] ?? '';

  String get optOut => _localizedValues[localeCode]['optout'] ?? '';

  String get welcomeToInvoiceNinja =>
      _localizedValues[localeCode]['welcome_to_invoice_ninja'] ?? '';

  String get tokenBilling =>
      _localizedValues[localeCode]['token_billing'] ?? '';

  String get copyLink => _localizedValues[localeCode]['copy_link'] ?? '';

  String get viewPortal => _localizedValues[localeCode]['view_portal'] ?? '';

  String get systemLogs => _localizedValues[localeCode]['system_logs'] ?? '';

  String get paymentReconciliationSuccess =>
      _localizedValues[localeCode]['payment_reconciliation_success'] ?? '';

  String get paymentReconciliationFailure =>
      _localizedValues[localeCode]['payment_reconciliation_failure'] ?? '';

  String get gatewaySuccess =>
      _localizedValues[localeCode]['gateway_success'] ?? '';

  String get gatewayFailure =>
      _localizedValues[localeCode]['gateway_failure'] ?? '';

  String get gatewayError =>
      _localizedValues[localeCode]['gateway_error'] ?? '';

  String get emailSend => _localizedValues[localeCode]['email_send'] ?? '';

  String get emailRetryQueue =>
      _localizedValues[localeCode]['email_retry_queue'] ?? '';

  String get failure => _localizedValues[localeCode]['failure'] ?? '';

  String get quotaExceeded =>
      _localizedValues[localeCode]['quota_exceeded'] ?? '';

  String get upstreamFailure =>
      _localizedValues[localeCode]['upstream_failure'] ?? '';

  String get opened => _localizedValues[localeCode]['opened'] ?? '';

  String get testMode => _localizedValues[localeCode]['test_mode'] ?? '';

  String get allowOverPayment =>
      _localizedValues[localeCode]['allow_over_payment'] ?? '';

  String get allowOverPaymentHelp =>
      _localizedValues[localeCode]['allow_over_payment_help'] ?? '';

  String get allowUnderPayment =>
      _localizedValues[localeCode]['allow_under_payment'] ?? '';

  String get allowUnderPaymentHelp =>
      _localizedValues[localeCode]['allow_under_payment_help'] ?? '';

  String get lineItem => _localizedValues[localeCode]['line_item'] ?? '';

  String get profit => _localizedValues[localeCode]['profit'] ?? '';

  String get sendDate => _localizedValues[localeCode]['send_date'] ?? '';

  String get minimumUnderPaymentAmount =>
      _localizedValues[localeCode]['minimum_under_payment_amount'] ?? '';

  String get autoBillOn => _localizedValues[localeCode]['auto_bill_on'] ?? '';

  String get nextSendDate =>
      _localizedValues[localeCode]['next_send_date'] ?? '';

  String get remainingCycles =>
      _localizedValues[localeCode]['remaining_cycles'] ?? '';

  String get endless => _localizedValues[localeCode]['endless'] ?? '';

  String get usePaymentTerms =>
      _localizedValues[localeCode]['use_payment_terms'] ?? '';

  String get firstDayOfTheMonth =>
      _localizedValues[localeCode]['first_day_of_the_month'] ?? '';

  String get lastDayOfTheMonth =>
      _localizedValues[localeCode]['last_day_of_the_month'] ?? '';

  String get dayCount => _localizedValues[localeCode]['day_count'] ?? '';

  String get markActive => _localizedValues[localeCode]['mark_active'] ?? '';

  String get paused => _localizedValues[localeCode]['paused'] ?? '';

  String get dueDateDays => _localizedValues[localeCode]['due_date_days'] ?? '';

  String get gatewayRefund =>
      _localizedValues[localeCode]['gateway_refund'] ?? '';

  String get gatewayRefundHelp =>
      _localizedValues[localeCode]['gateway_refund_help'] ?? '';

  String get totalFields => _localizedValues[localeCode]['total_fields'] ?? '';

  String get totalTaxes => _localizedValues[localeCode]['total_taxes'] ?? '';

  String get lineTaxes => _localizedValues[localeCode]['line_taxes'] ?? '';

  String get captureCard => _localizedValues[localeCode]['capture_card'] ?? '';

  String get autoBillEnabled =>
      _localizedValues[localeCode]['auto_bill_enabled'] ?? '';

  String get copyError => _localizedValues[localeCode]['copy_error'] ?? '';

  String get showPassword =>
      _localizedValues[localeCode]['show_password'] ?? '';

  String get hidePassword =>
      _localizedValues[localeCode]['hide_password'] ?? '';

  String get variables => _localizedValues[localeCode]['variables'] ?? '';

  String get userField => _localizedValues[localeCode]['user_field'] ?? '';

  String get paypal => _localizedValues[localeCode]['paypal'] ?? '';

  String get alipay => _localizedValues[localeCode]['alipay'] ?? '';

  String get sofort => _localizedValues[localeCode]['sofort'] ?? '';

  String get applePay => _localizedValues[localeCode]['apple_pay'] ?? '';

  String get crypto => _localizedValues[localeCode]['crypto'] ?? '';

  String get markPaidHelp =>
      _localizedValues[localeCode]['mark_paid_help'] ?? '';

  String get addDocumentsToInvoiceHelp =>
      _localizedValues[localeCode]['add_documents_to_invoice_help'] ?? '';

  String get expenseSettings =>
      _localizedValues[localeCode]['expense_settings'] ?? '';

  String get convertCurrencyHelp =>
      _localizedValues[localeCode]['convert_currency_help'] ?? '';

  String get cloneToRecurring =>
      _localizedValues[localeCode]['clone_to_recurring'] ?? '';

  String get forceUpdate => _localizedValues[localeCode]['force_update'] ?? '';

  String get forceUpdateHelp =>
      _localizedValues[localeCode]['force_update_help'] ?? '';

  String get negativePaymentError =>
      _localizedValues[localeCode]['negative_payment_error'] ?? '';

  String get viewChanges => _localizedValues[localeCode]['view_changes'] ?? '';

  String get showOption => _localizedValues[localeCode]['show_option'] ?? '';

  String get useAvailableCredits =>
      _localizedValues[localeCode]['use_available_credits'] ?? '';

  String get shouldBeInvoiced =>
      _localizedValues[localeCode]['should_be_invoiced'] ?? '';

  String get shouldBeInvoicedHelp =>
      _localizedValues[localeCode]['should_be_invoiced_help'] ?? '';

  String get isRunning => _localizedValues[localeCode]['is_running'] ?? '';

  String get timeLog => _localizedValues[localeCode]['time_log'] ?? '';

  String get bankId => _localizedValues[localeCode]['bank_id'] ?? '';

  String get expenseCategoryId =>
      _localizedValues[localeCode]['expense_category_id'] ?? '';

  String get invoiceCurrencyId =>
      _localizedValues[localeCode]['invoice_currency_id'] ?? '';

  String get taxName1 => _localizedValues[localeCode]['tax_name1'] ?? '';

  String get taxName2 => _localizedValues[localeCode]['tax_name2'] ?? '';

  String get taxName3 => _localizedValues[localeCode]['tax_name3'] ?? '';

  String get transactionId =>
      _localizedValues[localeCode]['transaction_id'] ?? '';

  String get configureCategories =>
      _localizedValues[localeCode]['configure_categories'] ?? '';

  String get configureStatuses =>
      _localizedValues[localeCode]['configure_statuses'] ?? '';

  String get taskSettings =>
      _localizedValues[localeCode]['task_settings'] ?? '';

  String get invoiceTaskTimelog =>
      _localizedValues[localeCode]['invoice_task_timelog'] ?? '';

  String get invoiceTaskTimelogHelp =>
      _localizedValues[localeCode]['invoice_task_timelog_help'] ?? '';

  String get invoiceTaskDatelog =>
      _localizedValues[localeCode]['invoice_task_datelog'] ?? '';

  String get invoiceTaskDatelogHelp =>
      _localizedValues[localeCode]['invoice_task_datelog_help'] ?? '';

  String get showTasksTable =>
      _localizedValues[localeCode]['show_tasks_table'] ?? '';

  String get showTasksTableHelp =>
      _localizedValues[localeCode]['show_tasks_table_help'] ?? '';

  String get sortOrder => _localizedValues[localeCode]['sort_order'] ?? '';

  String get clearCache => _localizedValues[localeCode]['clear_cache'] ?? '';

  String get calculatedRate =>
      _localizedValues[localeCode]['calculated_rate'] ?? '';

  String get requiredFields =>
      _localizedValues[localeCode]['required_fields'] ?? '';

  String get clientName => _localizedValues[localeCode]['client_name'] ?? '';

  String get clientPhone => _localizedValues[localeCode]['client_phone'] ?? '';

  String get cvv => _localizedValues[localeCode]['cvv'] ?? '';

  String get lateInvoice => _localizedValues[localeCode]['late_invoice'] ?? '';

  String get expiredQuote =>
      _localizedValues[localeCode]['expired_quote'] ?? '';

  String get remindInvoice =>
      _localizedValues[localeCode]['remind_invoice'] ?? '';

  String get companyDisabledWarning =>
      _localizedValues[localeCode]['company_disabled_warning'] ?? '';

  String get viewSettings =>
      _localizedValues[localeCode]['view_settings'] ?? '';

  String get warning => _localizedValues[localeCode]['warning'] ?? '';

  String get projectNumber =>
      _localizedValues[localeCode]['project_number'] ?? '';

  String get projectName => _localizedValues[localeCode]['project_name'] ?? '';

  String get taskNumber => _localizedValues[localeCode]['task_number'] ?? '';

  String get expenseNumber =>
      _localizedValues[localeCode]['expense_number'] ?? '';

  String get actions => _localizedValues[localeCode]['actions'] ?? '';

  String get invoiceTotal =>
      _localizedValues[localeCode]['invoice_total'] ?? '';

  String get quoteTotal => _localizedValues[localeCode]['quote_total'] ?? '';

  String get creditTotal => _localizedValues[localeCode]['credit_total'] ?? '';

  String get recurringInvoiceTotal =>
      _localizedValues[localeCode]['recurring_invoice_total'] ?? '';

  String get partialDue => _localizedValues[localeCode]['partial_due'] ?? '';

  String get invoiceLate => _localizedValues[localeCode]['invoice_late'] ?? '';

  String get quoteExpired =>
      _localizedValues[localeCode]['quote_expired'] ?? '';

  String get appPlatforms =>
      _localizedValues[localeCode]['app_platforms'] ?? '';

  String get sourceCode => _localizedValues[localeCode]['source_code'] ?? '';

  String get paidInvoicesArelocked =>
      _localizedValues[localeCode]['paid_invoices_are_locked'] ?? '';

  String get sentInvoicesArelocked =>
      _localizedValues[localeCode]['sent_invoices_are_locked'] ?? '';

  String get deliveryNote =>
      _localizedValues[localeCode]['delivery_note'] ?? '';

  String get whiteLabel => _localizedValues[localeCode]['white_label'] ?? '';

  String get unpaid => _localizedValues[localeCode]['unpaid'] ?? '';

  String get paymentTax => _localizedValues[localeCode]['payment_tax'] ?? '';

  String get addCustom => _localizedValues[localeCode]['add_custom'] ?? '';

  String get labels => _localizedValues[localeCode]['labels'] ?? '';

  String get cloneToOther =>
      _localizedValues[localeCode]['clone_to_other'] ?? '';

  String get cloneTo => _localizedValues[localeCode]['clone_to'] ?? '';

  String get service => _localizedValues[localeCode]['service'] ?? '';

  String get purge => _localizedValues[localeCode]['purge'] ?? '';

  String get pleaseTypeToConfirm =>
      _localizedValues[localeCode]['please_type_to_confirm'] ?? '';

  String get fullscreenEditor =>
      _localizedValues[localeCode]['fullscreen_editor'] ?? '';

  String get sidebarEditor =>
      _localizedValues[localeCode]['sidebar_editor'] ?? '';

  String get webhookUrl => _localizedValues[localeCode]['webhook_url'] ?? '';

  String get viewLicenses =>
      _localizedValues[localeCode]['view_licenses'] ?? '';

  String get csvFile => _localizedValues[localeCode]['csv_file'] ?? '';

  String get noFileSelected =>
      _localizedValues[localeCode]['no_file_selected'] ?? '';

  String get firstRowAsColumnNames =>
      _localizedValues[localeCode]['first_row_as_column_names'] ?? '';

  String get column => _localizedValues[localeCode]['column'] ?? '';

  String get sample => _localizedValues[localeCode]['sample'] ?? '';

  String get mapTo => _localizedValues[localeCode]['map_to'] ?? '';

  String get usesInclusiveTaxes =>
      _localizedValues[localeCode]['uses_inclusive_taxes'] ?? '';

  String get isAmountDiscount =>
      _localizedValues[localeCode]['is_amount_discount'] ?? '';

  String get duplicateColumnMapping =>
      _localizedValues[localeCode]['duplicate_column_mapping'] ?? '';

  String get startedImport =>
      _localizedValues[localeCode]['started_import'] ?? '';

  String get updateApp => _localizedValues[localeCode]['update_app'] ?? '';

  String get runningTasks =>
      _localizedValues[localeCode]['running_tasks'] ?? '';

  String get recentTasks => _localizedValues[localeCode]['recent_tasks'] ?? '';

  String get recentExpenses =>
      _localizedValues[localeCode]['recent_expenses'] ?? '';

  String get upcomingExpenses =>
      _localizedValues[localeCode]['upcoming_expenses'] ?? '';

  String get debugModeIsEnabled =>
      _localizedValues[localeCode]['debug_mode_is_enabled'] ?? '';

  String get debugModeIsEnabledHelp =>
      _localizedValues[localeCode]['debug_mode_is_enabled_help'] ?? '';

  String get emptyColumns =>
      _localizedValues[localeCode]['empty_columns'] ?? '';

  String get show => _localizedValues[localeCode]['show'] ?? '';

  String get hide => _localizedValues[localeCode]['hide'] ?? '';

  String get color => _localizedValues[localeCode]['color'] ?? '';

  String get enterTaxes => _localizedValues[localeCode]['enter_taxes'] ?? '';

  String get byRate => _localizedValues[localeCode]['by_rate'] ?? '';

  String get byAmount => _localizedValues[localeCode]['by_amount'] ?? '';

  String get enterAmount => _localizedValues[localeCode]['enter_amount'] ?? '';

  String get beforeTaxes => _localizedValues[localeCode]['before_taxes'] ?? '';

  String get afterTaxes => _localizedValues[localeCode]['after_taxes'] ?? '';

  String get isSent => _localizedValues[localeCode]['is_sent'] ?? '';

  String get defaultDocuments =>
      _localizedValues[localeCode]['default_documents'] ?? '';

  String get convertedAmount =>
      _localizedValues[localeCode]['converted_amount'] ?? '';

  String get convertedBalance =>
      _localizedValues[localeCode]['converted_balance'] ?? '';

  String get supportedEvents =>
      _localizedValues[localeCode]['supported_events'] ?? '';

  String get saveAndPreview =>
      _localizedValues[localeCode]['save_and_preview'] ?? '';

  String get saveAndEmail =>
      _localizedValues[localeCode]['save_and_email'] ?? '';

  String get searchPaymentTerm =>
      _localizedValues[localeCode]['search_payment_term'] ?? '';

  String get searchPaymentTerms =>
      _localizedValues[localeCode]['search_payment_terms'] ?? '';

  String get invoiceTask => _localizedValues[localeCode]['invoice_task'] ?? '';

  String get invoiceProject =>
      _localizedValues[localeCode]['invoice_project'] ?? '';

  String get invoiceExpense =>
      _localizedValues[localeCode]['invoice_expense'] ?? '';

  String get registrationUrl =>
      _localizedValues[localeCode]['registration_url'] ?? '';

  String get convertToInvoice =>
      _localizedValues[localeCode]['convert_to_invoice'] ?? '';

  String get toUpdateRun => _localizedValues[localeCode]['to_update_run'] ?? '';

  String get thisQuarter => _localizedValues[localeCode]['this_quarter'] ?? '';

  String get lastQuarter => _localizedValues[localeCode]['last_quarter'] ?? '';

  String get counterPatternError =>
      _localizedValues[localeCode]['counter_pattern_error'] ?? '';

  String get convertedPaidToDate =>
      _localizedValues[localeCode]['converted_paid_to_date'] ?? '';

  String get convertedCreditBalance =>
      _localizedValues[localeCode]['converted_credit_balance'] ?? '';

  String get convertedTotal =>
      _localizedValues[localeCode]['converted_total'] ?? '';

  String get emailSentToConfirmEmail =>
      _localizedValues[localeCode]['email_sent_to_confirm_email'] ?? '';

  String get listLongPress =>
      _localizedValues[localeCode]['list_long_press'] ?? '';

  String get showActions => _localizedValues[localeCode]['show_actions'] ?? '';

  String get startMultiselect =>
      _localizedValues[localeCode]['start_multiselect'] ?? '';

  String get selectAGmailUser =>
      _localizedValues[localeCode]['select_a_gmail_user'] ?? '';

  String get partiallyUnapplied =>
      _localizedValues[localeCode]['partially_unapplied'] ?? '';

  String get refundedPayment =>
      _localizedValues[localeCode]['refunded_payment'] ?? '';

  String get confirmYourEmailAddress =>
      _localizedValues[localeCode]['confirm_your_email_address'] ?? '';

  String get resendEmail => _localizedValues[localeCode]['resend_email'] ?? '';

  String get securitySettings =>
      _localizedValues[localeCode]['security_settings'] ?? '';

  String get webSessionTimeout =>
      _localizedValues[localeCode]['web_session_timeout'] ?? '';

  String get countHours => _localizedValues[localeCode]['count_hours'] ?? '';

  String get countDays => _localizedValues[localeCode]['count_days'] ?? '';

  String get countDay => _localizedValues[localeCode]['count_day'] ?? '';

  String get sessionAboutToExpire =>
      _localizedValues[localeCode]['session_about_to_expire'] ?? '';

  String get stayLoggedIn =>
      _localizedValues[localeCode]['stay_logged_in'] ?? '';

  String get requirePasswordWithSocialLogin =>
      _localizedValues[localeCode]['require_password_with_social_login'] ?? '';

  String get enableTwoFactor =>
      _localizedValues[localeCode]['enable_two_factor'] ?? '';

  String get disableTwoFactor =>
      _localizedValues[localeCode]['disable_two_factor'] ?? '';

  String get disabledTwoFactor =>
      _localizedValues[localeCode]['disabled_two_factor'] ?? '';

  String get connectGoogle =>
      _localizedValues[localeCode]['connect_google'] ?? '';

  String get disconnectGoogle =>
      _localizedValues[localeCode]['disconnect_google'] ?? '';

  String get twoFactorSetupHelp =>
      _localizedValues[localeCode]['two_factor_setup_help'] ?? '';

  String get enabledTwoFactor =>
      _localizedValues[localeCode]['enabled_two_factor'] ?? '';

  String get smsCode => _localizedValues[localeCode]['sms_code'] ?? '';

  String get sendSms => _localizedValues[localeCode]['send_sms'] ?? '';

  String get enterPhoneToEnableTwoFactor =>
      _localizedValues[localeCode]['enter_phone_to_enable_two_factor'] ?? '';

  String get viewDocs => _localizedValues[localeCode]['view_docs'] ?? '';

  String get delivered => _localizedValues[localeCode]['delivered'] ?? '';

  String get bounced => _localizedValues[localeCode]['bounced'] ?? '';

  String get spam => _localizedValues[localeCode]['spam'] ?? '';

  String get connectedGoogle =>
      _localizedValues[localeCode]['connected_google'] ?? '';

  String get disconnectedGoogle =>
      _localizedValues[localeCode]['disconnected_google'] ?? '';

  String get resendInvite =>
      _localizedValues[localeCode]['resend_invite'] ?? '';

  String get pleaseSelectACountry =>
      _localizedValues[localeCode]['please_select_a_country'] ?? '';

  String get helpTranslate =>
      _localizedValues[localeCode]['help_translate'] ?? '';

  String get changingPhoneDisablesTwoFactor =>
      _localizedValues[localeCode]['changing_phone_disables_two_factor'] ?? '';

  String get pleaseFirstSetAPassword =>
      _localizedValues[localeCode]['please_first_set_a_password'] ?? '';

  String get anErrorOccurredTryAgain =>
      _localizedValues[localeCode]['an_error_occurred_try_again'] ?? '';

  String get activateCompany =>
      _localizedValues[localeCode]['activate_company'] ?? '';

  String get activateCompanyHelp =>
      _localizedValues[localeCode]['activate_company_help'] ?? '';

  String get useLastEmail =>
      _localizedValues[localeCode]['use_last_email'] ?? '';

  String get sharedInvoiceCreditCounter =>
      _localizedValues[localeCode]['shared_invoice_credit_counter'] ?? '';

  String get passwordTimeout =>
      _localizedValues[localeCode]['password_timeout'] ?? '';

  String get countMinutes =>
      _localizedValues[localeCode]['count_minutes'] ?? '';

  String get clientIdNumber =>
      _localizedValues[localeCode]['client_id_number'] ?? '';

  String get updateFailHelp =>
      _localizedValues[localeCode]['update_fail_help'] ?? '';

  String get connectGmail =>
      _localizedValues[localeCode]['connect_gmail'] ?? '';

  String get disconnectGmail =>
      _localizedValues[localeCode]['disconnect_gmail'] ?? '';

  String get connectedGmail =>
      _localizedValues[localeCode]['connected_gmail'] ?? '';

  String get disconnectedGmail =>
      _localizedValues['en']['disconnected_gmail'] ?? '';

  String get subdomainIsNotAvailable =>
      _localizedValues[localeCode]['subdomain_is_not_available'] ?? '';

  String get subscription => _localizedValues[localeCode]['subscription'] ?? '';

  String get subscriptions =>
      _localizedValues[localeCode]['subscriptions'] ?? '';

  String get newSubscription =>
      _localizedValues[localeCode]['new_subscription'] ?? '';

  String get createdSubscription =>
      _localizedValues[localeCode]['created_subscription'] ?? '';

  String get updatedSubscription =>
      _localizedValues[localeCode]['updated_subscription'] ?? '';

  String get archivedSubscription =>
      _localizedValues[localeCode]['archived_subscription'] ?? '';

  String get deletedSubscription =>
      _localizedValues[localeCode]['deleted_subscription'] ?? '';

  String get restoredSubscription =>
      _localizedValues[localeCode]['restored_subscription'] ?? '';

  String get editSubscription =>
      _localizedValues[localeCode]['edit_subscription'] ?? '';

  String get searchSubscription =>
      _localizedValues[localeCode]['search_subscription'] ?? '';

  String get searchSubscriptions =>
      _localizedValues[localeCode]['search_subscriptions'] ?? '';

  String get security => _localizedValues[localeCode]['security'] ?? '';

  String get emailBounced =>
      _localizedValues[localeCode]['email_bounced'] ?? '';

  String get emailSpamComplaint =>
      _localizedValues[localeCode]['email_spam_complaint'] ?? '';

  String get emailDelivery =>
      _localizedValues[localeCode]['email_delivery'] ?? '';

  String get webhookResponse =>
      _localizedValues[localeCode]['webhook_response'] ?? '';

  String get pdfResponse => _localizedValues[localeCode]['pdf_response'] ?? '';

  String get authenticationFailure =>
      _localizedValues[localeCode]['authentication_failure'] ?? '';

  String get pdfFailed => _localizedValues[localeCode]['pdf_failed'] ?? '';

  String get pdfSuccess => _localizedValues[localeCode]['pdf_success'] ?? '';

  String get modified => _localizedValues[localeCode]['modified'] ?? '';

  String get recurringProducts =>
      _localizedValues[localeCode]['recurring_products'] ?? '';

  String get promoCode => _localizedValues[localeCode]['promo_code'] ?? '';

  String get promoDiscount =>
      _localizedValues[localeCode]['promo_discount'] ?? '';

  String get allowCancellation =>
      _localizedValues[localeCode]['allow_cancellation'] ?? '';

  String get perSeatEnabled =>
      _localizedValues[localeCode]['per_seat_enabled'] ?? '';

  String get maxSeatsLimit =>
      _localizedValues[localeCode]['max_seats_limit'] ?? '';

  String get trialEnabled =>
      _localizedValues[localeCode]['trial_enabled'] ?? '';

  String get trialDuration =>
      _localizedValues[localeCode]['trial_duration'] ?? '';

  String get allowQueryOverrides =>
      _localizedValues[localeCode]['allow_query_overrides'] ?? '';

  String get allowPlanChanges =>
      _localizedValues[localeCode]['allow_plan_changes'] ?? '';

  String get planMap => _localizedValues[localeCode]['plan_map'] ?? '';

  String get refundPeriod =>
      _localizedValues[localeCode]['refund_period'] ?? '';

  String get webhookConfiguration =>
      _localizedValues[localeCode]['webhook_configuration'] ?? '';

  String get purchasePage =>
      _localizedValues[localeCode]['purchase_page'] ?? '';

  String get returnUrl => _localizedValues[localeCode]['return_url'] ?? '';

  String get restMethod => _localizedValues[localeCode]['rest_method'] ?? '';

  String get headerKey => _localizedValues[localeCode]['header_key'] ?? '';

  String get invoicePaymentTerms =>
      _localizedValues[localeCode]['invoice_payment_terms'] ?? '';

  String get quoteValidUntil =>
      _localizedValues[localeCode]['quote_valid_until'] ?? '';

  String get headerValue => _localizedValues[localeCode]['header_value'] ?? '';

  String get addHeader => _localizedValues[localeCode]['add_header'] ?? '';

  String get removeHeader =>
      _localizedValues[localeCode]['remove_header'] ?? '';

  String get noHeaders => _localizedValues[localeCode]['no_headers'] ?? '';

  String get upgradeToPaidPlan =>
      _localizedValues[localeCode]['upgrade_to_paid_plan'] ?? '';

  String get ownerUpgradeToPaidPlan =>
      _localizedValues[localeCode]['owner_upgrade_to_paid_plan'] ?? '';

  String get pageLayout => _localizedValues[localeCode]['page_layout'] ?? '';

  String get portrait => _localizedValues[localeCode]['portrait'] ?? '';

  String get landscape => _localizedValues[localeCode]['landscape'] ?? '';

  String get letter => _localizedValues[localeCode]['letter'] ?? '';

  String get legal => _localizedValues[localeCode]['legal'] ?? '';

  String get isViewed => _localizedValues[localeCode]['is_viewed'] ?? '';

  String get viewDateFormats =>
      _localizedValues[localeCode]['view_date_formats'] ?? '';

  String get addBodyVariableMessage =>
      _localizedValues[localeCode]['add_body_variable_message'] ?? '';

  String get bodyVariableMissing =>
      _localizedValues[localeCode]['body_variable_missing'] ?? '';

  String get subdomainHelp =>
      _localizedValues[localeCode]['subdomain_help'] ?? '';

  String get moveTop => _localizedValues[localeCode]['move_top'] ?? '';

  String get moveUp => _localizedValues[localeCode]['move_up'] ?? '';

  String get moveDown => _localizedValues[localeCode]['move_down'] ?? '';

  String get moveBottom => _localizedValues[localeCode]['move_bottom'] ?? '';

  String get enable => _localizedValues[localeCode]['enable'] ?? '';

  String get kanban =>
      _localizedValues[localeCode]['kanban'] ??
      _localizedValues['en']['kanban'];

  String get searchKanban =>
      _localizedValues[localeCode]['search_kanban'] ??
      _localizedValues['en']['search_kanban'];

  String get searchKanbans =>
      _localizedValues[localeCode]['search_kanbans'] ??
      _localizedValues['en']['search_kanbans'];

  String get partialValue =>
      _localizedValues[localeCode]['partial_value'] ??
      _localizedValues['en']['partial_value'];

  String get unassigned =>
      _localizedValues[localeCode]['unassigned'] ??
      _localizedValues['en']['unassigned'];

  String get order =>
      _localizedValues[localeCode]['order'] ?? _localizedValues['en']['order'];

  String get contactFirstName =>
      _localizedValues[localeCode]['contact_first_name'] ??
      _localizedValues['en']['contact_first_name'];

  String get contactLastName =>
      _localizedValues[localeCode]['contact_last_name'] ??
      _localizedValues['en']['contact_last_name'];

  String get invoiceItem =>
      _localizedValues[localeCode]['invoice_item'] ??
      _localizedValues['en']['invoice_item'];

  String get quoteItem =>
      _localizedValues[localeCode]['quote_item'] ??
      _localizedValues['en']['quote_item'];

  String get lastUpdated =>
      _localizedValues[localeCode]['last_updated'] ??
      _localizedValues['en']['last_updated'];

  String get pro =>
      _localizedValues[localeCode]['pro'] ?? _localizedValues['en']['pro'];

  String get enterprise =>
      _localizedValues[localeCode]['enterprise'] ??
      _localizedValues['en']['enterprise'];

  String get invoiceCreated =>
      _localizedValues[localeCode]['invoice_created'] ??
      _localizedValues['en']['invoice_created'];

  String get quoteCreated =>
      _localizedValues[localeCode]['quote_created'] ??
      _localizedValues['en']['quote_created'];

  String get creditCreated =>
      _localizedValues[localeCode]['credit_created'] ??
      _localizedValues['en']['credit_created'];

  String get endAllSessions =>
      _localizedValues[localeCode]['end_all_sessions'] ??
      _localizedValues['en']['end_all_sessions'];

  String get countSession =>
      _localizedValues[localeCode]['count_session'] ??
      _localizedValues['en']['count_session'];

  String get countSessions =>
      _localizedValues[localeCode]['count_sessions'] ??
      _localizedValues['en']['count_sessions'];

  String get endedAllSessions =>
      _localizedValues[localeCode]['ended_all_sessions'] ??
      _localizedValues['en']['ended_all_sessions'];

  String get yearsDataShown =>
      _localizedValues[localeCode]['years_data_shown'] ??
      _localizedValues['en']['years_data_shown'];

  String get gatewaySetup =>
      _localizedValues[localeCode]['gateway_setup'] ??
      _localizedValues['en']['gateway_setup'];

  String get showTaskEndDate =>
      _localizedValues[localeCode]['show_task_end_date'] ??
      _localizedValues['en']['show_task_end_date'];

  String get showTaskEndDateHelp =>
      _localizedValues[localeCode]['show_task_end_date_help'] ??
      _localizedValues['en']['show_task_end_date_help'];

  String get migrationNotYetCompleted =>
      _localizedValues[localeCode]['migration_not_yet_completed'] ??
      _localizedValues['en']['migration_not_yet_completed'];

  String get accountId =>
      _localizedValues[localeCode]['account_id'] ??
      _localizedValues['en']['account_id'];

  String get step1SignIn =>
      _localizedValues[localeCode]['step_1_sign_in'] ??
      _localizedValues['en']['step_1_sign_in'];

  String get step2Authorize =>
      _localizedValues[localeCode]['step_2_authorize'] ??
      _localizedValues['en']['step_2_authorize'];

  String get includeDeletedClients =>
      _localizedValues[localeCode]['include_deleted_clients'] ??
      _localizedValues['en']['include_deleted_clients'];

  String get includeDeletedClientsHelp =>
      _localizedValues[localeCode]['include_deleted_clients_help'] ??
      _localizedValues['en']['include_deleted_clients_help'];

  String get exportedData =>
      _localizedValues[localeCode]['exported_data'] ??
      _localizedValues['en']['exported_data'];

  String get loginSuccess =>
      _localizedValues[localeCode]['login_success'] ??
      _localizedValues['en']['login_success'];

  String get loginFailure =>
      _localizedValues[localeCode]['login_failure'] ??
      _localizedValues['en']['login_failure'];

  String get importCustomers =>
      _localizedValues[localeCode]['import_customers'] ??
      _localizedValues['en']['import_customers'];

  String get importedCustomers =>
      _localizedValues[localeCode]['imported_customers'] ??
      _localizedValues['en']['imported_customers'];

  String get hasTaxes =>
      _localizedValues[localeCode]['has_taxes'] ??
      _localizedValues['en']['has_taxes'];

  String get netTotal =>
      _localizedValues[localeCode]['net_total'] ??
      _localizedValues['en']['net_total'];

  String get draftMode =>
      _localizedValues[localeCode]['html_mode'] ??
      _localizedValues['en']['html_mode'];

  String get draftModeHelp =>
      _localizedValues[localeCode]['html_mode_help'] ??
      _localizedValues['en']['html_mode_help'];

  String get waitForData =>
      _localizedValues[localeCode]['wait_for_data'] ??
      _localizedValues['en']['wait_for_data'];

  String get noPaymentTypesEnabled =>
      _localizedValues[localeCode]['no_payment_types_enabled'] ??
      _localizedValues['en']['no_payment_types_enabled'];

  String get json =>
      _localizedValues[localeCode]['json'] ?? _localizedValues['en']['json'];

  String get jsonFileMissing =>
      _localizedValues[localeCode]['json_file_missing'] ??
      _localizedValues['en']['json_file_missing'];

  String get jsonOptionMissing =>
      _localizedValues[localeCode]['json_option_missing'] ??
      _localizedValues['en']['json_option_missing'];

  String get importData =>
      _localizedValues[localeCode]['import_data'] ??
      _localizedValues['en']['import_data'];

  String get importSettings =>
      _localizedValues[localeCode]['import_settings'] ??
      _localizedValues['en']['import_settings'];

  String get sendTime =>
      _localizedValues[localeCode]['send_time'] ??
      _localizedValues['en']['send_time'];

  String get subdomainGuide =>
      _localizedValues[localeCode]['subdomain_guide'] ??
      _localizedValues['en']['subdomain_guide'];

  String get tasksShownInPortal =>
      _localizedValues[localeCode]['tasks_shown_in_portal'] ??
      _localizedValues['en']['tasks_shown_in_portal'];

  String get uninvoiced =>
      _localizedValues[localeCode]['uninvoiced'] ??
      _localizedValues['en']['uninvoiced'];

  String get clientPortalDomainHint =>
      _localizedValues[localeCode]['client_portal_domain_hint'] ??
      _localizedValues['en']['client_portal_domain_hint'];

  String get changeEmail =>
      _localizedValues[localeCode]['change_email'] ??
      _localizedValues['en']['change_email'];

  String get freeTrial =>
      _localizedValues[localeCode]['free_trial'] ??
      _localizedValues['en']['free_trial'];

  String get freeTrialHelp =>
      _localizedValues[localeCode]['free_trial_help'] ??
      _localizedValues['en']['free_trial_help'];

  String get freeTrialEndsInDays =>
      _localizedValues[localeCode]['free_trial_ends_in_days'] ??
      _localizedValues['en']['free_trial_ends_in_days'];

  String get freeTrialEndsToday =>
      _localizedValues[localeCode]['free_trial_ends_today'] ??
      _localizedValues['en']['free_trial_ends_today'];

  String get checkStatus =>
      _localizedValues[localeCode]['check_status'] ??
      _localizedValues['en']['check_status'];

  String get reviewApp =>
      _localizedValues[localeCode]['review_app'] ??
      _localizedValues['en']['review_app'];

  String get netSubtotal =>
      _localizedValues[localeCode]['net_subtotal'] ??
      _localizedValues['en']['net_subtotal'];

  String get sidebarActiveBackgroundColor =>
      _localizedValues[localeCode]['sidebar_active_background_color'] ??
      _localizedValues['en']['sidebar_active_background_color'];

  String get sidebarActiveFontColor =>
      _localizedValues[localeCode]['sidebar_active_font_color'] ??
      _localizedValues['en']['sidebar_active_font_color'];

  String get sidebarInactiveBackgroundColor =>
      _localizedValues[localeCode]['sidebar_inactive_background_color'] ??
      _localizedValues['en']['sidebar_inactive_background_color'];

  String get sidebarInactiveFontColor =>
      _localizedValues[localeCode]['sidebar_inactive_font_color'] ??
      _localizedValues['en']['sidebar_inactive_font_color'];

  String get tableAlternateRowBackgroundColor =>
      _localizedValues[localeCode]['table_alternate_row_background_color'] ??
      _localizedValues['en']['table_alternate_row_background_color'];

  String get invoiceHeaderBackgroundColor =>
      _localizedValues[localeCode]['invoice_header_background_color'] ??
      _localizedValues['en']['invoice_header_background_color'];

  String get invoiceHeaderFontColor =>
      _localizedValues[localeCode]['invoice_header_font_color'] ??
      _localizedValues['en']['invoice_header_font_color'];

  String get colors =>
      _localizedValues[localeCode]['colors'] ??
      _localizedValues['en']['colors'];

  String get customColors =>
      _localizedValues[localeCode]['custom_colors'] ??
      _localizedValues['en']['custom_colors'];

  String get statusColorTheme =>
      _localizedValues[localeCode]['status_color_theme'] ??
      _localizedValues['en']['status_color_theme'];

  String get loadColorTheme =>
      _localizedValues[localeCode]['load_color_theme'] ??
      _localizedValues['en']['load_color_theme'];

  String get clearAll =>
      _localizedValues[localeCode]['clear_all'] ??
      _localizedValues['en']['clear_all'];

  String get contrast =>
      _localizedValues[localeCode]['contrast'] ??
      _localizedValues['en']['contrast'];

  String get exportColors =>
      _localizedValues[localeCode]['export_colors'] ??
      _localizedValues['en']['export_colors'];

  String get importColors =>
      _localizedValues[localeCode]['import_colors'] ??
      _localizedValues['en']['import_colors'];

  String get previousPage =>
      _localizedValues[localeCode]['previous_page'] ??
      _localizedValues['en']['previous_page'];

  String get nextPage =>
      _localizedValues[localeCode]['next_page'] ??
      _localizedValues['en']['next_page'];

  String get addSecondContact =>
      _localizedValues[localeCode]['add_second_contact'] ??
      _localizedValues['en']['add_second_contact'];

  String get userGuide =>
      _localizedValues[localeCode]['user_guide'] ??
      _localizedValues['en']['user_guide'];

  String get enableMarkdown =>
      _localizedValues[localeCode]['enable_markdown'] ??
      _localizedValues['en']['enable_markdown'];

  String get enableMarkdownHelp =>
      _localizedValues[localeCode]['enable_markdown_help'] ??
      _localizedValues['en']['enable_markdown_help'];

  String get expenseTaxHelp =>
      _localizedValues[localeCode]['expense_tax_help'] ??
      _localizedValues['en']['expense_tax_help'];

  String get useWebAppToConnectGmail =>
      _localizedValues[localeCode]['use_web_app_to_connect_gmail'] ??
      _localizedValues['en']['use_web_app_to_connect_gmail'];

  String get selectMethod =>
      _localizedValues[localeCode]['select_method'] ??
      _localizedValues['en']['select_method'];

  String get selectPlatform =>
      _localizedValues[localeCode]['select_platform'] ??
      _localizedValues['en']['select_platform'];

  String get googleAnalyticsTrackingId =>
      _localizedValues[localeCode]['google_analytics_tracking_id'] ??
      _localizedValues['en']['google_analytics_tracking_id'];

  String get googleAnalytics =>
      _localizedValues[localeCode]['google_analytics'] ??
      _localizedValues['en']['google_analytics'];

  String get useCommaAsDecimalPlace =>
      _localizedValues[localeCode]['use_comma_as_decimal_place'] ??
      _localizedValues['en']['use_comma_as_decimal_place'];

  String get decimalComma =>
      _localizedValues[localeCode]['decimal_comma'] ??
      _localizedValues['en']['decimal_comma'];

  String get verifyCustomers =>
      _localizedValues[localeCode]['verify_customers'] ??
      _localizedValues['en']['verify_customers'];

  String get customerCount =>
      _localizedValues[localeCode]['customer_count'] ??
      _localizedValues['en']['customer_count'];

  String get persistData =>
      _localizedValues[localeCode]['persist_data'] ??
      _localizedValues['en']['persist_data'];

  String get changePlan =>
      _localizedValues[localeCode]['change_plan'] ??
      _localizedValues['en']['change_plan'];

  String get isInvoiced =>
      _localizedValues[localeCode]['is_invoiced'] ??
      _localizedValues['en']['is_invoiced'];

  String get includeDrafts =>
      _localizedValues[localeCode]['include_drafts'] ??
      _localizedValues['en']['include_drafts'];

  String get includeDraftsHelp =>
      _localizedValues[localeCode]['include_drafts_help'] ??
      _localizedValues['en']['include_drafts_help'];

  String get lastSentDate =>
      _localizedValues[localeCode]['last_sent_date'] ??
      _localizedValues['en']['last_sent_date'];

  String get recurringExpense =>
      _localizedValues[localeCode]['recurring_expense'] ??
      _localizedValues['en']['recurring_expense'];

  String get newRecurringExpense =>
      _localizedValues[localeCode]['new_recurring_expense'] ??
      _localizedValues['en']['new_recurring_expense'];

  String get createdRecurringExpense =>
      _localizedValues[localeCode]['created_recurring_expense'] ??
      _localizedValues['en']['created_recurring_expense'];

  String get updatedRecurringExpense =>
      _localizedValues[localeCode]['updated_recurring_expense'] ??
      _localizedValues['en']['updated_recurring_expense'];

  String get archivedRecurringExpense =>
      _localizedValues[localeCode]['archived_recurring_expense'] ??
      _localizedValues['en']['archived_recurring_expense'];

  String get deletedRecurringExpense =>
      _localizedValues[localeCode]['deleted_recurring_expense'] ??
      _localizedValues['en']['deleted_recurring_expense'];

  String get restoredRecurringExpense =>
      _localizedValues[localeCode]['restored_recurring_expense'] ??
      _localizedValues['en']['restored_recurring_expense'];

  String get searchRecurringExpense =>
      _localizedValues[localeCode]['search_recurring_expense'] ??
      _localizedValues['en']['search_recurring_expense'];

  String get searchRecurringExpenses =>
      _localizedValues[localeCode]['search_recurring_expenses'] ??
      _localizedValues['en']['search_recurring_expenses'];

  String get cloneToExpense =>
      _localizedValues[localeCode]['clone_to_expense'] ??
      _localizedValues['en']['clone_to_expense'];

  String get fromName =>
      _localizedValues[localeCode]['from_name'] ??
      _localizedValues['en']['from_name'];

  String get bulkEmailInvoice =>
      _localizedValues[localeCode]['bulk_email_invoice'] ??
      _localizedValues['en']['bulk_email_invoice'];

  String get bulkEmailQuote =>
      _localizedValues[localeCode]['bulk_email_quote'] ??
      _localizedValues['en']['bulk_email_quote'];

  String get bulkEmailCredit =>
      _localizedValues[localeCode]['bulk_email_credit'] ??
      _localizedValues['en']['bulk_email_credit'];

  String get grossLineTotal =>
      _localizedValues[localeCode]['gross_line_total'] ??
      _localizedValues['en']['gross_line_total'];

  String get forBestPerformance =>
      _localizedValues[localeCode]['for_best_performance'] ??
      _localizedValues['en']['for_best_performance'];

  String get app =>
      _localizedValues[localeCode]['app'] ?? _localizedValues['en']['app'];

  String get errorCrossClientTasks =>
      _localizedValues[localeCode]['error_cross_client_tasks'] ??
      _localizedValues['en']['error_cross_client_tasks'];

  String get errorCrossClientExpenses =>
      _localizedValues[localeCode]['error_cross_client_expenses'] ??
      _localizedValues['en']['error_cross_client_expenses'];

  String get webhookSuccess =>
      _localizedValues[localeCode]['webhook_success'] ??
      _localizedValues['en']['webhook_success'];

  String get whyAreYouLeaving =>
      _localizedValues[localeCode]['why_are_you_leaving'] ??
      _localizedValues['en']['why_are_you_leaving'];

  String get updatedCompany =>
      _localizedValues[localeCode]['updated_company'] ??
      _localizedValues['en']['updated_company'];

  String get setDefaultCompany =>
      _localizedValues[localeCode]['set_default_company'] ??
      _localizedValues['en']['set_default_company'];

  String get system =>
      _localizedValues[localeCode]['system'] ??
      _localizedValues['en']['system'];

  String get updateAllRecords =>
      _localizedValues[localeCode]['update_all_records'] ??
      _localizedValues['en']['update_all_records'];

  String get ideal =>
      _localizedValues[localeCode]['ideal'] ?? _localizedValues['en']['ideal'];

  String get sepa =>
      _localizedValues[localeCode]['sepa'] ?? _localizedValues['en']['sepa'];

  String get giropay =>
      _localizedValues[localeCode]['giropay'] ??
      _localizedValues['en']['giropay'];

  String get przelewy24 =>
      _localizedValues[localeCode]['przelewy24'] ??
      _localizedValues['en']['przelewy24'];

  String get eps =>
      _localizedValues[localeCode]['eps'] ?? _localizedValues['en']['eps'];

  String get fpx =>
      _localizedValues[localeCode]['fpx'] ?? _localizedValues['en']['fpx'];

  String get bancontact =>
      _localizedValues[localeCode]['bancontact'] ??
      _localizedValues['en']['bancontact'];

  String get viewStatement =>
      _localizedValues[localeCode]['view_statement'] ??
      _localizedValues['en']['view_statement'];

  String get viewExpense =>
      _localizedValues[localeCode]['view_expense'] ??
      _localizedValues['en']['view_expense'];

  String get unauthorizedStripeWarning =>
      _localizedValues[localeCode]['unauthorized_stripe_warning'] ??
      _localizedValues['en']['unauthorized_stripe_warning'];

  String get registration =>
      _localizedValues[localeCode]['registration'] ??
      _localizedValues['en']['registration'];

  String get hasTasks =>
      _localizedValues[localeCode]['has_tasks'] ??
      _localizedValues['en']['has_tasks'];

  String get clientPostalCode =>
      _localizedValues[localeCode]['client_postal_code'] ??
      _localizedValues['en']['client_postal_code'];

  String get clientVatNumber =>
      _localizedValues[localeCode]['client_vat_number'] ??
      _localizedValues['en']['client_vat_number'];

  String get persistDataHelp =>
      _localizedValues[localeCode]['persist_data_help'] ??
      _localizedValues['en']['persist_data_help'];

  String get persistUiHelp =>
      _localizedValues[localeCode]['persist_ui_help'] ??
      _localizedValues['en']['persist_ui_help'];

  String get persistUi =>
      _localizedValues[localeCode]['persist_ui'] ??
      _localizedValues['en']['persist_ui'];

  String get acss =>
      _localizedValues[localeCode]['acss'] ?? _localizedValues['en']['acss'];

  String get becs =>
      _localizedValues[localeCode]['becs'] ?? _localizedValues['en']['becs'];

  String get addToInvoices =>
      _localizedValues[localeCode]['add_to_invoices'] ??
      _localizedValues['en']['add_to_invoices'];

  String get disconnect =>
      _localizedValues[localeCode]['disconnect'] ??
      _localizedValues['en']['disconnect'];

  String get disconnectedGateway =>
      _localizedValues[localeCode]['disconnected_gateway'] ??
      _localizedValues['en']['disconnected_gateway'];

  String get recommendDesktop =>
      _localizedValues[localeCode]['recommend_desktop'] ??
      _localizedValues['en']['recommend_desktop'];

  String get recommendMobile =>
      _localizedValues[localeCode]['recommend_mobile'] ??
      _localizedValues['en']['recommend_mobile'];

  String get pleaseSetAPassword =>
      _localizedValues[localeCode]['please_set_a_password'] ??
      _localizedValues['en']['please_set_a_password'];

  String get setPassword =>
      _localizedValues[localeCode]['set_password'] ??
      _localizedValues['en']['set_password'];

  String get directDebit =>
      _localizedValues[localeCode]['direct_debit'] ??
      _localizedValues['en']['direct_debit'];

  String get applyCredit =>
      _localizedValues[localeCode]['apply_credit'] ??
      _localizedValues['en']['apply_credit'];

  String get creditIsMoreThanInvoice =>
      _localizedValues[localeCode]['credit_is_more_than_invoice'] ??
      _localizedValues['en']['credit_is_more_than_invoice'];

  String get clickSelected =>
      _localizedValues[localeCode]['click_selected'] ??
      _localizedValues['en']['click_selected'];

  String get editRecord =>
      _localizedValues[localeCode]['edit_record'] ??
      _localizedValues['en']['edit_record'];

  String get hidePreview =>
      _localizedValues[localeCode]['hide_preview'] ??
      _localizedValues['en']['hide_preview'];

  String get instantBankPay =>
      _localizedValues[localeCode]['instant_bank_pay'] ??
      _localizedValues['en']['instant_bank_pay'];

  String get remindMe =>
      _localizedValues[localeCode]['remind_me'] ??
      _localizedValues['en']['remind_me'];

  String get printPdf =>
      _localizedValues[localeCode]['print_pdf'] ??
      _localizedValues['en']['print_pdf'];

  String get showPdfPreview =>
      _localizedValues[localeCode]['show_pdf_preview'] ??
      _localizedValues['en']['show_pdf_preview'];

  String get showPdfPreviewHelp =>
      _localizedValues[localeCode]['show_pdf_preview_help'] ??
      _localizedValues['en']['show_pdf_preview_help'];

  // STARTER: lang field - do not remove comment

  String lookup(String key) {
    final lookupKey = toSnakeCase(key);

    if ((key ?? '').isEmpty) {
      return '';
    }

    if (lookupKey.startsWith('_')) {
      return key;
    }

    final value = _localizedValues[localeCode][lookupKey] ??
        _localizedValues[localeCode][lookupKey.replaceFirst('_id', '')] ??
        '';

    if (value.isEmpty) {
      print('## ERROR: localization key not found - $key');

      final englishValue = _localizedValues['en'][lookupKey] ?? '';

      return englishValue.isEmpty ? key : englishValue;
    }

    return value;
  }
}
